@@ -9,11 +9,10 @@
 
 #include "cdi.h"
 #include "cdi_int.h"
+#include "cdf.h"
 #include "cdf_int.h"
 #include "namespace.h"
 
-extern int CDF_Debug;
-
 #if  defined  (HAVE_LIBNETCDF)
 /*
 #if ! defined (MIN_BUF_SIZE)
@@ -24,7 +23,6 @@ static size_t ChunkSizeMin = MIN_BUF_SIZE;
 */
 void cdf_create(const char *path, int cmode, int *ncidp)
 {
-  int status;
   int oldfill;
   size_t initialsz = 0, chunksizehint = 0;
   /*
@@ -59,7 +57,7 @@ void cdf_create(const char *path, int cmode, int *ncidp)
 
   cdi_nc__create_funcp my_nc__create =
     (cdi_nc__create_funcp)namespaceSwitchGet(NSSWITCH_NC__CREATE).func;
-  status = my_nc__create(path, cmode, initialsz, &chunksizehint, ncidp);
+  int status = my_nc__create(path, cmode, initialsz, &chunksizehint, ncidp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d  mode = %d  file = %s", *ncidp, cmode, path);
@@ -114,15 +112,13 @@ int cdf_open(const char *path, int omode, int *ncidp)
 
   if ( CDF_Debug && status != NC_NOERR ) Message("%s", nc_strerror(status));
 
-  return (status);
+  return status;
 }
 
 
 void cdf_close(int ncid)
 {
-  int status;
-
-  status = nc_close(ncid);
+  int status = nc_close(ncid);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -130,9 +126,7 @@ void cdf_close(int ncid)
 
 void cdf_redef(int ncid)
 {
-  int status;
-
-  status = nc_redef(ncid);
+  int status = nc_redef(ncid);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -140,9 +134,7 @@ void cdf_redef(int ncid)
 
 void cdf_enddef(int ncid)
 {
-  int status;
-
-  status = nc_enddef(ncid);
+  int status = nc_enddef(ncid);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -150,13 +142,12 @@ void cdf_enddef(int ncid)
 
 void cdf__enddef(const int ncid, const size_t hdr_pad)
 {
-  int status;
   const size_t v_align   = 4UL; /* [B] Alignment of beginning of data section for fixed variables */
   const size_t v_minfree = 0UL; /* [B] Pad at end of data section for fixed size variables */
   const size_t r_align   = 4UL; /* [B] Alignment of beginning of data section for record variables */
 
   /* nc_enddef(ncid) is equivalent to nc__enddef(ncid, 0, 4, 0, 4) */
-  status = nc__enddef(ncid, hdr_pad, v_align, v_minfree, r_align);
+  int status = nc__enddef(ncid, hdr_pad, v_align, v_minfree, r_align);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -164,9 +155,7 @@ void cdf__enddef(const int ncid, const size_t hdr_pad)
 
 void cdf_sync(int ncid)
 {
-  int status;
-
-  status = nc_sync(ncid);
+  int status = nc_sync(ncid);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -174,9 +163,7 @@ void cdf_sync(int ncid)
 
 void cdf_inq(int ncid, int *ndimsp, int *nvarsp, int *ngattsp, int *unlimdimidp)
 {
-  int status;
-
-  status = nc_inq(ncid, ndimsp, nvarsp, ngattsp, unlimdimidp);
+  int status = nc_inq(ncid, ndimsp, nvarsp, ngattsp, unlimdimidp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d ndims = %d nvars = %d ngatts = %d unlimid = %d",
@@ -188,9 +175,7 @@ void cdf_inq(int ncid, int *ndimsp, int *nvarsp, int *ngattsp, int *unlimdimidp)
 
 void cdf_def_dim(int ncid, const char *name, size_t len, int *dimidp)
 {
-  int status;
-
-  status = nc_def_dim(ncid, name, len, dimidp);
+  int status = nc_def_dim(ncid, name, len, dimidp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d  name = %s  len = %d", ncid, name, len);
@@ -201,9 +186,7 @@ void cdf_def_dim(int ncid, const char *name, size_t len, int *dimidp)
 
 void cdf_inq_dimid(int ncid, const char *name, int *dimidp)
 {
-  int status;
-
-  status = nc_inq_dimid(ncid, name, dimidp);
+  int status = nc_inq_dimid(ncid, name, dimidp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d  name = %s  dimid= %d", ncid, name, *dimidp);
@@ -214,9 +197,7 @@ void cdf_inq_dimid(int ncid, const char *name, int *dimidp)
 
 void cdf_inq_dim(int ncid, int dimid, char *name, size_t * lengthp)
 {
-  int status;
-
-  status = nc_inq_dim(ncid, dimid, name, lengthp);
+  int status = nc_inq_dim(ncid, dimid, name, lengthp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d  dimid = %d  length = %d name = %s", ncid, dimid, *lengthp, name);
@@ -227,9 +208,7 @@ void cdf_inq_dim(int ncid, int dimid, char *name, size_t * lengthp)
 
 void cdf_inq_dimname(int ncid, int dimid, char *name)
 {
-  int status;
-
-  status = nc_inq_dimname(ncid, dimid, name);
+  int status = nc_inq_dimname(ncid, dimid, name);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d  dimid = %d  name = %s", ncid, dimid, name);
@@ -240,9 +219,7 @@ void cdf_inq_dimname(int ncid, int dimid, char *name)
 
 void cdf_inq_dimlen(int ncid, int dimid, size_t * lengthp)
 {
-  int status;
-
-  status = nc_inq_dimlen(ncid, dimid, lengthp);
+  int status = nc_inq_dimlen(ncid, dimid, lengthp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d dimid = %d length = %d", ncid, dimid, *lengthp);
@@ -276,9 +253,7 @@ cdf_def_var_serial(int ncid, const char *name, nc_type xtype, int ndims,
 
 void cdf_inq_varid(int ncid, const char *name, int *varidp)
 {
-  int status;
-
-  status = nc_inq_varid(ncid, name, varidp);
+  int status = nc_inq_varid(ncid, name, varidp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d  name = %s  varid = %d ", ncid, name, *varidp);
@@ -289,9 +264,7 @@ void cdf_inq_varid(int ncid, const char *name, int *varidp)
 
 void cdf_inq_nvars(int ncid, int *nvarsp)
 {
-  int status;
-
-  status = nc_inq_nvars(ncid, nvarsp);
+  int status = nc_inq_nvars(ncid, nvarsp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d  nvars = %d", ncid, *nvarsp);
@@ -303,9 +276,7 @@ void cdf_inq_nvars(int ncid, int *nvarsp)
 void cdf_inq_var(int ncid, int varid, char *name, nc_type *xtypep, int *ndimsp,
 		 int dimids[], int *nattsp)
 {
-  int status;
-
-  status = nc_inq_var(ncid, varid, name, xtypep, ndimsp, dimids, nattsp);
+  int status = nc_inq_var(ncid, varid, name, xtypep, ndimsp, dimids, nattsp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d ndims = %d xtype = %d natts = %d name = %s",
@@ -317,9 +288,7 @@ void cdf_inq_var(int ncid, int varid, char *name, nc_type *xtypep, int *ndimsp,
 
 void cdf_inq_varname(int ncid, int varid, char *name)
 {
-  int status;
-
-  status = nc_inq_varname(ncid, varid, name);
+  int status = nc_inq_varname(ncid, varid, name);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d name = %s", ncid, varid, name);
@@ -330,9 +299,7 @@ void cdf_inq_varname(int ncid, int varid, char *name)
 
 void cdf_inq_vartype(int ncid, int varid, nc_type *xtypep)
 {
-  int status;
-
-  status = nc_inq_vartype(ncid, varid, xtypep);
+  int status = nc_inq_vartype(ncid, varid, xtypep);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d xtype = %s", ncid, varid, *xtypep);
@@ -343,9 +310,7 @@ void cdf_inq_vartype(int ncid, int varid, nc_type *xtypep)
 
 void cdf_inq_varndims(int ncid, int varid, int *ndimsp)
 {
-  int status;
-
-  status = nc_inq_varndims(ncid, varid, ndimsp);
+  int status = nc_inq_varndims(ncid, varid, ndimsp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -356,9 +321,7 @@ void cdf_inq_varndims(int ncid, int varid, int *ndimsp)
 
 void cdf_inq_vardimid(int ncid, int varid, int dimids[])
 {
-  int status;
-
-  status = nc_inq_vardimid(ncid, varid, dimids);
+  int status = nc_inq_vardimid(ncid, varid, dimids);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -369,9 +332,7 @@ void cdf_inq_vardimid(int ncid, int varid, int dimids[])
 
 void cdf_inq_varnatts(int ncid, int varid, int *nattsp)
 {
-  int status;
-
-  status = nc_inq_varnatts(ncid, varid, nattsp);
+  int status = nc_inq_varnatts(ncid, varid, nattsp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d nattsp = %d", ncid, varid, *nattsp);
@@ -382,9 +343,7 @@ void cdf_inq_varnatts(int ncid, int varid, int *nattsp)
 
 void cdf_put_var_text(int ncid, int varid, const char *tp)
 {
-  int status;
-
-  status = nc_put_var_text(ncid, varid, tp);
+  int status = nc_put_var_text(ncid, varid, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("%d %d %s", ncid, varid, tp);
@@ -395,9 +354,7 @@ void cdf_put_var_text(int ncid, int varid, const char *tp)
 
 void cdf_put_var_short(int ncid, int varid, const short *sp)
 {
-  int status;
-
-  status = nc_put_var_short(ncid, varid, sp);
+  int status = nc_put_var_short(ncid, varid, sp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("%d %d %hd", ncid, varid, *sp);
@@ -408,9 +365,7 @@ void cdf_put_var_short(int ncid, int varid, const short *sp)
 
 void cdf_put_var_int(int ncid, int varid, const int *ip)
 {
-  int status;
-
-  status = nc_put_var_int(ncid, varid, ip);
+  int status = nc_put_var_int(ncid, varid, ip);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("%d %d %d", ncid, varid, *ip);
@@ -421,9 +376,7 @@ void cdf_put_var_int(int ncid, int varid, const int *ip)
 
 void cdf_put_var_long(int ncid, int varid, const long *lp)
 {
-  int status;
-
-  status = nc_put_var_long(ncid, varid, lp);
+  int status = nc_put_var_long(ncid, varid, lp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("%d %d %ld", ncid, varid, *lp);
@@ -434,9 +387,7 @@ void cdf_put_var_long(int ncid, int varid, const long *lp)
 
 void cdf_put_var_float(int ncid, int varid, const float *fp)
 {
-  int status;
-
-  status = nc_put_var_float(ncid, varid, fp);
+  int status = nc_put_var_float(ncid, varid, fp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("%d %d %f", ncid, varid, *fp);
@@ -448,9 +399,7 @@ void cdf_put_var_float(int ncid, int varid, const float *fp)
 void cdf_put_vara_double(int ncid, int varid, const size_t start[],
                          const size_t count[], const double *dp)
 {
-  int status;
-
-  status = nc_put_vara_double(ncid, varid, start, count, dp);
+  int status = nc_put_vara_double(ncid, varid, start, count, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d val0 = %f", ncid, varid, *dp);
@@ -469,9 +418,7 @@ void cdf_put_vara_double(int ncid, int varid, const size_t start[],
 void  cdf_put_vara_float(int ncid, int varid, const size_t start[],
                          const size_t count[], const float *fp)
 {
-  int status;
-
-  status = nc_put_vara_float(ncid, varid, start, count, fp);
+  int status = nc_put_vara_float(ncid, varid, start, count, fp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d val0 = %f", ncid, varid, *fp);
@@ -483,9 +430,7 @@ void  cdf_put_vara_float(int ncid, int varid, const size_t start[],
 void  cdf_get_vara_int(int ncid, int varid, const size_t start[],
                        const size_t count[], int *dp)
 {
-  int status;
-
-  status = nc_get_vara_int(ncid, varid, start, count, dp);
+  int status = nc_get_vara_int(ncid, varid, start, count, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -497,9 +442,7 @@ void  cdf_get_vara_int(int ncid, int varid, const size_t start[],
 void  cdf_get_vara_double(int ncid, int varid, const size_t start[],
                           const size_t count[], double *dp)
 {
-  int status;
-
-  status = nc_get_vara_double(ncid, varid, start, count, dp);
+  int status = nc_get_vara_double(ncid, varid, start, count, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -511,9 +454,7 @@ void  cdf_get_vara_double(int ncid, int varid, const size_t start[],
 void  cdf_get_vara_float(int ncid, int varid, const size_t start[],
                          const size_t count[], float *fp)
 {
-  int status;
-
-  status = nc_get_vara_float(ncid, varid, start, count, fp);
+  int status = nc_get_vara_float(ncid, varid, start, count, fp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -525,9 +466,7 @@ void  cdf_get_vara_float(int ncid, int varid, const size_t start[],
 void  cdf_get_vara_text(int ncid, int varid, const size_t start[],
 			const size_t count[], char *tp)
 {
-  int status;
-
-  status = nc_get_vara_text(ncid, varid, start, count, tp);
+  int status = nc_get_vara_text(ncid, varid, start, count, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -538,9 +477,7 @@ void  cdf_get_vara_text(int ncid, int varid, const size_t start[],
 
 void  cdf_get_vara_uchar(int ncid, int varid, const size_t start[], const size_t count[], unsigned char *tp)
 {
-  int status;
-
-  status = nc_get_vara_uchar(ncid, varid, start, count, tp);
+  int status = nc_get_vara_uchar(ncid, varid, start, count, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -551,9 +488,7 @@ void  cdf_get_vara_uchar(int ncid, int varid, const size_t start[], const size_t
 
 void cdf_put_var_double(int ncid, int varid, const double *dp)
 {
-  int status;
-
-  status = nc_put_var_double(ncid, varid, dp);
+  int status = nc_put_var_double(ncid, varid, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d val0 = %f", ncid, varid, *dp);
@@ -564,9 +499,7 @@ void cdf_put_var_double(int ncid, int varid, const double *dp)
 
 void cdf_get_var1_text(int ncid, int varid, const size_t index[], char *tp)
 {
-  int status;
-
-  status = nc_get_var1_text(ncid, varid, index, tp);
+  int status = nc_get_var1_text(ncid, varid, index, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -577,9 +510,7 @@ void cdf_get_var1_text(int ncid, int varid, const size_t index[], char *tp)
 
 void cdf_get_var1_double(int ncid, int varid, const size_t index[], double *dp)
 {
-  int status;
-
-  status = nc_get_var1_double(ncid, varid, index, dp);
+  int status = nc_get_var1_double(ncid, varid, index, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -590,9 +521,7 @@ void cdf_get_var1_double(int ncid, int varid, const size_t index[], double *dp)
 
 void cdf_put_var1_double(int ncid, int varid, const size_t index[], const double *dp)
 {
-  int status;
-
-  status = nc_put_var1_double(ncid, varid, index, dp);
+  int status = nc_put_var1_double(ncid, varid, index, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d val = %f", ncid, varid, *dp);
@@ -603,9 +532,7 @@ void cdf_put_var1_double(int ncid, int varid, const size_t index[], const double
 
 void cdf_get_var_text(int ncid, int varid, char *tp)
 {
-  int status;
-
-  status = nc_get_var_text(ncid, varid, tp);
+  int status = nc_get_var_text(ncid, varid, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -616,9 +543,7 @@ void cdf_get_var_text(int ncid, int varid, char *tp)
 
 void cdf_get_var_short(int ncid, int varid, short *sp)
 {
-  int status;
-
-  status = nc_get_var_short(ncid, varid, sp);
+  int status = nc_get_var_short(ncid, varid, sp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -629,9 +554,7 @@ void cdf_get_var_short(int ncid, int varid, short *sp)
 
 void cdf_get_var_int(int ncid, int varid, int *ip)
 {
-  int status;
-
-  status = nc_get_var_int(ncid, varid, ip);
+  int status = nc_get_var_int(ncid, varid, ip);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -642,9 +565,7 @@ void cdf_get_var_int(int ncid, int varid, int *ip)
 
 void cdf_get_var_long(int ncid, int varid, long *lp)
 {
-  int status;
-
-  status = nc_get_var_long(ncid, varid, lp);
+  int status = nc_get_var_long(ncid, varid, lp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -655,9 +576,7 @@ void cdf_get_var_long(int ncid, int varid, long *lp)
 
 void cdf_get_var_float(int ncid, int varid, float *fp)
 {
-  int status;
-
-  status = nc_get_var_float(ncid, varid, fp);
+  int status = nc_get_var_float(ncid, varid, fp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d", ncid, varid);
@@ -668,9 +587,7 @@ void cdf_get_var_float(int ncid, int varid, float *fp)
 
 void cdf_get_var_double(int ncid, int varid, double *dp)
 {
-  int status;
-
-  status = nc_get_var_double(ncid, varid, dp);
+  int status = nc_get_var_double(ncid, varid, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d val[0] = %f", ncid, varid, *dp);
@@ -682,9 +599,7 @@ void cdf_get_var_double(int ncid, int varid, double *dp)
 void cdf_copy_att(int ncid_in, int varid_in, const char *name, int ncid_out,
 		  int varid_out)
 {
-  int status;
-
-  status = nc_copy_att(ncid_in, varid_in, name, ncid_out, varid_out);
+  int status = nc_copy_att(ncid_in, varid_in, name, ncid_out, varid_out);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("%d %d %s %d %d", ncid_in, varid_out, name, ncid_out, varid_out);
@@ -696,9 +611,7 @@ void cdf_copy_att(int ncid_in, int varid_in, const char *name, int ncid_out,
 void cdf_put_att_text(int ncid, int varid, const char *name, size_t len,
 		      const char *tp)
 {
-  int status;
-
-  status = nc_put_att_text(ncid, varid, name, len, tp);
+  int status = nc_put_att_text(ncid, varid, name, len, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d att = %s text = %.*s", ncid, varid, name, (int)len, tp);
@@ -710,9 +623,7 @@ void cdf_put_att_text(int ncid, int varid, const char *name, size_t len,
 void cdf_put_att_int(int ncid, int varid, const char *name, nc_type xtype,
 		     size_t len, const int *ip)
 {
-  int status;
-
-  status = nc_put_att_int(ncid, varid, name, xtype, len, ip);
+  int status = nc_put_att_int(ncid, varid, name, xtype, len, ip);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d att = %s val = %d", ncid, varid, name, *ip);
@@ -721,15 +632,25 @@ void cdf_put_att_int(int ncid, int varid, const char *name, nc_type xtype,
 }
 
 
+void cdf_put_att_float(int ncid, int varid, const char *name, nc_type xtype,
+                       size_t len, const float *dp)
+{
+  int status = nc_put_att_float(ncid, varid, name, xtype, len, dp);
+
+  if ( CDF_Debug || status != NC_NOERR )
+    Message("ncid = %d varid = %d att = %s val = %g", ncid, varid, name, *dp);
+
+  if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
+}
+
+
 void cdf_put_att_double(int ncid, int varid, const char *name, nc_type xtype,
 			size_t len, const double *dp)
 {
-  int status;
-
-  status = nc_put_att_double(ncid, varid, name, xtype, len, dp);
+  int status = nc_put_att_double(ncid, varid, name, xtype, len, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
-    Message("%d %d %f", ncid, varid, *dp);
+    Message("ncid = %d varid = %d att = %s val = %g", ncid, varid, name, *dp);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -737,9 +658,7 @@ void cdf_put_att_double(int ncid, int varid, const char *name, nc_type xtype,
 
 void cdf_get_att_text(int ncid, int varid, const char *name, char *tp)
 {
-  int status;
-
-  status = nc_get_att_text(ncid, varid, name, tp);
+  int status = nc_get_att_text(ncid, varid, name, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d name = %s", ncid, varid, name);
@@ -750,9 +669,7 @@ void cdf_get_att_text(int ncid, int varid, const char *name, char *tp)
 void cdf_get_att_string(int ncid, int varid, const char *name, char **tp)
 {
 #if  defined  (HAVE_NETCDF4)
-  int status;
-
-  status = nc_get_att_string(ncid, varid, name, tp);
+  int status = nc_get_att_string(ncid, varid, name, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d name = %s", ncid, varid, name);
@@ -764,9 +681,7 @@ void cdf_get_att_string(int ncid, int varid, const char *name, char **tp)
 
 void cdf_get_att_int(int ncid, int varid, const char *name, int *ip)
 {
-  int status;
-
-  status = nc_get_att_int(ncid, varid, name, ip);
+  int status = nc_get_att_int(ncid, varid, name, ip);
 
   if ( CDF_Debug || status != NC_NOERR )
     Message("ncid = %d varid = %d att = %s val = %d", ncid, varid, name, *ip);
@@ -775,6 +690,19 @@ void cdf_get_att_int(int ncid, int varid, const char *name, int *ip)
 }
 
 
+void cdf_get_att_long(int ncid, int varid, const char *name, long *ip)
+{
+#if  defined  (HAVE_NETCDF4)
+  int status = nc_get_att_long(ncid, varid, name, ip);
+
+  if ( CDF_Debug || status != NC_NOERR )
+    Message("ncid = %d varid = %d att = %s val = %ld", ncid, varid, name, *ip);
+
+  if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
+#endif
+}
+
+
 void cdf_get_att_double(int ncid, int varid, const char *name, double *dp)
 {
   int status;
@@ -782,8 +710,7 @@ void cdf_get_att_double(int ncid, int varid, const char *name, double *dp)
   status = nc_get_att_double(ncid, varid, name, dp);
 
   if ( CDF_Debug || status != NC_NOERR )
-    Message("ncid = %d varid = %d att = %s val = %.9g",
-	    ncid, varid, name, *dp);
+    Message("ncid = %d varid = %d att = %s val = %.9g", ncid, varid, name, *dp);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -792,12 +719,10 @@ void cdf_get_att_double(int ncid, int varid, const char *name, double *dp)
 void cdf_inq_att(int ncid, int varid, const char *name, nc_type *xtypep,
 		 size_t *lenp)
 {
-  int status;
-
-  status = nc_inq_att(ncid, varid, name, xtypep, lenp);
+  int status = nc_inq_att(ncid, varid, name, xtypep, lenp);
 
   if ( CDF_Debug || status != NC_NOERR )
-    Message("ncid = %d varid = %d", ncid, varid);
+    Message("ncid = %d varid = %d att = %s", ncid, varid, name);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -805,12 +730,10 @@ void cdf_inq_att(int ncid, int varid, const char *name, nc_type *xtypep,
 
 void cdf_inq_atttype(int ncid, int varid, const char *name, nc_type * xtypep)
 {
-  int status;
-
-  status = nc_inq_atttype(ncid, varid, name, xtypep);
+  int status = nc_inq_atttype(ncid, varid, name, xtypep);
 
   if ( CDF_Debug || status != NC_NOERR )
-    Message("ncid = %d varid = %d", ncid, varid);
+    Message("ncid = %d varid = %d att = %s", ncid, varid, name);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -818,12 +741,10 @@ void cdf_inq_atttype(int ncid, int varid, const char *name, nc_type * xtypep)
 
 void cdf_inq_attlen(int ncid, int varid, const char *name, size_t * lenp)
 {
-  int status;
-
-  status = nc_inq_attlen(ncid, varid, name, lenp);
+  int status = nc_inq_attlen(ncid, varid, name, lenp);
 
   if ( CDF_Debug || status != NC_NOERR )
-    Message("ncid = %d varid = %d name = %s len = %d", ncid, varid, name, *lenp);
+    Message("ncid = %d varid = %d att = %s len = %d", ncid, varid, name, *lenp);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -831,12 +752,10 @@ void cdf_inq_attlen(int ncid, int varid, const char *name, size_t * lenp)
 
 void cdf_inq_attname(int ncid, int varid, int attnum, char *name)
 {
-  int status;
-
-  status = nc_inq_attname(ncid, varid, attnum, name);
+  int status = nc_inq_attname(ncid, varid, attnum, name);
 
   if ( CDF_Debug || status != NC_NOERR )
-    Message("ncid = %d varid = %d attnum = %d name = %s", ncid, varid, attnum, name);
+    Message("ncid = %d varid = %d attnum = %d att = %s", ncid, varid, attnum, name);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -844,12 +763,10 @@ void cdf_inq_attname(int ncid, int varid, int attnum, char *name)
 
 void cdf_inq_attid(int ncid, int varid, const char *name, int *attnump)
 {
-  int status;
-
-  status = nc_inq_attid(ncid, varid, name, attnump);
+  int status = nc_inq_attid(ncid, varid, name, attnump);
 
   if ( CDF_Debug || status != NC_NOERR )
-    Message("ncid = %d varid = %d", ncid, varid);
+    Message("ncid = %d varid = %d att = %s", ncid, varid, name);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
