@@ -16,10 +16,6 @@
 #include "vlist.h"
 
 
-#undef  UNDEFID
-#define UNDEFID  CDI_UNDEFID
-
-
 static
 void cdfReadGridTraj(stream_t *streamptr, int gridID)
 {
@@ -27,8 +23,8 @@ void cdfReadGridTraj(stream_t *streamptr, int gridID)
   int fileID  = streamptr->fileID;
 
   int gridindex = vlistGridIndex(vlistID, gridID);
-  int lonID = streamptr->xdimID[gridindex];
-  int latID = streamptr->ydimID[gridindex];
+  int lonID = streamptr->ncgrid[gridindex].xdimID;
+  int latID = streamptr->ncgrid[gridindex].ydimID;
 
   int tsID = streamptr->curTsID;
   size_t index = (size_t)tsID;
@@ -53,15 +49,15 @@ void cdfGetSlapDescription(stream_t *streamptr, int varID, size_t (*start)[4], s
 
   if ( CDI_Debug ) Message("tsID = %d", tsID);
 
-  int xid = UNDEFID, yid = UNDEFID;
+  int xid = CDI_UNDEFID, yid = CDI_UNDEFID;
   if ( gridInqType(gridID) == GRID_TRAJECTORY )
     {
       cdfReadGridTraj(streamptr, gridID);
     }
   else
     {
-      xid = streamptr->xdimID[gridindex];
-      yid = streamptr->ydimID[gridindex];
+      xid = streamptr->ncgrid[gridindex].xdimID;
+      yid = streamptr->ncgrid[gridindex].ydimID;
     }
   int zaxisindex = vlistZaxisIndex(vlistID, zaxisID);
   int zid = streamptr->zaxisID[zaxisindex];
@@ -74,9 +70,9 @@ void cdfGetSlapDescription(stream_t *streamptr, int varID, size_t (*start)[4], s
       ndims++; \
     } while(0)
   if ( tsteptype != TSTEP_CONSTANT ) addDimension((size_t)tsID, 1);
-  if ( zid != UNDEFID ) addDimension(0, (size_t)zaxisInqSize(zaxisID));
-  if ( yid != UNDEFID ) addDimension(0, (size_t)gridInqYsize(gridID));
-  if ( xid != UNDEFID ) addDimension(0, (size_t)gridInqXsize(gridID));
+  if ( zid != CDI_UNDEFID ) addDimension(0, (size_t)zaxisInqSize(zaxisID));
+  if ( yid != CDI_UNDEFID ) addDimension(0, (size_t)gridInqYsize(gridID));
+  if ( xid != CDI_UNDEFID ) addDimension(0, (size_t)gridInqXsize(gridID));
 #undef addDimension
 
   assert(ndims <= (int)(sizeof(*start)/sizeof(**start)));
@@ -314,18 +310,12 @@ void transpose2dArrayDP(size_t inWidth, size_t inHeight, double *data)
   */
 
   for ( size_t yBlock = 0; yBlock < inHeight; yBlock += cacheBlockSize )
-    {
-      for ( size_t xBlock = 0; xBlock < inWidth; xBlock += cacheBlockSize )
-        {
-          for ( size_t y = yBlock, yEnd = min_size(yBlock + cacheBlockSize, inHeight); y < yEnd; y++ )
-            {
-              for ( size_t x = xBlock, xEnd = min_size(xBlock + cacheBlockSize, inWidth); x < xEnd; x++ )
-                {
-                  out[x][y] = temp[y][x];
-                }
-            }
-        }
-    }
+    for ( size_t xBlock = 0; xBlock < inWidth; xBlock += cacheBlockSize )
+      for ( size_t y = yBlock, yEnd = min_size(yBlock + cacheBlockSize, inHeight); y < yEnd; y++ )
+        for ( size_t x = xBlock, xEnd = min_size(xBlock + cacheBlockSize, inWidth); x < xEnd; x++ )
+          {
+            out[x][y] = temp[y][x];
+          }
 
   Free(temp[0]);
 }
@@ -355,18 +345,12 @@ void transpose2dArraySP(size_t inWidth, size_t inHeight, float *data)
   */
 
   for ( size_t yBlock = 0; yBlock < inHeight; yBlock += cacheBlockSize )
-    {
-      for ( size_t xBlock = 0; xBlock < inWidth; xBlock += cacheBlockSize )
-        {
-          for ( size_t y = yBlock, yEnd = min_size(yBlock + cacheBlockSize, inHeight); y < yEnd; y++ )
-            {
-              for ( size_t x = xBlock, xEnd = min_size(xBlock + cacheBlockSize, inWidth); x < xEnd; x++ )
-                {
-                  out[x][y] = temp[y][x];
-                }
-            }
-        }
-    }
+    for ( size_t xBlock = 0; xBlock < inWidth; xBlock += cacheBlockSize )
+      for ( size_t y = yBlock, yEnd = min_size(yBlock + cacheBlockSize, inHeight); y < yEnd; y++ )
+        for ( size_t x = xBlock, xEnd = min_size(xBlock + cacheBlockSize, inWidth); x < xEnd; x++ )
+          {
+            out[x][y] = temp[y][x];
+          }
 
   Free(temp);
 }
@@ -377,7 +361,7 @@ void cdfInqDimIds(stream_t *streamptr, int varId, int (*outDimIds)[3])
   int gridId = vlistInqVarGrid(streamptr->vlistID, varId);
   int gridindex = vlistGridIndex(streamptr->vlistID, gridId);
 
-  (*outDimIds)[0] = (*outDimIds)[1] = (*outDimIds)[2] = UNDEFID;
+  (*outDimIds)[0] = (*outDimIds)[1] = (*outDimIds)[2] = CDI_UNDEFID;
   switch ( gridInqType(gridId) )
     {
       case GRID_TRAJECTORY:
@@ -385,12 +369,12 @@ void cdfInqDimIds(stream_t *streamptr, int varId, int (*outDimIds)[3])
         break;
 
       case GRID_UNSTRUCTURED:
-        (*outDimIds)[0] = streamptr->xdimID[gridindex];
+        (*outDimIds)[0] = streamptr->ncgrid[gridindex].xdimID;
         break;
 
       default:
-        (*outDimIds)[0] = streamptr->xdimID[gridindex];
-        (*outDimIds)[1] = streamptr->ydimID[gridindex];
+        (*outDimIds)[0] = streamptr->ncgrid[gridindex].xdimID;
+        (*outDimIds)[1] = streamptr->ncgrid[gridindex].ydimID;
         break;
     }
 
@@ -402,8 +386,8 @@ void cdfInqDimIds(stream_t *streamptr, int varId, int (*outDimIds)[3])
 static
 int cdfGetSkipDim(int fileId, int ncvarid, int (*dimIds)[3])
 {
-  if((*dimIds)[0] != UNDEFID) return 0;
-  if((*dimIds)[1] != UNDEFID) return 0;
+  if((*dimIds)[0] != CDI_UNDEFID) return 0;
+  if((*dimIds)[1] != CDI_UNDEFID) return 0;
   int nvdims;
   cdf_inq_varndims(fileId, ncvarid, &nvdims);
   if(nvdims != 3) return 0;
@@ -448,7 +432,7 @@ void cdfGetSliceSlapDescription(stream_t *streamptr, int varId, int levelId, boo
   int dimorder[3];
   vlistInqVarDimorder(vlistId, varId, &dimorder);
 
-  *outSwapXY = (dimorder[2] == 2 || dimorder[0] == 1) && dimIds[0] != UNDEFID && dimIds[1] != UNDEFID ;
+  *outSwapXY = (dimorder[2] == 2 || dimorder[0] == 1) && dimIds[0] != CDI_UNDEFID && dimIds[1] != CDI_UNDEFID ;
 
   int ndims = 0;
 
@@ -465,7 +449,7 @@ void cdfGetSliceSlapDescription(stream_t *streamptr, int varId, int levelId, boo
     {
       size_t size;
       int curDimId = dimIds[dimorder[id]-1];
-      if ( curDimId == UNDEFID ) continue;
+      if ( curDimId == CDI_UNDEFID ) continue;
       switch ( dimorder[id] )
         {
           Error("Internal errror: Malformed dimension order encountered. Please report this bug.\n");
@@ -578,15 +562,13 @@ void cdf_read_var(stream_t *streamptr, int varID, int memtype, void *data, int *
 static
 void cdfReadVarSliceDP(stream_t *streamptr, int varID, int levelID, double *data, int *nmiss)
 {
-  size_t start[4];
-  size_t count[4];
-
   if ( CDI_Debug )
     Message("streamID = %d  varID = %d  levelID = %d", streamptr->self, varID, levelID);
 
   int vlistID = streamptr->vlistID;
   int fileID = streamptr->fileID;
 
+  size_t start[4], count[4];
   bool swapxy;
   cdfGetSliceSlapDescription(streamptr, varID, levelID, &swapxy, &start, &count);
 
@@ -596,7 +578,7 @@ void cdfReadVarSliceDP(stream_t *streamptr, int varID, int levelID, double *data
   size_t xsize = (size_t)gridInqXsize(gridId);
   size_t ysize = (size_t)gridInqYsize(gridId);
 
-  if ( vlistInqVarDatatype(vlistID, varID) == DATATYPE_FLT32 )
+  if ( vlistInqVarDatatype(vlistID, varID) == CDI_DATATYPE_FLT32 )
     {
       float *data_fp = (float *) Malloc(gridsize*sizeof(*data_fp));
       cdf_get_vara_float(fileID, ncvarid, start, count, data_fp);
@@ -604,19 +586,20 @@ void cdfReadVarSliceDP(stream_t *streamptr, int varID, int levelID, double *data
         data[i] = (double) data_fp[i];
       Free(data_fp);
     }
-  else if ( vlistInqVarDatatype(vlistID, varID) == DATATYPE_UINT8 )
-    {
-      nc_type xtype;
-      cdf_inq_vartype(fileID, ncvarid, &xtype);
-      if ( xtype == NC_BYTE )
-        {
-          for ( size_t i = 0; i < gridsize; i++ )
-            if ( data[i] < 0 ) data[i] += 256;
-        }
-    }
   else
     {
       cdf_get_vara_double(fileID, ncvarid, start, count, data);
+      
+      if ( vlistInqVarDatatype(vlistID, varID) == CDI_DATATYPE_UINT8 )
+        {
+          nc_type xtype;
+          cdf_inq_vartype(fileID, ncvarid, &xtype);
+          if ( xtype == NC_BYTE )
+            {
+              for ( size_t i = 0; i < gridsize; i++ )
+                if ( data[i] < 0 ) data[i] += 256;
+            }
+        }
     }
 
   if ( swapxy ) transpose2dArrayDP(ysize, xsize, data);
@@ -636,15 +619,13 @@ void cdfReadVarSliceDP(stream_t *streamptr, int varID, int levelID, double *data
 static
 void cdfReadVarSliceSP(stream_t *streamptr, int varID, int levelID, float *data, int *nmiss)
 {
-  size_t start[4];
-  size_t count[4];
-
   if ( CDI_Debug )
     Message("streamID = %d  varID = %d  levelID = %d", streamptr->self, varID, levelID);
 
   int vlistID = streamptr->vlistID;
   int fileID = streamptr->fileID;
 
+  size_t start[4], count[4];
   bool swapxy;
   cdfGetSliceSlapDescription(streamptr, varID, levelID, &swapxy, &start, &count);
 
@@ -654,7 +635,7 @@ void cdfReadVarSliceSP(stream_t *streamptr, int varID, int levelID, float *data,
   size_t xsize = (size_t)gridInqXsize(gridId);
   size_t ysize = (size_t)gridInqYsize(gridId);
 
-  if ( vlistInqVarDatatype(vlistID, varID) == DATATYPE_FLT64 )
+  if ( vlistInqVarDatatype(vlistID, varID) == CDI_DATATYPE_FLT64 )
     {
       double *data_dp = (double *) Malloc(gridsize*sizeof(*data_dp));
       cdf_get_vara_double(fileID, ncvarid, start, count, data_dp);
@@ -662,19 +643,20 @@ void cdfReadVarSliceSP(stream_t *streamptr, int varID, int levelID, float *data,
         data[i] = (float) data_dp[i];
       Free(data_dp);
     }
-  else if ( vlistInqVarDatatype(vlistID, varID) == DATATYPE_UINT8 )
-    {
-      nc_type xtype;
-      cdf_inq_vartype(fileID, ncvarid, &xtype);
-      if ( xtype == NC_BYTE )
-        {
-          for ( size_t i = 0; i < gridsize; i++ )
-            if ( data[i] < 0 ) data[i] += 256;
-        }
-    }
   else
     {
       cdf_get_vara_float(fileID, ncvarid, start, count, data);
+
+      if ( vlistInqVarDatatype(vlistID, varID) == CDI_DATATYPE_UINT8 )
+        {
+          nc_type xtype;
+          cdf_inq_vartype(fileID, ncvarid, &xtype);
+          if ( xtype == NC_BYTE )
+            {
+              for ( size_t i = 0; i < gridsize; i++ )
+                if ( data[i] < 0 ) data[i] += 256;
+            }
+        }
     }
 
   if ( swapxy ) transpose2dArraySP(ysize, xsize, data);
