@@ -28,7 +28,7 @@ int cdiDefaultModelID  = CDI_UNDEFID;
 int cdiDefaultTableID  = CDI_UNDEFID;
 //int cdiNcMissingValue  = CDI_UNDEFID;
 int cdiNcChunksizehint = CDI_UNDEFID;
-int cdiChunkType       = CHUNK_GRID;
+int cdiChunkType       = CDI_CHUNK_GRID;
 int cdiSplitLtype105   = CDI_UNDEFID;
 
 int cdiIgnoreAttCoordinates = FALSE;
@@ -67,6 +67,7 @@ int cdiGribApiDebug     = 0;
 int cdiDefaultLeveltype = -1;
 int cdiDataUnreduced = 0;
 int cdiSortName = 0;
+int cdiSortParam = 0;
 int cdiHaveMissval = 0;
 
 
@@ -132,7 +133,7 @@ void cdiPrintVersion(void)
 #if  defined  (HAVE_LIBNETCDF)
   fprintf(stderr, "  NetCDF library version : %s\n", cdfLibraryVersion());
 #endif
-#if  defined  (HAVE_LIBHDF5)
+#if  defined  (HAVE_NC4HDF5)
   fprintf(stderr, "    HDF5 library version : %s\n", hdfLibraryVersion());
 #endif
 #if  defined  (HAVE_LIBSERVICE)
@@ -191,30 +192,30 @@ int cdiHaveFiletype(int filetype)
   switch (filetype)
     {
 #if  defined  (HAVE_LIBSERVICE)
-    case FILETYPE_SRV:  { status = 1; break; }
+    case CDI_FILETYPE_SRV:  { status = 1; break; }
 #endif
 #if  defined  (HAVE_LIBEXTRA)
-    case FILETYPE_EXT:  { status = 1; break; }
+    case CDI_FILETYPE_EXT:  { status = 1; break; }
 #endif
 #if  defined  (HAVE_LIBIEG)
-    case FILETYPE_IEG:  { status = 1; break; }
+    case CDI_FILETYPE_IEG:  { status = 1; break; }
 #endif
 #if  defined  (HAVE_LIBGRIB)
 #if  defined  (HAVE_LIBGRIB_API) || defined  (HAVE_LIBCGRIBEX)
-    case FILETYPE_GRB:  { status = 1; break; }
+    case CDI_FILETYPE_GRB:  { status = 1; break; }
 #endif
 #if  defined  (HAVE_LIBGRIB_API)
-    case FILETYPE_GRB2: { status = 1; break; }
+    case CDI_FILETYPE_GRB2: { status = 1; break; }
 #endif
 #endif
 #if  defined  (HAVE_LIBNETCDF)
-    case FILETYPE_NC:   { status = 1; break; }
+    case CDI_FILETYPE_NC:   { status = 1; break; }
 #if  defined  (HAVE_NETCDF2)
-    case FILETYPE_NC2:  { status = 1; break; }
+    case CDI_FILETYPE_NC2:  { status = 1; break; }
 #endif
 #if  defined  (HAVE_NETCDF4)
-    case FILETYPE_NC4:  { status = 1; break; }
-    case FILETYPE_NC4C: { status = 1; break; }
+    case CDI_FILETYPE_NC4:  { status = 1; break; }
+    case CDI_FILETYPE_NC4C: { status = 1; break; }
 #endif
 #endif
     default: { status = 0; break; }
@@ -237,9 +238,9 @@ void cdiSetChunk(const char *chunkAlgo)
   //size_t len = strlen(chunkAlgo);
   int algo = -1;
 
-  if      ( strcmp("auto",  chunkAlgo)   == 0 ) algo = CHUNK_AUTO;
-  else if ( strcmp("grid",  chunkAlgo)   == 0 ) algo = CHUNK_GRID;
-  else if ( strcmp("lines", chunkAlgo)   == 0 ) algo = CHUNK_LINES;
+  if      ( strcmp("auto",  chunkAlgo)   == 0 ) algo = CDI_CHUNK_AUTO;
+  else if ( strcmp("grid",  chunkAlgo)   == 0 ) algo = CDI_CHUNK_GRID;
+  else if ( strcmp("lines", chunkAlgo)   == 0 ) algo = CDI_CHUNK_LINES;
   /*
   else if ( (pch = strstr(chunkAlgo,"x")) != 0 )
     {
@@ -297,6 +298,9 @@ void cdiInitialize(void)
       value = cdiGetenvInt("CDI_SORTNAME");
       if ( value >= 0 ) cdiSortName = (int) value;
 
+      value = cdiGetenvInt("CDI_SORTPARAM");
+      if ( value >= 0 ) cdiSortParam = (int) value;
+
       value = cdiGetenvInt("CDI_HAVE_MISSVAL");
       if ( value >= 0 ) cdiHaveMissval = (int) value;
 
@@ -420,6 +424,7 @@ void cdiDefGlobal(const char *string, int val)
   if      ( strcmp(string, "REGULARGRID")      == 0 ) cdiDataUnreduced = val;
   else if ( strcmp(string, "GRIBAPI_DEBUG")    == 0 ) cdiGribApiDebug = val;
   else if ( strcmp(string, "SORTNAME")         == 0 ) cdiSortName = val;
+  else if ( strcmp(string, "SORTPARAM")        == 0 ) cdiSortParam = val;
   else if ( strcmp(string, "HAVE_MISSVAL")     == 0 ) cdiHaveMissval = val;
   else if ( strcmp(string, "NC_CHUNKSIZEHINT") == 0 ) cdiNcChunksizehint = val;
   else if ( strcmp(string, "CMOR_MODE")        == 0 ) CDI_cmor_mode = val;
