@@ -2,7 +2,7 @@
 #define _CDI_INT_H
 
 #if defined (HAVE_CONFIG_H)
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <assert.h>
@@ -13,8 +13,12 @@
 #include <math.h>
 #include <sys/types.h>
 
+#include "cdi.h"
+
 /* dummy use of unused parameters to silence compiler warnings */
+#ifndef UNUSED
 #define  UNUSED(x) (void)x
+#endif
 
 #ifndef strdupx
 #ifndef strdup
@@ -26,7 +30,7 @@ char *strdup(const char *s);
 ({					      	  \
    const char *__old = (s);			  \
    size_t __len = strlen(__old) + 1;		  \
-   char *__new = (char *) malloc(__len);	  \
+   char *__new = (char *) Malloc(__len);	  \
    (char *) memcpy(__new, __old, __len);	  \
 })
 */
@@ -65,7 +69,6 @@ char *strdup(const char *s);
 #  include "resource_handle.h"
 #endif
 
-#include "cdi.h"
 
 #define check_parg(arg)  if ( arg == 0 ) Warning("Argument '" #arg "' not allocated!")
 
@@ -95,7 +98,6 @@ char *strdup(const char *s);
 #  define IS_EQUAL(x,y)     (!IS_NOT_EQUAL(x,y))
 #endif
 
-
 #define  FALSE  0
 #define  TRUE   1
 
@@ -127,6 +129,18 @@ typedef struct
 }
 Record;
 
+/* data structure specifying tile-related meta-data. structure
+ * contains "-1" if this is no tile-variable. */
+typedef struct {
+  int
+    tileindex,
+    totalno_of_tileattr_pairs,
+    tileClassification,
+    numberOfTiles,
+    numberOfAttributes,
+    attribute;
+} var_tile_t;
+
 
 typedef struct
 {
@@ -137,19 +151,20 @@ typedef struct
   int       ilevel;
   int       ilevel2;
   int       ltype;
-  int       tsteptype;
+  short     tsteptype;
   short     used;
   short     varID;
   short     levelID;
   char      varname[32]; /* needed for grib decoding with GRIB_API */
+  var_tile_t tiles;      /* tile-related meta-data, currently for GRIB-API only. */
 }
 record_t;
 
 
 typedef struct {
   record_t *records;
-  int       recordSize;  /* number of allocated records           */
   int      *recIDs;      /* IDs of non constant records           */
+  int       recordSize;  /* number of allocated records           */
   int       nrecs;       /* number of used records                */
                          /* tsID=0 nallrecs                       */
                          /* tsID>0 number of non constant records */
@@ -163,16 +178,24 @@ tsteps_t;
 
 
 typedef struct {
-  int       ncvarid;
   int       nlevs;
-  int      *level;       /* record IDs */
-  int      *lindex;      /* level index */
-  int       defmiss;     /* TRUE if missval is defined in file */
+  int       subtypeIndex; /* corresponding tile in subtype_t structure (subtype->self) */
+  int      *recordID;     /* record IDs: [nlevs] */
+  int      *lindex;       /* level index */
+} sleveltable_t;
 
-  int       isUsed;
-  int       gridID;
-  int       zaxisID;
-  int       tsteptype;   /* TSTEP_* */
+
+typedef struct {
+  int            ncvarid;
+  int            subtypeSize;
+  sleveltable_t *recordTable; // record IDs for each subtype
+  bool           defmiss;     // true: if missval is defined in file
+  bool           isUsed;
+
+  int            gridID;
+  int            zaxisID;
+  int            tsteptype;   // TSTEP_*
+  int            subtypeID;   // subtype ID, e.g. for tile-related meta-data (currently for GRIB-API only).
 }
 svarinfo_t;
 
@@ -185,6 +208,17 @@ typedef struct {
 }
 VCT;
 
+#ifdef HAVE_LIBNETCDF
+typedef struct {
+  int gridID;
+  int xdimID;
+  int ydimID;
+  int xvarID;
+  int yvarID;
+  int avarID;
+}
+ncgrid_t;
+#endif
 
 typedef struct {
   int         self;
@@ -194,12 +228,12 @@ typedef struct {
   int         byteorder;
   int         fileID;
   int         filemode;
+  int         nrecs;        /* number of records                  */
   off_t       numvals;
   char       *filename;
   Record     *record;
-  int         nrecs;        /* number of records                  */
-  int         nvars;        /* number of variables                */
   svarinfo_t *vars;
+  int         nvars;        /* number of variables                */
   int         varsAllocated;
   int         curTsID;      /* current timestep ID */
   int         rtsteps;      /* number of tsteps accessed       */
@@ -210,52 +244,91 @@ typedef struct {
   basetime_t  basetime;
   int         ncmode;
   int         vlistID;
-  int         xdimID[MAX_GRIDS_PS];	//Warning: synchronous array to vlist_to_pointer(vlistID)->gridIDs
-  int         ydimID[MAX_GRIDS_PS];	//Warning: synchronous array to vlist_to_pointer(vlistID)->gridIDs
+#ifdef HAVE_LIBNETCDF
+  ncgrid_t    ncgrid[MAX_GRIDS_PS];
   int         zaxisID[MAX_ZAXES_PS];	//Warning: synchronous array to vlist_to_pointer(vlistID)->zaxisIDs
-  int         ncxvarID[MAX_GRIDS_PS];
-  int         ncyvarID[MAX_GRIDS_PS];
-  int         ncavarID[MAX_GRIDS_PS];
+  int         nczvarID[MAX_ZAXES_PS];
+  VCT         vct;
+#endif
   int         historyID;
   int         globalatts;
   int         localatts;
-  VCT         vct;
   int         unreduced;
-  int         sortname;
   int         have_missval;
   int         comptype;      // compression type
   int         complevel;     // compression level
+  bool        sortname;
+  bool        sortparam;
 #if defined (GRIBCONTAINER2D)
   void      **gribContainers;
 #else
   void       *gribContainers;
 #endif
-  int         vlistIDorig;
-  /* only used by MPI-parallelized version of library */
-  int       ownerRank;    // MPI rank of owner process
-  /* ---------------------------------- */
-  /* Local change: 2013-02-18, FP (DWD) */
-  /* ---------------------------------- */
 
   void *gh; // grib handle
 }
 stream_t;
 
 
+/* Length of optional keyword/value pair list */
+#define MAX_OPT_GRIB_ENTRIES 500
+
+enum cdi_convention {CDI_CONVENTION_ECHAM, CDI_CONVENTION_CF};
+
+/* Data type specification for optional key/value pairs (GRIB) */
+typedef enum {
+  t_double = 0,
+  t_int    = 1
+} key_val_pair_datatype;
+
+/* Data structure holding optional key/value pairs for GRIB */
+typedef struct
+{
+  char*                  keyword;        /* keyword string */
+  int                    update;
+  key_val_pair_datatype  data_type;      /* data type of this key/value pair */
+  double                 dbl_val;        /* double value (data_type == t_double) */
+  int                    int_val;        /* integer value (data_type == t_int) */
+  int                    subtype_index;  /* tile index for this key-value pair */
+} opt_key_val_pair_t;
+
+//enum for differenciating between the different times that we handle
+typedef enum {
+  kCdiTimeType_referenceTime,
+  kCdiTimeType_startTime,
+  kCdiTimeType_endTime
+} CdiTimeType;
+
+
+#define  CDI_FILETYPE_UNDEF          -1   /* Unknown/not yet defined file type */
+
+
 extern int CDI_Debug;      /* If set to 1, debuggig (default 0)            */
+extern int CDI_Recopt;
 extern int cdiGribApiDebug;
 extern double cdiDefaultMissval;
 extern int cdiDefaultInstID;
 extern int cdiDefaultModelID;
 extern int cdiDefaultTableID;
 extern int cdiDefaultLeveltype;
+extern int cdiDefaultCalendar;
 //extern int cdiNcMissingValue;
 extern int cdiNcChunksizehint;
 extern int cdiChunkType;
 extern int cdiSplitLtype105;
 extern int cdiDataUnreduced;
 extern int cdiSortName;
+extern int cdiSortParam;
 extern int cdiHaveMissval;
+extern int cdiIgnoreAttCoordinates;
+extern int cdiIgnoreValidRange;
+extern int cdiSkipRecords;
+extern int cdiConvention;
+extern int cdiInventoryMode;
+extern int CDI_Version_Info;
+extern int CDI_cmor_mode;
+extern size_t CDI_netcdf_hdr_pad;
+extern bool CDI_netcdf_lazy_grid_load;
 extern int STREAM_Debug;
 
 
@@ -266,7 +339,7 @@ extern const resOps streamOps;
 static inline stream_t *
 stream_to_pointer(int idx)
 {
-  return reshGetVal(idx, &streamOps);
+  return (stream_t *)reshGetVal(idx, &streamOps);
 }
 
 static inline void
@@ -280,8 +353,6 @@ int     streamInqFileID(int streamID);
 
 void    gridDefHasDims(int gridID, int hasdims);
 int     gridInqHasDims(int gridID);
-const char *gridNamePtr(int gridtype);
-char   *zaxisNamePtr(int leveltype);
 int     zaxisInqLevelID(int zaxisID, double level);
 
 void    streamCheckID(const char *caller, int streamID);
@@ -290,8 +361,10 @@ void    streamDefineTaxis(int streamID);
 
 int     streamsNewEntry(int filetype);
 void    streamsInitEntry(int streamID);
-void    cdiStreamSetupVlist(stream_t *streamptr, int vlistID, int vlistIDorig);
-int     stream_new_var(stream_t *streamptr, int gridID, int zaxisID);
+void    cdiStreamSetupVlist(stream_t *streamptr, int vlistID);
+/* default implementation of the overridable function */
+void    cdiStreamSetupVlist_(stream_t *streamptr, int vlistID);
+int     stream_new_var(stream_t *streamptr, int gridID, int zaxisID, int tilesetID);
 
 int     tstepsNewEntry(stream_t *streamptr);
 
@@ -303,6 +376,8 @@ void    vlist_check_contents(int vlistID);
 
 void    cdi_create_records(stream_t *streamptr, int tsID);
 
+void streamFCopyRecord(stream_t *streamptr2, stream_t *streamptr1, const char *container_name);
+
 int     recordNewEntry(stream_t *streamptr, int tsID);
 
 void    cdiCreateTimesteps(stream_t *streamptr);
@@ -318,22 +393,12 @@ int     cdiInqAccesstype(int streamID);
 
 int     getByteswap(int byteorder);
 
-void cdiStreamGetIndexList(unsigned numIDs, int IDs[numIDs]);
-
+void cdiStreamGetIndexList(unsigned numIDs, int IDs[]);
 
 void  cdiInitialize(void);
 
-void uuid2str(const unsigned char *uuid, char *uuidstr);
-int str2uuid(const char *uuidstr, unsigned char *uuid);
-
-static inline int cdiUUIDIsNull(const unsigned char uuid[CDI_UUID_SIZE])
-{
-  static const unsigned char uuid_nil[CDI_UUID_SIZE];
-  return !memcmp(uuid, uuid_nil, CDI_UUID_SIZE);
-}
-
-char* cdiEscapeSpaces(const char* string);
-char* cdiUnescapeSpaces(const char* string, const char** outStringEnd);
+char *cdiEscapeSpaces(const char *string);
+char *cdiUnescapeSpaces(const char *string, const char **outStringEnd);
 
 #define CDI_UNIT_PA   1
 #define CDI_UNIT_HPA  2
@@ -344,25 +409,30 @@ char* cdiUnescapeSpaces(const char* string, const char** outStringEnd);
 
 struct streamAssoc
 {
-  int streamID, vlistID, vlistIDorig;
+  int streamID, vlistID;
 };
 
 struct streamAssoc
-streamUnpack(char * unpackBuffer, int unpackBufferSize,
-             int * unpackBufferPos, int originNamespace, void *context);
+streamUnpack(char *unpackBuffer, int unpackBufferSize,
+             int *unpackBufferPos, int originNamespace, void *context);
 
 int
-cdiStreamOpenDefaultDelegate(const char *filename, const char *filemode,
+cdiStreamOpenDefaultDelegate(const char *filename, char filemode,
                              int filetype, stream_t *streamptr,
                              int recordBufIsToBeCreated);
 
+int
+streamOpenID(const char *filename, char filemode, int filetype,
+             int resH);
+
 void
 cdiStreamDefVlist_(int streamID, int vlistID);
+
+int
+cdiStreamWriteVar_(int streamID, int varID, int memtype, const void *data, int nmiss);
+
 void
-cdiStreamWriteVar_(int streamID, int varID, int memtype, const void *data,
-                   int nmiss);
-void
-cdiStreamwriteVarChunk_(int streamID, int varID, int memtype,
+cdiStreamWriteVarChunk_(int streamID, int varID, int memtype,
                         const int rect[][2], const void *data, int nmiss);
 void
 cdiStreamCloseDefaultDelegate(stream_t *streamptr,
@@ -372,13 +442,35 @@ int cdiStreamDefTimestep_(stream_t *streamptr, int tsID);
 
 void cdiStreamSync_(stream_t *streamptr);
 
-char *cdiUnitNamePtr(int cdi_unit);
+const char *cdiUnitNamePtr(int cdi_unit);
+
+enum {
+  /* 8192 is known to work on most systems (4096 isn't on Alpha) */
+  commonPageSize = 8192,
+};
+
+size_t cdiGetPageSize(bool largePageAlign);
 
 void zaxisGetIndexList(int nzaxis, int *zaxisIndexList);
 
 void zaxisDefLtype2(int zaxisID, int ltype2);
 int  zaxisInqLtype2(int zaxisID);
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+// functions used in CDO !!!
+
+void cdiDefTableID(int tableID);
+
+void gridGenXvals(int xsize, double xfirst, double xlast, double xinc, double *xvals);
+void gridGenYvals(int gridtype, int ysize, double yfirst, double ylast, double yinc, double *yvals);
+
+#if defined (__cplusplus)
+}
+#endif
+
 #endif  /* _CDI_INT_H */
 /*
  * Local Variables:
