@@ -16,6 +16,7 @@
 #define  GRIB1_LTYPE_ATMOSPHERE           10
 #define  GRIB1_LTYPE_99                   99
 #define  GRIB1_LTYPE_ISOBARIC            100
+#define  GRIB1_LTYPE_ISOBARIC_PA         210
 #define  GRIB1_LTYPE_MEANSEA             102
 #define  GRIB1_LTYPE_ALTITUDE            103
 #define  GRIB1_LTYPE_HEIGHT              105
@@ -32,7 +33,6 @@
 #define  GRIB1_LTYPE_SEDIMENT_BOTTOM_TA  164  /* Bottom Of Thermally Active Sediment Layer             */
 #define  GRIB1_LTYPE_SEDIMENT_BOTTOM_TW  165  /* Bottom Of Sediment Layer Penetrated By Thermal Wave   */
 #define  GRIB1_LTYPE_MIX_LAYER           166  /* Mixing Layer                                          */
-#define  GRIB1_LTYPE_99_MARGIN          1000
 
 /* GRIB1 Data representation type (Grid Type) [Table 6] */
 #define  GRIB1_GTYPE_LATLON                0  /*  latitude/longitude                                   */
@@ -57,13 +57,14 @@
 /*
  *  Macros for the product definition section ( Section 1 )
  */
-#define  ISEC1_TABLE4_MINUTE    0
-#define  ISEC1_TABLE4_HOUR      1
-#define  ISEC1_TABLE4_DAY       2
-#define  ISEC1_TABLE4_3HOURS   10
-#define  ISEC1_TABLE4_6HOURS   11
-#define  ISEC1_TABLE4_12HOURS  12
-#define  ISEC1_TABLE4_QUARTER  13
+#define  ISEC1_TABLE4_MINUTE      0
+#define  ISEC1_TABLE4_HOUR        1
+#define  ISEC1_TABLE4_DAY         2
+#define  ISEC1_TABLE4_3HOURS     10
+#define  ISEC1_TABLE4_6HOURS     11
+#define  ISEC1_TABLE4_12HOURS    12
+#define  ISEC1_TABLE4_QUARTER    13
+#define  ISEC1_TABLE4_30MINUTES  14
 
 
 #define  ISEC1_CodeTable            (isec1[ 0])  /*  Version number of code table                 */
@@ -174,6 +175,9 @@
 #define  ISEC4_NumNonMissValues     (isec4[20])  /* Number of non-missing values                  */
 
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 
 void  gribFixZSE(int flag);     /* 1: Fix ZeroShiftError of simple packed spherical harmonics */
@@ -187,11 +191,11 @@ void  gribSetValueCheck(int vcheck);
 
 void  gribExSP(int *isec0, int *isec1, int *isec2, float *fsec2, int *isec3,
                float *fsec3, int *isec4, float *fsec4, int klenp, int *kgrib,
-               int kleng, int *kword, char *hoper, int *kret);
+               int kleng, int *kword, const char *hoper, int *kret);
 
 void  gribExDP(int *isec0, int *isec1, int *isec2, double *fsec2, int *isec3,
                double *fsec3, int *isec4, double *fsec4, int klenp, int *kgrib,
-               int kleng, int *kword, char *hoper, int *kret);
+               int kleng, int *kword, const char *hoper, int *kret);
 
 
 const char *cgribexLibraryVersion(void);
@@ -222,13 +226,7 @@ void  gribPrintBDS(int nrec, long recpos, long recsize, unsigned char *gribbuffe
 void  gribCheck1(int nrec, long recpos, long recsize, unsigned char *gribbuffer);
 void  gribRepair1(int nrec, long recsize, unsigned char *gribbuffer);
 
-int grib1Sections(unsigned char *gribbuffer, long recsize, unsigned char **pdsp,
-		  unsigned char **gdsp, unsigned char **bmsp, unsigned char **bdsp);
-int grib2Sections(unsigned char *gribbuffer, long recsize, unsigned char **idsp,
-		  unsigned char **lusp, unsigned char **gdsp, unsigned char **pdsp,
-		  unsigned char **drsp, unsigned char **bmsp, unsigned char **bdsp);
-
-int   gribGetZip(long recsize, unsigned char *gribbuffer, long *urecsize);
+int   gribGetZip(size_t recsize, unsigned char *gribbuffer, size_t *urecsize);
 
 int   gribBzip(unsigned char *dbuf, long dbufsize, unsigned char *sbuf, long sbufsize);
 int   gribZip(unsigned char *dbuf, long dbufsize, unsigned char *sbuf, long sbufsize);
@@ -240,15 +238,21 @@ void  gribClose(int fileID);
 int   gribRead(int fileID, unsigned char *buffer, size_t *buffersize);
 int   gribWrite(int fileID, unsigned char *buffer, size_t buffersize);
 off_t gribGetPos(int fileID);
-int   gribGetSize(int fileID);
+size_t gribGetSize(int fileID);
 int   gribCheckSeek(int fileID, long *offset, int *version);
 int   gribFileSeek(int fileID, long *offset);
-int   gribReadSize(int fileID);
+size_t gribReadSize(int fileID);
 int   gribVersion(unsigned char *buffer, size_t buffersize);
 
-int   gribGinfo(off_t recpos, long recsize, unsigned char *gribbuffer, int *intnum, float *fltnum, off_t *bignum);
+int   grib_info_for_grads(off_t recpos, long recsize, unsigned char *gribbuffer, int *intnum, float *fltnum, off_t *bignum);
 
-double calculate_pfactor(const double* spectralField, long fieldTruncation, long subsetTruncation);
+double calculate_pfactor_float(const float* spectralField, long fieldTruncation, long subsetTruncation);
+double calculate_pfactor_double(const double* spectralField, long fieldTruncation, long subsetTruncation);
+
+
+#if defined (__cplusplus)
+}
+#endif
 
 #endif  /* _CGRIBEX_H */ 
 
