@@ -3,16 +3,17 @@
 #endif
 
 #include <assert.h>
-#include <stdio.h>
-#include <stdarg.h>
-#include <string.h>
-#include <errno.h>
 #include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <stdint.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/time.h>  // gettimeofday()
-#include <fcntl.h>
 
 #include "dmemory.h"
 #include "error.h"
@@ -34,7 +35,7 @@ char *strdup(const char *s);
 ({                                                \
    const char *__old = (s);                       \
    size_t __len = strlen(__old) + 1;              \
-   char *__new = (char *) malloc(__len);          \
+   char *__new = (char *) Malloc(__len);          \
    (char *) memcpy(__new, __old, __len);          \
 })
 */
@@ -91,15 +92,15 @@ typedef struct
   int        eof;            /* end of file flag              */
   int        fd;             /* file descriptor used for read */
   FILE      *fp;             /* FILE pointer used for write   */
-  int        mode;           /* file access mode              */
   char      *name;           /* file name                     */
   off_t      size;           /* file size                     */
   off_t      position;       /* file position                 */
   long       access;         /* file access                   */
   off_t      byteTrans;      /*                               */
   size_t     blockSize;      /* file block size               */
-  int        type;           /* file type ( 1:open 2:fopen )  */
-  int        bufferType;     /* buffer type ( 1:std 2:mmap )  */
+  int        mode;           /* file access mode              */
+  short      type;           /* file type ( 1:open 2:fopen )  */
+  short      bufferType;     /* buffer type ( 1:std 2:mmap )  */
   size_t     bufferSize;     /* file buffer size              */
   size_t     mappedSize;     /* mmap buffer size              */
   char      *buffer;         /* file buffer                   */
@@ -134,10 +135,10 @@ static int FileInfo  = FALSE;
 
 static size_t FileBufferSizeMin = MIN_BUF_SIZE;
 static long   FileBufferSizeEnv = -1;
-static int    FileBufferTypeEnv =  0;
+static short  FileBufferTypeEnv =  0;
 
-static int    FileTypeRead  = FILE_TYPE_OPEN;
-static int    FileTypeWrite = FILE_TYPE_FOPEN;
+static short  FileTypeRead  = FILE_TYPE_OPEN;
+static short  FileTypeWrite = FILE_TYPE_FOPEN;
 static int    FileFlagWrite = 0;
 
 static int    FILE_Debug = 0;   /* If set to 1, debugging */
@@ -152,7 +153,7 @@ static void file_table_print(void);
 #define  LIBVERSION      1.8.2
 #define  XSTRING(x)	 #x
 #define  STRING(x) 	 XSTRING(x)
-const char file_libvers[] = STRING(LIBVERSION) " of "__DATE__" "__TIME__;
+static const char file_libvers[] = STRING(LIBVERSION) " of " __DATE__ " " __TIME__;
 
 /*
   21/05/2004  1.3.2 set min I/O Buffersize to 128k
@@ -189,7 +190,7 @@ void file_list_new(void)
 {
   assert(_fileList == NULL);
 
-  _fileList = (filePtrToIdx *) malloc(_file_max*sizeof(filePtrToIdx));
+  _fileList = (filePtrToIdx *) Malloc((size_t)_file_max * sizeof (filePtrToIdx));
 }
 
 static
@@ -197,7 +198,7 @@ void file_list_delete(void)
 {
   if ( _fileList )
     {
-      free(_fileList);
+      Free(_fileList);
       _fileList = NULL;
     }
 }
@@ -306,7 +307,7 @@ bfile_t *file_new_entry(void)
 {
   bfile_t *fileptr;
 
-  fileptr = (bfile_t *) malloc(sizeof(bfile_t));
+  fileptr = (bfile_t *) Malloc(sizeof(bfile_t));
 
   if ( fileptr ) file_init_entry(fileptr);
 
@@ -322,7 +323,7 @@ void file_delete_entry(bfile_t *fileptr)
 
   FILE_LOCK();
 
-  free(fileptr);
+  Free(fileptr);
 
   _fileList[idx].next = _fileAvail;
   _fileList[idx].ptr  = 0;
@@ -341,16 +342,15 @@ const char *fileLibraryVersion(void)
 }
 
 
-#ifndef POSIXIO_DEFAULT_PAGESIZE
-#define POSIXIO_DEFAULT_PAGESIZE 4096
-#endif
-
 static
 int pagesize(void)
 {
 #if defined(_SC_PAGESIZE)
   return ((int) sysconf(_SC_PAGESIZE));
 #else
+#ifndef POSIXIO_DEFAULT_PAGESIZE
+#define POSIXIO_DEFAULT_PAGESIZE 4096
+#endif
   return ((int) POSIXIO_DEFAULT_PAGESIZE);
 #endif
 }
@@ -408,7 +408,7 @@ int fileSetBufferType(int fileID, int type)
 	{
 	case FILE_BUFTYPE_STD:
 	case FILE_BUFTYPE_MMAP:
-	  fileptr->bufferType = type;
+	  fileptr->bufferType = (short)type;
 	  break;
 	default:
 	  Error("File type %d not implemented!", type);
@@ -422,20 +422,6 @@ int fileSetBufferType(int fileID, int type)
   return (ret);
 }
 
-
-int fileGetBufferType(int fileID)
-{
-  bfile_t *fileptr;
-  int bufferType = 0;
-
-  fileptr = file_to_pointer(fileID);
-
-  if ( fileptr ) bufferType = fileptr->bufferType;
-
-  return (bufferType);
-}
-
-
 int fileFlush(int fileID)
 {
   bfile_t *fileptr;
@@ -486,20 +472,6 @@ int fileEOF(int fileID)
   return (retval);
 }
 
-
-int fileError(int fileID)
-{
-  bfile_t *fileptr;
-  int retval = 0;
-
-  fileptr = file_to_pointer(fileID);
-
-  if ( fileptr ) retval = (fileptr->flag & FILE_ERROR) != 0;
-
-  return (retval);
-}
-
-
 void fileRewind(int fileID)
 {
   fileSetPos(fileID, (off_t) 0, SEEK_SET);
@@ -531,7 +503,6 @@ off_t fileGetPos(int fileID)
 int fileSetPos(int fileID, off_t offset, int whence)
 {
   int status = 0;
-  off_t position;
   bfile_t *fileptr;
 
   fileptr = file_to_pointer(fileID);
@@ -549,7 +520,7 @@ int fileSetPos(int fileID, off_t offset, int whence)
     case SEEK_SET:
       if ( fileptr->mode == 'r' && fileptr->type == FILE_TYPE_OPEN )
 	{
-	  position = offset;
+	  off_t position = offset;
 	  fileptr->position = position;
 	  if ( position < fileptr->bufferStart || position > fileptr->bufferEnd )
 	    {
@@ -571,7 +542,7 @@ int fileSetPos(int fileID, off_t offset, int whence)
 
 		  fileptr->bufferPos = fileptr->bufferEnd + 1;
 		}
-	      fileptr->bufferCnt = fileptr->bufferEnd - position + 1;
+	      fileptr->bufferCnt = (size_t)(fileptr->bufferEnd - position) + 1;
 	      fileptr->bufferPtr = fileptr->buffer + position - fileptr->bufferStart;
 	    }
 	}
@@ -584,7 +555,7 @@ int fileSetPos(int fileID, off_t offset, int whence)
       if ( fileptr->mode == 'r' && fileptr->type == FILE_TYPE_OPEN )
 	{
 	  fileptr->position += offset;
-	  position = fileptr->position;
+	  off_t position = fileptr->position;
 	  if ( position < fileptr->bufferStart || position > fileptr->bufferEnd )
 	    {
 	      if ( fileptr->bufferType == FILE_BUFTYPE_STD )
@@ -605,7 +576,7 @@ int fileSetPos(int fileID, off_t offset, int whence)
 
 		  fileptr->bufferPos = fileptr->bufferEnd + 1;
 		}
-	      fileptr->bufferCnt -= offset;
+	      fileptr->bufferCnt -= (size_t)offset;
 	      fileptr->bufferPtr += offset;
 	    }
 	}
@@ -780,7 +751,7 @@ void file_initialize(void)
 	{
 	case FILE_TYPE_OPEN:
 	case FILE_TYPE_FOPEN:
-	  FileTypeRead = value;
+	  FileTypeRead = (short)value;
 	  break;
 	default:
 	  Warning("File type %d not implemented!", value);
@@ -794,13 +765,16 @@ void file_initialize(void)
 	{
 	case FILE_TYPE_OPEN:
 	case FILE_TYPE_FOPEN:
-	  FileTypeWrite = value;
+	  FileTypeWrite = (short)value;
 	  break;
 	default:
 	  Warning("File type %d not implemented!", value);
 	}
     }
 
+#if defined (O_NONBLOCK)
+  FileFlagWrite = O_NONBLOCK;
+#endif
   envString = getenv("FILE_FLAG_WRITE");
   if ( envString )
     {
@@ -823,7 +797,7 @@ void file_initialize(void)
 	{
 	case FILE_BUFTYPE_STD:
 	case FILE_BUFTYPE_MMAP:
-	  FileBufferTypeEnv = value;
+	  FileBufferTypeEnv = (short)value;
 	  break;
 	default:
 	  Warning("File buffer type %d not implemented!", value);
@@ -900,7 +874,7 @@ void file_set_buffer(bfile_t *fileptr)
     {
       if ( buffersize > 0 )
         {
-          fileptr->buffer = (char *) malloc(buffersize);
+          fileptr->buffer = (char *) Malloc(buffersize);
           if ( fileptr->buffer == NULL )
             SysError("Allocation of file buffer failed!");
         }
@@ -916,9 +890,8 @@ void file_set_buffer(bfile_t *fileptr)
 static
 int file_fill_buffer(bfile_t *fileptr)
 {
-  long nread;
+  ssize_t nread;
   int fd;
-  int ret;
   long offset = 0;
   off_t retseek;
 
@@ -942,25 +915,24 @@ int file_fill_buffer(bfile_t *fileptr)
 	}
       else
 	{
-	  nread = fileptr->bufferSize;
+          xassert(fileptr->bufferSize <= SIZE_MAX);
+	  nread = (ssize_t)fileptr->bufferSize;
 	  if ( (nread + fileptr->bufferPos) > fileptr->size )
 	    nread = fileptr->size - fileptr->bufferPos;
 
 	  if ( fileptr->buffer )
 	    {
+              int ret;
 	      ret = munmap(fileptr->buffer, fileptr->mappedSize);
-	      if ( ret == -1 )
-		SysError("munmap error for read %s", fileptr->name);
+	      if ( ret == -1 ) SysError("munmap error for read %s", fileptr->name);
 	      fileptr->buffer = NULL;
 	    }
 
-	  fileptr->mappedSize = (size_t) nread;
+	  fileptr->mappedSize = (size_t)nread;
 
-	  fileptr->buffer =
-            (char *) mmap(0, (size_t) nread, PROT_READ, MAP_SHARED, fd, fileptr->bufferPos);
+	  fileptr->buffer = (char*) mmap(NULL, (size_t) nread, PROT_READ, MAP_PRIVATE, fd, fileptr->bufferPos);
 
-	  if ( fileptr->buffer == (void *)-1 )
-	    SysError("mmap error for read %s", fileptr->name);
+	  if ( fileptr->buffer == MAP_FAILED ) SysError("mmap error for read %s", fileptr->name);
 
 	  offset = fileptr->position - fileptr->bufferPos;
 	}
@@ -972,7 +944,7 @@ int file_fill_buffer(bfile_t *fileptr)
       if ( retseek == (off_t)-1 )
 	SysError("lseek error at pos %ld file %s", (long) fileptr->bufferPos, fileptr->name);
 
-      nread = (long) read(fd, fileptr->buffer, fileptr->bufferSize);
+      nread = read(fd, fileptr->buffer, fileptr->bufferSize);
     }
 
   if ( nread <= 0 )
@@ -987,7 +959,7 @@ int file_fill_buffer(bfile_t *fileptr)
     }
 
   fileptr->bufferPtr = fileptr->buffer;
-  fileptr->bufferCnt = nread;
+  fileptr->bufferCnt = (size_t)nread;
 
   fileptr->bufferStart = fileptr->bufferPos;
   fileptr->bufferPos  += nread;
@@ -1010,7 +982,7 @@ int file_fill_buffer(bfile_t *fileptr)
 	Error("Internal problem with buffer handling. nread = %d offset = %d", nread, offset);
 
       fileptr->bufferPtr += offset;
-      fileptr->bufferCnt -= offset;
+      fileptr->bufferCnt -= (size_t)offset;
     }
 
   fileptr->bufferNumFill++;
@@ -1050,10 +1022,10 @@ size_t file_read_from_buffer(bfile_t *fileptr, void *ptr, size_t size)
   size_t offset = 0;
 
   if ( FILE_Debug )
-    Message("size = %ld  Cnt = %d", size, (int) fileptr->bufferCnt);
+    Message("size = %ld  Cnt = %ld", size, (long) fileptr->bufferCnt);
 
-  if ( ((int)fileptr->bufferCnt) < 0 )
-    Error("Internal problem. bufferCnt = %d", (int) fileptr->bufferCnt);
+  if ( ((long)fileptr->bufferCnt) < 0L )
+    Error("Internal problem. bufferCnt = %ld", (long) fileptr->bufferCnt);
 
   rsize = size;
 
@@ -1087,11 +1059,9 @@ size_t file_read_from_buffer(bfile_t *fileptr, void *ptr, size_t size)
 
 void fileSetBufferSize(int fileID, long buffersize)
 {
-  bfile_t *fileptr;
-
-  fileptr = file_to_pointer(fileID);
-
-  if ( fileptr ) fileptr->bufferSize = buffersize;
+  bfile_t *fileptr = file_to_pointer(fileID);
+  xassert(buffersize >= 0);
+  if ( fileptr ) fileptr->bufferSize = (size_t)buffersize;
 }
 
 /*
@@ -1207,8 +1177,8 @@ int fileClose_serial(int fileID)
 {
   char *name;
   int ret;
-  char *fbtname[] = {"unknown", "standard", "mmap"};
-  char *ftname[] = {"unknown", "open", "fopen"};
+  const char *fbtname[] = {"unknown", "standard", "mmap"};
+  const char *ftname[] = {"unknown", "open", "fopen"};
   bfile_t *fileptr = file_to_pointer(fileID);
   double rout = 0;
 
@@ -1233,8 +1203,10 @@ int fileClose_serial(int fileID)
       if ( fileptr->type == FILE_TYPE_FOPEN )
 	fprintf(stderr, " file pointer     : %p\n",  (void *) fileptr->fp);
       else
-        fprintf(stderr, " file descriptor  : %d\n",  fileptr->fd);
-
+        {
+          fprintf(stderr, " file descriptor  : %d\n",  fileptr->fd);
+          fprintf(stderr, " file flag        : %d\n", FileFlagWrite);
+        }
       fprintf(stderr, " file mode        : %c\n",  fileptr->mode);
 
       if ( sizeof(off_t) > sizeof(long) )
@@ -1261,8 +1233,7 @@ int fileClose_serial(int fileID)
 
       if ( fileptr->time_in_sec > 0 )
         {
-          rout = fileptr->byteTrans;
-          rout /= 1024.*1014.*fileptr->time_in_sec;
+          rout = (double)fileptr->byteTrans / (1024.*1024.*fileptr->time_in_sec);
         }
 
       fprintf(stderr, " wall time [s]    : %.2f\n", fileptr->time_in_sec);
@@ -1292,8 +1263,7 @@ int fileClose_serial(int fileID)
       if ( fileptr->buffer && fileptr->mappedSize )
 	{
 	  ret = munmap(fileptr->buffer, fileptr->mappedSize);
-	  if ( ret == -1 )
-	    SysError("munmap error for close %s", fileptr->name);
+	  if ( ret == -1 ) SysError("munmap error for close %s", fileptr->name);
 	  fileptr->buffer = NULL;
 	}
 #endif
@@ -1302,8 +1272,8 @@ int fileClose_serial(int fileID)
 	SysError("EOF returned for close of %s!", name);
     }
 
-  if ( fileptr->name )    free((void*) fileptr->name);
-  if ( fileptr->buffer )  free((void*) fileptr->buffer);
+  if ( fileptr->name )    Free((void*) fileptr->name);
+  if ( fileptr->buffer )  Free((void*) fileptr->buffer);
 
   file_delete_entry(fileptr);
 
@@ -1384,8 +1354,8 @@ size_t filePtrRead(void *vfileptr, void *restrict ptr, size_t size)
 	    }
 	}
 
-      fileptr->position  += nread;
-      fileptr->byteTrans += nread;
+      fileptr->position  += (off_t)nread;
+      fileptr->byteTrans += (off_t)nread;
       fileptr->access++;
     }
 
@@ -1424,8 +1394,8 @@ size_t fileRead(int fileID, void *restrict ptr, size_t size)
 
       if ( FileInfo ) fileptr->time_in_sec += file_time() - t_begin;
 
-      fileptr->position  += nread;
-      fileptr->byteTrans += nread;
+      fileptr->position  += (off_t)nread;
+      fileptr->byteTrans += (off_t)nread;
       fileptr->access++;
     }
 
@@ -1453,12 +1423,21 @@ size_t fileWrite(int fileID, const void *restrict ptr, size_t size)
       if ( fileptr->type == FILE_TYPE_FOPEN )
         nwrite = fwrite(ptr, 1, size, fileptr->fp);
       else
-        nwrite =  write(fileptr->fd, ptr, size);
+        {
+          ssize_t temp = write(fileptr->fd, ptr, size);
+          if (temp == -1)
+            {
+              perror("error writing to file");
+              nwrite = 0;
+            }
+          else
+            nwrite = (size_t)temp;
+        }
 
       if ( FileInfo ) fileptr->time_in_sec += file_time() - t_begin;
 
-      fileptr->position  += nwrite;
-      fileptr->byteTrans += nwrite;
+      fileptr->position  += (off_t)nwrite;
+      fileptr->byteTrans += (off_t)nwrite;
       fileptr->access++;
     }
 
