@@ -53,12 +53,10 @@ const char *gribapiLibraryVersionString(void)
 
 void gribContainersNew(stream_t * streamptr)
 {
-  int editionNumber = 2;
+  int editionNumber = (streamptr->filetype == CDI_FILETYPE_GRB) ? 1 : 2;
 
-  if ( streamptr->filetype == FILETYPE_GRB ) editionNumber = 1;
-  (void)editionNumber;
 #if  defined  (HAVE_LIBCGRIBEX)
-  if ( streamptr->filetype == FILETYPE_GRB )
+  if ( editionNumber == 1 )
     {
     }
   else
