@@ -276,7 +276,7 @@ static int getAvailabilityOfRelativeTimes(grib_handle* gh, bool* outHaveForecast
         return 0;
 
       //case 55 and case 40455 are the same: 55 is the proposed standard value, 40455 is the value in the local use range that is used by the dwd until the standard is updated.
-      case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 15: case 32: case 33: case 40: case 41: case 44: case 45: case 48: case 51: case 53: case 54: case 55: case 60: case 1000: case 1002: case 1100: case 40033: case 40455:
+      case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 15: case 32: case 33: case 40: case 41: case 44: case 45: case 48: case 51: case 53: case 54: case 55: case 56: case 60: case 1000: case 1002: case 1100: case 40033: case 40455: case 40456:
         *outHaveForecastTime = true, *outHaveTimeRange = false;
         return 0;
 
@@ -289,6 +289,7 @@ static int getAvailabilityOfRelativeTimes(grib_handle* gh, bool* outHaveForecast
     }
 }
 
+
 char* gribMakeTimeString(grib_handle* gh, CdiTimeType timeType)
 {
   //Get the parts of the reference date.
@@ -336,9 +337,10 @@ char* gribMakeTimeString(grib_handle* gh, CdiTimeType timeType)
   return makeDateString(&date);
 }
 
+
 int gribapiTimeIsFC(grib_handle *gh)
 {
-  if(gribEditionNumber(gh) <= 1) return true;
+  if (gribEditionNumber(gh) <= 1) return true;
 
   long sigofrtime;
   FAIL_ON_GRIB_ERROR(grib_get_long, gh, "significanceOfReferenceTime", &sigofrtime);
@@ -380,24 +382,26 @@ int gribapiGetTsteptype(grib_handle *gh)
         }
     }
 
-  return (tsteptype);
+  return tsteptype;
 }
 
+
 int gribGetDatatype(grib_handle* gribHandle)
 {
   int datatype;
   if(gribEditionNumber(gribHandle) > 1 && gribCheckString(gribHandle, "packingType", "grid_ieee"))
     {
-      datatype = gribCheckLong(gribHandle, "precision", 1) ? DATATYPE_FLT32 : DATATYPE_FLT64;
+      datatype = gribCheckLong(gribHandle, "precision", 1) ? CDI_DATATYPE_FLT32 : CDI_DATATYPE_FLT64;
     }
   else
     {
       long bitsPerValue;
-      datatype = (!grib_get_long(gribHandle, "bitsPerValue", &bitsPerValue) && bitsPerValue > 0 && bitsPerValue <= 32) ? (int)bitsPerValue : DATATYPE_PACK;
+      datatype = (!grib_get_long(gribHandle, "bitsPerValue", &bitsPerValue) && bitsPerValue > 0 && bitsPerValue <= 32) ? (int)bitsPerValue : CDI_DATATYPE_PACK;
     }
   return datatype;
 }
 
+
 int gribapiGetParam(grib_handle *gh)
 {
   long pdis, pcat, pnum;
@@ -416,6 +420,7 @@ int gribapiGetParam(grib_handle *gh)
   return cdiEncodeParam((int)pnum, (int)pcat, (int)pdis);
 }
 
+
 int gribapiGetGridType(grib_handle *gh)
 {
   int gridtype = GRID_GENERIC;
@@ -424,12 +429,10 @@ int gribapiGetGridType(grib_handle *gh)
       case  GRIB2_GTYPE_LATLON:
         gridtype = ( gribGetLong(gh, "Ni") == (long) GRIB_MISSING_LONG ) ? GRID_GENERIC : GRID_LONLAT;
         break;
-
       case  GRIB2_GTYPE_GAUSSIAN:
         gridtype = ( gribGetLong(gh, "Ni") == (long) GRIB_MISSING_LONG ) ? GRID_GAUSSIAN_REDUCED : GRID_GAUSSIAN;
         break;
-
-      case GRIB2_GTYPE_LATLON_ROT:   gridtype = GRID_LONLAT; break;
+      case GRIB2_GTYPE_LATLON_ROT:   gridtype = GRID_PROJECTION; break;
       case GRIB2_GTYPE_LCC:          gridtype = GRID_LCC; break;
       case GRIB2_GTYPE_SPECTRAL:     gridtype = GRID_SPECTRAL; break;
       case GRIB2_GTYPE_GME:          gridtype = GRID_GME; break;
@@ -450,6 +453,7 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
 {
   long editionNumber = gribEditionNumber(gh);
   int gridtype = gribapiGetGridType(gh);
+  int projtype = (gridtype == GRID_PROJECTION && gribapiGetIsRotated(gh)) ? CDI_PROJ_RLL : CDI_UNDEFID;
   /*
   if ( streamptr->unreduced && gridtype == GRID_GAUSSIAN_REDUCED )
     {
@@ -466,308 +470,251 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
   long numberOfPoints;
   FAIL_ON_GRIB_ERROR(grib_get_long, gh, "numberOfPoints", &numberOfPoints);
 
-  switch (gridtype)
+  long lpar;
+
+  if ( gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_RLL )
     {
-    case GRID_LONLAT:
-    case GRID_GAUSSIAN:
+      long nlon, nlat;
+      FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Ni", &nlon);
+      FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Nj", &nlat);
+
+      if ( gridtype == GRID_GAUSSIAN )
+        {
+          FAIL_ON_GRIB_ERROR(grib_get_long, gh, "numberOfParallelsBetweenAPoleAndTheEquator", &lpar);
+          grid->np = (int)lpar;
+        }
+
+      if ( numberOfPoints != nlon*nlat )
+        Error("numberOfPoints (%ld) and gridSize (%ld) differ!", numberOfPoints, nlon*nlat);
+
+      grid->size  = (int)numberOfPoints;
+      grid->x.size = (int)nlon;
+      grid->y.size = (int)nlat;
+      grid->x.inc  = 0;
+      grid->y.inc  = 0;
+      grid->x.flag = 0;
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfFirstGridPointInDegrees", &grid->x.first);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfLastGridPointInDegrees",  &grid->x.last);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfFirstGridPointInDegrees",  &grid->y.first);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfLastGridPointInDegrees",   &grid->y.last);
+      if ( nlon > 1 )
+        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "iDirectionIncrementInDegrees", &grid->x.inc);
+      if ( gridtype == GRID_LONLAT && nlat > 1 )
+        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "jDirectionIncrementInDegrees", &grid->y.inc);
+
+      if ( grid->x.inc < -999 || grid->x.inc > 999 ) grid->x.inc = 0;
+      if ( grid->y.inc < -999 || grid->y.inc > 999 ) grid->y.inc = 0;
+
+      if ( grid->y.inc > 0 && grid->y.first > grid->y.last ) grid->y.inc = -grid->y.inc;
+
+      /* if ( IS_NOT_EQUAL(grid->x.first, 0) || IS_NOT_EQUAL(grid->x.last, 0) ) */
       {
-        long lpar;
-        FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Ni", &lpar);
-        /* FIXME: assert(lpar <= INT_MAX && lpar >= INT_MIN) */
-        int nlon = (int)lpar;
-        FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Nj", &lpar);
-        /* FIXME: assert(lpar <= INT_MAX && lpar >= INT_MIN) */
-        int nlat = (int)lpar;
-
-        if ( gridtype == GRID_GAUSSIAN )
+        if ( grid->x.size > 1 )
           {
-            FAIL_ON_GRIB_ERROR(grib_get_long, gh, "numberOfParallelsBetweenAPoleAndTheEquator", &lpar);
-            grid->np = (int)lpar;
-          }
+            if ( (grid->x.first >= grid->x.last) && (grid->x.first >= 180) ) grid->x.first -= 360;
 
-        if ( numberOfPoints != nlon*nlat )
-          Error("numberOfPoints (%ld) and gridSize (%d) differ!", numberOfPoints, nlon*nlat);
-
-        /* FIXME: assert(numberOfPoints <= INT_MAX && numberOfPoints >= INT_MIN) */
-        grid->size  = (int)numberOfPoints;
-        grid->xsize = nlon;
-        grid->ysize = nlat;
-        grid->xinc  = 0;
-        grid->yinc  = 0;
-        grid->xdef  = 0;
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfFirstGridPointInDegrees", &grid->xfirst);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfLastGridPointInDegrees",  &grid->xlast);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfFirstGridPointInDegrees",  &grid->yfirst);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfLastGridPointInDegrees",   &grid->ylast);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "iDirectionIncrementInDegrees", &grid->xinc);
-        if ( gridtype == GRID_LONLAT )
-          FAIL_ON_GRIB_ERROR(grib_get_double, gh, "jDirectionIncrementInDegrees", &grid->yinc);
-
-        if ( grid->xinc < -999 || grid->xinc > 999 ) grid->xinc = 0;
-        if ( grid->yinc < -999 || grid->yinc > 999 ) grid->yinc = 0;
-
-        /* if ( IS_NOT_EQUAL(grid->xfirst, 0) || IS_NOT_EQUAL(grid->xlast, 0) ) */
-          {
-            if ( grid->xsize > 1 )
+            if ( editionNumber <= 1 )
               {
-                if ( (grid->xfirst >= grid->xlast) && (grid->xfirst >= 180) ) grid->xfirst -= 360;
-
-                if ( editionNumber <= 1 )
+                /* correct xinc if necessary */
+                if ( IS_EQUAL(grid->x.first, 0) && grid->x.last > 354 )
                   {
-                    /* correct xinc if necessary */
-                    if ( IS_EQUAL(grid->xfirst, 0) && grid->xlast > 354 )
+                    double xinc = 360. / grid->x.size;
+                    if ( fabs(grid->x.inc-xinc) > 0.0 )
                       {
-                        double xinc = 360. / grid->xsize;
-
-                        if ( fabs(grid->xinc-xinc) > 0.0 )
-                          {
-                            grid->xinc = xinc;
-                            if ( CDI_Debug ) Message("set xinc to %g", grid->xinc);
-                          }
+                        grid->x.inc = xinc;
+                        if ( CDI_Debug ) Message("set xinc to %g", grid->x.inc);
                       }
                   }
               }
-            grid->xdef = 2;
           }
-        grid->ydef = 0;
-        /* if ( IS_NOT_EQUAL(grid->yfirst, 0) || IS_NOT_EQUAL(grid->ylast, 0) ) */
-          {
-            if ( grid->ysize > 1 )
-              {
-                if ( editionNumber <= 1 )
-                  {
-                  }
-              }
-            grid->ydef = 2;
-          }
-        break;
+        grid->x.flag = 2;
       }
-    case GRID_GAUSSIAN_REDUCED:
+      grid->y.flag = 0;
+      /* if ( IS_NOT_EQUAL(grid->y.first, 0) || IS_NOT_EQUAL(grid->y.last, 0) ) */
       {
-        size_t dummy;
-        long *pl;
-
-        long lpar;
-        FAIL_ON_GRIB_ERROR(grib_get_long, gh, "numberOfParallelsBetweenAPoleAndTheEquator", &lpar);
-        /* FIXME: assert(lpar <= INT_MAX && lpar >= INT_MIN) */
-        grid->np = (int)lpar;
-
-        FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Nj", &lpar);
-        /* FIXME: assert(lpar <= INT_MAX && lpar >= INT_MIN) */
-        int nlat = (int)lpar;
-
-        /* FIXME: assert(numberOfPoints <= INT_MAX && numberOfPoints >= INT_MIN) */
-        grid->size   = (int)numberOfPoints;
-
-        grid->rowlon = (int *) Malloc((size_t)nlat * sizeof (int));
-        pl          = (long *) Malloc((size_t)nlat * sizeof (long));
-        dummy       = (size_t)nlat;
-        FAIL_ON_GRIB_ERROR(grib_get_long_array, gh, "pl", pl, &dummy);
-        /* FIXME: assert(pl[i] >= INT_MIN && pl[i] <= INT_MIN) */
-        for (int i = 0; i < nlat; ++i ) grid->rowlon[i] = (int)pl[i];
-        Free(pl);
-
-        grid->ysize  = nlat;
-        grid->xinc   = 0;
-        grid->yinc   = 0;
-        grid->xdef   = 0;
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfFirstGridPointInDegrees", &grid->xfirst);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfLastGridPointInDegrees",  &grid->xlast);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfFirstGridPointInDegrees",  &grid->yfirst);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfLastGridPointInDegrees",   &grid->ylast);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "iDirectionIncrementInDegrees", &grid->xinc);
-
-        if ( IS_EQUAL(grid->xinc, GRIB_MISSING_DOUBLE) ) grid->xinc = 0;
-
-        /* if ( IS_NOT_EQUAL(grid->xfirst, 0) || IS_NOT_EQUAL(grid->xlast, 0) ) */
+        if ( grid->y.size > 1 )
           {
-            if ( grid->xsize > 1 )
+            if ( editionNumber <= 1 )
               {
-                if ( (grid->xfirst > grid->xlast) && (grid->xfirst >= 180) ) grid->xfirst -= 360;
+              }
+          }
+        grid->y.flag = 2;
+      }
+    }
+  else if ( gridtype == GRID_GAUSSIAN_REDUCED )
+    {
+      FAIL_ON_GRIB_ERROR(grib_get_long, gh, "numberOfParallelsBetweenAPoleAndTheEquator", &lpar);
+      grid->np = (int)lpar;
 
-                if ( editionNumber <= 1 )
+      FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Nj", &lpar);
+      int nlat = (int)lpar;
+
+      grid->size   = (int)numberOfPoints;
+
+      grid->nrowlon = nlat;
+      grid->rowlon = (int *) Malloc((size_t)nlat * sizeof (int));
+      long *pl     = (long *) Malloc((size_t)nlat * sizeof (long));
+      size_t dummy = (size_t)nlat;
+      FAIL_ON_GRIB_ERROR(grib_get_long_array, gh, "pl", pl, &dummy);
+      for ( int i = 0; i < nlat; ++i ) grid->rowlon[i] = (int)pl[i];
+      Free(pl);
+
+      grid->y.size  = nlat;
+      grid->x.inc   = 0;
+      grid->y.inc   = 0;
+      grid->x.flag  = 0;
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfFirstGridPointInDegrees", &grid->x.first);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfLastGridPointInDegrees",  &grid->x.last);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfFirstGridPointInDegrees",  &grid->y.first);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfLastGridPointInDegrees",   &grid->y.last);
+
+      // FAIL_ON_GRIB_ERROR(grib_get_double, gh, "iDirectionIncrementInDegrees", &grid->x.inc);
+      // if ( IS_EQUAL(grid->x.inc, GRIB_MISSING_DOUBLE) ) grid->x.inc = 0;
+
+      /* if ( IS_NOT_EQUAL(grid->x.first, 0) || IS_NOT_EQUAL(grid->x.last, 0) ) */
+      {
+        if ( grid->x.size > 1 )
+          {
+            if ( (grid->x.first > grid->x.last) && (grid->x.first >= 180) ) grid->x.first -= 360;
+
+            if ( editionNumber <= 1 )
+              {
+                /* correct xinc if necessary */
+                if ( IS_EQUAL(grid->x.first, 0) && grid->x.last > 354 )
                   {
-                    /* correct xinc if necessary */
-                    if ( IS_EQUAL(grid->xfirst, 0) && grid->xlast > 354 )
+                    double xinc = 360. / grid->x.size;
+                    if ( fabs(grid->x.inc-xinc) > 0.0 )
                       {
-                        double xinc = 360. / grid->xsize;
-
-                        if ( fabs(grid->xinc-xinc) > 0.0 )
-                          {
-                            grid->xinc = xinc;
-                            if ( CDI_Debug ) Message("set xinc to %g", grid->xinc);
-                          }
+                        grid->x.inc = xinc;
+                        if ( CDI_Debug ) Message("set xinc to %g", grid->x.inc);
                       }
                   }
               }
-            grid->xdef = 2;
           }
-        grid->ydef  = 0;
-        /* if ( IS_NOT_EQUAL(grid->yfirst, 0) || IS_NOT_EQUAL(grid->ylast, 0) ) */
+        grid->x.flag = 2;
+      }
+      grid->y.flag = 0;
+      /* if ( IS_NOT_EQUAL(grid->y.first, 0) || IS_NOT_EQUAL(grid->y.last, 0) ) */
+      {
+        if ( grid->y.size > 1 )
           {
-            if ( grid->ysize > 1 )
+            if ( editionNumber <= 1 )
               {
-                if ( editionNumber <= 1 )
-                  {
-                  }
               }
-            grid->ydef = 2;
           }
-        break;
+        grid->y.flag = 2;
       }
-    case GRID_LCC:
-      {
-        int nlon, nlat;
-        long lpar;
+    }
+  else if ( gridtype == GRID_LCC )
+    {
+      int nlon, nlat;
+      FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Nx", &lpar);
+      nlon = (int)lpar;
+      FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Ny", &lpar);
+      nlat = (int)lpar;
 
-        FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Nx", &lpar);
-        nlon = (int)lpar;
-        FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Ny", &lpar);
-        nlat = (int)lpar;
+      if ( numberOfPoints != nlon*nlat )
+        Error("numberOfPoints (%d) and gridSize (%d) differ!", (int)numberOfPoints, nlon*nlat);
 
-        if ( numberOfPoints != nlon*nlat )
-          Error("numberOfPoints (%d) and gridSize (%d) differ!", (int)numberOfPoints, nlon*nlat);
+      grid->size  = (int)numberOfPoints;
+      grid->x.size = nlon;
+      grid->y.size = nlat;
 
-        grid->size  = (int)numberOfPoints;
-        grid->xsize = nlon;
-        grid->ysize = nlat;
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "DxInMetres", &grid->lcc.xinc);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "DyInMetres", &grid->lcc.yinc);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfFirstGridPointInDegrees", &grid->lcc.originLon);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfFirstGridPointInDegrees", &grid->lcc.originLat);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "LoVInDegrees", &grid->lcc.lonParY);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "Latin1InDegrees", &grid->lcc.lat1);
+      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "Latin2InDegrees", &grid->lcc.lat2);
 
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "DxInMetres", &grid->lcc_xinc);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "DyInMetres", &grid->lcc_yinc);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfFirstGridPointInDegrees", &grid->lcc_originLon);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfFirstGridPointInDegrees", &grid->lcc_originLat);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "LoVInDegrees", &grid->lcc_lonParY);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "Latin1InDegrees", &grid->lcc_lat1);
-        FAIL_ON_GRIB_ERROR(grib_get_double, gh, "Latin2InDegrees", &grid->lcc_lat2);
+      if ( editionNumber <= 1 )
+        {
+          FAIL_ON_GRIB_ERROR(grib_get_long, gh, "projectionCenterFlag", &lpar);
+          grid->lcc.projflag  = (int) lpar;
+          FAIL_ON_GRIB_ERROR(grib_get_long, gh, "scanningMode", &lpar);
+          grid->lcc.scanflag  = (int) lpar;
+        }
 
-        if ( editionNumber <= 1 )
-          {
-            FAIL_ON_GRIB_ERROR(grib_get_long, gh, "projectionCenterFlag", &lpar);
-            grid->lcc_projflag  = (int) lpar;
-            FAIL_ON_GRIB_ERROR(grib_get_long, gh, "scanningMode", &lpar);
-            grid->lcc_scanflag  = (int) lpar;
-          }
+      grid->x.flag = 0;
+      grid->y.flag = 0;
+    }
+  else if ( gridtype == GRID_SPECTRAL )
+    {
+      size_t len = 256;
+      char typeOfPacking[256];
+      FAIL_ON_GRIB_ERROR(grib_get_string, gh, "packingType", typeOfPacking, &len);
+      grid->lcomplex = 0;
+      if ( strncmp(typeOfPacking, "spectral_complex", len) == 0 ) grid->lcomplex = 1;
 
-        grid->xdef   = 0;
-        grid->ydef   = 0;
+      grid->size  = (int)datasize;
 
-        break;
-      }
-    case GRID_SPECTRAL:
-      {
-        size_t len = 256;
-        char typeOfPacking[256];
-        FAIL_ON_GRIB_ERROR(grib_get_string, gh, "packingType", typeOfPacking, &len);
-        grid->lcomplex = 0;
-        if ( strncmp(typeOfPacking, "spectral_complex", len) == 0 ) grid->lcomplex = 1;
-
-        /* FIXME: assert(datasize >= INT_MIN && datasize <= INT_MAX) */
-        grid->size  = (int)datasize;
-        long lpar;
-        FAIL_ON_GRIB_ERROR(grib_get_long, gh, "J", &lpar);
-        /* FIXME: assert(lpar >= INT_MIN && lpar <= INT_MAX) */
-        grid->trunc = (int)lpar;
-
-        break;
-      }
-    case GRID_GME:
-      {
-        /* FIXME: assert(numberOfPoints <= INT_MAX && numberOfPoints >= INT_MIN) */
-        grid->size  = (int)numberOfPoints;
-        long lpar;
-        /* FIXME: assert(lpar >= INT_MIN && lpar <= INT_MAX) */
-        if ( grib_get_long(gh, "nd", &lpar) == 0 ) grid->nd  = (int)lpar;
-        /* FIXME: assert(lpar >= INT_MIN && lpar <= INT_MAX) */
-        if ( grib_get_long(gh, "Ni", &lpar) == 0 ) grid->ni  = (int)lpar;
-        /* FIXME: assert(lpar >= INT_MIN && lpar <= INT_MAX) */
-        if ( grib_get_long(gh, "n2", &lpar) == 0 ) grid->ni2 = (int)lpar;
-        /* FIXME: assert(lpar >= INT_MIN && lpar <= INT_MAX) */
-        if ( grib_get_long(gh, "n3", &lpar) == 0 ) grid->ni3 = (int)lpar;
-
-        break;
-      }
-    case GRID_UNSTRUCTURED:
-      {
-        unsigned char uuid[CDI_UUID_SIZE];
-        /*
+      FAIL_ON_GRIB_ERROR(grib_get_long, gh, "J", &lpar);
+      grid->trunc = (int)lpar;
+    }
+  else if ( gridtype == GRID_GME )
+    {
+      grid->size  = (int)numberOfPoints;
+      if ( grib_get_long(gh, "nd", &lpar) == 0 ) grid->gme.nd  = (int)lpar;
+      if ( grib_get_long(gh, "Ni", &lpar) == 0 ) grid->gme.ni  = (int)lpar;
+      if ( grib_get_long(gh, "n2", &lpar) == 0 ) grid->gme.ni2 = (int)lpar;
+      if ( grib_get_long(gh, "n3", &lpar) == 0 ) grid->gme.ni3 = (int)lpar;
+    }
+  else if ( gridtype == GRID_UNSTRUCTURED )
+    {
+      unsigned char uuid[CDI_UUID_SIZE];
+      /*
         char reference_link[8192];
         size_t len = sizeof(reference_link);
         reference_link[0] = 0;
-        */
+      */
+      grid->size  = (int)numberOfPoints;
 
-        /* FIXME: assert(numberOfPoints <= INT_MAX && numberOfPoints >= INT_MIN) */
-            grid->size  = (int)numberOfPoints;
-        long lpar;
-        if ( grib_get_long(gh, "numberOfGridUsed", &lpar) == 0 )
-          {
-            /* FIXME: assert(lpar <= INT_MAX && lpar >= INT_MIN) */
-            grid->number   = (int)lpar;
-            /* FIXME: assert(lpar <= INT_MAX && lpar >= INT_MIN) */
-            if ( grib_get_long(gh, "numberOfGridInReference", &lpar) == 0 )
-              grid->position = (int)lpar;
-            /*
+      if ( grib_get_long(gh, "numberOfGridUsed", &lpar) == 0 )
+        {
+          grid->number   = (int)lpar;
+          if ( grib_get_long(gh, "numberOfGridInReference", &lpar) == 0 )
+            grid->position = (int)lpar;
+          /*
             if ( grib_get_string(gh, "gridDescriptionFile", reference_link, &len) == 0 )
-              {
-                if ( strncmp(reference_link, "file://", 7) == 0 )
-                  grid->reference = strdupx(reference_link);
-              }
-            */
-            size_t len = (size_t)CDI_UUID_SIZE;
-            if ( grib_get_bytes(gh, "uuidOfHGrid", uuid, &len) == 0)
-              {
-                memcpy(grid->uuid, uuid, CDI_UUID_SIZE);
-              }
-          }
-        break;
-      }
-    case GRID_GENERIC:
-      {
-        int nlon = 0, nlat = 0;
-        long lpar;
-        /* FIXME: assert(lpar <= INT_MAX && lpar >= INT_MIN) */
-        if ( grib_get_long(gh, "Ni", &lpar) == 0 ) nlon = (int)lpar;
-        /* FIXME: assert(lpar <= INT_MAX && lpar >= INT_MIN) */
-        if ( grib_get_long(gh, "Nj", &lpar) == 0 ) nlat = (int)lpar;
-
-        /* FIXME: assert(numberOfPoints <= INT_MAX && numberOfPoints >= INT_MIN) */
-        grid->size  = (int)numberOfPoints;
-
-        if ( nlon > 0 && nlat > 0 && nlon*nlat == grid->size )
-          {
-            grid->xsize = nlon;
-            grid->ysize = nlat;
-          }
-        else
-          {
-            grid->xsize = 0;
-            grid->ysize = 0;
-          }
-
-        break;
-      }
-    default:
-      {
-        Error("Unsupported grid type: %s", gridNamePtr(gridtype));
-        break;
-      }
+            {
+            if ( strncmp(reference_link, "file://", 7) == 0 )
+            grid->reference = strdupx(reference_link);
+            }
+          */
+          size_t len = (size_t)CDI_UUID_SIZE;
+          if ( grib_get_bytes(gh, "uuidOfHGrid", uuid, &len) == 0)
+            {
+              memcpy(grid->uuid, uuid, CDI_UUID_SIZE);
+            }
+        }
     }
-
-  grid->isRotated = FALSE;
-  if ( gribapiGetIsRotated(gh) )
+  else if ( gridtype == GRID_GENERIC )
     {
-      grid->isRotated = TRUE;
-      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "latitudeOfSouthernPoleInDegrees",  &grid->ypole);
-      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfSouthernPoleInDegrees", &grid->xpole);
-      FAIL_ON_GRIB_ERROR(grib_get_double, gh, "angleOfRotation", &grid->angle);
-      /* change from south to north pole */
-      if ( fabs(grid->ypole) > 0 ) grid->ypole = -grid->ypole;
-      grid->xpole =  grid->xpole - 180;
-      if ( fabs(grid->angle) > 0 ) grid->angle = -grid->angle;
+      int nlon = 0, nlat = 0;
+      if ( grib_get_long(gh, "Ni", &lpar) == 0 ) nlon = (int)lpar;
+      if ( grib_get_long(gh, "Nj", &lpar) == 0 ) nlat = (int)lpar;
+
+      grid->size  = (int)numberOfPoints;
+
+      if ( nlon > 0 && nlat > 0 && nlon*nlat == grid->size )
+        {
+          grid->x.size = nlon;
+          grid->y.size = nlat;
+        }
+      else
+        {
+          grid->x.size = 0;
+          grid->y.size = 0;
+        }
+    }
+  else
+    {
+      Error("Unsupported grid type: %s", gridNamePtr(gridtype));
     }
 
-  grid->xvals = NULL;
-  grid->yvals = NULL;
   grid->type  = gridtype;
+  grid->projtype  = projtype;
 }
 #endif
 /*
