@@ -15,13 +15,10 @@
 #include "serialize.h"
 #include "institution.h"
 
-#undef  UNDEFID
-#define UNDEFID  -1
 
-int ECMWF  = UNDEFID;
-int MPIMET = UNDEFID;
-int DWD    = UNDEFID;
-int MCH    = UNDEFID;
+static int ECMWF  = CDI_UNDEFID,
+  MPIMET = CDI_UNDEFID,
+  MCH    = CDI_UNDEFID;
 
 typedef struct
 {
@@ -54,10 +51,10 @@ static const resOps instituteOps = {
 static
 void instituteDefaultValue ( institute_t * instituteptr )
 {
-  instituteptr->self       = UNDEFID;
+  instituteptr->self       = CDI_UNDEFID;
   instituteptr->used       = 0;
-  instituteptr->center     = UNDEFID;
-  instituteptr->subcenter  = UNDEFID;
+  instituteptr->center     = CDI_UNDEFID;
+  instituteptr->subcenter  = CDI_UNDEFID;
   instituteptr->name       = NULL;
   instituteptr->longname   = NULL;
 }
@@ -68,7 +65,7 @@ void instituteDefaultEntries ( void )
     = { ECMWF   = institutDef( 98,   0, "ECMWF",     "European Centre for Medium-Range Weather Forecasts"),
         MPIMET  = institutDef( 98, 232, "MPIMET",    "Max-Planck-Institute for Meteorology"),
         institutDef( 98, 255, "MPIMET",    "Max-Planck-Institute for Meteorology"),
-        institutDef( 98, 232, "MPIMET",    "Max-Planck Institute for Meteorology"),
+        institutDef( 98, 232, "MPIMET",    "Max Planck Institute for Meteorology"),
         institutDef( 78,   0, "DWD",       "Deutscher Wetterdienst"),
         institutDef( 78, 255, "DWD",       "Deutscher Wetterdienst"),
         MCH     = institutDef(215, 255, "MCH",       "MeteoSwiss"),
@@ -88,12 +85,6 @@ void instituteDefaultEntries ( void )
 }
 
 
-int instituteCount ( void )
-{
-  return reshCountType ( &instituteOps );
-}
-
-
 static int
 instituteCompareKernel(institute_t *  ip1, institute_t * ip2)
 {
@@ -157,18 +148,18 @@ findInstitute(int id, void *res, void *data)
 
 int institutInq(int center, int subcenter, const char *name, const char *longname)
 {
-  institute_t * ip_ref = (institute_t *)xmalloc(sizeof (*ip_ref));
-  ip_ref->self       = UNDEFID;
+  institute_t * ip_ref = (institute_t *) Malloc(sizeof (*ip_ref));
+  ip_ref->self       = CDI_UNDEFID;
   ip_ref->used       = 0;
   ip_ref->center     = center;
   ip_ref->subcenter  = subcenter;
   ip_ref->name       = name && name[0] ? (char *)name : NULL;
   ip_ref->longname   = longname && longname[0] ? (char *)longname : NULL;
 
-  struct instLoc state = { .ip = ip_ref, .id = UNDEFID };
+  struct instLoc state = { .ip = ip_ref, .id = CDI_UNDEFID };
   cdiResHFilterApply(&instituteOps, findInstitute, &state);
 
-  free(ip_ref);
+  Free(ip_ref);
 
   return state.id;
 }
@@ -177,7 +168,7 @@ static
 institute_t *instituteNewEntry(cdiResH resH, int center, int subcenter,
                                const char *name, const char *longname)
 {
-  institute_t *instituteptr = (institute_t*) xmalloc(sizeof(institute_t));
+  institute_t *instituteptr = (institute_t*) Malloc(sizeof(institute_t));
   instituteDefaultValue(instituteptr);
   if (resH == CDI_UNDEFID)
     instituteptr->self = reshPut(instituteptr, &instituteOps);
@@ -209,10 +200,10 @@ int institutInqCenter(int instID)
 {
   institute_t * instituteptr = NULL;
 
-  if ( instID != UNDEFID )
+  if ( instID != CDI_UNDEFID )
     instituteptr = ( institute_t * ) reshGetVal ( instID, &instituteOps );
 
-  return  instituteptr ? instituteptr->center : UNDEFID;
+  return  instituteptr ? instituteptr->center : CDI_UNDEFID;
 }
 
 
@@ -220,10 +211,10 @@ int institutInqSubcenter(int instID)
 {
   institute_t * instituteptr = NULL;
 
-  if ( instID != UNDEFID )
+  if ( instID != CDI_UNDEFID )
     instituteptr = ( institute_t * ) reshGetVal ( instID, &instituteOps );
 
-  return instituteptr ? instituteptr->subcenter: UNDEFID;
+  return instituteptr ? instituteptr->subcenter: CDI_UNDEFID;
 }
 
 
@@ -231,18 +222,18 @@ const char *institutInqNamePtr(int instID)
 {
   institute_t * instituteptr = NULL;
 
-  if ( instID != UNDEFID )
+  if ( instID != CDI_UNDEFID )
     instituteptr = ( institute_t * ) reshGetVal ( instID, &instituteOps );
 
   return instituteptr ? instituteptr->name : NULL;
 }
 
 
-char *institutInqLongnamePtr(int instID)
+const char *institutInqLongnamePtr(int instID)
 {
   institute_t * instituteptr = NULL;
 
-  if ( instID != UNDEFID )
+  if ( instID != CDI_UNDEFID )
     instituteptr = ( institute_t * ) reshGetVal ( instID, &instituteOps );
 
   return instituteptr ? instituteptr->longname : NULL;
@@ -272,10 +263,10 @@ instituteDestroyP(institute_t *instituteptr)
   xassert(instituteptr);
 
   int instituteID = instituteptr->self;
-  free(instituteptr->name);
-  free(instituteptr->longname);
+  Free(instituteptr->name);
+  Free(instituteptr->longname);
   reshRemove(instituteID, &instituteOps);
-  free(instituteptr);
+  Free(instituteptr);
 }
 
 
@@ -311,9 +302,9 @@ static int instituteGetPackSize(institute_t *ip, void *context)
 {
   size_t namelen = strlen(ip->name), longnamelen = strlen(ip->longname);
   xassert(namelen < INT_MAX && longnamelen < INT_MAX);
-  size_t txsize = (size_t)serializeGetSize(institute_nints, DATATYPE_INT, context)
-    + (size_t)serializeGetSize((int)namelen + 1, DATATYPE_TXT, context)
-    + (size_t)serializeGetSize((int)longnamelen + 1, DATATYPE_TXT, context);
+  size_t txsize = (size_t)serializeGetSize(institute_nints, CDI_DATATYPE_INT, context)
+    + (size_t)serializeGetSize((int)namelen + 1, CDI_DATATYPE_TXT, context)
+    + (size_t)serializeGetSize((int)longnamelen + 1, CDI_DATATYPE_TXT, context);
   xassert(txsize <= INT_MAX);
   return (int)txsize;
 }
@@ -327,9 +318,9 @@ static void institutePackP(void * instituteptr, void *buf, int size, int *positi
   tempbuf[2] = p->subcenter;
   tempbuf[3] = (int)strlen(p->name) + 1;
   tempbuf[4] = (int)strlen(p->longname) + 1;
-  serializePack(tempbuf, institute_nints, DATATYPE_INT, buf, size, position, context);
-  serializePack(p->name, tempbuf[3], DATATYPE_TXT, buf, size, position, context);
-  serializePack(p->longname, tempbuf[4], DATATYPE_TXT, buf, size, position, context);
+  serializePack(tempbuf, institute_nints, CDI_DATATYPE_INT, buf, size, position, context);
+  serializePack(p->name, tempbuf[3], CDI_DATATYPE_TXT, buf, size, position, context);
+  serializePack(p->longname, tempbuf[4], CDI_DATATYPE_TXT, buf, size, position, context);
 }
 
 int instituteUnpack(void *buf, int size, int *position, int originNamespace,
@@ -338,17 +329,19 @@ int instituteUnpack(void *buf, int size, int *position, int originNamespace,
   int tempbuf[institute_nints];
   int instituteID;
   char *name, *longname;
-  serializeUnpack(buf, size, position, tempbuf, institute_nints, DATATYPE_INT, context);
-  name = (char *)xmalloc((size_t)tempbuf[3] + (size_t)tempbuf[4]);
+  serializeUnpack(buf, size, position, tempbuf, institute_nints, CDI_DATATYPE_INT, context);
+  name = (char *) Malloc((size_t)tempbuf[3] + (size_t)tempbuf[4]);
   longname = name + tempbuf[3];
-  serializeUnpack(buf, size, position, name, tempbuf[3], DATATYPE_TXT, context);
-  serializeUnpack(buf, size, position, longname, tempbuf[4], DATATYPE_TXT, context);
+  serializeUnpack(buf, size, position, name, tempbuf[3], CDI_DATATYPE_TXT, context);
+  serializeUnpack(buf, size, position, longname, tempbuf[4], CDI_DATATYPE_TXT, context);
   int targetID = namespaceAdaptKey(tempbuf[0], originNamespace);
   institute_t *ip = instituteNewEntry(force_id?targetID:CDI_UNDEFID,
                                       tempbuf[1], tempbuf[2], name, longname);
   instituteID = ip->self;
   xassert(!force_id || instituteID == targetID);
-  free(name);
+  Free(name);
+  reshSetStatus(instituteID, &instituteOps,
+                reshGetStatus(instituteID, &instituteOps) & ~RESH_SYNC_BIT);
   return instituteID;
 }
 
