@@ -1,32 +1,52 @@
 #include "iterator_grib.h"
 
+#include "cdi.h"
 #include "cdi_int.h"
 #include "cgribex.h"
 #include "dmemory.h"
 #include "error.h"
 #include "gribapi.h"
 #include "gribapi_utilities.h"
-#include "proprietarySystemWorkarounds.h"
 #include "stream_grb.h"
 #include "zaxis.h"
 
 #include <assert.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
 
 #ifdef HAVE_LIBGRIB_API
 
+struct CdiGribIterator {
+  CdiIterator super;
+
+  CdiInputFile *file;
+  off_t fileOffset;
+  unsigned char *gribBuffer;
+  size_t bufferSize, curRecordSize;
+#ifdef HAVE_LIBGRIB_API
+  grib_handle *gribHandle;
+#else
+  void *gribHandle;
+#endif
+};
+
+CdiIterator *cdiGribIterator_getSuper(CdiGribIterator *me)
+{
+  return &me->super;
+}
+
 //Since the error handling in constructors is usually very closely related to the workings of a destructor,
 //this function combines both functions in one, using a centralized exit.
 //The mode of operation depends on whether me is a NULL pointer on entry:
 //If it is NULL, a new object is allocated and constructed, which is returned if construction is successful.
 //If a non-NULL pointer is passed in, the object is destructed and NULL is returned. In this case, the other arguments are ignored.
-static CdiGribIterator* cdiGribIterator_condestruct(CdiGribIterator* me, const char* path, int filetype)
+static CdiGribIterator *cdiGribIterator_condestruct(CdiGribIterator *me, const char *path, int filetype)
 {
-  #define super() (&me->super)
+#define super() (&me->super)
   if(me) goto destruct;
-  me = xmalloc(sizeof(*me));
+  me = (CdiGribIterator *) Malloc(sizeof(*me));
   baseIterConstruct(super(), filetype);
 
   me->file = cdiInputFile_make(path);
@@ -45,43 +65,42 @@ static CdiGribIterator* cdiGribIterator_condestruct(CdiGribIterator* me, const c
 
 destruct:
   if(me->super.gridId != CDI_UNDEFID) gridDestroy(me->super.gridId);
-  if(me->gribHandle) grib_handle_delete(me->gribHandle);
-  free(me->gribBuffer);
+  if(me->gribHandle) grib_handle_delete((struct grib_handle *)me->gribHandle);
+  Free(me->gribBuffer);
   cdiRefObject_release(&me->file->super);
 destructSuper:
   baseIterDestruct(super());
-  free(me);
+  Free(me);
   me = NULL;
 
 success:
   return me;
-  #undef super
+#undef super
 }
 
-CdiIterator* cdiGribIterator_new(const char* path, int filetype)
+CdiIterator *cdiGribIterator_new(const char *path, int filetype)
 {
   return &cdiGribIterator_condestruct(NULL, path, filetype)->super;
 }
 
-CdiGribIterator* cdiGribIterator_makeClone(CdiIterator* super)
+CdiGribIterator *cdiGribIterator_makeClone(CdiIterator *super)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
 
   //Allocate memory and copy data. (operations that may fail)
-  CdiGribIterator* result = xmalloc(sizeof(*result));
+  CdiGribIterator *result = (struct CdiGribIterator *) Malloc(sizeof(*result));
   if(!result) goto fail;
-  *result = (CdiGribIterator)
-    {
-      .file = me->file,
-      .fileOffset = me->fileOffset,
-      .gribBuffer = NULL,
-      .bufferSize = me->bufferSize,
-      .curRecordSize = me->curRecordSize,
-      .gribHandle = NULL
-    };
+
+  result->file = me->file;
+  result->fileOffset = me->fileOffset;
+  result->gribBuffer = NULL;
+  result->bufferSize = me->bufferSize;
+  result->curRecordSize = me->curRecordSize;
+  result->gribHandle = NULL;
+
   if(me->gribBuffer)
     {
-      result->gribBuffer = xmalloc(me->bufferSize);
+      result->gribBuffer = (unsigned char *) Malloc(me->bufferSize);
       if(!result->gribBuffer) goto freeResult;
       memcpy(result->gribBuffer, me->gribBuffer, me->curRecordSize);
     }
@@ -109,30 +128,30 @@ CdiGribIterator* cdiGribIterator_makeClone(CdiIterator* super)
 deleteGribHandle:
   if(result->gribHandle) grib_handle_delete(result->gribHandle);
 freeBuffer:
-  free(result->gribBuffer);
+  Free(result->gribBuffer);
 freeResult:
-  free(result);
+  Free(result);
 fail:
   return NULL;
 }
 
-char* cdiGribIterator_serialize(CdiIterator* super)
+char *cdiGribIterator_serialize(CdiIterator *super)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
 
-  char* path = cdiInputFile_copyPath(me->file);
-  char* escapedPath = cdiEscapeSpaces(path);
-  free(path);
-  char* result = myAsprintf("%s %zu", escapedPath, me->fileOffset);
-  free(escapedPath);
+  const char *path = cdiInputFile_getPath(me->file);
+  char *escapedPath = cdiEscapeSpaces(path);
+  char *result = (char *) Malloc(strlen(escapedPath) + 3 * sizeof(int) * CHAR_BIT/8);
+  sprintf(result, "%s %zu", escapedPath, (size_t)me->fileOffset);
+  Free(escapedPath);
   return result;
 }
 
 
-CdiGribIterator* cdiGribIterator_deserialize(const char* description)
+CdiGribIterator *cdiGribIterator_deserialize(const char *description)
 {
-  char* path;
-  CdiGribIterator* me = xmalloc(sizeof(*me));
+  char *path;
+  CdiGribIterator *me = (CdiGribIterator *) Malloc(sizeof(*me));
   if(!me) goto fail;
 
   description = baseIter_constructFromString(&me->super, description);
@@ -142,11 +161,11 @@ CdiGribIterator* cdiGribIterator_deserialize(const char* description)
   if(!path) goto destructSuper;
 
   me->file = cdiInputFile_make(path);
-  free(path);
+  Free(path);
   if(!me->file) goto destructSuper;
 
   {
-    const char* savedStart = description;
+    const char *savedStart = description;
     long long decodedOffset = strtoll(description, (char**)&description, 0);    //The cast is a workaround for the wrong signature of strtoll() (it should have been `long long strtoll(const char*, const char**, int)`, not `long long strtoll(const char*, char**, int)`.
     me->fileOffset = (off_t)decodedOffset;
     if(savedStart == description) goto closeFile;
@@ -166,20 +185,18 @@ closeFile:
   cdiRefObject_release(&me->file->super);
 destructSuper:
   baseIterDestruct(&me->super);
-  free(me);
+  Free(me);
 fail:
   return NULL;
 }
-#endif
 
-#ifdef HAVE_LIBGRIB_API
-static void cdiGribIterator_ensureBuffer(CdiGribIterator* me, size_t requiredSize)
+static void cdiGribIterator_ensureBuffer(CdiGribIterator *me, size_t requiredSize)
 {
   if(me->bufferSize < requiredSize)
     {
       me->bufferSize *= 2;
       if(me->bufferSize < requiredSize) me->bufferSize = requiredSize;
-      me->gribBuffer = xrealloc(me->gribBuffer, me->bufferSize);
+      me->gribBuffer = (unsigned char *) Realloc(me->gribBuffer, me->bufferSize);
     }
 }
 
@@ -195,11 +212,11 @@ static bool isGrib1DualLevel(int levelType)
     }
 }
 
-static const unsigned char* positionOfGribMarker(const unsigned char* data, size_t size)
+static const unsigned char *positionOfGribMarker(const unsigned char *data, size_t size)
 {
-  for(const unsigned char* currentPosition = data, *end = data + size; currentPosition < end; currentPosition++)
+  for(const unsigned char *currentPosition = data, *end = data + size; currentPosition < end; currentPosition++)
     {
-      currentPosition = memchr(currentPosition, 'G', size - (currentPosition - data) - 3);      //-3 to ensure that we don't overrun the buffer during the strncmp() call.
+      currentPosition = (unsigned char *)memchr(currentPosition, 'G', size - (size_t)(currentPosition - data) - 3);      //-3 to ensure that we don't overrun the buffer during the strncmp() call.
       if(!currentPosition) return NULL;
       if(!strncmp((const char*)currentPosition, "GRIB", 4)) return currentPosition;
     }
@@ -208,7 +225,7 @@ static const unsigned char* positionOfGribMarker(const unsigned char* data, size
 
 //This clobbers the contents of the gribBuffer!
 //Returns the file offset of the next 'GRIB' marker.
-static ssize_t scanToGribMarker(CdiGribIterator* me)
+static ssize_t scanToGribMarker(CdiGribIterator *me)
 {
   cdiGribIterator_ensureBuffer(me, 8*1024);
   const size_t kMaxScanSize = 16*1024*1024;
@@ -218,39 +235,39 @@ static ssize_t scanToGribMarker(CdiGribIterator* me)
       scanSize = me->bufferSize;
       if(scannedBytes + scanSize > kMaxScanSize) scanSize = kMaxScanSize - scannedBytes;
       assert(scanSize <= me->bufferSize);
-      int status = cdiInputFile_read(me->file, me->fileOffset + scannedBytes, scanSize, &scanSize, me->gribBuffer);
-      if(status != CDI_NOERR && status != CDI_EEOF) return status;
+      int status = cdiInputFile_read(me->file, me->fileOffset + (off_t)scannedBytes, scanSize, &scanSize, me->gribBuffer);
+      if(status != CDI_NOERR && status != CDI_EEOF) return -1;
 
-      const unsigned char* startPosition = positionOfGribMarker(me->gribBuffer, scanSize);
+      const unsigned char *startPosition = positionOfGribMarker(me->gribBuffer, scanSize);
       if(startPosition)
         {
-          return me->fileOffset + scannedBytes + (startPosition - me->gribBuffer);
+          return (ssize_t)(me->fileOffset + (off_t)scannedBytes + (off_t)(startPosition - me->gribBuffer));
         }
 
       //Get the offset for the next iteration if there is a next iteration.
       scanSize -= 3;        //so that we won't miss a 'GRIB' sequence that happens to be cut off
-      scannedBytes += scanSize;
-      scannedBytes &= ~0xf; //make 16 bytes aligned
+      scanSize &= ~(size_t)0xf; //make 16 bytes aligned
+      if((ssize_t)scanSize <= 0) return -1; //ensure that we make progress
     }
   return -1;
 }
 
-static unsigned decode24(void* beData)
+static unsigned decode24(void *beData)
 {
-  unsigned char* bytes = beData;
+  unsigned char *bytes = (unsigned char *)beData;
   return ((unsigned)bytes[0] << 16) + ((unsigned)bytes[1] << 8) + (unsigned)bytes[2];
 }
 
-static uint64_t decode64(void* beData)
+static uint64_t decode64(void *beData)
 {
-  unsigned char* bytes = beData;
+  unsigned char *bytes = (unsigned char *)beData;
   uint64_t result = 0;
   for(size_t i = 0; i < 8; i++) result = (result << 8) + bytes[i];
   return result;
 }
 
 //Determine the size of the GRIB record that begins at the given file offset.
-static int getRecordSize(CdiGribIterator* me, off_t gribFileOffset, size_t* outRecordSize)
+static int getRecordSize(CdiGribIterator *me, off_t gribFileOffset, size_t *outRecordSize)
 {
   char buffer[16];
   size_t readSize;
@@ -282,9 +299,9 @@ static int getRecordSize(CdiGribIterator* me, off_t gribFileOffset, size_t* outR
 }
 
 #if 0
-static void hexdump(void* data, size_t size)
+static void hexdump(void *data, size_t size)
 {
-  unsigned char* charData = data;
+  unsigned char *charData = data;
   for(size_t offset = 0; offset < size; )
     {
       printf("%016zx:", offset);
@@ -301,11 +318,11 @@ static void hexdump(void* data, size_t size)
 
 //Read a record into memory and wrap it in a grib_handle.
 //XXX: I have omitted checking for szip compression as it is done in grbReadVarDP() & friends since that appears to be a non-standard extension of the GRIB1 standard: bit 1 in octet 14 of the binary data section which is used to signal szip compressio is defined to be reserved in the standard. As such, it seems prudent not to support this and to encourage people with such szip compressed files to switch to the GRIB2/JPEG2000 format. However, in the case that this reasoning is wrong, this function is probably the place to add the check for zsip compression.
-static int readMessage(CdiGribIterator* me)
+static int readMessage(CdiGribIterator *me)
 {
   //Destroy the old grib_handle.
   if(me->gribHandle) grib_handle_delete(me->gribHandle), me->gribHandle = NULL;
-  me->fileOffset += me->curRecordSize;
+  me->fileOffset += (off_t)me->curRecordSize;
 
   //Find the next record and determine its size.
   ssize_t gribFileOffset = scanToGribMarker(me);
@@ -329,9 +346,9 @@ fail:
   return result;
 }
 
-int cdiGribIterator_nextField(CdiIterator* super)
+int cdiGribIterator_nextField(CdiIterator *super)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
 
   if(super->gridId != CDI_UNDEFID) gridDestroy(super->gridId), super->gridId = CDI_UNDEFID;
 
@@ -350,37 +367,37 @@ int cdiGribIterator_nextField(CdiIterator* super)
   return CDI_NOERR;
 }
 
-char* cdiGribIterator_inqTime(CdiIterator* super, bool getEndTime)
+char *cdiGribIterator_inqTime(CdiIterator *super, CdiTimeType timeType)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
-  return gribMakeTimeString(me->gribHandle, getEndTime);
+  CdiGribIterator *me = (CdiGribIterator*)super;
+  return gribMakeTimeString(me->gribHandle, timeType);
 }
 
-int cdiGribIterator_levelType(CdiIterator* super, int levelSelector, char** outName, char** outLongName, char** outStdName, char** outUnit)
+int cdiGribIterator_levelType(CdiIterator *super, int levelSelector, char **outName, char **outLongName, char **outStdName, char **outUnit)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
 
   //First determine the zaxis type corresponding to the given level.
   int zaxisType = ZAXIS_GENERIC;
   if(gribEditionNumber(me->gribHandle) <= 1)
     {
-      int levelType = gribGetLongDefault(me->gribHandle, "indicatorOfTypeOfLevel", 255);
+      int levelType = (int)gribGetLongDefault(me->gribHandle, "indicatorOfTypeOfLevel", 255);
       if(levelSelector && !isGrib1DualLevel(levelType)) levelType = 255;
       zaxisType = grib1ltypeToZaxisType(levelType);
     }
   else
     {
-      int levelType = gribGetLongDefault(me->gribHandle, levelSelector ? "typeOfSecondFixedSurface" : "typeOfFirstFixedSurface", 255);
+      int levelType = (int)gribGetLongDefault(me->gribHandle, levelSelector ? "typeOfSecondFixedSurface" : "typeOfFirstFixedSurface", 255);
       zaxisType = grib2ltypeToZaxisType(levelType);
     }
 
   //Then lookup the requested names.
-  const char* name, *longName, *stdName, *unit;
+  const char *name, *longName, *stdName, *unit;
   zaxisGetTypeDescription(zaxisType, NULL, &name, &longName, &stdName, &unit);
-  if(outName) *outName = myStrDup(name);
-  if(outLongName) *outLongName = myStrDup(longName);
-  if(outStdName) *outStdName = myStrDup(stdName);
-  if(outUnit) *outUnit = myStrDup(unit);
+  if(outName) *outName = strdup(name);
+  if(outLongName) *outLongName = strdup(longName);
+  if(outStdName) *outStdName = strdup(stdName);
+  if(outUnit) *outUnit = strdup(unit);
 
   return zaxisType;
 }
@@ -388,24 +405,25 @@ int cdiGribIterator_levelType(CdiIterator* super, int levelSelector, char** outN
 static double logicalLevelValue2(long gribType, long storedValue, long power)
 {
   double factor = 1;
+  assert(power >= 0);
   while(power--) factor *= 10;      //this is precise up to factor == 22.
   switch(gribType)
     {
       case GRIB2_LTYPE_LANDDEPTH:
       case GRIB2_LTYPE_ISOBARIC:
       case GRIB2_LTYPE_SIGMA:
-        return storedValue*(1000/factor);      //The evaluation order allows the factors of ten to cancel out before rounding.
+        return (double)storedValue * (1000.0/factor);      //The evaluation order allows the factors of ten to cancel out before rounding.
 
       case 255:
         return 0;
 
       default:
-        return storedValue/factor;
+        return (double)storedValue/factor;
     }
 }
 
 //The output values must be preinitialized, this function does not always write them.
-static int readLevel2(grib_handle* gribHandle, const char* levelTypeKey, const char* powerKey, const char* valueKey, double* outValue1, double* outValue2)
+static int readLevel2(grib_handle *gribHandle, const char *levelTypeKey, const char *powerKey, const char *valueKey, double *outValue1, double *outValue2)
 {
   assert(levelTypeKey && powerKey && valueKey && outValue1 && outValue2);
 
@@ -426,14 +444,14 @@ static int readLevel2(grib_handle* gribHandle, const char* levelTypeKey, const c
           int status;
           //XXX: I'm not 100% sure about how the coordinate pairs are stored in the file.
           //     I'm assuming an array of pairs due to the example code in grib_api-1.12.3/examples/F90/set_pv.f90, but that may be wrong.
-          if((status = grib_get_double_element(gribHandle, "pv", value*2    , outValue1))) return status;
-          if((status = grib_get_double_element(gribHandle, "pv", value*2 + 1, outValue2))) return status;
+          if((status = grib_get_double_element(gribHandle, "pv", (int)value*2    , outValue1))) return status;
+          if((status = grib_get_double_element(gribHandle, "pv", (int)value*2 + 1, outValue2))) return status;
           break;
         }
 
       default:
         {
-          long power = gribGetLongDefault(gribHandle, powerKey, 0);  //1 byte
+          long power = 255 & gribGetLongDefault(gribHandle, powerKey, 0);  //1 byte
           if(power == 255) power = 0;
           long value = gribGetLongDefault(gribHandle, valueKey, 0);   //4 bytes
           *outValue1 = logicalLevelValue2(levelType, value, power);
@@ -442,9 +460,9 @@ static int readLevel2(grib_handle* gribHandle, const char* levelTypeKey, const c
   return CDI_NOERR;
 }
 
-int cdiGribIterator_level(CdiIterator* super, int levelSelector, double* outValue1, double* outValue2)
+int cdiGribIterator_level(CdiIterator *super, int levelSelector, double *outValue1, double *outValue2)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
   double trash;
   if(!outValue1) outValue1 = &trash;
   if(!outValue2) outValue2 = &trash;
@@ -466,25 +484,25 @@ int cdiGribIterator_level(CdiIterator* super, int levelSelector, double* outValu
       long levelType = (uint8_t)gribGetLongDefault(me->gribHandle, "indicatorOfTypeOfLevel", -1);    //1 byte
       if(levelType == 255)
         {}
-      else if(isGrib1DualLevel(levelType))
+      else if(isGrib1DualLevel((int)levelType))
         {
-          *outValue1 = gribGetLongDefault(me->gribHandle, (levelSelector ? "bottomLevel" : "topLevel"), 0);
+          *outValue1 = (double)gribGetLongDefault(me->gribHandle, (levelSelector ? "bottomLevel" : "topLevel"), 0);
         }
       else if(levelType == 100)
         {
-          *outValue1 = 100*gribGetLongDefault(me->gribHandle, "level", 0);        //2 bytes
+          *outValue1 = 100 * (double)gribGetLongDefault(me->gribHandle, "level", 0);        //2 bytes
         }
       else
         {
-          *outValue1 = gribGetLongDefault(me->gribHandle, "level", 0);        //2 bytes
+          *outValue1 = (double)gribGetLongDefault(me->gribHandle, "level", 0);        //2 bytes
         }
     }
   return CDI_NOERR;
 }
 
-int cdiGribIterator_zaxisUuid(CdiIterator* super, int* outVgridNumber, int* outLevelCount, unsigned char (*outUuid)[16])
+int cdiGribIterator_zaxisUuid(CdiIterator *super, int *outVgridNumber, int *outLevelCount, unsigned char outUuid[CDI_UUID_SIZE])
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
 
   if(outVgridNumber)
     {
@@ -500,57 +518,104 @@ int cdiGribIterator_zaxisUuid(CdiIterator* super, int* outVgridNumber, int* outL
     }
   if(outUuid)
     {
-      size_t size = sizeof(*outUuid);
-      if(grib_get_bytes(me->gribHandle, "uuidOfVGrid", *outUuid, &size)) return CDI_EINVAL;
-      if(size != sizeof(*outUuid)) return CDI_EUFSTRUCT;
+      size_t size = CDI_UUID_SIZE;
+      if(grib_get_bytes(me->gribHandle, "uuidOfVGrid", outUuid, &size)) return CDI_EINVAL;
+      if(size != CDI_UUID_SIZE) return CDI_EUFSTRUCT;
     }
 
   return CDI_NOERR;
 }
 
-char* cdiGribIterator_copyVariableName(CdiIterator* super)
+int cdiGribIterator_inqTile(CdiIterator *super, int *outTileIndex, int *outTileAttribute)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
+  int trash;
+  if(!outTileIndex) outTileIndex = &trash;
+  if(!outTileAttribute) outTileAttribute = &trash;
+
+  //Get the values if possible.
+  int error = CDI_NOERR;
+  long value;
+  if(grib_get_long(me->gribHandle, "tileIndex", &value)) error = CDI_EINVAL;
+  *outTileIndex = (int)value;
+  if(grib_get_long(me->gribHandle, "tileAttribute", &value)) error = CDI_EINVAL;
+  *outTileAttribute = (int)value;
+
+  //Ensure defined return values in case of failure.
+  if(error) *outTileIndex = *outTileAttribute = -1;
+  return error;
+}
+
+int cdiGribIterator_inqTileCount(CdiIterator *super, int *outTileCount, int *outTileAttributeCount)
+{
+  CdiGribIterator *me = (CdiGribIterator*)super;
+  int trash;
+  if(!outTileCount) outTileCount = &trash;
+  if(!outTileAttributeCount) outTileAttributeCount = &trash;
+
+  //Get the values if possible.
+  int error = CDI_NOERR;
+  long value;
+  if(grib_get_long(me->gribHandle, "numberOfTiles", &value)) error = CDI_EINVAL;
+  *outTileCount = (int)value;
+  if(grib_get_long(me->gribHandle, "numberOfTileAttributes", &value)) error = CDI_EINVAL;
+  *outTileAttributeCount = (int)value;
+
+  //Ensure defined return values in case of failure.
+  if(error) *outTileCount = *outTileAttributeCount = 0;
+  return error;
+}
+
+char *cdiGribIterator_copyVariableName(CdiIterator *super)
+{
+  CdiGribIterator *me = (CdiGribIterator*)super;
   return gribCopyString(me->gribHandle, "shortName");
 }
 
-void cdiGribIterator_readField(CdiIterator* super, double* buffer, size_t* nmiss)
+void cdiGribIterator_readField(CdiIterator *super, double *buffer, size_t *nmiss)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
 
+  GRIB_CHECK(my_grib_set_double(me->gribHandle, "missingValue", cdiDefaultMissval), 0);
   gribGetDoubleArray(me->gribHandle, "values", buffer);
   long gridType = gribGetLong(me->gribHandle, "gridDefinitionTemplateNumber");
   if(nmiss)
     {
-      *nmiss = (gridType >= 50 && gridType <= 53) ? 0 : (int)gribGetLong(me->gribHandle, "numberOfMissing");        //The condition excludes harmonic data.
+      *nmiss = (gridType >= 50 && gridType <= 53) ? (size_t)0 : (size_t)gribGetLong(me->gribHandle, "numberOfMissing");        //The condition excludes harmonic data.
     }
 }
 
-void cdiGribIterator_readFieldF(CdiIterator* super, float* buffer, size_t* nmiss)
+void cdiGribIterator_readFieldF(CdiIterator *super, float *buffer, size_t *nmiss)
 {
-  CdiGribIterator* me = (CdiGribIterator*)super;
+  CdiGribIterator *me = (CdiGribIterator*)super;
 
   size_t valueCount = gribGetArraySize(me->gribHandle, "values");
-  double* temp = malloc(valueCount*sizeof(*temp));
+  double *temp = (double *) Malloc(valueCount*sizeof(*temp));
   cdiGribIterator_readField(super, temp, nmiss);
-  for(size_t i = valueCount; i--; ) buffer[i] = temp[i];
-  free(temp);
+  for(size_t i = valueCount; i--; ) buffer[i] = (float)temp[i];
+  Free(temp);
 }
+#endif
 
-/**
+/*
 @Function cdiGribIterator_delete
 @Title Dispose off a CdiGribIterator instance.
 
-@Prototype void cdiGribIterator_delete(CdiGribIterator* me)
+@Prototype void cdiGribIterator_delete(CdiGribIterator *me)
 @Parameter
     @item me The iterator to delete.
 
 @Description
     Combined destructor and deallocator. Make sure to match every call to cdiGribIterator_clone() with a call to this function.
 */
-void cdiGribIterator_delete(CdiGribIterator* me)
+void cdiGribIterator_delete(CdiGribIterator *me)
 {
+#ifdef HAVE_LIBGRIB_API
   if(me) cdiGribIterator_condestruct(me, NULL, 0);
+#else
+  if (me)
+    xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
 
@@ -558,11 +623,11 @@ void cdiGribIterator_delete(CdiGribIterator* me)
 // callthroughs to provide direct access to the grib keys //////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
-/**
+/*
 @Function cdiGribIterator_inqEdition
 @Title Get the version of the GRIB standard that is used
 
-@Prototype int cdiGribIterator_inqEdition(CdiGribIterator* me)
+@Prototype int cdiGribIterator_inqEdition(CdiGribIterator *me)
 @Parameter
     @item me The iterator to operate on.
 
@@ -571,16 +636,21 @@ void cdiGribIterator_delete(CdiGribIterator* me)
 @Description
     Returns the version of the file format.
 */
-int cdiGribIterator_inqEdition(CdiGribIterator* me)
+int cdiGribIterator_inqEdition(CdiGribIterator *me)
 {
-  return gribEditionNumber(me->gribHandle);
+#ifdef HAVE_LIBGRIB_API
+  return (int)gribEditionNumber(me->gribHandle);
+#else
+  (void)me;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_getLong
 @Title Access to grib_get_long()
 
-@Prototype int cdiGribIterator_getLong(CdiGribIterator* me, const char* key, long* result)
+@Prototype int cdiGribIterator_getLong(CdiGribIterator *me, const char *key, long *result)
 @Parameter
     @item me The iterator to operate on.
     @item ... The arguments to the underlying GRIB-API function.
@@ -590,16 +660,23 @@ int cdiGribIterator_inqEdition(CdiGribIterator* me)
 @Description
     Callthrough to grib_get_long().
 */
-int cdiGribIterator_getLong(CdiGribIterator* me, const char* key, long* result)
+int cdiGribIterator_getLong(CdiGribIterator *me, const char *key, long *result)
 {
+#ifdef HAVE_LIBGRIB_API
   return grib_get_long(me->gribHandle, key, result);
+#else
+  (void)me;
+  (void)key;
+  (void)result;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_getLength
 @Title Access to grib_get_length()
 
-@Prototype int cdiGribIterator_getLength(CdiGribIterator* me, const char* key, size_t* result)
+@Prototype int cdiGribIterator_getLength(CdiGribIterator *me, const char *key, size_t *result)
 @Parameter
     @item me The iterator to operate on.
     @item ... The arguments to the underlying GRIB-API function.
@@ -609,16 +686,29 @@ int cdiGribIterator_getLong(CdiGribIterator* me, const char* key, long* result)
 @Description
     Callthrough to grib_get_length().
 */
-int cdiGribIterator_getLength(CdiGribIterator* me, const char* key, size_t* result)
+int cdiGribIterator_getLength(CdiGribIterator *me, const char *key, size_t *result)
 {
+#ifdef HAVE_GRIB_GET_LENGTH
   return grib_get_length(me->gribHandle, key, result);
+#elif defined(HAVE_LIBGRIB_API)
+  (void)me;
+  (void)key;
+  (void)result;
+  Error("grib_get_length() is not available, so cdiGribIterator_getLength() can't be used");
+  return -1;
+#else
+  (void)me;
+  (void)key;
+  (void)result;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_getString
 @Title Access to grib_get_string()
 
-@Prototype int cdiGribIterator_getString(CdiGribIterator* me, const char* key, char* result, size_t* length)
+@Prototype int cdiGribIterator_getString(CdiGribIterator *me, const char *key, char *result, size_t *length)
 @Parameter
     @item me The iterator to operate on.
     @item ... The arguments to the underlying GRIB-API function.
@@ -628,16 +718,24 @@ int cdiGribIterator_getLength(CdiGribIterator* me, const char* key, size_t* resu
 @Description
     Callthrough to grib_get_string().
 */
-int cdiGribIterator_getString(CdiGribIterator* me, const char* key, char* result, size_t* length)
+int cdiGribIterator_getString(CdiGribIterator *me, const char *key, char *result, size_t *length)
 {
+#ifdef HAVE_LIBGRIB_API
   return grib_get_string(me->gribHandle, key, result, length);
+#else
+  (void)me;
+  (void)key;
+  (void)result;
+  (void)length;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_inqLongValue
 @Title Get the value of a GRIB-API key as a long
 
-@Prototype long cdiGribIterator_inqLongValue(CdiGribIterator* me, const char* key)
+@Prototype long cdiGribIterator_inqLongValue(CdiGribIterator *me, const char *key)
 @Parameter
     @item me The iterator to operate on.
     @item key The GRIB-API key to retrieve.
@@ -648,16 +746,22 @@ int cdiGribIterator_getString(CdiGribIterator* me, const char* key, char* result
     Use this to fetch a grib value if you are certain that the given key must be present.
     This will abort the process if the key cannot be retrieved.
 */
-long cdiGribIterator_inqLongValue(CdiGribIterator* me, const char* key)
+long cdiGribIterator_inqLongValue(CdiGribIterator *me, const char *key)
 {
+#ifdef HAVE_LIBGRIB_API
   return gribGetLong(me->gribHandle, key);
+#else
+  (void)me;
+  (void)key;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_inqLongDefaultValue
 @Title Get the value of a GRIB-API key as a long
 
-@Prototype long cdiGribIterator_inqLongDefaultValue(CdiGribIterator* me, const char* key, long defaultValue)
+@Prototype long cdiGribIterator_inqLongDefaultValue(CdiGribIterator *me, const char *key, long defaultValue)
 @Parameter
     @item me The iterator to operate on.
     @item key The GRIB-API key to retrieve.
@@ -669,16 +773,23 @@ long cdiGribIterator_inqLongValue(CdiGribIterator* me, const char* key)
     Use this if you can handle failure to fetch the key by supplying a default value.
     This function cannot fail.
 */
-long cdiGribIterator_inqLongDefaultValue(CdiGribIterator* me, const char* key, long defaultValue)
+long cdiGribIterator_inqLongDefaultValue(CdiGribIterator *me, const char *key, long defaultValue)
 {
+#ifdef HAVE_LIBGRIB_API
   return gribGetLongDefault(me->gribHandle, key, defaultValue);
+#else
+  (void)me;
+  (void)key;
+  (void)defaultValue;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_inqStringValue
 @Title Safely retrieve a GRIB-API key with a string value
 
-@Prototype char* cdiGribIterator_inqStringValue(CdiGribIterator* me, const char* key)
+@Prototype char *cdiGribIterator_inqStringValue(CdiGribIterator *me, const char *key)
 @Parameter
     @item me The iterator to operate on.
     @item key The GRIB-API key to retrieve.
@@ -690,16 +801,22 @@ long cdiGribIterator_inqLongDefaultValue(CdiGribIterator* me, const char* key, l
     allocate memory accordingly, call grib_get_string(), and return the pointer to the new string.
     Returns NULL on failure.
 */
-char* cdiGribIterator_inqStringValue(CdiGribIterator* me, const char* key)
+char *cdiGribIterator_inqStringValue(CdiGribIterator *me, const char *key)
 {
+#ifdef HAVE_LIBGRIB_API
   return gribCopyString(me->gribHandle, key);
+#else
+  (void)me;
+  (void)key;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_getDouble
 @Title Access to grib_get_double()
 
-@Prototype int cdiGribIterator_getDouble(CdiGribIterator* me, const char* key, double* result)
+@Prototype int cdiGribIterator_getDouble(CdiGribIterator *me, const char *key, double *result)
 @Parameter
     @item me The iterator to operate on.
     @item ... The arguments to the underlying GRIB-API function.
@@ -709,16 +826,23 @@ char* cdiGribIterator_inqStringValue(CdiGribIterator* me, const char* key)
 @Description
     Callthrough to grib_get_double().
 */
-int cdiGribIterator_getDouble(CdiGribIterator* me, const char* key, double* result)
+int cdiGribIterator_getDouble(CdiGribIterator *me, const char *key, double *result)
 {
+#ifdef HAVE_LIBGRIB_API
   return grib_get_double(me->gribHandle, key, result);
+#else
+  (void)me;
+  (void)key;
+  (void)result;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_getSize
 @Title Access to grib_get_size()
 
-@Prototype int cdiGribIterator_getSize(CdiGribIterator* me, const char* key, size_t* result)
+@Prototype int cdiGribIterator_getSize(CdiGribIterator *me, const char *key, size_t *result)
 @Parameter
     @item me The iterator to operate on.
     @item ... The arguments to the underlying GRIB-API function.
@@ -728,16 +852,23 @@ int cdiGribIterator_getDouble(CdiGribIterator* me, const char* key, double* resu
 @Description
     Callthrough to grib_get_size().
 */
-int cdiGribIterator_getSize(CdiGribIterator* me, const char* key, size_t* result)
+int cdiGribIterator_getSize(CdiGribIterator *me, const char *key, size_t *result)
 {
+#ifdef HAVE_LIBGRIB_API
   return grib_get_size(me->gribHandle, key, result);
+#else
+  (void)me;
+  (void)key;
+  (void)result;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_getLongArray
 @Title Access to grib_get_long_array()
 
-@Prototype int cdiGribIterator_getLongArray(CdiGribIterator* me, const char* key, long* result, size_t* size)
+@Prototype int cdiGribIterator_getLongArray(CdiGribIterator *me, const char *key, long *result, size_t *size)
 @Parameter
     @item me The iterator to operate on.
     @item ... The arguments to the underlying GRIB-API function.
@@ -747,16 +878,24 @@ int cdiGribIterator_getSize(CdiGribIterator* me, const char* key, size_t* result
 @Description
     Callthrough to grib_get_long_array().
 */
-int cdiGribIterator_getLongArray(CdiGribIterator* me, const char* key, long* result, size_t* size)
+int cdiGribIterator_getLongArray(CdiGribIterator *me, const char *key, long *result, size_t *size)
 {
+#ifdef HAVE_LIBGRIB_API
   return grib_get_long_array(me->gribHandle, key, result, size);
+#else
+  (void)me;
+  (void)key;
+  (void)result;
+  (void)size;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_getDoubleArray
 @Title Access to grib_get_double_array()
 
-@Prototype int cdiGribIterator_getDoubleArray(CdiGribIterator* me, const char* key, double* result, size_t* size)
+@Prototype int cdiGribIterator_getDoubleArray(CdiGribIterator *me, const char *key, double *result, size_t *size)
 @Parameter
     @item me The iterator to operate on.
     @item ... The arguments to the underlying GRIB-API function.
@@ -766,16 +905,24 @@ int cdiGribIterator_getLongArray(CdiGribIterator* me, const char* key, long* res
 @Description
     Callthrough to grib_get_double_array().
 */
-int cdiGribIterator_getDoubleArray(CdiGribIterator* me, const char* key, double* result, size_t* size)
+int cdiGribIterator_getDoubleArray(CdiGribIterator *me, const char *key, double *result, size_t *size)
 {
+#ifdef HAVE_LIBGRIB_API
   return grib_get_double_array(me->gribHandle, key, result, size);
+#else
+  (void)me;
+  (void)key;
+  (void)result;
+  (void)size;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_inqDoubleValue
 @Title Get the value of a GRIB-API key as a double
 
-@Prototype double cdiGribIterator_inqDoubleValue(CdiGribIterator* me, const char* key)
+@Prototype double cdiGribIterator_inqDoubleValue(CdiGribIterator *me, const char *key)
 @Parameter
     @item me The iterator to operate on.
     @item key The GRIB-API key to retrieve.
@@ -786,16 +933,22 @@ int cdiGribIterator_getDoubleArray(CdiGribIterator* me, const char* key, double*
     Use this to fetch a grib value if you are certain that the given key must be present.
     This will abort the process if the key cannot be retrieved.
 */
-double cdiGribIterator_inqDoubleValue(CdiGribIterator* me, const char* key)
+double cdiGribIterator_inqDoubleValue(CdiGribIterator *me, const char *key)
 {
+#ifdef HAVE_LIBGRIB_API
   return gribGetDouble(me->gribHandle, key);
+#else
+  (void)me;
+  (void)key;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-/**
+/*
 @Function cdiGribIterator_inqDoubleDefaultValue
 @Title Get the value of a GRIB-API key as a double
 
-@Prototype double cdiGribIterator_inqDoubleDefaultValue(CdiGribIterator* me, const char* key, double defaultValue)
+@Prototype double cdiGribIterator_inqDoubleDefaultValue(CdiGribIterator *me, const char *key, double defaultValue)
 @Parameter
     @item me The iterator to operate on.
     @item key The GRIB-API key to retrieve.
@@ -807,9 +960,24 @@ double cdiGribIterator_inqDoubleValue(CdiGribIterator* me, const char* key)
     Use this if you can handle failure to fetch the key by supplying a default value.
     This function cannot fail.
 */
-double cdiGribIterator_inqDoubleDefaultValue(CdiGribIterator* me, const char* key, double defaultValue)
+double cdiGribIterator_inqDoubleDefaultValue(CdiGribIterator *me, const char *key, double defaultValue)
 {
+#ifdef HAVE_LIBGRIB_API
   return gribGetDoubleDefault(me->gribHandle, key, defaultValue);
+#else
+  (void)me;
+  (void)key;
+  (void)defaultValue;
+  xabort("CDI was compiled without GribAPI support, so you can't possibly have a valid CdiGribIterator* to call this function with");
+#endif
 }
 
-#endif
+/*
+ * Local Variables:
+ * c-file-style: "Java"
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * show-trailing-whitespace: t
+ * require-trailing-newline: t
+ * End:
+ */
