@@ -10,8 +10,6 @@
 #include "cdi.h"
 #include "cdi_int.h"
 
-#undef  UNDEFID
-#define UNDEFID -1
 
 /*int TableDefine = 0; */ /* Define new table also if the entry already exist */
                           /* This is needed for createtable */
@@ -25,8 +23,8 @@
 typedef struct
 {
   int    used;
-  PAR   *pars;
   int    npars;
+  PAR   *pars;
   int    modelID;
   int    number;
   char  *name;
@@ -58,17 +56,17 @@ void tableDefEntry(int tableID, int id, const char *name,
   parTable[tableID].pars[item].longname = NULL;
   parTable[tableID].pars[item].units    = NULL;
 
-  if ( name && strlen(name) > 0 )
+  if ( name && name[0] )
     {
       parTable[tableID].pars[item].name     = strdupx(name);
       parTable[tableID].pars[item].dupflags |= TABLE_DUP_NAME;
     }
-  if ( longname && strlen(longname) > 0 )
+  if ( longname && longname[0] )
     {
       parTable[tableID].pars[item].longname = strdupx(longname);
       parTable[tableID].pars[item].dupflags |= TABLE_DUP_LONGNAME;
     }
-  if ( units && strlen(units) > 0 )
+  if ( units && units[0] )
     {
       parTable[tableID].pars[item].units    = strdupx(units);
       parTable[tableID].pars[item].dupflags |= TABLE_DUP_UNITS;
@@ -96,8 +94,8 @@ static void parTableInitEntry(int tableID)
   parTable[tableID].used    = 0;
   parTable[tableID].pars    = NULL;
   parTable[tableID].npars   = 0;
-  parTable[tableID].modelID = UNDEFID;
-  parTable[tableID].number  = UNDEFID;
+  parTable[tableID].modelID = CDI_UNDEFID;
+  parTable[tableID].number  = CDI_UNDEFID;
   parTable[tableID].name    = NULL;
 }
 
@@ -122,14 +120,14 @@ static void parTableFinalize(void)
         for (int item = 0; item < npars; ++item)
           {
             if (parTable[tableID].pars[item].dupflags & TABLE_DUP_NAME)
-              free((void *)parTable[tableID].pars[item].name);
+              Free((void *)parTable[tableID].pars[item].name);
             if (parTable[tableID].pars[item].dupflags & TABLE_DUP_LONGNAME)
-              free((void *)parTable[tableID].pars[item].longname);
+              Free((void *)parTable[tableID].pars[item].longname);
             if (parTable[tableID].pars[item].dupflags & TABLE_DUP_UNITS)
-              free((void *)parTable[tableID].pars[item].units);
+              Free((void *)parTable[tableID].pars[item].units);
           }
-        free(parTable[tableID].pars);
-        free(parTable[tableID].name);
+        Free(parTable[tableID].pars);
+        Free(parTable[tableID].name);
       }
 }
 
@@ -193,8 +191,7 @@ decodeForm1(char *pline, char *name, char *longname, char *units)
   else
     return (0);
 
-  len = strlen(pline);
-  if ( len == 0 ) return (0);
+  if ( pline[0] == 0 ) return (0);
 
   /* Format 1 : code name add mult longname [units] */
   /* FIXME: successful parse isn't verified */
@@ -251,7 +248,6 @@ decodeForm2(char *pline, char *name, char *longname, char *units)
 {
   /* Format 2 : code | name | longname | units */
   char *pend;
-  size_t len;
 
   pline = strchr(pline, '|');
   pline++;
@@ -264,7 +260,7 @@ decodeForm2(char *pline, char *name, char *longname, char *units)
         {
           pend = pline;
           while ( ! isspace((int) *pend) ) pend++;
-          len = (size_t)(pend - pline);
+          size_t len = (size_t)(pend - pline);
           if ( len > 0 )
             {
               memcpy(name, pline, len);
@@ -276,7 +272,7 @@ decodeForm2(char *pline, char *name, char *longname, char *units)
         {
           pend--;
           while ( isspace((int) *pend) ) pend--;
-          len = (size_t)(pend - pline + 1);
+          size_t len = (size_t)(pend - pline + 1);
           if ( len > 0 )
             {
               memcpy(name, pline, len);
@@ -294,12 +290,14 @@ decodeForm2(char *pline, char *name, char *longname, char *units)
   if ( !pend ) pend = strchr(pline, 0);
   pend--;
   while ( isspace((int) *pend) ) pend--;
-  len = (size_t)(pend - pline + 1);
-  if ( len > 0 )
-    {
-      memcpy(longname, pline, len);
-      longname[len] = 0;
-    }
+  {
+    size_t len = (size_t)(pend - pline + 1);
+    if ( len > 0 )
+      {
+        memcpy(longname, pline, len);
+        longname[len] = 0;
+      }
+  }
 
   pline = strchr(pline, '|');
   if ( pline )
@@ -325,7 +323,7 @@ int tableRead(const char *tablefile)
   int lnr = 0;
   int id;
   char name[256], longname[256], units[256];
-  int tableID = UNDEFID;
+  int tableID = CDI_UNDEFID;
   int err;
   char *tablename;
   FILE *tablefp;
@@ -333,7 +331,7 @@ int tableRead(const char *tablefile)
   tablefp = fopen(tablefile, "r");
   if ( tablefp == NULL ) return (tableID);
 
-  tablename = strrchr(tablefile, '/');
+  tablename = (char* )strrchr(tablefile, '/');
   if ( tablename == 0 ) tablename = (char *) tablefile;
   else                  tablename++;
 
@@ -360,7 +358,7 @@ int tableRead(const char *tablefile)
       */
       if ( id == 0 ) continue;
 
-      while ( isdigit((int) *pline) ) pline++; 
+      while ( isdigit((int) *pline) ) pline++;
 
       if ( strchr(pline, '|') )
 	err = decodeForm2(pline, name, longname, units);
@@ -369,7 +367,7 @@ int tableRead(const char *tablefile)
 
       if ( err ) continue;
 
-      if ( strlen(name) == 0 ) sprintf(name, "var%d", id);
+      if ( name[0] == 0 ) sprintf(name, "var%d", id);
 
       tableDefEntry(tableID, id, name, longname, units);
     }
@@ -379,7 +377,7 @@ int tableRead(const char *tablefile)
 
 static int tableFromEnv(int modelID, int tablenum)
 {
-  int tableID = UNDEFID;
+  int tableID = CDI_UNDEFID;
   char tablename[256] = {'\0'};
   int tablenamefound = 0;
 
@@ -397,7 +395,7 @@ static int tableFromEnv(int modelID, int tablenum)
   else
     {
       int instID = modelInqInstitut(modelID);
-      if ( instID != UNDEFID )
+      if ( instID != CDI_UNDEFID )
 	{
           const char *instName;
 	  if ( (instName = institutInqNamePtr(instID)) )
@@ -422,7 +420,7 @@ static int tableFromEnv(int modelID, int tablenum)
       lenf = strlen(tablename);
       /* if (tablePath) printf("tablePath = %s\n", tablePath); */
       /* if (tablename) printf("tableName = %s\n", tablename); */
-      tablefile = (char *) malloc(lenp+lenf+3);
+      tablefile = (char *) Malloc(lenp+lenf+3);
       if ( tablePath )
 	{
 	  strcpy(tablefile, tablePath);
@@ -434,14 +432,14 @@ static int tableFromEnv(int modelID, int tablenum)
       /* if (tablefile) printf("tableFile = %s\n", tablefile); */
 
       tableID = tableRead(tablefile);
-      if ( tableID != UNDEFID )
+      if ( tableID != CDI_UNDEFID )
 	{
 	  tableDefModelID(tableID, modelID);
 	  tableDefNum(tableID, tablenum);
 	}
       /* printf("tableID = %d %s\n", tableID, tablefile); */
 
-      free(tablefile);
+      Free(tablefile);
     }
 
   return (tableID);
@@ -449,8 +447,8 @@ static int tableFromEnv(int modelID, int tablenum)
 
 int tableInq(int modelID, int tablenum, const char *tablename)
 {
-  int tableID = UNDEFID;
-  int modelID2 = UNDEFID;
+  int tableID = CDI_UNDEFID;
+  int modelID2 = CDI_UNDEFID;
   char tablefile[256] = {'\0'};
 
   if ( ! ParTableInit ) parTableInit();
@@ -472,7 +470,7 @@ int tableInq(int modelID, int tablenum, const char *tablename)
 	      if ( memcmp(parTable[tableID].name, tablename, len) == 0 ) break;
 	    }
 	}
-      if ( tableID == MAX_TABLE ) tableID = UNDEFID;
+      if ( tableID == MAX_TABLE ) tableID = CDI_UNDEFID;
       if ( CDI_Debug )
 	Message("tableID = %d tablename = %s", tableID, tablename);
     }
@@ -487,11 +485,11 @@ int tableInq(int modelID, int tablenum, const char *tablename)
 	    }
 	}
 
-      if ( tableID == MAX_TABLE ) tableID = UNDEFID;
+      if ( tableID == MAX_TABLE ) tableID = CDI_UNDEFID;
 
-      if ( tableID == UNDEFID )
+      if ( tableID == CDI_UNDEFID )
 	{
-	  if ( modelID != UNDEFID )
+	  if ( modelID != CDI_UNDEFID )
 	    {
               const char *modelName;
 	      if ( (modelName = modelInqNamePtr(modelID)) )
@@ -503,7 +501,7 @@ int tableInq(int modelID, int tablenum, const char *tablename)
 		  modelID2 = modelInq(-1, 0, tablefile);
 		}
 	    }
-	  if ( modelID2 != UNDEFID )
+	  if ( modelID2 != CDI_UNDEFID )
 	    for ( tableID = 0; tableID < MAX_TABLE; tableID++ )
 	      {
 		if ( parTable[tableID].used )
@@ -514,9 +512,9 @@ int tableInq(int modelID, int tablenum, const char *tablename)
 	      }
 	}
 
-      if ( tableID == MAX_TABLE ) tableID = UNDEFID;
+      if ( tableID == MAX_TABLE ) tableID = CDI_UNDEFID;
 
-      if ( tableID == UNDEFID && modelID != UNDEFID )
+      if ( tableID == CDI_UNDEFID && modelID != CDI_UNDEFID )
 	tableID = tableFromEnv(modelID, tablenum);
 
       if ( CDI_Debug )
@@ -529,14 +527,14 @@ int tableInq(int modelID, int tablenum, const char *tablename)
 
 int tableDef(int modelID, int tablenum, const char *tablename)
 {
-  int tableID = UNDEFID;
+  int tableID = CDI_UNDEFID;
 
   if ( ! ParTableInit ) parTableInit();
   /*
-  if ( ! (modelID == UNDEFID && tablenum == 0) )
+  if ( ! (modelID == CDI_UNDEFID && tablenum == 0) )
     tableID = tableInq(modelID, tablenum, tablename);
     */
-  if ( tableID == UNDEFID )
+  if ( tableID == CDI_UNDEFID )
     {
       tableID = tableNewEntry();
 
@@ -545,7 +543,7 @@ int tableDef(int modelID, int tablenum, const char *tablename)
       if ( tablename )
 	parTable[tableID].name = strdupx(tablename);
 
-      parTable[tableID].pars = (PAR *) malloc(MAX_PARS * sizeof(PAR));
+      parTable[tableID].pars = (PAR *) Malloc(MAX_PARS * sizeof(PAR));
     }
 
   return (tableID);
@@ -618,7 +616,7 @@ void tableWrite(const char *ptfile, int tableID)
   if ( CDI_Debug )
     Message("write parameter table %d to %s", tableID, ptfile);
 
-  if ( tableID == UNDEFID )
+  if ( tableID == CDI_UNDEFID )
     {
       Warning("parameter table ID undefined");
       return;
@@ -730,7 +728,7 @@ void tableFWriteC(FILE *ptfp, int tableID)
   char tablename[256];
 
 
-  if ( tableID == UNDEFID )
+  if ( tableID == CDI_UNDEFID )
     {
       Warning("parameter table ID undefined");
       return;
@@ -798,7 +796,7 @@ int tableInqParCode(int tableID, char *varname, int *code)
 {
   int err = 1;
 
-  if ( tableID != UNDEFID && varname != NULL )
+  if ( tableID != CDI_UNDEFID && varname != NULL )
     {
       int npars = parTable[tableID].npars;
       for ( int item = 0; item < npars; item++ )
@@ -821,7 +819,7 @@ int tableInqParName(int tableID, int code, char *varname)
 {
   int err = 1;
 
-  if ( tableID != UNDEFID )
+  if ( tableID >= 0 && tableID < MAX_TABLE )
     {
       int npars = parTable[tableID].npars;
       for ( int item = 0; item < npars; item++ )
@@ -835,6 +833,10 @@ int tableInqParName(int tableID, int code, char *varname)
 	    }
 	}
     }
+  else if ( tableID == CDI_UNDEFID )
+    { }
+  else
+    Error("Invalid table ID %d", tableID);
 
   return (err);
 }
@@ -844,7 +846,7 @@ const char *tableInqParNamePtr(int tableID, int code)
 {
   const char *name = NULL;
 
-  if ( tableID != UNDEFID )
+  if ( tableID != CDI_UNDEFID )
     {
       int npars = parTable[tableID].npars;
       for ( int item = 0; item < npars; item++ )
@@ -865,7 +867,7 @@ const char *tableInqParLongnamePtr(int tableID, int code)
 {
   const char *longname = NULL;
 
-  if ( tableID != UNDEFID )
+  if ( tableID != CDI_UNDEFID )
     {
       int npars = parTable[tableID].npars;
       for ( int item = 0; item < npars; item++ )
@@ -886,7 +888,7 @@ const char *tableInqParUnitsPtr(int tableID, int code)
 {
   const char *units = NULL;
 
-  if ( tableID != UNDEFID )
+  if ( tableID != CDI_UNDEFID )
     {
       int npars = parTable[tableID].npars;
       for ( int item = 0; item < npars; item++ )
@@ -905,12 +907,12 @@ const char *tableInqParUnitsPtr(int tableID, int code)
 
 int tableInqParLongname(int tableID, int code, char *longname)
 {
-  if ( ((tableID >= 0) & (tableID < MAX_TABLE)) | (tableID == UNDEFID) ) { } else
+  if ( ((tableID >= 0) & (tableID < MAX_TABLE)) | (tableID == CDI_UNDEFID) ) { } else
     Error("Invalid table ID %d", tableID);
 
   int err = 1;
 
-  if ( tableID != UNDEFID )
+  if ( tableID != CDI_UNDEFID )
     {
       int npars = parTable[tableID].npars;
       for ( int item = 0; item < npars; item++ )
@@ -932,12 +934,12 @@ int tableInqParLongname(int tableID, int code, char *longname)
 int tableInqParUnits(int tableID, int code, char *units)
 {
 
-  if ( ((tableID >= 0) & (tableID < MAX_TABLE)) | (tableID == UNDEFID) ) { } else
+  if ( ((tableID >= 0) & (tableID < MAX_TABLE)) | (tableID == CDI_UNDEFID) ) { } else
     Error("Invalid table ID %d", tableID);
 
   int err = 1;
 
-  if ( tableID != UNDEFID )
+  if ( tableID != CDI_UNDEFID )
     {
       int npars = parTable[tableID].npars;
       for ( int item = 0; item < npars; item++ )
@@ -959,7 +961,7 @@ int tableInqParUnits(int tableID, int code, char *units)
 void tableInqPar(int tableID, int code, char *name, char *longname, char *units)
 {
 
-  if ( ((tableID >= 0) & (tableID < MAX_TABLE)) | (tableID == UNDEFID) ) { } else
+  if ( ((tableID >= 0) & (tableID < MAX_TABLE)) | (tableID == CDI_UNDEFID) ) { } else
     Error("Invalid table ID %d", tableID);
 
   int npars = parTable[tableID].npars;
