@@ -112,7 +112,7 @@ const char *tunitNamePtr(int unitID)
   else
     name = Timeunits[0];
 
-  return (name);
+  return name;
 }
 
 #if 0
@@ -150,7 +150,8 @@ static
 void taxisDefaultValue(taxis_t* taxisptr)
 {
   taxisptr->self        = CDI_UNDEFID;
-  taxisptr->used        = FALSE;
+  taxisptr->used        = false;
+  taxisptr->datatype    = CDI_DATATYPE_FLT64;
   taxisptr->type        = DefaultTimeType;
   taxisptr->vdate       = 0;
   taxisptr->vtime       = 0;
@@ -161,8 +162,8 @@ void taxisDefaultValue(taxis_t* taxisptr)
   taxisptr->calendar    = cdiDefaultCalendar;
   taxisptr->unit        = DefaultTimeUnit;
   taxisptr->numavg      = 0;
-  taxisptr->climatology = FALSE;
-  taxisptr->has_bounds  = FALSE;
+  taxisptr->climatology = false;
+  taxisptr->has_bounds  = false;
   taxisptr->vdate_lb    = 0;
   taxisptr->vtime_lb    = 0;
   taxisptr->vdate_ub    = 0;
@@ -187,20 +188,19 @@ taxisNewEntry(cdiResH resH)
       reshReplace(resH, taxisptr, &taxisOps);
     }
 
-  return (taxisptr);
+  return taxisptr;
 }
 
 static
 void taxisInit (void)
 {
-  static int taxisInitialized = 0;
-  char *env;
+  static bool taxisInitialized = false;
 
   if ( taxisInitialized ) return;
 
-  taxisInitialized = 1;
+  taxisInitialized = true;
 
-  env = getenv("TAXIS_DEBUG");
+  char *env = getenv("TAXIS_DEBUG");
   if ( env ) TAXIS_Debug = atoi(env);
 }
 
@@ -248,8 +248,7 @@ taxisDefRtime(taxisID, 120000);
 */
 int taxisCreate(int taxistype)
 {
-  if ( CDI_Debug )
-    Message("taxistype: %d", taxistype);
+  if ( CDI_Debug ) Message("taxistype: %d", taxistype);
 
   taxisInit ();
 
@@ -258,10 +257,9 @@ int taxisCreate(int taxistype)
 
   int taxisID = taxisptr->self;
 
-  if ( CDI_Debug )
-    Message("taxisID: %d", taxisID);
+  if ( CDI_Debug ) Message("taxisID: %d", taxisID);
 
-  return (taxisID);
+  return taxisID;
 }
 
 void taxisDestroyKernel(taxis_t *taxisptr)
@@ -303,11 +301,11 @@ int taxisDuplicate(int taxisID1)
 
   int taxisID2 = taxisptr2->self;
 
-  if ( CDI_Debug )
-    Message("taxisID2: %d", taxisID2);
+  if ( CDI_Debug ) Message("taxisID2: %d", taxisID2);
 
   ptaxisCopy(taxisptr2, taxisptr1);
-  return (taxisID2);
+
+  return taxisID2;
 }
 
 
@@ -554,16 +552,14 @@ The valid CDI time types are TAXIS_ABSOLUTE and TAXIS_RELATIVE.
 int taxisInqType(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
-
-  return (taxisptr->type);
+  return taxisptr->type;
 }
 
 
 int taxisHasBounds(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
-
-  return (taxisptr->has_bounds);
+  return taxisptr->has_bounds;
 }
 
 
@@ -571,9 +567,9 @@ void taxisDeleteBounds(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  if (taxisptr->has_bounds != FALSE)
+  if ( taxisptr->has_bounds )
     {
-      taxisptr->has_bounds = FALSE;
+      taxisptr->has_bounds = false;
       reshSetStatus(taxisID, &taxisOps, RESH_DESYNC_IN_USE);
     }
 }
@@ -582,7 +578,7 @@ void taxisDeleteBounds(int taxisID)
 void taxisCopyTimestep(int taxisID2, int taxisID1)
 {
   taxis_t *taxisptr1 = (taxis_t *)reshGetVal(taxisID1, &taxisOps),
-    *taxisptr2 = (taxis_t *)reshGetVal(taxisID2, &taxisOps);
+          *taxisptr2 = (taxis_t *)reshGetVal(taxisID2, &taxisOps);
 
   reshLock();
 
@@ -629,8 +625,7 @@ The function @func{taxisInqVdate} returns the verification date of a Time axis.
 int taxisInqVdate(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
-
-  return (taxisptr->vdate);
+  return taxisptr->vdate;
 }
 
 
@@ -647,13 +642,13 @@ void taxisDefVdateBounds(int taxisID, int vdate_lb, int vdate_ub)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  if (taxisptr->vdate_lb != vdate_lb
-      || taxisptr->vdate_ub != vdate_ub
-      || taxisptr->has_bounds != TRUE)
+  if ( taxisptr->vdate_lb != vdate_lb
+       || taxisptr->vdate_ub != vdate_ub
+       || taxisptr->has_bounds == false )
     {
       taxisptr->vdate_lb = vdate_lb;
       taxisptr->vdate_ub = vdate_ub;
-      taxisptr->has_bounds = TRUE;
+      taxisptr->has_bounds = true;
       reshSetStatus(taxisID, &taxisOps, RESH_DESYNC_IN_USE);
     }
 }
@@ -677,8 +672,7 @@ The function @func{taxisInqVtime} returns the verification time of a Time axis.
 int taxisInqVtime(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
-
-  return (taxisptr->vtime);
+  return taxisptr->vtime;
 }
 
 
@@ -695,13 +689,13 @@ void taxisDefVtimeBounds(int taxisID, int vtime_lb, int vtime_ub)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  if (taxisptr->vtime_lb != vtime_lb
-      || taxisptr->vtime_ub != vtime_ub
-      || taxisptr->has_bounds != TRUE)
+  if ( taxisptr->vtime_lb != vtime_lb
+       || taxisptr->vtime_ub != vtime_ub
+       || taxisptr->has_bounds == false )
     {
       taxisptr->vtime_lb = vtime_lb;
       taxisptr->vtime_ub = vtime_ub;
-      taxisptr->has_bounds = TRUE;
+      taxisptr->has_bounds = true;
       reshSetStatus(taxisID, &taxisOps, RESH_DESYNC_IN_USE);
     }
 }
@@ -733,7 +727,7 @@ int taxisInqRdate(int taxisID)
       reshSetStatus(taxisID, &taxisOps, RESH_DESYNC_IN_USE);
     }
 
-  return (taxisptr->rdate);
+  return taxisptr->rdate;
 }
 
 /*
@@ -763,7 +757,7 @@ int taxisInqRtime(int taxisID)
       reshSetStatus(taxisID, &taxisOps, RESH_DESYNC_IN_USE);
     }
 
-  return (taxisptr->rtime);
+  return taxisptr->rtime;
 }
 
 /*
@@ -792,7 +786,7 @@ int taxisInqFdate(int taxisID)
       taxisptr->ftime = taxisptr->vtime;
     }
 
-  return (taxisptr->fdate);
+  return taxisptr->fdate;
 }
 
 /*
@@ -821,7 +815,7 @@ int taxisInqFtime(int taxisID)
       taxisptr->ftime = taxisptr->vtime;
     }
 
-  return (taxisptr->ftime);
+  return taxisptr->ftime;
 }
 
 /*
@@ -845,55 +839,55 @@ The valid CDI calendar types are @func{CALENDAR_STANDARD}, @func{CALENDAR_PROLEP
 */
 int taxisInqCalendar(int taxisID)
 {
-  taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  return (taxisptr->calendar);
+  taxis_t *taxisptr = (taxis_t *) reshGetVal ( taxisID, &taxisOps );
+  return taxisptr->calendar;
 }
 
 
 int taxisInqTunit(int taxisID)
 {
-  taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  return (taxisptr->unit);
+  taxis_t *taxisptr = (taxis_t *) reshGetVal ( taxisID, &taxisOps );
+  return taxisptr->unit;
 }
 
 
 int taxisInqForecastTunit(int taxisID)
 {
-  taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  return (taxisptr->fc_unit);
+  taxis_t *taxisptr = (taxis_t *) reshGetVal ( taxisID, &taxisOps );
+  return taxisptr->fc_unit;
 }
 
 
 double taxisInqForecastPeriod(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  return (taxisptr->fc_period);
+  return taxisptr->fc_period;
 }
 
 
 int taxisInqNumavg(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  return (taxisptr->numavg);
+  return taxisptr->numavg;
 }
 
 
 taxis_t *taxisPtr(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
-
-  return (taxisptr);
+  return taxisptr;
 }
 
-void
-ptaxisDefName(taxis_t *taxisptr, const char *name)
+
+void ptaxisDefDatatype(taxis_t *taxisptr, int datatype)
 {
-  if (name)
+  taxisptr->datatype = datatype;
+}
+
+
+void ptaxisDefName(taxis_t *taxisptr, const char *name)
+{
+  if ( name )
     {
       size_t len = strlen(name);
       delete_refcount_string(taxisptr->name);
@@ -902,10 +896,10 @@ ptaxisDefName(taxis_t *taxisptr, const char *name)
     }
 }
 
-void
-ptaxisDefLongname(taxis_t *taxisptr, const char *longname)
+
+void ptaxisDefLongname(taxis_t *taxisptr, const char *longname)
 {
-  if (longname)
+  if ( longname )
     {
       size_t len = strlen(longname);
       delete_refcount_string(taxisptr->longname);
@@ -918,8 +912,6 @@ ptaxisDefLongname(taxis_t *taxisptr, const char *longname)
 static void
 cdiDecodeTimevalue(int timeunit, double timevalue, int *days, int *secs)
 {
-  static int lwarn = TRUE;
-
   *days = 0;
   *secs = 0;
 
@@ -964,10 +956,11 @@ cdiDecodeTimevalue(int timeunit, double timevalue, int *days, int *secs)
     }
   else
     {
+      static bool lwarn = true;
       if ( lwarn )
 	{
 	  Warning("timeunit %s unsupported!", tunitNamePtr(timeunit));
-	  lwarn = FALSE;
+	  lwarn = false;
 	}
     }
 }
@@ -975,8 +968,6 @@ cdiDecodeTimevalue(int timeunit, double timevalue, int *days, int *secs)
 static
 void cdiEncodeTimevalue(int days, int secs, int timeunit, double *timevalue)
 {
-  static int lwarn = TRUE;
-
   if ( timeunit == TUNIT_SECOND )
     {
       *timevalue = days*86400. + secs;
@@ -1000,10 +991,11 @@ void cdiEncodeTimevalue(int days, int secs, int timeunit, double *timevalue)
     }
   else
     {
+      static bool lwarn = true;
       if ( lwarn )
 	{
 	  Warning("timeunit %s unsupported!", tunitNamePtr(timeunit));
-	  lwarn = FALSE;
+	  lwarn = false;
 	}
     }
 }
@@ -1067,28 +1059,24 @@ void timeval2vtime(double timevalue, taxis_t *taxis, int *vdate, int *vtime)
 
 double vtime2timeval(int vdate, int vtime, taxis_t *taxis)
 {
-  int ryear, rmonth;
-  int year, month, day, hour, minute, second;
-  int rdate, rtime;
   double value = 0;
-  int timeunit;
-  int timeunit0;
-  int calendar;
   int julday1, secofday1, julday2, secofday2, days, secs;
 
-  timeunit = (*taxis).unit;
-  calendar = (*taxis).calendar;
+  int timeunit = (*taxis).unit;
+  int calendar = (*taxis).calendar;
 
-  rdate = (*taxis).rdate;
-  rtime = (*taxis).rtime;
+  int rdate = (*taxis).rdate;
+  int rtime = (*taxis).rtime;
   if ( rdate == -1 )
     {
       rdate  = (*taxis).vdate;
       rtime  = (*taxis).vtime;
     }
 
-  if ( rdate == 0 && rtime == 0 && vdate == 0 && vtime == 0 ) return(value);
+  if ( rdate == 0 && rtime == 0 && vdate == 0 && vtime == 0 ) return value;
 
+  int ryear, rmonth;
+  int year, month, day, hour, minute, second;
   cdiDecodeDate(rdate, &ryear, &rmonth, &day);
   cdiDecodeTime(rtime, &hour, &minute, &second);
 
@@ -1097,7 +1085,7 @@ double vtime2timeval(int vdate, int vtime, taxis_t *taxis)
   cdiDecodeDate(vdate, &year, &month, &day);
   cdiDecodeTime(vtime, &hour, &minute, &second);
 
-  timeunit0 = timeunit;
+  int timeunit0 = timeunit;
 
   if ( timeunit == TUNIT_MONTH && calendar == CALENDAR_360DAYS )
     {
@@ -1106,17 +1094,15 @@ double vtime2timeval(int vdate, int vtime, taxis_t *taxis)
 
   if ( timeunit == TUNIT_MONTH || timeunit == TUNIT_YEAR )
     {
-      int nmonth, dpm;
-
       value = (year-ryear)*12 - rmonth + month;
 
-      nmonth = (int) value;
+      int nmonth = (int) value;
       month -= nmonth;
 
       while ( month > 12 ) { month -= 12; year++; }
       while ( month <  1 ) { month += 12; year--; }
 
-      dpm = days_per_month(calendar, year, month);
+      int dpm = days_per_month(calendar, year, month);
 
       encode_caldaysec(calendar, year, month, day, hour, minute, second, &julday2, &secofday2);
 
@@ -1140,39 +1126,35 @@ double vtime2timeval(int vdate, int vtime, taxis_t *taxis)
       value /= 30;
     }
 
-  return (value);
+  return value;
 }
 
 
-static void conv_timeval(double timevalue, int *rvdate, int *rvtime)
+static
+void conv_timeval(double timevalue, int *rvdate, int *rvtime)
 {
-  int vdate = 0, vtime = 0;
-  int hour, minute, second;
   int daysec;
 
-  vdate = (int) timevalue;
+  int vdate = (int) timevalue;
   if ( vdate < 0 )
     daysec = (int) (-(timevalue - vdate)*86400 + 0.01);
   else
     daysec = (int) ( (timevalue - vdate)*86400 + 0.01);
 
-  hour   =  daysec / 3600;
-  minute = (daysec - hour*3600)/60;
-  second =  daysec - hour*3600 - minute*60;
-  vtime  = cdiEncodeTime(hour, minute, second);
+  int hour   =  daysec / 3600;
+  int minute = (daysec - hour*3600)/60;
+  int second =  daysec - hour*3600 - minute*60;
+  int vtime  = cdiEncodeTime(hour, minute, second);
 
   *rvdate = vdate;
   *rvtime = vtime;
 }
 
 
-static void
-splitTimevalue(double timevalue, int timeunit, int *date, int *time)
+static
+void splitTimevalue(double timevalue, int timeunit, int *date, int *time)
 {
   int vdate = 0, vtime = 0;
-  int hour, minute, second;
-  int year, month, day;
-  static int lwarn = TRUE;
 
   if ( timeunit == TUNIT_SECOND )
     {
@@ -1212,16 +1194,19 @@ splitTimevalue(double timevalue, int timeunit, int *date, int *time)
     }
   else
     {
+      static bool lwarn = true;
       if ( lwarn )
 	{
 	  Warning("timeunit %s unsupported!", tunitNamePtr(timeunit));
-	  lwarn = FALSE;
+	  lwarn = false;
 	}
     }
 
   /* verify date and time */
 
+  int year, month, day;
   cdiDecodeDate(vdate, &year, &month, &day);
+  int hour, minute, second;
   cdiDecodeTime(vtime, &hour, &minute, &second);
 
   if ( month > 17 || day > 31 || hour > 23 || minute > 59 || second > 59 )
@@ -1236,9 +1221,10 @@ splitTimevalue(double timevalue, int timeunit, int *date, int *time)
       vdate = cdiEncodeDate(year, month, day);
       vtime = cdiEncodeTime(hour, minute, second);
 
+      static bool lwarn = true;
       if ( lwarn )
         {
-          lwarn = FALSE;
+          lwarn = false;
           Warning("Reset wrong date/time to %4.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d!",
                   year, month, day, hour, minute, second);
         }
@@ -1251,22 +1237,19 @@ splitTimevalue(double timevalue, int timeunit, int *date, int *time)
 
 void cdiSetForecastPeriod(double timevalue, taxis_t *taxis)
 {
-  int year, month, day, hour, minute, second;
-  int vdate, vtime;
-  int timeunit;
-  int calendar;
   int julday, secofday, days, secs;
 
   (*taxis).fc_period = timevalue;
 
-  timeunit = (*taxis).fc_unit;
-  calendar = (*taxis).calendar;
+  int timeunit = (*taxis).fc_unit;
+  int calendar = (*taxis).calendar;
 
-  vdate  = (*taxis).vdate;
-  vtime  = (*taxis).vtime;
+  int vdate = (*taxis).vdate;
+  int vtime = (*taxis).vtime;
 
   if ( vdate == 0 && vtime == 0 && DBL_IS_EQUAL(timevalue, 0.) ) return;
 
+  int year, month, day, hour, minute, second;
   cdiDecodeDate(vdate, &year, &month, &day);
   cdiDecodeTime(vtime, &hour, &minute, &second);
 
@@ -1355,7 +1338,7 @@ double cdiEncodeTimeval(int date, int time, taxis_t *taxis)
   else
     timevalue = vtime2timeval(date, time, taxis);
 
-  return (timevalue);
+  return timevalue;
 }
 
 
@@ -1371,6 +1354,7 @@ void ptaxisCopy(taxis_t *dest, taxis_t *source)
 
   /* memcpy(dest, source, sizeof(taxis_t)); */
   dest->used        = source->used;
+  dest->datatype    = source->datatype;
   dest->type        = source->type;
   dest->vdate       = source->vdate;
   dest->vtime       = source->vtime;
@@ -1435,13 +1419,13 @@ taxisPrintKernel(taxis_t * taxisptr, FILE * fp)
           "fc_unit     = %d\n"
           "fc_period   = %g\n"
           "\n", taxisptr->self, taxisptr->self,
-          taxisptr->used, taxisptr->type,
+          (int)taxisptr->used, taxisptr->type,
           taxisptr->vdate, taxisptr->vtime,
           taxisptr->rdate, taxisptr->rtime,
           taxisptr->fdate, taxisptr->ftime,
           taxisptr->calendar, taxisptr->unit,
-          taxisptr->numavg, taxisptr->climatology,
-          taxisptr->has_bounds,
+          taxisptr->numavg, (int)taxisptr->climatology,
+          (int) taxisptr->has_bounds,
           vdate_lb, vtime_lb, vdate_ub, vtime_ub,
           taxisptr->fc_unit, taxisptr->fc_period );
 }
@@ -1488,12 +1472,12 @@ taxisGetPackSize(void *p, void *context)
 {
   taxis_t *taxisptr = (taxis_t*) p;
   int packBufferSize
-    = serializeGetSize(taxisNint, DATATYPE_INT, context)
-    + serializeGetSize(1, DATATYPE_UINT32, context)
+    = serializeGetSize(taxisNint, CDI_DATATYPE_INT, context)
+    + serializeGetSize(1, CDI_DATATYPE_UINT32, context)
     + (taxisptr->name ?
-       serializeGetSize((int)strlen(taxisptr->name), DATATYPE_TXT, context) : 0)
+       serializeGetSize((int)strlen(taxisptr->name), CDI_DATATYPE_TXT, context) : 0)
     + (taxisptr->longname ?
-       serializeGetSize((int)strlen(taxisptr->longname), DATATYPE_TXT,
+       serializeGetSize((int)strlen(taxisptr->longname), CDI_DATATYPE_TXT,
                         context) : 0);
   return packBufferSize;
 }
@@ -1508,11 +1492,11 @@ taxisUnpack(char * unpackBuffer, int unpackBufferSize, int * unpackBufferPos,
   int idx = 0;
 
   serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
-                  intBuffer, taxisNint, DATATYPE_INT, context);
+                  intBuffer, taxisNint, CDI_DATATYPE_INT, context);
   serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
-                  &d, 1, DATATYPE_UINT32, context);
+                  &d, 1, CDI_DATATYPE_UINT32, context);
 
-  xassert(cdiCheckSum(DATATYPE_INT, taxisNint, intBuffer) == d);
+  xassert(cdiCheckSum(CDI_DATATYPE_INT, taxisNint, intBuffer) == d);
 
   taxisInit();
 
@@ -1545,7 +1529,7 @@ taxisUnpack(char * unpackBuffer, int unpackBufferSize, int * unpackBufferPos,
       int len = intBuffer[idx];
       char *name = new_refcount_string((size_t)len);
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
-                      name, len, DATATYPE_TXT, context);
+                      name, len, CDI_DATATYPE_TXT, context);
       name[len] = '\0';
       taxisP->name = name;
     }
@@ -1555,7 +1539,7 @@ taxisUnpack(char * unpackBuffer, int unpackBufferSize, int * unpackBufferPos,
       int len = intBuffer[idx];
       char *longname = new_refcount_string((size_t)len);
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
-                      longname, len, DATATYPE_TXT, context);
+                      longname, len, CDI_DATATYPE_TXT, context);
       longname[len] = '\0';
       taxisP->longname = longname;
     }
@@ -1598,16 +1582,16 @@ taxisPack(void * voidP, void * packBuffer, int packBufferSize, int * packBufferP
   intBuffer[idx++] = taxisP->name ? (int)strlen(taxisP->name) : 0;
   intBuffer[idx++] = taxisP->longname ? (int)strlen(taxisP->longname) : 0;
 
-  serializePack(intBuffer, taxisNint, DATATYPE_INT,
+  serializePack(intBuffer, taxisNint, CDI_DATATYPE_INT,
                 packBuffer, packBufferSize, packBufferPos, context);
-  d = cdiCheckSum(DATATYPE_INT, taxisNint, intBuffer);
-  serializePack(&d, 1, DATATYPE_UINT32,
+  d = cdiCheckSum(CDI_DATATYPE_INT, taxisNint, intBuffer);
+  serializePack(&d, 1, CDI_DATATYPE_UINT32,
                 packBuffer, packBufferSize, packBufferPos, context);
   if (taxisP->name)
-    serializePack(taxisP->name, intBuffer[15], DATATYPE_TXT,
+    serializePack(taxisP->name, intBuffer[15], CDI_DATATYPE_TXT,
                   packBuffer, packBufferSize, packBufferPos, context);
   if (taxisP->longname)
-    serializePack(taxisP->longname, intBuffer[16], DATATYPE_TXT,
+    serializePack(taxisP->longname, intBuffer[16], CDI_DATATYPE_TXT,
                   packBuffer, packBufferSize, packBufferPos, context);
 
 }
