@@ -8,6 +8,7 @@
 #include <float.h>
 #include <inttypes.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 
@@ -29,6 +30,7 @@ void cdiPrintDatatypes(void)
            "+-------------+-------+\n"
            "| char        |   %3d |\n"
            "+-------------+-------+\n"
+           "| bool        |   %3d |\n"
            "| short       |   %3d |\n"
            "| int         |   %3d |\n"
            "| long        |   %3d |\n"
@@ -47,7 +49,7 @@ void cdiPrintDatatypes(void)
            "| FLT64       | %-9s |\n"
            "+-------------+-----------+\n"
            "\n  byte ordering is %s\n\n",
-           (int) sizeof(void *), (int) sizeof(char),
+           (int) sizeof(void *), (int) sizeof(char), (int) sizeof(bool),
            (int) sizeof(short), (int) sizeof(int), (int) sizeof(long), (int) sizeof(long long),
            (int) sizeof(size_t), (int) sizeof(off_t),
            (int) sizeof(float), (int) sizeof(double), (int) sizeof(long double),
@@ -149,7 +151,7 @@ char* cdiUnescapeSpaces(const char* string, const char** outStringEnd)
   return result;
 }
 
-#ifdef HAVE_DECL_UUID_GENERATE
+#if defined (HAVE_DECL_UUID_GENERATE) && defined (HAVE_UUID_UUID_H)
 #include <sys/time.h>
 #include <uuid/uuid.h>
 void cdiCreateUUID(unsigned char *uuid)
@@ -176,7 +178,7 @@ void cdiCreateUUID(unsigned char *uuid)
   uuid_generate(uuid);
   setstate(caller_rand_state);
 }
-#elif defined (HAVE_DECL_UUID_CREATE)
+#elif defined (HAVE_DECL_UUID_CREATE) && defined (HAVE_UUID_H)
 typedef uint8_t u_int8_t;
 typedef uint16_t u_int16_t;
 typedef uint32_t u_int32_t;
@@ -197,6 +199,7 @@ void cdiCreateUUID(unsigned char *uuid)
 {
   static int uuid_seeded = 0;
   static char uuid_rand_state[31 * sizeof (long)];
+#ifndef _SX
   char *caller_rand_state;
   if (uuid_seeded)
     caller_rand_state = setstate(uuid_rand_state);
@@ -216,11 +219,17 @@ void cdiCreateUUID(unsigned char *uuid)
     }
   for (size_t i = 0; i < CDI_UUID_SIZE; ++i)
     uuid[i] = (unsigned char)random();
+#else
+  for (size_t i = 0; i < CDI_UUID_SIZE; ++i)
+    uuid[i] = (unsigned char)lrand48();
+#endif
   /* encode variant into msb of octet 8 */
   uuid[8] = (unsigned char)((uuid[8] & 0x3f) | (1 << 7));
   /* encode version 4 ((pseudo-)random uuid) into msb of octet 7 */
   uuid[7] = (unsigned char)((uuid[7] & 0x0f) | (4 << 4));
+#ifndef _SX
   setstate(caller_rand_state);
+#endif
 }
 #endif
 
