@@ -17,30 +17,8 @@
 
 #define VALIDMISS 1.e+303
 
-/*
- * CDI attribute
- */
-typedef struct {
-  size_t    xsz;	  /* amount of space at xvalue                      */
-  size_t    namesz;       /* size of name                                   */
-  char     *name;         /* attribute name                                 */
-  int       indtype;	  /* internal data type of xvalue (INT, FLT or TXT) */
-  int       exdtype;      /* external data type                             */
-                          /* indtype    exdtype                             */
-                          /* TXT        TXT                                 */
-                          /* INT        INT16, INT32                        */
-                          /* FLT        FLT32, FLT64                        */
-  size_t    nelems;    	  /* number of elements                             */
-  void     *xvalue;       /* the actual data                                */
-} cdi_att_t;
-
-
-typedef struct {
-  size_t     nalloc;		/* number allocated >= nelems */
-  size_t     nelems;		/* length of the array */
-  cdi_att_t  value[MAX_ATTRIBUTES];
-} cdi_atts_t;
-
+#include "cdi_att.h"
+#include "grid.h"
 
 typedef struct
 {
@@ -65,25 +43,19 @@ typedef struct
 }
 ensinfo_t;
 
-/* ---------------------------------- */
-/* Local change: 2013-01-28, FP (DWD) */
-/* ---------------------------------- */
-
-/* Length of optional keyword/value pair list */
-#define MAX_OPT_GRIB_ENTRIES 50
 
 
 typedef struct
 {
+  bool        isUsed;
   int         flag;
-  int         isUsed;
   int         mvarID;
   int         fvarID;
   int         param;
   int         gridID;
   int         zaxisID;
   int         tsteptype; /* TSTEP_* */
-  int         datatype;  /* DATATYPE_PACKX for GRIB data, else DATATYPE_FLT32 or DATATYPE_FLT64 */
+  int         datatype;  /* CDI_DATATYPE_PACKX for GRIB data, else CDI_DATATYPE_FLT32 or CDI_DATATYPE_FLT64 */
   int         instID;
   int         modelID;
   int         tableID;
@@ -110,33 +82,27 @@ typedef struct
   ensinfo_t  *ensdata;      /* Ensemble information */
   cdi_atts_t  atts;
   int         iorank;
-#if  defined  (HAVE_LIBGRIB_API)
-  /* ---------------------------------- */
-  /* Local change: 2013-01-28, FP (DWD) */
-  /* ---------------------------------- */
 
-  /* (Optional) list of keyword/double value pairs */
-  int    opt_grib_dbl_nentries;
-  char*  opt_grib_dbl_keyword[MAX_OPT_GRIB_ENTRIES];
-  int    opt_grib_dbl_update[MAX_OPT_GRIB_ENTRIES];
-  double opt_grib_dbl_val[MAX_OPT_GRIB_ENTRIES];
-  /* (Optional) list of keyword/integer value pairs */
-  int    opt_grib_int_nentries;
-  char*  opt_grib_int_keyword[MAX_OPT_GRIB_ENTRIES];
-  int    opt_grib_int_update[MAX_OPT_GRIB_ENTRIES];
-  int    opt_grib_int_val[MAX_OPT_GRIB_ENTRIES];
-#endif
+  int         subtypeID;   /* subtype ID for tile-related meta-data, currently for GRIB-API only. */
+
+  int                 opt_grib_nentries;       /* current no. key-value pairs            */
+  int                 opt_grib_kvpair_size;    /* current allocated size                 */
+  opt_key_val_pair_t *opt_grib_kvpair;         /* (optional) list of keyword/value pairs */
 }
 var_t;
 
 
 typedef struct
 {
-  int         locked;
+  //set when a vlist is passed to streamDefVlist() to safeguard against modifications of the wrong vlist object
+  bool        immutable;
+  //set if this vlist has been created by CDI itself, and must not be destroyed by the user, consequently
+  bool        internal;
   int         self;
   int         nvars;        /* number of variables                */
   int         ngrids;
   int         nzaxis;
+  int         nsubtypes;    /* no. of variable subtypes (e.g. sets of tiles) */
   long        ntsteps;
   int         taxisID;
   int         tableID;
@@ -145,6 +111,7 @@ typedef struct
   int         varsAllocated;
   int         gridIDs[MAX_GRIDS_PS];
   int         zaxisIDs[MAX_ZAXES_PS];
+  int         subtypeIDs[MAX_SUBTYPES_PS];
   var_t      *vars;
   cdi_atts_t  atts;
 }
@@ -152,20 +119,20 @@ vlist_t;
 
 
 vlist_t *vlist_to_pointer(int vlistID);
+void cdiVlistMakeInternal(int vlistID);
+void cdiVlistMakeImmutable(int vlistID);
 void vlistCheckVarID(const char *caller, int vlistID, int varID);
-const char *vlistInqVarNamePtr(int vlistID, int varID);
-const char *vlistInqVarLongnamePtr(int vlistID, int varID);
 const char *vlistInqVarStdnamePtr(int vlistID, int varID);
-const char *vlistInqVarUnitsPtr(int vlistID, int varID);
 void     vlistDestroyVarName(int vlistID, int varID);
 void     vlistDestroyVarLongname(int vlistID, int varID);
+void     vlistDestroyVarStdname(int vlistID, int varID);
 void     vlistDestroyVarUnits(int vlistID, int varID);
+void     cdiVlistDestroy_(int vlistID);
 void     vlistDefVarTsteptype(int vlistID, int varID, int tsteptype);
 int      vlistInqVarMissvalUsed(int vlistID, int varID);
 int      vlistHasTime(int vlistID);
 
-int      vlistDelAtts(int vlistID, int varID);
-int      vlistCopyVarAtts(int vlistID1, int varID_1, int vlistID2, int varID_2);
+int      cdiDelAtts(int vlistID, int varID);
 
 void     vlistUnpack(char * buffer, int bufferSize, int * pos,
                      int originNamespace, void *context, int force_id);
@@ -178,28 +145,33 @@ int     vlistInqVarValidrange(int vlistID, int varID, double *validrange);
 
 void vlistInqVarDimorder(int vlistID, int varID, int (*outDimorder)[3]);
 
-int vlist_att_compare(vlist_t *a, int varIDA, vlist_t *b, int varIDB, int attnum);
+int cdi_att_compare(vlist_t *a, int varIDA, vlist_t *b, int varIDB, int attnum);
 
-void vlist_lock(int vlistID);
-void vlist_unlock(int vlistID);
+void resize_opt_grib_entries(var_t *var, int nentries);
 
-static inline void
-vlistAdd2GridIDs(vlist_t *vlistptr, int gridID)
+
+
+static inline
+void vlistAdd2GridIDs(vlist_t *vlistptr, int gridID)
 {
   int index, ngrids = vlistptr->ngrids;
   for ( index = 0; index < ngrids; index++ )
-    if (vlistptr->gridIDs[index] == gridID ) break;
+    {
+      if ( vlistptr->gridIDs[index] == gridID ) break;
+      //      if ( gridIsEqual(vlistptr->gridIDs[index], gridID) ) break;
+    }
+
   if ( index == ngrids )
     {
-      if (ngrids >= MAX_GRIDS_PS)
+      if ( ngrids >= MAX_GRIDS_PS )
         Error("Internal limit exceeded: more than %d grids.", MAX_GRIDS_PS);
-      ++(vlistptr->ngrids);
       vlistptr->gridIDs[ngrids] = gridID;
+      ++(vlistptr->ngrids);
     }
 }
 
-static inline void
-vlistAdd2ZaxisIDs(vlist_t *vlistptr, int zaxisID)
+static inline
+void vlistAdd2ZaxisIDs(vlist_t *vlistptr, int zaxisID)
 {
   int index, nzaxis = vlistptr->nzaxis;
   for ( index = 0; index < nzaxis; index++ )
@@ -210,10 +182,29 @@ vlistAdd2ZaxisIDs(vlist_t *vlistptr, int zaxisID)
       if ( nzaxis >= MAX_ZAXES_PS )
 	Error("Internal limit exceeded: more than %d zaxis.", MAX_ZAXES_PS);
       vlistptr->zaxisIDs[nzaxis] = zaxisID;
-      vlistptr->nzaxis++;
+      ++(vlistptr->nzaxis);
     }
 }
 
+static inline
+void vlistAdd2SubtypeIDs(vlist_t *vlistptr, int subtypeID)
+{
+  if ( subtypeID == CDI_UNDEFID ) return;
+
+  int index, nsubs = vlistptr->nsubtypes;
+  for ( index = 0; index < nsubs; index++ )
+    if ( vlistptr->subtypeIDs[index] == subtypeID ) break;
+
+  if ( index == nsubs )
+    {
+      if ( nsubs >= MAX_SUBTYPES_PS )
+        Error("Internal limit exceeded: more than %d subs.", MAX_SUBTYPES_PS);
+      vlistptr->subtypeIDs[nsubs] = subtypeID;
+      ++(vlistptr->nsubtypes);
+    }
+}
+
+
 
 #if  defined  (HAVE_LIBGRIB_API)
 extern int   cdiNAdditionalGRIBKeys;
