@@ -29,7 +29,7 @@ typedef int MPI_Comm;
 #include "pio_write.h"
 
 #include "simple_model_helper.h"
-#include "create_uuid.h"
+#include "cdi_uuid.h"
 
 enum {
   ntfiles     = 2,
@@ -63,7 +63,7 @@ modelRun(struct model_config setup, MPI_Comm comm)
 {
   static const char * const fname_prefix        = "example";
 
-  struct
+  struct varDesc_t
   {
     size_t size;
     int nlev, zaxisID, id, code;
@@ -74,7 +74,7 @@ modelRun(struct model_config setup, MPI_Comm comm)
 #endif
   } *varDesc;
   int gridID, taxisID, vlistID, streamID, tsID, tfID = 0;
-  int i, nmiss = 0;
+  enum { nmiss = 0 };
   double *lons, *lats, *levs;
   double *var = NULL, *varslice = NULL;
   double mscale, mrscale;
@@ -83,7 +83,7 @@ modelRun(struct model_config setup, MPI_Comm comm)
   int rank = 0;
   char filename[1024];
   int nlon = setup.nlon, nlat = setup.nlat;
-  int nVars = setup.nvars;
+  size_t nVars = (size_t)setup.nvars;
   size_t varslice_size = 0;
 #if USE_MPI
   int *chunks = NULL, *displs = NULL, comm_size = 1;
@@ -94,11 +94,13 @@ modelRun(struct model_config setup, MPI_Comm comm)
   xmpi ( MPI_Comm_size ( comm, &comm_size ));
   if (rank == 0 && setup.compute_checksum)
     {
-      chunks = xmalloc((size_t)comm_size * sizeof (chunks[0]));
-      displs = xmalloc((size_t)comm_size * sizeof (displs[0]));
-      var = xmalloc((size_t)nlon * (size_t)nlat
+      chunks = malloc((size_t)comm_size * sizeof (chunks[0]));
+      displs = malloc((size_t)comm_size * sizeof (displs[0]));
+      var = malloc((size_t)nlon * (size_t)nlat
                     * (size_t)setup.max_nlev * sizeof(var[0]));
     }
+#else
+  (void)comm;
 #endif
 
   var_scale(setup.datatype, &mscale, &mrscale);
@@ -106,33 +108,34 @@ modelRun(struct model_config setup, MPI_Comm comm)
   gridID = gridCreate ( GRID_LONLAT, nlon*nlat );
   gridDefXsize ( gridID, nlon );
   gridDefYsize ( gridID, nlat );
-  lons = xmalloc((size_t)nlon * sizeof (lons[0]));
-  for (i = 0; i < nlon; ++i)
+  lons = (double *)malloc((size_t)nlon * sizeof (lons[0]));
+  for (size_t i = 0; i < (size_t)nlon; ++i)
     lons[i] = ((double)(i * 360))/nlon;
-  lats = xmalloc((size_t)nlat * sizeof (lats[0]));
-  for (i = 0; i < nlat; ++i)
+  lats = (double *)malloc((size_t)nlat * sizeof (lats[0]));
+  for (size_t i = 0; i < (size_t)nlat; ++i)
     lats[i] = ((double)(i * 180))/nlat - 90.0;
   gridDefXvals ( gridID, lons );
   gridDefYvals ( gridID, lats );
+  if (setup.create_uuid)
   {
     unsigned char uuid[CDI_UUID_SIZE];
     if (rank == 0)
-      create_uuid(uuid);
+      cdiCreateUUID(uuid);
 #if USE_MPI
     MPI_Bcast(uuid, CDI_UUID_SIZE, MPI_UNSIGNED_CHAR, 0, comm);
 #endif
     gridDefUUID(gridID, uuid);
   }
-  levs = xmalloc((size_t)setup.max_nlev * sizeof (levs[0]));
+  levs = (double *)malloc((size_t)setup.max_nlev * sizeof (levs[0]));
   {
     double lscale = 1.0/(double)(setup.max_nlev - 1);
-    for (i = 0; i < setup.max_nlev; ++i)
+    for (size_t i = 0; i < (size_t)setup.max_nlev; ++i)
       levs[i] = 101300.0 - 13000.0 * expm1(2.173 * (double)i * lscale);
   }
   vlistID = vlistCreate ();
 
-  varDesc = xmalloc((size_t)nVars * sizeof (varDesc[0]));
-  for (int varIdx = 0; varIdx < nVars; varIdx++ )
+  varDesc = (struct varDesc_t *)malloc(nVars * sizeof (varDesc[0]));
+  for (size_t varIdx = 0; varIdx < nVars; varIdx++ )
     {
       int varLevs = (int)random()%4;
       switch (varLevs)
@@ -163,10 +166,11 @@ modelRun(struct model_config setup, MPI_Comm comm)
             = zaxisCreate(ZAXIS_PRESSURE, varDesc[varIdx].nlev);
           zaxisDefLevels(varDesc[varIdx].zaxisID, levs);
         }
+      if (setup.create_uuid)
       {
         unsigned char uuid[16];
         if (rank == 0)
-          create_uuid(uuid);
+          cdiCreateUUID(uuid);
 #if USE_MPI
         MPI_Bcast(uuid, CDI_UUID_SIZE, MPI_UNSIGNED_CHAR, 0, comm);
 #endif
@@ -179,20 +183,28 @@ modelRun(struct model_config setup, MPI_Comm comm)
         = (size_t)nlon * (size_t)nlat * (size_t)varDesc[varIdx].nlev;
 #ifdef USE_MPI
       {
+        for (size_t i = 0; i < varIdx; ++i)
+          if (varDesc[i].nlev == varLevs)
+            {
+              varDesc[varIdx].partDesc = varDesc[i].partDesc;
+              varDesc[varIdx].start = varDesc[i].start;
+              varDesc[varIdx].chunkSize = varDesc[i].chunkSize;
+              goto partDescriptionSet;
+            }
         struct PPM_extent range
           = PPM_uniform_partition((struct PPM_extent){ 0,
                 (int32_t)varDesc[varIdx].size }, comm_size, rank);
         int start = range.first;
         int chunkSize = range.size;
-        Xt_idxlist idxlist
-          = xt_idxstripes_new(&(struct Xt_stripe){ .start = start,
-                .nstrides = chunkSize, .stride = 1 }, 1);
         varDesc[varIdx].start = start;
         varDesc[varIdx].chunkSize = chunkSize;
-        varDesc[varIdx].partDesc = idxlist;
+        varDesc[varIdx].partDesc
+          = xt_idxstripes_new(&(struct Xt_stripe){ .start = start,
+                .nstrides = chunkSize, .stride = 1 }, 1);
+        partDescriptionSet: ;
       }
 #endif
-      varDesc[varIdx].code = 129 + varIdx;
+      varDesc[varIdx].code = GRIB_USERDEF + (int)varIdx;
       vlistDefVarCode(vlistID, varDesc[varIdx].id, varDesc[varIdx].code);
       vlistDefVarDatatype(vlistID, varDesc[varIdx].id, setup.datatype);
     }
@@ -211,7 +223,7 @@ modelRun(struct model_config setup, MPI_Comm comm)
 
   for ( tfID = 0; tfID < ntfiles; tfID++ )
     {
-      for (int varIdx = 0; varIdx < nVars; ++varIdx)
+      for (size_t varIdx = 0; varIdx < nVars; ++varIdx)
         varDesc[varIdx].checksum_state = 0;
       if ( tfID > 0 )
 	{
@@ -231,7 +243,7 @@ modelRun(struct model_config setup, MPI_Comm comm)
 	  taxisDefVdate(taxisID, vdatetime[1]);
 	  taxisDefVtime(taxisID, vdatetime[0]);
 	  streamDefTimestep ( streamID, tsID );
-          if (setup.filetype == FILETYPE_EXT)
+          if (setup.filetype == CDI_FILETYPE_EXT)
             {
               /* EXTRA doesn't store time, only date
                * set the value to 0 before checksumming, because a
@@ -239,36 +251,37 @@ modelRun(struct model_config setup, MPI_Comm comm)
                * return */
               vdatetime[0] = 0;
             }
-	  for (int varID = 0; varID < nVars; ++varID)
+	  for (size_t varIdx = 0; varIdx < nVars; ++varIdx)
 	    {
 #ifdef USE_MPI
-              int start = varDesc[varID].start;
-              int chunk = varDesc[varID].chunkSize;
+              int start = varDesc[varIdx].start;
+              size_t chunkSize = (size_t)varDesc[varIdx].chunkSize;
 #else
-              int chunk = (int)varDesc[varID].size;
+              size_t chunkSize = varDesc[varIdx].size;
               int start = 0;
 #endif
-              if (varslice_size < (size_t)chunk)
+              if (varslice_size < chunkSize)
                 {
-                  varslice = xrealloc(varslice, (size_t)chunk * sizeof (var[0]));
-                  varslice_size = (size_t)chunk;
+                  varslice = (double *)Realloc(varslice, chunkSize * sizeof (var[0]));
+                  varslice_size = chunkSize;
                 }
-              modelRegionCompute(varslice, (size_t)start, (size_t)chunk,
-                                 varDesc[varID].nlev, nlat, nlon,
+              modelRegionCompute(varslice, (size_t)start, chunkSize,
+                                 varDesc[varIdx].nlev, nlat, nlon,
                                  tsID, lons, lats,
                                  mscale, mrscale);
               if (setup.compute_checksum)
                 {
 #if USE_MPI
+                  int chunk = (int)chunkSize;
                   xmpi(MPI_Gather(&chunk, 1, MPI_INT,
                                   chunks, 1, MPI_INT, 0, comm));
                   if (rank == 0)
                     {
                       displs[0] = 0;
-                      for (i = 1; i < comm_size; ++i)
+                      for (size_t i = 1; i < (size_t)comm_size; ++i)
                         displs[i] = displs[i - 1] + chunks[i - 1];
                     }
-                  xmpi(MPI_Gatherv(varslice, chunk, MPI_DOUBLE,
+                  xmpi(MPI_Gatherv(varslice, (size_t)chunkSize, MPI_DOUBLE,
                                    var, chunks, displs, MPI_DOUBLE, 0, comm));
 #else
                   var = varslice;
@@ -276,24 +289,24 @@ modelRun(struct model_config setup, MPI_Comm comm)
                 }
               if (rank == 0 && setup.compute_checksum)
                 {
-                  memcrc_r(&varDesc[varID].checksum_state,
+                  memcrc_r(&varDesc[varIdx].checksum_state,
                            (const unsigned char *)vdatetime,
                            sizeof (vdatetime));
-                  memcrc_r(&varDesc[varID].checksum_state,
+                  memcrc_r(&varDesc[varIdx].checksum_state,
                            (const unsigned char *)var,
-                           varDesc[varID].size * sizeof (var[0]));
+                           varDesc[varIdx].size * sizeof (var[0]));
                 }
 
 #ifdef USE_MPI
-	      streamWriteVarPart(streamID, varDesc[varID].id, varslice, nmiss,
-                                 varDesc[varID].partDesc);
+	      streamWriteVarPart(streamID, varDesc[varIdx].id, varslice, nmiss,
+                                 varDesc[varIdx].partDesc);
 #else
-	      streamWriteVar(streamID, varDesc[varID].id, varslice, nmiss);
+	      streamWriteVar(streamID, varDesc[varIdx].id, varslice, nmiss);
 #endif
 	    }
           current_time += 86400;
 #ifdef USE_MPI
-	  pioWriteTimestep ( tsID, vdate, vtime );
+	  pioWriteTimestep();
 #endif
 	}
       if (rank == 0 && setup.compute_checksum)
@@ -306,7 +319,7 @@ modelRun(struct model_config setup, MPI_Comm comm)
                 perror("failed to open table file");
                 exit(EXIT_FAILURE);
               }
-            for (i = 0; i < nVars; ++i)
+            for (size_t i = 0; i < (size_t)nVars; ++i)
               {
                 uint32_t cksum;
                 int code;
@@ -327,36 +340,37 @@ modelRun(struct model_config setup, MPI_Comm comm)
           }
         }
     }
-  free(varslice);
+  Free(varslice);
 #ifdef USE_MPI
   pioEndTimestepping ();
 #endif
   streamClose ( streamID );
   vlistDestroy ( vlistID );
   taxisDestroy ( taxisID );
-  for (int varID = 0; varID < nVars; varID++ )
+  for (size_t varIdx = 0; varIdx < nVars; varIdx++ )
     {
-      int zID = varDesc[varID].zaxisID;
+      int zID = varDesc[varIdx].zaxisID;
       if (zID != CDI_UNDEFID)
         {
           zaxisDestroy(zID);
-          for (int j = varID + 1; j < nVars; ++j)
+#if USE_MPI
+          xt_idxlist_delete(varDesc[varIdx].partDesc);
+#endif
+          for (size_t j = varIdx + 1; j < nVars; ++j)
             if (zID == varDesc[j].zaxisID)
               varDesc[j].zaxisID = CDI_UNDEFID;
         }
     }
   gridDestroy ( gridID );
 #if USE_MPI
-  for (int varID = 0; varID < nVars; ++varID)
-    xt_idxlist_delete(varDesc[varID].partDesc);
-  free(displs);
-  free(chunks);
-  free(var);
+  Free(displs);
+  Free(chunks);
+  Free(var);
 #endif
-  free(varDesc);
-  free(levs);
-  free(lats);
-  free(lons);
+  Free(varDesc);
+  Free(levs);
+  Free(lats);
+  Free(lons);
 }
 
 /*
