@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,6 @@
       Adisit      adipot          compute potential from insitu temperature
 */
 
-#include <ctype.h>
 
 #include <cdi.h>
 #include "cdo.h"
@@ -54,26 +53,22 @@ double adisit_1(double tpot, double sal, double p)
          a_d = 4.1057E-9,
          a_e1 = 1.6056E-10, a_e2 = 5.0484E-12;
 
-  double dc, dv, dvs, fne, fst, qc, qn3, qnq, qv, qvs, t, tpo;
+  double qc = p * (a_a1 + p * (a_c1 - a_e1 * p));
+  double qv = p * (a_b1 - a_d * p);
+  double dc = 1. + p * (-a_a2 + p * (a_c2 - a_e2 * p));
+  double dv = a_b2 * p;
+  double qnq  = -p * (-a_a3 + p * a_c3);
+  double qn3  = -p * a_a4;
 
-  qc = p * (a_a1 + p * (a_c1 - a_e1 * p));
-  qv = p * (a_b1 - a_d * p);
-  dc = 1. + p * (-a_a2 + p * (a_c2 - a_e2 * p));
-  dv = a_b2 * p;
-  qnq  = -p * (-a_a3 + p * a_c3);
-  qn3  = -p * a_a4;
+  double tpo = tpot;
+  double qvs = qv*(sal - 35.) + qc;
+  double dvs = dv*(sal - 35.) + dc;
+  double t   = (tpo + qvs)/dvs;
+  double fne = - qvs + t*(dvs + t*(qnq + t*qn3)) - tpo;
+  double fst = dvs + t*(2.*qnq + 3.*qn3*t);
+  t = t - fne/fst;
 
-    {
-      tpo = tpot;
-      qvs = qv*(sal - 35.) + qc;
-      dvs = dv*(sal - 35.) + dc;
-      t   = (tpo + qvs)/dvs;
-      fne = - qvs + t*(dvs + t*(qnq + t*qn3)) - tpo;
-      fst = dvs + t*(2.*qnq + 3.*qn3*t);
-      t = t - fne/fst;
-    }
-
-  return (t);
+  return t;
 }
 
 /* compute potential temperature from insitu temperature */
@@ -87,39 +82,34 @@ double adipot(double t, double s, double p)
          a_d = 4.1057E-9,
          a_e1 = 1.6056E-10, a_e2 = 5.0484E-12;
 
-  double aa,bb,cc,cc1,dd,tpot,s_rel;
+  double s_rel = s - 35.0;
 
-  s_rel = s - 35.0;
+  double aa = (a_a1+ t*(a_a2 - t*(a_a3 - a_a4*t)));
+  double bb = s_rel*(a_b1 -a_b2*t)     ;
+  double cc = (a_c1 + t*(-a_c2 + a_c3*t));
+  double cc1 = a_d*s_rel;
+  double dd = (-a_e1 + a_e2*t);
 
-   aa = (a_a1+ t*(a_a2 - t*(a_a3 - a_a4*t)));
-   bb = s_rel*(a_b1 -a_b2*t)     ;
-   cc = (a_c1 + t*(-a_c2 + a_c3*t));
-  cc1 = a_d*s_rel;
-   dd = (-a_e1 + a_e2*t);
+  double tpot = t-p*(aa + bb + p*(cc - cc1 + p*dd));
 
-   tpot=t-p*(aa + bb + p*(cc - cc1 + p*dd));
-
-  return (tpot);
+  return tpot;
 }
 
 static
-void calc_adisit(long gridsize, long nlevel, double *pressure, field_t tho, field_t sao, field_t tis)
+void calc_adisit(long gridsize, long nlevel, double *pressure, field_type tho, field_type sao, field_type tis)
 {
   /* pressure units: hPa     */
   /* tho units:      Celsius */
   /* sao units:      psu     */
 
-  long i, levelID, offset;
-  double *tisptr, *thoptr, *saoptr;
-
-  for ( levelID = 0; levelID < nlevel; ++levelID )
+  for ( long levelID = 0; levelID < nlevel; ++levelID )
     {
-      offset = gridsize*levelID;
-      thoptr = tho.ptr + offset;
-      saoptr = sao.ptr + offset;
-      tisptr = tis.ptr + offset;
+      long offset = gridsize*levelID;
+      double *thoptr = tho.ptr + offset;
+      double *saoptr = sao.ptr + offset;
+      double *tisptr = tis.ptr + offset;
 
-      for ( i = 0; i < gridsize; ++i )
+      for ( long i = 0; i < gridsize; ++i )
 	{
 	  if ( DBL_IS_EQUAL(thoptr[i], tho.missval) ||
 	       DBL_IS_EQUAL(saoptr[i], sao.missval) )
@@ -135,23 +125,20 @@ void calc_adisit(long gridsize, long nlevel, double *pressure, field_t tho, fiel
 }
 
 static
-void calc_adipot(long gridsize, long nlevel, double *pressure, field_t t, field_t s, field_t tpot)
+void calc_adipot(long gridsize, long nlevel, double *pressure, field_type t, field_type s, field_type tpot)
 {
   /* pressure units: hPa     */
   /* t units:      Celsius */
   /* s units:      psu     */
 
-  long i, levelID, offset;
-  double *tpotptr, *tptr, *sptr;
-
-  for ( levelID = 0; levelID < nlevel; ++levelID )
+  for ( long levelID = 0; levelID < nlevel; ++levelID )
     {
-      offset = gridsize*levelID;
-      tptr = t.ptr + offset;
-      sptr = s.ptr + offset;
-      tpotptr = tpot.ptr + offset;
+      long offset = gridsize*levelID;
+      double *tptr = t.ptr + offset;
+      double *sptr = s.ptr + offset;
+      double *tpotptr = tpot.ptr + offset;
 
-      for ( i = 0; i < gridsize; ++i )
+      for ( long i = 0; i < gridsize; ++i )
 	{
 	  if ( DBL_IS_EQUAL(tptr[i], t.missval) ||
 	       DBL_IS_EQUAL(sptr[i], s.missval) )
@@ -169,48 +156,35 @@ void calc_adipot(long gridsize, long nlevel, double *pressure, field_t t, field_
 
 void *Adisit(void *argument)
 {
-  int operatorID, ADISIT, ADIPOT;
-  int streamID1, streamID2;
   int nrecs;
-  int tsID, recID, varID, levelID;
-  int nlevel1, nlevel2;
-  int gridsize;
-  int nvars, code, gridID, zaxisID;
-  int vlistID1, vlistID2;
+  int varID, levelID;
   int offset;
-  int nlevel;
   int i;
   int nmiss;
   int thoID = -1, saoID = -1;
-  int tisID2, saoID2;
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
-  int taxisID1, taxisID2;
   double pin = -1;
-  double *pressure;
   double *single;
-  field_t tho, sao, tis;
 
   cdoInitialize(argument);
-  ADISIT = cdoOperatorAdd("adisit", 1, 1, "");
-  ADIPOT = cdoOperatorAdd("adipot", 1, 1, "");
+  int ADISIT = cdoOperatorAdd("adisit", 1, 1, "");
+  int ADIPOT = cdoOperatorAdd("adipot", 1, 1, "");
 
   UNUSED(ADIPOT);
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   if ( operatorArgc() == 1 ) pin = parameter2double(operatorArgv()[0]);
   
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
+  int vlistID1 = streamInqVlist(streamID1);
 
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      gridID  = vlistInqVarGrid(vlistID1, varID);
-
-      code = vlistInqVarCode(vlistID1, varID);
+      int code = vlistInqVarCode(vlistID1, varID);
 
       if ( code <= 0 )
 	{
@@ -238,19 +212,19 @@ void *Adisit(void *argument)
   if ( saoID == -1 ) cdoAbort("Sea water salinity not found!");
   if ( thoID == -1 ) cdoAbort("Potential or Insitu temperature not found!");
 
-  gridID = vlistGrid(vlistID1, 0);
-  gridsize = vlist_check_gridsize(vlistID1);
+  int gridID = vlistGrid(vlistID1, 0);
+  int gridsize = vlist_check_gridsize(vlistID1);
 
-  zaxisID = vlistInqVarZaxis(vlistID1, saoID);
-  nlevel1 = zaxisInqSize(zaxisID);
-  zaxisID = vlistInqVarZaxis(vlistID1, thoID);
-  nlevel2 = zaxisInqSize(zaxisID);
+  int zaxisID = vlistInqVarZaxis(vlistID1, saoID);
+  int nlevel1 = zaxisInqSize(zaxisID);
+      zaxisID = vlistInqVarZaxis(vlistID1, thoID);
+  int nlevel2 = zaxisInqSize(zaxisID);
 
   if ( nlevel1 != nlevel2 ) cdoAbort("temperature and salinity have different number of levels!");
-  nlevel = nlevel1;
+  int nlevel = nlevel1;
 
-  pressure = (double*) malloc(nlevel*sizeof(double));
-  zaxisInqLevels(zaxisID, pressure);
+  double *pressure = (double*) Malloc(nlevel*sizeof(double));
+  cdoZaxisInqLevels(zaxisID, pressure);
 
   if ( pin >= 0 ) 
     for ( i = 0; i < nlevel; ++i ) pressure[i] = pin;
@@ -264,12 +238,13 @@ void *Adisit(void *argument)
 	cdoPrint("%5d  %g", i+1, pressure[i]);
     }
 
+  field_type tho, sao, tis;
   field_init(&tho);
   field_init(&sao);
   field_init(&tis);
-  tho.ptr = (double*) malloc(gridsize*nlevel*sizeof(double));
-  sao.ptr = (double*) malloc(gridsize*nlevel*sizeof(double));
-  tis.ptr = (double*) malloc(gridsize*nlevel*sizeof(double));
+  tho.ptr = (double*) Malloc(gridsize*nlevel*sizeof(double));
+  sao.ptr = (double*) Malloc(gridsize*nlevel*sizeof(double));
+  tis.ptr = (double*) Malloc(gridsize*nlevel*sizeof(double));
 
   tho.nmiss = 0;
   sao.nmiss = 0;
@@ -279,11 +254,15 @@ void *Adisit(void *argument)
   sao.missval = vlistInqVarMissval(vlistID1, saoID);
   tis.missval = tho.missval;
 
+  int datatype = CDI_DATATYPE_FLT32;
+  if ( vlistInqVarDatatype(vlistID1, thoID) == CDI_DATATYPE_FLT64 &&
+       vlistInqVarDatatype(vlistID1, saoID) == CDI_DATATYPE_FLT64 )
+    datatype = CDI_DATATYPE_FLT64;
 
-  vlistID2 = vlistCreate();
+  int vlistID2 = vlistCreate();
 
-  tisID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARIABLE);
-  if (operatorID == ADISIT)
+  int tisID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARIABLE);
+  if ( operatorID == ADISIT )
     {
       vlistDefVarParam(vlistID2, tisID2, cdiEncodeParam(20, 255, 255));
       vlistDefVarName(vlistID2, tisID2, "to");
@@ -299,49 +278,55 @@ void *Adisit(void *argument)
     }
   vlistDefVarUnits(vlistID2, tisID2, "K");
   vlistDefVarMissval(vlistID2, tisID2, tis.missval);
+  vlistDefVarDatatype(vlistID2, tisID2, datatype);
 
-  saoID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARIABLE);
+  int saoID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARIABLE);
   vlistDefVarParam(vlistID2, saoID2, cdiEncodeParam(5, 255, 255));
   vlistDefVarName(vlistID2, saoID2, "s");
   vlistDefVarLongname(vlistID2, saoID2, "Sea water salinity");
   vlistDefVarStdname(vlistID2, saoID2, "sea_water_salinity");
   vlistDefVarUnits(vlistID2, saoID2, "psu");
   vlistDefVarMissval(vlistID2, saoID2, sao.missval);
+  vlistDefVarDatatype(vlistID2, saoID2, datatype);
 
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; ++recID )
+      for ( int recID = 0; recID < nrecs; ++recID )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
 	  offset = gridsize*levelID;
 
-	  if ( varID == thoID ) streamReadRecord(streamID1, tho.ptr+offset, &(tho.nmiss));
-	  if ( varID == saoID ) streamReadRecord(streamID1, sao.ptr+offset, &(sao.nmiss));
-	}
+	  if ( varID == thoID )
+            {
+              streamReadRecord(streamID1, tho.ptr+offset, &nmiss);
+              tho.nmiss = (size_t) nmiss;
+            }
+	  if ( varID == saoID )
+            {
+              streamReadRecord(streamID1, sao.ptr+offset, &nmiss);
+              sao.nmiss = (size_t) nmiss;
+            }
+        }
 
-      if (operatorID == ADISIT )
-        {
-          calc_adisit(gridsize, nlevel, pressure, tho, sao, tis); 
-        }
+      if ( operatorID == ADISIT )
+        calc_adisit(gridsize, nlevel, pressure, tho, sao, tis); 
       else
-        {
-          calc_adipot(gridsize, nlevel, pressure, tho, sao, tis); 
-        }
+        calc_adipot(gridsize, nlevel, pressure, tho, sao, tis); 
 
 
       for ( levelID = 0; levelID < nlevel; ++levelID )
@@ -375,12 +360,12 @@ void *Adisit(void *argument)
 
   vlistDestroy(vlistID2);
 
-  free(pressure);
-  free(tis.ptr);
-  free(tho.ptr);
-  free(sao.ptr);
+  Free(pressure);
+  Free(tis.ptr);
+  Free(tho.ptr);
+  Free(sao.ptr);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
