@@ -64,7 +64,7 @@ int stdin_is_tty  = 0;  /* true if stdin  is character device */
 int stdout_is_tty = 0;  /* true if stdout is character device */
 #endif
 
-static int lstdout = 1;
+static bool lstdout = true;
  
 static int Source = 0;
 
@@ -89,11 +89,11 @@ static int oVertID = -1;
 
 static int Lhybrid2pressure = FALSE;
 
-int    TsID;
+static int TsID;
 #if  defined  (HAVE_LIBPTHREAD)
-int    ParallelRead = TRUE;
+static bool ParallelRead = true;
 #else
-int    ParallelRead = FALSE;
+static bool ParallelRead = false;
 #endif
 
 #define TIMESTEP_INTERVAL  -1
@@ -102,8 +102,8 @@ int    ParallelRead = FALSE;
 #define UNLIM_INTERVAL      2
 
 #define MaxHours  24
-int nrqh;
-int hours[MaxHours+1];
+static int nrqh;
+static int hours[MaxHours+1];
 
 static double *LevelFound;
 
@@ -182,7 +182,7 @@ void after_PostProcess(struct Control *globs)
 static
 void after_SwitchFile(struct Control *globs)
 {
-  int echam4 = FALSE;
+  bool echam4 = false;
   int i, n;
   char y3, y2, y1, y0;
   char         m1, m0;
@@ -201,7 +201,7 @@ void after_SwitchFile(struct Control *globs)
 
       if ( ifile[i-3] == '.' )
 	{
-	  echam4 = TRUE;
+	  echam4 = true;
 	  y3 = ifile[i-9]; y2 = ifile[i-8];
 	  y1 = ifile[i-7]; y0 = ifile[i-6];
 	  m1 = ifile[i-5]; m0 = ifile[i-4];
@@ -301,7 +301,7 @@ void after_setDateTime(struct Date *datetime, int date, int time)
 static
 void after_printProcessStatus(int tsID)
 {
-  static int counthead = FALSE;
+  static bool counthead = false;
 
   if ( tsID == -1 )
     {
@@ -311,16 +311,16 @@ void after_printProcessStatus(int tsID)
 	  fflush(stdout);
 	}
 
-      counthead = FALSE;
+      counthead = false;
     }
   else
     {
-      if ( counthead == FALSE )
+      if ( counthead == false )
 	{
 	  if ( stdout_is_tty )
 	    fprintf(stdout, " Process timestep :       ");
 
-	  counthead = TRUE;
+	  counthead = true;
 	}
 
       if ( stdout_is_tty )
@@ -338,7 +338,7 @@ int after_setNextDate(struct Control *globs)
   int i;
   int vdate, vtime;
 
-  int righttime = FALSE;
+  bool righttime = false;
   while ( TRUE )
     {
       nrecs = streamInqTimestep(globs->istreamID, TsID);
@@ -367,7 +367,7 @@ int after_setNextDate(struct Control *globs)
       for ( i = 0; i < nrqh; i++ )
 	if ( hours[i] < 0 || hours[i] == globs->NextDate.hr )
 	  {
-	    righttime = TRUE;
+	    righttime = true;
 	    break;
 	  }
 
@@ -377,7 +377,7 @@ int after_setNextDate(struct Control *globs)
 	TsID += 1;	  
     }
 
-  return (nrecs);
+  return nrecs;
 }
 
 
@@ -387,25 +387,22 @@ static
 void *after_readTimestep(void *arg)
 {
   int i;
-  int recID, varID, gridID, zaxisID, levelID, timeID;
+  int varID, gridID, zaxisID, levelID, timeID;
   int code, leveltype;
   int nmiss;
-  int analysisData, nrecs;
-  struct Variable *vars;
-  struct Control *globs;
   RARG *rarg = (RARG *) arg;
 
-  nrecs        = rarg->nrecs;
-  analysisData = rarg->lana;
-  vars         = rarg->vars;
-  globs        = rarg->globs;
+  int nrecs        = rarg->nrecs;
+  int analysisData = rarg->lana;
+  struct Variable *vars = rarg->vars;
+  struct Control *globs = rarg->globs;
 
   for ( code = 0; code < MaxCodes; code++ ) vars[code].nmiss0 = 0;
 
   int level = 0;
   int levelOffset = 0;
 
-  for ( recID = 0; recID < nrecs; recID++ )
+  for ( int recID = 0; recID < nrecs; recID++ )
     {
       streamInqRecord(globs->istreamID, &varID, &levelID);
 
@@ -470,7 +467,7 @@ void *after_readTimestep(void *arg)
   */
   num_recs = after_setNextDate(globs);
 
-  return ((void *) &num_recs);
+  return (void *) &num_recs;
 }
 
 static
@@ -486,6 +483,8 @@ void after_defineNextTimestep(struct Control *globs)
     {
       if ( otsID == 0 )
 	{
+          int nvars = vlistNvars(globs->ovlistID);
+          if ( nvars == 0 ) Error("No variable selected!");
 	  vlistDefTaxis(globs->ovlistID, globs->taxisID2);
 	  streamDefVlist(globs->ostreamID, globs->ovlistID);
 	}
@@ -559,11 +558,74 @@ void after_moveTimestep(struct Variable *vars)
       }
 }
 
+static
+void after_check_content(struct Variable *vars, int timestep)
+{
+  extern int labort_after;
+  for ( int code = 0; code < 272; code++ )
+    {
+      /*  if ( code == GEOPOTENTIAL ) continue; */
+      if ( code ==          SLP ) continue;
+      if ( code == GEOPOTHEIGHT ) continue;
+      if ( code ==       STREAM ) continue;
+      if ( code ==      VELOPOT ) continue;
+      if ( code ==       U_WIND ) continue;
+      if ( code ==       V_WIND ) continue;
+      if ( code ==        OMEGA ) continue;
+      if ( code ==    RHUMIDITY ) continue;
+      if ( code ==    LOW_CLOUD ) continue;
+      if ( code ==    MID_CLOUD ) continue;
+      if ( code ==    HIH_CLOUD ) continue;
+      if ( code ==           PS ) continue;
+      if ( code ==     HUMIDITY )
+	{
+	  if ( vars[code].needed && !vars[code].selected &&
+	       vars[code].spectral == NULL &&
+	       vars[code].hybrid   == NULL )
+	    {
+              static bool lwarn = true;
+              if ( lwarn ) Warning( "No humidity in data file, set to zero !");
+              lwarn = false;
+	      vars[code].needed = FALSE;
+	    }
+	}
+      else
+	{
+	  if ( vars[code].needed && !vars[code].comp &&
+	       vars[code].spectral == NULL &&
+	       vars[code].hybrid   == NULL )
+	    {
+	      if ( labort_after )
+		Error( "Code  %3d not found at timestep %d!", code, timestep);
+	      else
+		Warning( "Code  %3d not found at timestep %d!", code, timestep);
+	    }
+	}
+    }
+  /*
+  if ( NumLevelRequest > 0 )
+    {
+      vars[HALF_PRESS].needed = 1;
+      vars[FULL_PRESS].needed = 1;
+    }
+
+  code = HALF_PRESS;
+  if ( vars[code].needed && !vars[code].comp &&
+       vars[code].spectral == NULL && vars[code].hybrid == NULL )
+    Error( "Hybrid model level not found!");
+
+  code = FULL_PRESS;
+  if ( vars[code].needed && !vars[code].comp &&
+       vars[code].spectral == NULL && vars[code].hybrid == NULL )
+    Error( "Hybrid model level not found!");
+  */
+}
+
 static
 void after_control(struct Control *globs, struct Variable *vars)
 {
   int i;
-  int tsFirst, nrecs;
+  int nrecs;
   int rdate, rtime;
   int vdate, vtime;
   int code;
@@ -581,7 +643,7 @@ void after_control(struct Control *globs, struct Variable *vars)
       pthread_attr_init(&attr);
       pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
       int status = pthread_attr_getstacksize(&attr, &stacksize);
-      if ( stacksize < 2097152 )
+      if ( status && stacksize < 2097152 )
 	{
 	  stacksize = 2097152;
 	  pthread_attr_setstacksize(&attr, stacksize);
@@ -594,7 +656,7 @@ void after_control(struct Control *globs, struct Variable *vars)
 
   TsID = 0;
 
-  int righttime = FALSE;
+  bool righttime = false;
   while ( (nrecs = streamInqTimestep(globs->istreamID, TsID)) > 0 )
     {
       vdate = taxisInqVdate(globs->taxisID);
@@ -605,7 +667,7 @@ void after_control(struct Control *globs, struct Variable *vars)
       for ( i = 0; i < nrqh; i++ )
 	if ( hours[i] < 0 || hours[i] == globs->NewDate.hr )
 	  {
-	    righttime = TRUE;
+	    righttime = true;
 	    break;
 	  }
 
@@ -626,7 +688,7 @@ void after_control(struct Control *globs, struct Variable *vars)
       rtime = after_getTime(globs->StartDate);
     }
 
-  if ( ofiletype == FILETYPE_NC || ofiletype == FILETYPE_NC2 || ofiletype == FILETYPE_NC4 )
+  if ( ofiletype == CDI_FILETYPE_NC || ofiletype == CDI_FILETYPE_NC2 || ofiletype == CDI_FILETYPE_NC4 )
     {
       taxisDefCalendar(globs->taxisID2, CALENDAR_PROLEPTIC);
       taxisDefType(globs->taxisID2, TAXIS_RELATIVE);
@@ -637,7 +699,7 @@ void after_control(struct Control *globs, struct Variable *vars)
 
   globs->OldDate = globs->NewDate;
 
-  tsFirst = TRUE;
+  bool tsFirst = true;
 
   while ( nrecs > 0 )
     {
@@ -646,9 +708,9 @@ void after_control(struct Control *globs, struct Variable *vars)
       rarg.vars  = vars;
       rarg.globs = globs;
 
-      if ( tsFirst || ParallelRead == FALSE )
+      if ( tsFirst || ParallelRead == false )
 	{
-	  if ( ParallelRead == FALSE )
+	  if ( ParallelRead == false )
 	    {
 	      statusp = after_readTimestep(&rarg);
 	    }
@@ -673,7 +735,7 @@ void after_control(struct Control *globs, struct Variable *vars)
 		Error("after_readTimestep error! (status = %d)", *(int *)statusp);
 	    }
 #endif
-	  tsFirst = FALSE;
+	  tsFirst = false;
 	}
 #if defined(HAVE_LIBPTHREAD)
       else
@@ -704,8 +766,12 @@ void after_control(struct Control *globs, struct Variable *vars)
 
       if ( lstdout && globs->EndOfInterval ) after_printProcessStatus(-1);
       
-      if ( globs->Mean == 0 || globs->EndOfInterval ) after_defineNextTimestep(globs);
-
+      if ( globs->Mean == 0 || globs->EndOfInterval )
+        {
+          if ( !globs->AnalysisData ) after_check_content(vars, globs->TermCount+1);
+          after_defineNextTimestep(globs);
+        }
+      
       if ( globs->AnalysisData )
 	after_processPL(globs, vars);
       else
@@ -738,7 +804,7 @@ void after_setLevel(struct Control *globs)
   int k, l, found;
   int removeLevel[MaxLevel];
   double level;
-  int checkLevel = TRUE;
+  bool checkLevel = true;
   int numplevelDefault;  /* default pressure level */
   long plevelDefault[] = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 20000, 15000,
                             10000,  7000,  5000,  3000,  2000, 1000 };
@@ -795,20 +861,20 @@ void after_setLevel(struct Control *globs)
 	      oVertID = iVertID;
 	    }
 	}
-      checkLevel = FALSE;
+      checkLevel = false;
     }
   else
     {
       if ( iVertID == -1 )
 	{
 	  if ( globs->Verbose )	fprintf(stdout," No level detected\n");
-	  checkLevel = FALSE;
+	  checkLevel = false;
 	}
       else if ( globs->NumLevelRequest == 1 && IS_EQUAL(globs->LevelRequest[0], 0) )
 	{
 	  if ( globs->Verbose ) fprintf(stdout," No level selected\n");
 	  globs->NumLevelRequest = 0;
-	  checkLevel = FALSE;
+	  checkLevel = false;
 	}
       else if ( globs->Verbose )
 	{
@@ -982,11 +1048,14 @@ void after_defineLevel(struct Control *globs, struct Variable *vars)
 	  if ( vars[code].selected )
 	    {
 	      if ( vars[code].izaxisID != -1 )
-		if ( zaxisInqType(vars[code].izaxisID) == zaxisInqType(iVertID) &&
-		     zaxisInqSize(vars[code].izaxisID) == globs->NumLevel &&
-		     zaxisInqSize(vars[code].izaxisID) >  1 )
-		  vars[code].ozaxisID = oVertID;
-	    }
+                {
+                  int nlev = zaxisInqSize(vars[code].izaxisID);
+                  if ( zaxisInqType(vars[code].izaxisID) == zaxisInqType(iVertID) &&
+                       (nlev == globs->NumLevel || nlev == globs->NumLevel+1) &&
+                       nlev >  1 )
+                    vars[code].ozaxisID = oVertID;
+                }
+            }
 	}
 
       break;
@@ -1281,29 +1350,29 @@ void after_parini(struct Control *globs, struct Variable *vars)
 #if defined(CDO)
     case -1: ofiletype = -1;            break;
 #else
-    case -1: ofiletype = FILETYPE_SRV;  break;
+    case -1: ofiletype = CDI_FILETYPE_SRV;  break;
 #endif
-    case  0: ofiletype = FILETYPE_SRV;  break;
-    case  1: ofiletype = FILETYPE_GRB;  break;
-    case  2: ofiletype = FILETYPE_NC;   break;
-    case  3: ofiletype = FILETYPE_EXT;  break;
-    case  4: ofiletype = FILETYPE_NC2;  break;
-    case  6: ofiletype = FILETYPE_NC4;  break;
+    case  0: ofiletype = CDI_FILETYPE_SRV;  break;
+    case  1: ofiletype = CDI_FILETYPE_GRB;  break;
+    case  2: ofiletype = CDI_FILETYPE_NC;   break;
+    case  3: ofiletype = CDI_FILETYPE_EXT;  break;
+    case  4: ofiletype = CDI_FILETYPE_NC2;  break;
+    case  6: ofiletype = CDI_FILETYPE_NC4;  break;
     default: Error( "unknown file format %d", fileFormat);
     }
 
-  if ( gribFormat )  ofiletype = FILETYPE_GRB;
-  if ( cdfFormat  )  ofiletype = FILETYPE_NC;
+  if ( gribFormat )  ofiletype = CDI_FILETYPE_GRB;
+  if ( cdfFormat  )  ofiletype = CDI_FILETYPE_NC;
 
   int precision = scan_par(globs->Verbose, namelist, "precision", 0);
   if ( precision )
     switch ( precision )
       {
-      case  8: DataType = DATATYPE_PACK8;  break;
-      case 16: DataType = DATATYPE_PACK16; break;
-      case 24: DataType = DATATYPE_PACK24; break;
-      case 32: DataType = DATATYPE_FLT32;  break;
-      case 64: DataType = DATATYPE_FLT64;  break;
+      case  8: DataType = CDI_DATATYPE_PACK8;  break;
+      case 16: DataType = CDI_DATATYPE_PACK16; break;
+      case 24: DataType = CDI_DATATYPE_PACK24; break;
+      case 32: DataType = CDI_DATATYPE_FLT32;  break;
+      case 64: DataType = CDI_DATATYPE_FLT64;  break;
       default: Error( "unsupported data precision %d", precision);
       }
 
@@ -1424,13 +1493,13 @@ void after_precntl(struct Control *globs, struct Variable *vars)
       else if ( gridtype == GRID_GAUSSIAN && globs->Latitudes == 0 )
 	{
 	  gaussGridID = gridID;
-	  globs->Longitudes  = gridInqXsize(gridID);
-	  globs->Latitudes   = gridInqYsize(gridID);
+	  globs->Longitudes = gridInqXsize(gridID);
+	  globs->Latitudes  = gridInqYsize(gridID);
 	}
     }
 
   if ( globs->Truncation == 0 && globs->Latitudes == 0 )
-    Error("Unsupported file structure!");
+    Error("Unsupported file structure (no spectral or Gaussian data found)!");
 
   if ( globs->Truncation == 0 )
     {
@@ -1483,6 +1552,8 @@ void after_precntl(struct Control *globs, struct Variable *vars)
                       continue;
 		    }
 		}
+              else if ( leveltype == ZAXIS_HYBRID && globs->nvct == zaxisInqVctSize(zaxisID) )
+                continue;
 
 	      if ( iVertID != - 1 )
 		Warning( "More than %d different vertical grid structure found!", vertfound);
@@ -1643,7 +1714,7 @@ void after_postcntl(struct Control *globs, struct Variable *vars)
   char histstring[99];
   int datatype;
 
-  sprintf(histstring, "afterburner version %s  type = %d", VERSION, globs->Type);
+  snprintf(histstring, sizeof(histstring), "afterburner version %s  type = %d", VERSION, globs->Type);
 
 #if defined(AFTERBURNER)
   afterInqHistory(globs->istreamID);
@@ -1876,10 +1947,9 @@ static
 void after_printCodes(void)
 {
   int tableID = tableInq(-1, 128, "echam4");
-  int ncodes;
   int codes[] = {34,35,36,131,132,135,148,149,151,156,157,259,260,261,262,263,264,268,269,270,271,275};
 
-  ncodes = sizeof(codes)/sizeof(codes[0]);
+  int ncodes = sizeof(codes)/sizeof(codes[0]);
 
   lprintf(stdout);
 
@@ -1916,35 +1986,29 @@ void after_printCodes(void)
 static
 void after_procstat(char *procpath, int truncation)
 {
-  FILE *sf;
-  double MaxMBytes;
   time_t tp;
-  long  yy, mm, dd, hh, mi;
   char mtype[12];
-  char *proc;
-  char *name;
   char  stat_file[128];
-  double CPUTime;
 
-  CPUTime = ((double) clock() - starttime ) / CLOCKS_PER_SEC;
+  double CPUTime = ((double) clock() - starttime ) / CLOCKS_PER_SEC;
 
   (void) time(&tp);
-  yy    = gmtime(&tp)->tm_year + 1900;
-  mm    = gmtime(&tp)->tm_mon + 1;
-  dd    = gmtime(&tp)->tm_mday   ;
-  hh    = gmtime(&tp)->tm_hour   ;
-  mi    = gmtime(&tp)->tm_min    ;
-  name  = getpwuid(getuid())->pw_name;
+  long yy    = gmtime(&tp)->tm_year + 1900;
+  long mm    = gmtime(&tp)->tm_mon + 1;
+  long dd    = gmtime(&tp)->tm_mday   ;
+  long hh    = gmtime(&tp)->tm_hour   ;
+  long mi    = gmtime(&tp)->tm_min    ;
+  char *name = getpwuid(getuid())->pw_name;
 
-  proc = strrchr(procpath,'/');
+  char *proc = strrchr(procpath,'/');
   if (proc == 0) proc = procpath;
   else           proc++         ;
 
   strcpy(stat_file, "/pf/m/m214003/local/log/after.log");
 
-  MaxMBytes = (double) memTotal() / 1048576.;
+  double MaxMBytes = (double) memTotal() / 1048576.;
 
-  sf = fopen(stat_file, "a");
+  FILE *sf = fopen(stat_file, "a");
   if ( sf )
     {
       char unknown[] = "";
@@ -2020,7 +2084,7 @@ void after_processing(struct Control *globs, struct Variable *vars)
       if ( globs->ostreamID < 0 ) cdiError(globs->ostreamID, "Open failed on %s", ofile);
 #endif
 
-      if ( globs->Szip ) streamDefCompType(globs->ostreamID, COMPRESS_SZIP);
+      if ( globs->Szip ) streamDefCompType(globs->ostreamID, CDI_COMPRESS_SZIP);
 
       globs->ovlistID = vlistCreate();
     }
@@ -2030,7 +2094,7 @@ void after_processing(struct Control *globs, struct Variable *vars)
       globs->ostreamID2 = streamOpenWrite(ofile2, ofiletype);
       if ( globs->ostreamID2 < 0 ) cdiError(globs->ostreamID2, "Open failed on %s", ofile2);
 
-      if ( globs->Szip ) streamDefCompType(globs->ostreamID, COMPRESS_SZIP);
+      if ( globs->Szip ) streamDefCompType(globs->ostreamID, CDI_COMPRESS_SZIP);
 
       globs->ovlistID2 = vlistCreate();
     }
@@ -2053,7 +2117,7 @@ void after_processing(struct Control *globs, struct Variable *vars)
   if ( ! globs->AnalysisData )
     for (i = 0; i < globs->NumLevelRequest; i++)
       {
-	if ( (globs->LevelRequest[i] >= 65535) && globs->unitsel && ofiletype == FILETYPE_GRB )
+	if ( (globs->LevelRequest[i] >= 65535) && globs->unitsel && ofiletype == CDI_FILETYPE_GRB )
 	  {
 	    fprintf(stderr,"\n Level %9.2f out of range (max=65535)!\n", globs->LevelRequest[i]);
 	    exit(1);
@@ -2085,10 +2149,10 @@ void after_processing(struct Control *globs, struct Variable *vars)
 
   if ( globs->Type == 10 || globs->Type == 40 || globs->Type == 60 )
     {
-      if ( ofiletype == FILETYPE_GRB )
+      if ( ofiletype == CDI_FILETYPE_GRB )
 	Error("Can't write fourier coefficients to GRIB!");
-      else if ( ofiletype == FILETYPE_NC || ofiletype == FILETYPE_NC2 ||
-		ofiletype == FILETYPE_NC4 )
+      else if ( ofiletype == CDI_FILETYPE_NC || ofiletype == CDI_FILETYPE_NC2 ||
+		ofiletype == CDI_FILETYPE_NC4 )
 	Error("Can't write fourier coefficients to NetCDF!");
     }
 
@@ -2221,7 +2285,7 @@ int afterburner(int argc, char *argv[])
       fprintf(stderr, "byte size of type double %d\n", (int) sizeof(double));
       fprintf(stderr, "byte size of type int %d\n",    (int) sizeof(int));
       fprintf(stderr, "byte size of type size_t %d\n", (int) sizeof(size_t));
-      return(1);
+      return 1;
     }
 
   fp = fopen("/pf/m/m214003/doc/afterburner.doc","r");
@@ -2253,7 +2317,7 @@ int afterburner(int argc, char *argv[])
       case 'b': Message( "option -b not longer needed!"); break;
       case 'c': after_printCodes(); break;
       case 'd': globs->Debug = 1; break;
-      case 'p': ParallelRead = TRUE; break;
+      case 'p': ParallelRead = true; break;
       case 'P': numThreads = atoi(optarg); break;
       case 'V': after_version(); break;
       case 'v': Vctfile = optarg; break;
@@ -2262,7 +2326,7 @@ int afterburner(int argc, char *argv[])
       }
 
 #if defined (_OPENMP)
-  /* ParallelRead = TRUE; */
+  /* ParallelRead = true; */
 
   lprintf(stdout);
   if ( numThreads <= 0 ) numThreads = 1;
@@ -2274,7 +2338,7 @@ int afterburner(int argc, char *argv[])
   if ( numThreads > 0 )
     {
       fprintf(stderr, "Option -P failed, OpenMP support not compiled in!\n");
-      return(-1);
+      return -1;
     }
 #endif
 
@@ -2284,7 +2348,7 @@ int afterburner(int argc, char *argv[])
       fprintf(stdout, " Parallel read enabled\n");
 #else
       fprintf(stdout, " Parallel read disabled\n");
-      ParallelRead = FALSE;
+      ParallelRead = false;
 #endif
     }
 
@@ -2365,7 +2429,7 @@ int afterburner(int argc, char *argv[])
 
   Free(globs);
 
-  return(0);
+  return 0;
 }
 #endif
 
