@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -39,14 +39,9 @@ int stringToParam(const char *paramstr);
 
 void *Change(void *argument)
 {
-  int CHCODE, CHTABNUM, CHPARAM, CHNAME, CHUNIT, CHLEVEL, CHLEVELC, CHLEVELV, CHLTYPE;  
-  int operatorID;
-  int streamID1, streamID2 = CDI_UNDEFID;
   int nrecs, nvars;
-  int tsID1, recID, varID = 0, levelID;
-  int vlistID1, vlistID2;
-  int taxisID1, taxisID2;
-  int chints[MAXARG], nch = 0;
+  int varID = 0, levelID;
+  int chints[MAXARG];
   char *chnames[MAXARG];
   char varname[CDI_MAX_NAME];
   char *chname = NULL;
@@ -54,38 +49,36 @@ void *Change(void *argument)
   int param;
   int code, tabnum, i;
   int nmiss;
-  int gridsize;
   int nfound;
   int nzaxis, zaxisID1, zaxisID2, k, nlevs, index; 
   double chlevels[MAXARG];
   int  chltypes[MAXARG];              
   double *levels = NULL;
   double *newlevels = NULL;
-  double *array = NULL;
 
   cdoInitialize(argument);
 
-  CHCODE   = cdoOperatorAdd("chcode",   0, 0, "pairs of old and new code numbers");
-  CHTABNUM = cdoOperatorAdd("chtabnum", 0, 0, "pairs of old and new GRIB1 table numbers");
-  CHPARAM  = cdoOperatorAdd("chparam",  0, 0, "pairs of old and new parameter identifiers");
-  CHNAME   = cdoOperatorAdd("chname",   0, 0, "pairs of old and new variable names");
-  CHUNIT   = cdoOperatorAdd("chunit",   0, 0, "pairs of old and new variable units");
-  CHLEVEL  = cdoOperatorAdd("chlevel",  0, 0, "pairs of old and new levels");
-  CHLEVELC = cdoOperatorAdd("chlevelc", 0, 0, "code number, old and new level");
-  CHLEVELV = cdoOperatorAdd("chlevelv", 0, 0, "variable name, old and new level");
-  CHLTYPE  = cdoOperatorAdd("chltype",  0, 0, "pairs of old and new type");          
+  int CHCODE   = cdoOperatorAdd("chcode",   0, 0, "pairs of old and new code numbers");
+  int CHTABNUM = cdoOperatorAdd("chtabnum", 0, 0, "pairs of old and new GRIB1 table numbers");
+  int CHPARAM  = cdoOperatorAdd("chparam",  0, 0, "pairs of old and new parameter identifiers");
+  int CHNAME   = cdoOperatorAdd("chname",   0, 0, "pairs of old and new variable names");
+  int CHUNIT   = cdoOperatorAdd("chunit",   0, 0, "pairs of old and new variable units");
+  int CHLEVEL  = cdoOperatorAdd("chlevel",  0, 0, "pairs of old and new levels");
+  int CHLEVELC = cdoOperatorAdd("chlevelc", 0, 0, "code number, old and new level");
+  int CHLEVELV = cdoOperatorAdd("chlevelv", 0, 0, "variable name, old and new level");
+  int CHLTYPE  = cdoOperatorAdd("chltype",  0, 0, "pairs of old and new type");          
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  nch = operatorArgc();
+  int nch = operatorArgc();
 
   if ( operatorID == CHCODE || operatorID == CHTABNUM )
     {
       if ( nch%2 ) cdoAbort("Odd number of input arguments!");
       for ( i = 0; i < nch; i++ )
-	chints[i] = atoi(operatorArgv()[i]);
+	chints[i] = parameter2int(operatorArgv()[i]);
     }
   else if ( operatorID == CHPARAM || operatorID == CHNAME || operatorID == CHUNIT )
     {
@@ -97,38 +90,38 @@ void *Change(void *argument)
     {
       if ( nch%2 ) cdoAbort("Odd number of input arguments!");
       for ( i = 0; i < nch; i++ )
-	chlevels[i] = atof(operatorArgv()[i]);
+	chlevels[i] = parameter2double(operatorArgv()[i]);
     }
   else if ( operatorID == CHLEVELC )
     {
       operatorCheckArgc(3);
       
-      chcode = atoi(operatorArgv()[0]);
-      chlevels[0] = atof(operatorArgv()[1]);
-      chlevels[1] = atof(operatorArgv()[2]);
+      chcode = parameter2int(operatorArgv()[0]);
+      chlevels[0] = parameter2double(operatorArgv()[1]);
+      chlevels[1] = parameter2double(operatorArgv()[2]);
     }
   else if ( operatorID == CHLEVELV )
     {
       operatorCheckArgc(3);
       
       chname = operatorArgv()[0];
-      chlevels[0] = atof(operatorArgv()[1]);
-      chlevels[1] = atof(operatorArgv()[2]);
+      chlevels[0] = parameter2double(operatorArgv()[1]);
+      chlevels[1] = parameter2double(operatorArgv()[2]);
     }
   else if ( operatorID == CHLTYPE )                  
     {
       if ( nch%2 ) cdoAbort("Odd number of input arguments!");
       for ( i = 0; i < nch; i++ )
-	chltypes[i] = atoi(operatorArgv()[i]);
+	chltypes[i] = parameter2int(operatorArgv()[i]);
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   if ( operatorID == CHCODE )
@@ -203,33 +196,36 @@ void *Change(void *argument)
       for ( index = 0; index < nzaxis; index++ )
 	{
 	  zaxisID1 = vlistZaxis(vlistID2, index);
-	  nlevs = zaxisInqSize(zaxisID1);
-	  levels = malloc(nlevs*sizeof(double));
-	  newlevels = malloc(nlevs*sizeof(double));
-	  zaxisInqLevels(zaxisID1, levels);
+          if ( zaxisInqLevels(zaxisID1, NULL) )
+            {
+              nlevs = zaxisInqSize(zaxisID1);
+              levels = (double*) Malloc(nlevs*sizeof(double));
+              newlevels = (double*) Malloc(nlevs*sizeof(double));
+              zaxisInqLevels(zaxisID1, levels);
 
-	  for ( k = 0; k < nlevs; k++ ) newlevels[k] = levels[k];
+              for ( k = 0; k < nlevs; k++ ) newlevels[k] = levels[k];
 
-	  nfound = 0;
-	  for ( i = 0; i < nch; i += 2 )
-	    for ( k = 0; k < nlevs; k++ )
-	      if ( fabs(levels[k] - chlevels[i]) < 0.0001 ) nfound++;
+              nfound = 0;
+              for ( i = 0; i < nch; i += 2 )
+                for ( k = 0; k < nlevs; k++ )
+                  if ( fabs(levels[k] - chlevels[i]) < 0.0001 ) nfound++;
 
-	  if ( nfound )
-	    {
-	      zaxisID2 = zaxisDuplicate(zaxisID1);
-	      for ( i = 0; i < nch; i += 2 )
-		for ( k = 0; k < nlevs; k++ )
-		  if ( fabs(levels[k] - chlevels[i]) < 0.001 )
-		    newlevels[k] = chlevels[i+1];
+              if ( nfound )
+                {
+                  zaxisID2 = zaxisDuplicate(zaxisID1);
+                  for ( i = 0; i < nch; i += 2 )
+                    for ( k = 0; k < nlevs; k++ )
+                      if ( fabs(levels[k] - chlevels[i]) < 0.001 )
+                        newlevels[k] = chlevels[i+1];
 
-	      zaxisDefLevels(zaxisID2, newlevels);
-	      vlistChangeZaxis(vlistID2, zaxisID1, zaxisID2);
-	    }
+                  zaxisDefLevels(zaxisID2, newlevels);
+                  vlistChangeZaxis(vlistID2, zaxisID1, zaxisID2);
+                }
 
-	  free(levels);
-	  free(newlevels);
-	}
+              Free(levels);
+              Free(newlevels);
+            }
+        }
     }
   else if ( operatorID == CHLEVELC || operatorID == CHLEVELV )
     {
@@ -254,31 +250,34 @@ void *Change(void *argument)
 	}
 
       zaxisID1 = vlistInqVarZaxis(vlistID2, varID);
-      nlevs = zaxisInqSize(zaxisID1);
-      levels = malloc(nlevs*sizeof(double));
-      zaxisInqLevels(zaxisID1, levels);
-      nfound = 0;
-      for ( k = 0; k < nlevs; k++ )
-	if ( fabs(levels[k] - chlevels[0]) < 0.0001 ) nfound++;
+      if ( zaxisInqLevels(zaxisID1, NULL) )
+        {
+          nlevs = zaxisInqSize(zaxisID1);
+          levels = (double*) Malloc(nlevs*sizeof(double));
+          zaxisInqLevels(zaxisID1, levels);
+          nfound = 0;
+          for ( k = 0; k < nlevs; k++ )
+            if ( fabs(levels[k] - chlevels[0]) < 0.0001 ) nfound++;
 
-      if ( nfound )
-	{
-	  zaxisID2 = zaxisDuplicate(zaxisID1);
-	  for ( k = 0; k < nlevs; k++ )
-	    if ( fabs(levels[k] - chlevels[0]) < 0.001 )
-	      levels[k] = chlevels[1];
+          if ( nfound )
+            {
+              zaxisID2 = zaxisDuplicate(zaxisID1);
+              for ( k = 0; k < nlevs; k++ )
+                if ( fabs(levels[k] - chlevels[0]) < 0.001 )
+                  levels[k] = chlevels[1];
 
-	  zaxisDefLevels(zaxisID2, levels);
-	  vlistChangeVarZaxis(vlistID2, varID, zaxisID2);
-	}
-      else
-	cdoAbort("Level %g not found!", chlevels[0]);
+              zaxisDefLevels(zaxisID2, levels);
+              vlistChangeVarZaxis(vlistID2, varID, zaxisID2);
+            }
+          else
+            cdoAbort("Level %g not found!", chlevels[0]);
 
-      free(levels);
+          Free(levels);
+        }
     }
   else if ( operatorID == CHLTYPE )                
     {
-      int zaxistype, zaxistype1, zaxistype2, ltype, ltype1, ltype2, sameltype;
+      int ltype, ltype1, ltype2;
 
       nzaxis = vlistNzaxis(vlistID2);
       for ( index = 0; index < nzaxis; index++ )
@@ -286,47 +285,38 @@ void *Change(void *argument)
 	  zaxisID1 = vlistZaxis(vlistID2, index);
 	  zaxisID2 = zaxisDuplicate(zaxisID1);
 
-	  zaxistype = zaxisInqType(zaxisID1);
 	  ltype = zaxisInqLtype(zaxisID1);
 
 	  for ( i = 0; i < nch; i += 2 )
 	    {
-	      sameltype = FALSE;
 	      ltype1 = chltypes[i];
 	      ltype2 = chltypes[i+1];
 
-	      zaxistype1 = ltype2ztype(ltype1);
-	      zaxistype2 = ltype2ztype(ltype2);
-
-	      if ( zaxistype1 == zaxistype ) sameltype = TRUE;
-
-	      if ( !(zaxistype1 == ZAXIS_GENERIC && ltype1 == ltype) ) sameltype = FALSE;
-
-	      if ( sameltype)
+	      if ( ltype1 == ltype )
 		{
-		  zaxisChangeType(zaxisID2, zaxistype2);
-		  if ( zaxistype == ZAXIS_GENERIC ) zaxisDefLtype(zaxisID2, ltype2);
+		  zaxisChangeType(zaxisID2, ZAXIS_GENERIC);
+		  zaxisDefLtype(zaxisID2, ltype2);
 		  vlistChangeZaxis(vlistID2, zaxisID1, zaxisID2);
 		}
 	    }
 	}
     }
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  gridsize = vlistGridsizeMax(vlistID2);
-  array = malloc(gridsize*sizeof(double));
+  int gridsize = vlistGridsizeMax(vlistID2);
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID1 = 0;
+  int tsID1 = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID1)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID1);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamDefRecord(streamID2,  varID,  levelID);
@@ -340,9 +330,9 @@ void *Change(void *argument)
   streamClose(streamID1);
   streamClose(streamID2);
 
-  if ( array ) free(array);
+  if ( array ) Free(array);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
