@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,13 +15,12 @@
   GNU General Public License for more details.
 */
 
-#include <ctype.h>
 
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
-#include "vinterp.h"
+#include "after_vertint.h"
 
 
 #define  SCALESLP        (101325.0)
@@ -122,30 +121,20 @@ void pl_index(int *kmax, int *kmin, double pmax, double pmin, long nlevs, double
 
 void *Cloudlayer(void *argument)
 {
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
-  int taxisID1, taxisID2;
-  int gridID, zaxisID, tsID;
-  int surfaceID;
-  int nlevel, nhlev, nlevs, nrecs, recID, code;
+  int gridID, zaxisID;
+  int nlevel, nlevs, nrecs, code;
   int varID, levelID;
-  int zrev = FALSE;
-  int nvars;
-  int gridsize, i;
+  bool zrev = false;
+  int i;
   int offset;
   int nmiss;
-  int ngp = 0, ngrids;
-  int aclcac_code;
   int aclcacID = -1;
   int nvars2 = 0;
   int aclcac_code_found = 0;
   int kmin[NVARS], kmax[NVARS];
   char varname[CDI_MAX_NAME];
   double sfclevel = 0;
-  double *plevs = NULL;
-  double *aclcac = NULL;
   double *cloud[NVARS];
-  double missval;
   double pmin = 0, pmax = 0;
 
   cdoInitialize(argument);
@@ -154,50 +143,26 @@ void *Cloudlayer(void *argument)
     {
       operatorCheckArgc(2);
       nvars2 = 1;
-      pmin = atof(operatorArgv()[0]);
-      pmax = atof(operatorArgv()[1]);
+      pmin = parameter2double(operatorArgv()[0]);
+      pmax = parameter2double(operatorArgv()[1]);
     }
   else
     {
       nvars2 = NVARS;
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
+  int vlistID1 = streamInqVlist(streamID1);
 
-  ngrids  = vlistNgrids(vlistID1);
-  for ( i = 0; i < ngrids; i++ )
-    {
-      gridID = vlistGrid(vlistID1, i);
-      if ( gridInqType(gridID) != GRID_SPECTRAL )
-	{
-	  ngp = gridInqSize(gridID);
-	  break;
-	}
-    }
+  int gridsize = vlist_check_gridsize(vlistID1);
 
-  /* check gridsize */
-  for ( i = 0; i < ngrids; i++ )
-    {
-      gridID = vlistGrid(vlistID1, i);
-      if ( gridInqType(gridID) != GRID_SPECTRAL )
-	{
-	  if ( ngp != gridInqSize(gridID) )
-	    cdoAbort("Grids have different size!");
-	}
-    }
+  int aclcac_code = 223;
 
-  aclcac_code = 223;
-
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
   for ( varID = 0; varID < nvars; ++varID )
     {
-      gridID   = vlistInqVarGrid(vlistID1, varID);
       zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-      gridsize = gridInqSize(gridID);
-      nlevel   = zaxisInqSize(zaxisID);
-
       code = vlistInqVarCode(vlistID1, varID);
 
       if ( code <= 0 )
@@ -226,26 +191,25 @@ void *Cloudlayer(void *argument)
 	cdoAbort("Cloud cover (parameter 223) not found!");
     }
 
-  missval = vlistInqVarMissval(vlistID1, aclcacID);
+  double missval = vlistInqVarMissval(vlistID1, aclcacID);
   gridID  = vlistInqVarGrid(vlistID1, aclcacID);
   zaxisID = vlistInqVarZaxis(vlistID1, aclcacID);
 
-  gridsize = gridInqSize(gridID);
   nlevel = zaxisInqSize(zaxisID);
-  nhlev  = nlevel+1;
+  int nhlev  = nlevel+1;
 
-  aclcac = malloc(gridsize*nlevel*sizeof(double));
+  double *aclcac = (double*) Malloc(gridsize*nlevel*sizeof(double));
   for ( varID = 0; varID < nvars2; ++varID )
-    cloud[varID] = malloc(gridsize*sizeof(double));
+    cloud[varID] = (double*) Malloc(gridsize*sizeof(double));
 
   if ( zaxisInqType(zaxisID) == ZAXIS_PRESSURE )
     {
-      plevs = malloc(nlevel*sizeof(double));
+      double *plevs = (double*) Malloc(nlevel*sizeof(double));
       zaxisInqLevels(zaxisID, plevs);
       if ( plevs[0] > plevs[nlevel-1] )
 	{
 	  double ptmp;
-	  zrev = TRUE;
+	  zrev = true;
 	  for ( levelID = 0; levelID < nlevel/2; ++levelID )
 	    {
 	      ptmp = plevs[levelID];
@@ -270,24 +234,20 @@ void *Cloudlayer(void *argument)
 	  pl_index(&kmax[0], &kmin[0], 75000.,101300., nlevel, plevs);
 	}
 
-      free(plevs);
+      Free(plevs);
     }
   else if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID )
     {
-      int nvct;
-
-      nvct = zaxisInqVctSize(zaxisID);
+      int nvct = zaxisInqVctSize(zaxisID);
       if ( nlevel == (nvct/2 - 1) )
 	{
-	  double *vct;
-
-	  vct = malloc(nvct*sizeof(double));
+	  double *vct = (double*) Malloc(nvct*sizeof(double));
 	  zaxisInqVct(zaxisID, vct);
 
 	  nlevs = nlevel + 1;
-	  plevs = malloc(nlevs*sizeof(double));
+	  double *plevs = (double*) Malloc(nlevs*sizeof(double));
 	  vct2plev(vct, plevs, nlevs);
-	  free(vct);
+	  Free(vct);
 
 	  if ( nvars2 == 1 )
 	    {
@@ -300,7 +260,7 @@ void *Cloudlayer(void *argument)
 	      hl_index(&kmax[0], &kmin[0], 75000.,101300., nhlev, plevs);
 	    }
 
-	  free(plevs);
+	  Free(plevs);
  	}
       else
 	cdoAbort("Unsupported vertical coordinate table format!");
@@ -309,15 +269,15 @@ void *Cloudlayer(void *argument)
     cdoAbort("Unsupported Z-Axis type!");
 
 
-  surfaceID = zaxisCreate(ZAXIS_SURFACE, 1);
+  int surfaceID = zaxisCreate(ZAXIS_SURFACE, 1);
   zaxisDefLevels(surfaceID, &sfclevel);
 
-  vlistID2 = vlistCreate();
+  int vlistID2 = vlistCreate();
 
   if ( nvars2 == 1 )
     {
       varID = vlistDefVar(vlistID2, gridID, surfaceID, TSTEP_INSTANT);
-      vlistDefVarCode(vlistID2, varID, 33);
+      vlistDefVarParam(vlistID2, varID, cdiEncodeParam(33, 128, 255));
       vlistDefVarName(vlistID2, varID, "cld_lay");
       vlistDefVarLongname(vlistID2, varID, "cloud layer");
       vlistDefVarMissval(vlistID2, varID, missval);
@@ -325,40 +285,40 @@ void *Cloudlayer(void *argument)
   else
     {
       varID = vlistDefVar(vlistID2, gridID, surfaceID, TSTEP_INSTANT);
-      vlistDefVarCode(vlistID2, varID, 34);
+      vlistDefVarParam(vlistID2, varID, cdiEncodeParam(34, 128, 255));
       vlistDefVarName(vlistID2, varID, "low_cld");
       vlistDefVarLongname(vlistID2, varID, "low cloud");
       vlistDefVarMissval(vlistID2, varID, missval);
 
       varID = vlistDefVar(vlistID2, gridID, surfaceID, TSTEP_INSTANT);
-      vlistDefVarCode(vlistID2, varID, 35);
+      vlistDefVarParam(vlistID2, varID, cdiEncodeParam(35, 128, 255));
       vlistDefVarName(vlistID2, varID, "mid_cld");
       vlistDefVarLongname(vlistID2, varID, "mid cloud");
       vlistDefVarMissval(vlistID2, varID, missval);
 
       varID = vlistDefVar(vlistID2, gridID, surfaceID, TSTEP_INSTANT);
-      vlistDefVarCode(vlistID2, varID, 36);
+      vlistDefVarParam(vlistID2, varID, cdiEncodeParam(36, 128, 255));
       vlistDefVarName(vlistID2, varID, "hih_cld");
       vlistDefVarLongname(vlistID2, varID, "high cloud");
       vlistDefVarMissval(vlistID2, varID, missval);
     }
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID);
      
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -403,11 +363,11 @@ void *Cloudlayer(void *argument)
  
   vlistDestroy(vlistID2);
 
-  free(aclcac);
+  Free(aclcac);
   for ( varID = 0; varID < nvars2; ++varID )
-    free(cloud[varID]);
+    Free(cloud[varID]);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
