@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,12 +15,12 @@
   GNU General Public License for more details.
 */
 
-#include <ctype.h>
 
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
+#include "grid.h"
 #include "util.h"
 
 
@@ -82,12 +82,10 @@ int cmpxy_gt(const void *s1, const void *s2)
 static
 int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid, int nxblocks)
 {
-  int lsouthnorth = TRUE;
+  bool lsouthnorth = true;
+  bool lregular = false;
+  bool lcurvilinear = false;
   int gridID2 = -1;
-  int idx;
-  int ny, ix, iy, i, j, ij, offset;
-  int lregular = FALSE;
-  int lcurvilinear = FALSE;
   double *xvals2 = NULL, *yvals2 = NULL;
 
   int nx = -1;
@@ -107,11 +105,11 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid, int nxblocks
   for ( int fileID = 0; fileID < nfiles; fileID++ )
     {
       gridID   = vlistGrid(ef[fileID].vlistID, igrid);
-      gridtype = gridInqType(gridID);
+      int gridtype = gridInqType(gridID);
       if ( gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN )
-        lregular = TRUE;
+        lregular = true;
       else if ( gridtype == GRID_CURVILINEAR )
-        lcurvilinear = TRUE;
+        lcurvilinear = true;
       else if ( gridtype == GRID_GENERIC && gridInqXsize(gridID) > 0 && gridInqYsize(gridID) > 0 )
         ;
       else
@@ -151,7 +149,7 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid, int nxblocks
 
           if ( ysize[fileID] > 1 )
             {
-              if ( yvals[fileID][0] > yvals[fileID][ysize[fileID]-1] ) lsouthnorth = FALSE;
+              if ( yvals[fileID][0] > yvals[fileID][ysize[fileID]-1] ) lsouthnorth = false;
             }
         }
       else
@@ -198,13 +196,13 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid, int nxblocks
       if ( nx <= 0 ) nx = nfiles;
     }
 
-  ny = nfiles/nx;
+  int ny = nfiles/nx;
   if ( nx*ny != nfiles ) cdoAbort("Number of input files (%d) and number of blocks (%dx%d) differ!", nfiles, nx, ny);
  
   int xsize2 = 0;
-  for ( i = 0; i < nx; ++i ) xsize2 += xsize[xyinfo[i].id];
+  for ( int i = 0; i < nx; ++i ) xsize2 += xsize[xyinfo[i].id];
   int ysize2 = 0;
-  for ( j = 0; j < ny; ++j ) ysize2 += ysize[xyinfo[j*nx].id];
+  for ( int j = 0; j < ny; ++j ) ysize2 += ysize[xyinfo[j*nx].id];
   if ( cdoVerbose ) cdoPrint("xsize2 %d  ysize2 %d", xsize2, ysize2);
 
   if ( lregular )
@@ -222,17 +220,17 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid, int nxblocks
   int *yoff = (int*) Malloc((ny+1)*sizeof(int));
 
   xoff[0] = 0;
-  for ( i = 0; i < nx; ++i )
+  for ( int i = 0; i < nx; ++i )
     {
-      idx = xyinfo[i].id;
+      int idx = xyinfo[i].id;
       if ( lregular ) memcpy(xvals2+xoff[i], xvals[idx], xsize[idx]*sizeof(double));
       xoff[i+1] = xoff[i] + xsize[idx];
     }
 
   yoff[0] = 0;
-  for ( j = 0; j < ny; ++j )
+  for ( int j = 0; j < ny; ++j )
     {
-      idx = xyinfo[j*nx].id;
+      int idx = xyinfo[j*nx].id;
       if ( lregular ) memcpy(yvals2+yoff[j], yvals[idx], ysize[idx]*sizeof(double));
       yoff[j+1] = yoff[j] + ysize[idx];
     }
@@ -241,18 +239,18 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid, int nxblocks
     {
       for ( int fileID = 0; fileID < nfiles; fileID++ )
 	{
-	  idx = xyinfo[fileID].id;
-	  iy = fileID/nx;
-	  ix = fileID - iy*nx;
+	  int idx = xyinfo[fileID].id;
+	  int iy = fileID/nx;
+	  int ix = fileID - iy*nx;
 
-          offset = yoff[iy]*xsize2 + xoff[ix];
+          int offset = yoff[iy]*xsize2 + xoff[ix];
 	  /*
 	  printf("fileID %d %d, iy %d, ix %d, offset %d\n",
 		 fileID, xyinfo[fileID].id, iy, ix, offset);
 	  */
-	  ij = 0;
-	  for ( j = 0; j < ysize[idx]; ++j )
-	    for ( i = 0; i < xsize[idx]; ++i )
+	  int ij = 0;
+	  for ( int j = 0; j < ysize[idx]; ++j )
+	    for ( int i = 0; i < xsize[idx]; ++i )
 	      {
                 if ( lcurvilinear )
                   {
@@ -289,21 +287,11 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid, int nxblocks
   Free(yvals);
   Free(xyinfo);
 
-  char string[1024];
-  string[0] = 0;
   gridID = vlistGrid(ef[0].vlistID, igrid);
-  gridInqXname(gridID, string);
-  gridDefXname(gridID2, string);
-  gridInqYname(gridID, string);
-  gridDefYname(gridID2, string);
-  gridInqXlongname(gridID, string);
-  gridDefXlongname(gridID2, string);
-  gridInqYlongname(gridID, string);
-  gridDefYlongname(gridID2, string);
-  gridInqXunits(gridID, string);
-  gridDefXunits(gridID2, string);
-  gridInqYunits(gridID, string);
-  gridDefYunits(gridID2, string);
+
+  grid_copy_attributes(gridID, gridID2);
+
+  if ( gridtype == GRID_PROJECTION ) grid_copy_mapping(gridID, gridID2);
 
   return gridID2;
 }
@@ -312,7 +300,7 @@ int genGrid(int nfiles, ens_file_t *ef, int **gridindex, int igrid, int nxblocks
 void *Collgrid(void *argument)
 {
   int nxblocks = -1;
-  int varID, recID;
+  int varID;
   int nrecs, nrecs0;
   int levelID;
   int nmiss;
@@ -323,10 +311,8 @@ void *Collgrid(void *argument)
   int nfiles = cdoStreamCnt() - 1;
   const char *ofilename = cdoStreamName(nfiles)->args;
 
-  if ( !cdoSilentMode && !cdoOverwriteMode )
-    if ( fileExists(ofilename) )
-      if ( !userFileOverwrite(ofilename) )
-	cdoAbort("Outputfile %s already exists!", ofilename);
+  if ( !cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename) )
+    cdoAbort("Outputfile %s already exists!", ofilename);
 
   ens_file_t *ef = (ens_file_t*) Malloc(nfiles*sizeof(ens_file_t));
 
@@ -344,10 +330,10 @@ void *Collgrid(void *argument)
     vlistCompare(vlistID1, ef[fileID].vlistID, CMP_NAME | CMP_NLEVEL);
 
   int nvars = vlistNvars(vlistID1);
-  int *vars  = (int*) Malloc(nvars*sizeof(int));
-  for ( varID = 0; varID < nvars; varID++ ) vars[varID] = FALSE;
-  int *vars1  = (int*) Malloc(nvars*sizeof(int));
-  for ( varID = 0; varID < nvars; varID++ ) vars1[varID] = FALSE;
+  bool *vars  = (bool*) Malloc(nvars*sizeof(bool));
+  for ( varID = 0; varID < nvars; varID++ ) vars[varID] = false;
+  bool *vars1  = (bool*) Malloc(nvars*sizeof(bool));
+  for ( varID = 0; varID < nvars; varID++ ) vars1[varID] = false;
 
   int nsel = operatorArgc();
 
@@ -365,7 +351,7 @@ void *Collgrid(void *argument)
 
   if ( nsel == 0 )
     {
-      for ( varID = 0; varID < nvars; varID++ ) vars1[varID] = TRUE;
+      for ( varID = 0; varID < nvars; varID++ ) vars1[varID] = true;
     }
   else
     {
@@ -375,8 +361,8 @@ void *Collgrid(void *argument)
 	for ( int i = 0; i < nsel; i++ )
 	  fprintf(stderr, "name %d = %s\n", i+1, argnames[i]);
 
-      int *selfound = (int*) Malloc(nsel*sizeof(int));
-      for ( int i = 0; i < nsel; i++ ) selfound[i] = FALSE;
+      bool *selfound = (bool*) Malloc(nsel*sizeof(bool));
+      for ( int i = 0; i < nsel; i++ ) selfound[i] = false;
 
       char varname[CDI_MAX_NAME];
       for ( varID = 0; varID < nvars; varID++ )
@@ -387,14 +373,14 @@ void *Collgrid(void *argument)
 	    {
 	      if ( strcmp(argnames[isel], varname) == 0 )
 		{
-		  selfound[isel] = TRUE;
-		  vars1[varID] = TRUE;
+		  selfound[isel] = true;
+		  vars1[varID] = true;
 		}
 	    }
 	}
 
       for ( int isel = 0; isel < nsel; isel++ )
-	if ( selfound[isel] == FALSE )
+	if ( selfound[isel] == false )
 	  cdoAbort("Variable name %s not found!", argnames[isel]);
 
       Free(selfound);
@@ -402,7 +388,7 @@ void *Collgrid(void *argument)
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      if ( vars1[varID] == TRUE )
+      if ( vars1[varID] )
 	{
 	  int zaxisID  = vlistInqVarZaxis(vlistID1, varID);
 	  int nlevs    = zaxisInqSize(zaxisID);
@@ -435,7 +421,7 @@ void *Collgrid(void *argument)
   //int vlistID2 = vlistDuplicate(vlistID1);
   int nvars2 = vlistNvars(vlistID2);
   // int *vars  = (int*) Malloc(nvars*sizeof(int));
-  //for ( varID = 0; varID < nvars; varID++ ) vars[varID] = FALSE;
+  //for ( varID = 0; varID < nvars; varID++ ) vars[varID] = false;
 
   int ngrids1 = vlistNgrids(vlistID1);
   int ngrids2 = vlistNgrids(vlistID2);
@@ -445,7 +431,7 @@ void *Collgrid(void *argument)
   for ( int fileID = 0; fileID < nfiles; fileID++ )
     gridindex[fileID] = (int*) Malloc(gridsizemax*sizeof(int));
 
-  int ginit = FALSE;
+  bool ginit = false;
   for ( int i2 = 0; i2 < ngrids2; ++i2 )
     {
       int i1;
@@ -454,10 +440,10 @@ void *Collgrid(void *argument)
 
       //   printf("i1 %d i2 %d\n", i1, i2);
 
-      if ( ginit == FALSE )
+      if ( !ginit )
 	{
 	  gridIDs[i2] = genGrid(nfiles, ef, gridindex, i1, nxblocks);
-	  if ( gridIDs[i2] != -1 ) ginit = TRUE;
+	  if ( gridIDs[i2] != -1 ) ginit = true;
 	}
       else
 	gridIDs[i2] = genGrid(nfiles, ef, NULL, i1, nxblocks);
@@ -487,7 +473,7 @@ void *Collgrid(void *argument)
 	{
 	  if ( gridIDs[i] != -1 ) 
 	    {
-	      if ( gridID == vlistGrid(vlistID2, i) ) vars[varID] = TRUE;
+	      if ( gridID == vlistGrid(vlistID2, i) ) vars[varID] = true;
 	      break;
 	    }
 	}
@@ -514,7 +500,7 @@ void *Collgrid(void *argument)
 
       if ( nrecs0 > 0 ) streamDefTimestep(streamID2, tsID);
       
-      for ( recID = 0; recID < nrecs0; recID++ )
+      for ( int recID = 0; recID < nrecs0; recID++ )
 	{
 	  streamInqRecord(ef[0].streamID, &varID, &levelID);
 	  if ( cdoVerbose && tsID == 0 ) printf(" tsID, recID, varID, levelID %d %d %d %d\n", tsID, recID, varID, levelID);
