@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,69 +34,63 @@
 
 void *Comp(void *argument)
 {
-  int EQ, NE, LE, LT, GE, GT;
-  int operatorID;
   enum {FILL_NONE, FILL_TS, FILL_REC};
   int filltype = FILL_NONE;
-  int streamIDx1, streamIDx2, streamID1, streamID2, streamID3;
-  int gridsize, gridsize1, gridsize2;
-  int nrecs, nrecs2, nvars = 0, nlev, recID;
-  int tsID;
+  int gridsize1, gridsize2;
+  int nrecs, nrecs2, nvars = 0, nlev;
   int varID, levelID;
-  int offset;
-  int ntsteps1, ntsteps2;
-  int vlistIDx1, vlistIDx2, vlistID1, vlistID2, vlistID3;
-  int taxisIDx1, taxisID1, taxisID2, taxisID3;
-  int nmiss1, nmiss2, nmiss3;
-  int i;
   double missval1, missval2 = 0;
   double *missvalx1, *missvalx2;
-  double *arrayx1, *arrayx2, *array1, *array2, *array3;
   double **vardata = NULL;
 
   cdoInitialize(argument);
 
-  EQ = cdoOperatorAdd("eq", 0, 0, NULL);
-  NE = cdoOperatorAdd("ne", 0, 0, NULL);
-  LE = cdoOperatorAdd("le", 0, 0, NULL);
-  LT = cdoOperatorAdd("lt", 0, 0, NULL);
-  GE = cdoOperatorAdd("ge", 0, 0, NULL);
-  GT = cdoOperatorAdd("gt", 0, 0, NULL);
+  int EQ = cdoOperatorAdd("eq", 0, 0, NULL);
+  int NE = cdoOperatorAdd("ne", 0, 0, NULL);
+  int LE = cdoOperatorAdd("le", 0, 0, NULL);
+  int LT = cdoOperatorAdd("lt", 0, 0, NULL);
+  int GE = cdoOperatorAdd("ge", 0, 0, NULL);
+  int GT = cdoOperatorAdd("gt", 0, 0, NULL);
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
 
-  streamIDx1 = streamID1;
-  streamIDx2 = streamID2;
+  int streamIDx1 = streamID1;
+  int streamIDx2 = streamID2;
 
   missvalx1 = &missval1;
   missvalx2 = &missval2;
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistIDx1 = vlistID1;
-  vlistIDx2 = vlistID2;
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistIDx1 = vlistID1;
+  int vlistIDx2 = vlistID2;
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = vlistInqTaxis(vlistID2);
-  taxisIDx1 = taxisID1;
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = vlistInqTaxis(vlistID2);
+  int taxisIDx1 = taxisID1;
 
-  ntsteps1 = vlistNtsteps(vlistID1);
-  ntsteps2 = vlistNtsteps(vlistID2);
+  int ntsteps1 = vlistNtsteps(vlistID1);
+  int ntsteps2 = vlistNtsteps(vlistID2);
   if ( ntsteps1 == 0 ) ntsteps1 = 1;
   if ( ntsteps2 == 0 ) ntsteps2 = 1;
 
+  bool fillstream1 = false;
+
   if ( vlistNrecs(vlistID1) != 1 && vlistNrecs(vlistID2) == 1 )
     {
       filltype = FILL_REC;
       cdoPrint("Filling up stream2 >%s< by copying the first record.", cdoStreamName(1)->args);
+      if ( ntsteps2 != 1 ) cdoAbort("stream2 has more than 1 timestep!");
     }
   else if ( vlistNrecs(vlistID1) == 1 && vlistNrecs(vlistID2) != 1 )
     {
       filltype = FILL_REC;
       cdoPrint("Filling up stream1 >%s< by copying the first record.", cdoStreamName(0)->args);
+      if ( ntsteps1 != 1 ) cdoAbort("stream1 has more than 1 timestep!");
+      fillstream1 = true;
       streamIDx1 = streamID2;
       streamIDx2 = streamID1;
       vlistIDx1 = vlistID2;
@@ -110,14 +104,14 @@ void *Comp(void *argument)
   nospec(vlistID1);
   nospec(vlistID2);
 
-  gridsize = vlistGridsizeMax(vlistIDx1);
+  int gridsize = vlistGridsizeMax(vlistIDx1);
 
-  array1 = malloc(gridsize*sizeof(double));
-  array2 = malloc(gridsize*sizeof(double));
-  array3 = malloc(gridsize*sizeof(double));
+  double *array1 = (double*) Malloc(gridsize*sizeof(double));
+  double *array2 = (double*) Malloc(gridsize*sizeof(double));
+  double *array3 = (double*) Malloc(gridsize*sizeof(double));
 
-  arrayx1 = array1;
-  arrayx2 = array2;
+  double *arrayx1 = array1;
+  double *arrayx2 = array2;
 
   if ( cdoVerbose )
     cdoPrint("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
@@ -133,6 +127,7 @@ void *Comp(void *argument)
 	{
 	  filltype = FILL_TS;
 	  cdoPrint("Filling up stream1 >%s< by copying the first timestep.", cdoStreamName(0)->args);
+          fillstream1 = true;
 	  streamIDx1 = streamID2;
           streamIDx2 = streamID1;
 	  vlistIDx1 = vlistID2;
@@ -142,18 +137,18 @@ void *Comp(void *argument)
 
       if ( filltype == FILL_TS )
 	{
-	  nvars  = vlistNvars(vlistIDx2);
-	  vardata  = malloc(nvars*sizeof(double *));
+	  nvars = vlistNvars(vlistIDx2);
+	  vardata = (double **) Malloc(nvars*sizeof(double *));
 	  for ( varID = 0; varID < nvars; varID++ )
 	    {
 	      gridsize = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
 	      nlev     = zaxisInqSize(vlistInqVarZaxis(vlistIDx2, varID));
-	      vardata[varID]  = malloc(nlev*gridsize*sizeof(double));
+	      vardata[varID] = (double*) Malloc(nlev*gridsize*sizeof(double));
 	    }
 	}
     }
 
-  if ( filltype != FILL_NONE && ntsteps1 == 1 )
+  if ( fillstream1 )
     {
       arrayx1 = array2;
       arrayx2 = array1;
@@ -161,16 +156,16 @@ void *Comp(void *argument)
       missvalx2 = &missval1;
     }
 
-  vlistID3 = vlistDuplicate(vlistIDx1);
+  int vlistID3 = vlistDuplicate(vlistIDx1);
 
-  taxisID3 = taxisDuplicate(taxisIDx1);
+  int taxisID3 = taxisDuplicate(taxisIDx1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
+  int streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
 
   streamDefVlist(streamID3, vlistID3);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamIDx1, tsID)) )
     {
       if ( tsID == 0 || filltype == FILL_NONE )
@@ -184,8 +179,9 @@ void *Comp(void *argument)
 
       streamDefTimestep(streamID3, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
+          int nmiss1;
 	  streamInqRecord(streamIDx1, &varID, &levelID);
 	  streamReadRecord(streamIDx1, arrayx1, &nmiss1);
 
@@ -193,21 +189,22 @@ void *Comp(void *argument)
 	    {
 	      if ( recID == 0 || filltype != FILL_REC )
 		{
+                  int nmiss2;
 		  streamInqRecord(streamIDx2, &varID, &levelID);
 		  streamReadRecord(streamIDx2, arrayx2, &nmiss2);
 		}
 
 	      if ( filltype == FILL_TS )
 		{
-		  gridsize = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
-		  offset   = gridsize*levelID;
+		  int gridsize = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
+		  int offset = gridsize*levelID;
 		  memcpy(vardata[varID]+offset, arrayx2, gridsize*sizeof(double));
 		}
 	    }
 	  else if ( filltype == FILL_TS )
 	    {
-	      gridsize = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
-	      offset   = gridsize*levelID;
+	      int gridsize = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
+	      int offset = gridsize*levelID;
 	      memcpy(arrayx2, vardata[varID]+offset, gridsize*sizeof(double));
 	    }
 
@@ -232,37 +229,37 @@ void *Comp(void *argument)
 
 	  if ( operatorID == EQ )
 	    {
-	      for ( i = 0; i < gridsize; i++ )
+	      for ( int i = 0; i < gridsize; i++ )
 		array3[i] = (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) ?
 			     missval1 : DBL_IS_EQUAL(array1[i], array2[i]));
 	    }
 	  else if ( operatorID == NE )
 	    {
-	      for ( i = 0; i < gridsize; i++ )
+	      for ( int i = 0; i < gridsize; i++ )
 		array3[i] = (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) ?
 			     missval1 : !DBL_IS_EQUAL(array1[i], array2[i]));
 	    }
 	  else if ( operatorID == LE )
 	    {
-	      for ( i = 0; i < gridsize; i++ )
+	      for ( int i = 0; i < gridsize; i++ )
 		array3[i] = (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) ?
 			     missval1 : array1[i] <= array2[i]);
 	    }
 	  else if ( operatorID == LT )
 	    {
-	      for ( i = 0; i < gridsize; i++ )
+	      for ( int i = 0; i < gridsize; i++ )
 		array3[i] = (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) ?
 			     missval1 : array1[i] < array2[i]);
 	    }
 	  else if ( operatorID == GE )
 	    {
-	      for ( i = 0; i < gridsize; i++ )
+	      for ( int i = 0; i < gridsize; i++ )
 		array3[i] = (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) ?
 			     missval1 : array1[i] >= array2[i]);
 	    }
 	  else if ( operatorID == GT )
 	    {
-	      for ( i = 0; i < gridsize; i++ )
+	      for ( int i = 0; i < gridsize; i++ )
 		array3[i] = (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) ?
 			     missval1 : array1[i] > array2[i]);
 	    }
@@ -271,8 +268,8 @@ void *Comp(void *argument)
 	      cdoAbort("Operator not implemented!");
 	    }
 
-	  nmiss3 = 0;
-	  for ( i = 0; i < gridsize; i++ )
+	  int nmiss3 = 0;
+	  for ( int i = 0; i < gridsize; i++ )
 	    if ( DBL_IS_EQUAL(array3[i], missval1) ) nmiss3++;
 
 	  streamDefRecord(streamID3, varID, levelID);
@@ -288,15 +285,15 @@ void *Comp(void *argument)
 
   if ( vardata )
     {
-      for ( varID = 0; varID < nvars; varID++ ) free(vardata[varID]);
-      free(vardata);
+      for ( varID = 0; varID < nvars; varID++ ) Free(vardata[varID]);
+      Free(vardata);
     }
 
-  if ( array3 ) free(array3);
-  if ( array2 ) free(array2);
-  if ( array1 ) free(array1);
+  if ( array3 ) Free(array3);
+  if ( array2 ) Free(array2);
+  if ( array1 ) Free(array1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
