@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,61 +24,55 @@
 
 void *Complextorect(void *argument)
 {
-  // int COMPLEXTORECT;
-  // int operatorID;
-  int streamID1, streamID2, streamID3;
-  int tsID, nrecs;
-  int recID, varID, levelID;
-  int vlistID1, vlistID2, vlistID3;
-  int taxisID1, taxisID2, taxisID3;
-  int i, gridsize;
+  int nrecs;
+  int varID, levelID;
+  int i;
   int datatype;
-  int nmiss, nvars;
-  double *array1 = NULL, *array2 = NULL, *array3 = NULL;
+  int nmiss;
 
   cdoInitialize(argument);
 
-  //  COMPLEXTORECT = cdoOperatorAdd("complextorect", 0, 0, NULL);
+  // int COMPLEXTORECT = cdoOperatorAdd("complextorect", 0, 0, NULL);
 
-  //  operatorID = cdoOperatorID();
+  // int operatorID = cdoOperatorID();
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
-  vlistID3 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID3 = vlistDuplicate(vlistID1);
 
-  nvars = vlistNvars(vlistID2);
+  int nvars = vlistNvars(vlistID2);
   for ( varID = 0; varID < nvars; ++varID )
     {
       datatype = vlistInqVarDatatype(vlistID2, varID);
-      if ( datatype == DATATYPE_CPX64 )
-	datatype = DATATYPE_FLT64;
+      if ( datatype == CDI_DATATYPE_CPX64 )
+	datatype = CDI_DATATYPE_FLT64;
       else
-	datatype = DATATYPE_FLT32;
+	datatype = CDI_DATATYPE_FLT32;
 
       vlistDefVarDatatype(vlistID2, varID, datatype);
       vlistDefVarDatatype(vlistID3, varID, datatype);
     }
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
-  taxisID3 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
-  streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
   streamDefVlist(streamID3, vlistID3);
 
-  gridsize = vlistGridsizeMax(vlistID1);
-  array1 = (double*) malloc(2*gridsize*sizeof(double));
-  array2 = (double*) malloc(gridsize*sizeof(double));
-  array3 = (double*) malloc(gridsize*sizeof(double));
+  int gridsize = vlistGridsizeMax(vlistID1);
+  double *array1 = (double*) Malloc(2*gridsize*sizeof(double));
+  double *array2 = (double*) Malloc(gridsize*sizeof(double));
+  double *array3 = (double*) Malloc(gridsize*sizeof(double));
       
-  tsID  = 0;
+  int tsID  = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
@@ -87,7 +81,7 @@ void *Complextorect(void *argument)
       streamDefTimestep(streamID2, tsID);
       streamDefTimestep(streamID3, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamDefRecord(streamID2, varID, levelID);
@@ -114,14 +108,14 @@ void *Complextorect(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array1 ) free(array1);
-  if ( array2 ) free(array2);
-  if ( array3 ) free(array3);
+  if ( array1 ) Free(array1);
+  if ( array2 ) Free(array2);
+  if ( array3 ) Free(array3);
 
   vlistDestroy(vlistID2);
   vlistDestroy(vlistID3);
 
   cdoFinish();
 
-  return (NULL);
+  return 0;
 }
