@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -38,9 +38,9 @@
 enum {CONSECSUM, CONSECTS};
 #define SWITCHWARN "Hit default case!This should never happen (%s).\n"
 
-static void selEndOfPeriod(field_t *periods, field_t history, field_t current, int isLastTimestep)
+static void selEndOfPeriod(field_type *periods, field_type history, field_type current, int isLastTimestep)
 {
-  long   i, len;
+  long   i;
   double pmissval = periods->missval;
   double  *parray = periods->ptr;
   double hmissval = history.missval;
@@ -48,7 +48,7 @@ static void selEndOfPeriod(field_t *periods, field_t history, field_t current, i
   double cmissval = current.missval;
   double  *carray = current.ptr;
 
-  len = gridInqSize(periods->grid);
+  long len = gridInqSize(periods->grid);
   if ( len != gridInqSize(current.grid) || (gridInqSize(current.grid) != gridInqSize(history.grid)) )
     cdoAbort("Fields have different gridsize (%s)", __func__);
 
@@ -119,42 +119,40 @@ static void selEndOfPeriod(field_t *periods, field_t history, field_t current, i
     if ( DBL_IS_EQUAL(parray[i], pmissval) ) periods->nmiss++;
 }
 
+
 void *Consecstat(void *argument)
 {
-  int operatorID;
-  int istreamID, itaxisID, ivlistID, itsID;
-  int ostreamID, otaxisID, ovlistID, otsID;
   int vdate = 0, vtime = 0;
   int histvdate = 0, histvtime = 0;
-  int recID, nrecs;
-  int varID, nvars;
-  int levelID, nlevels; 
+  int nrecs;
+  int varID;
+  int levelID, nlevels;
+  int nmiss;
   double refval = 0.0;
 
-  field_t **vars = NULL, **hist = NULL, **periods = NULL;
-  field_t field;
-
   cdoInitialize(argument);
   cdoOperatorAdd("consecsum",CONSECSUM, 0, "refval");
   cdoOperatorAdd("consects" ,CONSECTS , 0, NULL);
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   if ( operatorID == CONSECSUM )
-    if ( operatorArgc() > 0 ) refval = atof(operatorArgv()[0]);
+    if ( operatorArgc() > 0 ) refval = parameter2double(operatorArgv()[0]);
 
-  istreamID = streamOpenRead(cdoStreamName(0));
+  int istreamID = streamOpenRead(cdoStreamName(0));
 
-  ivlistID = streamInqVlist(istreamID);
-  itaxisID = vlistInqTaxis(ivlistID);
-  ovlistID = vlistDuplicate(ivlistID);
-  otaxisID = taxisDuplicate(itaxisID);
+  int ivlistID = streamInqVlist(istreamID);
+  int itaxisID = vlistInqTaxis(ivlistID);
+  int ovlistID = vlistDuplicate(ivlistID);
+  int otaxisID = taxisDuplicate(itaxisID);
   vlistDefTaxis(ovlistID, otaxisID);
 
+  field_type field;
   field_init(&field);
-  field.ptr = (double*) malloc(vlistGridsizeMax(ovlistID)*sizeof(double));
+  field.ptr = (double*) Malloc(vlistGridsizeMax(ovlistID)*sizeof(double));
 
-  nvars     = vlistNvars(ivlistID);
-  vars      = field_calloc(ivlistID, FIELD_PTR);
+  int nvars = vlistNvars(ivlistID);
+  field_type **vars = field_calloc(ivlistID, FIELD_PTR);
+  field_type **hist = NULL, **periods = NULL;
   if ( operatorID == CONSECTS )
     {
       hist      = field_malloc(ivlistID, FIELD_PTR);
@@ -166,12 +164,12 @@ void *Consecstat(void *argument)
       vlistDefVarUnits(ovlistID, varID, "steps"); /* TODO */
     }
 
-  ostreamID = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int ostreamID = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(ostreamID, ovlistID);
 
-  itsID = 0;
-  otsID = 0;
+  int itsID = 0;
+  int otsID = 0;
   while ( (nrecs = streamInqTimestep(istreamID, itsID)) )
   {
     vdate = taxisInqVdate(itaxisID);
@@ -196,10 +194,11 @@ void *Consecstat(void *argument)
         break;
     }
 
-    for ( recID = 0; recID < nrecs; recID++ )
+    for ( int recID = 0; recID < nrecs; recID++ )
     {
       streamInqRecord(istreamID, &varID, &levelID);
-      streamReadRecord(istreamID, field.ptr, &field.nmiss);
+      streamReadRecord(istreamID, field.ptr, &nmiss);
+      field.nmiss   = (size_t)nmiss;
       field.grid    = vlistInqVarGrid(ovlistID, varID);
       field.missval = vlistInqVarMissval(ovlistID, varID);
 
@@ -209,14 +208,14 @@ void *Consecstat(void *argument)
       {
         case CONSECSUM:
           streamDefRecord(ostreamID, varID, levelID);
-          streamWriteRecord(ostreamID, vars[varID][levelID].ptr, vars[varID][levelID].nmiss);
+          streamWriteRecord(ostreamID, vars[varID][levelID].ptr, (int)vars[varID][levelID].nmiss);
           break;
         case CONSECTS:
           if ( itsID != 0 )
           {
             selEndOfPeriod(&periods[varID][levelID], hist[varID][levelID], vars[varID][levelID], FALSE);
             streamDefRecord(ostreamID, varID, levelID);
-            streamWriteRecord(ostreamID, periods[varID][levelID].ptr, periods[varID][levelID].nmiss);
+            streamWriteRecord(ostreamID, periods[varID][levelID].ptr, (int)periods[varID][levelID].nmiss);
           }
 #if defined(_OPENMP)
 #pragma omp parallel for default(shared) schedule(static)
@@ -251,7 +250,7 @@ void *Consecstat(void *argument)
       {
         selEndOfPeriod(&periods[varID][levelID], hist[varID][levelID], vars[varID][levelID], TRUE);
         streamDefRecord(ostreamID, varID, levelID);
-        streamWriteRecord(ostreamID, periods[varID][levelID].ptr, periods[varID][levelID].nmiss);
+        streamWriteRecord(ostreamID, periods[varID][levelID].ptr, (int)periods[varID][levelID].nmiss);
       }
     }
   }
@@ -265,5 +264,5 @@ void *Consecstat(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
