@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -35,7 +35,7 @@ void *Copy(void *argument)
   int vlistID2 = CDI_UNDEFID;
   int taxisID2 = CDI_UNDEFID;
   int nrecs;
-  int recID, varID, levelID;
+  int varID, levelID;
   int nmiss;
   int ntsteps, nvars;
   double *array = NULL;
@@ -43,18 +43,17 @@ void *Copy(void *argument)
 
   cdoInitialize(argument);
 
+  bool lcopy = UNCHANGED_RECORD;
+
                 cdoOperatorAdd("copy",   0, 0, NULL);
   int SELALL  = cdoOperatorAdd("selall", 0, 0, NULL);
   int SZIP    = cdoOperatorAdd("szip",   0, 0, NULL);
 
-  bool lcopy = false;
-  if ( UNCHANGED_RECORD ) lcopy = true;
-
   int operatorID = cdoOperatorID();
 
   if ( operatorID == SZIP )
     {
-      cdoCompType  = COMPRESS_SZIP;
+      cdoCompType  = CDI_COMPRESS_SZIP;
       cdoCompLevel = 0;
     }
 
@@ -120,7 +119,7 @@ void *Copy(void *argument)
 
 	  streamDefTimestep(streamID2, tsID2);
 	       
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    { 
 	      if ( lcopy && (operatorID == SELALL || operatorID == SZIP) )
 		{
