@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,6 @@
       Detrend    detrend         Detrend
 */
 
-#if defined(_OPENMP)
-#  include <omp.h>
-#endif
 
 #include <cdi.h>
 #include "cdo.h"
@@ -37,18 +34,15 @@
 static
 void detrend(long nts, double missval1, double *array1, double *array2)
 {
-  long n;
-  long j;
   double zj;
   double sumj, sumjj;
   double sumx, sumjx;
-  double work1, work2;
   double missval2 = missval1;
 
   sumx = sumjx = 0;
   sumj = sumjj = 0;
-  n = 0;
-  for ( j = 0; j < nts; j++ )
+  long n = 0;
+  for ( long j = 0; j < nts; j++ )
     if ( !DBL_IS_EQUAL(array1[j], missval1) )
       {
         zj = j;
@@ -59,100 +53,69 @@ void detrend(long nts, double missval1, double *array1, double *array2)
 	n++;
       }
 
-  work1 = DIV(SUB(sumjx, DIV(MUL(sumx, sumj), n) ),
-	      SUB(sumjj, DIV(MUL(sumj, sumj), n)) );
-  work2 = SUB(DIV(sumx, n), MUL(work1, DIV(sumj, n)));
+  double work1 = DIVMN( SUBMN(sumjx, DIVMN( MULMN(sumx, sumj), n) ),
+                        SUBMN(sumjj, DIVMN( MULMN(sumj, sumj), n)) );
+  double work2 = SUBMN( DIVMN(sumx, n), MULMN(work1, DIVMN(sumj, n)));
 
-  for ( j = 0; j < nts; j++ )
-    array2[j] = SUB(array1[j], ADD(work2, MUL(j, work1)));
-}
-
-
-void taxisInqDTinfo(int taxisID, dtinfo_t *dtinfo)
-{
-  dtinfo->v.date = taxisInqVdate(taxisID);
-  dtinfo->v.time = taxisInqVtime(taxisID);
-  if ( taxisHasBounds(taxisID) )
-    {
-      taxisInqVdateBounds(taxisID, &(dtinfo->b[0].date), &(dtinfo->b[1].date));
-      taxisInqVtimeBounds(taxisID, &(dtinfo->b[0].time), &(dtinfo->b[1].time));
-    }
-}
-
-
-void taxisDefDTinfo(int taxisID, dtinfo_t dtinfo)
-{
-  taxisDefVdate(taxisID, dtinfo.v.date);
-  taxisDefVtime(taxisID, dtinfo.v.time);
-  if ( taxisHasBounds(taxisID) )
-    {
-      taxisDefVdateBounds(taxisID, dtinfo.b[0].date, dtinfo.b[1].date);
-      taxisDefVtimeBounds(taxisID, dtinfo.b[0].time, dtinfo.b[1].time);
-    }
+  for ( long j = 0; j < nts; j++ )
+    array2[j] = SUBMN(array1[j], ADDMN(work2, MULMN(j, work1)));
 }
 
 
 void *Detrend(void *argument)
 {
-  int ompthID;
   int gridsize;
   int nrecs;
-  int gridID, varID, levelID, recID;
-  int tsID;
+  int gridID, varID, levelID;
   int i;
-  int nts;
   int nalloc = 0;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2, taxisID1, taxisID2;
   int nmiss;
-  int nvars, nlevel;
+  int nlevel;
   double missval;
-  field_t ***vars = NULL;
-  dtinfo_t *dtinfo = NULL;
+  field_type ***vars = NULL;
+  dtlist_type *dtlist = dtlist_new();
   typedef struct
   {
     double *array1;
     double *array2;
   } memory_t;
-  memory_t *mem = NULL;
 
   cdoInitialize(argument);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       if ( tsID >= nalloc )
 	{
 	  nalloc += NALLOC_INC;
-	  dtinfo = realloc(dtinfo, nalloc*sizeof(dtinfo_t));
-	  vars   = realloc(vars, nalloc*sizeof(field_t **));
+	  vars   = (field_type ***) Realloc(vars, nalloc*sizeof(field_type **));
 	}
 
-      taxisInqDTinfo(taxisID1, &dtinfo[tsID]);
+      dtlist_taxisInqTimestep(dtlist, taxisID1, tsID);
 
       vars[tsID] = field_malloc(vlistID1, FIELD_NONE);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridID   = vlistInqVarGrid(vlistID1, varID);
 	  gridsize = gridInqSize(gridID);
-	  vars[tsID][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+	  vars[tsID][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 	  streamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
 	  vars[tsID][varID][levelID].nmiss = nmiss;
 	}
@@ -160,13 +123,13 @@ void *Detrend(void *argument)
       tsID++;
     }
 
-  nts = tsID;
+  int nts = tsID;
 
-  mem = malloc(ompNumThreads*sizeof(memory_t));
+  memory_t *ompmem = (memory_t*) Malloc(ompNumThreads*sizeof(memory_t));
   for ( i = 0; i < ompNumThreads; i++ )
     {
-      mem[i].array1 = malloc(nts*sizeof(double));
-      mem[i].array2 = malloc(nts*sizeof(double));
+      ompmem[i].array1 = (double*) Malloc(nts*sizeof(double));
+      ompmem[i].array2 = (double*) Malloc(nts*sizeof(double));
     }
 
   for ( varID = 0; varID < nvars; varID++ )
@@ -178,36 +141,33 @@ void *Detrend(void *argument)
       for ( levelID = 0; levelID < nlevel; levelID++ )
 	{
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, ompthID, tsID)
+#pragma omp parallel for default(none) shared(ompmem, vars, varID, levelID, gridsize, nts, missval)
 #endif
 	  for ( i = 0; i < gridsize; i++ )
 	    {
-#if defined(_OPENMP)
-              ompthID = omp_get_thread_num();
-#else
-              ompthID = 0;
-#endif
-	      for ( tsID = 0; tsID < nts; tsID++ )
-		mem[ompthID].array1[tsID] = vars[tsID][varID][levelID].ptr[i];
+              int ompthID = cdo_omp_get_thread_num();
 
-	      detrend(nts, missval, mem[ompthID].array1, mem[ompthID].array2);
+	      for ( int tsID = 0; tsID < nts; tsID++ )
+		ompmem[ompthID].array1[tsID] = vars[tsID][varID][levelID].ptr[i];
 
-	      for ( tsID = 0; tsID < nts; tsID++ )
-		vars[tsID][varID][levelID].ptr[i] = mem[ompthID].array2[tsID];
+	      detrend(nts, missval, ompmem[ompthID].array1, ompmem[ompthID].array2);
+
+	      for ( int tsID = 0; tsID < nts; tsID++ )
+		vars[tsID][varID][levelID].ptr[i] = ompmem[ompthID].array2[tsID];
 	    }
 	}
     }
 
   for ( i = 0; i < ompNumThreads; i++ )
     {
-      free(mem[i].array1);
-      free(mem[i].array2);
+      Free(ompmem[i].array1);
+      Free(ompmem[i].array2);
     }
-  free(mem);
+  Free(ompmem);
 
-  for ( tsID = 0; tsID < nts; tsID++ )
+  for ( int tsID = 0; tsID < nts; tsID++ )
     {
-      taxisDefDTinfo(taxisID2, dtinfo[tsID]);
+      dtlist_taxisDefTimestep(dtlist, taxisID2, tsID);
       streamDefTimestep(streamID2, tsID);
 
       for ( varID = 0; varID < nvars; varID++ )
@@ -220,7 +180,7 @@ void *Detrend(void *argument)
 		  nmiss = vars[tsID][varID][levelID].nmiss;
 		  streamDefRecord(streamID2, varID, levelID);
 		  streamWriteRecord(streamID2, vars[tsID][varID][levelID].ptr, nmiss);
-		  free(vars[tsID][varID][levelID].ptr);
+		  Free(vars[tsID][varID][levelID].ptr);
 		  vars[tsID][varID][levelID].ptr = NULL;
 		}
 	    }
@@ -229,13 +189,14 @@ void *Detrend(void *argument)
       field_free(vars[tsID], vlistID1);
     }
 
-  if ( vars  ) free(vars);
-  if ( dtinfo ) free(dtinfo);
+  if ( vars  ) Free(vars);
+
+  dtlist_delete(dtlist);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
