@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -271,7 +271,7 @@ int import_e5ml(const char *filename, VAR **vars)
   cdoAbort("NetCDF support not compiled in!");
 #endif
 
-  return (nvars);
+  return nvars;
 }
 
 static
@@ -1078,7 +1078,7 @@ int import_e5res(const char *filename, VAR **vars, ATTS *atts)
   cdoAbort("NetCDF support not compiled in!");
 #endif
 
-  return (nvars);
+  return nvars;
 }
 
 static
@@ -1427,13 +1427,9 @@ void export_e5res(const char *filename, VAR *vars, int nvars)
 
 void *Echam5ini(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int IMPORT_E5ML, IMPORT_E5RES;
-  int EXPORT_E5ML, EXPORT_E5RES;
   int streamID1, streamID2 = CDI_UNDEFID;
   int nrecs = 0;
-  int recID, varID, levelID;
+  int varID, levelID;
   int vlistID1, vlistID2;
   int nvars = 0;
   int iv, nlev;
@@ -1442,13 +1438,13 @@ void *Echam5ini(void *argument)
 
   cdoInitialize(argument);
 
-  IMPORT_E5ML  = cdoOperatorAdd("import_e5ml",  func_read,  0, NULL);
-  IMPORT_E5RES = cdoOperatorAdd("import_e5res", func_read,  0, NULL);
-  EXPORT_E5ML  = cdoOperatorAdd("export_e5ml",  func_write, 0, NULL);
-  EXPORT_E5RES = cdoOperatorAdd("export_e5res", func_write, 0, NULL);
+  int IMPORT_E5ML  = cdoOperatorAdd("import_e5ml",  func_read,  0, NULL);
+  int IMPORT_E5RES = cdoOperatorAdd("import_e5res", func_read,  0, NULL);
+  int EXPORT_E5ML  = cdoOperatorAdd("export_e5ml",  func_write, 0, NULL);
+  int EXPORT_E5RES = cdoOperatorAdd("export_e5res", func_write, 0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
 
   if ( operatorID == EXPORT_E5ML && processSelf() != 0 )
     cdoAbort("This operator can't be linked with other operators!");
@@ -1482,13 +1478,13 @@ void *Echam5ini(void *argument)
 	  if ( vars[iv].name )     vlistDefVarName(vlistID2, varID, vars[iv].name);
 	  if ( vars[iv].longname ) vlistDefVarLongname(vlistID2, varID, vars[iv].longname);
           if ( vars[iv].units )    vlistDefVarUnits(vlistID2, varID, vars[iv].units);
-	  vlistDefVarDatatype(vlistID2, varID, DATATYPE_FLT64);
+	  vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT64);
 	}
 
       for ( iatt = 0; iatt < atts.natxt; ++iatt )
 	{
 	  /* printf("%s: %s\n", atts.atxtname[iatt], atts.atxtentry[iatt]); */
-	  vlistDefAttTxt(vlistID2, CDI_GLOBAL, atts.atxtname[iatt],
+	  cdiDefAttTxt(vlistID2, CDI_GLOBAL, atts.atxtname[iatt],
 			 (int)strlen(atts.atxtentry[iatt])+1, atts.atxtentry[iatt]);
 	}
 
@@ -1496,7 +1492,7 @@ void *Echam5ini(void *argument)
       vlistDefTaxis(vlistID2, taxisID);
 
       if ( cdoDefaultFileType == CDI_UNDEFID )
-	cdoDefaultFileType = FILETYPE_NC;
+	cdoDefaultFileType = CDI_FILETYPE_NC;
 
       streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
@@ -1600,7 +1596,7 @@ void *Echam5ini(void *argument)
 	  vtime = 120000;
 	}
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
