@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,6 @@
       Ensstat    enspctl         Ensemble percentiles
 */
 
-
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
@@ -40,14 +39,7 @@
 
 void *Ensstat(void *argument)
 {
-  int i;
-  int varID = 0, recID;
-  int gridID;
-  int nrecs, nrecs0;
-  int levelID = 0;
-  int streamID = 0;
-  int nmiss;
-  double missval;
+  int nrecs0;
   typedef struct
   {
     int streamID;
@@ -84,10 +76,10 @@ void *Ensstat(void *argument)
       argc--;
     }
 
-  int count_data = FALSE;
+  bool count_data = false;
   if ( argc == 1 )
     {
-      if ( strcmp("count", operatorArgv()[nargc-1]) == 0 ) count_data = TRUE;
+      if ( strcmp("count", operatorArgv()[nargc-1]) == 0 ) count_data = true;
       else cdoAbort("Unknown parameter: >%s<", operatorArgv()[nargc-1]); 
     }
     
@@ -97,15 +89,13 @@ void *Ensstat(void *argument)
 
   const char *ofilename = cdoStreamName(nfiles)->args;
 
-  if ( !cdoSilentMode && !cdoOverwriteMode )
-    if ( fileExists(ofilename) )
-      if ( !userFileOverwrite(ofilename) )
-	cdoAbort("Outputfile %s already exists!", ofilename);
+  if ( !cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename) )
+    cdoAbort("Outputfile %s already exists!", ofilename);
 
   ens_file_t *ef = (ens_file_t *) Malloc(nfiles*sizeof(ens_file_t));
 
-  field_t *field = (field_t *) Malloc(ompNumThreads*sizeof(field_t));
-  for ( i = 0; i < ompNumThreads; i++ )
+  field_type *field = (field_type *) Malloc(ompNumThreads*sizeof(field_type));
+  for ( int i = 0; i < ompNumThreads; i++ )
     {
       field_init(&field[i]);
       field[i].size   = nfiles;
@@ -142,17 +132,17 @@ void *Ensstat(void *argument)
   if ( count_data )
     {
       count2 = (double *) Malloc(gridsize*sizeof(double));
-      for ( varID = 0; varID < nvars; ++varID )
+      for ( int varID = 0; varID < nvars; ++varID )
 	{
 	  char name[CDI_MAX_NAME];
 	  vlistInqVarName(vlistID2, varID, name);
 	  strcat(name, "_count");
-	  gridID = vlistInqVarGrid(vlistID2, varID);
+	  int gridID = vlistInqVarGrid(vlistID2, varID);
 	  int zaxisID = vlistInqVarZaxis(vlistID2, varID);
 	  int tsteptype = vlistInqVarTsteptype(vlistID2, varID);
 	  int cvarID = vlistDefVar(vlistID2, gridID, zaxisID, tsteptype);
 	  vlistDefVarName(vlistID2, cvarID, name);
-	  vlistDefVarDatatype(vlistID2, cvarID, DATATYPE_INT16);
+	  vlistDefVarDatatype(vlistID2, cvarID, CDI_DATATYPE_INT16);
 	  if ( cvarID != (varID+nvars) ) cdoAbort("Internal error, varIDs do not match!");
 	}
     }
@@ -161,21 +151,35 @@ void *Ensstat(void *argument)
 
   streamDefVlist(streamID2, vlistID2);
 
+  bool lwarning = false;
+  bool lerror = false;
   int tsID = 0;
   do
     {
       nrecs0 = streamInqTimestep(ef[0].streamID, tsID);
       for ( int fileID = 1; fileID < nfiles; fileID++ )
 	{
-	  streamID = ef[fileID].streamID;
-	  nrecs = streamInqTimestep(streamID, tsID);
+	  int streamID = ef[fileID].streamID;
+	  int nrecs = streamInqTimestep(streamID, tsID);
 	  if ( nrecs != nrecs0 )
 	    {
 	      if ( nrecs == 0 )
-		cdoAbort("Inconsistent ensemble file, too few time steps in %s!", cdoStreamName(fileID)->args);
+                {
+                  lwarning = true;
+                  cdoWarning("Inconsistent ensemble file, too few time steps in %s!", cdoStreamName(fileID)->args);
+                }
+	      else if ( nrecs0 == 0 )
+                {
+                  lwarning = true;
+                  cdoWarning("Inconsistent ensemble file, too few time steps in %s!", cdoStreamName(0)->args);
+                }
 	      else
-		cdoAbort("Inconsistent ensemble file, number of records at time step %d of %s and %s differ!",
-			 tsID+1, cdoStreamName(0)->args, cdoStreamName(fileID)->args);
+                {
+                  lerror = true;
+                  cdoWarning("Inconsistent ensemble file, number of records at time step %d of %s and %s differ!",
+                             tsID+1, cdoStreamName(0)->args, cdoStreamName(fileID)->args);
+                }
+              goto CLEANUP;
 	    }
 	}
 
@@ -185,30 +189,30 @@ void *Ensstat(void *argument)
 	  streamDefTimestep(streamID2, tsID);
 	}
 
-      for ( recID = 0; recID < nrecs0; recID++ )
+      for ( int recID = 0; recID < nrecs0; recID++ )
 	{
+          int varID = 0, levelID;
 #if defined(_OPENMP)
-#pragma omp parallel for default(none) shared(ef, nfiles)         \
-                                         private(streamID, nmiss) \
-                                     lastprivate(varID, levelID)
+#pragma omp parallel for default(none) shared(ef, nfiles) lastprivate(varID, levelID)
 #endif
 	  for ( int fileID = 0; fileID < nfiles; fileID++ )
 	    {
-	      streamID = ef[fileID].streamID;
+              int nmiss;
+	      int streamID = ef[fileID].streamID;
 	      streamInqRecord(streamID, &varID, &levelID);
 	      streamReadRecord(streamID, ef[fileID].array, &nmiss);
               ef[fileID].missval = vlistInqVarMissval(ef[fileID].vlistID, varID);
 	    }
 
-	  gridID   = vlistInqVarGrid(vlistID1, varID);
+	  int gridID = vlistInqVarGrid(vlistID1, varID);
 	  gridsize = gridInqSize(gridID);
-	  missval  = vlistInqVarMissval(vlistID1, varID);
+	  double missval = vlistInqVarMissval(vlistID1, varID);
 
-	  nmiss = 0;
+	  int nmiss = 0;
 #if defined(_OPENMP)
 #pragma omp parallel for default(shared)
 #endif
-	  for ( i = 0; i < gridsize; ++i )
+	  for ( int i = 0; i < gridsize; ++i )
 	    {
 	      int ompthID = cdo_omp_get_thread_num();
 
@@ -254,6 +258,11 @@ void *Ensstat(void *argument)
     }
   while ( nrecs0 > 0 );
 
+ CLEANUP:
+
+  if ( lwarning ) cdoWarning("Inconsistent ensemble, processed only the first %d timesteps!", tsID);
+  if ( lerror ) cdoAbort("Inconsistent ensemble, processed only the first %d timesteps!", tsID);
+
   for ( int fileID = 0; fileID < nfiles; fileID++ )
     streamClose(ef[fileID].streamID);
 
@@ -266,7 +275,7 @@ void *Ensstat(void *argument)
   if ( array2 ) Free(array2);
   if ( count2 ) Free(count2);
 
-  for ( i = 0; i < ompNumThreads; i++ )
+  for ( int i = 0; i < ompNumThreads; i++ )
     {
       if ( field[i].ptr    ) Free(field[i].ptr);
       if ( field[i].weight ) Free(field[i].weight);
