@@ -2,7 +2,7 @@
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate model Data.
  
- Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+ Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
  See COPYING file for copying and redistribution conditions.
  
  This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,6 @@
  
  Eofcoeff             eofcoeff             process eof coefficients
 */
-#define WEIGHTS 1
 
 #include <cdi.h>
 #include "cdo.h"
@@ -34,119 +33,93 @@
 void *Eofcoeff(void * argument)
 {
   char eof_name[8], oname[1024], filesuffix[32];
-  const char *refname;
-  double *w;
   double missval1 = -999, missval2;
-  double *xvals, *yvals;  
-  field_t ***eof;  
-  field_t in;  
-  field_t out;
-  int operatorID, operfunc;  
-  int gridsize;
-  int i, varID, recID, levelID, tsID, eofID;    
-  int gridID1, gridID2, gridID3;
-  int nrecs, nvars, nlevs, neof, nchars, nmiss, ngrids; 
-  int reached_eof;
-  int streamID1, streamID2, *streamIDs;
-  int taxisID1, taxisID2, taxisID3;
-  int vlistID1, vlistID2, vlistID3;
+  field_type in;  
+  field_type out;
+  int i, varID, levelID;    
+  int nrecs, nmiss; 
    
   cdoInitialize(argument);
-  cdoOperatorAdd("eofcoeff",  0,       0, NULL);
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+
+  if ( processSelf() != 0 ) cdoAbort("This operator can't be combined with other operators!");
+
+  cdoOperatorAdd("eofcoeff",  0,  0, NULL);
      
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
   
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistID3 = vlistDuplicate(vlistID2);   
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistID3 = vlistDuplicate(vlistID2);   
   
-  taxisID1 = vlistInqTaxis(vlistID1);  
-  taxisID2 = vlistInqTaxis(vlistID2); 
-  taxisID3 = taxisDuplicate(taxisID2);
+  //taxisID1 = vlistInqTaxis(vlistID1);  
+  int taxisID2 = vlistInqTaxis(vlistID2); 
+  int taxisID3 = taxisDuplicate(taxisID2);
   
-  gridID1 = vlistInqVarGrid(vlistID1, 0);
-  gridID2 = vlistInqVarGrid(vlistID2, 0);
+  int gridID1 = vlistInqVarGrid(vlistID1, 0);
+  int gridID2 = vlistInqVarGrid(vlistID2, 0);
   
-  if ( vlistGridsizeMax(vlistID1)==vlistGridsizeMax(vlistID2) )
-    gridsize = vlistGridsizeMax(vlistID1);  
-  else 
-    {
-      gridsize = -1;
-      cdoAbort ("Gridsize of input files does not match");
-    }
+  int gridsize = vlistGridsizeMax(vlistID1);  
+  if ( gridsize != vlistGridsizeMax(vlistID2) )
+    cdoAbort("Gridsize of input files does not match!");
       
-  
   if ( vlistNgrids(vlistID2) > 1 || vlistNgrids(vlistID1) > 1 )
-    cdoAbort("Too many grids in input");
+    cdoAbort("Too many different grids in input!");
   
-  nvars = vlistNvars(vlistID1)==vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
-  nrecs = vlistNrecs(vlistID1); 
-  nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
-  w = malloc(gridsize*sizeof(double));
-  gridWeights(gridID2, &w[0]);
-  
-  
-  
-  if (vlistGridsizeMax(vlistID2)   != gridsize ||
-      vlistInqVarGrid(vlistID2, 0) != gridID1 )
-    cdoAbort("EOFs (%s) and data (%s) defined on different grids", cdoStreamName(0)->args, cdoStreamName(1)->args);    
- 
+  int nvars = vlistNvars(vlistID1)==vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
+  int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));  
+
+  if ( gridID1 != gridID2 ) cdoCompareGrids(gridID1, gridID2);
   
   strcpy(oname, cdoStreamName(2)->args);
-  nchars = strlen(oname);
+  int nchars = strlen(oname);
   
-  refname = cdoStreamName(0)->argv[cdoStreamName(0)->argc-1];
+  const char *refname = cdoStreamName(0)->argv[cdoStreamName(0)->argc-1];
   filesuffix[0] = 0;
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), streamInqFiletype(streamID1), vlistID1, refname);
   
-  eof = malloc (nvars * sizeof(field_t**) );
-  for ( varID=0; varID<nvars; varID++)
-    eof[varID] = malloc(nlevs*sizeof(field_t*));
-  reached_eof=0;
-  eofID = 0;
+  field_type ***eof = (field_type***) Malloc(nvars * sizeof(field_type**));
+  for ( varID=0; varID<nvars; varID++ )
+    eof[varID] = (field_type**) Malloc(nlevs*sizeof(field_type*));
+
+  int eofID = 0;
   while ( 1 )       
    {     
      nrecs = streamInqTimestep(streamID1, eofID);
-     if ( nrecs == 0)
-       {
-         reached_eof = 1;
-         break;
-       }
-     for ( recID = 0; recID < nrecs; recID++ )
+     if ( nrecs == 0) break;
+
+     for ( int recID = 0; recID < nrecs; recID++ )
        {         
          streamInqRecord(streamID1, &varID, &levelID);
          missval1 = vlistInqVarMissval(vlistID1, varID);
          if ( eofID == 0 )
-           eof[varID][levelID] = malloc (1*sizeof(field_t));
+           eof[varID][levelID] = (field_type*) Malloc(1*sizeof(field_type));
          else
-           eof[varID][levelID] = realloc (eof[varID][levelID], (eofID+1)*sizeof(field_t));
+           eof[varID][levelID] = (field_type*) Realloc(eof[varID][levelID], (eofID+1)*sizeof(field_type));
          eof[varID][levelID][eofID].grid   = gridID1;
          eof[varID][levelID][eofID].nmiss  = 0;
          eof[varID][levelID][eofID].missval= missval1;
-         eof[varID][levelID][eofID].ptr    = malloc(gridsize*sizeof(double));
+         eof[varID][levelID][eofID].ptr    = (double*) Malloc(gridsize*sizeof(double));
          memset(&eof[varID][levelID][eofID].ptr[0], missval1, gridsize*sizeof(double));
          if ( varID >= nvars )
            cdoAbort("Internal error - too high varID");
          if ( levelID >= nlevs )
            cdoAbort("Internal error - too high levelID");
          
-         streamReadRecord(streamID1, eof[varID][levelID][eofID].ptr, 
-                          &eof[varID][levelID][eofID].nmiss);
+         streamReadRecord(streamID1, eof[varID][levelID][eofID].ptr, &nmiss);
+         eof[varID][levelID][eofID].nmiss = (size_t) nmiss;
        }
      eofID++;
    }
-  neof = eofID;  
+  int neof = eofID;  
   
   if ( cdoVerbose ) cdoPrint("%s contains %i eof's", cdoStreamName(0)->args, neof);
   // Create 1x1 Grid for output
-  gridID3 = gridCreate(GRID_LONLAT, 1);
+  int gridID3 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID3, 1);
   gridDefYsize(gridID3, 1);
-  xvals=malloc(1*sizeof(double));
-  yvals=malloc(1*sizeof(double));
+  double *xvals = (double*) Malloc(1*sizeof(double));
+  double *yvals = (double*) Malloc(1*sizeof(double));
   xvals[0]=0;
   yvals[0]=0;
   gridDefXvals(gridID3, xvals);
@@ -154,7 +127,7 @@ void *Eofcoeff(void * argument)
   
   // Create var-list and time-axis for output
       
-  ngrids = vlistNgrids(vlistID3);
+  int ngrids = vlistNgrids(vlistID3);
   
   for ( i = 0; i < ngrids; i++ )
     vlistChangeGridIndex(vlistID3, i, gridID3);     
@@ -164,8 +137,7 @@ void *Eofcoeff(void * argument)
     vlistDefVarTsteptype(vlistID3, varID, TSTEP_INSTANT);
   
   // open streams for eofcoeff output
-  streamIDs = malloc (neof*sizeof(int)); 
-  eofID = 0;
+  int *streamIDs = (int*) Malloc(neof*sizeof(int)); 
   for ( eofID = 0; eofID < neof; eofID++)
     {
       oname[nchars] = '\0';                       
@@ -183,27 +155,20 @@ void *Eofcoeff(void * argument)
         cdoPrint("opened %s ('w')  as stream%i for %i. eof", oname, streamIDs[eofID], eofID+1);
       
       streamDefVlist(streamIDs[eofID], vlistID3);
-    
     }
   
   // ALLOCATE temporary fields for data read and write
-  in.ptr = malloc(gridsize*sizeof(double));
+  in.ptr = (double*) Malloc(gridsize*sizeof(double));
   in.grid = gridID1;  
   out.missval = missval1;
   out.nmiss = 0;
-  out.ptr = malloc (1*sizeof(double));
+  out.ptr = (double*) Malloc(1*sizeof(double));
  
-  // 
-  reached_eof=0;
-  tsID=0;
+  int tsID = 0;
   while ( 1 )
     {      
       nrecs = streamInqTimestep(streamID2, tsID);
-      if ( nrecs == 0 )
-        {
-          reached_eof=1;
-          break;
-        }
+      if ( nrecs == 0 ) break;
       
       taxisCopyTimestep(taxisID3, taxisID2);
       /*for ( eofID=0; eofID<neof; eofID++)
@@ -212,13 +177,14 @@ void *Eofcoeff(void * argument)
           streamDefTimestep(streamIDs[eofID],tsID);
         }
       */
-      for ( recID =0; recID< nrecs; recID++ )
+      for ( int recID = 0; recID< nrecs; recID++ )
         {
           streamInqRecord(streamID2, &varID, &levelID);
           missval2 = vlistInqVarMissval(vlistID2, varID);
-          streamReadRecord(streamID2, in.ptr, &in.nmiss);  
+          streamReadRecord(streamID2, in.ptr, &nmiss);  
+          in.nmiss = (size_t) nmiss;
           
-          for (eofID = 0; eofID < neof; eofID++ )
+          for ( eofID = 0; eofID < neof; eofID++ )
             {
               if ( recID == 0 ) streamDefTimestep(streamIDs[eofID],tsID);
               //if ( recID == 0 ) fprintf(stderr, "ts%i rec%i eof%i\n", tsID, recID, eofID);
@@ -230,7 +196,8 @@ void *Eofcoeff(void * argument)
                   if (! DBL_IS_EQUAL(in.ptr[i],missval2) && 
                       ! DBL_IS_EQUAL(eof[varID][levelID][eofID].ptr[i],missval1 ))
                     {
-                      double tmp = w[i]*in.ptr[i]*eof[varID][levelID][eofID].ptr[i];
+		      // double tmp = w[i]*in.ptr[i]*eof[varID][levelID][eofID].ptr[i];
+                      double tmp = in.ptr[i]*eof[varID][levelID][eofID].ptr[i];
                       out.ptr[0] += tmp;                   
                     }
                 }            
@@ -246,6 +213,7 @@ void *Eofcoeff(void * argument)
           if ( levelID >= nlevs )
             cdoAbort("Internal error - too high levelID");                              
         }
+
       tsID++;
     }
   
@@ -256,6 +224,6 @@ void *Eofcoeff(void * argument)
   
   cdoFinish();
 
-  return (0);
+  return 0;
 }
 
