@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -41,16 +41,16 @@ Constansts: M_PI, M_E
 #include "grid.h"
 #include "expr.h"
 
+
 void grid_cell_area(int gridID, double *array);
 int getSurfaceID(int vlistID);
 
+
 static
 char *exprs_from_arg(const char *arg)
-{
-  char *exprs = NULL;
-  
+{  
   size_t slen = strlen(arg);
-  exprs = (char*) Malloc(slen+2);
+  char *exprs = (char*) Malloc(slen+2);
   strcpy(exprs, operatorArgv()[0]);
   if ( exprs[slen-1] != ';' )
     {
@@ -63,9 +63,7 @@ char *exprs_from_arg(const char *arg)
 
 static
 char *exprs_from_file(const char *exprf)
-{
-  char *exprs = NULL;
-  
+{  
   /* Open expr script file for reading */
   FILE *fp = fopen(exprf, "r");
   if( fp == NULL ) cdoAbort("Open failed on %s", exprf);
@@ -74,7 +72,7 @@ char *exprs_from_file(const char *exprf)
   if ( stat(exprf, &filestat) != 0 ) cdoAbort("Stat failed on %s", exprf);
 
   size_t fsize = (size_t) filestat.st_size;
-  exprs = (char*) Malloc(fsize+1);
+  char *exprs = (char*) Malloc(fsize+1);
 
   int ichar, ipos = 0;
   while ( (ichar = fgetc(fp)) != EOF ) exprs[ipos++] = ichar;
@@ -101,6 +99,7 @@ paramType *params_new(int vlistID)
   char name[CDI_MAX_NAME];
   char longname[CDI_MAX_NAME];
   char units[CDI_MAX_NAME];
+
   for ( int varID = 0; varID < nvars1; varID++ )
     {
       int gridID     = vlistInqVarGrid(vlistID, varID);
@@ -175,6 +174,7 @@ void params_add_coordinates(int vlistID, parse_param_t *parse_arg)
 {
   char longname[CDI_MAX_NAME];
   char units[CDI_MAX_NAME];
+
   int ngrids = vlistNgrids(vlistID);
   for ( int index = 0; index < ngrids; ++index )
     {
@@ -188,6 +188,7 @@ void params_add_coordinates(int vlistID, parse_param_t *parse_arg)
       params_add_coord(parse_arg, 'a', gridID, size, "m^2", "grid cell area");
       params_add_coord(parse_arg, 'w', gridID, size, NULL, "grid cell area weights");
     }
+
   int nzaxis = vlistNzaxis(vlistID);
   for ( int index = 0; index < nzaxis; ++index )
     {
@@ -239,6 +240,7 @@ void params_delete(paramType *params)
     }
 }
 
+
 void *Expr(void *argument)
 {
   cdoInitialize(argument);
@@ -301,7 +303,7 @@ void *Expr(void *argument)
     parse_arg.needed[varID] = ! REPLACES_VARIABLES(operatorID);
 
   int vartsID = params_add_ts(&parse_arg);
-  parse_arg.tsID       = vartsID;
+  parse_arg.tsID = vartsID;
   params_add_coordinates(vlistID1, &parse_arg);
                   
   CDO_parser_errorno = 0;
@@ -319,7 +321,7 @@ void *Expr(void *argument)
   if ( cdoVerbose )
     for ( int varID = 0; varID < parse_arg.nparams; varID++ )
       cdoPrint("var: %d %s ngp=%lu nlev=%lu coord=%c",
-               varID, params[varID].name, params[varID].ngp, params[varID].nlev, params[varID].coord);
+               varID, params[varID].name, params[varID].ngp, params[varID].nlev, params[varID].coord==0?' ':params[varID].coord);
 
   int *varIDmap = (int*) Malloc(parse_arg.nparams*sizeof(int));
 
@@ -431,7 +433,7 @@ void *Expr(void *argument)
               size_t nlev = parse_arg.coords[i].size;
               double *data = (double*) Malloc(nlev*sizeof(double));
               parse_arg.coords[i].data = data;
-              zaxisInqLevels(zaxisID, data);
+              cdoZaxisInqLevels(zaxisID, data);
             }
           else
             cdoAbort("Computation of coordinate %c not implemented!", coord);
