@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -28,53 +28,46 @@
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
+#include "grid.h"
 #include "specspace.h"
-#include "list.h"
+#include "listarray.h"
 
 
 void *FC(void *argument)
 {
-  int FC2SP, SP2FC, FC2GP, GP2FC;
-  int operatorID;
-  int streamID1, streamID2;
-  int nrecs, nvars;
-  int tsID, recID, varID, levelID;
-  int gridsize;
-  int index, ngrids;
-  int vlistID1, vlistID2;
+  int nrecs;
+  int varID, levelID;
+  int index;
   int gridIDsp = -1, gridIDgp = -1, gridIDfc = -1;
   int gridID1 = -1, gridID2 = -1;
   int gridID;
   int nmiss;
-  int *vars;
-  int lcopy = FALSE;
-  int taxisID1, taxisID2;
   int nlon = 0, nlat = 0, ntr = 0;
   int nsp = 0, nfc = 0;
-  double *array1 = NULL, *array2 = NULL;
+  double *array2 = NULL;
   SPTRANS *sptrans = NULL;
 
   cdoInitialize(argument);
 
-  FC2SP  = cdoOperatorAdd("fc2sp",  0, 0, NULL);
-  SP2FC  = cdoOperatorAdd("sp2fc", 0, 0, NULL);
-  FC2GP  = cdoOperatorAdd("fc2gp",  0, 0, NULL);
-  GP2FC  = cdoOperatorAdd("gp2fc", 0, 0, NULL);
+  bool lcopy = UNCHANGED_RECORD;
 
-  operatorID = cdoOperatorID();
+  int FC2SP  = cdoOperatorAdd("fc2sp",  0, 0, NULL);
+  int SP2FC  = cdoOperatorAdd("sp2fc", 0, 0, NULL);
+  int FC2GP  = cdoOperatorAdd("fc2gp",  0, 0, NULL);
+  int GP2FC  = cdoOperatorAdd("gp2fc", 0, 0, NULL);
 
-  if ( UNCHANGED_RECORD ) lcopy = TRUE;
+  int operatorID = cdoOperatorID();
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  ngrids = vlistNgrids(vlistID1);
+  int ngrids = vlistNgrids(vlistID1);
   /* find first spectral grid */
   for ( index = 0; index < ngrids; index++ )
     {
@@ -117,7 +110,7 @@ void *FC(void *argument)
 	{
 	  nfc  = gridInqSize(gridID1);
 	  ntr  = gridInqTrunc(gridID1);
-	  nlat = nfc2nlat(nfc, ntr);
+	  nlat = nfc_to_nlat(nfc, ntr);
 
 	  if ( gridIDsp != -1 )
 	    if ( ntr != gridInqTrunc(gridIDsp) ) gridIDsp = -1;
@@ -146,7 +139,7 @@ void *FC(void *argument)
       if ( gridID1 != -1 )
 	{
 	  ntr  = gridInqTrunc(gridID1);
-	  nlat = ntr2nlat(ntr);
+	  nlat = ntr_to_nlat(ntr);
 
 	  if ( gridIDfc != -1 )
 	    {
@@ -176,7 +169,7 @@ void *FC(void *argument)
 	{
 	  nlon = gridInqXsize(gridID1);
 	  nlat = gridInqYsize(gridID1);
-	  ntr  = nlat2ntr(nlat);
+	  ntr  = nlat_to_ntr(nlat);
 
 	  if ( gridIDfc != -1 )
 	    if ( ntr != gridInqTrunc(gridIDfc) ) gridIDfc = -1;
@@ -202,7 +195,7 @@ void *FC(void *argument)
 	{
 	  nfc  = gridInqSize(gridID1);
 	  ntr  = gridInqTrunc(gridID1);
-	  nlat = nfc2nlat(nfc, ntr);
+	  nlat = nfc_to_nlat(nfc, ntr);
 
 	  if ( gridIDgp != -1 )
 	    {
@@ -212,9 +205,9 @@ void *FC(void *argument)
 	  if ( gridIDgp == -1 )
 	    {
 	      char gridname[20];
-	      sprintf(gridname, "t%dgrid", ntr);
+	      snprintf(gridname, sizeof(gridname), "t%dgrid", ntr);
 
-	      gridIDgp = gridFromName(gridname);
+	      gridIDgp = grid_from_name(gridname);
 	    }
 
 	  gridID2 = gridIDgp;
@@ -227,8 +220,8 @@ void *FC(void *argument)
 
   // printf("nfc %d, ntr %d, nlat %d, nlon %d\n", nfc, ntr, nlat, nlon);
 
-  nvars = vlistNvars(vlistID2);
-  vars  = (int*) Malloc(nvars*sizeof(int));
+  int nvars = vlistNvars(vlistID2);
+  int *vars  = (int*) Malloc(nvars*sizeof(int));
   for ( varID = 0; varID < nvars; varID++ )
     {
       if ( gridID1 == vlistInqVarGrid(vlistID1, varID) )
@@ -239,12 +232,12 @@ void *FC(void *argument)
 
   if ( gridID1 != -1 ) vlistChangeGrid(vlistID2, gridID1, gridID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  gridsize = vlistGridsizeMax(vlistID1);
-  array1 = (double*) Malloc(gridsize*sizeof(double));
+  int gridsize = vlistGridsizeMax(vlistID1);
+  double *array1 = (double*) Malloc(gridsize*sizeof(double));
 
   if ( gridID2 != -1 )
     {
@@ -252,14 +245,14 @@ void *FC(void *argument)
       array2 = (double*) Malloc(gridsize*sizeof(double));
     }
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
