@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,52 +30,63 @@
 #include "util.h"
 
 
-static
-void printAtts(FILE *fp, int vlistID, int varID)
+void cdo_print_grid(int gridID, int opt);
+void cdo_print_zaxis(int zaxisID);
+
+void cdo_print_attributes(FILE *fp, int cdiID, int varID, int nblanks)
 {
 #define MAXATT 8192
-  int natts, ia;
-  char attname[1024];
+  int natts;
+  char attname[CDI_MAX_NAME];
   int atttype, attlen;
   char atttxt[MAXATT];
   int attint[MAXATT];
   double attflt[MAXATT];
-  int i;
+  char fltstr[128];
 
-  vlistInqNatts(vlistID, varID, &natts);
+  cdiInqNatts(cdiID, varID, &natts);
 
-  for ( ia = 0; ia < natts; ++ia )
+  for ( int ia = 0; ia < natts; ++ia )
     {
-      vlistInqAtt(vlistID, varID, ia, attname, &atttype, &attlen);
-      if ( atttype == DATATYPE_INT )
-	{
+      cdiInqAtt(cdiID, varID, ia, attname, &atttype, &attlen);
+
+      if ( atttype == CDI_DATATYPE_INT8  || atttype == CDI_DATATYPE_UINT8  ||
+           atttype == CDI_DATATYPE_INT16 || atttype == CDI_DATATYPE_UINT16 ||
+           atttype == CDI_DATATYPE_INT32 || atttype == CDI_DATATYPE_UINT32 )
+        {
 	  if ( attlen > MAXATT ) attlen = MAXATT;
-	  vlistInqAttInt(vlistID, varID, attname, attlen, attint);
-	  fprintf(fp, "  %s=", attname);
-	  for ( i = 0; i < attlen; ++i)
+	  cdiInqAttInt(cdiID, varID, attname, attlen, attint);
+          fprintf(fp, "%*s", nblanks, "");
+	  fprintf(fp, "%s = ", attname);
+	  for ( int i = 0; i < attlen; ++i )
 	    {
 	      if ( i > 0 ) fprintf(fp, ", ");
 	      fprintf(fp, "%d", attint[i]);
 	    }
 	  fprintf(fp, "\n");
 	}
-      else if ( atttype == DATATYPE_FLT )
+      else if ( atttype == CDI_DATATYPE_FLT32 || atttype == CDI_DATATYPE_FLT64 )
 	{
 	  if ( attlen > MAXATT ) attlen = MAXATT;
-	  vlistInqAttFlt(vlistID, varID, attname, MAXATT, attflt);
-	  fprintf(fp, "  %s=", attname);
-	  for ( i = 0; i < attlen; ++i)
+	  cdiInqAttFlt(cdiID, varID, attname, MAXATT, attflt);
+          fprintf(fp, "%*s", nblanks, "");
+	  fprintf(fp, "%s = ", attname);
+	  for ( int i = 0; i < attlen; ++i )
 	    {
 	      if ( i > 0 ) fprintf(fp, ", ");
-	      fprintf(fp, "%g", attflt[i]);
+              if ( atttype == CDI_DATATYPE_FLT32 )
+                fprintf(fp, "%sf", double_to_attstr(CDO_flt_digits, fltstr, sizeof(fltstr), attflt[i]));
+              else
+                fprintf(fp, "%s", double_to_attstr(CDO_dbl_digits, fltstr, sizeof(fltstr), attflt[i]));
 	    }
 	  fprintf(fp, "\n");
 	}
-      else if ( atttype == DATATYPE_TXT )
+      else if ( atttype == CDI_DATATYPE_TXT )
 	{
-	  vlistInqAttTxt(vlistID, varID, attname, sizeof(atttxt), atttxt);
+	  cdiInqAttTxt(cdiID, varID, attname, sizeof(atttxt), atttxt);
 	  atttxt[attlen] = 0;
-	  fprintf(fp, "  %s=\"%s\"\n", attname, atttxt);
+          fprintf(fp, "%*s", nblanks, "");
+	  fprintf(fp, "%s = \"%s\"\n", attname, atttxt);
 	}
     }
 }
@@ -90,7 +101,8 @@ void printHistory(FILE *fp, int streamID)
       char *history = (char*) Malloc(historysize+1);
       history[historysize] = 0;
       streamInqHistoryString(fileID, history);
-      fprintf(fp, "  history=%s\n", history);
+      fprintf(fp, "  history = %s\n", history);
+      Free(history);
     }
 }
 
@@ -99,11 +111,11 @@ void printSource(FILE *fp, int vlistID, int varID)
 {
   /* institute info */
   const char *instptr = institutInqLongnamePtr(vlistInqVarInstitut(vlistID, varID));
-  if ( instptr ) fprintf(fp, "  institution=%s\n", instptr);
+  if ( instptr ) fprintf(fp, "  institution = %s\n", instptr);
 
   /* source info */
   const char *modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
-  if ( modelptr ) fprintf(fp, "  source=%s\n", modelptr);
+  if ( modelptr ) fprintf(fp, "  source = %s\n", modelptr);
 }
 
 static
@@ -111,28 +123,24 @@ void partab(FILE *fp, int streamID, int option)
 {
   int vlistID = streamInqVlist(streamID);
   int varID, datatype = -1;
-  int param;
   char pstr[32];
   char paramstr[32];
   char varname[CDI_MAX_NAME], varlongname[CDI_MAX_NAME], varstdname[CDI_MAX_NAME], varunits[CDI_MAX_NAME];
-  int natts;
-  int chunktype;
-  int linebreak = 1;
-  double missval;
       
   int nvars = vlistNvars(vlistID);
-  if ( option == 4 ) linebreak = 0;
+  bool linebreak = ( option == 4 ) ? false : true;
 
   if ( option == 2 )
     {
-      vlistInqNatts(vlistID, CDI_GLOBAL, &natts);
+      int natts;
+      cdiInqNatts(vlistID, CDI_GLOBAL, &natts);
       if ( natts > 0 )
 	{
 	  fprintf(fp, "&parameter\n");
-	  fprintf(fp, "  name=_GLOBAL_\n");
+	  fprintf(fp, "  name = _GLOBAL_\n");
           printHistory(fp, streamID);
           printSource(fp, vlistID, 0);
-	  printAtts(fp, vlistID, CDI_GLOBAL);
+	  cdo_print_attributes(fp, vlistID, CDI_GLOBAL, 2);
 	  fprintf(fp, "/\n");
 	}
     }
@@ -153,11 +161,11 @@ void partab(FILE *fp, int streamID, int option)
 	{
 	  fprintf(fp, "&parameter");
 	  if ( linebreak ) fprintf(fp, "\n");
-	  fprintf(fp, "  name=_default_");
+	  fprintf(fp, "  name = _default_");
 	  if ( linebreak ) fprintf(fp, "\n");
 	  if ( datatype2str(datatype, pstr) == 0 )
 	    {
-	      fprintf(fp, "  datatype=%s", pstr);
+	      fprintf(fp, "  datatype = %s", pstr);
 	      if ( linebreak ) fprintf(fp, "\n");
 	    }
 	  fprintf(fp, "/\n");
@@ -172,17 +180,14 @@ void partab(FILE *fp, int streamID, int option)
       varname[0]     = 0;
       varlongname[0] = 0;
       varunits[0]    = 0;
-      param    = vlistInqVarParam(vlistID, varID);
-      missval  = vlistInqVarMissval(vlistID, varID);
+      int param = vlistInqVarParam(vlistID, varID);
+      double missval = vlistInqVarMissval(vlistID, varID);
       vlistInqVarName(vlistID, varID, varname);
-      /* printf("1>%s<\n", varname); */
       vlistInqVarStdname(vlistID, varID, varstdname);
-      /* printf("2>%s<\n", varname); */
       vlistInqVarLongname(vlistID, varID, varlongname);
-      /* printf("3>%s<\n", varname); */
       vlistInqVarUnits(vlistID, varID, varunits);
             
-      fprintf(fp, "  name=%s", varname);
+      fprintf(fp, "  name = %s", varname);
       if ( linebreak ) fprintf(fp, "\n");
       // if ( code   > 0 ) fprintf(fp, "  code=%d\n", code);
       // if ( tabnum > 0 ) fprintf(fp, "  table=%d\n", tabnum);
@@ -194,47 +199,47 @@ void partab(FILE *fp, int streamID, int option)
 	}
       if ( strlen(varstdname) )
 	{
-	  fprintf(fp, "  standard_name=%s", varstdname);
+	  fprintf(fp, "  standard_name = %s", varstdname);
 	  if ( linebreak ) fprintf(fp, "\n");
 	}
       if ( strlen(varlongname) )
 	{
-	  fprintf(fp, "  long_name=\"%s\"", varlongname);
+	  fprintf(fp, "  long_name = \"%s\"", varlongname);
 	  if ( linebreak ) fprintf(fp, "\n");
 	}
       if ( strlen(varunits) )
 	{
-	  fprintf(fp, "  units=\"%s\"", varunits);
+	  fprintf(fp, "  units = \"%s\"", varunits);
 	  if ( linebreak ) fprintf(fp, "\n");
 	}
 
       if ( datatype == -1 )
 	if ( datatype2str(vlistInqVarDatatype(vlistID, varID), pstr) == 0 )
 	  {
-	    fprintf(fp, "  datatype=%s", pstr);
+	    fprintf(fp, "  datatype = %s", pstr);
 	    if ( linebreak ) fprintf(fp, "\n");
 	  }
 
-      chunktype = vlistInqVarChunkType(vlistID, varID);
-      if ( chunktype == CHUNK_AUTO )
+      int chunktype = vlistInqVarChunkType(vlistID, varID);
+      if ( chunktype == CDI_CHUNK_AUTO )
 	{
-	  fprintf(fp, "  chunktype=auto");
+	  fprintf(fp, "  chunktype = auto");
 	  if ( linebreak ) fprintf(fp, "\n");
 	}
-      else if ( chunktype == CHUNK_GRID )
+      else if ( chunktype == CDI_CHUNK_GRID )
 	{
-	  fprintf(fp, "  chunktype=grid");
+	  fprintf(fp, "  chunktype = grid");
 	  if ( linebreak ) fprintf(fp, "\n");
 	}
-      if ( chunktype == CHUNK_LINES )
+      if ( chunktype == CDI_CHUNK_LINES )
 	{
-	  fprintf(fp, "  chunktype=lines");
+	  fprintf(fp, "  chunktype = lines");
 	  if ( linebreak ) fprintf(fp, "\n");
 	}
       
-      if ( option == 2 ) printAtts(fp, vlistID, varID);
+      if ( option == 2 ) cdo_print_attributes(fp, vlistID, varID, 2);
       if ( option == 2 ) 
-	fprintf(fp, "  missing_value=%g\n", missval);
+	fprintf(fp, "  missing_value = %g\n", missval);
       
       if ( !linebreak ) fprintf(fp, "  ");
       fprintf(fp, "/\n");
@@ -244,70 +249,36 @@ void partab(FILE *fp, int streamID, int option)
 static
 void filedes(int streamID)
 {
-  int filetype;
-
   printf("\n");
-  filetype = streamInqFiletype(streamID);
+  int filetype = streamInqFiletype(streamID);
   switch ( filetype )
     {
-    case FILETYPE_GRB:
-      printf("  GRIB data\n");
-      break;
-    case FILETYPE_GRB2:
-      printf("  GRIB2 data\n");
-      break;
-    case FILETYPE_NC:
-      printf("  NetCDF data\n");
-      break;
-    case FILETYPE_NC2:
-      printf("  NetCDF2 data\n");
-      break;
-    case FILETYPE_NC4:
-      printf("  NetCDF4 data\n");
-      break;
-    case FILETYPE_NC4C:
-      printf("  NetCDF4 classic data\n");
-      break;
-    case FILETYPE_SRV:
-      printf("  SERVICE data\n");
-      switch ( streamInqByteorder(streamID) )
-	{
-	case CDI_BIGENDIAN:
-	  printf("  byteorder is BIGENDIAN\n"); break;
-	case CDI_LITTLEENDIAN:
-	  printf("  byteorder is LITTLEENDIAN\n"); break;
-	default:
-	  printf("  byteorder %d undefined\n", streamInqByteorder(streamID)); break;
-	}
-      break;
-    case FILETYPE_EXT:
-      printf("  EXTRA data\n");
-      switch ( streamInqByteorder(streamID) )
-	{
-	case CDI_BIGENDIAN:
-	  printf("  byteorder is BIGENDIAN\n"); break;
-	case CDI_LITTLEENDIAN:
-	  printf("  byteorder is LITTLEENDIAN\n"); break;
-	default:
-	  printf("  byteorder %d undefined\n", streamInqByteorder(streamID)); break;
-	}
-      break;
-    case FILETYPE_IEG:
-      printf("  IEG data\n");
-      switch ( streamInqByteorder(streamID) )
-	{
-	case CDI_BIGENDIAN:
-	  printf("  byteorder is BIGENDIAN\n"); break;
-	case CDI_LITTLEENDIAN:
-	  printf("  byteorder is LITTLEENDIAN\n"); break;
-	default:
-	  printf("  byteorder %d undefined\n", streamInqByteorder(streamID)); break;
-	}
-      break;
-    default:
-      printf("  unsupported filetype %d\n" , filetype);
-      break;
+    case CDI_FILETYPE_GRB:  printf("  GRIB data\n"); break;
+    case CDI_FILETYPE_GRB2: printf("  GRIB2 data\n"); break;
+    case CDI_FILETYPE_NC:   printf("  NetCDF data\n"); break;
+    case CDI_FILETYPE_NC2:  printf("  NetCDF2 data\n"); break;
+    case CDI_FILETYPE_NC4:  printf("  NetCDF4 data\n"); break;
+    case CDI_FILETYPE_NC4C: printf("  NetCDF4 classic data\n"); break;
+    case CDI_FILETYPE_SRV:  printf("  SERVICE data\n"); break;
+    case CDI_FILETYPE_EXT:  printf("  EXTRA data\n"); break;
+    case CDI_FILETYPE_IEG:  printf("  IEG data\n"); break;
+    default: printf("  unsupported filetype %d\n" , filetype);
     }
+
+  switch ( filetype )
+    {
+    case CDI_FILETYPE_SRV:
+    case CDI_FILETYPE_EXT:
+    case CDI_FILETYPE_IEG:
+      {
+        switch ( streamInqByteorder(streamID) )
+          {
+          case CDI_BIGENDIAN:    printf("  byteorder is BIGENDIAN\n"); break;
+          case CDI_LITTLEENDIAN: printf("  byteorder is LITTLEENDIAN\n"); break;
+          default:  printf("  byteorder %d undefined\n", streamInqByteorder(streamID)); break;
+          }
+       }
+    }  
   
   printf("\n");
 }
@@ -315,13 +286,6 @@ void filedes(int streamID)
 
 void *Filedes(void *argument)
 {
-  int operatorID;
-  int streamID = 0;
-  int zaxisID;
-  int nvars, ngrids, nzaxis;
-  int type, index;
-  int vlistID;
-
   cdoInitialize(argument);
 
   int GRIDDES  = cdoOperatorAdd("griddes",   0, 0, NULL);
@@ -336,57 +300,58 @@ void *Filedes(void *argument)
   int PARTAB   = cdoOperatorAdd("partab",    0, 0, NULL);
   int PARTAB2  = cdoOperatorAdd("partab2",   0, 0, NULL);
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
-  streamID = streamOpenRead(cdoStreamName(0));
+  int streamID = streamOpenRead(cdoStreamName(0));
 
-  vlistID = streamInqVlist(streamID);
+  int vlistID = streamInqVlist(streamID);
 
-  nvars  = vlistNvars(vlistID);
-  ngrids = vlistNgrids(vlistID);
-  nzaxis = vlistNzaxis(vlistID);
+  int nvars  = vlistNvars(vlistID);
+  int ngrids = vlistNgrids(vlistID);
+  int nzaxis = vlistNzaxis(vlistID);
 
   if ( operatorID == GRIDDES || operatorID == GRIDDES2 )
     {
-      int opt = 0;
-      if ( operatorID == GRIDDES ) opt = 1;
-      for ( index = 0; index < ngrids; index++ )
-	gridPrint(vlistGrid(vlistID, index), index+1, opt);
+      int opt = (operatorID == GRIDDES) ? 1 : 0;
+      for ( int index = 0; index < ngrids; index++ )
+        {
+          printf("#\n" "# gridID %d\n" "#\n", index+1);
+          cdo_print_grid(vlistGrid(vlistID, index), opt);
+        }
     }
   else if ( operatorID == ZAXISDES )
     {
-      for ( index = 0; index < nzaxis; index++ )
-	zaxisPrint(vlistZaxis(vlistID, index), index+1);
+      for ( int index = 0; index < nzaxis; index++ )
+        {
+          printf("#\n" "# zaxisID %d\n" "#\n", index+1);
+          cdo_print_zaxis(vlistZaxis(vlistID, index));
+        }
     }
   else if ( operatorID == VCT || operatorID == VCT2 )
     {
-      for ( index = 0; index < nzaxis; index++)
+      for ( int index = 0; index < nzaxis; index++ )
 	{
-	  zaxisID = vlistZaxis(vlistID, index);
-	  type = zaxisInqType(zaxisID);
+	  int zaxisID = vlistZaxis(vlistID, index);
+	  int type = zaxisInqType(zaxisID);
 	  if ( type == ZAXIS_HYBRID || type == ZAXIS_HYBRID_HALF )
 	    {
-	      int i, vctsize;
-	      const double *vct;
-
-	      vctsize = zaxisInqVctSize(zaxisID);
-	      vct     = zaxisInqVctPtr(zaxisID);
+	      int vctsize = zaxisInqVctSize(zaxisID);
+	      const double *vct = zaxisInqVctPtr(zaxisID);
 		
 	      if ( vctsize%2 == 0 )
 		{
 		  if ( operatorID == VCT )
 		    {
 		      fprintf(stdout, "#   k         vct_a(k) [Pa]             vct_b(k) []\n");
-		      for ( i = 0; i < vctsize/2; i++ )
+		      for ( int i = 0; i < vctsize/2; i++ )
 			fprintf(stdout, "%5d %25.17f %25.17f\n", i, vct[i], vct[vctsize/2+i]);
 		    }
 		  else
 		    {
-		      int nbyte0, nbyte;
 		      fprintf(stdout, "vctsize   = %d\n", vctsize);
-		      nbyte0 = fprintf(stdout, "vct       = ");
-		      nbyte = nbyte0;
-		      for ( i = 0; i < vctsize; i++ )
+		      int nbyte0 = fprintf(stdout, "vct       = ");
+		      int nbyte = nbyte0;
+		      for ( int i = 0; i < vctsize; i++ )
 			{
 			  if ( nbyte > 70 || i == vctsize/2 )
 			    {
@@ -399,7 +364,7 @@ void *Filedes(void *argument)
 		    }
 		}
 	      else
-		for ( i = 0; i < vctsize; i++ )
+		for ( int i = 0; i < vctsize; i++ )
 		  fprintf(stdout, "%5d %25.17f\n", i, vct[i]);
 
 	      break;
@@ -412,23 +377,22 @@ void *Filedes(void *argument)
     }
   else if ( operatorID == CODETAB )
     {
-      int varID, code;
       char varname[CDI_MAX_NAME], varlongname[CDI_MAX_NAME], varunits[CDI_MAX_NAME];
 
-      for ( varID = 0; varID < nvars; varID++ )
+      for ( int varID = 0; varID < nvars; varID++ )
 	{
 	  varname[0]     = 0;
 	  varlongname[0] = 0;
 	  varunits[0]    = 0;
-	  code     = vlistInqVarCode(vlistID, varID);
+	  int code = vlistInqVarCode(vlistID, varID);
 	  vlistInqVarName(vlistID, varID, varname);
 	  vlistInqVarLongname(vlistID, varID, varlongname);
 	  vlistInqVarUnits(vlistID, varID, varunits);
 	  fprintf(stdout, "%4d  %-12s", code, varname);
-	  if ( strlen(varlongname) )
+	  if ( varlongname[0] )
 	    {
 	      fprintf(stdout, "  %s", varlongname);
-	      if ( strlen(varunits) )
+	      if ( varunits[0] )
 		fprintf(stdout, " [%s]", varunits);
 	    }
 	  fprintf(stdout, "\n");
