@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,7 +32,7 @@
 #include "clipping/geometry.h"
 
 
-void fillmiss(field_t *field1, field_t *field2, int nfill)
+void fillmiss(field_type *field1, field_type *field2, int nfill)
 {
   int nx, ny, i, j;
   int nmiss2 = 0;
@@ -147,7 +147,7 @@ void fillmiss(field_t *field1, field_t *field2, int nfill)
 }
 
 
-void fillmiss_one_step(field_t *field1, field_t *field2, int maxfill)
+void fillmiss_one_step(field_type *field1, field_type *field2, int maxfill)
 {
   int gridID, nx, ny, i, j;
   int nmiss2 = 0;
@@ -271,14 +271,15 @@ void fillmiss_one_step(field_t *field1, field_t *field2, int maxfill)
 }
 
 
-void grid_search_nbr(struct gridsearch *gs, int num_neighbors, int *restrict nbr_add, double *restrict nbr_dist, double plon, double plat);
+int grid_search_nbr(struct gridsearch *gs, int num_neighbors, int *restrict nbr_add, double *restrict nbr_dist, double plon, double plat);
 double nbr_compute_weights(unsigned num_neighbors, const int *restrict src_grid_mask, int *restrict nbr_mask, const int *restrict nbr_add, double *restrict nbr_dist);
 unsigned nbr_normalize_weights(unsigned num_neighbors, double dist_tot, const int *restrict nbr_mask, int *restrict nbr_add, double *restrict nbr_dist);
 
 static
-void setmisstodis(field_t *field1, field_t *field2, int num_neighbors)
+void setmisstodis(field_type *field1, field_type *field2, int num_neighbors)
 {
   int gridID = field1->grid;
+  int gridID0 = gridID;
   double missval = field1->missval;
   double *array1 = field1->ptr;
   double *array2 = field2->ptr;
@@ -400,6 +401,8 @@ void setmisstodis(field_t *field1, field_t *field2, int num_neighbors)
 
   if ( gs ) gridsearch_delete(gs);
 
+  if ( gridID0 != gridID ) gridDestroy(gridID);
+
   if ( lons ) Free(lons);
   if ( lats ) Free(lats);
   Free(xvals);
@@ -410,8 +413,8 @@ void setmisstodis(field_t *field1, field_t *field2, int num_neighbors)
 void *Fillmiss(void *argument)
 {
   int nmiss;
-  int nrecs, recID, varID, levelID;
-  void (*fill_method) (field_t *fin , field_t *fout , int) = NULL;
+  int nrecs, varID, levelID;
+  void (*fill_method) (field_type *fin , field_type *fout , int) = NULL;
 
   cdoInitialize(argument);
 
@@ -473,7 +476,7 @@ void *Fillmiss(void *argument)
 
   int gridsize = vlistGridsizeMax(vlistID1);
 
-  field_t field1, field2;
+  field_type field1, field2;
   field_init(&field1);
   field_init(&field2);
   field1.ptr = (double*) Malloc(gridsize*sizeof(double));
@@ -486,7 +489,7 @@ void *Fillmiss(void *argument)
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamReadRecord(streamID1, field1.ptr, &nmiss);
