@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -35,14 +35,11 @@ static
 double correlation_s(const double * restrict in0, const double * restrict in1,
 		     const double * restrict weight, double missval1, double missval2, long gridsize)
 {
-  long i;
   double sum0, sum1, sum00, sum01, sum11, wsum0;
-  double out;
-
   sum0 = sum1 = sum00 = sum01 = sum11 = 0;
   wsum0 = 0;
 	
-  for ( i = 0; i < gridsize; ++i )
+  for ( long i = 0; i < gridsize; ++i )
     {
       if ( IS_NOT_EQUAL(weight[i], missval1) && IS_NOT_EQUAL(in0[i], missval1) && IS_NOT_EQUAL(in1[i], missval2) )
 	    {
@@ -55,12 +52,12 @@ double correlation_s(const double * restrict in0, const double * restrict in1,
 	    }
     }
 
-  out = IS_NOT_EQUAL(wsum0, 0) ?
-        DIV((sum01 * wsum0 - sum0 * sum1),
-	     SQRT((sum00 * wsum0 - sum0 * sum0) *
-	          (sum11 * wsum0 - sum1 * sum1))) : missval1;
+  double out = IS_NOT_EQUAL(wsum0, 0) ?
+               DIVMN((sum01 * wsum0 - sum0 * sum1),
+	            SQRTMN((sum00 * wsum0 - sum0 * sum0) *
+	                 (sum11 * wsum0 - sum1 * sum1))) : missval1;
 
-  return (out);
+  return out;
 }
 
 /* covariance in space */
@@ -68,14 +65,11 @@ static
 double covariance_s(const double * restrict in0, const double * restrict in1,
 		    const double * restrict weight, double missval1, double missval2, long gridsize)
 {
-  long i;
   double sum0, sum1, sum01, wsum0, wsum00;
-  double out;
-
   sum0 = sum1 = sum01 = 0;
   wsum0 = wsum00 = 0;
 
-  for ( i = 0; i < gridsize; ++i )
+  for ( long i = 0; i < gridsize; ++i )
     {
       if ( IS_NOT_EQUAL(weight[i], missval1) && IS_NOT_EQUAL(in0[i], missval1) && IS_NOT_EQUAL(in1[i], missval2) )
 	{
@@ -87,82 +81,83 @@ double covariance_s(const double * restrict in0, const double * restrict in1,
 	}
     }
 
-  out = IS_NOT_EQUAL(wsum0, 0) ?
-        (sum01 * wsum0 - sum0 * sum1) / (wsum0 * wsum0) : missval1;
+  double out = IS_NOT_EQUAL(wsum0, 0) ?
+               (sum01 * wsum0 - sum0 * sum1) / (wsum0 * wsum0) : missval1;
 
-  return (out);
+  return out;
 }
 
 
 void *Fldstat2(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int streamID1, streamID2, streamID3;
-  int vlistID1, vlistID2, vlistID3;
   int gridID, lastgridID = -1;
   int gridID3;
-  int wstatus = FALSE;
-  int code = 0, oldcode = 0;
-  int index, ngrids;
-  int recID, nrecs, nrecs2;
-  int tsID, varID, levelID;
-  long gridsize;
-  int needWeights = TRUE;
+  int index;
+  int nrecs, nrecs2;
+  int varID, levelID;
   int nmiss1, nmiss2, nmiss3;
+  bool wstatus = false;
+  bool needWeights = true;
   double missval1, missval2;
-  double slon, slat;
-  double sglval;
-  double *array1, *array2, *weight;
-  int taxisID1, taxisID3;
+  double sglval = 0;
+  char varname[CDI_MAX_NAME];
 
   cdoInitialize(argument);
 
   cdoOperatorAdd("fldcor",   func_cor,   0, NULL);
   cdoOperatorAdd("fldcovar", func_covar, 0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistID3 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistID3 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID3 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  slon = 0;
-  slat = 0;
-  gridID3 = gridCreate(GRID_LONLAT, 1);
-  gridDefXsize(gridID3, 1);
-  gridDefYsize(gridID3, 1);
-  gridDefXvals(gridID3, &slon);
-  gridDefYvals(gridID3, &slat);
+  if ( CDO_Reduce_Dim )
+    {
+      gridID3 = gridCreate(GRID_GENERIC, 1);
+      gridDefXsize(gridID3, 0);
+      gridDefYsize(gridID3, 0);
+    }
+  else
+    {
+      double slon = 0;
+      double slat = 0;
+      gridID3 = gridCreate(GRID_LONLAT, 1);
+      gridDefXsize(gridID3, 1);
+      gridDefYsize(gridID3, 1);
+      gridDefXvals(gridID3, &slon);
+      gridDefYvals(gridID3, &slat);
+    }
 
-  ngrids = vlistNgrids(vlistID1);
+  int ngrids = vlistNgrids(vlistID1);
 
   for ( index = 0; index < ngrids; index++ )
     vlistChangeGridIndex(vlistID3, index, gridID3);
 
-  streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
+  int streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
 
   streamDefVlist(streamID3, vlistID3);
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
 
-  array1 = malloc(gridsize*sizeof(double));
-  array2 = malloc(gridsize*sizeof(double));
-  weight = NULL;
+  double *array1 = (double*) Malloc(gridsize*sizeof(double));
+  double *array2 = (double*) Malloc(gridsize*sizeof(double));
+  double *weight = NULL;
   if ( needWeights )
-    weight = malloc(gridsize*sizeof(double));
+    weight = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       nrecs2 = streamInqTimestep(streamID2, tsID);
@@ -177,7 +172,7 @@ void *Fldstat2(void *argument)
 
       streamDefTimestep(streamID3, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamInqRecord(streamID2, &varID, &levelID);
@@ -189,11 +184,13 @@ void *Fldstat2(void *argument)
 	  if ( needWeights && gridID != lastgridID )
 	    {
 	      lastgridID = gridID;
-	      wstatus = gridWeights(gridID, weight);
+	      wstatus = gridWeights(gridID, weight) != 0;
+	    }
+	  if ( wstatus && tsID == 0 && levelID == 0 )
+	    {
+	      vlistInqVarName(vlistID1, varID, varname);
+	      cdoWarning("Using constant grid cell area weights for variable %s!", varname);
 	    }
-	  code = vlistInqVarCode(vlistID1, varID);
-	  if ( wstatus != 0 && tsID == 0 && code != oldcode )
-	    cdoWarning("Using constant grid cell area weights for code %d!", oldcode=code);
 
 	  missval1 = vlistInqVarMissval(vlistID1, varID);
 	  missval2 = vlistInqVarMissval(vlistID2, varID);
@@ -222,11 +219,11 @@ void *Fldstat2(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array1 ) free(array1);
-  if ( array2 ) free(array2);
-  if ( weight ) free(weight);
+  if ( array1 ) Free(array1);
+  if ( array2 ) Free(array2);
+  if ( weight ) Free(weight);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
