@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,8 +29,8 @@
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
-#include "pstream.h"
 #include "grid.h"
+#include "pstream.h"
 
 
 /*
@@ -405,7 +405,7 @@ void dumpmap()
 }
 
 static
-void ctl_xydef(FILE *gdp, int gridID, int *yrev)
+void ctl_xydef(FILE *gdp, int gridID, bool *yrev)
 {
   int gridtype;
   int i, j;
@@ -413,7 +413,7 @@ void ctl_xydef(FILE *gdp, int gridID, int *yrev)
   double xfirst, yfirst, xinc, yinc;
   double *xvals, *yvals;
 
-  *yrev = FALSE;
+  *yrev = false;
 
   xsize  = gridInqXsize(gridID);
   ysize  = gridInqYsize(gridID);
@@ -431,7 +431,7 @@ void ctl_xydef(FILE *gdp, int gridID, int *yrev)
       int nx, ny, ni;
       double inc[] = { 1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001 };
 
-      gridInqLCC(gridID, &originLon, &originLat, &lonParY, &lat1, &lat2, &xincm, &yincm,
+      gridInqParamLCC(gridID, &originLon, &originLat, &lonParY, &lat1, &lat2, &xincm, &yincm,
                  &projflag, &scanflag);
       fprintf(gdp, "PDEF %d %d LCCR %g %g 1 1 %g %g %g %g %g\n",
               xsize, ysize, originLat, originLon, lat1, lat2, lonParY, xincm, yincm);
@@ -521,7 +521,7 @@ void ctl_xydef(FILE *gdp, int gridID, int *yrev)
           j = 0;
           if ( yvals[0] > yvals[ysize-1] )
             {
-              *yrev = TRUE;
+              *yrev = true;
               for ( i = ysize-1; i >= 0; i-- )
                 {
                   fprintf(gdp, "%7.3f ", yvals[i]);
@@ -558,7 +558,7 @@ void ctl_xydef(FILE *gdp, int gridID, int *yrev)
           if ( IS_EQUAL(yinc, 0) ) yinc = 180.0/ysize;
           if ( yinc < 0)
             {
-              *yrev = TRUE;
+              *yrev = true;
               fprintf(gdp, "YDEF %d LINEAR %f %f\n", ysize, yfirst + yinc * (ysize-1 ), -yinc);
             }
           else
@@ -568,18 +568,17 @@ void ctl_xydef(FILE *gdp, int gridID, int *yrev)
 }
 
 static
-void ctl_zdef(FILE *gdp, int vlistID, int *zrev)
+void ctl_zdef(FILE *gdp, int vlistID, bool *zrev)
 {
   int i, j, index;
-  int zaxisIDmax = -1, nlevmax;
-  int nzaxis, zaxisID, nlev;
-  int lplev = FALSE;
-  double *levels, level0, levinc = 0;
+  int zaxisIDmax = -1;
+  int zaxisID, nlev;
+  double levinc = 0;
 
-  *zrev = FALSE;
-  nzaxis  = vlistNzaxis(vlistID);
+  *zrev = false;
+  int nzaxis  = vlistNzaxis(vlistID);
 
-  nlevmax = 0;
+  int nlevmax = 0;
   for ( index = 0; index < nzaxis; index++ )
     {
       zaxisID = vlistZaxis(vlistID, index);
@@ -591,18 +590,18 @@ void ctl_zdef(FILE *gdp, int vlistID, int *zrev)
         }
     }
 
-  levels = (double*) Malloc(nlevmax*sizeof(double));
-  zaxisInqLevels(zaxisIDmax, levels);
-  if ( zaxisInqType(zaxisIDmax) == ZAXIS_PRESSURE ) lplev = TRUE;
-  level0 = levels[0];
+  double *levels = (double*) Malloc(nlevmax*sizeof(double));
+  cdoZaxisInqLevels(zaxisIDmax, levels);
+  bool lplev = (zaxisInqType(zaxisIDmax) == ZAXIS_PRESSURE);
+  double level0 = levels[0];
   if ( nlevmax > 1 )
     {
       if ( levels[0] < levels[1] && zaxisInqType(zaxisIDmax) != ZAXIS_HYBRID )
-        *zrev = TRUE;
+        *zrev = true;
 
       levinc = levels[1] - levels[0];
 
-      if ( IS_EQUAL(levinc, 1) ) *zrev = FALSE;
+      if ( IS_EQUAL(levinc, 1) ) *zrev = false;
 
       for ( i = 1; i < nlevmax; i++ )
         {
@@ -659,9 +658,9 @@ void ctl_zdef(FILE *gdp, int vlistID, int *zrev)
 }
 
 static
-void ctl_options(FILE *gdp, int yrev, int zrev, int sequential, int bigendian, int littleendian, int flt64, int cal365day)
+void ctl_options(FILE *gdp, bool yrev, bool zrev, bool sequential, bool bigendian, bool littleendian, bool flt64, bool cal365day)
 {
-  /* if ( filetype == FILETYPE_GRB ) zrev = FALSE; */
+  /* if ( filetype == CDI_FILETYPE_GRB ) zrev = false; */
 
   if ( yrev || zrev || sequential || bigendian || littleendian || flt64 )
     {
@@ -722,7 +721,7 @@ void ctl_vars(FILE *gdp, int filetype, int vlistID, int nvarsout, int *vars)
 
           fprintf(gdp, "  %3d", nlev);
 
-          if ( filetype == FILETYPE_GRB )
+          if ( filetype == CDI_FILETYPE_GRB )
             {
               code = vlistInqVarCode(vlistID, varID);
               /*
@@ -733,7 +732,7 @@ void ctl_vars(FILE *gdp, int filetype, int vlistID, int nvarsout, int *vars)
               */
               fprintf(gdp, "  %d,%d", code, ltype);
             }
-          else if ( filetype == FILETYPE_NC )
+          else if ( filetype == CDI_FILETYPE_NC )
             {
               int xyz = vlistInqVarXYZ(vlistID, varID);
 
@@ -945,19 +944,18 @@ void *Gradsdes(void *argument)
   int gridtype = -1;
   int index;
   int varID;
-  int recID, levelID;
+  int levelID;
   int nrecs;
   int vdate, vtime;
   char *idxfile = NULL;
   char varname[CDI_MAX_NAME];
-  int yrev = FALSE;
-  int zrev = FALSE;
+  bool yrev = false;
+  bool zrev = false;
   int xyheader = 0;
   int nrecords = 0;
-  int bigendian = FALSE, littleendian = FALSE;
-  int flt64 = 0;
-  int cal365day = 0;
-  int sequential = FALSE;
+  bool bigendian = false, littleendian = false;
+  bool sequential = false;
+  bool flt64 = false;
   char Time[30], Incr[10] = {"1mn"};
   const char *IncrKey[] = {"mn","hr","dy","mo","yr"};
   int isd, imn, ihh, iyy, imm, idd;
@@ -1032,17 +1030,17 @@ void *Gradsdes(void *argument)
   int filetype  = streamInqFiletype(streamID);
   int byteorder = streamInqByteorder(streamID);
 
-  if ( filetype == FILETYPE_NC2 || filetype == FILETYPE_NC4 ) filetype = FILETYPE_NC;
+  if ( filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 ) filetype = CDI_FILETYPE_NC;
 
-  if ( filetype != FILETYPE_SRV &&
-       filetype != FILETYPE_EXT &&
-       filetype != FILETYPE_IEG &&
-       filetype != FILETYPE_GRB )
+  if ( filetype != CDI_FILETYPE_SRV &&
+       filetype != CDI_FILETYPE_EXT &&
+       filetype != CDI_FILETYPE_IEG &&
+       filetype != CDI_FILETYPE_GRB )
     {
-      if ( filetype == FILETYPE_NC )
+      if ( filetype == CDI_FILETYPE_NC )
         //        cdoAbort("Unsupported file format: NetCDF");
         ;
-      else if ( filetype == FILETYPE_GRB2 )
+      else if ( filetype == CDI_FILETYPE_GRB2 )
         //cdoAbort("Unsupported file format: GRIB2");
         ;
       else
@@ -1071,12 +1069,12 @@ void *Gradsdes(void *argument)
     {
       if ( vlistInqVarGrid(vlistID, varID) == gridID )
         {
-          if ( filetype == FILETYPE_SRV ||
-               filetype == FILETYPE_EXT ||
-               filetype == FILETYPE_IEG )
+          if ( filetype == CDI_FILETYPE_SRV ||
+               filetype == CDI_FILETYPE_EXT ||
+               filetype == CDI_FILETYPE_IEG )
             {
               prec = vlistInqVarDatatype(vlistID, varID);
-              if ( prec == DATATYPE_FLT64 ) flt64 = 1;
+              if ( prec == CDI_DATATYPE_FLT64 ) flt64 = true;
             }
           vars[varID] = TRUE;
           recoffset[varID] = nrecsout;
@@ -1095,34 +1093,34 @@ void *Gradsdes(void *argument)
         }
     }
 
-  if ( filetype != FILETYPE_GRB && nvars != nvarsout )
+  if ( filetype != CDI_FILETYPE_GRB && nvars != nvarsout )
     cdoAbort("Too many different grids!");
 
-  if ( filetype == FILETYPE_SRV )
+  if ( filetype == CDI_FILETYPE_SRV )
     {
       xyheader = 40;
       if ( flt64 ) xyheader = 72;
-      sequential = TRUE;
-      if ( byteorder == CDI_BIGENDIAN )    bigendian = TRUE;
-      if ( byteorder == CDI_LITTLEENDIAN ) littleendian = TRUE;
+      sequential = true;
+      if ( byteorder == CDI_BIGENDIAN )    bigendian = true;
+      if ( byteorder == CDI_LITTLEENDIAN ) littleendian = true;
     }
 
-  if ( filetype == FILETYPE_EXT )
+  if ( filetype == CDI_FILETYPE_EXT )
     {
       xyheader = 24;
       if ( flt64 ) xyheader = 40;
-      sequential = TRUE;
-      if ( byteorder == CDI_BIGENDIAN )    bigendian = TRUE;
-      if ( byteorder == CDI_LITTLEENDIAN ) littleendian = TRUE;
+      sequential = true;
+      if ( byteorder == CDI_BIGENDIAN )    bigendian = true;
+      if ( byteorder == CDI_LITTLEENDIAN ) littleendian = true;
     }
 
-  if ( filetype == FILETYPE_IEG )
+  if ( filetype == CDI_FILETYPE_IEG )
     {
       xyheader = 644;
       if ( flt64 ) xyheader = 1048;
-      sequential = TRUE;
-      if ( byteorder == CDI_BIGENDIAN )    bigendian = TRUE;
-      if ( byteorder == CDI_LITTLEENDIAN ) littleendian = TRUE;
+      sequential = true;
+      if ( byteorder == CDI_BIGENDIAN )    bigendian = true;
+      if ( byteorder == CDI_LITTLEENDIAN ) littleendian = true;
     }
 
   /* ctl file name */
@@ -1156,19 +1154,19 @@ void *Gradsdes(void *argument)
    * DTYPE Print file type
    * INDEX Print filename of the control/index file .ctl/.idx
    */
-  if ( filetype == FILETYPE_GRB ||  filetype == FILETYPE_GRB2 )
+  if ( filetype == CDI_FILETYPE_GRB ||  filetype == CDI_FILETYPE_GRB2 )
     {
       idxfile = strdup(ctlfile);
       char *pidxfile = idxfile;
 
       // print GRIB[12] file type
       // generate the index file
-      if ( filetype == FILETYPE_GRB )
+      if ( filetype == CDI_FILETYPE_GRB )
         {
           fprintf(gdp, "DTYPE  GRIB\n");
           repl_filetypeext(idxfile, ".ctl", ".gmp");
         }
-      else if ( filetype == FILETYPE_GRB2 )
+      else if ( filetype == CDI_FILETYPE_GRB2 )
         {
           fprintf(gdp, "DTYPE  GRIB2\n");
           repl_filetypeext(pidxfile, ".ctl", ".idx");
@@ -1188,7 +1186,7 @@ void *Gradsdes(void *argument)
       gridsize = vlistGridsizeMax(vlistID);
       array = (double*) Malloc(gridsize*sizeof(double));
     }
-  else if ( filetype == FILETYPE_NC )
+  else if ( filetype == CDI_FILETYPE_NC )
     {
       fprintf(gdp, "DTYPE  NetCDF\n");
     }
@@ -1200,7 +1198,7 @@ void *Gradsdes(void *argument)
 
   int taxisID = vlistInqTaxis(vlistID);
 
-  if ( taxisInqCalendar(taxisID) == CALENDAR_365DAYS ) cal365day = 1;
+  bool cal365day = (taxisInqCalendar(taxisID) == CALENDAR_365DAYS);
 
   int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID, tsID)) )
@@ -1296,7 +1294,7 @@ void *Gradsdes(void *argument)
           iyys = iyy;
         }
 
-      if ( filetype == FILETYPE_GRB )
+      if ( filetype == CDI_FILETYPE_GRB )
         {
           nrecords += nrecsout;
           if ( nrecords >= maxrecs )
@@ -1307,7 +1305,7 @@ void *Gradsdes(void *argument)
               bignum = (off_t*) Realloc(bignum, 2*maxrecs*sizeof(off_t));
             }
 
-          for ( recID = 0; recID < nrecs; recID++ )
+          for ( int recID = 0; recID < nrecs; recID++ )
             {
               streamInqRecord(streamID, &varID, &levelID);
               if ( vars[varID] == TRUE )
@@ -1358,18 +1356,18 @@ void *Gradsdes(void *argument)
         }
     }
 
-  sprintf(Time, "%02d:%02dZ%02d%s%04d", ihh0, imn0, idd0, cmons[imm0-1], iyy0);
-  sprintf(Incr, "%d%s", dt, IncrKey[iik]);
+  snprintf(Time, sizeof(Time), "%02d:%02dZ%02d%s%04d", ihh0, imn0, idd0, cmons[imm0-1], iyy0);
+  snprintf(Incr, sizeof(Incr), "%d%s", dt, IncrKey[iik]);
 
   fprintf(gdp, "TDEF %d LINEAR %s %s\n", tsID, Time, Incr);
 
   /* TITLE */
 
-  int xsize  = gridInqXsize(gridID);
-  int ysize  = gridInqYsize(gridID);
+  int xsize = gridInqXsize(gridID);
+  int ysize = gridInqYsize(gridID);
 
   int res = 0;
-  if ( gridtype == GRID_GAUSSIAN ) res = nlat2ntr(ysize);
+  if ( gridtype == GRID_GAUSSIAN ) res = nlat_to_ntr(ysize);
 
   if ( res )
     fprintf(gdp, "TITLE  %s  T%d grid\n", datfile, res);
@@ -1387,13 +1385,14 @@ void *Gradsdes(void *argument)
 
 
   /* INDEX file */
-  if ( filetype == FILETYPE_GRB )
+  if ( filetype == CDI_FILETYPE_GRB )
     {
       write_map_grib1(idxfile, map_version, nrecords, intnum, fltnum, bignum);
     }
-  if ( filetype == FILETYPE_GRB2 )
+  if ( filetype == CDI_FILETYPE_GRB2 )
     {
-      cdoAbort("\nThe fileformat GRIB2 is not fully supported yet\nfor the gradsdes operator.\nThe .ctl file %s was generated.\nYou can add the necessary .idx file by running\n\tgribmap -i %s", ctlfile, ctlfile);
+      cdoAbort("The fileformat GRIB2 is not fully supported yet for the gradsdes operator.\n"
+               "The .ctl file %s was generated. You can add the necessary .idx file by running\n\tgribmap -i %s", ctlfile, ctlfile);
       // write_map_grib2(idxfile, map_version, nrecords, intnum, fltnum, bignum);
     }
 
@@ -1408,8 +1407,6 @@ void *Gradsdes(void *argument)
   if ( intnum ) Free(intnum);
   if ( fltnum ) Free(fltnum);
 
- END_LABEL:
-
   cdoFinish();
 
   return 0;
