@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,7 @@
 #include "cdo_int.h"
 #include "pstream.h"
 #include "grid.h"
-
+#include "constants.h"
 
 static
 double orthodrome(double px1, double py1, double px2, double py2)
@@ -38,67 +38,92 @@ double orthodrome(double px1, double py1, double px2, double py2)
 }
 
 
+void grid_cell_area(int gridID, double *array)
+{
+  int gridtype = gridInqType(gridID);
+  int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+
+  if ( gridtype == GRID_LONLAT      ||
+       gridtype == GRID_GAUSSIAN    ||
+       projtype == CDI_PROJ_RLL     ||
+       projtype == CDI_PROJ_LAEA    ||
+       projtype == CDI_PROJ_SINU    ||
+       projtype == CDI_PROJ_LCC     ||
+       gridtype == GRID_LCC         ||
+       gridtype == GRID_GME         ||
+       gridtype == GRID_CURVILINEAR ||
+       gridtype == GRID_UNSTRUCTURED )
+    {
+      if ( gridHasArea(gridID) )
+        {
+          if ( cdoVerbose ) cdoPrint("Using existing grid cell area!");
+          gridInqArea(gridID, array);
+        }
+      else
+        {
+          int status = gridGenArea(gridID, array);
+          if ( status == 1 )
+            cdoAbort("%s: Grid corner missing!", __func__);
+          else if ( status == 2 )
+            cdoAbort("%s: Can't compute grid cell area for this grid!", __func__);
+
+          int ngp = gridInqSize(gridID);
+          for ( int i = 0; i < ngp; ++i )
+            array[i] *= PlanetRadius*PlanetRadius;
+        }
+    }
+  else
+    {
+      if ( gridtype == GRID_GAUSSIAN_REDUCED )
+        cdoAbort("Unsupported grid type: %s, use CDO option -R to convert reduced to regular grid!",
+                 gridNamePtr(gridtype));
+      else
+        cdoAbort("%s: Unsupported grid type: %s", __func__, gridNamePtr(gridtype));
+    }
+}
+
 void *Gridcell(void *argument)
 {
-  int GRIDAREA, GRIDWGTS, GRIDMASK, GRIDDX, GRIDDY;
-  int operatorID;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
-  int gridID, zaxisID;
-  int gridtype;
   int status;
-  int ngrids;
-  int need_radius;
-  int tsID, varID, levelID, taxisID;
-  long i, gridsize;
-  char *envstr;
-  double *array = NULL;
-  double  EarthRadius = 6371000; /* default radius of the earth in m */
-  double PlanetRadius = EarthRadius;
 
   cdoInitialize(argument);
 
-  GRIDAREA = cdoOperatorAdd("gridarea",     1,  0, NULL);
-  GRIDWGTS = cdoOperatorAdd("gridweights",  1,  0, NULL);
-  GRIDMASK = cdoOperatorAdd("gridmask",     0,  0, NULL);
-  GRIDDX   = cdoOperatorAdd("griddx",       1,  0, NULL);
-  GRIDDY   = cdoOperatorAdd("griddy",       1,  0, NULL);
+  int GRIDAREA = cdoOperatorAdd("gridarea",     1,  0, NULL);
+  int GRIDWGTS = cdoOperatorAdd("gridweights",  1,  0, NULL);
+  int GRIDMASK = cdoOperatorAdd("gridmask",     0,  0, NULL);
+  int GRIDDX   = cdoOperatorAdd("griddx",       1,  0, NULL);
+  int GRIDDY   = cdoOperatorAdd("griddy",       1,  0, NULL);
 
-  operatorID = cdoOperatorID();
-
-  need_radius = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
 
+  bool need_radius = cdoOperatorF1(operatorID) > 0;
   if ( need_radius )
     {
-      envstr = getenv("PLANET_RADIUS");
+      char *envstr = getenv("PLANET_RADIUS");
       if ( envstr )
 	{
-	  double fval;
-	  fval = atof(envstr);
-	  if ( fval > 0 )
-	    {
-	      PlanetRadius = fval;
-	      if ( cdoVerbose )
-		cdoPrint("Set PlanetRadius to %g", PlanetRadius);
-	    }
+	  double fval = atof(envstr);
+	  if ( fval > 0 ) PlanetRadius = fval;
 	}
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  if ( cdoVerbose ) cdoPrint("PlanetRadius: %g", PlanetRadius);
 
-  vlistID1 = streamInqVlist(streamID1);
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  ngrids = vlistNgrids(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+
+  int ngrids = vlistNgrids(vlistID1);
 
   if ( ngrids > 1 )
     cdoWarning("Found more than 1 grid, using the first one!");
 
-  gridID  = vlistGrid(vlistID1, 0);
+  int gridID  = vlistGrid(vlistID1, 0);
 
-  zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+  int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
 
-  vlistID2 = vlistCreate();
-  varID    = vlistDefVar(vlistID2, gridID, zaxisID, TSTEP_CONSTANT);
+  int vlistID2 = vlistCreate();
+  int varID    = vlistDefVar(vlistID2, gridID, zaxisID, TSTEP_CONSTANT);
   vlistDefNtsteps(vlistID2, 0);
 
   if ( operatorID == GRIDAREA )
@@ -107,17 +132,17 @@ void *Gridcell(void *argument)
       vlistDefVarStdname(vlistID2, varID, "area");
       vlistDefVarLongname(vlistID2, varID, "area of grid cell");
       vlistDefVarUnits(vlistID2, varID, "m2");
-      vlistDefVarDatatype(vlistID2, varID, DATATYPE_FLT64);
+      vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT64);
     }
   else if ( operatorID == GRIDWGTS )
     {
       vlistDefVarName(vlistID2, varID, "cell_weights");
-      vlistDefVarDatatype(vlistID2, varID, DATATYPE_FLT64);
+      vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT64);
     }
   else if ( operatorID == GRIDMASK )
     {
       vlistDefVarName(vlistID2, varID, "grid_mask");
-      vlistDefVarDatatype(vlistID2, varID, DATATYPE_UINT8);
+      vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_UINT8);
     }
   else if ( operatorID == GRIDDX )
     {
@@ -132,82 +157,47 @@ void *Gridcell(void *argument)
       vlistDefVarUnits(vlistID2, varID, "m");
     }
 
-  taxisID = taxisCreate(TAXIS_ABSOLUTE);
+  int taxisID = taxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID2, taxisID);
 
 
-  gridsize = gridInqSize(gridID);
-  array = malloc(gridsize*sizeof(double));
+  long gridsize = gridInqSize(gridID);
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
 
   if ( operatorID == GRIDAREA )
     {
-      gridtype = gridInqType(gridID);
-      if ( gridtype == GRID_LONLAT      ||
-	   gridtype == GRID_GAUSSIAN    ||
-	   gridtype == GRID_LCC         ||
-	   gridtype == GRID_GME         ||
-	   gridtype == GRID_CURVILINEAR ||
-	   gridtype == GRID_UNSTRUCTURED )
-	{
-	  if ( gridHasArea(gridID) )
-	    {
-	      if ( cdoVerbose ) cdoPrint("Using existing grid cell area!");
-	      gridInqArea(gridID, array);
-	    }
-	  else
-	    {
-	      status = gridGenArea(gridID, array);
-	      if ( status == 1 )
-		cdoAbort("Grid corner missing!");
-	      else if ( status == 2 )
-		cdoAbort("Can't compute grid cell areas for this grid!");
-
-	      for ( i = 0; i < gridsize; ++i )
-		array[i] *= PlanetRadius*PlanetRadius;
-	    }
-	}
-      else
-	{
-	  if ( gridtype == GRID_GAUSSIAN_REDUCED )
-	    cdoAbort("Unsupported grid type: %s, use CDO option -R to convert reduced to regular grid!",
-		     gridNamePtr(gridtype));
-	  else
-	    cdoAbort("Unsupported grid type: %s", gridNamePtr(gridtype));
-	}
+      grid_cell_area(gridID, array);
     }
   else if ( operatorID == GRIDWGTS )
     {
       status = gridWeights(gridID, array);
-      if ( status != 0 )
-	  cdoWarning("Using constant grid cell area weights!");
+      if ( status != 0 ) cdoWarning("Using constant grid cell area weights!");
     }
   else if ( operatorID == GRIDMASK )
     {
       int *mask;
-      mask = malloc(gridsize*sizeof(int));
+      mask = (int*) Malloc(gridsize*sizeof(int));
       if ( gridInqMask(gridID, NULL) )
 	{
 	  gridInqMask(gridID, mask);
 	}
       else
 	{
-	  for ( i = 0; i < gridsize; ++i ) mask[i] = 1;
+	  for ( int i = 0; i < gridsize; ++i ) mask[i] = 1;
 	}
 
-      for ( i = 0; i < gridsize; ++i ) array[i] = mask[i];
-      free(mask);
+      for ( int i = 0; i < gridsize; ++i ) array[i] = mask[i];
+      Free(mask);
     }
   else if ( operatorID == GRIDDX || operatorID == GRIDDY )
     {
-      gridtype = gridInqType(gridID);
+      int gridtype = gridInqType(gridID);
       if ( gridtype == GRID_LONLAT      ||
 	   gridtype == GRID_GAUSSIAN    ||
 	   gridtype == GRID_LCC         ||
 	   gridtype == GRID_CURVILINEAR )
 	{
-	  long i, j, xsize, ysize;
-	  double *xv, *yv;
 	  double len1 = 0, len2 = 0;
 	  char units[CDI_MAX_NAME];
 
@@ -215,11 +205,11 @@ void *Gridcell(void *argument)
 	    gridID = gridToCurvilinear(gridID, 1);
 
 	  gridsize = gridInqSize(gridID);
-	  xsize = gridInqXsize(gridID);
-	  ysize = gridInqYsize(gridID);
+	  long xsize = gridInqXsize(gridID);
+	  long ysize = gridInqYsize(gridID);
 
-	  xv = malloc(gridsize*sizeof(double));
-	  yv = malloc(gridsize*sizeof(double));
+	  double *xv = (double*) Malloc(gridsize*sizeof(double));
+	  double *yv = (double*) Malloc(gridsize*sizeof(double));
 
 	  gridInqXvals(gridID, xv);
 	  gridInqYvals(gridID, yv);
@@ -228,27 +218,13 @@ void *Gridcell(void *argument)
 
 	  gridInqXunits(gridID, units);
 
-	  if ( memcmp(units, "degree", 6) == 0 )
-	    {
-	      for ( i = 0; i < gridsize; ++i )
-		{
-		  xv[i] *= DEG2RAD;
-		  yv[i] *= DEG2RAD;
-		}
-	    }
-	  else if ( memcmp(units, "radian", 6) == 0 )
-	    {
-	      /* No conversion necessary */
-	    }
-	  else
-	    {
-	      cdoWarning("Unknown units supplied for grid1 center lat/lon: proceeding assuming radians");
-	    }
+	  grid_to_radian(units, gridsize, xv, "grid longitudes");
+	  grid_to_radian(units, gridsize, yv, "grid latitudes");
 
 	  if ( operatorID == GRIDDX )
 	    {
-	      for ( j = 0; j < ysize; ++j )
-		for ( i = 0; i < xsize; ++i )
+	      for ( long j = 0; j < ysize; ++j )
+		for ( long i = 0; i < xsize; ++i )
 		  {
 		    if ( i == 0 )
 		      {
@@ -271,8 +247,8 @@ void *Gridcell(void *argument)
 	    }
 	  else
 	    {
-	      for ( i = 0; i < xsize; ++i )
-	        for ( j = 0; j < ysize; ++j )
+	      for ( long i = 0; i < xsize; ++i )
+	        for ( long j = 0; j < ysize; ++j )
 		  {
 		    if ( j == 0 )
 		      {
@@ -294,8 +270,8 @@ void *Gridcell(void *argument)
 		  }
 	    }
 
-	  free(xv);
-	  free(yv);
+	  Free(xv);
+	  Free(yv);
 	}
       else
 	{
@@ -308,24 +284,24 @@ void *Gridcell(void *argument)
     }
 
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  tsID = 0;
+  int tsID = 0;
   streamDefTimestep(streamID2, tsID);
 
-  varID   = 0;
-  levelID = 0;
+  varID = 0;
+  int levelID = 0;
   streamDefRecord(streamID2, varID, levelID);
   streamWriteRecord(streamID2, array, 0);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array ) free(array);
+  if ( array ) Free(array);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
