@@ -46,9 +46,8 @@ static double humidityIndex(double t, double e, double r, double missval)
 }
 
 
-static void farexpr(field_t *field1, field_t field2, field_t field3, double (*expression)(double, double, double, double))
+static void farexpr(field_type *field1, field_type field2, field_type field3, double (*expression)(double, double, double, double))
 {
-  int   i, len;
   const int     grid1    = field1->grid;
   const int     nmiss1   = field1->nmiss;
   const double  missval1 = field1->missval;
@@ -62,14 +61,14 @@ static void farexpr(field_t *field1, field_t field2, field_t field3, double (*ex
   const double  missval3 = field3.missval;
   const double *array3   = field3.ptr;
 
-  len = gridInqSize(grid1);
+  int len = gridInqSize(grid1);
 
   if ( len != gridInqSize(grid2) || len != gridInqSize(grid3) )
     cdoAbort("Fields have different gridsize (%s)", __func__);
 
   if ( nmiss1 > 0 || nmiss2 > 0 || nmiss3 > 0 )
     {
-      for ( i = 0; i < len; i++ )
+      for ( int i = 0; i < len; i++ )
         if ( DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) || DBL_IS_EQUAL(array3[i], missval3))  
 	  array1[i] = missval1;
 	else
@@ -77,66 +76,61 @@ static void farexpr(field_t *field1, field_t field2, field_t field3, double (*ex
     }
   else
     {
-      for ( i = 0; i < len; i++ )
+      for ( int i = 0; i < len; i++ )
         array1[i] = expression(array1[i], array2[i], array3[i], missval1);  
     }
 
   field1->nmiss = 0;
-  for ( i = 0; i < len; i++ )
+  for ( int i = 0; i < len; i++ )
     if ( DBL_IS_EQUAL(array1[i], missval1) ) field1->nmiss++;
 }
 
    
 void *Hi(void *argument)
 {
-  int streamID1, streamID2, streamID3, streamID4;
-  int gridsize;
-  int nrecs, nrecs2, nrecs3, recID;
-  int tsID;
-  int gridID, zaxisID;
-  int varID1, varID2, varID3, varID4;
+  int nmiss;
+  int nrecs;
+  int varID1, varID2, varID3;
   int levelID1, levelID2, levelID3;
-  int vlistID1, vlistID2, vlistID3, vlistID4;
-  int taxisID1, taxisID2, taxisID3, taxisID4;
-  field_t field1, field2, field3;
 
   cdoInitialize(argument);
   cdoOperatorAdd("hi", 0, 0, NULL);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
-  streamID3 = streamOpenRead(cdoStreamName(2));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID3 = streamOpenRead(cdoStreamName(2));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistID3 = streamInqVlist(streamID3);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistID3 = streamInqVlist(streamID3);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = vlistInqTaxis(vlistID2);
-  taxisID3 = vlistInqTaxis(vlistID3);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  //taxisID2 = vlistInqTaxis(vlistID2);
+  //taxisID3 = vlistInqTaxis(vlistID3);
 
   vlistCompare(vlistID1, vlistID2, CMP_DIM);
   vlistCompare(vlistID1, vlistID3, CMP_DIM);
   
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
 
+  field_type field1, field2, field3;
   field_init(&field1);
   field_init(&field2);
   field_init(&field3);
-  field1.ptr = malloc(gridsize*sizeof(double));
-  field2.ptr = malloc(gridsize*sizeof(double));
-  field3.ptr = malloc(gridsize*sizeof(double));
+  field1.ptr = (double*) Malloc(gridsize*sizeof(double));
+  field2.ptr = (double*) Malloc(gridsize*sizeof(double));
+  field3.ptr = (double*) Malloc(gridsize*sizeof(double));
 
   if ( cdoVerbose )
     cdoPrint("Number of timesteps: file1 %d, file2 %d, file3 %d",
 	     vlistNtsteps(vlistID1), vlistNtsteps(vlistID2), vlistNtsteps(vlistID3));
 
-  vlistID4 = vlistCreate();
-  gridID   = vlistInqVarGrid(vlistID1, FIRST_VAR);
-  zaxisID  = vlistInqVarZaxis(vlistID1, FIRST_VAR);
-  varID4   = vlistDefVar(vlistID4, gridID, zaxisID, TSTEP_INSTANT);
+  int vlistID4 = vlistCreate();
+  int gridID   = vlistInqVarGrid(vlistID1, FIRST_VAR);
+  int zaxisID  = vlistInqVarZaxis(vlistID1, FIRST_VAR);
+  int varID4   = vlistDefVar(vlistID4, gridID, zaxisID, TSTEP_INSTANT);
 
-  taxisID4 = taxisCreate(TAXIS_RELATIVE);
+  int taxisID4 = taxisCreate(TAXIS_RELATIVE);
   taxisDefTunit(taxisID4, TUNIT_MINUTE);
   taxisDefCalendar(taxisID4, CALENDAR_STANDARD);
   taxisDefRdate(taxisID4, 19550101);
@@ -147,31 +141,34 @@ void *Hi(void *argument)
   vlistDefVarLongname(vlistID4, varID4, HI_LONGNAME);
   vlistDefVarUnits(vlistID4, varID4, HI_UNITS);
 
-  streamID4 = streamOpenWrite(cdoStreamName(3), cdoFiletype());
+  int streamID4 = streamOpenWrite(cdoStreamName(3), cdoFiletype());
 
   streamDefVlist(streamID4, vlistID4);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
-      nrecs2 = streamInqTimestep(streamID2, tsID);
-      nrecs3 = streamInqTimestep(streamID3, tsID);
+      int nrecs2 = streamInqTimestep(streamID2, tsID);
+      int nrecs3 = streamInqTimestep(streamID3, tsID);
       if ( nrecs2 == 0 || nrecs3 == 0 )
         cdoAbort("Input streams have different number of timesteps!");
 
       taxisCopyTimestep(taxisID4, taxisID1);
       streamDefTimestep(streamID4, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID1, &levelID1);
-	  streamReadRecord(streamID1, field1.ptr, &field1.nmiss);
-
+	  streamReadRecord(streamID1, field1.ptr, &nmiss);
+          field1.nmiss = (size_t) nmiss;
+          
 	  streamInqRecord(streamID2, &varID2, &levelID2);
-	  streamReadRecord(streamID2, field2.ptr, &field2.nmiss);
+	  streamReadRecord(streamID2, field2.ptr, &nmiss);
+          field2.nmiss = (size_t) nmiss;
 	  
 	  streamInqRecord(streamID3, &varID3, &levelID3);
-	  streamReadRecord(streamID3, field3.ptr, &field3.nmiss);
+	  streamReadRecord(streamID3, field3.ptr, &nmiss);
+          field3.nmiss = (size_t) nmiss;
 	  
 	  if ( varID1 != varID2 || varID1 != varID3 || levelID1 != levelID2 || levelID1 != levelID3 )
 	    cdoAbort("Input streams have different structure!");
@@ -190,7 +187,7 @@ void *Hi(void *argument)
 	  farexpr(&field1, field2, field3, humidityIndex);
 	  
 	  streamDefRecord(streamID4, varID4, levelID1);
-	  streamWriteRecord(streamID4, field1.ptr, field1.nmiss);
+	  streamWriteRecord(streamID4, field1.ptr, (int)field1.nmiss);
 	}
 
       tsID++;
@@ -201,11 +198,11 @@ void *Hi(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( field1.ptr ) free(field1.ptr);
-  if ( field2.ptr ) free(field2.ptr);
-  if ( field3.ptr ) free(field3.ptr);
+  if ( field1.ptr ) Free(field1.ptr);
+  if ( field2.ptr ) Free(field2.ptr);
+  if ( field3.ptr ) Free(field3.ptr);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
