@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -45,8 +45,8 @@ void init_amsr_day(int vlistID, int gridID, int zaxisID, int nvars)
     5:cloud	cloud liquid water in millimeters
     6:rain   	rain rate in millimeters/hour
   */
-  char *name[]  = {"hours", "sst", "wind", "vapor", "cloud", "rain"};
-  char *units[] = {"h", "deg Celcius", "m/s", "mm", "mm", "mm/h"};
+  const char *name[]  = {"hours", "sst", "wind", "vapor", "cloud", "rain"};
+  const char *units[] = {"h", "deg Celcius", "m/s", "mm", "mm", "mm/h"};
   double xscale[]  = {0.1, 0.15, 0.2, 0.3, 0.01, 0.1};
   double xminval[] = {0., -3., 0., 0., 0., 0.};
   int i, varID;
@@ -56,7 +56,7 @@ void init_amsr_day(int vlistID, int gridID, int zaxisID, int nvars)
       varID = vlistDefVar(vlistID, gridID, zaxisID, TSTEP_INSTANT);
       vlistDefVarName(vlistID, varID, name[i]);
       vlistDefVarUnits(vlistID, varID, units[i]);
-      vlistDefVarDatatype(vlistID, varID, DATATYPE_INT16);
+      vlistDefVarDatatype(vlistID, varID, CDI_DATATYPE_INT16);
       vlistDefVarMissval(vlistID, varID, 254);
       vlistDefVarScalefactor(vlistID, varID, xscale[i]);
       vlistDefVarAddoffset(vlistID, varID, xminval[i]);
@@ -91,8 +91,8 @@ void init_amsr_averaged(int vlistID, int gridID, int zaxisID, int nvars)
     4:cloud     cloud liquid water in millimeters
     5:rain	rain rate in millimeters/hour
   */
-  char *name[]  = {"sst", "wind", "vapor", "cloud", "rain"};
-  char *units[] = {"deg Celcius", "m/s", "mm", "mm", "mm/h"};
+  const char *name[]  = {"sst", "wind", "vapor", "cloud", "rain"};
+  const char *units[] = {"deg Celcius", "m/s", "mm", "mm", "mm/h"};
   double xscale[]  = {0.15, 0.2, 0.3, 0.01, 0.1};
   double xminval[] = {-3., 0., 0., 0., 0.};
   int i, varID;
@@ -103,7 +103,7 @@ void init_amsr_averaged(int vlistID, int gridID, int zaxisID, int nvars)
       varID = vlistDefVar(vlistID, gridID, zaxisID, TSTEP_INSTANT);
       vlistDefVarName(vlistID, varID, name[i]);
       vlistDefVarUnits(vlistID, varID, units[i]);
-      vlistDefVarDatatype(vlistID, varID, DATATYPE_INT16);
+      vlistDefVarDatatype(vlistID, varID, CDI_DATATYPE_INT16);
       vlistDefVarMissval(vlistID, varID, 254);
       vlistDefVarScalefactor(vlistID, varID, xscale[i]);
       vlistDefVarAddoffset(vlistID, varID, xminval[i]);
@@ -121,7 +121,7 @@ void read_amsr(FILE *fp, int vlistID, int nvars, double *data[], int *nmiss)
   for ( varID = 0; varID < nvars; ++varID )
     {
       gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-      amsr_data = realloc(amsr_data, gridsize);
+      amsr_data = (unsigned char *) Realloc(amsr_data, gridsize);
       size = fread(amsr_data, 1, gridsize, fp);
       if ( (int)size != gridsize ) cdoAbort("Read error!");
 
@@ -144,15 +144,13 @@ void read_amsr(FILE *fp, int vlistID, int nvars, double *data[], int *nmiss)
 	}
     } 
 
-  free(amsr_data);
+  Free(amsr_data);
 }
 
 static
 void write_data(int streamID, int nvars, double *data[], int *nmiss)
 {
-  int varID;
-
-  for ( varID = 0; varID < nvars; ++varID )
+  for ( int varID = 0; varID < nvars; ++varID )
     {
       streamDefRecord(streamID, varID, 0);
       streamWriteRecord(streamID, data[varID], nmiss[varID]);
@@ -163,72 +161,61 @@ static
 int getDate(const char *name)
 {
   int date = 0;
-  size_t len;
-  char *pname;
-
-  len = strlen(name);
-
-  pname = strchr(name, '_');
+  char *pname = (char *)strchr(name, '_');
 
   if ( pname ) date = atoi(pname+1);
 
-  return(date);
+  return date;
 }
 
 
 void *Importamsr(void *argument)
 {
-  int streamID;
   int tsID;
-  int gridID, zaxisID, taxisID, vlistID;
-  int gridsize;
-  int i;
   int nvars;
-  int vdate = 0, vtime = 0;
+  int vtime = 0;
   double xvals[NLON], yvals[NLAT];
   double *data[MAX_VARS];
   int nmiss[MAX_VARS];
-  FILE *fp;
-  size_t fsize;
 
   cdoInitialize(argument);
 
-  fp = fopen(cdoStreamName(0)->args, "r");
+  FILE *fp = fopen(cdoStreamName(0)->args, "r");
   if ( fp == NULL ) { perror(cdoStreamName(0)->args); exit(EXIT_FAILURE); }
 
   fseek(fp, 0L, SEEK_END);
-  fsize = (size_t) ftell(fp);
+  size_t fsize = (size_t) ftell(fp);
   fseek(fp, 0L, SEEK_SET);
 
-  vdate = getDate(cdoStreamName(0)->args);
+  int vdate = getDate(cdoStreamName(0)->args);
   if ( vdate <= 999999 ) vdate = vdate*100 + 1;
 
-  streamID = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   /*
     Longitude  is 0.25*xdim-0.125    degrees east
     Latitude   is 0.25*ydim-90.125
   */
-  gridsize = NLON*NLAT;
-  gridID = gridCreate(GRID_LONLAT, gridsize);
+  int gridsize = NLON*NLAT;
+  int gridID = gridCreate(GRID_LONLAT, gridsize);
   gridDefXsize(gridID, NLON);
   gridDefYsize(gridID, NLAT);
-  for ( i = 0; i < NLON; ++i ) xvals[i] = 0.25*(i+1) - 0.125;
-  for ( i = 0; i < NLAT; ++i ) yvals[i] = 0.25*(i+1) - 90.125;
+  for ( int i = 0; i < NLON; ++i ) xvals[i] = 0.25*(i+1) - 0.125;
+  for ( int i = 0; i < NLAT; ++i ) yvals[i] = 0.25*(i+1) - 90.125;
   gridDefXvals(gridID, xvals);
   gridDefYvals(gridID, yvals);
 
-  zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+  int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
 
-  taxisID = taxisCreate(TAXIS_ABSOLUTE);
+  int taxisID = taxisCreate(TAXIS_ABSOLUTE);
 
-  vlistID = vlistCreate();
+  int vlistID = vlistCreate();
   vlistDefTaxis(vlistID, taxisID);
 
   if ( fsize == 12441600 )
     {
       nvars = 6;
-      for ( i = 0; i < nvars; ++i ) data[i] = malloc(gridsize*sizeof(double));
+      for ( int i = 0; i < nvars; ++i ) data[i] = (double*) Malloc(gridsize*sizeof(double));
 
       init_amsr_day(vlistID, gridID, zaxisID, nvars);
 
@@ -248,12 +235,12 @@ void *Importamsr(void *argument)
 	  write_data(streamID, nvars, data, nmiss);
 	}
 
-      for ( i = 0; i < nvars; ++i ) free(data[i]);
+      for ( int i = 0; i < nvars; ++i ) Free(data[i]);
     }
   else if ( fsize == 5184000 )
     {
       nvars = 5;
-      for ( i = 0; i < nvars; ++i ) data[i] = malloc(gridsize*sizeof(double));
+      for ( int i = 0; i < nvars; ++i ) data[i] = (double*) Malloc(gridsize*sizeof(double));
 
       init_amsr_averaged(vlistID, gridID, zaxisID, nvars);
 
@@ -270,7 +257,7 @@ void *Importamsr(void *argument)
 
       write_data(streamID, nvars, data, nmiss);
 
-      for ( i = 0; i < nvars; ++i ) free(data[i]);
+      for ( int i = 0; i < nvars; ++i ) Free(data[i]);
     }
   else
     cdoAbort("Unexpected file size for AMSR data!");
@@ -288,5 +275,5 @@ void *Importamsr(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
