@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,10 @@
 #  include "config.h"
 #endif
 
-#include <ctype.h>
+#ifndef _XOPEN_SOURCE
+#define _XOPEN_SOURCE 600 /* fseeko */
+#endif
+
 
 #include <cdi.h>
 #include "cdo.h"
@@ -66,10 +69,9 @@ void get_dim_vals(dsets_t *pfi, double *vals, int dimlen, int dim)
 static
 void rev_vals(double *vals, int n)
 {
-  int i;
   double dum;
 
-  for ( i = 0; i < n/2; ++i )
+  for ( int i = 0; i < n/2; ++i )
     {
       dum = vals[i];
       vals[i] = vals[n-1-i];
@@ -86,10 +88,10 @@ int y_is_gauss(double *gridyvals, int ysize)
   if ( ysize > 2 )
     {
       double *yvals, *yw;
-      yvals = malloc(ysize*sizeof(double));
-      yw    = malloc(ysize*sizeof(double));
-      gaussaw(yvals, yw, ysize);
-      free(yw);
+      yvals = (double*) Malloc(ysize*sizeof(double));
+      yw    = (double*) Malloc(ysize*sizeof(double));
+      gaussaw(yvals, yw, (size_t)ysize);
+      Free(yw);
       for ( i = 0; i < (int) ysize; i++ )
 	yvals[i] = asin(yvals[i])/M_PI*180.0;
 
@@ -109,32 +111,30 @@ int y_is_gauss(double *gridyvals, int ysize)
 	  if ( i == (int) ysize ) lgauss = TRUE;
 	}
 
-      free(yvals);
+      Free(yvals);
     }
 
-  return (lgauss);
+  return lgauss;
 }
 
 static
 int define_grid(dsets_t *pfi)
 {
   int gridID, gridtype;
-  int nx, ny;
-  double *xvals, *yvals;
   int lgauss = FALSE;
 
-  nx = pfi->dnum[0];
-  ny = pfi->dnum[1];
+  int nx = pfi->dnum[0];
+  int ny = pfi->dnum[1];
 
-  xvals = malloc(nx*sizeof(double));
-  yvals = malloc(ny*sizeof(double));
+  double *xvals = (double*) Malloc(nx*sizeof(double));
+  double *yvals = (double*) Malloc(ny*sizeof(double));
 
   get_dim_vals(pfi, xvals, nx, 0);
   get_dim_vals(pfi, yvals, ny, 1);
 
   if ( pfi->yrflg ) rev_vals(yvals, ny);
 
-  if ( pfi->linear[1] == 0 ) lgauss = y_is_gauss(yvals, ny);
+  if ( pfi->linear[1] == 0 ) lgauss = y_is_gauss(yvals, (size_t)ny);
 
   if ( lgauss ) gridtype = GRID_GAUSSIAN;
   else          gridtype = GRID_LONLAT;
@@ -146,25 +146,21 @@ int define_grid(dsets_t *pfi)
   gridDefXvals(gridID, xvals);
   gridDefYvals(gridID, yvals);
 
-  free(xvals);
-  free(yvals);
+  Free(xvals);
+  Free(yvals);
   
-  return (gridID);
+  return gridID;
 }
 
 static
 int define_level(dsets_t *pfi, int nlev)
 {
   int zaxisID = -1;
-  int nz;
-
-  nz = pfi->dnum[2];
+  int nz = pfi->dnum[2];
 
   if ( nz )
     {
-      double *zvals = NULL;
-
-      zvals = malloc(nz*sizeof(double));
+      double *zvals = (double*) Malloc(nz*sizeof(double));
 
       get_dim_vals(pfi, zvals, nz, 2);
 
@@ -178,7 +174,7 @@ int define_level(dsets_t *pfi, int nlev)
 	}
       zaxisDefLevels(zaxisID, zvals);
 
-      free(zvals);
+      Free(zvals);
     }
   else
     {
@@ -189,75 +185,64 @@ int define_level(dsets_t *pfi, int nlev)
       zaxisDefLevels(zaxisID, &level);
     }
 
-  return (zaxisID);
+  return zaxisID;
 }
 
 
 void *Importbinary(void *argument)
 {
-  int streamID;
-  int gridID = -1, zaxisID, zaxisIDsfc, taxisID, vlistID;
   int i;
   int nmiss = 0, n_nan;
   int ivar;
   int varID = -1, levelID, tsID;
   int gridsize;
-  int  status;
   int datatype;
   dsets_t pfi;
   int vdate, vtime;
   int tcur, told,fnum;
   int tmin=0,tmax=0;
   char *ch = NULL;
-  int nvars, nlevels, nrecs;
-  int recID;
+  int nlevels;
   int e, flag;
   size_t rc, recsize;
   int recoffset;
   char *rec = NULL;
-  struct gavar *pvar;
   struct dt dtim, dtimi;
   double missval;
   double fmin, fmax;
   double *array;
   double sfclevel = 0;
-  int *recVarID, *recLevelID;
-  int *var_zaxisID;
-  int *var_dfrm = NULL;
   char vdatestr[32], vtimestr[32];	  
 
   cdoInitialize(argument);
 
   dsets_init(&pfi);
 
-  status = read_gradsdes(cdoStreamName(0)->args, &pfi);
+  int status = read_gradsdes(cdoStreamName(0)->args, &pfi);
   if ( cdoVerbose ) fprintf(stderr, "status %d\n", status);
   //if ( status ) cdoAbort("Open failed on %s!", pfi.name);
   if ( status ) cdoAbort("Open failed!");
 
-  nrecs = pfi.trecs;
-  nvars = pfi.vnum;
-  pvar  = pfi.pvar1;
+  int nrecs = pfi.trecs;
+  int nvars = pfi.vnum;
+  struct gavar *pvar  = pfi.pvar1;
 
   if ( nvars == 0 ) cdoAbort("No variables found!");
 
-  gridID = define_grid(&pfi);
-  if ( cdoVerbose ) gridPrint(gridID, 1);
+  int gridID = define_grid(&pfi);
+  int zaxisID = define_level(&pfi, 0);
 
-  zaxisID = define_level(&pfi, 0);
-  if ( cdoVerbose ) zaxisPrint(zaxisID);
-
-  zaxisIDsfc = zaxisCreate(ZAXIS_SURFACE, 1);
+  int zaxisIDsfc = zaxisCreate(ZAXIS_SURFACE, 1);
   zaxisDefLevels(zaxisIDsfc, &sfclevel);
 
-  vlistID = vlistCreate();
+  int vlistID = vlistCreate();
 
-  var_zaxisID = malloc(nvars*sizeof(int));
-  recVarID    = malloc(nrecs*sizeof(int));
-  recLevelID  = malloc(nrecs*sizeof(int));
-  var_dfrm    = malloc(nrecs*sizeof(int));
+  int *var_zaxisID = (int*) Malloc(nvars*sizeof(int));
+  int *recVarID    = (int*) Malloc(nrecs*sizeof(int));
+  int *recLevelID  = (int*) Malloc(nrecs*sizeof(int));
+  int *var_dfrm    = (int*) Malloc(nrecs*sizeof(int));
 
-  recID = 0;
+  int recID = 0;
   for ( ivar = 0; ivar < nvars; ++ivar )
     {
       /*
@@ -309,26 +294,26 @@ void *Importbinary(void *argument)
       }
 
       missval  = pfi.undef;
-      datatype = DATATYPE_FLT32;
+      datatype = CDI_DATATYPE_FLT32;
 
       if      ( pvar->dfrm ==  1 ) {
-	datatype = DATATYPE_UINT8;
+	datatype = CDI_DATATYPE_UINT8;
 	if ( missval < 0 || missval > 255 ) missval = 255;
       }
       else if ( pvar->dfrm ==  2 )  {
-	datatype = DATATYPE_UINT16;
+	datatype = CDI_DATATYPE_UINT16;
 	if ( missval < 0 || missval > 65535 ) missval = 65535;
       }
       else if ( pvar->dfrm == -2 )  {
-	datatype = DATATYPE_INT16;
+	datatype = CDI_DATATYPE_INT16;
 	if ( missval < -32768 || missval > 32767 ) missval = -32768;
       }
       else if ( pvar->dfrm ==  4 )  {
-	datatype = DATATYPE_INT32;
+	datatype = CDI_DATATYPE_INT32;
 	if ( missval < -2147483648 || missval > 2147483647 ) missval = -2147483646;
       }
       else if ( pfi.flt64 )
-	datatype = DATATYPE_FLT64;
+	datatype = CDI_DATATYPE_FLT64;
       
       vlistDefVarDatatype(vlistID, varID, datatype);
       vlistDefVarMissval(vlistID, varID, missval);
@@ -345,13 +330,15 @@ void *Importbinary(void *argument)
       pvar++;
     }
 
-  taxisID = taxisCreate(TAXIS_RELATIVE);
+  int calendar = CALENDAR_STANDARD;
 
-  taxisDefCalendar(taxisID, CALENDAR_STANDARD);
+  int taxisID = taxisCreate(TAXIS_RELATIVE);
+
+  taxisDefCalendar(taxisID, calendar);
 
   vlistDefTaxis(vlistID, taxisID);
 
-  streamID = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID, vlistID);
 
@@ -366,9 +353,9 @@ void *Importbinary(void *argument)
 
   //recsize = pfi.gsiz*4;
   recsize = pfi.gsiz*8;
-  rec = malloc(recsize);
+  rec = (char*) Malloc(recsize);
 
-  array = malloc(gridsize*sizeof(double));
+  array = (double*) Malloc(gridsize*sizeof(double));
 
   /*
   if (pfi.tmplat)
@@ -438,7 +425,7 @@ void *Importbinary(void *argument)
       pfi.infile = fopen(ch,"rb");
       if (pfi.infile==NULL) {
 	if (pfi.tmplat) {
-	  if ( cdoVerbose ) cdoPrint("Could not open file: %s",ch);
+	  cdoWarning("Could not open file: %s",ch);
 	  break;
 	} else {
 	  cdoAbort("Could not open file: %s",ch);
@@ -474,7 +461,7 @@ void *Importbinary(void *argument)
 	  taxisDefVtime(taxisID, vtime);
 	  streamDefTimestep(streamID, tsID);
 
-	  for ( recID = 0; recID < nrecs; ++recID )
+	  for ( int recID = 0; recID < nrecs; ++recID )
 	    {
 	      /* record size depends on data type */
 	      if (var_dfrm[recID] == 1) {
@@ -495,21 +482,21 @@ void *Importbinary(void *argument)
 
 	      /* convert */
 	      if (var_dfrm[recID] == 1) {
-		unsigned char *carray = (void*)(rec + recoffset);
+		unsigned char *carray = (unsigned char *)(rec + recoffset);
 		for (i = 0; i < gridsize; ++i) array[i] = (double) carray[i];
 	      }
 	      else if (var_dfrm[recID] == 2) {
-		unsigned short *sarray = (void*)(rec + recoffset);
+		unsigned short *sarray = (unsigned short *)(rec + recoffset);
 	        if (pfi.bswap) gabswp2(sarray, gridsize);
 		for (i = 0; i < gridsize; ++i) array[i] = (double) sarray[i];
 	      }
 	      else if (var_dfrm[recID] == -2) {
-		short *sarray = (void*)(rec + recoffset);
+		short *sarray = (short *)(rec + recoffset);
 	        if (pfi.bswap) gabswp2(sarray, gridsize);
 		for (i = 0; i < gridsize; ++i) array[i] = (double) sarray[i];
 	      }
 	      else if (var_dfrm[recID] == 4) {
-		int *iarray = (void*)(rec + recoffset);
+		int *iarray = (int *)(rec + recoffset);
 	        if (pfi.bswap) gabswp(iarray, gridsize);
 		for (i = 0; i < gridsize; ++i) array[i] = (double) iarray[i];
 	      }
@@ -551,9 +538,10 @@ void *Importbinary(void *argument)
 		      if ( array[i] > fmax ) fmax = array[i];
 		    }
 		}
+	      /*
 	      if ( cdoVerbose )
 		printf("%3d %4d %3d %6d %6d %12.5g %12.5g\n", tsID, recID, recoffset, nmiss, n_nan, fmin, fmax);
-
+	      */
 	      varID   = recVarID[recID];
 	      levelID = recLevelID[recID];
 	      streamDefRecord(streamID,  varID,  levelID);
@@ -576,15 +564,15 @@ void *Importbinary(void *argument)
   zaxisDestroy(zaxisID);
   taxisDestroy(taxisID);
 
-  free(array);
-  free(rec);
+  Free(array);
+  Free(rec);
 
-  if ( var_zaxisID ) free(var_zaxisID);
-  if ( recVarID    ) free(recVarID);
-  if ( recLevelID  ) free(recLevelID);
-  if ( var_dfrm    ) free(var_dfrm);
+  if ( var_zaxisID ) Free(var_zaxisID);
+  if ( recVarID    ) Free(recVarID);
+  if ( recLevelID  ) Free(recLevelID);
+  if ( var_dfrm    ) Free(var_dfrm);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
