@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,11 +25,11 @@
 #  include "hdf5.h"
 #endif
 
-#include <ctype.h>
 
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
+#include "grid.h"
 #include "pstream.h"
 
 
@@ -133,25 +133,22 @@ static
 double det_lon_atovs(double r, double r0, double lts, double c, double re)
 {
   const double pi = M_PI;
-  double xla;
 
-  xla=(r-r0)*c/re/cos(lts*pi/180.); /* longitude */
+  double xla=(r-r0)*c/re/cos(lts*pi/180.); /* longitude */
   xla=180.*xla/pi;
 
-  return (xla);
+  return xla;
 }
 
 static
 double det_lat_atovs(double s, double s0, double lts, double c, double re)
 {
   const double pi = M_PI;
-  double siphi;
-  double phi;
 
-  siphi=(s-s0)*c*cos(lts*pi/180.)/re;
-  phi=180.*asin(siphi)/pi; /* latitude */
+  double siphi=(s-s0)*c*cos(lts*pi/180.)/re;
+  double phi=180.*asin(siphi)/pi; /* latitude */
 
-  return (phi);
+  return phi;
 }
 
 static
@@ -163,20 +160,19 @@ int defLonLatGrid(int nx, int ny, double c0, double lts, double re)
   double r0, s0;
   double r, s;
   double xla, phi;
-  double *xvals, *yvals, *xbounds, *ybounds;
 
   get_grid_info(c0, re, &nrx, &nry, &r0, &s0, &c);
 
   if ( nx != nrx || ny != nry )
     {
       printf("nrx=%d nry=%d\n", nrx, nry);
-      return(-1);
+      return -1;
     }
 
-  xvals = (double*) Malloc(nx*sizeof(double));
-  yvals = (double*) Malloc(ny*sizeof(double));
-  xbounds = (double*) Malloc(nx*2*sizeof(double));
-  ybounds = (double*) Malloc(nx*2*sizeof(double));
+  double *xvals = (double*) Malloc(nx*sizeof(double));
+  double *yvals = (double*) Malloc(ny*sizeof(double));
+  double *xbounds = (double*) Malloc(nx*2*sizeof(double));
+  double *ybounds = (double*) Malloc(nx*2*sizeof(double));
 
   for ( i = 0; i < nx; ++i )
     {
@@ -216,7 +212,7 @@ int defLonLatGrid(int nx, int ny, double c0, double lts, double re)
   Free(xbounds);
   Free(ybounds);
 
-  return (gridID);
+  return gridID;
 }
 
 static
@@ -233,18 +229,15 @@ int defSinusoidalGrid(int nx, int ny, double xmin, double xmax, double ymin, dou
   double *yvals = (double*) Malloc(ny*sizeof(double));
 
   for ( int i = 0; i < nx; ++i )
-    {
-      xvals[i] = xmin + i*dx + dx/2;
-      /* printf("x[%d]=%g\n", i, xvals[i]); */
-    }
+    xvals[i] = xmin + i*dx + dx/2;
 
   for ( int i = 0; i < ny; ++i )
-    {
-      yvals[i] = ymax - i*dy - dy/2;
-      /* printf("y[%d]=%g\n", i, yvals[i]); */
-    }
+    yvals[i] = ymax - i*dy - dy/2;
+
+  int gridID = gridCreate(GRID_PROJECTION, nx*ny);
+
+  grid_def_param_sinu(gridID);
 
-  int gridID = gridCreate(GRID_SINUSOIDAL, nx*ny);
   gridDefXsize(gridID, nx);
   gridDefYsize(gridID, ny);
   gridDefXvals(gridID, xvals);
@@ -253,7 +246,7 @@ int defSinusoidalGrid(int nx, int ny, double xmin, double xmax, double ymin, dou
   Free(xvals);
   Free(yvals);
 
-  return (gridID);
+  return gridID;
 }
 
 static
@@ -266,29 +259,24 @@ int defLaeaGrid(int nx, int ny, double xmin, double xmax, double ymin, double ym
   double *yvals = (double*) Malloc(ny*sizeof(double));
 
   for ( int i = 0; i < nx; ++i )
-    {
-      xvals[i] = xmin + i*dx + dx/2;
-      /* printf("x[%d]=%g\n", i, xvals[i]); */
-    }
+    xvals[i] = xmin + i*dx + dx/2;
 
   for ( int i = 0; i < ny; ++i )
-    {
-      yvals[i] = ymax - i*dy - dy/2;
-      /* printf("y[%d]=%g\n", i, yvals[i]); */
-    }
+    yvals[i] = ymax - i*dy - dy/2;
+
+  int gridID = gridCreate(GRID_PROJECTION, nx*ny);
 
-  int gridID = gridCreate(GRID_LAEA, nx*ny);
   gridDefXsize(gridID, nx);
   gridDefYsize(gridID, ny);
   gridDefXvals(gridID, xvals);
   gridDefYvals(gridID, yvals);
 
-  gridDefLaea(gridID, a, lon0, lat0);
+  grid_def_param_laea(gridID, a, lon0, lat0);
 
   Free(xvals);
   Free(yvals);
 
-  return (gridID);
+  return gridID;
 }
 
 static
@@ -342,7 +330,7 @@ int scan_pcs_def(char *pcs_def, char proj[128], double *a, double *lon0, double
 	}
     }
 
-  return (nfound);
+  return nfound;
 }
 
 static
@@ -606,7 +594,7 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       gridID = defLonLatGrid(nx, ny, c0, lts, re);
     }
 
-  return (gridID);
+  return gridID;
 }
 
 static
@@ -747,7 +735,7 @@ int read_region(hid_t loc_id, int nx, int ny)
 			   dx, dy, a, lon0, lat0);
     }
 
-  return (gridID);
+  return gridID;
 }
 
 static
@@ -776,7 +764,7 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
   int i, k;
   int ftype = 0;
   int len;
-  int dtype = DATATYPE_FLT32;
+  int dtype = CDI_DATATYPE_FLT32;
   char attstring[4096];     /* Buffer to read string attribute back */
   char varname[CDI_MAX_NAME];
   short *mask = NULL;
@@ -813,14 +801,14 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
   }
   */
   native_type = H5Tget_native_type(type_id, H5T_DIR_ASCEND);
-  if      ( H5Tequal(native_type, H5T_NATIVE_SCHAR)  > 0 ) {ftype=0; dtype = DATATYPE_INT8;}
-  else if ( H5Tequal(native_type, H5T_NATIVE_UCHAR)  > 0 ) {ftype=0; dtype = DATATYPE_UINT8;}
-  else if ( H5Tequal(native_type, H5T_NATIVE_SHORT)  > 0 ) {ftype=0; dtype = DATATYPE_INT16;}
-  else if ( H5Tequal(native_type, H5T_NATIVE_USHORT) > 0 ) {ftype=0; dtype = DATATYPE_UINT16;}
-  else if ( H5Tequal(native_type, H5T_NATIVE_INT)    > 0 ) {ftype=0; dtype = DATATYPE_INT32;}
-  else if ( H5Tequal(native_type, H5T_NATIVE_UINT)   > 0 ) {ftype=0; dtype = DATATYPE_UINT32;}
-  else if ( H5Tequal(native_type, H5T_NATIVE_FLOAT)  > 0 ) {ftype=1; dtype = DATATYPE_FLT32;}
-  else if ( H5Tequal(native_type, H5T_NATIVE_DOUBLE) > 0 ) {ftype=1; dtype = DATATYPE_FLT64;}
+  if      ( H5Tequal(native_type, H5T_NATIVE_SCHAR)  > 0 ) {ftype=0; dtype = CDI_DATATYPE_INT8;}
+  else if ( H5Tequal(native_type, H5T_NATIVE_UCHAR)  > 0 ) {ftype=0; dtype = CDI_DATATYPE_UINT8;}
+  else if ( H5Tequal(native_type, H5T_NATIVE_SHORT)  > 0 ) {ftype=0; dtype = CDI_DATATYPE_INT16;}
+  else if ( H5Tequal(native_type, H5T_NATIVE_USHORT) > 0 ) {ftype=0; dtype = CDI_DATATYPE_UINT16;}
+  else if ( H5Tequal(native_type, H5T_NATIVE_INT)    > 0 ) {ftype=0; dtype = CDI_DATATYPE_INT32;}
+  else if ( H5Tequal(native_type, H5T_NATIVE_UINT)   > 0 ) {ftype=0; dtype = CDI_DATATYPE_UINT32;}
+  else if ( H5Tequal(native_type, H5T_NATIVE_FLOAT)  > 0 ) {ftype=1; dtype = CDI_DATATYPE_FLT32;}
+  else if ( H5Tequal(native_type, H5T_NATIVE_DOUBLE) > 0 ) {ftype=1; dtype = CDI_DATATYPE_FLT64;}
   else
     {
       cdoWarning("Dataset %s skipped, unsupported native datatype!", varname);
@@ -1063,7 +1051,7 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 
       if ( ftype )
 	{
-	  if ( dtype == DATATYPE_FLT32 )
+	  if ( dtype == CDI_DATATYPE_FLT32 )
 	    {
 	      float *farray;
 	      int i;
@@ -1137,13 +1125,13 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 	cdoPrint("Dataset %s: dtype = %d  minval = %g  maxval = %g  missval = %g",
 		 varname, dtype,  minval, maxval, missval);
 
-      if ( dtype == DATATYPE_UINT8 )
+      if ( dtype == CDI_DATATYPE_UINT8 )
 	{
-	  if ( minval >= 0 && maxval <= 127 ) dtype = DATATYPE_INT8;
+	  if ( minval >= 0 && maxval <= 127 ) dtype = CDI_DATATYPE_INT8;
 	}
-      else if ( dtype == DATATYPE_UINT16 )
+      else if ( dtype == CDI_DATATYPE_UINT16 )
 	{
-	  if ( minval >= 0 && maxval <= 32767 ) dtype = DATATYPE_INT16;
+	  if ( minval >= 0 && maxval <= 32767 ) dtype = CDI_DATATYPE_INT16;
 	}
 
       laddoffset   = IS_NOT_EQUAL(addoffset,   0);
@@ -1183,10 +1171,10 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 	{
 	  if ( ! (missval < minval || missval > maxval) )
 	    {
-	      if ( DBL_IS_EQUAL(missval, 255.) && dtype == DATATYPE_UINT8 )
+	      if ( DBL_IS_EQUAL(missval, 255.) && dtype == CDI_DATATYPE_UINT8 )
 		{
 		  missval = -255;
-		  dtype   = DATATYPE_INT16;
+		  dtype   = CDI_DATATYPE_INT16;
 		  cdoPrint("Dataset %s: changed missval to %g and datatype to INT16!",
 			   varname, missval);
 
@@ -1195,7 +1183,7 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 		}
 	      else
 		cdoWarning(" Missing value is inside the range of valid values!\n"
-                           " Dataset %s,  Missval: %g,  Range: %g - %g",
+                           "\tDataset %s,  Missval: %g,  Range: %g - %g",
 			   varname, missval, minval, maxval);
 	    }
 	}
@@ -1317,17 +1305,17 @@ void get_global_att(hid_t file_id, const char *obj_path, int vlistID)
       if ( type_class == H5T_STRING )
 	{
 	  H5Aread(attr, atype_mem, attstring);
-	  vlistDefAttTxt(vlistID, CDI_GLOBAL, attname, (int)strlen(attstring), attstring);
+	  cdiDefAttTxt(vlistID, CDI_GLOBAL, attname, (int)strlen(attstring), attstring);
 	}
       else if ( type_class == H5T_INTEGER )
 	{
 	  H5Aread(attr, H5T_NATIVE_INT, &attint);
-	  vlistDefAttInt(vlistID, CDI_GLOBAL, attname, DATATYPE_INT32, 1, &attint);
+	  cdiDefAttInt(vlistID, CDI_GLOBAL, attname, CDI_DATATYPE_INT32, 1, &attint);
 	}
       else if ( type_class == H5T_FLOAT )
 	{
 	  H5Aread(attr, H5T_NATIVE_DOUBLE, &attflt);
-	  vlistDefAttFlt(vlistID, CDI_GLOBAL, attname, DATATYPE_FLT64, 1, &attflt);
+	  cdiDefAttFlt(vlistID, CDI_GLOBAL, attname, CDI_DATATYPE_FLT64, 1, &attflt);
 	}
       H5Tclose(atype_mem);
       H5Aclose(attr);
@@ -1347,17 +1335,17 @@ int get_vdate(int vlistID)
   char name[CDI_MAX_NAME];
   char attstr[CDI_MAX_NAME];
 
-  vlistInqNatts(vlistID, CDI_GLOBAL, &natts);
+  cdiInqNatts(vlistID, CDI_GLOBAL, &natts);
 
   for ( i = 0; i < natts; ++i )
     {
-      vlistInqAtt(vlistID, CDI_GLOBAL, i, name, &type, &len);
-      if ( type == DATATYPE_TXT )
+      cdiInqAtt(vlistID, CDI_GLOBAL, i, name, &type, &len);
+      if ( type == CDI_DATATYPE_TXT )
 	{
 	  if ( strcmp(name, "DateAndTime") == 0 ||
 	       strcmp(name, "Date_Time") == 0 )
 	    {
-	      vlistInqAttTxt(vlistID, CDI_GLOBAL, name, CDI_MAX_NAME, attstr);
+	      cdiInqAttTxt(vlistID, CDI_GLOBAL, name, CDI_MAX_NAME, attstr);
 	      if ( len > 8 ) len = 8;
 	      attstr[len] = 0;
 	      vdate = atoi(attstr);
@@ -1366,7 +1354,7 @@ int get_vdate(int vlistID)
 	}
     }
 
-  return (vdate);
+  return vdate;
 }
 
 static
@@ -1423,7 +1411,7 @@ void *Importcmsaf(void *argument)
   cdoInitialize(argument);
 
   if ( cdoDefaultFileType == CDI_UNDEFID )
-    cdoDefaultFileType = FILETYPE_NC;
+    cdoDefaultFileType = CDI_FILETYPE_NC;
 
 #if defined(HAVE_LIBHDF5)
   dsets_init(&dsets);
@@ -1540,7 +1528,7 @@ void *Importcmsaf(void *argument)
       if ( dsets.obj[ivar].units )
 	vlistDefVarUnits(vlistID, varID,  dsets.obj[ivar].units);
       if ( dsets.obj[ivar].title )
-	vlistDefAttTxt(vlistID, varID, "title", (int)strlen(dsets.obj[ivar].title),
+	cdiDefAttTxt(vlistID, varID, "title", (int)strlen(dsets.obj[ivar].title),
 		       dsets.obj[ivar].title);
 
       /*
