@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -28,18 +28,18 @@
 #include "cdo_int.h"
 #include "pstream.h"
 
-
+static
 void printMap(int nlon, int nlat, double *array, double missval, double min, double max)
 {
   /* source code from PINGO */
   int ilon, ilat, i;
   double x, a, b;
-  double step;
   double level[10];
   int min_n, max_n;
-  char c;
+  int bmin = 1, bmax = 1;
+  unsigned char c;
 
-  step = (max - min) / 10;
+  double step = (max - min) / 10;
 
   if ( IS_NOT_EQUAL(step, 0) )
     {
@@ -151,7 +151,29 @@ void printMap(int nlon, int nlat, double *array, double missval, double min, dou
 		    break;
 		  }
 	    }
+
+	  if      ( c == '0' ) { set_text_color(stdout, BRIGHT, BLUE); }
+	  else if ( c == '1' ) { set_text_color(stdout, RESET, BLUE); }
+	  else if ( c == '2' ) { set_text_color(stdout, BRIGHT, CYAN); }
+	  else if ( c == '3' ) { set_text_color(stdout, RESET, CYAN); }
+	  else if ( c == '4' ) { set_text_color(stdout, RESET, GREEN); }
+	  else if ( c == '5' ) { set_text_color(stdout, RESET, YELLOW); }
+	  else if ( c == '6' ) { set_text_color(stdout, RESET, RED); }
+	  else if ( c == '7' ) { set_text_color(stdout, BRIGHT, RED); }
+	  else if ( c == '8' ) { set_text_color(stdout, RESET, MAGENTA); }
+	  else if ( c == '9' ) { set_text_color(stdout, BRIGHT, MAGENTA); }
+	  else if ( c == 'M' )
+	    {
+	      if ( bmax ) { bmax = 0; set_text_color(stdout, BLINK, BLACK); }
+	      else        {           set_text_color(stdout, RESET, BLACK); }
+	    }
+	  else if ( c == 'm' )
+	    {
+	      if ( bmin ) { bmin = 0; set_text_color(stdout, BLINK, BLACK); }
+	      else        {           set_text_color(stdout, RESET, BLACK); }
+	    }
 	  putchar (c);
+	  reset_text_color(stdout);
 	}
       printf (" %0*d\n", nlat < 10 ? 1 : nlat < 100 ? 2 : nlat < 1000 ? 3 : 4, ilat + 1);
       fflush (stdout);
@@ -219,70 +241,63 @@ void printMap(int nlon, int nlat, double *array, double missval, double min, dou
 
 void *Info(void *argument)
 {
-  int INFO, INFOP, INFON, INFOC, MAP;
-  int operatorID;
-  int i;
-  int indf, indg;
-  int varID, recID;
-  int gridsize = 0;
-  int gridID, zaxisID;
-  int code, param;
-  int vdate, vtime;
+  int fpeRaised = 0;
+  int varID, levelID;
   int nrecs;
-  int levelID;
-  int tsID, taxisID;
-  int streamID = 0;
-  int vlistID;
   int nmiss;
-  int number;
   int ivals = 0, nvals = 0;
   int imiss = 0;
   char varname[CDI_MAX_NAME];
   char paramstr[32];
   char vdatestr[32], vtimestr[32];
-  double missval;
-  double *array = NULL;
-  double level;
-  double arrmin = 0, arrmax = 0, arrmean = 0, arrvar = 0;
+  double arrmin = 0, arrmax = 0, arrmean = 0;
+  // double arrvar = 0;
 
   cdoInitialize(argument);
 
-  INFO  = cdoOperatorAdd("info",  0, 0, NULL);
-  INFOP = cdoOperatorAdd("infop", 0, 0, NULL);
-  INFON = cdoOperatorAdd("infon", 0, 0, NULL);
-  INFOC = cdoOperatorAdd("infoc", 0, 0, NULL);
-  MAP   = cdoOperatorAdd("map",   0, 0, NULL);
+  int INFO  = cdoOperatorAdd("info",  0, 0, NULL);
+  int INFOP = cdoOperatorAdd("infop", 0, 0, NULL);
+  int INFON = cdoOperatorAdd("infon", 0, 0, NULL);
+  int INFOC = cdoOperatorAdd("infoc", 0, 0, NULL);
+  int MAP   = cdoOperatorAdd("map",   0, 0, NULL);
 
-  operatorID = cdoOperatorID();
+  UNUSED(INFO);
+  UNUSED(INFOP);
 
-  for ( indf = 0; indf < cdoStreamCnt(); indf++ )
+  int operatorID = cdoOperatorID();
+
+  dtlist_type *dtlist = dtlist_new();
+
+  for ( int indf = 0; indf < cdoStreamCnt(); indf++ )
     {
-      streamID = streamOpenRead(cdoStreamName(indf));
+      int streamID = streamOpenRead(cdoStreamName(indf));
 
-      vlistID = streamInqVlist(streamID);
+      int vlistID = streamInqVlist(streamID);
+      int taxisID = vlistInqTaxis(vlistID);
 
       if ( vlistNvars(vlistID) == 0 ) continue;
 
-      gridsize = vlistGridsizeMax(vlistID);
-      if ( vlistNumber(vlistID) != CDI_REAL ) gridsize *= 2;
+      int gridsizemax = vlistGridsizeMax(vlistID);
+      if ( vlistNumber(vlistID) != CDI_REAL ) gridsizemax *= 2;
 
-      array = malloc(gridsize*sizeof(double));
+      double *array = (double*) Malloc(gridsizemax*sizeof(double));
 
-      indg = 0;
-      tsID = 0;
-      taxisID = vlistInqTaxis(vlistID);
+      int indg = 0;
+      int tsID = 0;
       while ( (nrecs = streamInqTimestep(streamID, tsID)) )
 	{
-	  vdate = taxisInqVdate(taxisID);
-	  vtime = taxisInqVtime(taxisID);
+	  dtlist_taxisInqTimestep(dtlist, taxisID, 0);
+	  int vdate = dtlist_get_vdate(dtlist, 0);
+	  int vtime = dtlist_get_vtime(dtlist, 0);
 
 	  date2str(vdate, vdatestr, sizeof(vdatestr));
 	  time2str(vtime, vtimestr, sizeof(vtimestr));
 
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      if ( (tsID == 0 && recID == 0) || operatorID == MAP )
 		{
+		  set_text_color(stdout, BRIGHT, BLACK);
 		  fprintf(stdout, "%6d :       Date     Time   Level Gridsize    Miss :"
 			  "     Minimum        Mean     Maximum : ",  -(indf+1));
 
@@ -290,36 +305,52 @@ void *Info(void *argument)
 		  else if ( operatorID == INFOC ) fprintf(stdout, "Code number");
 		  else                            fprintf(stdout, "Parameter ID");
 
-		  if ( cdoVerbose ) fprintf(stdout, " : Extra" );              
-		  fprintf(stdout, "\n" );              
+		  if ( cdoVerbose ) fprintf(stdout, " : Extra" );
+		  reset_text_color(stdout);
+		  fprintf(stdout, "\n" );
 		}
 
 	      streamInqRecord(streamID, &varID, &levelID);
 	      streamReadRecord(streamID, array, &nmiss);
 
 	      indg += 1;
-	      param    = vlistInqVarParam(vlistID, varID);
-	      code     = vlistInqVarCode(vlistID, varID);
-	      gridID   = vlistInqVarGrid(vlistID, varID);
-	      zaxisID  = vlistInqVarZaxis(vlistID, varID);
-	      missval  = vlistInqVarMissval(vlistID, varID);
-	      gridsize = gridInqSize(gridID);
-	      number   = vlistInqVarNumber(vlistID, varID);
+	      int param    = vlistInqVarParam(vlistID, varID);
+	      int code     = vlistInqVarCode(vlistID, varID);
+	      int gridID   = vlistInqVarGrid(vlistID, varID);
+	      int zaxisID  = vlistInqVarZaxis(vlistID, varID);
+	      int gridsize = gridInqSize(gridID);
+	      int number   = vlistInqVarNumber(vlistID, varID);
+	      double level = cdoZaxisInqLevel(zaxisID, levelID);
+	      double missval = vlistInqVarMissval(vlistID, varID);
 
 	      cdiParamToString(param, paramstr, sizeof(paramstr));
 
 	      if ( operatorID == INFON ) vlistInqVarName(vlistID, varID, varname);
 
-	      fprintf(stdout, "%6d :%s %s ", indg, vdatestr, vtimestr);
+	      set_text_color(stdout, BRIGHT, BLACK);
+	      fprintf(stdout, "%6d ", indg);
+	      reset_text_color(stdout);
+	      set_text_color(stdout, RESET, BLACK);
+	      fprintf(stdout, ":");
+	      reset_text_color(stdout);
+	      
+              set_text_color(stdout, RESET, MAGENTA);
+	      fprintf(stdout, "%s %s ", vdatestr, vtimestr);
+	      reset_text_color(stdout);
 
-	      level = zaxisInqLevel(zaxisID, levelID);
+	      set_text_color(stdout, RESET, GREEN);
 	      fprintf(stdout, "%7g ", level);
+	      fprintf(stdout, "%8d %7d ", gridsize, nmiss);
+		
+	      set_text_color(stdout, RESET, BLACK);
+	      fprintf(stdout, ":");
+	      reset_text_color(stdout);
 
-	      fprintf(stdout, "%8d %7d :", gridsize, nmiss);
-
+              set_text_color(stdout, RESET, BLUE);
 	      if ( /* gridInqType(gridID) == GRID_SPECTRAL || */
 		   (gridsize == 1 && nmiss == 0 && number == CDI_REAL) )
 		{
+                  //fpeRaised = array_minmaxmean_val(gridsize, array, NULL, NULL, NULL);
 		  fprintf(stdout, "            %#12.5g            ", array[0]);
 		}
 	      else
@@ -330,49 +361,34 @@ void *Info(void *argument)
 			{
 			  ivals   = 0;
 			  arrmean = 0;
-			  arrvar  = 0;
+			  //arrvar  = 0;
 			  arrmin  =  1.e300;
 			  arrmax  = -1.e300;
-			  for ( i = 0; i < gridsize; ++i )
+			  for ( int i = 0; i < gridsize; ++i )
 			    {
 			      if ( !DBL_IS_EQUAL(array[i], missval) )
 				{
 				  if ( array[i] < arrmin ) arrmin = array[i];
 				  if ( array[i] > arrmax ) arrmax = array[i];
 				  arrmean += array[i];
-				  arrvar  += array[i]*array[i];
+				  //arrvar  += array[i]*array[i];
 				  ivals++;
 				}
 			    }
+                          fpeRaised = 0;
 			  imiss = gridsize - ivals;
 			  nvals = ivals;
+                          if ( nvals ) arrmean /= nvals;
 			}
 		      else
 			{
-			  arrmean = array[0];
-			  arrvar  = array[0];
-			  arrmin  = array[0];
-			  arrmax  = array[0];
-			  /*
-#pragma omp parallel for default(none) shared(arrmin, arrmax, array, gridsize)	\
-                                       reduction(+:arrmean, arrvar)
-			  */
-			  for ( i = 1; i < gridsize; i++ )
-			    {
-			      /* #pragma omp critical */
-			      if ( array[i] < arrmin ) arrmin = array[i];
-			      /* #pragma omp critical */
-			      if ( array[i] > arrmax ) arrmax = array[i];
-			      arrmean += array[i];
-			      arrvar  += array[i]*array[i];
-			    }
+                          fpeRaised = array_minmaxmean_val(gridsize, array, &arrmin, &arrmax, &arrmean);
 			  nvals = gridsize;
 			}
 
 		      if ( nvals )
 			{
-			  arrmean = arrmean/nvals;
-			  arrvar  = arrvar/nvals - arrmean*arrmean;
+			  // arrvar  = arrvar/nvals - arrmean*arrmean;
 			  fprintf(stdout, "%#12.5g%#12.5g%#12.5g", arrmin, arrmean, arrmax);
 			}
 		      else
@@ -387,7 +403,7 @@ void *Info(void *argument)
 		      arrsum_r = 0;
 		      arrsum_i = 0;
 		      
-		      for ( i = 0; i < gridsize; i++ )
+		      for ( int i = 0; i < gridsize; i++ )
 			{
 			  if ( !DBL_IS_EQUAL(array[i*2],   missval) && 
 			       !DBL_IS_EQUAL(array[i*2+1], missval) )
@@ -398,6 +414,7 @@ void *Info(void *argument)
 			      nvals_i++;
 			    }
 			}
+                      fpeRaised = 0;
 
 		      imiss = gridsize - nvals_r;
 
@@ -406,13 +423,20 @@ void *Info(void *argument)
 		      fprintf(stdout, "   -  (%#12.5g,%#12.5g)  -", arrmean_r, arrmean_i);
 		    }
 		}
+	      reset_text_color(stdout);
 
+	      set_text_color(stdout, RESET, BLACK);
+	      fprintf(stdout, " : ");
+	      reset_text_color(stdout);
+
+	      set_text_color(stdout, BRIGHT, GREEN);
 	      if ( operatorID == INFON )
-		fprintf(stdout, " : %-14s", varname);
+		fprintf(stdout, "%-14s", varname);
 	      else if ( operatorID == INFOC )
-		fprintf(stdout, " : %4d   ", code);
+		fprintf(stdout, "%4d   ", code);
 	      else
-		fprintf(stdout, " : %-14s", paramstr);
+		fprintf(stdout, "%-14s", paramstr);
+	      reset_text_color(stdout);
 
 	      if ( cdoVerbose )
 		{
@@ -424,14 +448,14 @@ void *Info(void *argument)
 	      fprintf(stdout, "\n");
 
 	      if ( imiss != nmiss && nmiss > 0 )
-		fprintf(stdout, "Found %d of %d missing values!\n", imiss, nmiss);
+		cdoPrint("Found %d of %d missing values!", imiss, nmiss);
+
+              if ( fpeRaised > 0 ) cdoWarning("floating-point exception reported: %s!", fpe_errstr(fpeRaised));
 
 	      if ( operatorID == MAP )
-		{
-		  int nlon, nlat;
-		  
-		  nlon = gridInqXsize(gridID);
-		  nlat = gridInqYsize(gridID);
+		{		  
+		  int nlon = gridInqXsize(gridID);
+		  int nlat = gridInqYsize(gridID);
 
 		  if ( gridInqType(gridID) == GRID_GAUSSIAN    ||
 		       gridInqType(gridID) == GRID_LONLAT      ||
@@ -445,12 +469,15 @@ void *Info(void *argument)
 	    }
 	  tsID++;
 	}
+
       streamClose(streamID);
 
-      if ( array ) free(array);
+      if ( array ) Free(array);
     }
 
+  dtlist_delete(dtlist);
+
   cdoFinish();
 
-  return (0);
+  return 0;
 }
