@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -31,85 +31,71 @@
 
 void *Intntime(void *argument)
 {
-  int streamID1, streamID2;
-  int nrecs, nvars, nlevel;
-  int i, nrecords;
-  int tsID, tsIDo, recID, varID, levelID;
-  int gridsize;
-  int vlistID1, vlistID2;
-  int taxisID1, taxisID2;
+  int nlevel;
+  int varID, levelID;
   int vdate, vtime;
-  int vdate1, vtime1;
-  int vdate2, vtime2;
   int offset;
-  int calendar;
-  int numts, it;
-  int *recVarID, *recLevelID;
-  int **nmiss1, **nmiss2, nmiss3;
-  double missval1, missval2;
-  juldate_t juldate1, juldate2, juldate;
-  double fac1, fac2;
-  double *array, *single1, *single2;
-  double **vardata1, **vardata2, *vardatap;
+  double *single1, *single2;
+  double *vardatap;
 
   cdoInitialize(argument);
 
   operatorInputArg("number of timesteps between 2 timesteps");
   if ( operatorArgc() < 1 ) cdoAbort("Too few arguments!");
 
-  numts = atoi(operatorArgv()[0]);
+  int numts = parameter2int(operatorArgv()[0]);
   if ( numts < 2 ) cdoAbort("parameter must be greater than 1!");
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  nvars    = vlistNvars(vlistID1);
-  nrecords = vlistNrecs(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = malloc(nrecords*sizeof(int));
-  recLevelID = malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
-  gridsize = vlistGridsizeMax(vlistID1);
-  array = malloc(gridsize*sizeof(double));
+  int gridsize = vlistGridsizeMax(vlistID1);
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
-  nmiss1   = malloc(nvars*sizeof(int *));
-  nmiss2   = malloc(nvars*sizeof(int *));
-  vardata1 = malloc(nvars*sizeof(double *));
-  vardata2 = malloc(nvars*sizeof(double *));
+  int **nmiss1   = (int **) Malloc(nvars*sizeof(int *));
+  int **nmiss2   = (int **) Malloc(nvars*sizeof(int *));
+  double **vardata1 = (double **) Malloc(nvars*sizeof(double *));
+  double **vardata2 = (double **) Malloc(nvars*sizeof(double *));
 
   for ( varID = 0; varID < nvars; varID++ )
     {
       gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
       nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      nmiss1[varID]   = malloc(nlevel*sizeof(int));
-      nmiss2[varID]   = malloc(nlevel*sizeof(int));
-      vardata1[varID] = malloc(gridsize*nlevel*sizeof(double));
-      vardata2[varID] = malloc(gridsize*nlevel*sizeof(double));
+      nmiss1[varID]   = (int*) Malloc(nlevel*sizeof(int));
+      nmiss2[varID]   = (int*) Malloc(nlevel*sizeof(int));
+      vardata1[varID] = (double*) Malloc(gridsize*nlevel*sizeof(double));
+      vardata2[varID] = (double*) Malloc(gridsize*nlevel*sizeof(double));
     }
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   if ( taxisHasBounds(taxisID2) ) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  calendar = taxisInqCalendar(taxisID1);
+  int calendar = taxisInqCalendar(taxisID1);
 
-  tsID = 0;
-  tsIDo = 0;
-  nrecs = streamInqTimestep(streamID1, tsID++);
-  vdate1 = taxisInqVdate(taxisID1);
-  vtime1 = taxisInqVtime(taxisID1);
-  juldate1 = juldate_encode(calendar, vdate1, vtime1);
+  int tsID = 0;
+  int tsIDo = 0;
+  int nrecs = streamInqTimestep(streamID1, tsID++);
+  int vdate1 = taxisInqVdate(taxisID1);
+  int vtime1 = taxisInqVtime(taxisID1);
+  juldate_t juldate1 = juldate_encode(calendar, vdate1, vtime1);
 
   taxisCopyTimestep(taxisID2, taxisID1);
   streamDefTimestep(streamID2, tsIDo++);
-  for ( recID = 0; recID < nrecs; recID++ )
+  for ( int recID = 0; recID < nrecs; recID++ )
     {
       streamInqRecord(streamID1, &varID, &levelID);
       gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
@@ -123,11 +109,11 @@ void *Intntime(void *argument)
 
   while ( (nrecs = streamInqTimestep(streamID1, tsID++)) )
     {
-      vdate2 = taxisInqVdate(taxisID1);
-      vtime2 = taxisInqVtime(taxisID1);
-      juldate2 = juldate_encode(calendar, vdate2, vtime2);
+      int vdate2 = taxisInqVdate(taxisID1);
+      int vtime2 = taxisInqVtime(taxisID1);
+      juldate_t juldate2 = juldate_encode(calendar, vdate2, vtime2);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -140,11 +126,10 @@ void *Intntime(void *argument)
 	  streamReadRecord(streamID1, single2, &nmiss2[varID][levelID]);
 	}
 
-      for ( it = 1; it < numts; it++ )
+      for ( int it = 1; it < numts; it++ )
 	{
-	  double seconds;
-	  seconds = it * juldate_to_seconds(juldate_sub(juldate2, juldate1)) / numts;
-	  juldate = juldate_add_seconds(NINT(seconds), juldate1);
+	  double seconds = it * juldate_to_seconds(juldate_sub(juldate2, juldate1)) / numts;
+	  juldate_t juldate = juldate_add_seconds((int)lround(seconds), juldate1);
 
 	  juldate_decode(calendar, juldate, &vdate, &vtime);
 
@@ -165,12 +150,12 @@ void *Intntime(void *argument)
 	  taxisDefVtime(taxisID2, vtime);
 	  streamDefTimestep(streamID2, tsIDo++);
 
-	  fac1 = juldate_to_seconds(juldate_sub(juldate2, juldate)) / 
-	         juldate_to_seconds(juldate_sub(juldate2, juldate1));
-	  fac2 = juldate_to_seconds(juldate_sub(juldate,  juldate1)) / 
-	         juldate_to_seconds(juldate_sub(juldate2, juldate1));
+	  double fac1 = juldate_to_seconds(juldate_sub(juldate2, juldate)) / 
+	                juldate_to_seconds(juldate_sub(juldate2, juldate1));
+	  double fac2 = juldate_to_seconds(juldate_sub(juldate,  juldate1)) / 
+	                juldate_to_seconds(juldate_sub(juldate2, juldate1));
 
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      varID    = recVarID[recID];
 	      levelID  = recLevelID[recID];
@@ -179,14 +164,14 @@ void *Intntime(void *argument)
 	      single1  = vardata1[varID] + offset;
 	      single2  = vardata2[varID] + offset;
 
-	      nmiss3 = 0;
+	      int nmiss3 = 0;
 
 	      if ( nmiss1[varID][levelID] > 0 || nmiss2[varID][levelID] > 0 )
 		{
-		  missval1 = vlistInqVarMissval(vlistID1, varID);
-		  missval2 = vlistInqVarMissval(vlistID2, varID);
+		  double missval1 = vlistInqVarMissval(vlistID1, varID);
+		  double missval2 = vlistInqVarMissval(vlistID2, varID);
 
-		  for ( i = 0; i < gridsize; i++ )
+		  for ( int i = 0; i < gridsize; i++ )
 		    {
 		      if ( !DBL_IS_EQUAL(single1[i], missval1) &&
 			   !DBL_IS_EQUAL(single2[i], missval2) )
@@ -206,7 +191,7 @@ void *Intntime(void *argument)
 		}
 	      else
 		{
-		  for ( i = 0; i < gridsize; i++ )
+		  for ( int i = 0; i < gridsize; i++ )
 		    array[i] = single1[i]*fac1 + single2[i]*fac2;
 		}
 
@@ -218,7 +203,7 @@ void *Intntime(void *argument)
       taxisDefVdate(taxisID2, vdate2);
       taxisDefVtime(taxisID2, vtime2);
       streamDefTimestep(streamID2, tsIDo++);
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  varID   = recVarID[recID];
 	  levelID = recLevelID[recID];
@@ -238,30 +223,30 @@ void *Intntime(void *argument)
 	  vardata2[varID] = vardatap;
 	}
 
-      vdate1 = vdate2;
-      vtime1 = vtime2;
+      //vdate1 = vdate2;
+      //vtime1 = vtime2;
       juldate1 = juldate2;
     }
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      free(nmiss1[varID]);
-      free(nmiss2[varID]);
-      free(vardata1[varID]);
-      free(vardata2[varID]);
+      Free(nmiss1[varID]);
+      Free(nmiss2[varID]);
+      Free(vardata1[varID]);
+      Free(vardata2[varID]);
     }
 
-  free(nmiss1);
-  free(nmiss2);
-  free(vardata1);
-  free(vardata2);
+  Free(nmiss1);
+  Free(nmiss2);
+  Free(vardata1);
+  Free(vardata2);
 
-  if ( array )  free(array);
+  if ( array )  Free(array);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
