@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -36,21 +36,13 @@
 static
 void invertLonDes(int vlistID)
 {
-  int index, ngrids;
-  int gridID1, gridID2;
-  int nlat, nlon, size;
-  int ilat, ilon;
-  int gridtype, nv, iv;
-  double *xv1, *xv2;
-  double *xb1, *xb2;
-
-  ngrids = vlistNgrids(vlistID);
-  for ( index = 0; index < ngrids; index++ )
+  int ngrids = vlistNgrids(vlistID);
+  for ( int index = 0; index < ngrids; index++ )
     {
-      gridID1 = vlistGrid(vlistID, index);
-      gridID2 = gridDuplicate(gridID1);
+      int gridID1 = vlistGrid(vlistID, index);
+      int gridID2 = gridDuplicate(gridID1);
 
-      gridtype = gridInqType(gridID1);
+      int gridtype = gridInqType(gridID1);
 
       if ( gridtype != GRID_GENERIC && gridtype != GRID_GAUSSIAN &&
 	   gridtype != GRID_LONLAT  && gridtype != GRID_CURVILINEAR )
@@ -58,64 +50,55 @@ void invertLonDes(int vlistID)
 
       if ( gridInqXvals(gridID1, NULL) )
 	{
-	  nlon  = gridInqXsize(gridID1);
-	  nlat  = gridInqYsize(gridID1);
+	  int nlon = gridInqXsize(gridID1);
+	  int nlat = gridInqYsize(gridID1);
+	  int size = (gridtype == GRID_CURVILINEAR) ? nlon*nlat : nlon;
 
-	  if ( gridtype == GRID_CURVILINEAR )
-	    size = nlon*nlat;
-	  else
-            size = nlon;
-
-	  xv1 = (double*) malloc(size*sizeof(double));
-	  xv2 = (double*) malloc(size*sizeof(double));
+	  double *xv1 = (double*) Malloc(size*sizeof(double));
+	  double *xv2 = (double*) Malloc(size*sizeof(double));
 
 	  gridInqXvals(gridID1, xv1);
 
 	  if ( gridtype == GRID_CURVILINEAR )
 	    {
-	      for ( ilat = 0; ilat < nlat; ilat++ )
-		for ( ilon = 0; ilon < nlon; ilon++ )
+	      for ( int ilat = 0; ilat < nlat; ilat++ )
+		for ( int ilon = 0; ilon < nlon; ilon++ )
 		  xv2[ilat*nlon + nlon-ilon-1] = xv1[ilat*nlon + ilon];
 	    }
 	  else
 	    {
-	      for ( ilon = 0; ilon < nlon; ilon++ )
+	      for ( int ilon = 0; ilon < nlon; ilon++ )
 		xv2[nlon-ilon-1] = xv1[ilon];
 	    }
 
 	  gridDefXvals(gridID2, xv2);
 
-	  if ( xv2 ) free(xv2);
-	  if ( xv1 ) free(xv1);
+	  if ( xv2 ) Free(xv2);
+	  if ( xv1 ) Free(xv1);
 	}
 
       if ( gridInqXbounds(gridID1, NULL) )
 	{
-	  nlon  = gridInqXsize(gridID1);
-	  nlat  = gridInqYsize(gridID1);
+	  int nlon = gridInqXsize(gridID1);
+	  int nlat = gridInqYsize(gridID1);
+	  int nv   = gridInqNvertex(gridID1);
+	  int size = (gridtype == GRID_CURVILINEAR) ? nv*nlon*nlat : nv*nlon;
 
-	  nv = gridInqNvertex(gridID1);
-
-	  if ( gridtype == GRID_CURVILINEAR )
-	    size = nv*nlon*nlat;
-	  else
-            size = nv*nlon;
-
-	  xb1 = (double*) malloc(size*sizeof(double));
-	  xb2 = (double*) malloc(size*sizeof(double));
+	  double *xb1 = (double*) Malloc(size*sizeof(double));
+	  double *xb2 = (double*) Malloc(size*sizeof(double));
 
 	  gridInqXbounds(gridID1, xb1);
 
 	  if ( gridtype == GRID_CURVILINEAR )
 	    {
-	      for ( ilat = 0; ilat < nlat; ilat++ )
-		for ( ilon = 0; ilon < nlon; ilon++ )
-		  for ( iv = 0; iv < nv; iv++ )
+	      for ( int ilat = 0; ilat < nlat; ilat++ )
+		for ( int ilon = 0; ilon < nlon; ilon++ )
+		  for ( int iv = 0; iv < nv; iv++ )
 		    xb2[ilat*nlon*nv + (nlon-ilon-1)*nv + iv] = xb1[ilat*nlon*nv + ilon*nv + iv];
 	    }
 	  else
 	    {
-		for ( ilon = 0; ilon < nlon; ilon++ )
+		for ( int ilon = 0; ilon < nlon; ilon++ )
 		  {
 		    xb2[nlon*2-ilon*2-1] = xb1[ilon*2];
 		    xb2[nlon*2-ilon*2-2] = xb1[ilon*2+1];
@@ -124,8 +107,8 @@ void invertLonDes(int vlistID)
 
 	  gridDefXbounds(gridID2, xb2);
 
-	  if ( xb2 ) free(xb2);
-	  if ( xb1 ) free(xb1);
+	  if ( xb2 ) Free(xb2);
+	  if ( xb1 ) Free(xb1);
 	}
 
       vlistChangeGrid(vlistID, gridID1, gridID2);
@@ -135,21 +118,13 @@ void invertLonDes(int vlistID)
 static
 void invertLatDes(int vlistID)
 {
-  int index, ngrids;
-  int gridID1, gridID2;
-  int nlat, nlon, size;
-  int ilat, ilon;
-  int gridtype, nv, iv;
-  double *yv1, *yv2;
-  double *yb1, *yb2;
-
-  ngrids = vlistNgrids(vlistID);
-  for ( index = 0; index < ngrids; index++ )
+  int ngrids = vlistNgrids(vlistID);
+  for ( int index = 0; index < ngrids; index++ )
     {
-      gridID1 = vlistGrid(vlistID, index);
-      gridID2 = gridDuplicate(gridID1);
+      int gridID1 = vlistGrid(vlistID, index);
+      int gridID2 = gridDuplicate(gridID1);
 
-      gridtype = gridInqType(gridID1);
+      int gridtype = gridInqType(gridID1);
 
       if ( gridtype != GRID_GENERIC && gridtype != GRID_GAUSSIAN &&
 	   gridtype != GRID_LONLAT  && gridtype != GRID_CURVILINEAR )
@@ -157,32 +132,28 @@ void invertLatDes(int vlistID)
 
       if ( gridInqYvals(gridID1, NULL) )
 	{
-	  nlon  = gridInqXsize(gridID1);
-	  nlat  = gridInqYsize(gridID1);
+	  int nlon = gridInqXsize(gridID1);
+	  int nlat = gridInqYsize(gridID1);
+	  int size = (gridtype == GRID_CURVILINEAR) ? nlon*nlat : nlat;
 
-	  if ( gridtype == GRID_CURVILINEAR )
-	    size = nlon*nlat;
-	  else
-            size = nlat;
-
-	  yv1 = (double*) malloc(size*sizeof(double));
-	  yv2 = (double*) malloc(size*sizeof(double));
+	  double *yv1 = (double*) Malloc(size*sizeof(double));
+	  double *yv2 = (double*) Malloc(size*sizeof(double));
 
 
 	  if ( gridtype == GRID_CURVILINEAR )
 	    {
 	      gridInqXvals(gridID1, yv1);
 
-	      for ( ilat = 0; ilat < nlat; ilat++ )
-		for ( ilon = 0; ilon < nlon; ilon++ )
+	      for ( int ilat = 0; ilat < nlat; ilat++ )
+		for ( int ilon = 0; ilon < nlon; ilon++ )
 		  yv2[(nlat-ilat-1)*nlon + ilon] = yv1[ilat*nlon + ilon];
 
 	      gridDefXvals(gridID2, yv2);
 
 	      gridInqYvals(gridID1, yv1);
 
-	      for ( ilat = 0; ilat < nlat; ilat++ )
-		for ( ilon = 0; ilon < nlon; ilon++ )
+	      for ( int ilat = 0; ilat < nlat; ilat++ )
+		for ( int ilon = 0; ilon < nlon; ilon++ )
 		  yv2[(nlat-ilat-1)*nlon + ilon] = yv1[ilat*nlon + ilon];
 
 	      gridDefYvals(gridID2, yv2);
@@ -191,43 +162,38 @@ void invertLatDes(int vlistID)
 	    {
 	      gridInqYvals(gridID1, yv1);
 
-	      for ( ilat = 0; ilat < nlat; ilat++ )
+	      for ( int ilat = 0; ilat < nlat; ilat++ )
 		yv2[nlat-ilat-1] = yv1[ilat];
 
 	      gridDefYvals(gridID2, yv2);
 	    }
 
-	  if ( yv2 ) free(yv2);
-	  if ( yv1 ) free(yv1);
+	  if ( yv2 ) Free(yv2);
+	  if ( yv1 ) Free(yv1);
 	}
 
       if ( gridInqYbounds(gridID1, NULL) )
 	{
-	  nlon  = gridInqXsize(gridID1);
-	  nlat  = gridInqYsize(gridID1);
+	  int nlon = gridInqXsize(gridID1);
+	  int nlat = gridInqYsize(gridID1);
+	  int nv   = gridInqNvertex(gridID1);
+	  int size = (gridtype == GRID_CURVILINEAR) ? nv*nlon*nlat : nv*nlat;
 
-	  nv = gridInqNvertex(gridID1);
-
-	  if ( gridtype == GRID_CURVILINEAR )
-	    size = nv*nlon*nlat;
-	  else
-            size = nv*nlat;
-
-	  yb1 = (double*) malloc(size*sizeof(double));
-	  yb2 = (double*) malloc(size*sizeof(double));
+	  double *yb1 = (double*) Malloc(size*sizeof(double));
+	  double *yb2 = (double*) Malloc(size*sizeof(double));
 
 	  gridInqYbounds(gridID1, yb1);
 
 	  if ( gridtype == GRID_CURVILINEAR )
 	    {
-	      for ( ilat = 0; ilat < nlat; ilat++ )
-		for ( ilon = 0; ilon < nlon; ilon++ )
-		  for ( iv = 0; iv < nv; iv++ )
+	      for ( int ilat = 0; ilat < nlat; ilat++ )
+		for ( int ilon = 0; ilon < nlon; ilon++ )
+		  for ( int iv = 0; iv < nv; iv++ )
 		    yb2[(nlat-ilat-1)*nlon*nv + ilon*nv + iv] = yb1[ilat*nlon*nv + ilon*nv + iv];
 	    }
 	  else
 	    {
-		for ( ilat = 0; ilat < nlat; ilat++ )
+		for ( int ilat = 0; ilat < nlat; ilat++ )
 		  {
 		    yb2[nlat*2-ilat*2-1] = yb1[ilat*2];
 		    yb2[nlat*2-ilat*2-2] = yb1[ilat*2+1];
@@ -236,8 +202,8 @@ void invertLatDes(int vlistID)
 
 	  gridDefYbounds(gridID2, yb2);
 
-	  if ( yb2 ) free(yb2);
-	  if ( yb1 ) free(yb1);
+	  if ( yb2 ) Free(yb2);
+	  if ( yb1 ) Free(yb1);
 	}
 
       vlistChangeGrid(vlistID, gridID1, gridID2);
@@ -247,30 +213,26 @@ void invertLatDes(int vlistID)
 static
 void invertLonData(double *array1, double *array2, int gridID1)
 {
-  int nlat, nlon;
-  int ilat, ilon;
-  double **field1, **field2;
-
-  nlon = gridInqXsize(gridID1);
-  nlat = gridInqYsize(gridID1);
+  int nlon = gridInqXsize(gridID1);
+  int nlat = gridInqYsize(gridID1);
 
   if ( nlat > 0 )
     {
-      field1 = (double **) malloc(nlat*sizeof(double *));
-      field2 = (double **) malloc(nlat*sizeof(double *));
+      double **field1 = (double **) Malloc(nlat*sizeof(double *));
+      double **field2 = (double **) Malloc(nlat*sizeof(double *));
   
-      for ( ilat = 0; ilat < nlat; ilat++ )
+      for ( int ilat = 0; ilat < nlat; ilat++ )
 	{
 	  field1[ilat] = array1 + ilat*nlon;
 	  field2[ilat] = array2 + ilat*nlon;
 	}
 
-      for ( ilat = 0; ilat < nlat; ilat++ )
-	for ( ilon = 0; ilon < nlon; ilon++ )
+      for ( int ilat = 0; ilat < nlat; ilat++ )
+	for ( int ilon = 0; ilon < nlon; ilon++ )
 	  field2[ilat][nlon-ilon-1] = field1[ilat][ilon];
   
-      if ( field1 ) free(field1);
-      if ( field2 ) free(field2);
+      if ( field1 ) Free(field1);
+      if ( field2 ) Free(field2);
     }
   else
     {
@@ -281,29 +243,25 @@ void invertLonData(double *array1, double *array2, int gridID1)
 static
 void invertLatData(double *array1, double *array2, int gridID1)
 {
-  int nlat, nlon;
-  int ilat;
-  double **field1, **field2;
-
-  nlon = gridInqXsize(gridID1);
-  nlat = gridInqYsize(gridID1);
+  int nlon = gridInqXsize(gridID1);
+  int nlat = gridInqYsize(gridID1);
 
   if ( nlat > 0 )
     {
-      field1 = (double **) malloc(nlat*sizeof(double *));
-      field2 = (double **) malloc(nlat*sizeof(double *));
+      double **field1 = (double **) Malloc(nlat*sizeof(double *));
+      double **field2 = (double **) Malloc(nlat*sizeof(double *));
   
-      for ( ilat = 0; ilat < nlat; ilat++ )
+      for ( int ilat = 0; ilat < nlat; ilat++ )
 	{
 	  field1[ilat] = array1 + ilat*nlon;
 	  field2[ilat] = array2 + ilat*nlon;
 	}
 
-      for ( ilat = 0; ilat < nlat; ilat++ )
+      for ( int ilat = 0; ilat < nlat; ilat++ )
 	memcpy(field2[nlat-ilat-1], field1[ilat], nlon*sizeof(double));
       
-      if ( field1 ) free(field1);
-      if ( field2 ) free(field2);
+      if ( field1 ) Free(field1);
+      if ( field2 ) Free(field2);
     }
   else
     {
@@ -315,7 +273,7 @@ void invertLatData(double *array1, double *array2, int gridID1)
 void *Invert(void *argument)
 {
   int nrecs;
-  int recID, varID, levelID;
+  int varID, levelID;
   int gridID1;
   int nmiss;
 
@@ -355,8 +313,8 @@ void *Invert(void *argument)
 
   int gridsize = vlistGridsizeMax(vlistID1);
 
-  double *array1 = (double*) malloc(gridsize*sizeof(double));
-  double *array2 = (double*) malloc(gridsize*sizeof(double));
+  double *array1 = (double*) Malloc(gridsize*sizeof(double));
+  double *array2 = (double*) Malloc(gridsize*sizeof(double));
 
   int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
@@ -365,7 +323,7 @@ void *Invert(void *argument)
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamReadRecord(streamID1, array1, &nmiss);
@@ -394,10 +352,10 @@ void *Invert(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array1 ) free(array1);
-  if ( array2 ) free(array2);
+  if ( array1 ) Free(array1);
+  if ( array2 ) Free(array2);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
