@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -41,7 +41,7 @@ void invertLevDes(int vlistID)
       int nlev = zaxisInqSize(zaxisID1);
       if ( nlev <= 1 ) continue;
 
-      /* if ( zaxisInqLevels(zaxisID1, NULL) ) */
+      if ( zaxisInqLevels(zaxisID1, NULL) )
 	{
 	  double yv1[nlev], yv2[nlev];
 	  zaxisInqLevels(zaxisID1, yv1);
@@ -86,7 +86,7 @@ void invertLevDes(int vlistID)
 void *Invertlev(void *argument)
 {
   int nrecs;
-  int recID, varID, levelID;
+  int varID, levelID;
   int nmiss;
   int nlev, nlevel;
   int gridID, zaxisID, offset;
@@ -154,7 +154,7 @@ void *Invertlev(void *argument)
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
