@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -38,14 +38,14 @@ void mastrfu(int gridID, int zaxisID, double *array1, double *array2, int nmiss,
 
   int nlat = gridInqSize(gridID);
   int nlev = zaxisInqSize(zaxisID);
-  double *phi    = (double*) malloc(nlat*sizeof(double));
-  double *dummy  = (double*) malloc(nlat*sizeof(double));
-  double *cosphi = (double*) malloc(nlat*sizeof(double));
-  double *plevel = (double*) malloc(nlev*sizeof(double));
-  double **field1 = (double**) malloc(nlev*sizeof(double*));
-  double **field2 = (double**) malloc(nlev*sizeof(double*));
+  double *phi    = (double*) Malloc(nlat*sizeof(double));
+  double *dummy  = (double*) Malloc(nlat*sizeof(double));
+  double *cosphi = (double*) Malloc(nlat*sizeof(double));
+  double *plevel = (double*) Malloc(nlev*sizeof(double));
+  double **field1 = (double**) Malloc(nlev*sizeof(double*));
+  double **field2 = (double**) Malloc(nlev*sizeof(double*));
 
-  zaxisInqLevels(zaxisID, plevel);
+  cdoZaxisInqLevels(zaxisID, plevel);
 
   // gaussaw(phi, dummy, nlat);
   
@@ -95,19 +95,19 @@ void mastrfu(int gridID, int zaxisID, double *array1, double *array2, int nmiss,
 	    }
     }
 
-  free(field2);
-  free(field1);
-  free(plevel);
-  free(cosphi);
-  free(dummy);
-  free(phi);
+  Free(field2);
+  Free(field1);
+  Free(plevel);
+  Free(cosphi);
+  Free(dummy);
+  Free(phi);
 }
 
 
 void *Mastrfu(void *argument)
 {
   int nrecs;
-  int recID, varID, levelID;
+  int varID, levelID;
   int offset;
   int nmiss, nmiss1;
 
@@ -150,14 +150,14 @@ void *Mastrfu(void *argument)
   vlistDefVarName(vlistID2, 0, "mastrfu");
   vlistDefVarLongname(vlistID2, 0, "mass stream function");
   vlistDefVarUnits(vlistID2, 0, "kg/s");
-  vlistDefVarDatatype(vlistID2, 0, DATATYPE_FLT32);
+  vlistDefVarDatatype(vlistID2, 0, CDI_DATATYPE_FLT32);
 
   int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  double *array1 = (double*) malloc(gridsize*nlev*sizeof(double));
-  double *array2 = (double*) malloc(gridsize*nlev*sizeof(double));
+  double *array1 = (double*) Malloc(gridsize*nlev*sizeof(double));
+  double *array2 = (double*) Malloc(gridsize*nlev*sizeof(double));
 
   int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
@@ -167,7 +167,7 @@ void *Mastrfu(void *argument)
       streamDefTimestep(streamID2, tsID);
 
       nmiss = 0;
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  offset  = gridsize*levelID;
@@ -177,7 +177,7 @@ void *Mastrfu(void *argument)
 
       mastrfu(gridID, zaxisID, array1, array2, nmiss, missval);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  varID = 0;
 	  levelID = recID;
@@ -191,10 +191,10 @@ void *Mastrfu(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array1 ) free(array1);
-  if ( array2 ) free(array2);
+  if ( array1 ) Free(array1);
+  if ( array2 ) Free(array2);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
