@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,47 +34,39 @@ void checkDupEntry(int vlistID1, int vlistID2, const char *filename)
 {
   char vname1[CDI_MAX_NAME], vname2[CDI_MAX_NAME];
   int k;
-  int gridID1, gridID2;
-  int zaxisID1, zaxisID2;
-  int varID1, varID2;
-  int param1, param2;
-  int ztype1, ztype2;
-  int gtype1, gtype2;
-  int nlev1,  nlev2;
-  int gsize1, gsize2;
   int mlev1 = 0, mlev2 = 0;
   double *lev1 = NULL, *lev2 = NULL;
 
   int nvars1 = vlistNvars(vlistID1);
   int nvars2 = vlistNvars(vlistID2);
 
-  for ( varID1 = 0; varID1 < nvars1; ++varID1 )
+  for ( int varID1 = 0; varID1 < nvars1; ++varID1 )
     {
       vlistInqVarName(vlistID1, varID1, vname1);
-      param1   = vlistInqVarParam(vlistID1, varID1);
-      gridID1  = vlistInqVarGrid(vlistID1, varID1);
-      zaxisID1 = vlistInqVarZaxis(vlistID1, varID1);
-      gtype1   = gridInqType(gridID1);
-      gsize1   = gridInqSize(gridID1);
-      ztype1   = zaxisInqType(zaxisID1);
-      nlev1    = zaxisInqSize(zaxisID1);
+      int param1   = vlistInqVarParam(vlistID1, varID1);
+      int gridID1  = vlistInqVarGrid(vlistID1, varID1);
+      int zaxisID1 = vlistInqVarZaxis(vlistID1, varID1);
+      int gtype1   = gridInqType(gridID1);
+      int gsize1   = gridInqSize(gridID1);
+      int ztype1   = zaxisInqType(zaxisID1);
+      int nlev1    = zaxisInqSize(zaxisID1);
       if ( nlev1 > mlev1 )
 	{
 	  mlev1 = nlev1;
 	  lev1 = (double*) Realloc(lev1, mlev1*sizeof(double));
 	}
-      zaxisInqLevels(zaxisID1, lev1);
+      cdoZaxisInqLevels(zaxisID1, lev1);
 
-      for ( varID2 = 0; varID2 < nvars2; ++varID2 )
+      for ( int varID2 = 0; varID2 < nvars2; ++varID2 )
 	{
 	  vlistInqVarName(vlistID2, varID2, vname2);
-	  param2   = vlistInqVarParam(vlistID2, varID2);
-	  gridID2  = vlistInqVarGrid(vlistID2, varID2);
-	  zaxisID2 = vlistInqVarZaxis(vlistID2, varID2);
-	  gtype2   = gridInqType(gridID2);
-	  gsize2   = gridInqSize(gridID2);
-	  ztype2   = zaxisInqType(zaxisID2);
-	  nlev2    = zaxisInqSize(zaxisID2);
+	  int param2   = vlistInqVarParam(vlistID2, varID2);
+	  int gridID2  = vlistInqVarGrid(vlistID2, varID2);
+	  int zaxisID2 = vlistInqVarZaxis(vlistID2, varID2);
+	  int gtype2   = gridInqType(gridID2);
+	  int gsize2   = gridInqSize(gridID2);
+	  int ztype2   = zaxisInqType(zaxisID2);
+	  int nlev2    = zaxisInqSize(zaxisID2);
 	  if ( gtype1 == gtype2 && gsize1 == gsize2 && ztype1 == ztype2 && nlev1 == nlev2 )
 	    {
 	      if ( nlev2 > mlev2 )
@@ -82,29 +74,30 @@ void checkDupEntry(int vlistID1, int vlistID2, const char *filename)
 		  mlev2 = nlev2;
 		  lev2 = (double*) Realloc(lev2, mlev2*sizeof(double));
 		}
-	      zaxisInqLevels(zaxisID2, lev2);
+	      cdoZaxisInqLevels(zaxisID2, lev2);
 
-	      for ( k = 0; k < nlev2; ++k )
-		if ( !IS_EQUAL(lev1[k], lev2[k]) ) break;
+              if ( zaxisInqLevels(zaxisID1, NULL) && zaxisInqLevels(zaxisID2, NULL) )
+                {
+                  for ( k = 0; k < nlev2; ++k )
+                    if ( !IS_EQUAL(lev1[k], lev2[k]) ) break;
 
-	      if ( k == nlev2 )
-		{
-		  if ( param1 < 0 || param2 < 0 )
-		    {
-		      if ( strcmp(vname1, vname2) == 0 )
-			{
-			  cdoWarning("Duplicate entry of parameter %s in %s!", vname2, filename);
-			}
-		    }
-		  else
-		    {
-		      if ( param1 == param2 )
-			{
-			  char paramstr[32];
-			  cdiParamToString(param2, paramstr, sizeof(paramstr));
-			  cdoWarning("Duplicate entry of parameter %s in %s!", paramstr, filename);
-			}
-		    }
+                  if ( k == nlev2 )
+                    {
+                      if ( param1 < 0 || param2 < 0 )
+                        {
+                          if ( strcmp(vname1, vname2) == 0 )
+                            cdoWarning("Duplicate entry of parameter %s in %s!", vname2, filename);
+                        }
+                      else
+                        {
+                          if ( param1 == param2 )
+                            {
+                              char paramstr[32];
+                              cdiParamToString(param2, paramstr, sizeof(paramstr));
+                              cdoWarning("Duplicate entry of parameter %s in %s!", paramstr, filename);
+                            }
+                        }
+                    }
 		}
 	    }
 	}
@@ -122,7 +115,7 @@ int vlistConstVars(int vlistID)
   for ( int varID = 0; varID < nvars; ++varID )
     if ( vlistInqVarTsteptype(vlistID, varID) != TSTEP_CONSTANT ) return 0;
 
-  return (1);
+  return 1;
 }
 */
 
@@ -131,44 +124,22 @@ void *Merge(void *argument)
   int streamID1 = -1;
   int varID, varID2;
   int nrecs = 0;
-  int recID, levelID, levelID2;
+  int levelID, levelID2;
   int index;
-  int lcopy = FALSE;
   int gridsize;
   int nmiss;
-  //int skip_same_var = FALSE;
 
   cdoInitialize(argument);
 
-  if ( UNCHANGED_RECORD ) lcopy = TRUE;
-
-  /*
-  {
-    char *envstr;
-    envstr = getenv("SKIP_SAME_VAR");
-    if ( envstr )
-      {
-	int ival;
-	ival = atoi(envstr);
-	if ( ival == 1 )
-	  {
-	    skip_same_var = TRUE;
-	    if ( cdoVerbose )
-	      cdoPrint("Set SKIP_SAME_VAR to %d", ival);
-	  }
-      }
-  }
-  */
+  bool lcopy = UNCHANGED_RECORD;
 
   int streamCnt = cdoStreamCnt();
   int nmerge    = streamCnt - 1;
 
   const char *ofilename = cdoStreamName(streamCnt-1)->args;
 
-  if ( !cdoSilentMode && !cdoOverwriteMode )
-    if ( fileExists(ofilename) )
-      if ( !userFileOverwrite(ofilename) )
-	cdoAbort("Outputfile %s already exists!", ofilename);
+  if ( !cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename) )
+    cdoAbort("Outputfile %s already exists!", ofilename);
 
   int *streamIDs = (int*) Malloc(nmerge*sizeof(int));
   int *vlistIDs  = (int*) Malloc(nmerge*sizeof(int));
@@ -313,7 +284,7 @@ void *Merge(void *argument)
 
 	  if ( vlistID1 == -1 ) continue;
 
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      streamInqRecord(streamID1, &varID, &levelID);
 
