@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,9 @@
       Merstat    mermean         Meridional mean
       Merstat    meravg          Meridional average
       Merstat    merstd          Meridional standard deviation
+      Merstat    merstd          Meridional standard deviation [Normalize by (n-1)]
       Merstat    mervar          Meridional variance
+      Merstat    mervar          Meridional variance [Normalize by (n-1)]
       Merstat    merpctl         Meridional percentiles
 */
 
@@ -38,25 +40,14 @@
 
 void *Merstat(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
   int gridID1, gridID2 = -1, lastgrid = -1;
   int wstatus = FALSE;
-  int code = 0, oldcode = 0;
-  int nlonmax;
-  int index, ngrids;
-  int recID, nrecs;
-  int tsID, varID, levelID;
-  int lim;
-  int ndiffgrids;
-  int taxisID1, taxisID2;
-  int needWeights = FALSE;
-  field_t field1, field2;
-  /* RQ */
-  int pn = 0;
-  /* QR */
+  int index;
+  int nmiss;
+  int nrecs;
+  int varID, levelID;
+  bool needWeights = false;
+  char varname[CDI_MAX_NAME];
 
   cdoInitialize(argument);
 
@@ -66,40 +57,38 @@ void *Merstat(void *argument)
   cdoOperatorAdd("mermean", func_mean, 0, NULL);
   cdoOperatorAdd("meravg",  func_avg,  0, NULL);
   cdoOperatorAdd("mervar",  func_var,  0, NULL);
+  cdoOperatorAdd("mervar1", func_var1, 0, NULL);
   cdoOperatorAdd("merstd",  func_std,  0, NULL);
-  /* RQ */
+  cdoOperatorAdd("merstd1", func_std1, 0, NULL);
   cdoOperatorAdd("merpctl", func_pctl, 0, NULL);
-  /* QR */
  
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
 
-  /* RQ */
+  double pn = 0;
   if ( operfunc == func_pctl )
     {
       operatorInputArg("percentile number");
-      pn = atoi(operatorArgv()[0]);
-      
-      if ( pn < 1 || pn > 99 )
-        cdoAbort("Illegal argument: percentile number %d is not in the range 1..99!", pn);
+      pn = parameter2double(operatorArgv()[0]);
+      percentile_check_number(pn);
     }
-  /* QR */
 
   if ( operfunc == func_mean || operfunc == func_avg ||
-       operfunc == func_var  || operfunc == func_std )
-    needWeights = TRUE;
+       operfunc == func_var  || operfunc == func_std ||
+       operfunc == func_var1 || operfunc == func_std1 )
+    needWeights = true;
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  ngrids = vlistNgrids(vlistID1);
-  ndiffgrids = 0;
+  int ngrids = vlistNgrids(vlistID1);
+  int ndiffgrids = 0;
   for ( index = 1; index < ngrids; index++ )
     if ( vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index))
       ndiffgrids++;
@@ -122,59 +111,59 @@ void *Merstat(void *argument)
 
   vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
   gridID1 = vlistInqVarGrid(vlistID1, 0);
-  nlonmax = gridInqXsize(gridID1); /* max nlon ? */
+  int nlonmax = gridInqXsize(gridID1); /* max nlon ? */
+  int lim = vlistGridsizeMax(vlistID1);
 
+  field_type field1, field2;
   field_init(&field1);
   field_init(&field2);
 
-  lim = vlistGridsizeMax(vlistID1);
-  field1.ptr    = malloc(lim*sizeof(double));
+  field1.ptr    = (double*) Malloc(lim*sizeof(double));
   field1.weight = NULL;
   if ( needWeights )
-    field1.weight = malloc(lim*sizeof(double));
+    field1.weight = (double*) Malloc(lim*sizeof(double));
 
-  field2.ptr  = malloc(nlonmax*sizeof(double));
+  field2.ptr  = (double*) Malloc(nlonmax*sizeof(double));
   field2.grid = gridID2;
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
-	  streamReadRecord(streamID1, field1.ptr, &field1.nmiss);
-
-	  field1.grid    = vlistInqVarGrid(vlistID1, varID);
+	  streamReadRecord(streamID1, field1.ptr, &nmiss);
+          field1.nmiss = (size_t) nmiss;
+	  field1.grid = vlistInqVarGrid(vlistID1, varID);
 	  if ( needWeights && field1.grid != lastgrid )
 	    {
 	      lastgrid = field1.grid;
 	      wstatus = gridWeights(field1.grid, field1.weight);
 	    }
-	  code = vlistInqVarCode(vlistID1, varID);
-	  if ( wstatus != 0 && tsID == 0 && code != oldcode )
-	    cdoWarning("Using constant area weights for code %d!", oldcode=code);
-
+	  if ( wstatus != 0 && tsID == 0 && levelID == 0 )
+	    {
+	      vlistInqVarName(vlistID1, varID, varname);
+	      cdoWarning("Using constant grid cell area weights for variable %s!", varname);
+	    }
 	  field1.missval = vlistInqVarMissval(vlistID1, varID);
 	  field2.missval = vlistInqVarMissval(vlistID1, varID);
 
-	  /* RQ */
 	  if ( operfunc == func_pctl )
 	    merpctl(field1, & field2, pn);
 	  else  
 	    merfun(field1, &field2, operfunc);
-	  /* QR */  
 
 	  streamDefRecord(streamID2, varID,  levelID);
-	  streamWriteRecord(streamID2, field2.ptr, field2.nmiss);
+	  streamWriteRecord(streamID2, field2.ptr, (int)field2.nmiss);
 	}
       tsID++;
     }
@@ -182,11 +171,11 @@ void *Merstat(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( field1.ptr )    free(field1.ptr);
-  if ( field1.weight ) free(field1.weight);
-  if ( field2.ptr )    free(field2.ptr);
+  if ( field1.ptr )    Free(field1.ptr);
+  if ( field1.weight ) Free(field1.weight);
+  if ( field2.ptr )    Free(field2.ptr);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
