@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2013 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,8 @@
       Ninfo      nmon            Number of months
       Ninfo      ndate           Number of dates
       Ninfo      ntime           Number of timesteps
+      Ninfo      ngridpoints     Number of gridpoints
+      Ninfo      ngrids          Number of grids
 */
 
 
@@ -35,49 +37,45 @@
 
 void *Ninfo(void *argument)
 {
-  enum {NYEAR, NMON, NDATE, NTIME, NPAR, NLEVEL};
-  int operatorID;
-  int operfunc;
-  int varID, zaxisID;
-  int vdate;
-  int nrecs, nvars, ntsteps;
-  int levelsize;
-  int tsID, ndate, date0 = 0;
-  int day, mon0 = 0, mon, nmon, year0 = 0, year, nyear;
-  int taxisID;
-  int streamID;
-  int vlistID;
+  enum {NYEAR, NMON, NDATE, NTIME, NPAR, NLEVEL, NGRIDPOINTS, NGRIDS};
+  int varID;
+  int nrecs;
+  int date0 = 0;
+  int day, mon0 = 0, mon, year0 = 0, year;
 
   cdoInitialize(argument);
 
-  cdoOperatorAdd("nyear",  NYEAR,  0, NULL);
-  cdoOperatorAdd("nmon",   NMON,   0, NULL);
-  cdoOperatorAdd("ndate",  NDATE,  0, NULL);
-  cdoOperatorAdd("ntime",  NTIME,  0, NULL);
-  cdoOperatorAdd("npar",   NPAR,   0, NULL);
-  cdoOperatorAdd("nlevel", NLEVEL, 0, NULL);
+  cdoOperatorAdd("nyear"       , NYEAR       , 0 , NULL);
+  cdoOperatorAdd("nmon"        , NMON        , 0 , NULL);
+  cdoOperatorAdd("ndate"       , NDATE       , 0 , NULL);
+  cdoOperatorAdd("ntime"       , NTIME       , 0 , NULL);
+  cdoOperatorAdd("npar"        , NPAR        , 0 , NULL);
+  cdoOperatorAdd("nlevel"      , NLEVEL      , 0 , NULL);
+  cdoOperatorAdd("ngridpoints" , NGRIDPOINTS , 0 , NULL);
+  cdoOperatorAdd("ngrids"      , NGRIDS      , 0 , NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc   = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc   = cdoOperatorF1(operatorID);
 
-  streamID = streamOpenRead(cdoStreamName(0));
+  int streamID = streamOpenRead(cdoStreamName(0));
 
-  vlistID = streamInqVlist(streamID);
+  int vlistID = streamInqVlist(streamID);
 
-  nvars   = vlistNvars(vlistID);
-  taxisID = vlistInqTaxis(vlistID);
-  ntsteps = vlistNtsteps(vlistID);
+  int nvars   = vlistNvars(vlistID);
+  int taxisID = vlistInqTaxis(vlistID);
+  int ntsteps = vlistNtsteps(vlistID);
+  int ngrids  = vlistNgrids(vlistID);
 
   switch ( operfunc )
     {
     case NYEAR:
-      nyear = 0;
-      tsID = 0;
+      {
+      int nyear = 0;
+      int tsID = 0;
       if ( ntsteps != 0 )
 	while ( (nrecs = streamInqTimestep(streamID, tsID)) )
 	  {
-	    vdate = taxisInqVdate(taxisID);
-
+	    int vdate = taxisInqVdate(taxisID);
 	    cdiDecodeDate(vdate, &year, &mon, &day);
 	 
 	    if ( tsID == 0 || year0 != year )
@@ -90,14 +88,15 @@ void *Ninfo(void *argument)
 	  }
       fprintf(stdout, "%d\n", nyear);
       break;
+      }
     case NMON:
-      nmon = 0;
-      tsID = 0;
+      {
+      int nmon = 0;
+      int tsID = 0;
       if ( ntsteps != 0 )
 	while ( (nrecs = streamInqTimestep(streamID, tsID)) )
 	  {
-	    vdate = taxisInqVdate(taxisID);
-	    
+	    int vdate = taxisInqVdate(taxisID);
 	    cdiDecodeDate(vdate, &year, &mon, &day);
 	 
 	    if ( tsID == 0 || mon0 != mon )
@@ -110,13 +109,15 @@ void *Ninfo(void *argument)
 	  }
       fprintf(stdout, "%d\n", nmon);
       break;
+      }
     case NDATE:
-      ndate = 0;
-      tsID = 0;
+      {
+      int ndate = 0;
+      int tsID = 0;
       if ( ntsteps != 0 )
 	while ( (nrecs = streamInqTimestep(streamID, tsID)) )
 	  {
-	    vdate = taxisInqVdate(taxisID);
+	    int vdate = taxisInqVdate(taxisID);
 	    
 	    if ( tsID == 0 || date0 != vdate )
 	      {
@@ -128,23 +129,37 @@ void *Ninfo(void *argument)
 	  }
       fprintf(stdout, "%d\n", ndate);
       break;
+      }
     case NTIME:
-      tsID = 0;
+      {
+      int tsID = 0;
       if ( ntsteps != 0 )
 	while ( (nrecs = streamInqTimestep(streamID, tsID)) ) tsID++;
       fprintf(stdout, "%d\n", tsID);
       break;
+      }
     case NPAR:
       fprintf(stdout, "%d\n", nvars);
       break;
     case NLEVEL:
       for ( varID = 0; varID < nvars; varID++ )
 	{
-	  zaxisID = vlistInqVarZaxis(vlistID, varID);
-	  levelsize = zaxisInqSize(zaxisID);
+	  int zaxisID = vlistInqVarZaxis(vlistID, varID);
+	  int levelsize = zaxisInqSize(zaxisID);
 	  fprintf(stdout, "%d\n", levelsize);
 	}
       break;
+    case NGRIDPOINTS:
+      for ( varID = 0; varID < nvars; varID++ )
+	{
+	  int gridID = vlistInqVarGrid(vlistID, varID);
+	  int gridsize = gridInqSize(gridID);
+	  fprintf(stdout, "%d\n", gridsize);
+	}
+      break;
+    case NGRIDS:
+      fprintf(stdout, "%d\n", ngrids);
+      break;
     default:
       cdoAbort("operator not implemented!");
       break;
@@ -154,5 +169,5 @@ void *Ninfo(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
