@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -158,7 +158,7 @@ int maptype2operfunc(int map_type, int submap_type, int num_neighbors, int remap
   else
     cdoAbort("Unsupported mapping method (map_type = %d)", map_type);
 
-  return (operfunc);
+  return operfunc;
 } 
 
 static
@@ -186,22 +186,22 @@ void print_remap_info(int operfunc, int remap_genweights, remapgrid_t *src_grid,
 
   strcat(line, gridNamePtr(gridInqType(src_grid->gridID)));
   if ( src_grid->rank == 2 )
-    sprintf(tmpstr, " (%dx%d)", src_grid->dims[0], src_grid->dims[1]);
+    snprintf(tmpstr, sizeof(tmpstr), " (%dx%d)", src_grid->dims[0], src_grid->dims[1]);
   else
-    sprintf(tmpstr, " (%d)", src_grid->dims[0]);
+    snprintf(tmpstr, sizeof(tmpstr), " (%d)", src_grid->dims[0]);
   strcat(line, tmpstr);
   strcat(line, " to ");
   strcat(line, gridNamePtr(gridInqType(tgt_grid->gridID)));
   if ( tgt_grid->rank == 2 )
-    sprintf(tmpstr, " (%dx%d)", tgt_grid->dims[0], tgt_grid->dims[1]);
+    snprintf(tmpstr, sizeof(tmpstr), " (%dx%d)", tgt_grid->dims[0], tgt_grid->dims[1]);
   else
-    sprintf(tmpstr, " (%d)", tgt_grid->dims[0]);
+    snprintf(tmpstr, sizeof(tmpstr), " (%d)", tgt_grid->dims[0]);
   strcat(line, tmpstr);
   strcat(line, " grid");
 
   if ( nmiss > 0 )
     {
-      sprintf(tmpstr, ", with source mask (%d)", gridInqSize(src_grid->gridID)-nmiss);
+      snprintf(tmpstr, sizeof(tmpstr), ", with source mask (%d)", gridInqSize(src_grid->gridID)-nmiss);
       strcat(line, tmpstr);
     }
 
@@ -215,6 +215,7 @@ void print_remap_warning(const char *remap_file, int operfunc, remapgrid_t *src_
   char tmpstr[256];
 
   line[0] = 0;
+  (void)operfunc;
   /*
   if      ( operfunc == REMAPBIL  || operfunc == GENBIL  )  strcpy(line, "SCRIP bilinear");
   else if ( operfunc == REMAPBIC  || operfunc == GENBIC  )  strcpy(line, "SCRIP bicubic");
@@ -233,15 +234,15 @@ void print_remap_warning(const char *remap_file, int operfunc, remapgrid_t *src_
   strcat(line, " not used, ");
   strcat(line, gridNamePtr(gridInqType(src_grid->gridID)));
   if ( src_grid->rank == 2 )
-    sprintf(tmpstr, " (%dx%d)", src_grid->dims[0], src_grid->dims[1]);
+    snprintf(tmpstr, sizeof(tmpstr), " (%dx%d)", src_grid->dims[0], src_grid->dims[1]);
   else
-    sprintf(tmpstr, " (%d)", src_grid->dims[0]);
+    snprintf(tmpstr, sizeof(tmpstr), " (%d)", src_grid->dims[0]);
   strcat(line, tmpstr);
   strcat(line, " grid");
 
   if ( nmiss > 0 )
     {
-      sprintf(tmpstr, " with mask (%d)", gridInqSize(src_grid->gridID)-nmiss);
+      snprintf(tmpstr, sizeof(tmpstr), " with mask (%d)", gridInqSize(src_grid->gridID)-nmiss);
       strcat(line, tmpstr);
     }
 
@@ -257,9 +258,9 @@ int remap_test = 0;
 int remap_order = 1;
 int remap_non_global = FALSE;
 int remap_num_srch_bins = 180;
-int lremap_num_srch_bins = FALSE;
-int remap_extrapolate = FALSE;
-int lextrapolate = FALSE;
+static bool lremap_num_srch_bins = false;
+static bool remap_extrapolate = false;
+static bool lextrapolate = false;
 int max_remaps = -1;
 int sort_mode = HEAP_SORT;
 double remap_frac_min = 0;
@@ -273,8 +274,7 @@ void get_remap_env(void)
   envstr = getenv("MAX_REMAPS");
   if ( envstr )
     {
-      int ival;
-      ival = atoi(envstr);
+      int ival = atoi(envstr);
       if ( ival > 0 )
 	{
 	  max_remaps = ival;
@@ -286,8 +286,7 @@ void get_remap_env(void)
   envstr = getenv("REMAP_MAX_ITER");
   if ( envstr )
     {
-      int ival;
-      ival = atoi(envstr);
+      int ival = atoi(envstr);
       if ( ival > 0 )
 	{
 	  remap_set_int(REMAP_MAX_ITER, ival);
@@ -299,8 +298,7 @@ void get_remap_env(void)
   envstr = getenv("REMAP_ORDER");
   if ( envstr )
     {
-      int ival;
-      ival = atoi(envstr);
+      int ival = atoi(envstr);
       if ( ival > 0 )
 	{
 	  remap_order = ival;
@@ -315,8 +313,7 @@ void get_remap_env(void)
   envstr = getenv("REMAP_TEST");
   if ( envstr )
     {
-      int ival;
-      ival = atoi(envstr);
+      int ival = atoi(envstr);
       if ( ival > 0 )
 	{
 	  remap_test = ival;
@@ -350,8 +347,7 @@ void get_remap_env(void)
   envstr = getenv("REMAP_THRESHHOLD");
   if ( envstr )
     {
-      double fval;
-      fval = atof(envstr);
+      double fval = atof(envstr);
       if ( fval > 0 )
 	{
 	  remap_threshhold = fval;
@@ -365,45 +361,28 @@ void get_remap_env(void)
   envstr = getenv("CDO_REMAP_RADIUS");
   if ( envstr )
     {
-      double fval;
-      fval = atof(envstr);
-      if ( fval < 0 || fval > 180 )
-	{
-	  cdoAbort("CDO_REMAP_RADIUS=%g out of bounds (0-180)", fval);
-	}
-      else
-	{
-	  gridsearch_radius = fval;
-	  if ( cdoVerbose )
-	    cdoPrint("Set CDO_REMAP_RADIUS to %g", gridsearch_radius);
-	}
+      double fval = radius_str_to_deg(envstr);
+      if ( fval < 0 || fval > 180 ) cdoAbort("%s=%g out of bounds (0-180 deg)!", "CDO_REMAP_RADIUS", fval);
+      gridsearch_radius = fval;
+      if ( cdoVerbose ) cdoPrint("Set CDO_REMAP_RADIUS to %g", gridsearch_radius);
     }
 
   envstr = getenv("CDO_GRIDSEARCH_RADIUS");
   if ( envstr )
     {
-      double fval;
-      fval = atof(envstr);
-      if ( fval < 0 || fval > 180 )
-	{
-	  cdoAbort("CDO_GRIDSEARCH_RADIUS=%g out of bounds (0-180)", fval);
-	}
-      else
-	{
-	  gridsearch_radius = fval;
-	  if ( cdoVerbose )
-	    cdoPrint("Set CDO_GRIDSEARCH_RADIUS to %g", gridsearch_radius);
-	}
+      double fval = radius_str_to_deg(envstr);
+      if ( fval < 0 || fval > 180 ) cdoAbort("%s=%g out of bounds (0-180 deg)!", "CDO_GRIDSEARCH_RADIUS", fval);
+      gridsearch_radius = fval;
+      if ( cdoVerbose ) cdoPrint("Set CDO_GRIDSEARCH_RADIUS to %g", gridsearch_radius);
     }
   
   if ( cdoVerbose )
-    cdoPrint("remap_radius = %g", gridsearch_radius);
+    cdoPrint("remap_radius = %g deg", gridsearch_radius);
 
   envstr = getenv("REMAP_AREA_MIN");
   if ( envstr )
     {
-      double fval;
-      fval = atof(envstr);
+      double fval = atof(envstr);
       if ( fval > 0 )
 	{
 	  remap_frac_min = fval;
@@ -415,12 +394,11 @@ void get_remap_env(void)
   envstr = getenv("REMAP_NUM_SRCH_BINS");
   if ( envstr )
     {
-      int ival;
-      ival = atoi(envstr);
+      int ival = atoi(envstr);
       if ( ival > 0 )
 	{
 	  remap_num_srch_bins = ival;
-	  lremap_num_srch_bins = TRUE;
+	  lremap_num_srch_bins = true;
 	  if ( cdoVerbose )
 	    cdoPrint("Set REMAP_NUM_SRCH_BINS to %d", remap_num_srch_bins);
 	}
@@ -429,8 +407,7 @@ void get_remap_env(void)
   envstr = getenv("REMAP_NON_GLOBAL");
   if ( envstr )
     {
-      int ival;
-      ival = atoi(envstr);
+      int ival = atoi(envstr);
       if ( ival >= 0 )
 	{
 	  remap_non_global = ival;
@@ -444,8 +421,7 @@ void get_remap_env(void)
   envstr = getenv("REMAP_STORE_LINK_FAST");
   if ( envstr )
     {
-      int ival;
-      ival = atoi(envstr);
+      int ival = atoi(envstr);
       if ( ival >= 0 )
 	{
 	  remap_store_link_fast = ival;
@@ -463,22 +439,22 @@ void get_remap_env(void)
 	{
 	  if ( memcmp(envstr, "ON", 2) == 0 || memcmp(envstr, "on", 2) == 0 )
 	    {
-	      lextrapolate = TRUE;
-	      remap_extrapolate = TRUE;
+	      lextrapolate = true;
+	      remap_extrapolate = true;
 	    }
 	  else if ( memcmp(envstr, "OFF", 3) == 0 || memcmp(envstr, "off", 3) == 0 )
 	    {
-	      lextrapolate = TRUE;
-	      remap_extrapolate = FALSE;
+	      lextrapolate = true;
+	      remap_extrapolate = false;
 	    }
 	  else
 	    cdoWarning("Environment variable REMAP_EXTRAPOLATE has wrong value!");
 
 	  if ( cdoVerbose )
 	    {
-	      if ( remap_extrapolate == TRUE )
+	      if ( remap_extrapolate )
 		cdoPrint("Extrapolation enabled!");
-	      else if ( remap_extrapolate == FALSE )
+	      else
 		cdoPrint("Extrapolation disabled!");
 	    }
 	}
@@ -504,7 +480,7 @@ void get_remap_env(void)
 	    {
 	      if ( remap_genweights == TRUE )
 		cdoPrint("Generation of weights enabled!");
-	      else if ( remap_genweights == FALSE )
+	      else
 		cdoPrint("Generation of weights disabled!");
 	    }
 	}
@@ -514,134 +490,138 @@ void get_remap_env(void)
 static
 void set_halo_to_missval(int nx, int ny, double *array, double missval)
 {
-  int i, j;
-
-  for ( j = 0; j < ny+4; j++ ) array[j*(nx+4)+0]      = missval;
-  for ( j = 0; j < ny+4; j++ ) array[j*(nx+4)+1]      = missval;
-  for ( j = 0; j < ny+4; j++ ) array[j*(nx+4)+nx+2]   = missval;
-  for ( j = 0; j < ny+4; j++ ) array[j*(nx+4)+nx+3]   = missval;
-  for ( i = 0; i < nx+4; i++ ) array[     0*(nx+4)+i] = missval;
-  for ( i = 0; i < nx+4; i++ ) array[     1*(nx+4)+i] = missval;
-  for ( i = 0; i < nx+4; i++ ) array[(ny+2)*(nx+4)+i] = missval;
-  for ( i = 0; i < nx+4; i++ ) array[(ny+3)*(nx+4)+i] = missval;
+  for ( int j = 0; j < ny+4; j++ ) array[j*(nx+4)+0]      = missval;
+  for ( int j = 0; j < ny+4; j++ ) array[j*(nx+4)+1]      = missval;
+  for ( int j = 0; j < ny+4; j++ ) array[j*(nx+4)+nx+2]   = missval;
+  for ( int j = 0; j < ny+4; j++ ) array[j*(nx+4)+nx+3]   = missval;
+  for ( int i = 0; i < nx+4; i++ ) array[     0*(nx+4)+i] = missval;
+  for ( int i = 0; i < nx+4; i++ ) array[     1*(nx+4)+i] = missval;
+  for ( int i = 0; i < nx+4; i++ ) array[(ny+2)*(nx+4)+i] = missval;
+  for ( int i = 0; i < nx+4; i++ ) array[(ny+3)*(nx+4)+i] = missval;
 }
 
 static
-int is_global_grid(int gridID)
+bool is_global_grid(int gridID)
 {
-  int global_grid = TRUE;
-  int non_global = remap_non_global || !gridIsCircular(gridID);
-  int gridtype = gridInqType(gridID);
+  bool global_grid = true;
+  bool non_global = remap_non_global || !gridIsCircular(gridID);
 
-  if ( (gridtype == GRID_LONLAT && gridIsRotated(gridID)) ||
+  int gridtype = gridInqType(gridID);
+  int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+  if ( (projtype == CDI_PROJ_RLL)  ||
+       (projtype == CDI_PROJ_LAEA) ||
+       (projtype == CDI_PROJ_SINU) ||
+       (projtype == CDI_PROJ_LCC)  ||
        (gridtype == GRID_LONLAT && non_global) ||
        (gridtype == GRID_LCC) ||
-       (gridtype == GRID_LAEA) ||
-       (gridtype == GRID_SINUSOIDAL) ||
-       (gridtype == GRID_CURVILINEAR && non_global) ) global_grid = FALSE;
+       (gridtype == GRID_CURVILINEAR && non_global) ) global_grid = false;
 
-  return (global_grid);
+  return global_grid;
 }
 
 static
 void scale_gridbox_area(long gridsize, const double *restrict array1, long gridsize2, double *restrict array2, const double *restrict grid2_area)
 {
-  static int lgridboxinfo = TRUE;
-  long i;
   double array1sum = 0;
+  for ( long i = 0; i < gridsize; i++ ) array1sum += array1[i];
+
   double array2sum = 0;
+  for ( long i = 0; i < gridsize2; i++ ) array2sum += grid2_area[i];
 
-  for ( i = 0; i < gridsize; i++ )
-    array1sum += array1[i];
-
-  for ( i = 0; i < gridsize2; i++ )
-    array2sum += grid2_area[i];
-
-  for ( i = 0; i < gridsize2; i++ )
-    array2[i] = grid2_area[i]/array2sum*array1sum;
+  for ( long i = 0; i < gridsize2; i++ ) array2[i] = grid2_area[i]/array2sum*array1sum;
 
+  static bool lgridboxinfo = true;
   if ( lgridboxinfo )
     {
       cdoPrint("gridbox_area replaced and scaled to %g", array1sum);
-      lgridboxinfo = FALSE;
+      lgridboxinfo = false;
     }
 }
 
 static
-int set_remapgrids(int filetype, int vlistID, int ngrids, int *remapgrids)
+int set_remapgrids(int filetype, int vlistID, int ngrids, bool *remapgrids)
 {
-  int index, gridID, gridtype;
-
+  int index;
   for ( index = 0; index < ngrids; index++ )
     {
-      remapgrids[index] = TRUE;
-
-      gridID = vlistGrid(vlistID, index);
-      gridtype = gridInqType(gridID);
+      remapgrids[index] = true;
 
+      int gridID = vlistGrid(vlistID, index);
+      int gridtype = gridInqType(gridID);
+      int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+  
       if ( gridtype != GRID_LONLAT      &&
+           projtype != CDI_PROJ_RLL     &&
+           projtype != CDI_PROJ_LAEA    &&
+           projtype != CDI_PROJ_SINU    &&
+           projtype != CDI_PROJ_LCC     &&
 	   gridtype != GRID_GAUSSIAN    &&
 	   gridtype != GRID_LCC         &&
-	   gridtype != GRID_LAEA        &&
-	   gridtype != GRID_SINUSOIDAL  &&
 	   gridtype != GRID_GME         &&
 	   gridtype != GRID_CURVILINEAR &&
 	   gridtype != GRID_UNSTRUCTURED )
 	{
 	  if ( gridtype == GRID_GAUSSIAN_REDUCED )
 	    {
-	      if ( !cdoRegulargrid && filetype == FILETYPE_GRB )
+	      if ( !cdoRegulargrid && filetype == CDI_FILETYPE_GRB )
 		cdoAbort("Unsupported grid type: %s, use CDO option -R to convert reduced to regular grid!", gridNamePtr(gridtype));
 	      else
 		cdoAbort("Unsupported grid type: %s, use CDO operator -setgridtype,regular to convert reduced to regular grid!", gridNamePtr(gridtype));
 	    }
-	  else if ( gridtype == GRID_GENERIC && gridInqSize(gridID) == 1 )
-	    remapgrids[index] = FALSE;
+	  else if ( gridtype == GRID_GENERIC && gridInqSize(gridID) <= 2 )
+            {
+              remapgrids[index] = false;
+            }
 	  else
-	    cdoAbort("Unsupported grid type: %s", gridNamePtr(gridtype));
+            {
+              int nvars = vlistNvars(vlistID);
+              int varID;
+              for ( varID = 0; varID < nvars; ++varID )
+                if ( gridID == vlistInqVarGrid(vlistID, varID) ) break;
+              char varname[CDI_MAX_NAME];
+              vlistInqVarName(vlistID, varID, varname);
+              cdoAbort("Variable %s has an unsupported %s grid!", varname, gridNamePtr(gridtype));
+            }
 	}
     }
 
   for ( index = 0; index < ngrids; index++ )
-    if ( remapgrids[index] == TRUE ) break;
+    if ( remapgrids[index] ) break;
 
   if ( index == ngrids ) cdoAbort("No remappable grid found!");
 
-  return (index);
+  return index;
 }
 
 static
 int set_max_remaps(int vlistID)
 {
   int max_remaps = 0;
-  int nzaxis, nvars, index;
-  int zaxisID, zaxissize;
 
-  nzaxis = vlistNzaxis(vlistID);
-  for ( index = 0; index < nzaxis; index++ )
+  const int nzaxis = vlistNzaxis(vlistID);
+  for ( int index = 0; index < nzaxis; index++ )
     {
-      zaxisID = vlistZaxis(vlistID, index);
-      zaxissize = zaxisInqSize(zaxisID);
+      const int zaxisID = vlistZaxis(vlistID, index);
+      const int zaxissize = zaxisInqSize(zaxisID);
       if ( zaxissize > max_remaps ) max_remaps = zaxissize;
     }
   
-  nvars = vlistNvars(vlistID);
+  const int nvars = vlistNvars(vlistID);
   if ( nvars > max_remaps ) max_remaps = nvars;
 
   max_remaps++;
 
   if ( cdoVerbose ) cdoPrint("Set max_remaps to %d", max_remaps);
 
-  return (max_remaps);
+  return max_remaps;
 }
 
 static
 int get_norm_opt(void)
 {
   int norm_opt = NORM_OPT_FRACAREA;
-  char *envstr;
 
-  envstr = getenv("CDO_REMAP_NORMALIZE_OPT"); // obsolate
+  char *envstr = getenv("CDO_REMAP_NORMALIZE_OPT"); // obsolate
   if ( envstr && *envstr )
     {
       if      ( memcmp(envstr, "frac", 4) == 0 ) norm_opt = NORM_OPT_FRACAREA;
@@ -666,19 +646,18 @@ int get_norm_opt(void)
       else                                      cdoPrint("Normalization option: none");
     }
 
-  return (norm_opt);
+  return norm_opt;
 }
 
 static
 void remap_normalize(int norm_opt, int gridsize, double *array, double missval, remapgrid_t *tgt_grid)
 {
   /* used only to check the result of remapcon */
-  int i;
   double grid_err;
 
   if ( norm_opt == NORM_OPT_NONE )
     {
-      for ( i = 0; i < gridsize; i++ )
+      for ( int i = 0; i < gridsize; i++ )
 	{
 	  if ( !DBL_IS_EQUAL(array[i], missval) )
 	    {
@@ -693,7 +672,7 @@ void remap_normalize(int norm_opt, int gridsize, double *array, double missval,
     }
   else if ( norm_opt == NORM_OPT_DESTAREA )
     {
-      for ( i = 0; i < gridsize; i++ )
+      for ( int i = 0; i < gridsize; i++ )
 	{
 	  if ( !DBL_IS_EQUAL(array[i], missval) )
 	    {
@@ -737,6 +716,37 @@ void init_remap_timer(void)
   timer_remap_con_l2 = timer_new("remap con loop2");
 }
 
+static
+void links_per_value(remapvars_t *remapvars)
+{
+  long num_links = remapvars->num_links;
+  const int *restrict dst_add = remapvars->tgt_cell_add;
+
+  int lpv = 1;
+  int ival = dst_add[0];
+  for ( long n = 1; n < num_links; ++n )
+    if ( dst_add[n] == ival ) lpv++;
+    else break;
+
+  if ( num_links%lpv != 0 ) lpv = -1;
+  else if ( lpv > 1 )
+    {
+      for ( long n = 1; n < num_links/lpv; ++n )
+        {
+          ival = dst_add[n*lpv];
+          for ( int k = 1; k < lpv; ++k )
+            if ( dst_add[n*lpv+k] != ival )
+              {
+                lpv = -1;
+                break;
+              }
+          if ( lpv == -1 ) break;
+        }
+    }
+
+  remapvars->links_per_value = lpv;
+}
+
 static
 void sort_remap_add(remapvars_t *remapvars)
 {
@@ -766,21 +776,18 @@ void *Remap(void *argument)
   int streamID2 = -1;
   int nrecs;
   int index;
-  int tsID, recID, varID, levelID;
+  int varID, levelID;
   int gridsize, gridsize2;
   int gridID1 = -1, gridID2;
   int nmiss1, nmiss2, i, j, r = -1;
-  int *imask = NULL;
   int nremaps = 0;
   int norm_opt = NORM_OPT_NONE;
   int map_type = -1;
   int submap_type = SUBMAP_TYPE_NONE;
   int num_neighbors = 0;
   int need_gradiants = FALSE;
-  int grid1sizemax;
   char varname[CDI_MAX_NAME];
   double missval;
-  double *array1 = NULL, *array2 = NULL;
   double *grad1_lat = NULL, *grad1_lon = NULL, *grad1_latlon = NULL;
   remap_t *remaps = NULL;
   char *remap_file = NULL;
@@ -817,18 +824,20 @@ void *Remap(void *argument)
 
   if ( operfunc == REMAPDIS || operfunc == GENDIS ||
        operfunc == REMAPNN  || operfunc == GENNN )
-    remap_extrapolate = TRUE;
+    remap_extrapolate = true;
 
   get_remap_env();
 
   if ( cdoVerbose )
     {
-      if ( remap_extrapolate == TRUE )
+      if ( remap_extrapolate )
 	cdoPrint("Extrapolation enabled!");
-      else if ( remap_extrapolate == FALSE )
+      else
 	cdoPrint("Extrapolation disabled!");
     }
 
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+
   if ( lremapxxx )
     {
       operatorInputArg("grid description file or name, remap weights file (SCRIP NetCDF)");
@@ -855,7 +864,6 @@ void *Remap(void *argument)
 
   if ( gridInqType(gridID2) == GRID_GENERIC ) cdoAbort("Unsupported target grid type (generic)!");
 
-  int streamID1 = streamOpenRead(cdoStreamName(0));
   int filetype = streamInqFiletype(streamID1);
 
   int vlistID1 = streamInqVlist(streamID1);
@@ -866,7 +874,7 @@ void *Remap(void *argument)
   vlistDefTaxis(vlistID2, taxisID2);
 
   int ngrids = vlistNgrids(vlistID1);
-  int remapgrids[ngrids];
+  bool remapgrids[ngrids];
   index = set_remapgrids(filetype, vlistID1, ngrids, remapgrids);
   gridID1 = vlistGrid(vlistID1, index);
 
@@ -897,18 +905,21 @@ void *Remap(void *argument)
 
       read_remap_scrip(remap_file, gridID1, gridID2, &map_type, &submap_type, &num_neighbors,
 		       &remap_order, &remaps[0].src_grid, &remaps[0].tgt_grid, &remaps[0].vars);
+
+      if ( remaps[0].vars.links_per_value == -1 ) links_per_value(&remaps[0].vars);
+            
       nremaps = 1;
       gridsize = remaps[0].src_grid.size;
       remaps[0].gridID   = gridID1;
       remaps[0].gridsize = gridInqSize(gridID1);
 
-      if ( map_type == MAP_TYPE_DISTWGT && !lextrapolate ) remap_extrapolate = TRUE;
-      if ( gridIsCircular(gridID1)      && !lextrapolate ) remap_extrapolate = TRUE;
+      if ( map_type == MAP_TYPE_DISTWGT && !lextrapolate ) remap_extrapolate = true;
+      if ( gridIsCircular(gridID1)      && !lextrapolate ) remap_extrapolate = true;
 
-      if ( map_type == MAP_TYPE_DISTWGT && !remap_extrapolate && gridInqSize(gridID1) > 1 &&  !is_global_grid(gridID1) )
+      if ( map_type == MAP_TYPE_DISTWGT && !remap_extrapolate && gridInqSize(gridID1) > 1 && !is_global_grid(gridID1) )
 	{
 	  remaps[0].gridsize += 4*(gridInqXsize(gridID1)+2) + 4*(gridInqYsize(gridID1)+2);
-	  remaps[0].src_grid.non_global = TRUE;
+	  remaps[0].src_grid.non_global = true;
 	}
 
       if ( gridInqType(gridID1) == GRID_GME ) gridsize = remaps[0].src_grid.nvgp;
@@ -959,7 +970,7 @@ void *Remap(void *argument)
 
   if ( map_type == MAP_TYPE_CONSERV || map_type == MAP_TYPE_CONSERV_YAC ) norm_opt = get_norm_opt();
 
-  grid1sizemax = vlistGridsizeMax(vlistID1);
+  int grid1sizemax = vlistGridsizeMax(vlistID1);
 
   if ( map_type == MAP_TYPE_BICUBIC ) need_gradiants = TRUE;
   if ( map_type == MAP_TYPE_CONSERV && remap_order == 2 )
@@ -977,11 +988,11 @@ void *Remap(void *argument)
       grad1_latlon = (double*) Malloc(grid1sizemax*sizeof(double));
     }
 
-  array1 = (double*) Malloc(grid1sizemax*sizeof(double));
-  imask  = (int*) Malloc(grid1sizemax*sizeof(int));
+  double *array1 = (double*) Malloc(grid1sizemax*sizeof(double));
+  int *imask = (int*) Malloc(grid1sizemax*sizeof(int));
 
   gridsize = gridInqSize(gridID2);
-  array2   = (double*) Malloc(gridsize*sizeof(double));
+  double *array2 = (double*) Malloc(gridsize*sizeof(double));
 
   if ( ! lwrite_remap )
     {
@@ -989,7 +1000,7 @@ void *Remap(void *argument)
       streamDefVlist(streamID2, vlistID2);
     }
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
@@ -997,14 +1008,14 @@ void *Remap(void *argument)
       if ( ! lwrite_remap ) 
 	streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamReadRecord(streamID1, array1, &nmiss1);
 
 	  gridID1 = vlistInqVarGrid(vlistID1, varID);
 
-	  if ( remapgrids[vlistGridIndex(vlistID1, gridID1)] == FALSE )
+	  if ( !remapgrids[vlistGridIndex(vlistID1, gridID1)] )
 	    {
 	      if ( lwrite_remap ) continue;
 	      else
@@ -1018,21 +1029,16 @@ void *Remap(void *argument)
 	  if ( map_type != MAP_TYPE_CONSERV && map_type != MAP_TYPE_CONSERV_YAC && 
 	       gridInqType(gridID1) == GRID_GME && gridInqType(gridID2) == GRID_GME )
 	    cdoAbort("Only conservative remapping is available to remap between GME grids!");
-	  /*
-	  if ( gridIsRotated(gridID1) && map_type != MAP_TYPE_CONSERV )
-	    cdoAbort("Only conservative remapping is available for rotated grids!");
-	  */
+
 	  missval = vlistInqVarMissval(vlistID1, varID);
 	  gridsize = gridInqSize(gridID1);
 
-	  if ( gridIsCircular(gridID1) && !lextrapolate ) remap_extrapolate = TRUE;
+	  if ( gridIsCircular(gridID1) && !lextrapolate ) remap_extrapolate = true;
 	  if ( map_type == MAP_TYPE_DISTWGT && !remap_extrapolate && gridInqSize(gridID1) > 1 && !is_global_grid(gridID1) )
 	    {
-	      int gridsize_new;
-	      int nx, ny;
-	      nx = gridInqXsize(gridID1);
-	      ny = gridInqYsize(gridID1);
-	      gridsize_new = gridsize + 4*(nx+2) + 4*(ny+2);
+	      int nx = gridInqXsize(gridID1);
+	      int ny = gridInqYsize(gridID1);
+	      int gridsize_new = gridsize + 4*(nx+2) + 4*(ny+2);
 	      if ( gridsize_new > grid1sizemax )
 		{
 		  grid1sizemax = gridsize_new;
@@ -1107,17 +1113,17 @@ void *Remap(void *argument)
 
 	      if ( remaps[r].gridID != gridID1 )
 		{
-		  if ( gridIsCircular(gridID1) && !lextrapolate ) remap_extrapolate = TRUE;
-		  remaps[r].src_grid.non_global = FALSE;
+		  if ( gridIsCircular(gridID1) && !lextrapolate ) remap_extrapolate = true;
+		  remaps[r].src_grid.non_global = false;
 		  if ( map_type == MAP_TYPE_DISTWGT && !remap_extrapolate && gridInqSize(gridID1) > 1 && !is_global_grid(gridID1) )
 		    {
-		      remaps[r].src_grid.non_global = TRUE;
+		      remaps[r].src_grid.non_global = true;
 		    }
 		  /*
 		    remaps[r].src_grid.luse_cell_area = FALSE;
 		    remaps[r].tgt_grid.luse_cell_area = FALSE;
 		  */
-		  if ( gridInqType(gridID1) != GRID_UNSTRUCTURED && lremap_num_srch_bins == FALSE )
+		  if ( gridInqType(gridID1) != GRID_UNSTRUCTURED && lremap_num_srch_bins == false )
 		    {
 		      if ( !remap_extrapolate && map_type == MAP_TYPE_DISTWGT )
 			{
@@ -1136,7 +1142,7 @@ void *Remap(void *argument)
 		  remap_set_int(REMAP_NUM_SRCH_BINS, remap_num_srch_bins);
 
 		  remaps[r].vars.norm_opt = norm_opt;
-		  remaps[r].vars.pinit = FALSE;
+		  remaps[r].vars.pinit = false;
 		  
 		  if ( (map_type == MAP_TYPE_BILINEAR || map_type == MAP_TYPE_BICUBIC) &&
 		       (gridInqType(gridID1) == GRID_GME || gridInqType(gridID1) == GRID_UNSTRUCTURED) )
@@ -1187,6 +1193,7 @@ void *Remap(void *argument)
 		    resize_remap_vars(&remaps[r].vars, remaps[r].vars.num_links-remaps[r].vars.max_links);
 		  
 		  if ( remaps[r].vars.sort_add ) sort_remap_add(&remaps[r].vars);
+		  if ( remaps[r].vars.links_per_value == -1 ) links_per_value(&remaps[r].vars);
 
 		  if ( lwrite_remap ) goto WRITE_REMAP;
 
@@ -1222,7 +1229,7 @@ void *Remap(void *argument)
 	      else
 		remap(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
 		      remaps[r].vars.num_wts, remaps[r].vars.tgt_cell_add, remaps[r].vars.src_cell_add,
-		      array1, grad1_lat, grad1_lon, grad1_latlon, remaps[r].vars.links);
+		      array1, grad1_lat, grad1_lon, grad1_latlon, remaps[r].vars.links, remaps[r].vars.links_per_value);
 	    }
 	  else
 	    {
@@ -1244,16 +1251,15 @@ void *Remap(void *argument)
 
 	  if ( operfunc == REMAPSUM )
 	    {
-	      double array1sum = 0;
-	      double array2sum = 0;
-   
 	      for ( i = 0; i < gridsize; i++ )
 		printf("1 %d %g %g %g %g\n", i, array1[i], remaps[r].src_grid.cell_frac[i], remaps[r].src_grid.cell_area[i],remaps[r].src_grid.cell_frac[i]);
+	      double array1sum = 0;
 	      for ( i = 0; i < gridsize; i++ )
 		array1sum += remaps[r].src_grid.cell_area[i];
 
 	      for ( i = 0; i < gridsize2; i++ )
 		printf("2 %d %g %g %g %g\n", i, array2[i], remaps[r].tgt_grid.cell_frac[i],remaps[r].tgt_grid.cell_area[i],remaps[r].tgt_grid.cell_frac[i]);
+	      double array2sum = 0;
 	      for ( i = 0; i < gridsize2; i++ )
 		array2sum += remaps[r].tgt_grid.cell_area[i];
 
@@ -1273,9 +1279,8 @@ void *Remap(void *argument)
 
 	  if ( gridInqType(gridID2) == GRID_GME )
 	    {
-	      int ni, nd;
- 	      ni = gridInqGMEni(gridID2);
-	      nd = gridInqGMEnd(gridID2);
+	      int nd, ni, ni2, ni3;
+ 	      gridInqParamGME(gridID2, &nd, &ni, &ni2, &ni3);
 	      j = remaps[r].tgt_grid.size;
 
 	      for ( i = gridsize2-1; i >=0 ; i-- )
