@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,26 +20,23 @@
       Remapeta     remapeta          Model to model level interpolation
 */
 
-#include <ctype.h>
 
+#include "hetaeta.h"
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
 #include "after_vertint.h"
-#include "list.h"
+#include "listarray.h"
 #include "stdnametable.h"
-#include "hetaeta.h"
 
 
 static 
 void setmissval(long nvals, int *imiss, double missval, double *array)
 {
-  long i;
-
   if ( imiss )
     {
-      for ( i = 0; i < nvals; ++i )
+      for ( long i = 0; i < nvals; ++i )
 	if ( imiss[i] ) array[i] = missval;
     }
 }
@@ -47,9 +44,7 @@ void setmissval(long nvals, int *imiss, double missval, double *array)
 static
 void corr_hum(long gridsize, double *q, double q_min)
 {
-  long i;
-
-  for ( i = 0; i < gridsize; ++i )
+  for ( long i = 0; i < gridsize; ++i )
     {
       if ( q[i] < q_min ) q[i] = q_min;
     }
@@ -94,7 +89,7 @@ long ncctop(double cptop, long nlev, long nlevp1, double *vct_a, double *vct_b)
       if ( zp[jk] >= cptop ) break;
     }
 
-  return (nctop);
+  return nctop;
 }
 
 
@@ -102,21 +97,16 @@ double *vctFromFile(const char *filename, int *nvct)
 {
   char line[1024], *pline;
   int num, i = 0;
-  int nlevh2, nvct2;
   int maxvct = 8192;
-  double *vct2;
-  FILE *fp;
 
-
-  fp = fopen(filename, "r");
+  FILE *fp = fopen(filename, "r");
   if ( fp == NULL ) { perror(filename); exit(EXIT_FAILURE); }
 
-  vct2 = (double*) malloc(maxvct*sizeof(double));
+  double *vct2 = (double*) Malloc(maxvct*sizeof(double));
 
   while ( readline(fp, line, 1024) )
     {
-      if ( line[0] == '#' ) continue;
-      if ( line[0] == '\0' ) continue;
+      if ( line[0] == '#' || line[0] == '\0' ) continue;
 
       pline = line;
       num = (int) strtod(pline, &pline);
@@ -135,17 +125,17 @@ double *vctFromFile(const char *filename, int *nvct)
 
   fclose(fp);
 
-  nvct2 = 2*i;
-  nlevh2 = i - 1;
+  int nvct2 = 2*i;
+  int nlevh2 = i - 1;
 
   for ( i = 0; i < nlevh2+1; ++i )
     vct2[i+nvct2/2] = vct2[i+maxvct/2];
   
-  vct2 = (double*) realloc(vct2, nvct2*sizeof(double));
+  vct2 = (double*) Realloc(vct2, nvct2*sizeof(double));
 
   *nvct = nvct2;
 
-  return (vct2);
+  return vct2;
 }
 
 static
@@ -177,44 +167,76 @@ void vert_sumw(double *sum, double *var3d, long gridsize, long nlevel, double *d
 }
 
 
+double *vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlevf)
+{
+  int zaxisIDh = -1;
+  int nhlevf = 0;
+  int nvct = 0;
+  double *vct = NULL;
+
+  bool lhavevct = false;
+  int nzaxis = vlistNzaxis(vlistID);
+  for ( int i = 0; i < nzaxis; i++ )
+    {
+      int zaxisID = vlistZaxis(vlistID, i);
+      int nlevel  = zaxisInqSize(zaxisID);
+
+      if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevel > 1 )
+	{
+          nvct = zaxisInqVctSize(zaxisID);
+
+          if ( nlevel == (nvct/2 - 1) )
+            {
+	      if ( lhavevct == false )
+		{
+		  lhavevct = true;
+                  zaxisIDh = zaxisID;
+                  nhlevf  = nlevel;
+ 
+                  vct = (double*) Malloc(nvct*sizeof(double));
+                  zaxisInqVct(zaxisID, vct);
+                }
+            }
+          else 
+            {
+              if ( cdoVerbose )
+                cdoPrint("nlevel = (nvct1/2 - 1): nlevel = %d", nlevel);
+              if ( nlevel < (nvct/2 - 1) )
+                cdoPrint("z-axis %d has only %d of %d hybrid sigma pressure levels!", i+1, nlevel, (nvct/2 - 1));
+            }
+	}
+    }
+
+  *rzaxisIDh = zaxisIDh;
+  *rnvct   = nvct;
+  *rnhlevf = nhlevf;
+  
+  return vct;
+}
+
 #define  MAX_VARS3D  1024
 
 void *Remapeta(void *argument)
 {
-  int REMAPETA, REMAPETAS, REMAPETAZ;
-  int operatorID;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
   int nfis2gp = 0;
-  int recID, nrecs;
-  int i, offset, iv;
-  int tsID, varID, levelID;
-  int nvars, nvars3D = 0;
-  int zaxisID2, zaxisIDh = -1, nzaxis, surfaceID;
-  int gridID, zaxisID;
-  int nlevel;
-  int nvct1, nvct2 = 0;
-  int sgeopotID = -1, tempID = -1, sqID = -1, psID = -1, lnpsID = -1, presID = -1;
-  int code;
+  int nrecs;
+  int i, iv;
+  int varID, levelID;
+  int nvars3D = 0;
+  int sgeopotID = -1, tempID = -1, sqID = -1, psID = -1, lnpsID = -1;
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
   double *single2;
-  int taxisID1, taxisID2;
-  int lhavevct;
-  int nhlevf1 = 0, nhlevf2 = 0;
-  double *lev2;
-  double *vct1 = NULL, *vct2 = NULL;
-  double *a1 = NULL, *b1 = NULL, *a2 = NULL, *b2 = NULL;
-  double *fis1 = NULL, *ps1 = NULL, *t1 = NULL, *q1 = NULL;
-  double *fis2 = NULL, *ps2 = NULL, *t2 = NULL, *q2 = NULL;
+  double *fis2 = NULL;
+  double *t1 = NULL, *q1 = NULL;
+  double *t2 = NULL, *q2 = NULL;
   double *tscor = NULL, *pscor = NULL, *secor = NULL;
   int nmiss, nmissout = 0;
-  int ltq = FALSE;
-  int lfis2 = FALSE;
+  bool ltq = false;
+  bool lfis2 = false;
   int varids[MAX_VARS3D];
   int *imiss = NULL;
   int timer_hetaeta = 0;
   long nctop = 0;
-  double *array = NULL;
   double *deltap1 = NULL, *deltap2 = NULL;
   double *half_press1 = NULL, *half_press2 = NULL;
   double *sum1 = NULL, *sum2 = NULL;
@@ -222,23 +244,21 @@ void *Remapeta(void *argument)
   double minval, maxval;
   double missval = 0;
   double cconst = 1.E-6;
-  const char *fname;
-  char *envstr;
   double cptop  = 0; /* min. pressure level for cond. */
 
   if ( cdoTimer ) timer_hetaeta = timer_new("Remapeta_hetaeta");
 
   cdoInitialize(argument);
 
-  REMAPETA  = cdoOperatorAdd("remapeta",   0, 0, "VCT file name");
-  REMAPETAS = cdoOperatorAdd("remapeta_s", 0, 0, "VCT file name");
-  REMAPETAZ = cdoOperatorAdd("remapeta_z", 0, 0, "VCT file name");
+  int REMAPETA  = cdoOperatorAdd("remapeta",   0, 0, "VCT file name");
+  int REMAPETAS = cdoOperatorAdd("remapeta_s", 0, 0, "VCT file name");
+  int REMAPETAZ = cdoOperatorAdd("remapeta_z", 0, 0, "VCT file name");
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  envstr = getenv("REMAPETA_PTOP");
+  char *envstr = getenv("REMAPETA_PTOP");
   if ( envstr )
     {
       double fval = atof(envstr);
@@ -250,43 +270,42 @@ void *Remapeta(void *argument)
 	}
     }  
 
-  vct2 = vctFromFile(operatorArgv()[0], &nvct2);
-  nhlevf2 = nvct2/2 - 1;
+  int nvct2 = 0;
+  double *vct2 = vctFromFile(operatorArgv()[0], &nvct2);
+  int nhlevf2 = nvct2/2 - 1;
 
-  a2 = vct2;
-  b2 = vct2 + nvct2/2;
+  double *a2 = vct2;
+  double *b2 = vct2 + nvct2/2;
 
   if ( cdoVerbose )
     for ( i = 0; i < nhlevf2+1; ++i )
       cdoPrint("vct2: %5d %25.17f %25.17f", i, vct2[i], vct2[nvct2/2+i]);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
   if ( operatorArgc() == 2 )
     {
-      int streamID;
+      lfis2 = true;
 
-      lfis2 = TRUE;
-      fname = operatorArgv()[1];
-      
+      const char *fname = operatorArgv()[1];
       argument_t *fileargument = file_argument_new(fname);
-      streamID = streamOpenRead(fileargument);
+      int streamID = streamOpenRead(fileargument);
       file_argument_free(fileargument);
 
-      vlistID1 = streamInqVlist(streamID);
+      int vlistID1 = streamInqVlist(streamID);
 
       streamInqRecord(streamID, &varID, &levelID);
-      gridID  = vlistInqVarGrid(vlistID1, varID);
+      int gridID  = vlistInqVarGrid(vlistID1, varID);
       nfis2gp = gridInqSize(gridID);
 
-      fis2 = (double*) malloc(nfis2gp*sizeof(double));
+      fis2 = (double*) Malloc(nfis2gp*sizeof(double));
 
       streamReadRecord(streamID, fis2, &nmiss);
 
       if ( nmiss )
 	{
 	  missval = vlistInqVarMissval(vlistID1, varID);
-	  imiss = (int*) malloc (nfis2gp*sizeof(int));
+	  imiss = (int*) Malloc(nfis2gp*sizeof(int));
 	  for ( i = 0; i < nfis2gp; ++i )
 	    {
 	      if ( DBL_IS_EQUAL(fis2[i], missval) )
@@ -309,98 +328,54 @@ void *Remapeta(void *argument)
       streamClose(streamID); 
     }
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  gridID = vlistGrid(vlistID1, 0);
+  int gridID = vlistGrid(vlistID1, 0);
   if ( gridInqType(gridID) == GRID_SPECTRAL )
     cdoAbort("Spectral data unsupported!");
 
   int gridsize = vlist_check_gridsize(vlistID1);
 
-  zaxisID2 = zaxisCreate(ZAXIS_HYBRID, nhlevf2);
-  lev2 = (double*) malloc(nhlevf2*sizeof(double));
+  int zaxisID2 = zaxisCreate(ZAXIS_HYBRID, nhlevf2);
+  double *lev2 = (double*) Malloc(nhlevf2*sizeof(double));
   for ( i = 0; i < nhlevf2; ++i ) lev2[i] = i+1;
   zaxisDefLevels(zaxisID2, lev2);
-  free(lev2);
+  Free(lev2);
 
   if ( nvct2 == 0 ) cdoAbort("Internal problem, vct2 undefined!");
   zaxisDefVct(zaxisID2, nvct2, vct2);
 
-  surfaceID = zaxisFromName("surface");
+  int surfaceID = zaxisFromName("surface");
 
-  nzaxis  = vlistNzaxis(vlistID1);
-  lhavevct = FALSE;
+  int zaxisIDh = -1;
+  int nvct1 = 0;
+  int nhlevf1 = 0;
+  double *vct1 = vlist_hybrid_vct(vlistID1, &zaxisIDh, &nvct1, &nhlevf1);
 
-  if ( cdoVerbose )
-    cdoPrint("nzaxis: %d", nzaxis);
+  vlist_change_hybrid_zaxis(vlistID1, vlistID2, zaxisIDh, zaxisID2);
 
-  for ( i = 0; i < nzaxis; i++ )
+  int nzaxis = vlistNzaxis(vlistID1);
+  for ( int i = 0; i < nzaxis; i++ )
     {
-      zaxisID = vlistZaxis(vlistID1, i);
-      nlevel  = zaxisInqSize(zaxisID);
-      if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID )
-	{
-	  if ( nlevel > 1 )
-	    {
-	      nvct1 = zaxisInqVctSize(zaxisID);
-
-              if ( cdoVerbose )
-                cdoPrint("i: %d, vct1 size of zaxisID %d = %d", i, zaxisID, nvct1);
-
-	      if ( nlevel == (nvct1/2 - 1) )
-		{
-		  if ( lhavevct == FALSE )
-		    {
-		      lhavevct = TRUE;
-		      zaxisIDh = zaxisID;
-		      nhlevf1  = nlevel;
-	      
-                      if ( cdoVerbose )
-                        cdoPrint("lhavevct=TRUE  zaxisIDh = %d, nhlevf1   = %d", zaxisIDh, nlevel);
- 
-		      vct1 = (double*) malloc(nvct1*sizeof(double));
-		      zaxisInqVct(zaxisID, vct1);
-		      
-		      vlistChangeZaxisIndex(vlistID2, i, zaxisID2);
-
-		      a1 = vct1;
-		      b1 = vct1 + nvct1/2;
-		      if ( cdoVerbose )
-			for ( i = 0; i < nvct1/2; ++i )
-			  cdoPrint("vct1: %5d %25.17f %25.17f", i, vct1[i], vct1[nvct1/2+i]);
-		    }
-		  else
-		    {
-		      if ( memcmp(vct1, zaxisInqVctPtr(zaxisID), nvct1*sizeof(double)) == 0 )
-			vlistChangeZaxisIndex(vlistID2, i, zaxisID2);
-		    }
-		}
-              else 
-                {
-		  if ( cdoVerbose )
-		    cdoPrint("nlevel = (nvct1/2 - 1): nlevel = %d", nlevel);
-		  if ( nlevel < (nvct1/2 - 1) )
-		    cdoPrint("z-axis %d has only %d of %d hybrid sigma pressure levels!", i+1, nlevel, (nvct1/2 - 1));
-                }
-	    }
-	  else
-	    {
-	      vlistChangeZaxisIndex(vlistID2, i, surfaceID);
-	    }
-	}
-      else
-        {
-	  if ( cdoVerbose )
-	    cdoPrint("i: %d, type of zaxisID %d not ZAXIS_HYBRID", i, zaxisID);
-        }
+      int zaxisID = vlistZaxis(vlistID1, i);
+      int nlevel  = zaxisInqSize(zaxisID);
+      if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevel == 1 )
+        vlistChangeZaxisIndex(vlistID2, i, surfaceID);
     }
+  
+  double *a1 = vct1;
+  double *b1 = vct1 + nvct1/2;
+  if ( cdoVerbose )
+    for ( i = 0; i < nvct1/2; ++i )
+      cdoPrint("vct1: %5d %25.17f %25.17f", i, vct1[i], vct1[nvct1/2+i]);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
@@ -408,15 +383,15 @@ void *Remapeta(void *argument)
   if ( zaxisIDh == -1 )
     cdoWarning("No 3D variable with hybrid sigma pressure coordinate found!");
 
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      gridID  = vlistInqVarGrid(vlistID1, varID);
-      zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      nlevel  = zaxisInqSize(zaxisID);
+      int gridID  = vlistInqVarGrid(vlistID1, varID);
+      int zaxisID = vlistInqVarZaxis(vlistID1, varID);
+      int nlevel  = zaxisInqSize(zaxisID);
 
-      code = vlistInqVarCode(vlistID1, varID);
+      int code = vlistInqVarCode(vlistID1, varID);
       /* code = -1; */
       if ( code <= 0 || code == 255 )
 	{
@@ -475,7 +450,7 @@ void *Remapeta(void *argument)
 
   if ( tempID != -1 && sqID != -1 )
     {
-      ltq = TRUE;
+      ltq = true;
     }
   else
     {
@@ -483,59 +458,59 @@ void *Remapeta(void *argument)
       if ( sqID   != -1 ) cdoAbort("Humidity without temperature unsupported!");
     }
   /*
-  if ( ltq == FALSE )
+  if ( ltq == false )
     {
       cdoWarning("Temperature and Humidity not found!");
     }
   */
   if ( operatorID == REMAPETA ) {}
 
-  if ( operatorID == REMAPETAS || operatorID == REMAPETAZ)
+  if ( operatorID == REMAPETAS || operatorID == REMAPETAZ )
     {
-      sum1 = (double*) malloc(gridsize*sizeof(double));
-      sum2 = (double*) malloc(gridsize*sizeof(double));
+      sum1 = (double*) Malloc(gridsize*sizeof(double));
+      sum2 = (double*) Malloc(gridsize*sizeof(double));
     }
 
   if ( operatorID == REMAPETAZ )
     {
-      deltap1 = (double*) malloc(gridsize*nhlevf1*sizeof(double));
-      deltap2 = (double*) malloc(gridsize*nhlevf2*sizeof(double));
-      half_press1 = (double*) malloc(gridsize*(nhlevf1+1)*sizeof(double));
-      half_press2 = (double*) malloc(gridsize*(nhlevf2+1)*sizeof(double));
+      deltap1 = (double*) Malloc(gridsize*nhlevf1*sizeof(double));
+      deltap2 = (double*) Malloc(gridsize*nhlevf2*sizeof(double));
+      half_press1 = (double*) Malloc(gridsize*(nhlevf1+1)*sizeof(double));
+      half_press2 = (double*) Malloc(gridsize*(nhlevf2+1)*sizeof(double));
     }
 
-  array = (double*) malloc(gridsize*sizeof(double));
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
-  fis1  = (double*) malloc(gridsize*sizeof(double));
-  ps1   = (double*) malloc(gridsize*sizeof(double));
+  double *fis1  = (double*) Malloc(gridsize*sizeof(double));
+  double *ps1   = (double*) Malloc(gridsize*sizeof(double));
 
-  if ( lfis2 == FALSE ) fis2  = (double*) malloc(gridsize*sizeof(double));
-  if ( lfis2 == TRUE && gridsize != nfis2gp ) cdoAbort("Orographies have different grid size!");
+  if ( lfis2 == false ) fis2 = (double*) Malloc(gridsize*sizeof(double));
+  if ( lfis2 == true && gridsize != nfis2gp ) cdoAbort("Orographies have different grid size!");
 
-  ps2   = (double*) malloc(gridsize*sizeof(double));
+  double *ps2   = (double*) Malloc(gridsize*sizeof(double));
 
   if ( ltq )
     {
-      tscor = (double*) malloc(gridsize*sizeof(double));
-      pscor = (double*) malloc(gridsize*sizeof(double));
-      secor = (double*) malloc(gridsize*sizeof(double));
+      tscor = (double*) Malloc(gridsize*sizeof(double));
+      pscor = (double*) Malloc(gridsize*sizeof(double));
+      secor = (double*) Malloc(gridsize*sizeof(double));
 
-      t1    = (double*) malloc(gridsize*nhlevf1*sizeof(double));
-      q1    = (double*) malloc(gridsize*nhlevf1*sizeof(double));
+      t1    = (double*) Malloc(gridsize*nhlevf1*sizeof(double));
+      q1    = (double*) Malloc(gridsize*nhlevf1*sizeof(double));
 
-      t2    = (double*) malloc(gridsize*nhlevf2*sizeof(double));
-      q2    = (double*) malloc(gridsize*nhlevf2*sizeof(double));
+      t2    = (double*) Malloc(gridsize*nhlevf2*sizeof(double));
+      q2    = (double*) Malloc(gridsize*nhlevf2*sizeof(double));
     }
 
   if ( nvars3D )
     {
-      vars1  = (double**) malloc(nvars*sizeof(double*));
-      vars2  = (double**) malloc(nvars*sizeof(double*));
+      vars1  = (double**) Malloc(nvars*sizeof(double*));
+      vars2  = (double**) Malloc(nvars*sizeof(double*));
 
       for ( varID = 0; varID < nvars3D; ++varID )
 	{
-	  vars1[varID] = (double*) malloc(gridsize*nhlevf1*sizeof(double));
-	  vars2[varID] = (double*) malloc(gridsize*nhlevf2*sizeof(double));
+	  vars1[varID] = (double*) Malloc(gridsize*nhlevf1*sizeof(double));
+	  vars2[varID] = (double*) Malloc(gridsize*nhlevf2*sizeof(double));
 	}
     }
 
@@ -547,7 +522,7 @@ void *Remapeta(void *argument)
       memset(fis1, 0, gridsize*sizeof(double));
     }
 
-  presID = lnpsID;
+  int presID = lnpsID;
   if ( zaxisIDh != -1 && lnpsID == -1 )
     {
       if ( psID == -1 )
@@ -564,21 +539,20 @@ void *Remapeta(void *argument)
 	cdoPrint("using %s", var_stdname(surface_air_pressure));
     }
 
-  if ( cdoVerbose ) cdoPrint("nvars3D = %d   ltq = %d", nvars3D, ltq);
+  if ( cdoVerbose ) cdoPrint("nvars3D = %d   ltq = %d", nvars3D, (int)ltq);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-
       streamDefTimestep(streamID2, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
-	  zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-	  nlevel   = zaxisInqSize(zaxisID);
-	  offset   = gridsize*levelID;
+	  int zaxisID = vlistInqVarZaxis(vlistID1, varID);
+	  int nlevel  = zaxisInqSize(zaxisID);
+	  int offset = gridsize*levelID;
 	  streamReadRecord(streamID1, array, &nmiss);
 
 	  if ( zaxisIDh != -1 )
@@ -632,16 +606,16 @@ void *Remapeta(void *argument)
 	    cdoWarning("Orography out of range (min=%g max=%g)!", minval, maxval);
 	}
 
-      if ( lfis2 == FALSE )
-	for ( i = 0; i < gridsize; i++ ) fis2[i] = fis1[i];
+      if ( lfis2 == false )
+	for ( int i = 0; i < gridsize; i++ ) fis2[i] = fis1[i];
 
       if ( ltq )
 	{
 	  varID = tempID;
-	  nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+	  int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	  for ( levelID = 0; levelID < nlevel; levelID++ )
 	    {
-	      offset   = gridsize*levelID;
+	      int offset = gridsize*levelID;
 	      single2  = t1 + offset;
 
 	      minmaxval(gridsize, single2, imiss, &minval, &maxval);
@@ -654,7 +628,7 @@ void *Remapeta(void *argument)
 	  nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	  for ( levelID = 0; levelID < nlevel; levelID++ )
 	    {
-	      offset   = gridsize*levelID;
+	      int offset = gridsize*levelID;
 	      single2  = q1 + offset;
 
 	      corr_hum(gridsize, single2, MIN_Q);
@@ -708,10 +682,10 @@ void *Remapeta(void *argument)
       if ( ltq )
 	{
 	  varID = tempID;
-	  nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+	  int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
 	  for ( levelID = 0; levelID < nlevel; levelID++ )
 	    {
-	      offset   = gridsize*levelID;
+	      int offset = gridsize*levelID;
 	      single2  = t2 + offset;
 
 	      minmaxval(gridsize, single2, imiss, &minval, &maxval);
@@ -728,7 +702,7 @@ void *Remapeta(void *argument)
 	  nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
 	  for ( levelID = 0; levelID < nlevel; levelID++ )
 	    {
-	      offset   = gridsize*levelID;
+	      int offset = gridsize*levelID;
 	      single2  = q2 + offset;
 
 	      corr_hum(gridsize, single2, MIN_Q);
@@ -751,22 +725,17 @@ void *Remapeta(void *argument)
 	{
 	  varID = varids[iv];
 
-	  nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+	  int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
 
 	  if ( operatorID == REMAPETAS )
 	    {
-	      nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	      vert_sum(sum1, vars1[iv], gridsize, nhlevf1);
-
-	      nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
 	      vert_sum(sum2, vars2[iv], gridsize, nhlevf2);
 	    }
 	  else if ( operatorID == REMAPETAZ )
 	    {
 	      int k;
 
-	      nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-
 	      presh(NULL, half_press1, vct1, ps1, nhlevf1, gridsize);
 	      for ( k = 0; k < nhlevf1; ++k )
 		for ( i = 0; i < gridsize; ++i )
@@ -776,8 +745,6 @@ void *Remapeta(void *argument)
 		  }
 	      vert_sumw(sum1, vars1[iv], gridsize, nhlevf1, deltap1);
 
-	      nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-
 	      presh(NULL, half_press2, vct2, ps1, nhlevf2, gridsize);
 	      for ( k = 0; k < nhlevf2; ++k )
 		for ( i = 0; i < gridsize; ++i )
@@ -790,7 +757,7 @@ void *Remapeta(void *argument)
 
 	  for ( levelID = 0; levelID < nlevel; levelID++ )
 	    {
-	      offset   = gridsize*levelID;
+	      int offset = gridsize*levelID;
 	      single2  = vars2[iv] + offset;
 
 	      if ( operatorID == REMAPETAS || operatorID == REMAPETAZ )
@@ -820,45 +787,45 @@ void *Remapeta(void *argument)
     {
       for ( varID = 0; varID < nvars3D; varID++ )
 	{
-	  free(vars2[varID]);
-	  free(vars1[varID]);
+	  Free(vars2[varID]);
+	  Free(vars1[varID]);
 	}
-      free(vars2);
-      free(vars1);
+      Free(vars2);
+      Free(vars1);
     }
 
   if ( ltq )
     {
-      free(q2);
-      free(t2);
-      free(q1);
-      free(t1);
-      free(secor);
-      free(pscor);
-      free(tscor);
+      Free(q2);
+      Free(t2);
+      Free(q1);
+      Free(t1);
+      Free(secor);
+      Free(pscor);
+      Free(tscor);
     }
 
-  if ( imiss ) free(imiss);
+  if ( imiss ) Free(imiss);
 
-  free(ps2);
-  free(fis2);
-  free(ps1);
-  free(fis1);
+  Free(ps2);
+  Free(fis2);
+  Free(ps1);
+  Free(fis1);
 
-  if ( sum1 ) free(sum1);
-  if ( sum2 ) free(sum2);
+  if ( sum1 ) Free(sum1);
+  if ( sum2 ) Free(sum2);
 
-  if ( deltap1 ) free(deltap1);
-  if ( deltap2 ) free(deltap2);
+  if ( deltap1 ) Free(deltap1);
+  if ( deltap2 ) Free(deltap2);
 
-  if ( half_press1 ) free(half_press1);
-  if ( half_press2 ) free(half_press2);
+  if ( half_press1 ) Free(half_press1);
+  if ( half_press2 ) Free(half_press2);
 
-  free(array);
-  free(vct2);
-  if ( vct1 ) free(vct1);
+  Free(array);
+  Free(vct2);
+  if ( vct1 ) Free(vct1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
