@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -31,78 +31,54 @@
 
 void *Replace(void *argument)
 {
-  int varID;
-  int varID1, nvars1;
-  int varID2, nvars2;
+  int varID, varID1, varID2;
   int nrecs = 0;
-  int tsID, recID, levelID, levelID2;
+  int levelID, levelID2;
   int nrecs2;
   int nchvars = 0;
   int idx;
-  int streamID1, streamID2, streamID3;
-  int vlistID1 , vlistID2, vlistID3;
-  int code1 = 0, code2;
-  int nlevel1, nlevel2;
   char varname1[CDI_MAX_NAME], varname2[CDI_MAX_NAME];
-  int gridsize;
   int nmiss;
-  int taxisID1, taxisID3;
-  int offset;
-  int nts2;
   int varlist1[MAX_VARS], varlist2[MAX_VARS];
   int **varlevel = NULL;
   int **varnmiss2 = NULL;
   double **vardata2 = NULL;
-  double *array = NULL, *parray;
+  double *parray;
 
   cdoInitialize(argument);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID3 = taxisDuplicate(taxisID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID3 = taxisDuplicate(taxisID1);
 
-  streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
 
-  vlistID2 = streamInqVlist(streamID2);
+  int vlistID2 = streamInqVlist(streamID2);
 
   /* compare all variables in vlistID2 */
 
-  nvars1 = vlistNvars(vlistID1);
-  nvars2 = vlistNvars(vlistID2);
+  int nvars1 = vlistNvars(vlistID1);
+  int nvars2 = vlistNvars(vlistID2);
 
   for ( varID2 = 0; varID2 < nvars2; varID2++ )
     {
-      code2 = vlistInqVarCode(vlistID2, varID2);
       vlistInqVarName(vlistID2, varID2, varname2);
 
       for ( varID1 = 0; varID1 < nvars1; varID1++ )
 	{
-	  code1 = vlistInqVarCode(vlistID1, varID1);
 	  vlistInqVarName(vlistID1, varID1, varname1);
 	  if ( strcmp(varname1, varname2) == 0 ) break;
 	}
 
-      if ( code2 > 0 && varID1 == nvars1 )
-	{
-	  for ( varID1 = 0; varID1 < nvars1; varID1++ )
-	    {
-	      code1 = vlistInqVarCode(vlistID1, varID1);
-	      vlistInqVarName(vlistID1, varID1, varname1);
-	      if ( code1 == code2 ) break;
-	    }
-	}
-
       if ( varID1 < nvars1 )
 	{
-	  int gridsize1, gridsize2, nlevel1, nlevel2;
+	  int gridsize1 = gridInqSize(vlistInqVarGrid(vlistID1, varID1));
+	  int nlevel1   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
 
-	  gridsize1 = gridInqSize(vlistInqVarGrid(vlistID1, varID1));
-	  nlevel1   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
-
-	  gridsize2 = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
-	  nlevel2   = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID2));
+	  int gridsize2 = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
+	  int nlevel2   = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID2));
 
 	  if ( gridsize1 != gridsize2 )
 	    cdoAbort("Variables have different gridsize!");
@@ -110,9 +86,7 @@ void *Replace(void *argument)
 	  if ( nlevel1 < nlevel2 )
 	    cdoAbort("Variables have different number of levels!");
 
-	  if ( cdoVerbose )
-	    cdoPrint("Variable %s (code %d) replaced by  %s (code %d)",
-		     varname1, code1, varname2, code2);
+	  if ( cdoVerbose ) cdoPrint("Variable %s replaced.", varname1);
 
 	  varlist1[nchvars] = varID1;
 	  varlist2[nchvars] = varID2;
@@ -121,35 +95,35 @@ void *Replace(void *argument)
 	}
       else
 	{
-	  cdoPrint("Variable %s (code %d) not found!", varname2, code2);
+	  cdoPrint("Variable %s not found!", varname2);
 	}
     }
 
   if ( nchvars )
     {
-      vardata2  = (double **) malloc(nchvars*sizeof(double *));
-      varnmiss2 = (int **) malloc(nchvars*sizeof(int *));
-      varlevel  = (int **) malloc(nchvars*sizeof(int *));
+      vardata2  = (double **) Malloc(nchvars*sizeof(double *));
+      varnmiss2 = (int **) Malloc(nchvars*sizeof(int *));
+      varlevel  = (int **) Malloc(nchvars*sizeof(int *));
       for ( idx = 0; idx < nchvars; idx++ )
 	{
 	  varID1 = varlist1[idx];
 	  varID2 = varlist2[idx];
-	  nlevel1  = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
-	  nlevel2  = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID2));
-	  gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
-	  vardata2[idx]  = (double*) malloc(nlevel2*gridsize*sizeof(double));
-	  varnmiss2[idx] = (int*) malloc(nlevel2*sizeof(int));
-	  varlevel[idx] = (int*) malloc(nlevel1*sizeof(int));
+	  int nlevel1  = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
+	  int nlevel2  = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID2));
+	  int gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
+	  vardata2[idx]  = (double*) Malloc(nlevel2*gridsize*sizeof(double));
+	  varnmiss2[idx] = (int*) Malloc(nlevel2*sizeof(int));
+	  varlevel[idx] = (int*) Malloc(nlevel1*sizeof(int));
 	  /*
 	  for ( levelID = 0; levelID < nlevel1; levelID++ )
 	    varlevel[idx][levelID] = levelID;
 	  */
 	  if ( nlevel2 <= nlevel1 )
 	    {
-	      double *level1 = (double*) malloc(nlevel1*sizeof(double));
-	      double *level2 = (double*) malloc(nlevel2*sizeof(double));
-	      zaxisInqLevels(vlistInqVarZaxis(vlistID1, varID1), level1);
-	      zaxisInqLevels(vlistInqVarZaxis(vlistID2, varID2), level2);
+	      double *level1 = (double*) Malloc(nlevel1*sizeof(double));
+	      double *level2 = (double*) Malloc(nlevel2*sizeof(double));
+	      cdoZaxisInqLevels(vlistInqVarZaxis(vlistID1, varID1), level1);
+	      cdoZaxisInqLevels(vlistInqVarZaxis(vlistID2, varID2), level2);
 
 	      for ( levelID = 0; levelID < nlevel1; levelID++ )
 		varlevel[idx][levelID] = -1;
@@ -167,25 +141,25 @@ void *Replace(void *argument)
 		  if ( l1 == nlevel1 ) cdoWarning("Level %g not found!", level2[l2]);
 		}
 
-	      free(level1);
-	      free(level2);
+	      Free(level1);
+	      Free(level2);
 	    }
 	}
     }
 
-  vlistID3 = vlistDuplicate(vlistID1);
+  int vlistID3 = vlistDuplicate(vlistID1);
 
-  streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
+  int streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
 
   vlistDefTaxis(vlistID3, taxisID3);
   streamDefVlist(streamID3, vlistID3);
 
-  gridsize = vlistGridsizeMax(vlistID1);
-  array = (double*) malloc(gridsize*sizeof(double));
+  int gridsize = vlistGridsizeMax(vlistID1);
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
-  nts2 = vlistNtsteps(vlistID2);
+  int nts2 = vlistNtsteps(vlistID2);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID3, taxisID1);
@@ -196,15 +170,15 @@ void *Replace(void *argument)
 	  if ( nrecs2 == 0 )
 	    cdoAbort("Input streams have different number of timesteps!");
 
-	  for ( recID = 0; recID < nrecs2; recID++ )
+	  for ( int recID = 0; recID < nrecs2; recID++ )
 	    {
 	      streamInqRecord(streamID2, &varID, &levelID);
 	      
 	      for ( idx = 0; idx < nchvars; idx++ )
 		if ( varlist2[idx] == varID )
 		  {
-		    gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-		    offset   = gridsize*levelID;
+		    int gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
+		    int offset   = gridsize*levelID;
 		    parray   = vardata2[idx]+offset;
 		    streamReadRecord(streamID2, parray, &nmiss);
 		    varnmiss2[idx][levelID] = nmiss;
@@ -215,7 +189,7 @@ void *Replace(void *argument)
 
       streamDefTimestep(streamID3, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -227,8 +201,8 @@ void *Replace(void *argument)
 		levelID2 = varlevel[idx][levelID];
 		if ( levelID2 != -1 )
 		  {
-		    gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-		    offset   = gridsize*levelID2;
+		    int gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+		    int offset   = gridsize*levelID2;
 		    parray   = vardata2[idx]+offset;
 		    nmiss    = varnmiss2[idx][levelID2];
 		    break;
@@ -252,19 +226,19 @@ void *Replace(void *argument)
     {
       for ( idx = 0; idx < nchvars; idx++ )
 	{
-	  free(vardata2[idx]);
-	  free(varnmiss2[idx]);
-	  free(varlevel[idx]);
+	  Free(vardata2[idx]);
+	  Free(varnmiss2[idx]);
+	  Free(varlevel[idx]);
 	}
 
-      free(vardata2);
-      free(varnmiss2);
-      free(varlevel);
+      Free(vardata2);
+      Free(varnmiss2);
+      Free(varlevel);
     }
 
-  if ( array ) free(array);
+  if ( array ) Free(array);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
