@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
       Rotuv      rotuvb          Backward rotation
 */
 
-#include <ctype.h>
 
 #include <cdi.h>
 #include "cdo.h"
@@ -33,43 +32,35 @@
 static
 void rot_uv_back(int gridID, double *us, double *vs)
 {
-  long i, ilat, ilon, nlat, nlon;
-  double u, v;
-  double xval, yval;
   double xpole, ypole, angle;
-  double *xvals, *yvals;
+  if ( gridInqType(gridID) == GRID_PROJECTION && gridInqProjType(gridID) == CDI_PROJ_RLL )
+    gridInqParamRLL(gridID, &xpole, &ypole, &angle);
 
-  nlon = gridInqXsize(gridID);
-  nlat = gridInqYsize(gridID);
-
-  xpole = gridInqXpole(gridID);
-  ypole = gridInqYpole(gridID);
-  angle = gridInqAngle(gridID);
-
-  xvals = (double*) Malloc(nlon*sizeof(double));
-  yvals = (double*) Malloc(nlat*sizeof(double));
+  long nlon = gridInqXsize(gridID);
+  long nlat = gridInqYsize(gridID);
 
+  double *xvals = (double*) Malloc(nlon*sizeof(double));
+  double *yvals = (double*) Malloc(nlat*sizeof(double));
   gridInqXvals(gridID, xvals);
   gridInqYvals(gridID, yvals);
 
   /* Convert lat/lon units if required */
-  {
-    char units[CDI_MAX_NAME];
-    gridInqXunits(gridID, units);
-    grid_to_degree(units, 1, &xpole, "xpole");
-    grid_to_degree(units, nlon, xvals, "grid center lon");
-    gridInqYunits(gridID, units);
-    grid_to_degree(units, 1, &ypole, "ypole");
-    grid_to_degree(units, nlat, yvals, "grid center lat");
-  }
+  char units[CDI_MAX_NAME];
+  gridInqXunits(gridID, units);
+  grid_to_degree(units, 1, &xpole, "xpole");
+  grid_to_degree(units, nlon, xvals, "grid center lon");
+  gridInqYunits(gridID, units);
+  grid_to_degree(units, 1, &ypole, "ypole");
+  grid_to_degree(units, nlat, yvals, "grid center lat");
 
-  for ( ilat = 0; ilat < nlat; ilat++ )
-    for ( ilon = 0; ilon < nlon; ilon++ )
+  double u, v;
+  for ( long ilat = 0; ilat < nlat; ilat++ )
+    for ( long ilon = 0; ilon < nlon; ilon++ )
       {
-	i = ilat*nlon + ilon;
+	long i = ilat*nlon + ilon;
 
-        xval = lamrot_to_lam(yvals[ilat], xvals[ilon], ypole, xpole, angle);
-        yval = phirot_to_phi(yvals[ilat], xvals[ilon], ypole, angle);
+        double xval = lamrot_to_lam(yvals[ilat], xvals[ilon], ypole, xpole, angle);
+        double yval = phirot_to_phi(yvals[ilat], xvals[ilon], ypole, angle);
 
 	usvs_to_uv(us[i], vs[i], yval, xval, ypole, xpole, &u, &v);
 	/*
@@ -89,15 +80,12 @@ void rot_uv_back(int gridID, double *us, double *vs)
 
 void *Rotuv(void *argument)
 {
-  int recID, varID, levelID;
+  int varID, levelID;
   int varID1, varID2, nlevel1, nlevel2;
   int gridsize;
   int code, gridID;
   int offset;
   int nlevel;
-  int lvar = FALSE;
-  int i;
-  int lfound[MAXARG];
   int chcodes[MAXARG];
   char *chvars[MAXARG];
   char varname[CDI_MAX_NAME];
@@ -110,25 +98,26 @@ void *Rotuv(void *argument)
   int nch = operatorArgc();
   if ( nch%2 ) cdoAbort("Odd number of input arguments!");
 
-  int lcode = TRUE;
+  bool lvar = false;
+  bool lcode = true;
   int len = (int)strlen(operatorArgv()[0]);
-  for ( i = 0; i < len; ++i )
+  for ( int i = 0; i < len; ++i )
     if ( !isdigit(operatorArgv()[0][i]) )
       {
-        lcode = FALSE;
+        lcode = false;
         break;
       }
 
   if ( lcode )
     {
-      lvar = FALSE;
-      for ( i = 0; i < nch; i++ )
+      lvar = false;
+      for ( int i = 0; i < nch; i++ )
 	chcodes[i] = parameter2int(operatorArgv()[i]);
     }
   else
     {
-      lvar = TRUE;
-      for ( i = 0; i < nch; i++ )
+      lvar = true;
+      for ( int i = 0; i < nch; i++ )
 	chvars[i] = operatorArgv()[i];
     }
 
@@ -146,17 +135,18 @@ void *Rotuv(void *argument)
   int **varnmiss    = (int **) Malloc(nvars*sizeof(int *));
   double **vardata  = (double **) Malloc(nvars*sizeof(double *));
 
-  for ( i = 0; i < nch; i++ ) lfound[i] = FALSE;
+  bool lfound[MAXARG];
+  for ( int i = 0; i < nch; i++ ) lfound[i] = false;
 
   if ( lvar )
     {
       for ( varID = 0; varID < nvars; varID++ )
 	{
 	  vlistInqVarName(vlistID2, varID, varname);
-	  for ( i = 0; i < nch; i++ )
-	    if ( strcmp(varname, chvars[i]) == 0 ) lfound[i] = TRUE;
+	  for ( int i = 0; i < nch; i++ )
+	    if ( strcmp(varname, chvars[i]) == 0 ) lfound[i] = true;
 	}
-      for ( i = 0; i < nch; i++ )
+      for ( int i = 0; i < nch; i++ )
 	if ( ! lfound[i] ) cdoAbort("Variable %s not found!", chvars[i]);
     }
   else
@@ -164,18 +154,18 @@ void *Rotuv(void *argument)
       for ( varID = 0; varID < nvars; varID++ )
 	{
 	  code = vlistInqVarCode(vlistID2, varID);
-	  for ( i = 0; i < nch; i++ )
-	    if ( code == chcodes[i] ) lfound[i] = TRUE;
+	  for ( int i = 0; i < nch; i++ )
+	    if ( code == chcodes[i] ) lfound[i] = true;
 	}
-      for ( i = 0; i < nch; i++ )
+      for ( int i = 0; i < nch; i++ )
 	if ( ! lfound[i] ) cdoAbort("Code %d not found!", chcodes[i]);
     }
 
   for ( varID = 0; varID < nvars; varID++ )
     {
       gridID = vlistInqVarGrid(vlistID1, varID);
-      if ( ! (gridInqType(gridID) == GRID_LONLAT && gridIsRotated(gridID)) )
-	cdoAbort("Only rotated lon/lat grids supported");
+      if ( ! (gridInqType(gridID) == GRID_PROJECTION && gridInqProjType(gridID) == CDI_PROJ_RLL) )
+	cdoAbort("Only rotated lon/lat grids supported!");
 
       gridsize = gridInqSize(gridID);
       nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
@@ -198,7 +188,7 @@ void *Rotuv(void *argument)
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -213,7 +203,7 @@ void *Rotuv(void *argument)
 	    cdoAbort("Missing values unsupported for this operator!");
 	}
 
-      for ( i = 0; i < nch; i += 2 )
+      for ( int i = 0; i < nch; i += 2 )
 	{
 	  for ( varID = 0; varID < nvars; varID++ )
 	    {
@@ -277,7 +267,7 @@ void *Rotuv(void *argument)
 	    }
 	}
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  varID    = recVarID[recID];
 	  levelID  = recLevelID[recID];
