@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,9 @@
       Seasstat   seasmean        Seasonal mean
       Seasstat   seasavg         Seasonal average
       Seasstat   seasvar         Seasonal variance
+      Seasstat   seasvar1        Seasonal variance [Normalize by (n-1)]
       Seasstat   seasstd         Seasonal standard deviation
+      Seasstat   seasstd1        Seasonal standard deviation [Normalize by (n-1)]
 */
 
 
@@ -38,24 +40,15 @@
 void *Seasstat(void *argument)
 {
   int timestat_date = TIMESTAT_MEAN;
-  int gridsize;
-  int vdate = 0, vtime = 0;
   int vdate0 = 0, vtime0 = 0;
   int vdate1 = 0, vtime1 = 0;
   int nrecs;
-  int varID, levelID, recID;
-  int tsID;
-  int otsID;
-  long nsets;
-  int i;
-  int year, month, day, seas, seas0 = 0;
+  int varID, levelID;
+  int year, month, day, seas0 = 0;
   int nmiss;
   int nlevel;
-  int newseas, oldmon = 0, newmon;
+  int oldmon = 0;
   int nseason = 0;
-  field_t **vars1 = NULL, **vars2 = NULL, **samp1 = NULL;
-  field_t field;
-  int season_start;
   const char *seas_name[4];
 
   cdoInitialize(argument);
@@ -66,14 +59,21 @@ void *Seasstat(void *argument)
   cdoOperatorAdd("seasmean", func_mean, 0, NULL);
   cdoOperatorAdd("seasavg",  func_avg,  0, NULL);
   cdoOperatorAdd("seasvar",  func_var,  0, NULL);
+  cdoOperatorAdd("seasvar1", func_var1, 0, NULL);
   cdoOperatorAdd("seasstd",  func_std,  0, NULL);
+  cdoOperatorAdd("seasstd1", func_std1, 0, NULL);
 
   int operatorID = cdoOperatorID();
   int operfunc = cdoOperatorF1(operatorID);
 
-  season_start = get_season_start();
+  int season_start = get_season_start();
   get_season_name(seas_name);
 
+  int lmean   = operfunc == func_mean || operfunc == func_avg;
+  int lstd    = operfunc == func_std || operfunc == func_std1;
+  int lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  int divisor = operfunc == func_std1 || operfunc == func_var1;
+
   int streamID1 = streamOpenRead(cdoStreamName(0));
 
   int vlistID1 = streamInqVlist(streamID1);
@@ -91,60 +91,43 @@ void *Seasstat(void *argument)
   int nvars    = vlistNvars(vlistID1);
   int nrecords = vlistNrecs(vlistID1);
 
-  int *recVarID   = (int*) malloc(nrecords*sizeof(int));
-  int *recLevelID = (int*) malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
   dtlist_type *dtlist = dtlist_new();
   dtlist_set_stat(dtlist, timestat_date);
   dtlist_set_calendar(dtlist, taxisInqCalendar(taxisID1));
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
 
+  field_type field;
   field_init(&field);
-  field.ptr = (double*) malloc(gridsize*sizeof(double));
+  field.ptr = (double*) Malloc(gridsize*sizeof(double));
 
-  vars1 = field_malloc(vlistID1, FIELD_PTR);
-  samp1 = field_malloc(vlistID1, FIELD_NONE);
-  if ( operfunc == func_std || operfunc == func_var )
-    vars2 = field_malloc(vlistID1, FIELD_PTR);
+  field_type **samp1 = field_malloc(vlistID1, FIELD_NONE);
+  field_type **vars1 = field_malloc(vlistID1, FIELD_PTR);
+  field_type **vars2 = NULL;
+  if ( lvarstd ) vars2 = field_malloc(vlistID1, FIELD_PTR);
 
-  tsID    = 0;
-  otsID   = 0;
+  int tsID    = 0;
+  int otsID   = 0;
   while ( TRUE )
     {
-      nsets = 0;
-      newseas = FALSE;
+      long nsets = 0;
+      bool newseas = false;
       while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
 	{
 	  dtlist_taxisInqTimestep(dtlist, taxisID1, nsets);
-	  vdate = dtlist_get_vdate(dtlist, nsets);
-	  vtime = dtlist_get_vtime(dtlist, nsets);
+	  int vdate = dtlist_get_vdate(dtlist, nsets);
+	  int vtime = dtlist_get_vtime(dtlist, nsets);
 
 	  cdiDecodeDate(vdate, &year, &month, &day);
-	  if ( month < 1 || month > 12 )
-	    cdoAbort("Month %d out of range!", month);
 
-	  newmon = month;
+	  int newmon = month;
 
-	  if ( season_start == START_DEC )
-	    {
-	      if ( newmon == 12 ) newmon = 0;
+	  if ( season_start == START_DEC && newmon == 12 ) newmon = 0;
 
-	      if ( month <= 12 )
-		seas = (month % 12) / 3;
-	      else
-		seas = month - 13;
-	    }
-	  else
-	    {
-	      if ( month <= 12 )
-		seas = (month - 1) / 3;
-	      else
-		seas = month - 13;
-	    }
-
-	  if ( seas < 0 || seas > 3 )
-	    cdoAbort("Season %d out of range!", seas+1);
+          int seas = month_to_season(month);
 
 	  if ( nsets == 0 )
 	    {
@@ -155,13 +138,13 @@ void *Seasstat(void *argument)
 	      oldmon = newmon;
 	    }
 
-	  if ( newmon < oldmon ) newseas = TRUE;
+	  if ( newmon < oldmon ) newseas = true;
 
 	  if ( (seas != seas0) || newseas ) break;
 
 	  oldmon = newmon;
 
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      streamInqRecord(streamID1, &varID, &levelID);
 
@@ -181,9 +164,9 @@ void *Seasstat(void *argument)
 		  if ( nmiss > 0 || samp1[varID][levelID].ptr )
 		    {
 		      if ( samp1[varID][levelID].ptr == NULL )
-			samp1[varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
+			samp1[varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 
-		      for ( i = 0; i < gridsize; i++ )
+		      for ( int i = 0; i < gridsize; i++ )
 			if ( DBL_IS_EQUAL(vars1[varID][levelID].ptr[i],
 					  vars1[varID][levelID].missval) )
 			  samp1[varID][levelID].ptr[i] = 0;
@@ -193,7 +176,8 @@ void *Seasstat(void *argument)
 		}
 	      else
 		{
-		  streamReadRecord(streamID1, field.ptr, &field.nmiss);
+		  streamReadRecord(streamID1, field.ptr, &nmiss);
+                  field.nmiss   = (size_t)nmiss;
 		  field.grid    = vars1[varID][levelID].grid;
 		  field.missval = vars1[varID][levelID].missval;
 
@@ -201,17 +185,17 @@ void *Seasstat(void *argument)
 		    {
 		      if ( samp1[varID][levelID].ptr == NULL )
 			{
-			  samp1[varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
-			  for ( i = 0; i < gridsize; i++ )
+			  samp1[varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
+			  for ( int i = 0; i < gridsize; i++ )
 			    samp1[varID][levelID].ptr[i] = nsets;
 			}
 
-		      for ( i = 0; i < gridsize; i++ )
+		      for ( int i = 0; i < gridsize; i++ )
 			if ( !DBL_IS_EQUAL(field.ptr[i], vars1[varID][levelID].missval) )
 			  samp1[varID][levelID].ptr[i]++;
 		    }
 
-		  if ( operfunc == func_std || operfunc == func_var )
+		  if ( lvarstd )
 		    {
 		      farsumq(&vars2[varID][levelID], field);
 		      farsum(&vars1[varID][levelID], field);
@@ -223,7 +207,7 @@ void *Seasstat(void *argument)
 		}
 	    }
 
-	  if ( nsets == 0 && (operfunc == func_std || operfunc == func_var) )
+	  if ( nsets == 0 && lvarstd )
 	    for ( varID = 0; varID < nvars; varID++ )
 	      {
 		if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
@@ -240,7 +224,7 @@ void *Seasstat(void *argument)
 
       if ( nrecs == 0 && nsets == 0 ) break;
 
-      if ( operfunc == func_mean || operfunc == func_avg )
+      if ( lmean )
 	for ( varID = 0; varID < nvars; varID++ )
 	  {
 	    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
@@ -248,12 +232,12 @@ void *Seasstat(void *argument)
 	    for ( levelID = 0; levelID < nlevel; levelID++ )
 	      {
 		if ( samp1[varID][levelID].ptr == NULL )
-		  farcmul(&vars1[varID][levelID], 1.0/nsets);
+		  farcdiv(&vars1[varID][levelID], (double)nsets);
 		else
 		  fardiv(&vars1[varID][levelID], samp1[varID][levelID]);
 	      }
 	  }
-      else if ( operfunc == func_std || operfunc == func_var )
+      else if ( lvarstd )
 	for ( varID = 0; varID < nvars; varID++ )
 	  {
 	    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
@@ -262,18 +246,17 @@ void *Seasstat(void *argument)
 	      {
 		if ( samp1[varID][levelID].ptr == NULL )
 		  {
-		    if ( operfunc == func_std )
-		      farcstd(&vars1[varID][levelID], vars2[varID][levelID], 1.0/nsets);
+		    if ( lstd )
+		      farcstd(&vars1[varID][levelID], vars2[varID][levelID], nsets, divisor);
 		    else
-		      farcvar(&vars1[varID][levelID], vars2[varID][levelID], 1.0/nsets);
+		      farcvar(&vars1[varID][levelID], vars2[varID][levelID], nsets, divisor);
 		  }
 		else
 		  {
-		    farinv(&samp1[varID][levelID]);
-		    if ( operfunc == func_std )
-		      farstd(&vars1[varID][levelID], vars2[varID][levelID], samp1[varID][levelID]);
+		    if ( lstd )
+		      farstd(&vars1[varID][levelID], vars2[varID][levelID], samp1[varID][levelID], divisor);
 		    else
-		      farvar(&vars1[varID][levelID], vars2[varID][levelID], samp1[varID][levelID]);
+		      farvar(&vars1[varID][levelID], vars2[varID][levelID], samp1[varID][levelID], divisor);
 		  }
 	      }
 	  }
@@ -286,9 +269,8 @@ void *Seasstat(void *argument)
 	  time2str(vtime0, vtimestr0, sizeof(vtimestr0));
 	  date2str(vdate1, vdatestr1, sizeof(vdatestr1));
 	  time2str(vtime1, vtimestr1, sizeof(vtimestr1));
-	  cdoPrint("season %3d %3s start %s %s end %s %s ntimesteps %d", 
-		   nseason, seas_name[seas0],
-		   vdatestr0, vtimestr0, vdatestr1, vtimestr1, nsets);
+	  cdoPrint("season: %3d %3s  start: %s %s  end: %s %s ntimesteps: %d", 
+		   nseason, seas_name[seas0], vdatestr0, vtimestr0, vdatestr1, vtimestr1, nsets);
 	}
 
       dtlist_stat_taxisDefTimestep(dtlist, taxisID2, nsets);
@@ -302,7 +284,7 @@ void *Seasstat(void *argument)
 		     otsID+1, vdatestr, nsets, nsets == 1 ? "" : "s");
 	}
 
-      for ( recID = 0; recID < nrecords; recID++ )
+      for ( int recID = 0; recID < nrecords; recID++ )
 	{
 	  varID   = recVarID[recID];
 	  levelID = recLevelID[recID];
@@ -310,7 +292,7 @@ void *Seasstat(void *argument)
 	  if ( otsID && vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 
 	  streamDefRecord(streamID2, varID, levelID);
-	  streamWriteRecord(streamID2, vars1[varID][levelID].ptr,  vars1[varID][levelID].nmiss);
+	  streamWriteRecord(streamID2, vars1[varID][levelID].ptr, (int)vars1[varID][levelID].nmiss);
 	}
 
       if ( nrecs == 0 ) break;
@@ -320,12 +302,12 @@ void *Seasstat(void *argument)
 
   field_free(vars1, vlistID1);
   field_free(samp1, vlistID1);
-  if ( operfunc == func_std || operfunc == func_var ) field_free(vars2, vlistID1);
+  if ( lvarstd ) field_free(vars2, vlistID1);
 
-  if ( field.ptr ) free(field.ptr);
+  if ( field.ptr ) Free(field.ptr);
 
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
 
   dtlist_delete(dtlist);
 
@@ -334,5 +316,5 @@ void *Seasstat(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
