@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2013 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,75 +27,69 @@
 #include "pstream.h"
 #include "error.h"
 #include "util.h"
-#include "list.h"
+#include "listarray.h"
 
 
 void *Selrec(void *argument)
 {
-  int streamID1, streamID2;
-  int tsID, nrecs;
-  int recID, varID, levelID;
-  int *intarr, nsel = 0;
-  int vlistID1 = -1, vlistID2 = -1;
-  int i;
-  int recordID;
-  int filetype;
-  int taxisID1, taxisID2;
-  LIST *ilist = listNew(INT_LIST);
+  int nrecs;
+  int varID, levelID;
+  lista_t *ilista = lista_new(INT_LISTA);
 
   cdoInitialize(argument);
 
-  if ( processSelf() != 0 && *(char *)argument == '-' )
-    cdoAbort("This operator does not work with pipes!");
+  if ( processSelf() != 0 ) cdoAbort("This operator can't be combined with other operators!");
 
   operatorInputArg("records");
 
-  nsel = args2intlist(operatorArgc(), operatorArgv(), ilist);
+  int nsel = args2int_lista(operatorArgc(), operatorArgv(), ilista);
 
-  intarr = (int *) listArrayPtr(ilist);
+  int *intarr = (int *) lista_dataptr(ilista);
 
   if ( cdoVerbose )
     {
-      for ( i = 0; i < nsel; i++ )
+      for ( int i = 0; i < nsel; i++ )
 	cdoPrint("intarr entry: %d %d", i, intarr[i]);
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  filetype = streamInqFiletype(streamID1);
+  int filetype = streamInqFiletype(streamID1);
 
-  if ( filetype == FILETYPE_NC || filetype == FILETYPE_NC2 )
-    cdoAbort("This operator does not work on netCDF data!");
+  if ( filetype == CDI_FILETYPE_NC || filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C )
+    cdoAbort("This operator does not work on NetCDF data!");
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  recordID = 0;
-  tsID = 0;
+  int recordID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID);
      
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  recordID++;
 	  streamInqRecord(streamID1, &varID, &levelID);
-	  for ( i = 0; i < nsel; i++ )
+
+	  for ( int i = 0; i < nsel; i++ )
 	    {
 	      if ( recordID == intarr[i] )
 		{
 		  streamDefRecord(streamID2, varID, levelID);
 		  streamCopyRecord(streamID2, streamID1);
+
 		  break;
 		}
 	    }
@@ -107,9 +101,9 @@ void *Selrec(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  listDelete(ilist);
+  lista_destroy(ilista);
 
   cdoFinish();
 
-  return (NULL);
+  return 0;
 }
