@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,6 @@
       Seltime    selsmon         Select single month
 */
 
-#include <ctype.h>
 
 #include <cdi.h>
 #include "cdo.h"
@@ -37,7 +36,7 @@
 #include "pstream.h"
 #include "error.h"
 #include "util.h"
-#include "list.h"
+#include "listarray.h"
 
 
 void season_to_months(const char *season, int *imonths)
@@ -64,7 +63,7 @@ void season_to_months(const char *season, int *imonths)
 }
 
 static
-int seaslist(LIST *ilist)
+int seaslist(lista_t *ilista)
 {
   int imon[13]; /* 1-12 ! */
   for ( int i = 0; i < 13; ++i ) imon[i] = 0;
@@ -89,7 +88,7 @@ int seaslist(LIST *ilist)
     }
 
   nsel = 0;
-  for ( int i = 1; i < 13; ++i ) if ( imon[i] ) listSetInt(ilist, nsel++, i);
+  for ( int i = 1; i < 13; ++i ) if ( imon[i] ) lista_set_int(ilista, nsel++, i);
 
   return nsel;
 }
@@ -125,9 +124,9 @@ double datestr_to_double(const char *datestr, int opt)
 }
 
 static
-int datelist(LIST *flist)
+int datelist(lista_t *flista)
 {
-  int set2 = TRUE;
+  bool set2 = true;
   double fval = 0;
 
   int nsel = operatorArgc();
@@ -143,28 +142,28 @@ int datelist(LIST *flist)
 	  else
 	    fval =  99999999999.;
 
-	  listSetFlt(flist, i,  fval);
+	  lista_set_flt(flista, i,  fval);
 	}
       else
 	{
 	  fval = datestr_to_double(operatorArgv()[i], 0);
 	  if ( strchr(operatorArgv()[i], 'T') )
-	    set2 = FALSE;
+	    set2 = false;
 	  else if ( nsel > 1 && i > 0 )
 	    fval += 0.999;
 	}
 
-      listSetFlt(flist, i, fval);
+      lista_set_flt(flista, i, fval);
     }
 
-  if ( nsel == 1 && set2 == TRUE )
+  if ( nsel == 1 && set2 )
     {
       fval += 0.999;
-      listSetFlt(flist, nsel, fval);
+      lista_set_flt(flista, nsel, fval);
       nsel = 2;
     }
 
-  return (nsel);
+  return nsel;
 }
 
 
@@ -172,33 +171,30 @@ void *Seltime(void *argument)
 {
   int streamID2 = -1;
   int nrecs;
-  int recID, varID, levelID;
-  int nsel = 0, selival;
-  int vdate, vtime;
-  int copytimestep;
-  int copy_nts2 = FALSE;
-  int i, isel;
-  int gridsize;
+  int varID, levelID;
+  int nsel = 0;
+  int i;
   int nmiss;
   int ncts = 0, nts, it;
-  int *selfound = NULL;
   int hour = 0, minute = 0, second = 0;
   int nts1 = 0, nts2 = 0;
   int its1 = 0, its2 = 0;
   double selfval = 0;
   double *array = NULL;
-  LIST *ilist = listNew(INT_LIST);
-  LIST *flist = listNew(FLT_LIST);
-  int gridID;
-  int nlevel;
-  int lconstout = FALSE;
-  int process_nts1 = FALSE, process_nts2 = FALSE;
+  lista_t *ilista = lista_new(INT_LISTA);
+  lista_t *flista = lista_new(FLT_LISTA);
+  bool copy_nts2 = false;
+  bool lconstout = false;
+  bool process_nts1 = false, process_nts2 = false;
+  bool *selfound = NULL;
   int *vdate_list = NULL, *vtime_list = NULL;
   double *single;
-  field_t ***vars = NULL;
+  field_type ***vars = NULL;
 
   cdoInitialize(argument);
 
+  bool lcopy = UNCHANGED_RECORD;
+
   int SELTIMESTEP = cdoOperatorAdd("seltimestep", func_step,     0, "timesteps");
   int SELDATE     = cdoOperatorAdd("seldate",     func_datetime, 0, "start date and end date (format YYYY-MM-DDThh:mm:ss)");
   int SELTIME     = cdoOperatorAdd("seltime",     func_time,     0, "times (format hh:mm:ss)");
@@ -213,18 +209,15 @@ void *Seltime(void *argument)
 
   int operfunc = cdoOperatorF1(operatorID);
 
-  int lcopy = FALSE;
-  if ( UNCHANGED_RECORD ) lcopy = TRUE;
-
   operatorInputArg(cdoOperatorEnter(operatorID));
 
   if ( operatorID == SELSEASON )
     {
-      nsel = seaslist(ilist);
+      nsel = seaslist(ilista);
     }
   else if ( operatorID == SELDATE )
     {
-      nsel = datelist(flist);
+      nsel = datelist(flista);
     }
   else if ( operatorID == SELTIME )
     {
@@ -235,23 +228,23 @@ void *Seltime(void *argument)
 	  if ( strchr(operatorArgv()[i], ':') )
 	    {
 	      sscanf(operatorArgv()[i], "%d:%d:%d", &hour, &minute, &second);
-	      listSetInt(ilist, i, cdiEncodeTime(hour, minute, second));
+	      lista_set_int(ilista, i, cdiEncodeTime(hour, minute, second));
 	    }
 	  else
 	    {
-	      listSetInt(ilist, i, parameter2int(operatorArgv()[i]));
+	      lista_set_int(ilista, i, parameter2int(operatorArgv()[i]));
 	    }
 	}
     }
   else
     {
-      nsel = args2intlist(operatorArgc(), operatorArgv(), ilist);
+      nsel = args2int_lista(operatorArgc(), operatorArgv(), ilista);
     }
 
   if ( nsel < 1 ) cdoAbort("No timestep selected!");
 
-  int *intarr = (int *) listArrayPtr(ilist);
-  double *fltarr = (double *) listArrayPtr(flist);
+  int *intarr = (int *) lista_dataptr(ilista);
+  double *fltarr = (double *) lista_dataptr(flista);
 
   if ( operatorID == SELSMON )
     {
@@ -269,8 +262,8 @@ void *Seltime(void *argument)
 
   if ( nsel )
     {
-      selfound = (int*) Malloc(nsel*sizeof(int));
-      for ( i = 0; i < nsel; i++ ) selfound[i] = FALSE;
+      selfound = (bool*) Malloc(nsel*sizeof(bool));
+      for ( i = 0; i < nsel; i++ ) selfound[i] = false;
     }
 
   int streamID1 = streamOpenRead(cdoStreamName(0));
@@ -287,7 +280,7 @@ void *Seltime(void *argument)
 
   if ( ! lcopy )
     {
-      gridsize = vlistGridsizeMax(vlistID1);
+      int gridsize = vlistGridsizeMax(vlistID1);
       if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
       array = (double*) Malloc(gridsize*sizeof(double));
     }
@@ -322,7 +315,7 @@ void *Seltime(void *argument)
   for ( varID = 0; varID < nvars; varID++ )
     if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) nconst++;
       
-  int lnts1 = operatorID == SELSMON && nts1 > 0;
+  bool lnts1 = (operatorID == SELSMON) && (nts1 > 0);
 
   if ( lnts1 || nconst )
     {
@@ -336,7 +329,7 @@ void *Seltime(void *argument)
 	  nts1 = 1;
 	}
 
-      vars  = (field_t ***) Malloc(nts1*sizeof(field_t **));
+      vars  = (field_type ***) Malloc(nts1*sizeof(field_type **));
 
       for ( int tsID = 0; tsID < nts1; tsID++ )
 	{
@@ -346,9 +339,9 @@ void *Seltime(void *argument)
 	    {
 	      if ( lnts1 || (vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT) )
 		{
-		  gridID  = vlistInqVarGrid(vlistID1, varID);
-		  nlevel  = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-		  gridsize = gridInqSize(gridID);
+		  int gridID  = vlistInqVarGrid(vlistID1, varID);
+		  int nlevel  = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+		  int gridsize = gridInqSize(gridID);
 		  
 		  for ( levelID = 0; levelID < nlevel; levelID++ )
 		    {
@@ -363,11 +356,11 @@ void *Seltime(void *argument)
   int tsID2 = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
-      vdate = taxisInqVdate(taxisID1);
-      vtime = taxisInqVtime(taxisID1);
+      int vdate = taxisInqVdate(taxisID1);
+      int vtime = taxisInqVtime(taxisID1);
 
-      copytimestep = FALSE;
-      selival = -1;
+      bool copytimestep = false;
+      int selival = -1;
 
       if ( operfunc == func_step )
 	{
@@ -398,9 +391,9 @@ void *Seltime(void *argument)
 	{
 	  if ( selfval >= fltarr[0] && selfval <= fltarr[nsel-1] )
 	    {
-	      copytimestep = TRUE;
-	      selfound[0]      = TRUE;
-	      selfound[nsel-1] = TRUE;
+	      copytimestep = true;
+	      selfound[0]      = true;
+	      selfound[nsel-1] = true;
 	    }
           else if ( selfval > fltarr[nsel-1] )
             {
@@ -412,29 +405,29 @@ void *Seltime(void *argument)
 	  for ( i = 0; i < nsel; i++ )
 	    if ( selival == intarr[i] )
 	      {
-		copytimestep = TRUE;
-		selfound[i] = TRUE;
+		copytimestep = true;
+		selfound[i] = true;
 		break;
 	      }
 	}
 
-      if ( operatorID == SELSMON && copytimestep == FALSE )
+      if ( operatorID == SELSMON && copytimestep == false )
 	{
-	  copy_nts2 = FALSE;
+	  copy_nts2 = false;
 
-	  if ( process_nts1 == TRUE )
+	  if ( process_nts1 )
 	    {
-	      process_nts2 = TRUE;
+	      process_nts2 = true;
 	      its2 = 0;
-	      process_nts1 = FALSE;
+	      process_nts1 = false;
 	    }
 
-	  if ( process_nts2 == TRUE )
+	  if ( process_nts2 )
 	    {
 	      if ( its2++ < nts2 )
-		copy_nts2 = TRUE;
+		copy_nts2 = true;
 	      else
-		process_nts2 = FALSE;
+		process_nts2 = false;
 	    }
 	}
 
@@ -465,7 +458,7 @@ void *Seltime(void *argument)
 		  for ( varID = 0; varID < nvars; varID++ )
 		    {
 		      if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT && tsID2 > 1 ) continue;
-		      nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+		      int nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 		      for ( levelID = 0; levelID < nlevel; levelID++ )
 			{
 			  streamDefRecord(streamID2, varID, levelID);
@@ -480,10 +473,10 @@ void *Seltime(void *argument)
 	    }
 
 	  ncts++;
-	  if ( process_nts2 == FALSE )
+	  if ( !process_nts2 )
 	    {
 	      its2 = 0;
-	      process_nts1 = TRUE;
+	      process_nts1 = true;
 	    }
 
 	  taxisCopyTimestep(taxisID2, taxisID1);
@@ -492,13 +485,13 @@ void *Seltime(void *argument)
 
 	  if ( tsID > 0 && lconstout )
 	    {
-	      lconstout = FALSE;
+	      lconstout = false;
 	      nts = nts1 - 1;
 	      for ( varID = 0; varID < nvars; varID++ )
 		{
 		  if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT )
 		    {
-		      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+		      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 		      for ( levelID = 0; levelID < nlevel; levelID++ )
 			{
 			  streamDefRecord(streamID2, varID, levelID);
@@ -510,7 +503,7 @@ void *Seltime(void *argument)
 		}
 	    }
 
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      streamInqRecord(streamID1, &varID, &levelID);
 	      streamDefRecord(streamID2, varID, levelID);
@@ -531,7 +524,7 @@ void *Seltime(void *argument)
 
 	  if ( lnts1 || tsID == 0 )
 	    {
-	      if ( tsID == 0 && nconst && (!lnts1) ) lconstout = TRUE;
+	      if ( tsID == 0 && nconst && (!lnts1) ) lconstout = true;
 
 	      nts = nts1-1;
 	      if ( lnts1 )
@@ -546,9 +539,9 @@ void *Seltime(void *argument)
 			for ( varID = 0; varID < nvars; varID++ )
 			  {
 			    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
-			    gridID   = vlistInqVarGrid(vlistID1, varID);
-			    gridsize = gridInqSize(gridID);
-			    nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+			    int gridID   = vlistInqVarGrid(vlistID1, varID);
+			    int gridsize = gridInqSize(gridID);
+			    int nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 			    for ( levelID = 0; levelID < nlevel; levelID++ )
 			      {
 				memcpy(vars[it][varID][levelID].ptr,
@@ -565,7 +558,7 @@ void *Seltime(void *argument)
 		  its1++;
 		}
 
-	      for ( recID = 0; recID < nrecs; recID++ )
+	      for ( int recID = 0; recID < nrecs; recID++ )
 		{
 		  streamInqRecord(streamID1, &varID, &levelID);
 		  if ( lnts1 || (vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT) )
@@ -590,15 +583,15 @@ void *Seltime(void *argument)
 
   if ( array ) Free(array);
 
-  for ( isel = 0; isel < nsel; isel++ )
+  for ( int isel = 0; isel < nsel; isel++ )
     {
-      if ( selfound[isel] == FALSE )
+      if ( selfound[isel] == false )
 	{
 	 
 	  int isel2;
 	  int lcont = FALSE;
 	  for ( isel2 = isel+1; isel2 < nsel; isel2++ )
-	    if ( selfound[isel2] == TRUE ) break;
+	    if ( selfound[isel2] ) break;
 	  if ( isel2 == nsel && (nsel-isel) > 1 ) lcont = TRUE;
 
 	  if ( operatorID == SELTIMESTEP )
@@ -654,7 +647,8 @@ void *Seltime(void *argument)
 
   if ( selfound ) Free(selfound);
 
-  listDelete(ilist);
+  lista_destroy(ilista);
+  lista_destroy(flista);
 
   if ( lnts1 || nconst )
     {
