@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,6 @@
       Selvar     selltype        Select GRIB level type 
 */
 
-#include <ctype.h>  /* isdigit */
 
 #include <cdi.h>
 #include "cdo.h"
@@ -41,19 +40,14 @@
 #include "pstream.h"
 #include "error.h"
 #include "util.h"
-#include "list.h"
+#include "listarray.h"
 
 
 void *Selvar(void *argument)
 {
-  int nlevs;
-  int code, tabnum, param, gridID, zaxisID, levID;
-  int grididx, zaxisidx;
-  double level;
   int varID2, levelID2;
-  int recID, varID, levelID;
+  int varID, levelID;
   int *intarr = NULL, nsel = 0;
-  int *selfound = NULL;
   double *fltarr = NULL;
   char paramstr[32];
   char varname[CDI_MAX_NAME];
@@ -62,18 +56,15 @@ void *Selvar(void *argument)
   char zaxistypename[CDI_MAX_NAME];
   char zaxisname[CDI_MAX_NAME];
   char **argnames = NULL;
-  int isel;
-  int i;
-  int lcopy = FALSE;
-  int gridsize;
   int nmiss;
   int gridnum = 0;
-  double *array = NULL;
-  LIST *ilist = listNew(INT_LIST);
-  LIST *flist = listNew(FLT_LIST);
+  lista_t *ilista = lista_new(INT_LISTA);
+  lista_t *flista = lista_new(FLT_LISTA);
 
   cdoInitialize(argument);
 
+  bool lcopy = UNCHANGED_RECORD;
+
 # define INVERTS_SELECTION(id) (cdoOperatorF2(id) & 1)
 # define TAKES_STRINGS(id) (cdoOperatorF2(id) & 2)
 # define TAKES_INTEGERS(id) (cdoOperatorF2(id) & 4)
@@ -89,17 +80,17 @@ void *Selvar(void *argument)
   int SELZAXIS     = cdoOperatorAdd("selzaxis",     0, 4|2, "list of zaxis types or numbers");
   int SELZAXISNAME = cdoOperatorAdd("selzaxisname", 0, 2,   "list of zaxis names");
   int SELTABNUM    = cdoOperatorAdd("seltabnum",    0, 4,   "table numbers");
-  int DELPARAM     = cdoOperatorAdd("delparam",     0, 2|1, "parameter");
-  int DELCODE      = cdoOperatorAdd("delcode",      0, 1,   "code numbers");
-  int DELNAME      = cdoOperatorAdd("delname",      0, 2|1, "variable names");
+  int DELPARAM     = cdoOperatorAdd("delparam",     1, 2|1, "parameter");
+  int DELCODE      = cdoOperatorAdd("delcode",      1, 1,   "code numbers");
+  int DELNAME      = cdoOperatorAdd("delname",      1, 2|1, "variable names");
   int SELLTYPE     = cdoOperatorAdd("selltype",     0, 4,   "GRIB level types"); 
 
-  if ( UNCHANGED_RECORD ) lcopy = TRUE;
-
   int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
+  bool ldelete = (cdoOperatorF1(operatorID) == 1) ? true : false;
+
   int args_are_numeric = operatorArgc() > 0 && isdigit(*operatorArgv()[0]);
 
   if ( TAKES_STRINGS(operatorID) && !( TAKES_INTEGERS(operatorID) && args_are_numeric ) )
@@ -108,32 +99,33 @@ void *Selvar(void *argument)
       argnames = operatorArgv();
 
       if ( cdoVerbose )
-	for ( i = 0; i < nsel; i++ )
-	  fprintf(stderr, "name %d = %s\n", i+1, argnames[i]);
+	for ( int i = 0; i < nsel; i++ )
+	  cdoPrint("name %d = %s", i+1, argnames[i]);
     }
   else if ( TAKES_FLOATS(operatorID) )
     {
-      nsel = args2fltlist(operatorArgc(), operatorArgv(), flist);
-      fltarr = (double *) listArrayPtr(flist);
+      nsel = args2flt_lista(operatorArgc(), operatorArgv(), flista);
+      fltarr = (double *) lista_dataptr(flista);
 
       if ( cdoVerbose )
-	for ( i = 0; i < nsel; i++ )
-	  printf("flt %d = %g\n", i+1, fltarr[i]);
+	for ( int i = 0; i < nsel; i++ )
+	  cdoPrint("flt %d = %g", i+1, fltarr[i]);
     }
   else
     {
-      nsel = args2intlist(operatorArgc(), operatorArgv(), ilist);
-      intarr = (int *) listArrayPtr(ilist);
+      nsel = args2int_lista(operatorArgc(), operatorArgv(), ilista);
+      intarr = (int *) lista_dataptr(ilista);
 
       if ( cdoVerbose )
-	for ( i = 0; i < nsel; i++ )
-	  printf("int %d = %d\n", i+1, intarr[i]);
+	for ( int i = 0; i < nsel; i++ )
+	  cdoPrint("int %d = %d", i+1, intarr[i]);
     }
 
+  bool *selfound = NULL;
   if ( nsel )
     {
-      selfound = (int*) Malloc(nsel*sizeof(int));
-      for ( i = 0; i < nsel; i++ ) selfound[i] = FALSE;
+      selfound = (bool*) Malloc(nsel*sizeof(bool));
+      for ( int i = 0; i < nsel; i++ ) selfound[i] = false;
     }
 
   /*
@@ -144,7 +136,7 @@ void *Selvar(void *argument)
 
   int vlistID1 = streamInqVlist(streamID1);
   int nvars = vlistNvars(vlistID1);
-  int *vars = (int*) Malloc(nvars*sizeof(int));
+  bool *vars = (bool*) Malloc(nvars*sizeof(bool));
 
   if ( operatorID == SELGRID && !args_are_numeric && nsel == 1 && strncmp(argnames[0], "var=", 4) == 0 )
     {
@@ -157,7 +149,7 @@ void *Selvar(void *argument)
           vlistInqVarName(vlistID1, varID, varname);
           if ( strcmp(varname, gridvarname) == 0 )
             {
-              gridID = vlistInqVarGrid(vlistID1, varID);
+              int gridID = vlistInqVarGrid(vlistID1, varID);
               gridnum = 1 + vlistGridIndex(vlistID1, gridID);
               args_are_numeric = TRUE;
               intarr = &gridnum;
@@ -171,104 +163,70 @@ void *Selvar(void *argument)
   vlistClearFlag(vlistID1);
   for ( varID = 0; varID < nvars; varID++ )
     {
-      vars[varID] = FALSE;
+      vars[varID] = ldelete;
 
       vlistInqVarName(vlistID1, varID, varname);
       vlistInqVarStdname(vlistID1, varID, stdname);
-      param    = vlistInqVarParam(vlistID1, varID);
-      code     = vlistInqVarCode(vlistID1, varID);
-      tabnum   = tableInqNum(vlistInqVarTable(vlistID1, varID));
-      gridID   = vlistInqVarGrid(vlistID1, varID);
-      grididx  = vlistGridIndex(vlistID1, gridID);
-      zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-      zaxisidx = vlistZaxisIndex(vlistID1, zaxisID);
-      nlevs    = zaxisInqSize(zaxisID);
+      int param    = vlistInqVarParam(vlistID1, varID);
+      int code     = vlistInqVarCode(vlistID1, varID);
+      int tabnum   = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      int gridID   = vlistInqVarGrid(vlistID1, varID);
+      int grididx  = vlistGridIndex(vlistID1, gridID);
+      int zaxisID  = vlistInqVarZaxis(vlistID1, varID);
+      int zaxisidx = vlistZaxisIndex(vlistID1, zaxisID);
+      int nlevs    = zaxisInqSize(zaxisID);
       gridName(gridInqType(gridID), gridname);
       zaxisInqName(zaxisID, zaxisname);
       zaxisName(zaxisInqType(zaxisID), zaxistypename);
 
       cdiParamToString(param, paramstr, sizeof(paramstr));
 
-      for ( levID = 0; levID < nlevs; levID++ )
+      for ( int levID = 0; levID < nlevs; levID++ )
 	{
-	  level = zaxisInqLevel(zaxisID, levID);
+	  double level = cdoZaxisInqLevel(zaxisID, levID);
 
-	  if ( operatorID == DELCODE || operatorID == DELNAME || operatorID == DELPARAM )
-	    vlistDefFlag(vlistID1, varID, levID, TRUE);
-
-	  for ( isel = 0; isel < nsel; isel++ )
+	  if ( ldelete ) vlistDefFlag(vlistID1, varID, levID, TRUE);
+          
+	  for ( int isel = 0; isel < nsel; isel++ )
 	    {
-              int found = 0;
-	      if ( operatorID == SELCODE )
-		{
-		  found = intarr[isel] == code;
-		}
+              bool found = false;
+	      if ( operatorID == SELCODE ) found = intarr[isel] == code;
 	      else if ( operatorID == SELPARAM )
-		{
-		  found = wildcardmatch(argnames[isel], paramstr) == 0;
-		}
+                found = wildcardmatch(argnames[isel], paramstr) == 0;
 	      else if ( operatorID == SELNAME )
-		{
-		  found = wildcardmatch(argnames[isel], varname) == 0;
-		}
+                found = wildcardmatch(argnames[isel], varname) == 0;
 	      else if ( operatorID == SELSTDNAME )
-		{
-		  found = wildcardmatch(argnames[isel], stdname) == 0;
-		}
+                found = wildcardmatch(argnames[isel], stdname) == 0;
 	      else if ( operatorID == SELLEVEL )
-		{
-		  found = fabs(fltarr[isel] - level) < 0.0001;
-		}
+                found = fabs(fltarr[isel] - level) < 0.0001;
 	      else if ( operatorID == SELLEVIDX )
-		{
-		  found = intarr[isel] == (levID+1);
-		}
+                found = intarr[isel] == (levID+1);
 	      else if ( operatorID == SELGRID && args_are_numeric )
-		{
-		  found = intarr[isel] == (grididx+1);
-		}
+                found = intarr[isel] == (grididx+1);
 	      else if ( operatorID == SELGRID && !args_are_numeric )
-		{
-		  found = memcmp(argnames[isel], gridname, strlen(argnames[isel])) == 0;
-		}
+                found = memcmp(argnames[isel], gridname, strlen(argnames[isel])) == 0;
 	      else if ( operatorID == SELZAXIS && args_are_numeric )
-		{
-		  found = intarr[isel] == (zaxisidx+1);
-		}
+                found = intarr[isel] == (zaxisidx+1);
 	      else if ( operatorID == SELZAXIS && !args_are_numeric )
-		{
-		  found = memcmp(argnames[isel], zaxistypename, strlen(argnames[isel])) == 0;
-		}
+                found = memcmp(argnames[isel], zaxistypename, strlen(argnames[isel])) == 0;
 	      else if ( operatorID == SELZAXISNAME )
-		{
-		  found = wildcardmatch(argnames[isel], zaxisname) == 0;
-		}
+                found = wildcardmatch(argnames[isel], zaxisname) == 0;
 	      else if ( operatorID == SELTABNUM )
-		{
-		  found = intarr[isel] == tabnum;
-		}
+                found = intarr[isel] == tabnum;
 	      else if ( operatorID == DELCODE )
-		{
-		  found = intarr[isel] == code;
-		}
+                found = intarr[isel] == code;
 	      else if ( operatorID == DELNAME )
-		{
-		  found = wildcardmatch(argnames[isel], varname) == 0;
-		}
+                found = wildcardmatch(argnames[isel], varname) == 0;
 	      else if ( operatorID == DELPARAM )
-		{
-		  found = strcmp(argnames[isel], paramstr) == 0;
-		}
+                found = strcmp(argnames[isel], paramstr) == 0;
 	      else if ( operatorID == SELLTYPE )
-		{
-		  found = intarr[isel] == zaxis2ltype(zaxisID);
-		}
+                found = intarr[isel] == zaxis2ltype(zaxisID);
 
 	      if ( found )
 	        {
 		  vlistDefFlag(vlistID1, varID, levID, !INVERTS_SELECTION(operatorID));
-		  selfound[isel] = TRUE;
-                  vars[varID] = TRUE;
+		  selfound[isel] = true;
+                  vars[varID] = ldelete ? false : true;
 	        }
 	    }
 	}
@@ -287,69 +245,43 @@ void *Selvar(void *argument)
               int psvarid = vlist_get_psvarid(vlistID1, zaxisID);
               if ( psvarid != -1 && !vars[psvarid] )
                 {
-                  vars[psvarid] = TRUE;
+                  vars[psvarid] = true;
                   vlistDefFlag(vlistID1, psvarid, 0, !INVERTS_SELECTION(operatorID));
                 }
             }
         }
     }
 
-  for ( isel = 0; isel < nsel; isel++ )
+  for ( int isel = 0; isel < nsel; isel++ )
     {
-      if ( selfound[isel] == FALSE )
+      if ( selfound[isel] == false )
 	{
 	  if ( operatorID == SELCODE || operatorID == DELCODE )
-	    {
-	      cdoWarning("Code number %d not found!", intarr[isel]);
-	    }
+            cdoWarning("Code number %d not found!", intarr[isel]);
 	  else if ( operatorID == SELPARAM || operatorID == DELPARAM )
-	    {
-	      cdoWarning("Parameter %s not found!", argnames[isel]);
-	    }
+            cdoWarning("Parameter %s not found!", argnames[isel]);
 	  else if ( operatorID == SELNAME || operatorID == DELNAME )
-	    {
-	      cdoWarning("Variable name %s not found!", argnames[isel]);
-	    }
+            cdoWarning("Variable name %s not found!", argnames[isel]);
 	  else if ( operatorID == SELSTDNAME )
-	    {
-	      cdoWarning("Variable with standard name %s not found!", argnames[isel]);
-	    }
+            cdoWarning("Variable with standard name %s not found!", argnames[isel]);
 	  else if ( operatorID == SELLEVEL )
-	    {
-	      cdoWarning("Level %g not found!", fltarr[isel]);
-	    }
+            cdoWarning("Level %g not found!", fltarr[isel]);
 	  else if ( operatorID == SELLEVIDX )
-	    {
-	      cdoWarning("Level index %d not found!", intarr[isel]);
-	    }
+            cdoWarning("Level index %d not found!", intarr[isel]);
 	  else if ( operatorID == SELGRID && args_are_numeric )
-	    {
-	      cdoWarning("Grid %d not found!", intarr[isel]);
-	    }
+            cdoWarning("Grid %d not found!", intarr[isel]);
 	  else if ( operatorID == SELGRID && !args_are_numeric )
-	    {
-	      cdoWarning("Grid name %s not found!", argnames[isel]);
-	    }
+            cdoWarning("Grid name %s not found!", argnames[isel]);
 	  else if ( operatorID == SELZAXIS && args_are_numeric )
-	    {
-	      cdoWarning("Zaxis %d not found!", intarr[isel]);
-	    }
+            cdoWarning("Zaxis %d not found!", intarr[isel]);
 	  else if ( operatorID == SELZAXIS && !args_are_numeric )
-	    {
-	      cdoWarning("Zaxis type %s not found!", argnames[isel]);
-	    }
+            cdoWarning("Zaxis type %s not found!", argnames[isel]);
 	  else if ( operatorID == SELZAXISNAME )
-	    {
-	      cdoWarning("Zaxis name %s not found!", argnames[isel]);
-	    }
+            cdoWarning("Zaxis name %s not found!", argnames[isel]);
 	  else if ( operatorID == SELTABNUM )
-	    {
-	      cdoWarning("Table number %d not found!", intarr[isel]);
-	    }
+            cdoWarning("Table number %d not found!", intarr[isel]);
 	  else if ( operatorID == SELLTYPE )
-	    {
-	      cdoWarning("GRIB level type %d not found!", intarr[isel]);
-	    }
+            cdoWarning("GRIB level type %d not found!", intarr[isel]);
 	}
     }
 
@@ -373,9 +305,10 @@ void *Selvar(void *argument)
 
   streamDefVlist(streamID2, vlistID2);
 
+  double *array = NULL;
   if ( ! lcopy )
     {
-      gridsize = vlistGridsizeMax(vlistID1);
+      int gridsize = vlistGridsizeMax(vlistID1);
       if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
       array = (double*) Malloc(gridsize*sizeof(double));
     }
@@ -387,7 +320,7 @@ void *Selvar(void *argument)
 
       streamDefTimestep(streamID2, tsID);
      
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  if ( vlistInqFlag(vlistID1, varID, levelID) == TRUE )
@@ -423,8 +356,8 @@ void *Selvar(void *argument)
 
   if ( selfound ) Free(selfound);
 
-  listDelete(ilist);
-  listDelete(flist);
+  lista_destroy(ilista);
+  lista_destroy(flista);
 
   cdoFinish();
 
