@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,8 +22,6 @@
       Setgatt    setgatts        Set global attributes
 */
 
-#include <ctype.h>
-
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
@@ -32,24 +30,18 @@
 
 void *Setgatt(void *argument)
 {
-  int SETGATT, SETGATTS;
-  int operatorID;
-  int streamID1, streamID2 = CDI_UNDEFID;
   int nrecs;
-  int tsID, recID, varID, levelID;
-  int vlistID1, vlistID2;
+  int varID, levelID;
   int gridsize;
   int nmiss;
   char *attname = NULL, *attstring = NULL, *attfile = NULL;
-  double *array = NULL;
-  int taxisID1, taxisID2;
 
   cdoInitialize(argument);
 
-  SETGATT  = cdoOperatorAdd("setgatt",  0, 0, "attribute name and string");
-  SETGATTS = cdoOperatorAdd("setgatts", 0, 0, NULL);
+  int SETGATT  = cdoOperatorAdd("setgatt",  0, 0, "attribute name and string");
+                 cdoOperatorAdd("setgatts", 0, 0, NULL);
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   if ( operatorID == SETGATT )
     {
@@ -62,28 +54,27 @@ void *Setgatt(void *argument)
       attfile   = operatorArgv()[0];
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   if ( operatorID == SETGATT )
     {
-      vlistDefAttTxt(vlistID2, CDI_GLOBAL, attname, (int)strlen(attstring), attstring);
+      cdiDefAttTxt(vlistID2, CDI_GLOBAL, attname, (int)strlen(attstring), attstring);
     }
   else
     {
       char line[1024];
-      FILE *fp;
       int attlen = 0;
 
-      fp = fopen(attfile, "r");
+      FILE *fp = fopen(attfile, "r");
       if ( fp == 0 ) cdoAbort("Open failed on %s", attfile);
 
       while ( readline(fp, line, 1024) )
@@ -111,7 +102,7 @@ void *Setgatt(void *argument)
 	    }
 
 	  if ( attstring && attlen)
-	    vlistDefAttTxt(vlistID2, CDI_GLOBAL, attname, attlen, attstring);
+	    cdiDefAttTxt(vlistID2, CDI_GLOBAL, attname, attlen, attstring);
 	}
 
       fclose(fp);
@@ -121,16 +112,16 @@ void *Setgatt(void *argument)
 
   gridsize = vlistGridsizeMax(vlistID1);
   if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-  array = (double*) malloc(gridsize*sizeof(double));
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamDefRecord(streamID2,  varID,  levelID);
@@ -144,9 +135,9 @@ void *Setgatt(void *argument)
   streamClose(streamID1);
   streamClose(streamID2);
 
-  if ( array ) free(array);
+  if ( array ) Free(array);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
