@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,24 +33,17 @@
 
 void *Setgrid(void *argument)
 {
-  int SETGRID, SETGRIDTYPE, SETGRIDAREA, SETGRIDMASK, UNSETGRIDMASK, SETGRIDNUMBER, SETGRIDURI;
-  int operatorID;
-  int streamID1, streamID2 = CDI_UNDEFID;
   int nrecs;
-  int tsID, recID, varID, levelID;
-  int vlistID1, vlistID2;
-  int taxisID1, taxisID2;
-  int gridID1, gridID2 = -1;
-  int ngrids, index;
+  int varID, levelID;
+  int gridID2 = -1;
   int gridtype = -1;
   int nmiss;
-  int found;
-  long i, gridsize;
-  long areasize = 0;
-  long masksize = 0;
-  int lregular = 0;
-  int ldereference = 0;
-  int ligme = 0;
+  int gridsize;
+  int areasize = 0;
+  int  masksize = 0;
+  bool lregular = false;
+  bool lregularnn = false;
+  bool ldereference = false;
   int number = 0, position = 0;
   int grid2_nvgp;
   int lbounds = TRUE;
@@ -59,19 +52,18 @@ void *Setgrid(void *argument)
   char *griduri = NULL;
   double *gridmask = NULL;
   double *areaweight = NULL;
-  double *array = NULL;
 
   cdoInitialize(argument);
 
-  SETGRID       = cdoOperatorAdd("setgrid",       0, 0, "grid description file or name");
-  SETGRIDTYPE   = cdoOperatorAdd("setgridtype",   0, 0, "grid type");
-  SETGRIDAREA   = cdoOperatorAdd("setgridarea",   0, 0, "filename with area weights");
-  SETGRIDMASK   = cdoOperatorAdd("setgridmask",   0, 0, "filename with grid mask");
-  UNSETGRIDMASK = cdoOperatorAdd("unsetgridmask", 0, 0, NULL);
-  SETGRIDNUMBER = cdoOperatorAdd("setgridnumber", 0, 0, "grid number and optionally grid position");
-  SETGRIDURI    = cdoOperatorAdd("setgriduri",    0, 0, "reference URI of the horizontal grid");
+  int SETGRID       = cdoOperatorAdd("setgrid",       0, 0, "grid description file or name");
+  int SETGRIDTYPE   = cdoOperatorAdd("setgridtype",   0, 0, "grid type");
+  int SETGRIDAREA   = cdoOperatorAdd("setgridarea",   0, 0, "filename with area weights");
+  int SETGRIDMASK   = cdoOperatorAdd("setgridmask",   0, 0, "filename with grid mask");
+  int UNSETGRIDMASK = cdoOperatorAdd("unsetgridmask", 0, 0, NULL);
+  int SETGRIDNUMBER = cdoOperatorAdd("setgridnumber", 0, 0, "grid number and optionally grid position");
+  int SETGRIDURI    = cdoOperatorAdd("setgriduri",    0, 0, "reference URI of the horizontal grid");
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   if ( operatorID != UNSETGRIDMASK )
     operatorInputArg(cdoOperatorEnter(operatorID));  
@@ -92,45 +84,40 @@ void *Setgrid(void *argument)
       else if ( strcmp(gridname, "unstructured0") == 0 ) {gridtype = GRID_UNSTRUCTURED; lbounds = 0;}
       else if ( strcmp(gridname, "unstructured") == 0 )  {gridtype = GRID_UNSTRUCTURED; lbounds = 1;}
       else if ( strcmp(gridname, "generic") == 0 )        gridtype = GRID_GENERIC;
-      else if ( strcmp(gridname, "dereference") == 0 )    ldereference = 1;
+      else if ( strcmp(gridname, "dereference") == 0 )    ldereference = true;
       else if ( strcmp(gridname, "lonlat") == 0 )         gridtype = GRID_LONLAT;
       else if ( strcmp(gridname, "gaussian") == 0 )       gridtype = GRID_GAUSSIAN;
-      else if ( strcmp(gridname, "regular") == 0 )       {gridtype = GRID_GAUSSIAN; lregular = 1;}
+      else if ( strcmp(gridname, "regularnn") == 0 )     {gridtype = GRID_GAUSSIAN; lregularnn = true;}
+      else if ( strcmp(gridname, "regular") == 0 )       {gridtype = GRID_GAUSSIAN; lregular = true;}
       else cdoAbort("Unsupported grid name: %s", gridname);
     }
   else if ( operatorID == SETGRIDAREA )
     {
-      int streamID, vlistID, gridID;
-      char *areafile;
-
       operatorCheckArgc(1);
-      areafile = operatorArgv()[0];
+      char *areafile = operatorArgv()[0];
 
       argument_t *fileargument = file_argument_new(areafile);
-      streamID = streamOpenRead(fileargument);
+      int streamID = streamOpenRead(fileargument);
       file_argument_free(fileargument);
 
-      vlistID = streamInqVlist(streamID);
+      int vlistID = streamInqVlist(streamID);
 
       nrecs = streamInqTimestep(streamID, 0);
       streamInqRecord(streamID, &varID, &levelID);
 
-      gridID = vlistInqVarGrid(vlistID, varID);
+      int gridID = vlistInqVarGrid(vlistID, varID);
       areasize = gridInqSize(gridID);
       areaweight = (double*) Malloc(areasize*sizeof(double));
   
       streamReadRecord(streamID, areaweight, &nmiss);
-
       streamClose(streamID);
 
       if ( cdoVerbose )
 	{
-	  double arrmean, arrmin, arrmax;
-
-	  arrmean = areaweight[0];
-	  arrmin  = areaweight[0];
-	  arrmax  = areaweight[0];
-	  for ( i = 1; i < areasize; i++ )
+	  double arrmean = areaweight[0];
+	  double arrmin  = areaweight[0];
+	  double arrmax  = areaweight[0];
+	  for ( int i = 1; i < areasize; i++ )
 	    {
 	      if ( areaweight[i] < arrmin ) arrmin = areaweight[i];
 	      if ( areaweight[i] > arrmax ) arrmax = areaweight[i];
@@ -143,31 +130,26 @@ void *Setgrid(void *argument)
     }
   else if ( operatorID == SETGRIDMASK )
     {
-      int streamID, vlistID, gridID;
-      char *maskfile;
-      double missval;
-
       operatorCheckArgc(1);
-      maskfile = operatorArgv()[0];
+      char *maskfile = operatorArgv()[0];
       argument_t *fileargument = file_argument_new(maskfile);
-      streamID = streamOpenRead(fileargument);
+      int streamID = streamOpenRead(fileargument);
       file_argument_free(fileargument);
 
-      vlistID = streamInqVlist(streamID);
+      int vlistID = streamInqVlist(streamID);
 
       nrecs = streamInqTimestep(streamID, 0);
       streamInqRecord(streamID, &varID, &levelID);
 
-      missval  = vlistInqVarMissval(vlistID, varID);
-      gridID   = vlistInqVarGrid(vlistID, varID);
+      double missval  = vlistInqVarMissval(vlistID, varID);
+      int gridID   = vlistInqVarGrid(vlistID, varID);
       masksize = gridInqSize(gridID);
       gridmask = (double*) Malloc(masksize*sizeof(double));
   
       streamReadRecord(streamID, gridmask, &nmiss);
-
       streamClose(streamID);
 
-      for ( i = 0; i < masksize; i++ )
+      for ( int i = 0; i < masksize; i++ )
 	if ( DBL_IS_EQUAL(gridmask[i], missval) ) gridmask[i] = 0;
     }
   else if ( operatorID == SETGRIDNUMBER )
@@ -188,22 +170,22 @@ void *Setgrid(void *argument)
       griduri = operatorArgv()[0];
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   if ( operatorID == SETGRID )
     {
-      found = 0;
-      ngrids = vlistNgrids(vlistID1);
-      for ( index = 0; index < ngrids; index++ )
+      int found = 0;
+      int ngrids = vlistNgrids(vlistID1);
+      for ( int index = 0; index < ngrids; index++ )
 	{
-	  gridID1 = vlistGrid(vlistID1, index);
+	  int gridID1 = vlistGrid(vlistID1, index);
 
 	  if ( gridInqSize(gridID1) == gridInqSize(gridID2) )
 	    {
@@ -215,7 +197,7 @@ void *Setgrid(void *argument)
     }
   else if ( operatorID == SETGRIDNUMBER || operatorID == SETGRIDURI )
     {
-      gridID1 = vlistGrid(vlistID1, 0);
+      int gridID1 = vlistGrid(vlistID1, 0);
 
       if ( operatorID == SETGRIDNUMBER )
 	{
@@ -229,9 +211,9 @@ void *Setgrid(void *argument)
 	  gridDefReference(gridID2, griduri);
 	}
 
-      found = 0;
-      ngrids = vlistNgrids(vlistID1);
-      for ( index = 0; index < ngrids; index++ )
+      int found = 0;
+      int ngrids = vlistNgrids(vlistID1);
+      for ( int index = 0; index < ngrids; index++ )
 	{
 	  gridID1 = vlistGrid(vlistID1, index);
 
@@ -245,20 +227,19 @@ void *Setgrid(void *argument)
     }
   else if ( operatorID == SETGRIDTYPE )
     {
-      ngrids = vlistNgrids(vlistID1);
-      for ( index = 0; index < ngrids; index++ )
+      bool lrgrid = false;
+      int ngrids = vlistNgrids(vlistID1);
+      for ( int index = 0; index < ngrids; index++ )
 	{
-	  gridID1 = vlistGrid(vlistID1, index);
+	  int gridID1 = vlistGrid(vlistID1, index);
 	  gridID2 = -1;
 
 	  if ( gridInqType(gridID1) == GRID_GENERIC && gridInqSize(gridID1) == 1 ) continue;
 	  
-	  if ( lregular )
+	  if ( lregular || lregularnn )
 	    {
 	      if ( gridInqType(gridID1) == GRID_GAUSSIAN_REDUCED )
-		{
-		  gridID2 = gridToRegular(gridID1);
-		}
+                gridID2 = gridToRegular(gridID1);
 	    }
 	  else if ( ldereference )
 	    {
@@ -273,7 +254,8 @@ void *Setgrid(void *argument)
 		}
 	      else if ( gridtype == GRID_UNSTRUCTURED )
 		{
-		  if ( gridInqType(gridID1) == GRID_GME ) ligme = 1;
+                  bool ligme = false;
+		  if ( gridInqType(gridID1) == GRID_GME ) ligme = true;
 		  gridID2 = gridToUnstructured(gridID1, 1);
 
 		  if ( ligme )
@@ -306,17 +288,28 @@ void *Setgrid(void *argument)
 	      else cdoAbort("Unsupported grid name: %s", gridname);
 	    }
 
-	  if ( gridID2 == -1 ) cdoAbort("Unsupported grid type!");
+	  if ( gridID2 == -1 )
+            {
+              if ( !(lregular || lregularnn) )
+                cdoAbort("Unsupported grid type!");
+            }
 
-	  vlistChangeGridIndex(vlistID2, index, gridID2);
-	}
+	  if ( gridID2 != -1 )
+            {
+              if ( lregular || lregularnn ) lrgrid = true;
+              vlistChangeGridIndex(vlistID2, index, gridID2);
+            }
+        }
+
+      if ( (lregular || lregularnn) && !lrgrid )
+        cdoWarning("No reduced Gaussian grid found!");
     }
   else if ( operatorID == SETGRIDAREA )
     {
-      ngrids = vlistNgrids(vlistID1);
-      for ( index = 0; index < ngrids; index++ )
+      int ngrids = vlistNgrids(vlistID1);
+      for ( int index = 0; index < ngrids; index++ )
 	{
-	  gridID1  = vlistGrid(vlistID1, index);
+	  int gridID1  = vlistGrid(vlistID1, index);
 	  gridsize = gridInqSize(gridID1);
 	  if ( gridsize == areasize )
 	    {
@@ -328,15 +321,15 @@ void *Setgrid(void *argument)
     }
   else if ( operatorID == SETGRIDMASK )
     {
-      ngrids = vlistNgrids(vlistID1);
-      for ( index = 0; index < ngrids; index++ )
+      int ngrids = vlistNgrids(vlistID1);
+      for ( int index = 0; index < ngrids; index++ )
 	{
-	  gridID1  = vlistGrid(vlistID1, index);
+	  int gridID1  = vlistGrid(vlistID1, index);
 	  gridsize = gridInqSize(gridID1);
 	  if ( gridsize == masksize )
 	    {
 	      int *mask = (int*) Malloc(masksize*sizeof(int));
-	      for ( i = 0; i < masksize; i++ )
+	      for ( int i = 0; i < masksize; i++ )
 		{
 		  if ( gridmask[i] < 0 || gridmask[i] > 255 )
 		    mask[i] = 0;
@@ -352,58 +345,59 @@ void *Setgrid(void *argument)
     }
   else if ( operatorID == UNSETGRIDMASK )
     {
-      ngrids = vlistNgrids(vlistID1);
-      for ( index = 0; index < ngrids; index++ )
+      int ngrids = vlistNgrids(vlistID1);
+      for ( int index = 0; index < ngrids; index++ )
 	{
-	  gridID1  = vlistGrid(vlistID1, index);
+	  int gridID1  = vlistGrid(vlistID1, index);
 	  gridID2 = gridDuplicate(gridID1);
 	  gridDefMask(gridID2, NULL);
 	  vlistChangeGridIndex(vlistID2, index, gridID2);
 	}
     }
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
   //vlistPrint(vlistID2);
 
-  if ( lregular )
+  if ( lregular || lregularnn )
     gridsize = vlistGridsizeMax(vlistID2);
   else
     gridsize = vlistGridsizeMax(vlistID1);
 
   if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-  array = (double*) Malloc(gridsize*sizeof(double));
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamDefRecord(streamID2,  varID,  levelID);
 	  
 	  streamReadRecord(streamID1, array, &nmiss);
 
-	  gridID1 = vlistInqVarGrid(vlistID1, varID);
-	  if ( lregular )
+	  int gridID1 = vlistInqVarGrid(vlistID1, varID);
+	  if ( lregular || lregularnn )
 	    {
 	      gridID2 = vlistInqVarGrid(vlistID2, varID);
 	      if ( gridInqType(gridID1) == GRID_GAUSSIAN_REDUCED )
 		{
 		  double missval = vlistInqVarMissval(vlistID1, varID);
-		  field2regular(gridID1, gridID2, missval, array, nmiss);
+                  int lnearst = lregularnn ? 1 : 0;
+                  field2regular(gridID1, gridID2, missval, array, nmiss, lnearst);
 		}
 	    }
 	  else if ( gridInqType(gridID1) == GRID_GME )
 	    {
-	      int j = 0;
 	      gridsize = gridInqSize(gridID1);
-	      for ( i = 0; i < gridsize; i++ )
+	      int j = 0;
+	      for ( int i = 0; i < gridsize; i++ )
 		if ( grid2_vgpm[i] ) array[j++] = array[i];
 	    }
 
