@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,6 @@
       Settime    shifttime       Shift timesteps
 */
 
-#include <ctype.h>  /* isdigit */
 
 #include <cdi.h>
 #include "cdo.h"
@@ -41,9 +40,7 @@
 
 int get_tunits(const char *unit, int *incperiod, int *incunit, int *tunit)
 {
-  size_t len;
-	
-  len = strlen(unit);
+  size_t len = strlen(unit);
   
   if      ( memcmp(unit, "seconds", len) == 0 ) { *incunit =     1; *tunit = TUNIT_SECOND;  }
   else if ( memcmp(unit, "minutes", len) == 0 ) { *incunit =    60; *tunit = TUNIT_MINUTE;  }
@@ -54,7 +51,7 @@ int get_tunits(const char *unit, int *incperiod, int *incunit, int *tunit)
   else if ( memcmp(unit, "days", len)    == 0 ) { *incunit = 86400; *tunit = TUNIT_DAY;     }
   else if ( memcmp(unit, "months", len)  == 0 ) { *incunit =     1; *tunit = TUNIT_MONTH;   }
   else if ( memcmp(unit, "years", len)   == 0 ) { *incunit =    12; *tunit = TUNIT_YEAR;    }
-  else cdoAbort("time unit >%s< unsupported", unit);
+  else cdoAbort("Time unit >%s< unsupported!", unit);
 
   if ( *tunit == TUNIT_HOUR )
     {
@@ -102,12 +99,46 @@ void shifttime(int calendar, int tunit, int ijulinc, int *pdate, int *ptime)
     }
 }
 
+static
+void gen_bounds(int calendar, int tunit, int vdate, int vtime, int *vdateb, int *vtimeb)
+{
+  juldate_t juldate;
+  int year, month, day;
+  //int hour, minute, second;
+  UNUSED(vtime);
+  
+  vdateb[0] = vdate;
+  vdateb[1] = vdate;
+  vtimeb[0] = 0;
+  vtimeb[1] = 0;
+          
+  cdiDecodeDate(vdate, &year, &month, &day);
+  if ( tunit == TUNIT_MONTH )
+    {
+      vdateb[0] = cdiEncodeDate(year, month, 1);
+      month++;
+      if ( month > 12 ) { month = 1; year++; }
+      vdateb[1] = cdiEncodeDate(year, month, 1);
+    }
+  else if ( tunit == TUNIT_YEAR )
+    {
+      vdateb[0] = cdiEncodeDate(year,   1, 1);
+      vdateb[1] = cdiEncodeDate(year+1, 1, 1);
+    }
+  else if ( tunit == TUNIT_DAY )
+    {
+      vdateb[0] = vdate;
+      juldate = juldate_encode(calendar, vdateb[0], vtimeb[0]);
+      juldate = juldate_add_seconds(86400, juldate);
+      juldate_decode(calendar, juldate, &vdateb[1], &vtimeb[1]);
+    }
+}
+
 
 void *Settime(void *argument)
 {
   int nrecs, newval = 0;
-  int tsID1, recID, varID, levelID;
-  int vdate, vtime;
+  int varID, levelID;
   int vdateb[2], vtimeb[2];
   int sdate = 0, stime = 0;
   int taxisID2 = CDI_UNDEFID;
@@ -117,12 +148,10 @@ void *Settime(void *argument)
   int ijulinc = 0, incperiod = 1, incunit = 86400;
   int year = 1, month = 1, day = 1, hour = 0, minute = 0, second = 0;
   int day0;
-  int taxis_has_bounds, copy_timestep = FALSE;
+  bool copy_timestep = false;
   int newcalendar = CALENDAR_STANDARD;
   // int nargs;
-  const char *datestr, *timestr;
   juldate_t juldate;
-  double *array = NULL;
 
   cdoInitialize(argument);
 
@@ -133,6 +162,7 @@ void *Settime(void *argument)
   int SETTIME     = cdoOperatorAdd("settime",      0,  1, "time (format: hh:mm:ss)");
   int SETTUNITS   = cdoOperatorAdd("settunits",    0,  1, "time units (seconds, minutes, hours, days, months, years)");
   int SETTAXIS    = cdoOperatorAdd("settaxis",     0, -2, "date,time<,increment> (format YYYY-MM-DD,hh:mm:ss)");
+  int SETTBOUNDS  = cdoOperatorAdd("settbounds",   0,  1, "frequency (day, month, year)");
   int SETREFTIME  = cdoOperatorAdd("setreftime",   0, -2, "date,time<,units> (format YYYY-MM-DD,hh:mm:ss)");
   int SETCALENDAR = cdoOperatorAdd("setcalendar",  0,  1, "calendar (standard, proleptic_gregorian, 360_day, 365_day, 366_day)");
   int SHIFTTIME   = cdoOperatorAdd("shifttime",    0,  1, "shift value");
@@ -149,8 +179,8 @@ void *Settime(void *argument)
       if ( operatorArgc() < 2 ) cdoAbort("Too few arguments!");
       if ( operatorArgc() > 3 ) cdoAbort("Too many arguments!");
 
-      datestr = operatorArgv()[0];
-      timestr = operatorArgv()[1];
+      const char *datestr = operatorArgv()[0];
+      const char *timestr = operatorArgv()[1];
 
       if ( strchr(datestr+1, '-') )
 	{
@@ -190,7 +220,7 @@ void *Settime(void *argument)
   else if ( operatorID == SETDATE )
     {
       operatorCheckArgc(1);
-      datestr = operatorArgv()[0];
+      const char *datestr = operatorArgv()[0];
       if ( strchr(datestr, '-') )
 	{
 	  sscanf(datestr, "%d-%d-%d", &year, &month, &day);
@@ -204,7 +234,7 @@ void *Settime(void *argument)
   else if ( operatorID == SETTIME )
     {
       operatorCheckArgc(1);
-      timestr = operatorArgv()[0];
+      const char *timestr = operatorArgv()[0];
 
       if ( strchr(timestr, ':') )
 	{
@@ -229,13 +259,17 @@ void *Settime(void *argument)
       /* increment in seconds */
       ijulinc = incperiod * incunit;
     }
-  else if ( operatorID == SETTUNITS )
+  else if ( operatorID == SETTUNITS || operatorID == SETTBOUNDS )
     {
       operatorCheckArgc(1);
       int idum;
       const char *timeunits = operatorArgv()[0];
       incperiod = 0;
       get_tunits(timeunits, &incperiod, &idum, &tunit);
+
+      if ( operatorID == SETTBOUNDS &&
+           !(tunit == TUNIT_DAY || tunit == TUNIT_MONTH || tunit == TUNIT_YEAR) )
+        cdoAbort("Unsupported frequency %s! Use day, month or year.", timeunits);
     }
   else if ( operatorID == SETCALENDAR )
     {
@@ -268,7 +302,7 @@ void *Settime(void *argument)
   int vlistID2 = vlistDuplicate(vlistID1);
 
   int taxisID1 = vlistInqTaxis(vlistID1);
-  taxis_has_bounds = taxisHasBounds(taxisID1);
+  int taxis_has_bounds = taxisHasBounds(taxisID1);
   int ntsteps  = vlistNtsteps(vlistID1);
   int nvars    = vlistNvars(vlistID1);
 
@@ -292,7 +326,7 @@ void *Settime(void *argument)
 
   if ( operatorID == SETREFTIME )
     {
-      copy_timestep = TRUE;
+      copy_timestep = true;
 
       if ( taxisInqType(taxisID1) == TAXIS_ABSOLUTE )
 	{
@@ -308,7 +342,7 @@ void *Settime(void *argument)
     }
   else if ( operatorID == SETTUNITS )
     {
-      copy_timestep = TRUE;
+      copy_timestep = true;
 
       if ( taxisInqType(taxisID1) == TAXIS_ABSOLUTE )
 	{
@@ -322,14 +356,14 @@ void *Settime(void *argument)
     }
   else if ( operatorID == SETCALENDAR )
     {
-      copy_timestep = TRUE;
+      copy_timestep = true;
       /*
       if ( ((char *)argument)[0] == '-' )
 	cdoAbort("This operator does not work with pipes!");
       */
       if ( taxisInqType(taxisID1) == TAXIS_ABSOLUTE )
 	{/*
-	  if ( cdoFiletype() != FILETYPE_NC )
+	  if ( cdoFiletype() != CDI_FILETYPE_NC )
 	    cdoAbort("This operator does not work on an absolute time axis!");
 	 */
 	  cdoPrint("Changing absolute to relative time axis!");
@@ -358,7 +392,7 @@ void *Settime(void *argument)
     }
 
   if ( operatorID != SHIFTTIME )
-    if ( taxis_has_bounds && copy_timestep == FALSE )
+    if ( taxis_has_bounds && !copy_timestep )
       {
 	cdoWarning("Time bounds unsupported by this operator, removed!");
 	taxisDeleteBounds(taxisID2);
@@ -373,13 +407,13 @@ void *Settime(void *argument)
 
   gridsize = vlistGridsizeMax(vlistID1);
   if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-  array = (double*) Malloc(gridsize*sizeof(double));
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID1 = 0;
+  int tsID1 = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID1)) )
     {
-      vdate = taxisInqVdate(taxisID1);
-      vtime = taxisInqVtime(taxisID1);
+      int vdate = taxisInqVdate(taxisID1);
+      int vtime = taxisInqVtime(taxisID1);
 
       if ( operatorID == SETTAXIS )
 	{
@@ -412,6 +446,19 @@ void *Settime(void *argument)
 	      juldate = juldate_add_seconds(ijulinc, juldate);
 	    }
 	}
+      else if ( operatorID == SETTBOUNDS )
+	{
+          gen_bounds(calendar, tunit, vdate, vtime, vdateb, vtimeb);
+          
+          if ( CDO_CMOR_Mode )
+            {
+              juldate_t juldate1 = juldate_encode(calendar, vdateb[0], vtimeb[0]);
+              juldate_t juldate2 = juldate_encode(calendar, vdateb[1], vtimeb[1]);
+              double seconds = juldate_to_seconds(juldate_sub(juldate2, juldate1)) / 2;
+              juldate_t juldatem = juldate_add_seconds((int)lround(seconds), juldate1);
+              juldate_decode(calendar, juldatem, &vdate, &vtime);
+            }
+	}
       else if ( operatorID == SHIFTTIME )
 	{
 	  shifttime(calendar, tunit, ijulinc, &vdate, &vtime);
@@ -459,7 +506,7 @@ void *Settime(void *argument)
 
 	  taxisDefVdate(taxisID2, vdate);
 	  taxisDefVtime(taxisID2, vtime);
-	  if ( taxis_has_bounds )
+	  if ( taxis_has_bounds || operatorID == SETTBOUNDS )
 	    {
 	      taxisDefVdateBounds(taxisID2, vdateb[0], vdateb[1]);
 	      taxisDefVtimeBounds(taxisID2, vtimeb[0], vtimeb[1]);
@@ -468,7 +515,7 @@ void *Settime(void *argument)
 
       streamDefTimestep(streamID2, tsID1);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamDefRecord(streamID2,  varID,  levelID);
