@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -54,12 +54,12 @@ int getkeyval_dp(const char *keyval, const char *key, double *val)
 void *Setzaxis(void *argument)
 {
   int nrecs;
-  int recID, varID, levelID;
+  int varID, levelID;
   int zaxisID1, zaxisID2 = -1;
   int nzaxis, index;
   int nmiss;
   int found;
-  int lztop = FALSE, lzbot = FALSE;
+  bool lztop = false, lzbot = false;
   double ztop = 0, zbot = 0;
 
   cdoInitialize(argument);
@@ -84,8 +84,8 @@ void *Setzaxis(void *argument)
         {
           if ( cdoVerbose ) cdoPrint("keyval[%d]: %s", i+1, parnames[i]);
           
-          if      ( !lzbot && getkeyval_dp(parnames[i], "zbot", &zbot) ) lzbot = TRUE;
-          else if ( !lztop && getkeyval_dp(parnames[i], "ztop", &ztop) ) lztop = TRUE;
+          if      ( !lzbot && getkeyval_dp(parnames[i], "zbot", &zbot) ) lzbot = true;
+          else if ( !lztop && getkeyval_dp(parnames[i], "ztop", &ztop) ) lztop = true;
           else cdoAbort("Parameter >%s< unsupported! Supported parameter are: zbot, ztop", parnames[i]);
         }
     }
@@ -124,14 +124,15 @@ void *Setzaxis(void *argument)
 	{
 	  zaxisID1 = vlistZaxis(vlistID1, index);
           int nlev = zaxisInqSize(zaxisID1);
-          double *levels  = (double *) malloc(nlev*sizeof(double));
-          double *lbounds = (double *) malloc(nlev*sizeof(double));
-          double *ubounds = (double *) malloc(nlev*sizeof(double));
+          double *levels  = (double *) Malloc(nlev*sizeof(double));
+          double *lbounds = (double *) Malloc(nlev*sizeof(double));
+          double *ubounds = (double *) Malloc(nlev*sizeof(double));
 
           if ( nlev > 1 )
             {
-              zaxisInqLevels(zaxisID1, levels);
+              cdoZaxisInqLevels(zaxisID1, levels);
               zaxisID2 = zaxisDuplicate(zaxisID1);
+              if ( !zaxisInqLevels(zaxisID1, NULL) ) zaxisDefLevels(zaxisID2, levels);
 
               genLayerBounds(nlev, levels, lbounds, ubounds);
 
@@ -142,9 +143,9 @@ void *Setzaxis(void *argument)
 	      vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
             }
 
-          free(levels);
-          free(lbounds);
-          free(ubounds);
+          Free(levels);
+          Free(lbounds);
+          Free(ubounds);
 	}
     }
 
@@ -152,7 +153,7 @@ void *Setzaxis(void *argument)
 
   int gridsize = vlistGridsizeMax(vlistID1);
   if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-  double *array = (double*) malloc(gridsize*sizeof(double));
+  double *array = (double*) Malloc(gridsize*sizeof(double));
 
   int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
@@ -161,7 +162,7 @@ void *Setzaxis(void *argument)
 
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamDefRecord(streamID2,  varID,  levelID);
@@ -175,9 +176,9 @@ void *Setzaxis(void *argument)
   streamClose(streamID1);
   streamClose(streamID2);
 
-  if ( array ) free(array);
+  if ( array ) Free(array);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
