@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -41,15 +41,15 @@ static
 int cmpdatetime(const void *s1, const void *s2)
 {
   int cmp = 0;
-  const timeinfo_t *x = s1;
-  const timeinfo_t *y = s2;
+  const timeinfo_t *x = (timeinfo_t *)s1;
+  const timeinfo_t *y = (timeinfo_t *)s2;
   /*
   printf("%g %g  %d %d\n", x->datetime, y->datetime, x, y);
   */
   if      ( x->datetime < y->datetime ) cmp = -1;
   else if ( x->datetime > y->datetime ) cmp =  1;
 
-  return (cmp);
+  return cmp;
 }
 
 
@@ -57,30 +57,26 @@ void *Sorttimestamp(void *argument)
 {
   int gridsize;
   int nrecs;
-  int gridID, varID, levelID, recID;
-  int tsID, tsID2, xtsID, lasttsID = -1;
-  int nfiles, fileID;
-  int nts;
+  int gridID, varID, levelID;
+  int tsID, lasttsID = -1;
   int nalloc = 0;
-  int streamID1, streamID2;
-  int vlistID1 = -1, vlistID2 = -1, taxisID1, taxisID2 = -1;
+  int vlistID2 = -1, taxisID2 = -1;
   int nmiss;
   int nvars = 0, nlevel;
   int *vdate = NULL, *vtime = NULL;
-  field_t ***vars = NULL;
-  timeinfo_t *timeinfo;
+  field_type ***vars = NULL;
 
   cdoInitialize(argument);
 
-  nfiles = cdoStreamCnt() - 1;
+  int nfiles = cdoStreamCnt() - 1;
 
-  xtsID = 0;
-  for ( fileID = 0; fileID < nfiles; fileID++ )
+  int xtsID = 0;
+  for ( int fileID = 0; fileID < nfiles; fileID++ )
     {
-      streamID1 = streamOpenRead(cdoStreamName(fileID));
+      int streamID1 = streamOpenRead(cdoStreamName(fileID));
 
-      vlistID1 = streamInqVlist(streamID1);
-      taxisID1 = vlistInqTaxis(vlistID1);
+      int vlistID1 = streamInqVlist(streamID1);
+      int taxisID1 = vlistInqTaxis(vlistID1);
 
       if ( fileID == 0 )
 	{
@@ -99,15 +95,15 @@ void *Sorttimestamp(void *argument)
 
       nvars = vlistNvars(vlistID1);
 
-      tsID = 0;
+      int tsID = 0;
       while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
 	{
 	  if ( xtsID >= nalloc )
 	    {
 	      nalloc += NALLOC_INC;
-	      vdate = realloc(vdate, nalloc*sizeof(int));
-	      vtime = realloc(vtime, nalloc*sizeof(int));
-	      vars  = realloc(vars, nalloc*sizeof(field_t **));
+	      vdate = (int*) Realloc(vdate, nalloc*sizeof(int));
+	      vtime = (int*) Realloc(vtime, nalloc*sizeof(int));
+	      vars  = (field_type ***) Realloc(vars, nalloc*sizeof(field_type **));
 	    }
 
 	  vdate[xtsID] = taxisInqVdate(taxisID1);
@@ -115,12 +111,12 @@ void *Sorttimestamp(void *argument)
 
 	  vars[xtsID] = field_malloc(vlistID1, FIELD_NONE);
 
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      streamInqRecord(streamID1, &varID, &levelID);
 	      gridID   = vlistInqVarGrid(vlistID1, varID);
 	      gridsize = gridInqSize(gridID);
-	      vars[xtsID][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+	      vars[xtsID][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 	      streamReadRecord(streamID1, vars[xtsID][varID][levelID].ptr, &nmiss);
 	      vars[xtsID][varID][levelID].nmiss = nmiss;
 	    }
@@ -132,19 +128,16 @@ void *Sorttimestamp(void *argument)
       streamClose(streamID1);
     }
 
-  nts = xtsID;
+  int nts = xtsID;
 
-  timeinfo= malloc(nts*sizeof(timeinfo_t));
+  timeinfo_t *timeinfo= (timeinfo_t*) Malloc(nts*sizeof(timeinfo_t));
 
   for ( tsID = 0; tsID < nts; tsID++ )
     {
-      int calendar, julday, secofday;
-      double vdatetime;
-
-      calendar = taxisInqCalendar(taxisID2);
-      julday = date_to_julday(calendar, vdate[tsID]);
-      secofday = time_to_sec(vtime[tsID]);
-      vdatetime = julday + secofday / 86400.;
+      int calendar = taxisInqCalendar(taxisID2);
+      int julday = date_to_julday(calendar, vdate[tsID]);
+      int secofday = time_to_sec(vtime[tsID]);
+      double vdatetime = julday + secofday / 86400.;
       timeinfo[tsID].index    = tsID;
       timeinfo[tsID].datetime = vdatetime;
     }
@@ -155,11 +148,11 @@ void *Sorttimestamp(void *argument)
 
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(nfiles), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(nfiles), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  tsID2 = 0;
+  int tsID2 = 0;
   for ( tsID = 0; tsID < nts; tsID++ )
     {
       xtsID = timeinfo[tsID].index;
@@ -202,13 +195,13 @@ void *Sorttimestamp(void *argument)
       field_free(vars[xtsID], vlistID2);      
     }
 
-  if ( vars  ) free(vars);
-  if ( vdate ) free(vdate);
-  if ( vtime ) free(vtime);
+  if ( vars  ) Free(vars);
+  if ( vdate ) Free(vdate);
+  if ( vtime ) Free(vtime);
 
   streamClose(streamID2);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
