@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,34 +29,29 @@
 
 void *Splitrec(void *argument)
 {
-  int nchars;
-  int streamID1, streamID2;
   int varID;
-  int nrecs;
-  int tsID, recID, levelID;
-  int varID2, levelID2;
-  int vlistID1, vlistID2;
+  int levelID;
   char filesuffix[32];
   char filename[8192];
   const char *refname;
-  int index;
-  int lcopy = FALSE;
   int gridsize;
   int nmiss;
   double *array = NULL;
 
   cdoInitialize(argument);
 
-  if ( UNCHANGED_RECORD ) lcopy = TRUE;
+  if ( processSelf() != 0 ) cdoAbort("This operator can't be combined with other operators!");
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  bool lcopy = UNCHANGED_RECORD;
 
-  vlistID1 = streamInqVlist(streamID1);
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  nrecs  = vlistNrecs(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+
+  int nrecs  = vlistNrecs(vlistID1);
 
   strcpy(filename, cdoStreamName(1)->args);
-  nchars = strlen(filename);
+  int nchars = strlen(filename);
 
   refname = cdoStreamName(0)->argv[cdoStreamName(0)->argc-1];
   filesuffix[0] = 0;
@@ -66,21 +61,21 @@ void *Splitrec(void *argument)
     {
       gridsize = vlistGridsizeMax(vlistID1);
       if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-      array = (double*) malloc(gridsize*sizeof(double));
+      array = (double*) Malloc(gridsize*sizeof(double));
     }
 
-  index = 0;
-  tsID = 0;
+  int index = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
 	  vlistClearFlag(vlistID1);
 	  vlistDefFlag(vlistID1, varID, levelID, TRUE);
 
-	  vlistID2 = vlistCreate();
+	  int vlistID2 = vlistCreate();
 	  vlistCopyFlag(vlistID2, vlistID1);
 
 	  index++;
@@ -91,13 +86,13 @@ void *Splitrec(void *argument)
 	  if ( cdoVerbose ) cdoPrint("create file %s", filename);
 
 	  argument_t *fileargument = file_argument_new(filename);
-	  streamID2 = streamOpenWrite(fileargument, cdoFiletype());
+	  int streamID2 = streamOpenWrite(fileargument, cdoFiletype());
 	  file_argument_free(fileargument);
 
 	  streamDefVlist(streamID2, vlistID2);
 
-	  varID2   = vlistFindVar(vlistID2, varID);
-	  levelID2 = vlistFindLevel(vlistID2, varID, levelID);
+	  int varID2   = vlistFindVar(vlistID2, varID);
+	  int levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
 	  streamDefTimestep(streamID2, 0);
 	  streamDefRecord(streamID2, varID2, levelID2);
@@ -121,9 +116,9 @@ void *Splitrec(void *argument)
   streamClose(streamID1);
 
   if ( ! lcopy )
-    if ( array ) free(array);
+    if ( array ) Free(array);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
