@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,44 +30,33 @@
 void *Splitsel(void *argument)
 {
   int gridsize;
-  //int vdate = 0, vtime = 0;
   int nrecs = 0;
-  int varID, levelID, recID;
-  int tsID, tsID2;
-  int nsets;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2, taxisID1, taxisID2;
+  int varID, levelID;
+  int tsID;
   int nmiss;
   int gridID;
-  int nvars, nlevel;
-  int nconst;
-/*   int ndates = 0, noffset = 0, nskip = 0, nargc; */
-  double ndates, noffset, nskip;
+  int nlevel;
   int i2 = 0;
-  int nargc;
-
   /* from Splittime.c */
   int nchars;
   char filesuffix[32];
   char filename[8192];
   const char *refname;
-  int index = 0;
-  int lcopy = FALSE;
+  double ndates, noffset, nskip;
   double *array = NULL;
-  field_t **vars = NULL;
+  field_type **vars = NULL;
 
   cdoInitialize(argument);
 
   if ( processSelf() != 0 ) cdoAbort("This operator can't be combined with other operators!");
 
-  cdoOperatorAdd("splitsel",  0,  0, NULL);
+  bool lcopy = UNCHANGED_RECORD;
 
-  if ( UNCHANGED_RECORD ) lcopy = TRUE;
+  cdoOperatorAdd("splitsel",  0,  0, NULL);
 
   /*  operatorInputArg("nsets <noffset <nskip>>"); */
 
-  nargc = operatorArgc();
-
+  int nargc = operatorArgc();
   if ( nargc < 1 )
     cdoAbort("Too few arguments! Need %d found %d.", 1, nargc);
 
@@ -75,21 +64,21 @@ void *Splitsel(void *argument)
 /*   if ( nargc > 1 ) noffset = parameter2int(operatorArgv()[1]); */
 /*   if ( nargc > 2 ) nskip   = parameter2int(operatorArgv()[2]); */
 /*   printf("%s %s %s\n", operatorArgv()[0],operatorArgv()[1],operatorArgv()[2]); */
-  ndates = noffset = nskip = 0.0;
+  noffset = nskip = 0.0;
   ndates = parameter2double(operatorArgv()[0]);
   if ( nargc > 1 ) noffset = parameter2double(operatorArgv()[1]);
   if ( nargc > 2 ) nskip   = parameter2double(operatorArgv()[2]);
 
   if ( cdoVerbose ) cdoPrint("nsets = %f, noffset = %f, nskip = %f", ndates, noffset, nskip);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
 /*   taxisID2 = taxisCreate(TAXIS_ABSOLUTE); */
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   strcpy(filename, cdoStreamName(1)->args);
@@ -106,14 +95,14 @@ void *Splitsel(void *argument)
       array = (double*) Malloc(gridsize*sizeof(double));
     }
 
-  nvars = vlistNvars(vlistID1);
-  nconst = 0;
+  int nvars = vlistNvars(vlistID1);
+  int nconst = 0;
   for ( varID = 0; varID < nvars; varID++ )
     if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) nconst++;
 
   if ( nconst )
     {
-      vars = (field_t **) Malloc(nvars*sizeof(field_t *));
+      vars = (field_type **) Malloc(nvars*sizeof(field_type *));
 
       for ( varID = 0; varID < nvars; varID++ )
 	{
@@ -123,7 +112,7 @@ void *Splitsel(void *argument)
 	      nlevel  = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	      gridsize = gridInqSize(gridID);
 		  
-	      vars[varID] = (field_t*) Malloc(nlevel*sizeof(field_t));
+	      vars[varID] = (field_type*) Malloc(nlevel*sizeof(field_type));
 
 	      for ( levelID = 0; levelID < nlevel; levelID++ )
 		{
@@ -135,6 +124,8 @@ void *Splitsel(void *argument)
 	}
     }
 
+  int index = 0;
+  int nsets = 0;
 
   /* offset */
   for ( tsID = 0; tsID < noffset; tsID++ )
@@ -147,7 +138,7 @@ void *Splitsel(void *argument)
 	}
 
       if ( tsID == 0 && nconst )
-	for ( recID = 0; recID < nrecs; recID++ )
+	for ( int recID = 0; recID < nrecs; recID++ )
 	  {
 	    streamInqRecord(streamID1, &varID, &levelID);
 	    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT )
@@ -158,8 +149,6 @@ void *Splitsel(void *argument)
           }
     }
 
-  index = 0;
-  nsets = 0;
   while ( TRUE )
     {
       sprintf(filename+nchars, "%06d", index);
@@ -167,24 +156,18 @@ void *Splitsel(void *argument)
 	  
       if ( cdoVerbose ) cdoPrint("create file %s", filename);
       argument_t *fileargument = file_argument_new(filename);
-      streamID2 = streamOpenWrite(fileargument, cdoFiletype());
+      int streamID2 = streamOpenWrite(fileargument, cdoFiletype());
       file_argument_free(fileargument);
 
       streamDefVlist(streamID2, vlistID2);
 
-      tsID2 = 0;
+      int tsID2 = 0;
 
       for ( ; nsets < (int)(ndates*(index+1)); nsets++ ) 
 	{
 	  nrecs = streamInqTimestep(streamID1, tsID);
 	  if ( nrecs == 0 ) break;
 
-	  /*
-	  vdate = taxisInqVdate(taxisID1);
-	  vtime = taxisInqVtime(taxisID1);
-	  printf("vdate: %d vtime: %d\n", vdate, vtime);
-	   */
-
 	  taxisCopyTimestep(taxisID2, taxisID1);
 	  streamDefTimestep(streamID2, tsID2);
 
@@ -205,7 +188,7 @@ void *Splitsel(void *argument)
 		}
 	    }
 
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      
 	      streamInqRecord(streamID1, &varID, &levelID);
