@@ -26,32 +26,17 @@
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
-#include "percentiles.h"
+#include "percentiles_hist.h"
 
 
 void *Timselpctl(void *argument)
 {
-  int gridsize;
-  int vdate1 = 0, vtime1 = 0;
-  int vdate2 = 0, vtime2 = 0;
-  int vdate3 = 0, vtime3 = 0;
-  int vdate4 = 0, vtime4 = 0;
-  int nrecs = 0, nrecords;
-  int gridID, varID, levelID, recID;
+  int timestat_date = TIMESTAT_MEAN;
+  int nrecs = 0;
+  int gridID, varID, levelID;
   int tsID;
-  int otsID;
-  int nsets = 0;
-  int i;
-  int streamID1, streamID2, streamID3, streamID4;
-  int vlistID1, vlistID2, vlistID3, vlistID4, taxisID1, taxisID2, taxisID3, taxisID4;
   int nmiss;
-  int nvars, nlevels;
-  int ndates = 0, noffset = 0, nskip = 0, nargc;
-  int *recVarID, *recLevelID;
-  field_t **vars1 = NULL;
-  field_t field;
-  double pn;
-  HISTOGRAM_SET *hset = NULL;
+  int nlevels;
 
   cdoInitialize(argument);
 
@@ -59,57 +44,60 @@ void *Timselpctl(void *argument)
 
   operatorInputArg("percentile number, nsets <,noffset <,nskip>>");
 
-  nargc = operatorArgc();
-  if ( nargc < 2 )
-    cdoAbort("Too few arguments! Need %d found %d.", 2, nargc);
+  int nargc = operatorArgc();
+  if ( nargc < 2 ) cdoAbort("Too few arguments! Need %d found %d.", 2, nargc);
 
-  pn     = atof(operatorArgv()[0]);
-  ndates = atoi(operatorArgv()[1]);
-  if ( nargc > 2 ) noffset = atoi(operatorArgv()[2]);
-  if ( nargc > 3 ) nskip   = atoi(operatorArgv()[3]);
-
-  if ( !(pn > 0 && pn < 100) )
-    cdoAbort("Illegal argument: percentile number %g is not in the range 0..100!", pn);
+  double pn  = parameter2double(operatorArgv()[0]);
+  percentile_check_number(pn);
+  int ndates = parameter2int(operatorArgv()[1]);
+  int noffset = 0, nskip = 0;
+  if ( nargc > 2 ) noffset = parameter2int(operatorArgv()[2]);
+  if ( nargc > 3 ) nskip   = parameter2int(operatorArgv()[3]);
 
   if ( cdoVerbose ) cdoPrint("nsets = %d, noffset = %d, nskip = %d", ndates, noffset, nskip);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
-  streamID3 = streamOpenRead(cdoStreamName(2));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID3 = streamOpenRead(cdoStreamName(2));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistID3 = streamInqVlist(streamID3);
-  vlistID4 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistID3 = streamInqVlist(streamID3);
+  int vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = vlistInqTaxis(vlistID2);
-  taxisID3 = vlistInqTaxis(vlistID3);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = vlistInqTaxis(vlistID2);
+  int taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  taxisID4 = taxisDuplicate(taxisID1);
+  int taxisID4 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  streamID4 = streamOpenWrite(cdoStreamName(3), cdoFiletype());
+  int streamID4 = streamOpenWrite(cdoStreamName(3), cdoFiletype());
 
   streamDefVlist(streamID4, vlistID4);
 
-  nvars    = vlistNvars(vlistID1);
-  nrecords = vlistNrecs(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = malloc(nrecords*sizeof(int));
-  recLevelID = malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  dtlist_type *dtlist = dtlist_new();
+  dtlist_set_stat(dtlist, timestat_date);
+  dtlist_set_calendar(dtlist, taxisInqCalendar(taxisID1));
 
+  int gridsize = vlistGridsizeMax(vlistID1);
+
+  field_type field;
   field_init(&field);
-  field.ptr = malloc(gridsize * sizeof(double));
+  field.ptr = (double*) Malloc(gridsize * sizeof(double));
 
-  vars1 = field_malloc(vlistID1, FIELD_PTR);
-  hset = hsetCreate(nvars);
+  field_type **vars1 = field_malloc(vlistID1, FIELD_PTR);
+  HISTOGRAM_SET *hset = hsetCreate(nvars);
 
   for ( varID = 0; varID < nvars; varID++ )
     {
@@ -124,7 +112,7 @@ void *Timselpctl(void *argument)
       nrecs = streamInqTimestep(streamID1, tsID);
       if ( nrecs == 0 ) break;
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -136,33 +124,34 @@ void *Timselpctl(void *argument)
 	}
     }
 
+  int otsID = 0;
   if ( tsID < noffset )
     {
       cdoWarning("noffset is larger than number of timesteps!");
       goto LABEL_END;
     }
 
-  otsID = 0;
   while ( TRUE )
     {
       nrecs = streamInqTimestep(streamID2, otsID);
       if ( nrecs != streamInqTimestep(streamID3, otsID) )
         cdoAbort("Number of records at time step %d of %s and %s differ!", otsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args);
 
-      vdate2 = taxisInqVdate(taxisID2);
-      vtime2 = taxisInqVtime(taxisID2);
-      vdate3 = taxisInqVdate(taxisID3);
-      vtime3 = taxisInqVtime(taxisID3);
+      int vdate2 = taxisInqVdate(taxisID2);
+      int vtime2 = taxisInqVtime(taxisID2);
+      int vdate3 = taxisInqVdate(taxisID3);
+      int vtime3 = taxisInqVtime(taxisID3);
       if ( vdate2 != vdate3 || vtime2 != vtime3 )
         cdoAbort("Verification dates at time step %d of %s and %s differ!", otsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args);
       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
         {
           streamInqRecord(streamID2, &varID, &levelID);
           streamReadRecord(streamID2, vars1[varID][levelID].ptr, &nmiss);
           vars1[varID][levelID].nmiss = nmiss;
         }
-      for ( recID = 0; recID < nrecs; recID++ )
+
+      for ( int recID = 0; recID < nrecs; recID++ )
         {
           streamInqRecord(streamID3, &varID, &levelID);
           streamReadRecord(streamID3, field.ptr, &nmiss);
@@ -172,46 +161,37 @@ void *Timselpctl(void *argument)
           
           hsetDefVarLevelBounds(hset, varID, levelID, &vars1[varID][levelID], &field);
         }
-      
+
+      int nsets = 0;
       if ( nrecs )
-      for ( nsets = 0; nsets < ndates; nsets++ )
-	{
-	  nrecs = streamInqTimestep(streamID1, tsID);
-	  if ( nrecs == 0 ) break;
+	for ( nsets = 0; nsets < ndates; nsets++ )
+	  {
+	    nrecs = streamInqTimestep(streamID1, tsID);
+	    if ( nrecs == 0 ) break;
 
-	  vdate1 = taxisInqVdate(taxisID1);
-	  vtime1 = taxisInqVtime(taxisID1);
+	    dtlist_taxisInqTimestep(dtlist, taxisID1, nsets);
 
-	  for ( recID = 0; recID < nrecs; recID++ )
-	    {
-	      streamInqRecord(streamID1, &varID, &levelID);
+	    for ( int recID = 0; recID < nrecs; recID++ )
+	      {
+		streamInqRecord(streamID1, &varID, &levelID);
 
-	      if ( tsID == 0 )
-		{
-		  recVarID[recID]   = varID;
-		  recLevelID[recID] = levelID;
-		}
+		if ( tsID == 0 )
+		  {
+		    recVarID[recID]   = varID;
+		    recLevelID[recID] = levelID;
+		  }
 
-	      streamReadRecord(streamID1, vars1[varID][levelID].ptr, &nmiss);
-	      vars1[varID][levelID].nmiss = nmiss;
+		streamReadRecord(streamID1, vars1[varID][levelID].ptr, &nmiss);
+		vars1[varID][levelID].nmiss = nmiss;
                   
-              hsetAddVarLevelValues(hset, varID, levelID, &vars1[varID][levelID]);
-	    }
+		hsetAddVarLevelValues(hset, varID, levelID, &vars1[varID][levelID]);
+	      }
 
-	  vdate4 = vdate1;
-	  vtime4 = vtime1;
-	  tsID++;
-	}
+	    tsID++;
+	  }
 
       if ( nrecs == 0 && nsets == 0 ) break;
 
-      if ( vdate2 != vdate4 )
-        cdoAbort("Verification dates at time step %d of %s, %s and %s differ!",
-		 otsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args, cdoStreamName(3)->args);
-      if ( vtime2 != vtime4 )
-        cdoAbort("Verification times at time step %d of %s, %s and %s differ!",
-		 otsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args, cdoStreamName(3)->args);
-
       for ( varID = 0; varID < nvars; varID++ )
         {
           if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
@@ -221,11 +201,10 @@ void *Timselpctl(void *argument)
             hsetGetVarLevelPercentiles(&vars1[varID][levelID], hset, varID, levelID, pn);
         }
 
-      taxisDefVdate(taxisID4, vdate4);
-      taxisDefVtime(taxisID4, vtime4);
+      dtlist_stat_taxisDefTimestep(dtlist, taxisID4, nsets);
       streamDefTimestep(streamID4, otsID);
 
-      for ( recID = 0; recID < nrecords; recID++ )
+      for ( int recID = 0; recID < nrecords; recID++ )
 	{
 	  varID   = recVarID[recID];
 	  levelID = recLevelID[recID];
@@ -239,7 +218,7 @@ void *Timselpctl(void *argument)
       if ( nrecs == 0 ) break;
       otsID++;
 
-      for ( i = 0; i < nskip; i++ )
+      for ( int i = 0; i < nskip; i++ )
 	{
 	  nrecs = streamInqTimestep(streamID1, tsID);
 	  if ( nrecs == 0 ) break;
@@ -254,10 +233,12 @@ void *Timselpctl(void *argument)
   field_free(vars1, vlistID1);
   hsetDestroy(hset);
 
-  if ( field.ptr ) free(field.ptr);
+  dtlist_delete(dtlist);
 
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
+  if ( field.ptr ) Free(field.ptr);
+
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
 
   streamClose(streamID4);
   streamClose(streamID3);
@@ -266,5 +247,5 @@ void *Timselpctl(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
