@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -40,10 +40,9 @@ void *Timselstat(void *argument)
 {
   int timestat_date = TIMESTAT_MEAN;
   int nrecs = 0;
-  int varID, levelID, recID;
+  int varID, levelID;
   int tsID;
   int nsets;
-  int i;
   int nmiss;
   int nlevel;
 
@@ -102,13 +101,13 @@ void *Timselstat(void *argument)
 
   int gridsize = vlistGridsizeMax(vlistID1);
 
-  field_t field;
+  field_type field;
   field_init(&field);
   field.ptr = (double*) Malloc(gridsize*sizeof(double));
 
-  field_t **samp1 = field_malloc(vlistID1, FIELD_NONE);
-  field_t **vars1 = field_malloc(vlistID1, FIELD_PTR);
-  field_t **vars2 = NULL;
+  field_type **samp1 = field_malloc(vlistID1, FIELD_NONE);
+  field_type **vars1 = field_malloc(vlistID1, FIELD_PTR);
+  field_type **vars2 = NULL;
   if ( lvarstd ) vars2 = field_malloc(vlistID1, FIELD_PTR);
 
   for ( tsID = 0; tsID < noffset; tsID++ )
@@ -116,7 +115,7 @@ void *Timselstat(void *argument)
       nrecs = streamInqTimestep(streamID1, tsID);
       if ( nrecs == 0 ) break;
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -144,7 +143,7 @@ void *Timselstat(void *argument)
 
 	  dtlist_taxisInqTimestep(dtlist, taxisID1, nsets);
 
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      streamInqRecord(streamID1, &varID, &levelID);
 
@@ -166,7 +165,7 @@ void *Timselstat(void *argument)
 		      if ( samp1[varID][levelID].ptr == NULL )
 			samp1[varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 
-		      for ( i = 0; i < gridsize; i++ )
+		      for ( int i = 0; i < gridsize; i++ )
 			if ( DBL_IS_EQUAL(vars1[varID][levelID].ptr[i],
 					  vars1[varID][levelID].missval) )
 			  samp1[varID][levelID].ptr[i] = 0;
@@ -186,11 +185,11 @@ void *Timselstat(void *argument)
 		      if ( samp1[varID][levelID].ptr == NULL )
 			{
 			  samp1[varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
-			  for ( i = 0; i < gridsize; i++ )
+			  for ( int i = 0; i < gridsize; i++ )
 			    samp1[varID][levelID].ptr[i] = nsets;
 			}
 
-		      for ( i = 0; i < gridsize; i++ )
+		      for ( int i = 0; i < gridsize; i++ )
 			if ( !DBL_IS_EQUAL(field.ptr[i], vars1[varID][levelID].missval) )
 			  samp1[varID][levelID].ptr[i]++;
 		    }
@@ -261,7 +260,7 @@ void *Timselstat(void *argument)
       dtlist_stat_taxisDefTimestep(dtlist, taxisID2, nsets);
       streamDefTimestep(streamID2, otsID);
 
-      for ( recID = 0; recID < nrecords; recID++ )
+      for ( int recID = 0; recID < nrecords; recID++ )
 	{
 	  varID   = recVarID[recID];
 	  levelID = recLevelID[recID];
@@ -275,7 +274,7 @@ void *Timselstat(void *argument)
       if ( nrecs == 0 ) break;
       otsID++;
 
-      for ( i = 0; i < nskip; i++ )
+      for ( int i = 0; i < nskip; i++ )
 	{
 	  nrecs = streamInqTimestep(streamID1, tsID);
 	  if ( nrecs == 0 ) break;
