@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,13 +34,13 @@ static
 int cmpdarray(const void *s1, const void *s2)
 {
   int cmp = 0;
-  const double *x = s1;
-  const double *y = s2;
+  const double *x = (double *)s1;
+  const double *y = (double *)s2;
 
   if      ( *x < *y ) cmp = -1;
   else if ( *x > *y ) cmp =  1;
 
-  return (cmp);
+  return cmp;
 }
 
 
@@ -48,45 +48,39 @@ void *Timsort(void *argument)
 {
   int gridsize;
   int nrecs;
-  int gridID, varID, levelID, recID;
-  int tsID;
-  int i;
-  int nts;
+  int gridID, varID, levelID;
   int nalloc = 0;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2, taxisID1, taxisID2;
   int nmiss;
-  int nvars, nlevel;
+  int nlevel;
   int *vdate = NULL, *vtime = NULL;
-  double **sarray = NULL;
-  field_t ***vars = NULL;
+  field_type ***vars = NULL;
 
   cdoInitialize(argument);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisCreate(TAXIS_ABSOLUTE);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       if ( tsID >= nalloc )
 	{
 	  nalloc += NALLOC_INC;
-	  vdate = (int*) realloc(vdate, nalloc*sizeof(int));
-	  vtime = (int*) realloc(vtime, nalloc*sizeof(int));
-	  vars  = (field_t ***) realloc(vars, nalloc*sizeof(field_t **));
+	  vdate = (int*) Realloc(vdate, nalloc*sizeof(int));
+	  vtime = (int*) Realloc(vtime, nalloc*sizeof(int));
+	  vars  = (field_type ***) Realloc(vars, nalloc*sizeof(field_type **));
 	}
 
       vdate[tsID] = taxisInqVdate(taxisID1);
@@ -94,12 +88,12 @@ void *Timsort(void *argument)
 
       vars[tsID] = field_malloc(vlistID1, FIELD_NONE);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridID   = vlistInqVarGrid(vlistID1, varID);
 	  gridsize = gridInqSize(gridID);
-	  vars[tsID][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
+	  vars[tsID][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 	  streamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
 	  vars[tsID][varID][levelID].nmiss = nmiss;
 	}
@@ -107,11 +101,11 @@ void *Timsort(void *argument)
       tsID++;
     }
 
-  nts = tsID;
+  int nts = tsID;
 
-  sarray = (double **) malloc(ompNumThreads*sizeof(double *));
-  for ( i = 0; i < ompNumThreads; i++ )
-    sarray[i] = (double*) malloc(nts*sizeof(double));
+  double **sarray = (double **) Malloc(ompNumThreads*sizeof(double *));
+  for ( int i = 0; i < ompNumThreads; i++ )
+    sarray[i] = (double*) Malloc(nts*sizeof(double));
 
   for ( varID = 0; varID < nvars; varID++ )
     {
@@ -123,27 +117,27 @@ void *Timsort(void *argument)
       for ( levelID = 0; levelID < nlevel; levelID++ )
 	{
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, tsID)
+#pragma omp parallel for default(none) shared(gridsize,nts,sarray,vars,varID,levelID)
 #endif
-	  for ( i = 0; i < gridsize; i++ )
+	  for ( int i = 0; i < gridsize; i++ )
 	    {
 	      int ompthID = cdo_omp_get_thread_num();
 
-	      for ( tsID = 0; tsID < nts; tsID++ )
+	      for ( int tsID = 0; tsID < nts; tsID++ )
 		sarray[ompthID][tsID] = vars[tsID][varID][levelID].ptr[i];
 
 	      qsort(sarray[ompthID], nts, sizeof(double), cmpdarray);  	      
 
-	      for ( tsID = 0; tsID < nts; tsID++ )
+	      for ( int tsID = 0; tsID < nts; tsID++ )
 		vars[tsID][varID][levelID].ptr[i] = sarray[ompthID][tsID];
 	    }
 	}
     }
 
-  for ( i = 0; i < ompNumThreads; i++ )
-    if ( sarray[i] ) free(sarray[i]);
+  for ( int i = 0; i < ompNumThreads; i++ )
+    if ( sarray[i] ) Free(sarray[i]);
 
-  if ( sarray ) free(sarray);
+  if ( sarray ) Free(sarray);
 
   for ( tsID = 0; tsID < nts; tsID++ )
     {
@@ -168,14 +162,14 @@ void *Timsort(void *argument)
       field_free(vars[tsID], vlistID1);      
     }
 
-  if ( vars  ) free(vars);
-  if ( vdate ) free(vdate);
-  if ( vtime ) free(vtime);
+  if ( vars  ) Free(vars);
+  if ( vdate ) Free(vdate);
+  if ( vtime ) Free(vtime);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
