@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,8 +24,8 @@
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
+#include "calendar.h"
 #include "pstream.h"
-#include "dtypes.h"
 
 
 #define MAX_GAPS   64
@@ -33,15 +33,12 @@
 #define LIM_NTSM 1024
 
 enum {TU_SECONDS=0, TU_MINUTES, TU_HOURS, TU_DAYS, TU_MONTHS, TU_YEARS};
-char *tunits[] = {"second", "minute", "hour", "day", "month", "year"};
+const char *tunits[] = {"second", "minute", "hour", "day", "month", "year"};
 int   iunits[] = {1, 60, 3600, 86400, 1, 12};
 
 
 void getTimeInc(double jdelta, int vdate0, int vdate1, int *incperiod, int *incunit)
 {
-  int year0, month0, day0;
-  int year1, month1, day1;
-  int deltam, deltay;
   int lperiod;
   int sign = 1;
 
@@ -55,8 +52,7 @@ void getTimeInc(double jdelta, int vdate0, int vdate1, int *incperiod, int *incu
 
   if ( lperiod < 0 )
     {
-      int tmp;
-      tmp = vdate1;
+      int tmp = vdate1;
       vdate1 = vdate0;
       vdate0 = tmp;
       lperiod = -lperiod;
@@ -65,38 +61,42 @@ void getTimeInc(double jdelta, int vdate0, int vdate1, int *incperiod, int *incu
 
   // printf("\n%d %d %d\n",lperiod, vdate0, vdate1);
 
+  int year0, month0, day0;
   cdiDecodeDate(vdate0, &year0, &month0, &day0);
+  int year1, month1, day1;
   cdiDecodeDate(vdate1, &year1, &month1, &day1);
 
-  deltay = year1-year0;
-  deltam = deltay*12 + (month1-month0);
+  int deltay = year1-year0;
+  int deltam = deltay*12 + (month1-month0);
+  if ( deltay == 0 ) deltay = 1;
+  if ( deltam == 0 ) deltam = 1;
 
-  if ( lperiod/60 > 0 && lperiod/60 < 60 )
+  if ( lperiod/60 >= 1 && lperiod/60 < 60 )
     {
       *incperiod = lperiod/60;
       *incunit = TU_MINUTES;
     }
-  else if ( lperiod/3600 > 0 && lperiod/3600 < 24 )
+  else if ( lperiod/3600 >= 1 && lperiod/3600 < 24 )
     {
       *incperiod = lperiod/3600;
       *incunit = TU_HOURS;
     }
-  else if ( lperiod/(3600*24) > 0 && lperiod/(3600*24) < 32 )
+  else if ( lperiod/(3600*24) >= 1 && lperiod/(3600*24) < 32 )
     {
       *incperiod = lperiod/(3600*24);
       *incunit = TU_DAYS;
-      if ( *incperiod > 27 && deltam == 1 )
+      if ( *incperiod >= 27 && deltam == 1 )
 	{
 	  *incperiod = 1;
 	  *incunit = TU_MONTHS;
 	}
     }
-  else if ( lperiod/(3600*24*30) > 0 && lperiod/(3600*24*30) < 12 )
+  else if ( lperiod/(3600*24*30) >= 1 && lperiod/(3600*24*30) < 12 )
     {
       *incperiod = deltam;
       *incunit = TU_MONTHS;
     }
-  else if ( lperiod/(3600*24*30*12) > 0 )
+  else if ( lperiod/(3600*24*30*12) >= 1 )
     {
       *incperiod = deltay;
       *incunit = TU_YEARS;
@@ -114,17 +114,12 @@ static
 void printBounds(int taxisID, int calendar)
 {
   int vdate0, vdate1;
-  int vtime0, vtime1;
-  int incperiod = 0, incunit = 0;
-  juldate_t juldate1, juldate0;
-  double jdelta;
-  int i, len;
-  char vdatestr[32], vtimestr[32];
-
   taxisInqVdateBounds(taxisID, &vdate0, &vdate1);
+  int vtime0, vtime1;
   taxisInqVtimeBounds(taxisID, &vtime0, &vtime1);
 
-  date2str(vdate0, vdatestr, sizeof(vdatestr));
+   char vdatestr[32], vtimestr[32];
+   date2str(vdate0, vdatestr, sizeof(vdatestr));
   time2str(vtime0, vtimestr, sizeof(vtimestr));	  
   fprintf(stdout, " %s %s", vdatestr, vtimestr);
 
@@ -132,15 +127,16 @@ void printBounds(int taxisID, int calendar)
   time2str(vtime1, vtimestr, sizeof(vtimestr));	  
   fprintf(stdout, " %s %s", vdatestr, vtimestr);
 
-  juldate0  = juldate_encode(calendar, vdate0, vtime0);
-  juldate1  = juldate_encode(calendar, vdate1, vtime1);
-  jdelta    = juldate_to_seconds(juldate_sub(juldate1, juldate0));
+  juldate_t juldate0  = juldate_encode(calendar, vdate0, vtime0);
+  juldate_t juldate1  = juldate_encode(calendar, vdate1, vtime1);
+  double    jdelta    = juldate_to_seconds(juldate_sub(juldate1, juldate0));
 
+  int incperiod = 0, incunit = 0;
   getTimeInc(jdelta, vdate0, vdate1, &incperiod, &incunit);
   
   /* fprintf(stdout, "  %g  %g  %g  %d", jdelta, jdelta/3600, fmod(jdelta,3600), incperiod%3600);*/
-  len = fprintf(stdout, " %3d %s%s", incperiod, tunits[incunit], abs(incperiod)>1?"s":"");
-  for ( i = 0; i < 11-len; ++i ) fprintf(stdout, " ");
+  int len = fprintf(stdout, " %3d %s%s", incperiod, tunits[incunit], abs(incperiod)!=1?"s":"");
+  for ( int i = 0; i < 11-len; ++i ) fprintf(stdout, " ");
 }
 
 static
@@ -213,7 +209,7 @@ int fill_gap(int ngaps, int ntsm[MAX_NTSM], int rangetsm[MAX_GAPS][2],
       ntsm[ngaps] = its;
     }
 
-  return (its);
+  return its;
 }
 
 
@@ -222,16 +218,13 @@ void *Tinfo(void *argument)
   int vdate_first = 0, vtime_first = 0;
   int vdate0 = 0, vtime0 = 0;
   int vdate = 0, vtime = 0;
-  int fdate = 0, ftime = 0;
-  int nrecs, ntsteps;
+  int nrecs;
   int tsID = 0, ntimeout;
-  int taxisID;
-  int streamID;
-  int vlistID;
+  int calendar;
   int year0, month0, day0;
   int year, month, day;
-  int calendar, unit;
-  int lforecast = FALSE;
+  int unit;
+  bool lforecast = false;
   int incperiod0 = 0, incunit0 = 0;
   int incperiod = 0, incunit = 0;
   int its = 0, igap;
@@ -248,14 +241,14 @@ void *Tinfo(void *argument)
 
   cdoInitialize(argument);
 
-  streamID = streamOpenRead(cdoStreamName(0));
+  int streamID = streamOpenRead(cdoStreamName(0));
 
-  vlistID = streamInqVlist(streamID);
+  int vlistID = streamInqVlist(streamID);
 
   fprintf(stdout, "\n");
 
-  taxisID = vlistInqTaxis(vlistID);
-  ntsteps = vlistNtsteps(vlistID);
+  int taxisID = vlistInqTaxis(vlistID);
+  int ntsteps = vlistNtsteps(vlistID);
 
   if ( ntsteps != 0 )
     {
@@ -289,8 +282,8 @@ void *Tinfo(void *argument)
 
 	      if ( taxisInqType(taxisID) == TAXIS_FORECAST )
 		{
-		  fdate = taxisInqFdate(taxisID);
-		  ftime = taxisInqFtime(taxisID);
+		  int fdate = taxisInqFdate(taxisID);
+		  int ftime = taxisInqFtime(taxisID);
 	      
 		  date2str(fdate, vdatestr, sizeof(vdatestr));
 		  time2str(ftime, vtimestr, sizeof(vtimestr));
@@ -302,12 +295,12 @@ void *Tinfo(void *argument)
 
 		  fprintf(stdout, "\n");
 
-		  lforecast = TRUE;
+		  lforecast = true;
 		}
 	    }
 	}
 
-      calendar = taxisInqCalendar(taxisID);
+      int calendar = taxisInqCalendar(taxisID);
 
       fprintf(stdout, "\n");
       fprintf(stdout, "         Verification Time              ");
@@ -346,7 +339,7 @@ void *Tinfo(void *argument)
 	      getTimeInc(jdelta, vdate0, vdate, &incperiod, &incunit);
 
 	      /* fprintf(stdout, "  %g  %g  %g  %d", jdelta, jdelta/3600, fmod(jdelta,3600), incperiod%3600);*/
-	      len = fprintf(stdout, " %3d %s%s", incperiod, tunits[incunit], abs(incperiod)>1?"s":"");
+	      len = fprintf(stdout, " %3d %s%s", incperiod, tunits[incunit], abs(incperiod)!=1?"s":"");
 	      for ( i = 0; i < 11-len; ++i ) fprintf(stdout, " ");
 	    }
 	  else
@@ -358,8 +351,8 @@ void *Tinfo(void *argument)
 
 	  if ( lforecast )
 	    {
-	      fdate = taxisInqFdate(taxisID);
-	      ftime = taxisInqFtime(taxisID);
+	      int fdate = taxisInqFdate(taxisID);
+	      int ftime = taxisInqFtime(taxisID);
 	      
 	      date2str(fdate, vdatestr, sizeof(vdatestr));
 	      time2str(ftime, vtimestr, sizeof(vtimestr));
@@ -408,7 +401,7 @@ void *Tinfo(void *argument)
 		  ngaps++;
 		  if ( cdoVerbose )
 		    fprintf(stdout, "  %c--- Gap %d, missing %s%d timestep%s",
-			    arrow, ngaps, its>=LIM_NTSM?"more than ":"", its, its>1?"s":"");
+			    arrow, ngaps, its>=LIM_NTSM?"more than ":"", its, its!=1?"s":"");
 		}
 	      else if ( its < 0 )
 		{
@@ -446,13 +439,13 @@ void *Tinfo(void *argument)
   fprintf(stdout, " End date            : %s %s\n", vdatestr, vtimestr);
 
   fprintf(stdout, " Increment           : %3d %s%s\n", 
-	  incperiod0, tunits[incunit0], incperiod0>1?"s":"");
+	  incperiod0, tunits[incunit0], incperiod0!=1?"s":"");
   fprintf(stdout, " Number of timesteps : %d\n", tsID);
   fprintf(stdout, " Gaps identified     : %d\n", ngaps);
 
   if ( cdoVerbose && ngaps )
     {
-      fprintf(stdout, "\nFound potentially %d gap%s in the time series", ngaps, ngaps>1?"s":"");
+      fprintf(stdout, "\nFound potentially %d gap%s in the time series", ngaps, ngaps!=1?"s":"");
       if ( ngaps >= MAX_GAPS )
 	{
 	  ngaps = MAX_GAPS;
@@ -462,7 +455,7 @@ void *Tinfo(void *argument)
       for ( igap = 0; igap < ngaps; ++igap )
 	{
 	  fprintf(stdout, "  Gap %d between timestep %d and %d, missing %d timestep%s",
-		  igap+1, rangetsm[igap][0], rangetsm[igap][1], ntsm[igap], ntsm[igap]>1?"s":"");
+		  igap+1, rangetsm[igap][0], rangetsm[igap][1], ntsm[igap], ntsm[igap]!=1?"s":"");
 	  if ( ntsm[igap] >= MAX_NTSM )
 	    {
 	      ntsm[igap] = MAX_NTSM;
@@ -495,5 +488,5 @@ void *Tinfo(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
