@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,72 +30,62 @@
 
 void *Trend(void *argument)
 {
-  int gridsize;
   int vdate = 0, vtime = 0;
-  int nrecs, nrecords;
-  int gridID, varID, levelID, recID;
-  int tsID;
-  int i, w;
-  int streamID1, streamID2, streamID3;
-  int vlistID1, vlistID2, taxisID1, taxisID2;
+  int varID, levelID;
   int nmiss;
-  int nvars;
-  int *recVarID, *recLevelID;
-  int nwork = 5;
-  double zj;
+  int nrecs;
   double temp1, temp2;
-  double missval, missval1, missval2;
-  field_t **work[5];
-  field_t field1, field2;
+  enum {nwork = 5};
+  field_type **work[5];
 
   cdoInitialize(argument);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
   vlistDefNtsteps(vlistID2, 1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  nvars    = vlistNvars(vlistID1);
-  nrecords = vlistNrecs(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
   for ( varID = 0; varID < nvars; varID++ )
-    vlistDefVarDatatype(vlistID2, varID, DATATYPE_FLT64);
+    vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT64);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
-  streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
   streamDefVlist(streamID3, vlistID2);
 
-  recVarID   = malloc(nrecords*sizeof(int));
-  recLevelID = malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
 
+  field_type field1, field2;
   field_init(&field1);
   field_init(&field2);
+  field1.ptr = (double*) Malloc(gridsize*sizeof(double));
+  field2.ptr = (double*) Malloc(gridsize*sizeof(double));
 
-  field1.ptr = malloc(gridsize*sizeof(double));
-  field2.ptr = malloc(gridsize*sizeof(double));
-
-  for ( w = 0; w < nwork; w++ )
+  for ( int w = 0; w < nwork; w++ )
     work[w] = field_calloc(vlistID1, FIELD_PTR);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       vdate = taxisInqVdate(taxisID1);
       vtime = taxisInqVtime(taxisID1);
 
-      zj = tsID;
+      double zj = (double)tsID;
       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -107,11 +97,11 @@ void *Trend(void *argument)
 
 	  streamReadRecord(streamID1, field1.ptr, &nmiss);
 
-	  missval  = vlistInqVarMissval(vlistID1, varID);
-	  gridID   = vlistInqVarGrid(vlistID1, varID);
-	  gridsize = gridInqSize(gridID);
+	  double missval  = vlistInqVarMissval(vlistID1, varID);
+	  int gridID   = vlistInqVarGrid(vlistID1, varID);
+	  int gridsize = gridInqSize(gridID);
 
-	  for ( i = 0; i < gridsize; i++ )
+	  for ( int i = 0; i < gridsize; i++ )
 	    if ( !DBL_IS_EQUAL(field1.ptr[i], missval) )
 	      {
 		work[0][varID][levelID].ptr[i] += zj;
@@ -131,39 +121,39 @@ void *Trend(void *argument)
   streamDefTimestep(streamID2, 0);
   streamDefTimestep(streamID3, 0);
 
-  for ( recID = 0; recID < nrecords; recID++ )
+  for ( int recID = 0; recID < nrecords; recID++ )
     {
       varID   = recVarID[recID];
       levelID = recLevelID[recID];
 
-      missval  = vlistInqVarMissval(vlistID1, varID);
-      gridID   = vlistInqVarGrid(vlistID1, varID);
-      gridsize = gridInqSize(gridID);
+      double missval  = vlistInqVarMissval(vlistID1, varID);
+      int gridID   = vlistInqVarGrid(vlistID1, varID);
+      int gridsize = gridInqSize(gridID);
 
-      missval1  = missval;
-      missval2  = missval;
+      double missval1  = missval;
+      double missval2  = missval;
 
-      for ( i = 0; i < gridsize; i++ )
+      for ( int i = 0; i < gridsize; i++ )
 	{
-	  temp1 = SUB(work[2][varID][levelID].ptr[i],
-		      DIV(MUL(work[0][varID][levelID].ptr[i], work[3][varID][levelID].ptr[i]), work[4][varID][levelID].ptr[i]));
-	  temp2 = SUB(work[1][varID][levelID].ptr[i],
-		      DIV(MUL(work[0][varID][levelID].ptr[i], work[0][varID][levelID].ptr[i]), work[4][varID][levelID].ptr[i]));
+	  temp1 = SUBMN(work[2][varID][levelID].ptr[i],
+		      DIVMN( MULMN(work[0][varID][levelID].ptr[i], work[3][varID][levelID].ptr[i]), work[4][varID][levelID].ptr[i]));
+	  temp2 = SUBMN(work[1][varID][levelID].ptr[i],
+		      DIVMN( MULMN(work[0][varID][levelID].ptr[i], work[0][varID][levelID].ptr[i]), work[4][varID][levelID].ptr[i]));
 
-	  field2.ptr[i] = DIV(temp1, temp2);
-	  field1.ptr[i] = SUB(DIV(work[3][varID][levelID].ptr[i], work[4][varID][levelID].ptr[i]),
-			      MUL(DIV(work[0][varID][levelID].ptr[i], work[4][varID][levelID].ptr[i]), field2.ptr[i]));
+	  field2.ptr[i] = DIVMN(temp1, temp2);
+	  field1.ptr[i] = SUBMN( DIVMN(work[3][varID][levelID].ptr[i], work[4][varID][levelID].ptr[i]),
+			      MULMN( DIVMN(work[0][varID][levelID].ptr[i], work[4][varID][levelID].ptr[i]), field2.ptr[i]));
 	}
 
       nmiss = 0;
-      for ( i = 0; i < gridsize; i++ )
+      for ( int i = 0; i < gridsize; i++ )
 	if ( DBL_IS_EQUAL(field1.ptr[i], missval) ) nmiss++;
 
       streamDefRecord(streamID2, varID, levelID);
       streamWriteRecord(streamID2, field1.ptr, nmiss);
 
       nmiss = 0;
-      for ( i = 0; i < gridsize; i++ )
+      for ( int i = 0; i < gridsize; i++ )
 	if ( DBL_IS_EQUAL(field2.ptr[i], missval) ) nmiss++;
 
       streamDefRecord(streamID3, varID, levelID);
@@ -171,13 +161,13 @@ void *Trend(void *argument)
     }
 
 
-  for ( w = 0; w < nwork; w++ ) field_free(work[w], vlistID1);
+  for ( int w = 0; w < nwork; w++ ) field_free(work[w], vlistID1);
 
-  if ( field1.ptr ) free(field1.ptr);
-  if ( field2.ptr ) free(field2.ptr);
+  if ( field1.ptr ) Free(field1.ptr);
+  if ( field2.ptr ) Free(field2.ptr);
 
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
 
   streamClose(streamID3);
   streamClose(streamID2);
@@ -185,5 +175,5 @@ void *Trend(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
