@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -36,7 +36,7 @@ double tstepcount(long nts, double missval1, double *array1, double refval)
   long j;
   long n = 0;
 
-  if ( DBL_IS_EQUAL(refval, missval1) ) return (missval1);
+  if ( DBL_IS_EQUAL(refval, missval1) ) return missval1;
 
   for ( j = 0; j < nts; j++ )
     {
@@ -45,9 +45,9 @@ double tstepcount(long nts, double missval1, double *array1, double refval)
     }
 
   if ( j == nts )
-    return (missval1);
+    return missval1;
   else
-    return ((double) n);
+    return (double) n;
 }
 
 
@@ -55,58 +55,51 @@ void *Tstepcount(void *argument)
 {
   int gridsize;
   int nrecs;
-  int gridID, varID, levelID, recID;
-  int tsID;
-  int i;
-  int nts;
+  int gridID, varID, levelID;
   int nalloc = 0;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2, taxisID1, taxisID2;
   int nmiss;
-  int nvars, nlevel;
+  int nlevel;
   int vdate = 0, vtime = 0;
   double missval;
   double refval = 0;
-  double count;
-  field_t ***vars = NULL;
+  field_type ***vars = NULL;
   typedef struct
   {
     double *array1;
   } memory_t;
-  memory_t *mem = NULL;
 
   cdoInitialize(argument);
 
-  if ( operatorArgc() == 1 ) refval = atof(operatorArgv()[0]);
+  if ( operatorArgc() == 1 ) refval = parameter2double(operatorArgv()[0]);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
   vlistDefNtsteps(vlistID2, 1);
 
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
   for ( varID = 0; varID < nvars; varID++ )
     {
       vlistDefVarUnits(vlistID2, varID, "steps");
     }
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       if ( tsID >= nalloc )
 	{
 	  nalloc += NALLOC_INC;
-	  vars  = (field_t ***) realloc(vars, nalloc*sizeof(field_t **));
+	  vars  = (field_type ***) Realloc(vars, nalloc*sizeof(field_type **));
 	}
 
       vdate = taxisInqVdate(taxisID1);
@@ -114,12 +107,12 @@ void *Tstepcount(void *argument)
 
       vars[tsID] = field_malloc(vlistID1, FIELD_NONE);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridID   = vlistInqVarGrid(vlistID1, varID);
 	  gridsize = gridInqSize(gridID);
-	  vars[tsID][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
+	  vars[tsID][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 	  streamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
 	  vars[tsID][varID][levelID].nmiss = nmiss;
 	}
@@ -127,12 +120,12 @@ void *Tstepcount(void *argument)
       tsID++;
     }
 
-  nts = tsID;
+  int nts = tsID;
 
-  mem = (memory_t*) malloc(ompNumThreads*sizeof(memory_t));
-  for ( i = 0; i < ompNumThreads; i++ )
+  memory_t *mem = (memory_t*) Malloc(ompNumThreads*sizeof(memory_t));
+  for ( int i = 0; i < ompNumThreads; i++ )
     {
-      mem[i].array1 = (double*) malloc(nts*sizeof(double));
+      mem[i].array1 = (double*) Malloc(nts*sizeof(double));
     }
 
   for ( varID = 0; varID < nvars; varID++ )
@@ -144,27 +137,24 @@ void *Tstepcount(void *argument)
       for ( levelID = 0; levelID < nlevel; levelID++ )
 	{
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, tsID) \
-  schedule(dynamic,1)
+#pragma omp parallel for default(none) shared(gridsize,mem,vars,varID,levelID,nts,missval,refval) schedule(dynamic,1)
 #endif
-	  for ( i = 0; i < gridsize; i++ )
+	  for ( int i = 0; i < gridsize; i++ )
 	    {
 	      int ompthID = cdo_omp_get_thread_num();
 
-	      for ( tsID = 0; tsID < nts; tsID++ )
+	      for ( int tsID = 0; tsID < nts; tsID++ )
 		mem[ompthID].array1[tsID] = vars[tsID][varID][levelID].ptr[i];
 
-	      count = tstepcount(nts, missval, mem[ompthID].array1, refval);
+	      double count = tstepcount(nts, missval, mem[ompthID].array1, refval);
 	      vars[0][varID][levelID].ptr[i] = count;
 	    }
 	}
     }
 
-  for ( i = 0; i < ompNumThreads; i++ )
-    {
-      free(mem[i].array1);
-    }
-  free(mem);
+  for ( int i = 0; i < ompNumThreads; i++ )
+    Free(mem[i].array1);
+  Free(mem);
 
   taxisDefVdate(taxisID2, vdate);
   taxisDefVtime(taxisID2, vtime);
@@ -182,7 +172,7 @@ void *Tstepcount(void *argument)
 	  streamDefRecord(streamID2, varID, levelID);
 
 	  nmiss = 0;
-	  for ( i = 0; i < gridsize; i++ )
+	  for ( int i = 0; i < gridsize; i++ )
 	    if ( DBL_IS_EQUAL(vars[0][varID][levelID].ptr[i], missval) ) nmiss++;
 
 	  streamWriteRecord(streamID2, vars[0][varID][levelID].ptr, nmiss);
@@ -191,12 +181,12 @@ void *Tstepcount(void *argument)
 
   for ( tsID = 0; tsID < nts; tsID++ ) field_free(vars[tsID], vlistID1);
 
-  if ( vars  ) free(vars);
+  if ( vars ) Free(vars);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
