@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,7 @@
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
-#include "list.h"
+#include "listarray.h"
 #include "grid.h"
 #include "constants.h"
 #include "stdnametable.h"
@@ -87,10 +87,54 @@ double std_atm_pressure(double height)
   return (P_ZERO * exp((-1)*TMP4PRESSURE*log((exp(height/SCALEHEIGHT)*T_ZERO + T_DELTA)/(T_ZERO + T_DELTA))));
 }
 
+static
+void conv_generic_grid(int gridID, int gridsize, double *xvals2D, double *yvals2D)
+{
+  int xsize = gridInqXsize(gridID);
+  int ysize = gridInqYsize(gridID);
+
+  assert(gridsize==xsize*ysize);
+
+  double *xcoord = (double*) Malloc(xsize*sizeof(double));
+  double *ycoord = (double*) Malloc(ysize*sizeof(double));
+
+  gridInqXvals(gridID, xcoord);
+  gridInqYvals(gridID, ycoord);
+
+  double xmin = xcoord[0];
+  double xmax = xcoord[0];
+  for ( int i = 1; i < xsize; ++i )
+    {
+      if ( xcoord[i] < xmin ) xmin = xcoord[i];
+      if ( xcoord[i] > xmax ) xmax = xcoord[i];
+    }
+
+  double ymin = ycoord[0];
+  double ymax = ycoord[0];
+  for ( int i = 1; i < ysize; ++i )
+    {
+      if ( ycoord[i] < ymin ) ymin = ycoord[i];
+      if ( ycoord[i] > ymax ) ymax = ycoord[i];
+    }
+
+  double xrange = xmax - xmin;
+  double yrange = ymax - ymin;
+  
+  for ( int j = 0; j < ysize; ++j )
+    for ( int i = 0; i < xsize; ++i )
+      {
+        xvals2D[j*xsize+i] = xcoord[i]*M_PI/xrange; 
+        yvals2D[j*xsize+i] = ycoord[j]*M_PI/yrange; 
+      }
+  
+  Free(xcoord);
+  Free(ycoord);
+}
+
 static
 void remap_nn_reg2d_reg2d(int nx, int ny, const double *restrict data, int gridID, double *restrict array)
 {
-  if ( !(gridInqType(gridID) == GRID_LONLAT && !gridIsRotated(gridID)) )
+  if ( gridInqType(gridID) != GRID_LONLAT )
     cdoAbort("Internal error, wrong grid type!");
 
   int nxvals = gridInqXsize(gridID);
@@ -177,26 +221,26 @@ void remap_nn_reg2d_nonreg2d(int nx, int ny, const double *restrict data, int gr
 static
 void remap_nn_reg2d(int nx, int ny, const double *restrict data, int gridID, double *restrict array)
 {
-  if ( gridInqType(gridID) == GRID_LONLAT && !gridIsRotated(gridID) )
+  if ( gridInqType(gridID) == GRID_LONLAT )
     remap_nn_reg2d_reg2d(nx, ny, data, gridID, array);
   else
     remap_nn_reg2d_nonreg2d(nx, ny, data, gridID, array);
 }
 
+#define NLON 720
+#define NLAT 360
 
 void *Vargen(void *argument)
 {
   int ntimesteps, nlevels = 1;
   int varID, varID2 = -1, levelID;
-  int i;
   int gridID = -1, gridIDdata = -1, zaxisID;
-  int vdate, vtime;
-  double rval, rstart = 0, rstop = 0, rinc = 0;
+  double rstart = 0, rstop = 0, rinc = 0;
   double rconst = 0;
   double *levels = NULL;
-  double lon[720], lat[360];
-  int nlon = 720;
-  int nlat = 360;
+  double lon[NLON], lat[NLAT];
+  int nlon = NLON;
+  int nlat = NLAT;
 
   cdoInitialize(argument);
 
@@ -280,14 +324,14 @@ void *Vargen(void *argument)
   else if ( operatorID == STDATM )
     {
       double lon = 0, lat = 0;
-      LIST *flist = listNew(FLT_LIST);
+      lista_t *flista = lista_new(FLT_LISTA);
 
       operatorInputArg(cdoOperatorEnter(operatorID));
-      nlevels = args2fltlist(operatorArgc(), operatorArgv(), flist);
-      levels  = (double *) listArrayPtr(flist);
-      //listDelete(flist);
+      nlevels = args2flt_lista(operatorArgc(), operatorArgv(), flista);
+      levels  = (double *) lista_dataptr(flista);
+      //lista_destroy(flista);
 
-      if ( cdoVerbose ) for ( i = 0; i < nlevels; ++i ) printf("levels %d: %g\n", i, levels[i]);
+      if ( cdoVerbose ) for ( int i = 0; i < nlevels; ++i ) printf("levels %d: %g\n", i, levels[i]);
 
       gridID = gridCreate(GRID_LONLAT, 1);
       gridDefXsize(gridID, 1);
@@ -325,7 +369,7 @@ void *Vargen(void *argument)
     varID2 = vlistDefVar(vlistID, gridID, zaxisID, TSTEP_CONSTANT);
 
   if ( operatorID == MASK )
-    vlistDefVarDatatype(vlistID, varID, DATATYPE_INT8);
+    vlistDefVarDatatype(vlistID, varID, CDI_DATATYPE_INT8);
 
   if ( operatorID == STDATM )
     {
@@ -383,9 +427,9 @@ void *Vargen(void *argument)
 
   for ( int tsID = 0; tsID < ntimesteps; tsID++ )
     {
-      rval  = rstart + rinc*tsID;
-      vdate = julday_to_date(CALENDAR_PROLEPTIC, julday + tsID);
-      vtime = 0;
+      double rval  = rstart + rinc*tsID;
+      int vdate = julday_to_date(CALENDAR_PROLEPTIC, julday + tsID);
+      int vtime = 0;
       taxisDefVdate(taxisID, vdate);
       taxisDefVtime(taxisID, vtime);
       streamDefTimestep(streamID, tsID);
@@ -399,7 +443,7 @@ void *Vargen(void *argument)
 
               if ( operatorID == RANDOM )
                 {
-                  for ( i = 0; i < gridsize; i++ )
+                  for ( int i = 0; i < gridsize; i++ )
                     array[i] = ((double)rand())/((double)RAND_MAX);
                 }
               else if ( operatorID == SINCOS || operatorID == COSHILL )
@@ -407,29 +451,36 @@ void *Vargen(void *argument)
 		  double *xvals = (double*) Malloc(gridsize*sizeof(double));
 		  double *yvals = (double*) Malloc(gridsize*sizeof(double));
 
-		  if ( gridInqType(gridID) == GRID_GME ) gridID = gridToUnstructured(gridID, 0);
+                  if ( grid_is_distance_generic(gridID) )
+                    {
+                      conv_generic_grid(gridID, gridsize, xvals, yvals);
+                    }
+                  else
+                    {
+                      if ( gridInqType(gridID) == GRID_GME ) gridID = gridToUnstructured(gridID, 0);
 
-		  if ( gridInqType(gridID) != GRID_UNSTRUCTURED && gridInqType(gridID) != GRID_CURVILINEAR )
-		    gridID = gridToCurvilinear(gridID, 0);
+                      if ( gridInqType(gridID) != GRID_UNSTRUCTURED && gridInqType(gridID) != GRID_CURVILINEAR )
+                        gridID = gridToCurvilinear(gridID, 0);
 
-		  gridInqXvals(gridID, xvals);
-		  gridInqYvals(gridID, yvals);
-
-		  /* Convert lat/lon units if required */
-		  char units[CDI_MAX_NAME];
-		  gridInqXunits(gridID, units);
-		  grid_to_radian(units, gridsize, xvals, "grid center lon");
-		  gridInqYunits(gridID, units);
-		  grid_to_radian(units, gridsize, yvals, "grid center lat");
+                      gridInqXvals(gridID, xvals);
+                      gridInqYvals(gridID, yvals);
 
+                      /* Convert lat/lon units if required */
+                      char units[CDI_MAX_NAME];
+                      gridInqXunits(gridID, units);
+                      grid_to_radian(units, gridsize, xvals, "grid center lon");
+                      gridInqYunits(gridID, units);
+                      grid_to_radian(units, gridsize, yvals, "grid center lat");
+                    }
+                  
 		  if ( operatorID == SINCOS )
 		    {
-		      for ( i = 0; i < gridsize; i++ )
+		      for ( int i = 0; i < gridsize; i++ )
 			array[i] = cos(1.0 * xvals[i]) * sin(2.0 * yvals[i]);
 		    }
 		  else if ( operatorID == COSHILL )
 		    {		     
-		      for ( i = 0; i < gridsize; i++ )
+		      for ( int i = 0; i < gridsize; i++ )
 			array[i] = 2 - cos(acos(cos(xvals[i]) * cos(yvals[i]))/1.2);
 		    }
 
@@ -438,13 +489,13 @@ void *Vargen(void *argument)
 		}
               else if ( operatorID == CONST )
                 {
-                  for ( i = 0; i < gridsize; i++ )
+                  for ( int i = 0; i < gridsize; i++ )
                     array[i] = rconst;
                 }
               else if ( operatorID == TOPO )
                 {
 #if defined(ENABLE_DATA)
-                  for ( i = 0; i < datasize; i++ )
+                  for ( int i = 0; i < datasize; i++ )
                     data[i] = etopo[i]/etopo_scale - etopo_offset;
 #else
                   cdoAbort("Operator support disabled!");
@@ -453,7 +504,7 @@ void *Vargen(void *argument)
               else if ( operatorID == TEMP )
                 {
 #if defined(ENABLE_DATA)
-                  for ( i = 0; i < datasize; i++ )
+                  for ( int i = 0; i < datasize; i++ )
                     data[i] = temp[i]/temp_scale - temp_offset;
 #else
                   cdoAbort("Operator support disabled!");
@@ -462,7 +513,7 @@ void *Vargen(void *argument)
               else if ( operatorID == MASK )
                 {
 #if defined(ENABLE_DATA)
-                  for ( i = 0; i < datasize; i++ )
+                  for ( int i = 0; i < datasize; i++ )
                     data[i] = mask[i]/mask_scale - mask_offset;
 #else
                   cdoAbort("Operator support disabled!");
