@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,20 +22,18 @@
 */
 
 
-#include <ctype.h>
-
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
 #include "after_vertint.h"
-#include "list.h"
+#include "listarray.h"
 #include "stdnametable.h"
 
 static
-int is_height_axis(int zaxisID, int nlevel)
+bool is_height_axis(int zaxisID, int nlevel)
 {
-  int isheight = FALSE;
+  bool isheight = false;
   if ( nlevel > 1 )
     {
       if ( zaxisInqType(zaxisID) == ZAXIS_REFERENCE )
@@ -45,12 +43,12 @@ int is_height_axis(int zaxisID, int nlevel)
           zaxisInqUnits(zaxisID, units);
           zaxisInqStdname(zaxisID, stdname);
           if ( strcmp(stdname, "height") == 0 && *units == 0 )
-            isheight = TRUE;
+            isheight = true;
         }
     }
   return isheight;
 }
-
+/*
 static
 int is_hybrid_axis(int zaxisID, int nlevel)
 {
@@ -59,41 +57,38 @@ int is_hybrid_axis(int zaxisID, int nlevel)
        nlevel > 1 ) ishybrid = TRUE;
   return ishybrid;
 }
-
+*/
 
 void *Vertintap(void *argument)
 {
   enum {func_pl, func_hl};
   enum {type_lin, type_log};
-  int recID, nrecs;
+  int nrecs;
   int i, k, offset;
   int varID, levelID;
-  int zaxisIDp, zaxisIDh = -1, nzaxis;
-  int gridID, zaxisID;
+  int zaxisIDp, zaxisIDh = -1;
   int nhlev = 0, nhlevf = 0, nhlevh = 0, nlevel;
   int *vert_index = NULL;
-  int nvct;
   int apressID = -1, dpressID = -1;
   int psID = -1, tempID = -1;
-  int param;
   //int sortlevels = TRUE;
   int *pnmiss = NULL;
   char paramstr[32];
   char stdname[CDI_MAX_NAME];
-  double minval, maxval;
-  double missval;
+  char varname[CDI_MAX_NAME];
   double *vct = NULL;
   double *single1, *single2;
   double *ps_prog = NULL, *full_press = NULL, *dpress = NULL;
   double *hyb_press = NULL;
   int Extrapolate = 0;
-  int mono_level;
-  LIST *flist = listNew(FLT_LIST);
+  lista_t *flista = lista_new(FLT_LISTA);
 
   cdoInitialize(argument);
 
   int AP2PL     = cdoOperatorAdd("ap2pl",     func_pl, type_lin, "pressure levels in pascal");
   int AP2PLX    = cdoOperatorAdd("ap2plx",    func_pl, type_lin, "pressure levels in pascal");
+  int AP2HL     = cdoOperatorAdd("ap2hl",     func_hl, type_lin, "height levels in meter");
+  int AP2HLX    = cdoOperatorAdd("ap2hlx",    func_hl, type_lin, "height levels in meter");
   int AP2PL_LP  = cdoOperatorAdd("ap2pl_lp",  func_pl, type_log, "pressure levels in pascal");
   int AP2PLX_LP = cdoOperatorAdd("ap2plx_lp", func_pl, type_log, "pressure levels in pascal");
 
@@ -101,7 +96,7 @@ void *Vertintap(void *argument)
   int operfunc   = cdoOperatorF1(operatorID);
   int opertype   = cdoOperatorF2(operatorID);
 
-  if ( operatorID == AP2PL || operatorID == AP2PL_LP )
+  if ( operatorID == AP2PL || operatorID == AP2HL || operatorID == AP2PL_LP )
     {
       char *envstr = getenv("EXTRAPOLATE");
 
@@ -112,7 +107,7 @@ void *Vertintap(void *argument)
             cdoPrint("Extrapolation of missing values enabled!");
 	}
     }
-  else if ( operatorID == AP2PLX || operatorID == AP2PLX_LP )
+  else if ( operatorID == AP2PLX ||  operatorID == AP2HLX || operatorID == AP2PLX_LP )
     {
       Extrapolate = 1;
     }
@@ -123,20 +118,30 @@ void *Vertintap(void *argument)
   double *plev = NULL;
   if ( operatorArgc() == 1 && strcmp(operatorArgv()[0], "default") == 0 )
     {
-      /*
-      double stdlev[] = {100000, 92500, 85000, 77500, 70000, 60000, 50000, 40000, 30000, 25000, 20000,
-                          15000, 10000, 7000, 5000, 3000, 2000, 1000, 700, 500, 300, 200, 100, 50, 20, 10};
-      */
-      double stdlev[] = {100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 20000, 15000,
-                          10000,  7000,  5000,  3000,  2000, 1000 };
-      nplev = sizeof(stdlev)/sizeof(*stdlev);
-      plev  = (double *) Malloc(nplev*sizeof(double));
-      for ( i = 0; i < nplev; ++i ) plev[i] = stdlev[i];
+      if ( operfunc == func_hl )
+        {
+          double stdlev[] = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
+          nplev = sizeof(stdlev)/sizeof(*stdlev);
+          plev  = (double *) Malloc(nplev*sizeof(double));
+          for ( i = 0; i < nplev; ++i ) plev[i] = stdlev[i];
+        }
+      else
+        {
+          /*
+            double stdlev[] = {100000, 92500, 85000, 77500, 70000, 60000, 50000, 40000, 30000, 25000, 20000,
+            15000, 10000, 7000, 5000, 3000, 2000, 1000, 700, 500, 300, 200, 100, 50, 20, 10};
+          */
+          double stdlev[] = {100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 20000, 15000,
+                             10000,  7000,  5000,  3000,  2000, 1000 };
+          nplev = sizeof(stdlev)/sizeof(*stdlev);
+          plev  = (double *) Malloc(nplev*sizeof(double));
+          for ( i = 0; i < nplev; ++i ) plev[i] = stdlev[i];
+        }
     }
   else
     {
-      nplev = args2fltlist(operatorArgc(), operatorArgv(), flist);
-      plev  = (double *) listArrayPtr(flist);
+      nplev = args2flt_lista(operatorArgc(), operatorArgv(), flista);
+      plev  = (double *) lista_dataptr(flista);
     }
 
   int streamID1 = streamOpenRead(cdoStreamName(0));
@@ -156,119 +161,71 @@ void *Vertintap(void *argument)
     zaxisIDp = zaxisCreate(ZAXIS_PRESSURE, nplev);
 
   zaxisDefLevels(zaxisIDp, plev);
-  nzaxis  = vlistNzaxis(vlistID1);
 
-  int lheight = FALSE;
-  for ( i = 0; i < nzaxis; i++ )
-    {
-      mono_level = TRUE;
-      zaxisID = vlistZaxis(vlistID1, i);
-      nlevel  = zaxisInqSize(zaxisID);
-
-      if ( is_height_axis(zaxisID, nlevel) )
-        {
-	  double *level = (double *) Malloc(nlevel*sizeof(double));
-	  zaxisInqLevels(zaxisID, level);
-	  int l;
-	  for ( l = 0; l < nlevel; l++ )
-	    {
-	      if ( (l+1) != (int) (level[l]+0.5) ) break;
-	    }
-	  if ( l == nlevel ) mono_level = TRUE; 
-	  Free(level);          
-        }
-      
-      if ( is_height_axis(zaxisID, nlevel) && mono_level )
-        {
-          lheight  = TRUE;
-          zaxisIDh = zaxisID;
-          nhlev    = nlevel;
-          nhlevf   = nhlev;
-          nhlevh   = nhlevf + 1;
-
-          vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-          break;
-        }
-    }
-
-  if ( ! lheight ) { // old version, not needed anymore!!
-  int lhavevct = FALSE;
-  for ( i = 0; i < nzaxis; i++ )
-    {
-      /* mono_level = FALSE; */
-      mono_level = TRUE;
-      zaxisID = vlistZaxis(vlistID1, i);
-      nlevel  = zaxisInqSize(zaxisID);
-
-      if ( is_hybrid_axis(zaxisID, nlevel) )
-	{
-	  double *level = (double *) Malloc(nlevel*sizeof(double));
-	  zaxisInqLevels(zaxisID, level);
-	  int l;
-	  for ( l = 0; l < nlevel; l++ )
-	    {
-	      if ( (l+1) != (int) (level[l]+0.5) ) break;
-	    }
-	  if ( l == nlevel ) mono_level = TRUE; 
-	  Free(level);
-	}
-
-      if ( is_hybrid_axis(zaxisID, nlevel) && mono_level )
-	{
-	  nvct = zaxisInqVctSize(zaxisID);
-	  if ( nlevel == (nvct/2 - 1) )
-	    {
-	      if ( lhavevct == FALSE )
-		{
-		  lhavevct = TRUE;
-		  zaxisIDh = zaxisID;
-		  nhlev    = nlevel;
-		  nhlevf   = nhlev;
-		  nhlevh   = nhlevf + 1;
-	      
-		  vct = (double*) Malloc(nvct*sizeof(double));
-		  zaxisInqVct(zaxisID, vct);
-
-		  vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	      else
-		{
-		  if ( memcmp(vct, zaxisInqVctPtr(zaxisID), nvct*sizeof(double)) == 0 )
-		    vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	    }
-	  else if ( nlevel == (nvct/2) )
-	    {
-	      if ( lhavevct == FALSE )
-		{
-		  lhavevct = TRUE;
-		  zaxisIDh = zaxisID;
-		  nhlev    = nlevel;
-		  nhlevf   = nhlev - 1;
-		  nhlevh   = nhlev;
-	      
-		  vct = (double*) Malloc(nvct*sizeof(double));
-		  zaxisInqVct(zaxisID, vct);
-
-		  vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	      else
-		{
-		  if ( memcmp(vct, zaxisInqVctPtr(zaxisID), nvct*sizeof(double)) == 0 )
-		    vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	    }
-	}
-    }
-  }
-  
   int nvars = vlistNvars(vlistID1);
 
-  int vars[nvars];
+  for ( varID = 0; varID < nvars; varID++ )
+    {
+      vlistInqVarStdname(vlistID1, varID, stdname);
+      strtolower(stdname);
+
+      if      ( strcmp(stdname, var_stdname(surface_air_pressure)) == 0 ) psID = varID; 
+      else if ( strcmp(stdname, var_stdname(air_pressure))         == 0 ) apressID = varID; 
+      else if ( strcmp(stdname, var_stdname(pressure_thickness))   == 0 ) dpressID = varID; 
+      else if ( strcmp(stdname, var_stdname(air_temperature))      == 0 ) tempID = varID; 
+    }
+
+  if ( cdoVerbose )
+    {
+      cdoPrint("Found:");
+      if ( psID      != -1 ) cdoPrint("  %s", var_stdname(surface_air_pressure));
+      if ( apressID  != -1 ) cdoPrint("  %s", var_stdname(air_pressure));
+      if ( dpressID  != -1 ) cdoPrint("  %s", var_stdname(pressure_thickness));
+      if ( tempID    != -1 ) cdoPrint("  %s", var_stdname(air_temperature));
+    }
+
+  if ( apressID == -1 ) cdoAbort("%s not found!", var_stdname(air_pressure));
+
+  int nzaxis = vlistNzaxis(vlistID1);
+  for ( int i = 0; i < nzaxis; i++ )
+    {
+      int zaxisID = vlistZaxis(vlistID1, i);
+      if ( zaxisID == vlistInqVarZaxis(vlistID1, apressID) )
+        {
+          bool mono_level = true;
+          nlevel  = zaxisInqSize(zaxisID);
+
+          if ( is_height_axis(zaxisID, nlevel) )
+            {
+              double *level = (double *) Malloc(nlevel*sizeof(double));
+              cdoZaxisInqLevels(zaxisID, level);
+              int l;
+              for ( l = 0; l < nlevel; l++ )
+                {
+                  if ( (l+1) != (int) (level[l]+0.5) ) break;
+                }
+              if ( l == nlevel ) mono_level = true; 
+              Free(level);          
+            }
+      
+          if ( is_height_axis(zaxisID, nlevel) && mono_level )
+            {
+              zaxisIDh = zaxisID;
+              nhlev    = nlevel;
+              nhlevf   = nhlev;
+              nhlevh   = nhlevf + 1;
+
+              vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
+              break;
+            }
+        }
+    }
+  
+  bool vars[nvars];
   double *vardata1[nvars];
   double *vardata2[nvars];
   int *varnmiss[nvars];
-  int varinterp[nvars];
+  bool varinterp[nvars];
 
   int maxlev = nhlevh > nplev ? nhlevh : nplev;
 
@@ -309,7 +266,7 @@ void *Vertintap(void *argument)
       height2pressure(phlev, plev, nplev);
 
       if ( cdoVerbose )
-	for ( i = 0; i < nplev; ++i )
+	for ( int i = 0; i < nplev; ++i )
 	  cdoPrint("level = %d   height = %g   pressure = %g", i+1, plev[i], phlev[i]);
 
       memcpy(plev, phlev, nplev*sizeof(double));
@@ -320,54 +277,42 @@ void *Vertintap(void *argument)
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      gridID   = vlistInqVarGrid(vlistID1, varID);
-      zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-      nlevel   = zaxisInqSize(zaxisID);
-
-      vlistInqVarStdname(vlistID1, varID, stdname);
-      strtolower(stdname);
-
-      if      ( strcmp(stdname, var_stdname(surface_air_pressure)) == 0 ) psID = varID; 
-      else if ( strcmp(stdname, var_stdname(air_pressure))         == 0 ) apressID = varID; 
-      else if ( strcmp(stdname, var_stdname(pressure_thickness))   == 0 ) dpressID = varID; 
-      else if ( strcmp(stdname, var_stdname(air_temperature))      == 0 ) tempID = varID; 
+      int gridID   = vlistInqVarGrid(vlistID1, varID);
+      int zaxisID  = vlistInqVarZaxis(vlistID1, varID);
+      int nlevel   = zaxisInqSize(zaxisID);
 
       if ( gridInqType(gridID) == GRID_SPECTRAL )
 	cdoAbort("Spectral data unsupported!");
 
       vardata1[varID] = (double *) Malloc(gridsize*nlevel*sizeof(double));
-
+           
       if ( zaxisID == zaxisIDh )
 	{
-	  varinterp[varID] = TRUE;
+	  varinterp[varID] = true;
 	  vardata2[varID]  = (double *) Malloc(gridsize*nplev*sizeof(double));
 	  varnmiss[varID]  = (int *) Malloc(maxlev*sizeof(int));
 	  memset(varnmiss[varID], 0, maxlev*sizeof(int));
 	}
       else
 	{
-	  varinterp[varID] = FALSE;
+          vlistInqVarName(vlistID1, varID, varname);
+
+          if ( is_height_axis(zaxisID, nlevel) && nlevel == nhlevh )
+            cdoWarning("Interpolation from half level not supported, parameter %s not interpolated!", varname);
+          
+	  varinterp[varID] = false;
 	  vardata2[varID]  = vardata1[varID];
 	  varnmiss[varID]  = (int *) Malloc(nlevel*sizeof(int));
 	}
     }
-
-  if ( cdoVerbose )
-    {
-      cdoPrint("Found:");
-      if ( psID      != -1 ) cdoPrint("  %s", var_stdname(surface_air_pressure));
-      if ( apressID  != -1 ) cdoPrint("  %s", var_stdname(air_pressure));
-      if ( dpressID  != -1 ) cdoPrint("  %s", var_stdname(pressure_thickness));
-      if ( tempID    != -1 ) cdoPrint("  %s", var_stdname(air_temperature));
-    }
-
-  if ( apressID == -1 ) cdoAbort("%s not found!", var_stdname(air_pressure));
+  
   if ( zaxisIDh != -1 && psID == -1 && dpressID )
-    cdoWarning("Surface pressure not found - set to upper level of %s!", var_stdname(air_pressure));
+    cdoWarning("Surface pressure not found - set to vertical sum of %s!", var_stdname(pressure_thickness));
+  //  cdoWarning("Surface pressure not found - set to upper level of %s!", var_stdname(air_pressure));
 
   for ( varID = 0; varID < nvars; ++varID )
     {
-      if ( varinterp[varID] == TRUE && vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT )
+      if ( varinterp[varID] && vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT )
 	vlistDefVarTsteptype(vlistID2, varID, TSTEP_INSTANT);
     }
 
@@ -380,17 +325,16 @@ void *Vertintap(void *argument)
     {
       for ( varID = 0; varID < nvars; ++varID )
 	{
-          vars[varID] = FALSE;
+          vars[varID] = false;
 	  nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	  for ( levelID = 0; levelID < nlevel; levelID++ )
 	    varnmiss[varID][levelID] = 0;
 	}
 
       taxisCopyTimestep(taxisID2, taxisID1);
-
       streamDefTimestep(streamID2, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  /*
@@ -407,11 +351,11 @@ void *Vertintap(void *argument)
 
 	  streamReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
 
-	  vars[varID] = TRUE;
+	  vars[varID] = true;
 	}
 
       for ( varID = 0; varID < nvars; varID++ )
-	if ( varinterp[varID] == TRUE ) vars[varID] = TRUE;
+	if ( varinterp[varID]  ) vars[varID] = true;
 
       if ( zaxisIDh != -1 )
 	{
@@ -434,6 +378,7 @@ void *Vertintap(void *argument)
 	    }
 
 	  /* check range of ps_prog */
+          double minval, maxval;
 	  minmaxval(gridsize, ps_prog, NULL, &minval, &maxval);
 	  if ( minval < MIN_PS || maxval > MAX_PS )
 	    cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
@@ -459,9 +404,9 @@ void *Vertintap(void *argument)
 	{
 	  if ( vars[varID] )
 	    {
-	      zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-	      missval  = vlistInqVarMissval(vlistID1, varID);
-	      nlevel   = zaxisInqSize(zaxisID);
+	      int zaxisID  = vlistInqVarZaxis(vlistID1, varID);
+	      int nlevel   = zaxisInqSize(zaxisID);
+	      double missval  = vlistInqVarMissval(vlistID1, varID);
 	      if ( varinterp[varID] )
 		{
 		  if ( nlevel == nhlevf )
@@ -470,7 +415,7 @@ void *Vertintap(void *argument)
 		    }
 		  else
 		    {
-		      param = vlistInqVarParam(vlistID1, varID);
+		      int param = vlistInqVarParam(vlistID1, varID);
 		      cdiParamToString(param, paramstr, sizeof(paramstr));
 		      cdoAbort("Number of generalized height level differ from full/half level (param=%s)!", paramstr);
 		    }
@@ -494,7 +439,7 @@ void *Vertintap(void *argument)
 	{
 	  if ( vars[varID] )
 	    {
-	      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+	      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
 	      for ( levelID = 0; levelID < nlevel; levelID++ )
 		{
 		  offset   = gridsize*levelID;
@@ -525,7 +470,7 @@ void *Vertintap(void *argument)
   if ( dpress     ) Free(dpress);
   if ( vct        ) Free(vct);
 
-  listDelete(flist);
+  lista_destroy(flista);
 
   cdoFinish();
 
