@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,53 +23,60 @@
 */
 
 
-#include <ctype.h>
-
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
 #include "after_vertint.h"
-#include "list.h"
+#include "listarray.h"
 #include "stdnametable.h"
 #include "constants.h"
 
 
+static
+bool zaxis_is_hybrid(int zaxistype)
+{
+  return (zaxistype == ZAXIS_HYBRID || zaxistype == ZAXIS_HYBRID_HALF);
+}
+
+static
+void change_hybrid_zaxis(int vlistID1, int vlistID2, int nvct, double *vct, int zaxisID2)
+{
+  int nzaxis = vlistNzaxis(vlistID1);
+  for ( int iz = 0; iz < nzaxis; ++iz )
+    {
+      int zaxisID = vlistZaxis(vlistID1, iz);
+      int nlevel  = zaxisInqSize(zaxisID);
+      int zaxistype = zaxisInqType(zaxisID);
+
+      if ( zaxis_is_hybrid(zaxistype) && nlevel > 1 )
+	{
+	  int nvct2 = zaxisInqVctSize(zaxisID);
+          if ( nvct2 == nvct && memcmp(vct, zaxisInqVctPtr(zaxisID), nvct*sizeof(double)) == 0 )
+            vlistChangeZaxisIndex(vlistID2, iz, zaxisID2);
+	}
+    }
+}
+
+
 void *Vertintml(void *argument)
 {
   int mode;
   enum {ECHAM_MODE, WMO_MODE};
   enum {func_pl, func_hl};
   enum {type_lin, type_log};
-  int recID, nrecs;
-  int i, k, offset;
+  int nrecs;
   int varID, levelID;
-  int zaxisIDp, zaxisIDh = -1;
-  int gridID, zaxisID;
-  int nhlev = 0, nhlevf = 0, nhlevh = 0, nlevel;
-  int *vert_index = NULL;
-  int nvct = 0;
-  int sgeopot_needed = FALSE;
-  int sgeopotID = -1, geopotID = -1, tempID = -1, psID = -1, lnpsID = -1, presID = -1, gheightID = -1;
-  int code, param;
-  int pnum, pcat, pdis;
-  //int sortlevels = TRUE;
-  int *pnmiss = NULL;
+  bool sgeopot_needed = false;
+  bool extrapolate = false;
+  int sgeopotID = -1, geopotID = -1, tempID = -1, psID = -1, lnpsID = -1, gheightID = -1;
+  // bool sortlevels = true;
   char paramstr[32];
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
   double minval, maxval;
-  double missval;
-  double *vct = NULL;
-  double *rvct = NULL; /* reduced VCT for LM */
-  double *single1, *single2;
-  double *sgeopot = NULL, *ps_prog = NULL, *full_press = NULL, *half_press = NULL;
-  double *hyb_press = NULL;
-  int Extrapolate = 0;
-  int mono_level;
-  int instNum, tableNum;
-  int useTable;
+  double *sgeopot = NULL;
   gribcode_t gribcodes = {0};
-  LIST *flist = listNew(FLT_LIST);
+  lista_t *flista = lista_new(FLT_LISTA);
 
   cdoInitialize(argument);
 
@@ -89,17 +96,16 @@ void *Vertintml(void *argument)
   if ( operatorID == ML2PL || operatorID == ML2HL || operatorID == ML2PL_LP || operatorID == ML2HL_LP )
     {
       char *envstr = getenv("EXTRAPOLATE");
-
       if ( envstr && isdigit((int) envstr[0]) )
 	{
-          Extrapolate = atoi(envstr);
-          if ( Extrapolate == 1 )
+          if ( atoi(envstr) == 1 ) extrapolate = true;
+          if ( extrapolate )
             cdoPrint("Extrapolation of missing values enabled!");
 	}
     }
   else if ( operatorID == ML2PLX || operatorID == ML2HLX || operatorID == ML2PLX_LP || operatorID == ML2HLX_LP )
     {
-      Extrapolate = 1;
+      extrapolate = true;
     }
 
   operatorInputArg(cdoOperatorEnter(operatorID));
@@ -108,20 +114,30 @@ void *Vertintml(void *argument)
   double *plev = NULL;
   if ( operatorArgc() == 1 && strcmp(operatorArgv()[0], "default") == 0 )
     {
-      /*
-      double stdlev[] = {100000, 92500, 85000, 77500, 70000, 60000, 50000, 40000, 30000, 25000, 20000,
-                          15000, 10000, 7000, 5000, 3000, 2000, 1000, 700, 500, 300, 200, 100, 50, 20, 10};
-      */
-      double stdlev[] = {100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 20000, 15000,
-                          10000,  7000,  5000,  3000,  2000, 1000 };
-      nplev = sizeof(stdlev)/sizeof(*stdlev);
-      plev  = (double *) Malloc(nplev*sizeof(double));
-      for ( i = 0; i < nplev; ++i ) plev[i] = stdlev[i];
+      if ( operfunc == func_hl )
+        {
+          double stdlev[] = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
+          nplev = sizeof(stdlev)/sizeof(*stdlev);
+          plev  = (double *) Malloc(nplev*sizeof(double));
+          for ( int i = 0; i < nplev; ++i ) plev[i] = stdlev[i];
+        }
+      else
+        {
+          /*
+            double stdlev[] = {100000, 92500, 85000, 77500, 70000, 60000, 50000, 40000, 30000, 25000, 20000,
+            15000, 10000, 7000, 5000, 3000, 2000, 1000, 700, 500, 300, 200, 100, 50, 20, 10};
+          */
+          double stdlev[] = {100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 20000, 15000,
+                             10000,  7000,  5000,  3000,  2000, 1000 };
+          nplev = sizeof(stdlev)/sizeof(*stdlev);
+          plev  = (double *) Malloc(nplev*sizeof(double));
+          for ( int i = 0; i < nplev; ++i ) plev[i] = stdlev[i];
+        }
     }
   else
     {
-      nplev = args2fltlist(operatorArgc(), operatorArgv(), flist);
-      plev  = (double *) listArrayPtr(flist);
+      nplev = args2flt_lista(operatorArgc(), operatorArgv(), flista);
+      plev  = (double *) lista_dataptr(flista);
     }
   
   int streamID1 = streamOpenRead(cdoStreamName(0));
@@ -135,154 +151,35 @@ void *Vertintml(void *argument)
 
   int gridsize = vlist_check_gridsize(vlistID1);
 
-  if ( operfunc == func_hl )
-    zaxisIDp = zaxisCreate(ZAXIS_HEIGHT, nplev);
-  else
-    zaxisIDp = zaxisCreate(ZAXIS_PRESSURE, nplev);
-
+  int zaxisIDp = (operfunc == func_hl) ? zaxisCreate(ZAXIS_HEIGHT, nplev) : zaxisCreate(ZAXIS_PRESSURE, nplev);
   zaxisDefLevels(zaxisIDp, plev);
-  int nzaxis  = vlistNzaxis(vlistID1);
-  int lhavevct = FALSE;
-  for ( i = 0; i < nzaxis; i++ )
-    {
-      /* mono_level = FALSE; */
-      mono_level = TRUE;
-      zaxisID = vlistZaxis(vlistID1, i);
-      nlevel  = zaxisInqSize(zaxisID);
 
-      if ( (zaxisInqType(zaxisID) == ZAXIS_HYBRID || zaxisInqType(zaxisID) == ZAXIS_HYBRID_HALF) &&
-	   nlevel > 1 )
-	{
-	  double level[nlevel];
-	  zaxisInqLevels(zaxisID, level);
-	  int l;
-	  for ( l = 0; l < nlevel; l++ )
-	    {
-	      if ( (l+1) != (int) (level[l]+0.5) ) break;
-	    }
-	  if ( l == nlevel ) mono_level = TRUE; 
-	}
+  int nvct = 0;
+  int zaxisIDh = -1;
+  int nhlev = 0, nhlevf = 0, nhlevh = 0;
+  double *vct = vlist_read_vct(vlistID1, &zaxisIDh, &nvct, &nhlev, &nhlevf, &nhlevh);
 
-      if ( (zaxisInqType(zaxisID) == ZAXIS_HYBRID || zaxisInqType(zaxisID) == ZAXIS_HYBRID_HALF) &&
-	   nlevel > 1 && mono_level )
-	{
-	  nvct = zaxisInqVctSize(zaxisID);
-	  if ( nlevel == (nvct/2 - 1) )
-	    {
-	      if ( lhavevct == FALSE )
-		{
-		  lhavevct = TRUE;
-		  zaxisIDh = zaxisID;
-		  nhlev    = nlevel;
-		  nhlevf   = nhlev;
-		  nhlevh   = nhlev + 1;
-
-		  vct = (double*) Malloc(nvct*sizeof(double));
-		  zaxisInqVct(zaxisID, vct);
-
-		  vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	      else
-		{
-		  if ( memcmp(vct, zaxisInqVctPtr(zaxisID), nvct*sizeof(double)) == 0 )
-		    vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	    }
-	  else if ( nlevel == (nvct/2) )
-	    {
-	      if ( lhavevct == FALSE )
-		{
-		  lhavevct = TRUE;
-		  zaxisIDh = zaxisID;
-		  nhlev    = nlevel;
-		  nhlevf   = nhlev - 1;
-		  nhlevh   = nhlev;
-	      
-		  vct = (double*) Malloc(nvct*sizeof(double));
-		  zaxisInqVct(zaxisID, vct);
-
-		  vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	      else
-		{
-		  if ( memcmp(vct, zaxisInqVctPtr(zaxisID), nvct*sizeof(double)) == 0 )
-		    vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	    }
-	  else if ( nlevel == (nvct - 4 - 1) )
-	    {
-	      if ( lhavevct == FALSE )
-		{
-		  int vctsize;
-		  int voff = 4;
-		  
-		  rvct = (double*) Malloc(nvct*sizeof(double));
-		  zaxisInqVct(zaxisID, rvct);
-
-		  if ( (int)(rvct[0]+0.5) == 100000 && rvct[voff] < rvct[voff+1] )
-		    {
-		      lhavevct = TRUE;
-		      zaxisIDh = zaxisID;
-		      nhlev    = nlevel;
-		      nhlevf   = nhlev;
-		      nhlevh   = nhlev + 1;
-
-		      vctsize = 2*nhlevh;
-		      vct = (double*) Malloc(vctsize*sizeof(double));
-
-		      vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-
-		      /* calculate VCT for LM */
-
-		      for ( i = 0; i < vctsize/2; i++ )
-			{
-			  if ( rvct[voff+i] >= rvct[voff] && rvct[voff+i] <= rvct[3] )
-			    {
-			      vct[i] = rvct[0]*rvct[voff+i];
-			      vct[vctsize/2+i] = 0;
-			    }
-			  else
-			    {
-			      vct[i] = (rvct[0]*rvct[3]*(1-rvct[voff+i]))/(1-rvct[3]);
-			      vct[vctsize/2+i] = (rvct[voff+i]-rvct[3])/(1-rvct[3]);
-			    }
-			}
-		      
-		      if ( cdoVerbose )
-			{
-			  for ( i = 0; i < vctsize/2; i++ )
-			    fprintf(stdout, "%5d %25.17f %25.17f\n", i, vct[i], vct[vctsize/2+i]);
-			}
-		    }
-		}
-	      else
-		{
-		  if ( memcmp(rvct, zaxisInqVctPtr(zaxisID), nvct*sizeof(double)) == 0 )
-		    vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
-		}
-	    }
-	}
-    }
+  change_hybrid_zaxis(vlistID1, vlistID2, nvct, vct, zaxisIDp);
 
   int psvarID = -1;
-  int linvertvct = FALSE;
-  if ( lhavevct && nvct && nvct%2 == 0 )
+  bool linvertvct = false;
+  if ( vct && nvct && nvct%2 == 0 )
     {
       psvarID = vlist_get_psvarid(vlistID1, zaxisIDh);
 
+      int i;
       for ( i = nvct/2+1; i < nvct; i++ )
         if ( vct[i] > vct[i-1] ) break;
-
-      if ( i == nvct ) linvertvct = TRUE;
+      if ( i == nvct ) linvertvct = true;
     }
 
-  if ( cdoVerbose ) cdoPrint("linvertvct = %d", linvertvct);
+  if ( cdoVerbose ) cdoPrint("linvertvct = %d", (int) linvertvct);
 
   if ( linvertvct )
     {
       double vctbuf[nvct];
       memcpy(vctbuf, vct, nvct*sizeof(double));
-      for ( i = 0; i < nvct/2; i++ )
+      for ( int i = 0; i < nvct/2; i++ )
         {
           vct[nvct/2-1-i] = vctbuf[i];
           vct[nvct-1-i] = vctbuf[i+nvct/2];
@@ -291,16 +188,15 @@ void *Vertintml(void *argument)
 
   int nvars = vlistNvars(vlistID1);
 
-  int vars[nvars];
+  bool vars[nvars];
+  bool varinterp[nvars];
+  int *varnmiss[nvars];
   double *vardata1[nvars];
   double *vardata2[nvars];
-  int *varnmiss[nvars];
-  int varinterp[nvars];
 
-  int maxlev   = nhlevh > nplev ? nhlevh : nplev;
+  int maxlev = nhlevh > nplev ? nhlevh : nplev;
 
-  if ( Extrapolate == 0 )
-    pnmiss = (int*) Malloc(nplev*sizeof(int));
+  int *pnmiss = extrapolate ? NULL : (int*) Malloc(nplev*sizeof(int));
 
   // check levels
   if ( zaxisIDh != -1 )
@@ -308,18 +204,20 @@ void *Vertintml(void *argument)
       int nlev = zaxisInqSize(zaxisIDh);
       if ( nlev != nhlev ) cdoAbort("Internal error, wrong number of hybrid level!");
       double levels[nlev];
-      zaxisInqLevels(zaxisIDh, levels);
+      cdoZaxisInqLevels(zaxisIDh, levels);
 
       for ( int ilev = 0; ilev < nlev; ++ilev )
 	{
 	  if ( (ilev+1) != (int)levels[ilev] )
 	    {
-	      //sortlevels = FALSE;
+	      //sortlevels = false;
 	      break;
 	    }
 	}
     }
 
+  int *vert_index = NULL;
+  double *ps_prog = NULL, *full_press = NULL, *half_press = NULL;
   if ( zaxisIDh != -1 && gridsize > 0 )
     {
       vert_index = (int*) Malloc(gridsize*nplev*sizeof(int));
@@ -336,22 +234,22 @@ void *Vertintml(void *argument)
       height2pressure(phlev, plev, nplev);
 
       if ( cdoVerbose )
-	for ( i = 0; i < nplev; ++i )
+	for ( int i = 0; i < nplev; ++i )
 	  cdoPrint("level = %d   height = %g   pressure = %g", i+1, plev[i], phlev[i]);
 
       memcpy(plev, phlev, nplev*sizeof(double));
     }
 
   if ( opertype == type_log )
-    for ( k = 0; k < nplev; k++ ) plev[k] = log(plev[k]);
+    for ( int k = 0; k < nplev; k++ ) plev[k] = log(plev[k]);
 
-  useTable = FALSE;
+  bool useTable = false;
   for ( varID = 0; varID < nvars; varID++ )
     {
-      tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
       if ( tableNum > 0 && tableNum != 255 )
 	{
-	  useTable = TRUE;
+	  useTable = true;
 	  break;
 	}
     }
@@ -360,17 +258,19 @@ void *Vertintml(void *argument)
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      gridID   = vlistInqVarGrid(vlistID1, varID);
-      zaxisID  = vlistInqVarZaxis(vlistID1, varID);
+      int gridID   = vlistInqVarGrid(vlistID1, varID);
+      int zaxisID  = vlistInqVarZaxis(vlistID1, varID);
+      int zaxistype = zaxisInqType(zaxisID);
       // gridsize = gridInqSize(gridID);
-      nlevel   = zaxisInqSize(zaxisID);
-      instNum  = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
-      tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
-
-      code     = vlistInqVarCode(vlistID1, varID);
-      param    = vlistInqVarParam(vlistID1, varID);
+      int nlevel   = zaxisInqSize(zaxisID);
+      int instNum  = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
+      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
 
+      int code     = vlistInqVarCode(vlistID1, varID);
+      
+      int param = vlistInqVarParam(vlistID1, varID);
       cdiParamToString(param, paramstr, sizeof(paramstr));
+      int pnum, pcat, pdis;
       cdiDecodeParam(param, &pnum, &pcat, &pdis);
       if ( pdis >= 0 && pdis < 255 ) code = -1;
 
@@ -437,7 +337,7 @@ void *Vertintml(void *argument)
 	  else if ( code == gribcodes.ps      && nlevel == 1      ) psID      = varID;
 	}
 
-      if ( gridInqType(gridID) == GRID_SPECTRAL && zaxisInqType(zaxisID) == ZAXIS_HYBRID )
+      if ( gridInqType(gridID) == GRID_SPECTRAL && zaxis_is_hybrid(zaxistype) )
 	cdoAbort("Spectral data on model level unsupported!");
 
       if ( gridInqType(gridID) == GRID_SPECTRAL )
@@ -448,21 +348,21 @@ void *Vertintml(void *argument)
       else
 	vardata1[varID] = (double*) Malloc(gridsize*nlevel*sizeof(double));
 
-      /* if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID && zaxisIDh != -1 && nlevel == nhlev ) */
+      /* if ( zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && nlevel == nhlev ) */
       if ( zaxisID == zaxisIDh ||
-	   (zaxisInqType(zaxisID) == ZAXIS_HYBRID && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)) )
+	   (zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)) )
 	{
-	  varinterp[varID] = TRUE;
+	  varinterp[varID] = true;
 	  vardata2[varID]  = (double*) Malloc(gridsize*nplev*sizeof(double));
 	  varnmiss[varID]  = (int*) Malloc(maxlev*sizeof(int));
 	  memset(varnmiss[varID], 0, maxlev*sizeof(int));
 	}
       else
 	{
-	  if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID && zaxisIDh != -1 && nlevel > 1 )
+	  if ( zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && nlevel > 1 )
 	    cdoWarning("Parameter %d has wrong number of levels, skipped! (param=%s nlevel=%d)",
 		       varID+1, paramstr, nlevel);
-	  varinterp[varID] = FALSE;
+	  varinterp[varID] = false;
 	  vardata2[varID]  = vardata1[varID];
 	  varnmiss[varID]  = (int*) Malloc(nlevel*sizeof(int));
 	}
@@ -479,18 +379,20 @@ void *Vertintml(void *argument)
       if ( gheightID != -1 ) cdoPrint("  %s", var_stdname(geopotential_height));
     }
 
-  if ( tempID != -1 || gheightID != -1 ) sgeopot_needed = TRUE;
+  if ( tempID != -1 || gheightID != -1 ) sgeopot_needed = true;
 
   if ( zaxisIDh != -1 && sgeopot_needed )
     {
       sgeopot = (double*) Malloc(gridsize*sizeof(double));
       if ( sgeopotID == -1 )
 	{
-	  if ( geopotID == -1 )
-	    cdoWarning("%s not found - set to zero!", var_stdname(surface_geopotential));
-	  else
-	    cdoPrint("%s not found - using bottom layer of %s!", var_stdname(surface_geopotential), var_stdname(geopotential));
-
+          if ( extrapolate )
+            {
+              if ( geopotID == -1 )
+                cdoWarning("%s not found - set to zero!", var_stdname(surface_geopotential));
+              else
+                cdoPrint("%s not found - using bottom layer of %s!", var_stdname(surface_geopotential), var_stdname(geopotential));
+            }
 	  memset(sgeopot, 0, gridsize*sizeof(double));
 	}
     }
@@ -498,7 +400,7 @@ void *Vertintml(void *argument)
   if ( zaxisIDh != -1 && gheightID != -1 && tempID == -1 )
     cdoAbort("Temperature not found, needed for vertical interpolation of geopotheight!");
 
-  presID = lnpsID;
+  int presID = lnpsID;
   if ( psvarID != -1 ) presID = psvarID;
 
   if ( zaxisIDh != -1 && presID == -1 )
@@ -522,7 +424,7 @@ void *Vertintml(void *argument)
   if ( zaxisIDh != -1 )
     {
       double suma = 0, sumb = 0;
-      for ( i = 0; i < nhlevh; i++ )
+      for ( int i = 0; i < nhlevh; i++ )
         {
           suma += vct[i];
           sumb += vct[i+nhlevh];
@@ -537,18 +439,17 @@ void *Vertintml(void *argument)
   int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
-      for ( varID = 0; varID < nvars; ++varID ) vars[varID] = FALSE;
+      for ( varID = 0; varID < nvars; ++varID ) vars[varID] = false;
 
       taxisCopyTimestep(taxisID2, taxisID1);
-
       streamDefTimestep(streamID2, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  //gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-	  zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-	  nlevel   = zaxisInqSize(zaxisID);
+	  int zaxisID  = vlistInqVarZaxis(vlistID1, varID);
+	  int nlevel   = zaxisInqSize(zaxisID);
 	  /*
 	  if ( sortlevels && zaxisIDh != -1 && zaxisID == zaxisIDh && nlevel == nhlev )
 	    {
@@ -559,11 +460,11 @@ void *Vertintml(void *argument)
           if ( linvertvct && zaxisIDh != -1 && zaxisID == zaxisIDh )
             levelID = nlevel-1-levelID;
 
-	  offset   = gridsize*levelID;
-	  single1  = vardata1[varID] + offset;
+	  size_t offset  = gridsize*levelID;
+	  double *single = vardata1[varID] + offset;
 
-	  streamReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
-	  vars[varID] = TRUE;
+	  streamReadRecord(streamID1, single, &varnmiss[varID][levelID]);
+	  vars[varID] = true;
 	}
 
       if ( zaxisIDh != -1 )
@@ -576,7 +477,7 @@ void *Vertintml(void *argument)
 		memcpy(sgeopot, vardata1[geopotID]+gridsize*(nhlevf-1), gridsize*sizeof(double));
 
 	      /* check range of surface geopot */
-	      if ( sgeopotID != -1 || geopotID != -1 )
+	      if ( extrapolate && (sgeopotID != -1 || geopotID != -1) )
 		{
 		  minmaxval(gridsize, sgeopot, NULL, &minval, &maxval);
 		  if ( minval < MIN_FIS || maxval > MAX_FIS )
@@ -587,7 +488,7 @@ void *Vertintml(void *argument)
 	    }
 
 	  if ( presID == lnpsID )
-	    for ( i = 0; i < gridsize; i++ ) ps_prog[i] = exp(vardata1[lnpsID][i]);
+	    for ( int i = 0; i < gridsize; i++ ) ps_prog[i] = exp(vardata1[lnpsID][i]);
 	  else if ( presID != -1 )
 	    memcpy(ps_prog, vardata1[presID], gridsize*sizeof(double));
 
@@ -601,32 +502,29 @@ void *Vertintml(void *argument)
 
 	  if ( opertype == type_log )
 	    {
-	      for ( i = 0; i < gridsize; i++ ) ps_prog[i] = log(ps_prog[i]);
+	      for ( int i = 0; i < gridsize; i++ ) ps_prog[i] = log(ps_prog[i]);
 
-	      for ( k = 0; k < nhlevh; k++ )
-		for ( i = 0; i < gridsize; i++ )
+	      for ( int k = 0; k < nhlevh; k++ )
+		for ( int i = 0; i < gridsize; i++ )
 		  half_press[k*gridsize+i] = log(half_press[k*gridsize+i]);
 
-	      for ( k = 0; k < nhlevf; k++ )
-		for ( i = 0; i < gridsize; i++ )
+	      for ( int k = 0; k < nhlevf; k++ )
+		for ( int i = 0; i < gridsize; i++ )
 		  full_press[k*gridsize+i] = log(full_press[k*gridsize+i]);
 	    }
 
 	  genind(vert_index, plev, full_press, gridsize, nplev, nhlevf);
 
-	  if ( Extrapolate == 0 )
-	    genindmiss(vert_index, plev, gridsize, nplev, ps_prog, pnmiss);
+	  if ( !extrapolate ) genindmiss(vert_index, plev, gridsize, nplev, ps_prog, pnmiss);
 	}
 
       for ( varID = 0; varID < nvars; varID++ )
 	{
 	  if ( vars[varID] )
 	    {
-	      gridID   = vlistInqVarGrid(vlistID1, varID);
-	      zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-	      missval  = vlistInqVarMissval(vlistID1, varID);
-	      //gridsize = gridInqSize(gridID);
-	      nlevel   = zaxisInqSize(zaxisID);
+	      int zaxisID  = vlistInqVarZaxis(vlistID1, varID);
+	      int nlevel   = zaxisInqSize(zaxisID);
+	      double missval = vlistInqVarMissval(vlistID1, varID);
 	      if ( varinterp[varID] )
 		{
 		  /*
@@ -635,28 +533,23 @@ void *Vertintml(void *argument)
 		      int i, k;
 		      double *vl1, *vl2;
 
-		      for ( k = 1; k < nlevel; k++ )
+		      for ( int k = 1; k < nlevel; k++ )
 			{
 			  vl1  = vardata1[varID] + gridsize*(k-1);
 			  vl2  = vardata1[varID] + gridsize*(k);
-			  for ( i = 0; i < gridsize; i++ )
+			  for ( int i = 0; i < gridsize; i++ )
 			    vl1[i] = 0.5*(vl1[i] + vl2[i]);
 			}
 		      
 		      nlevel = nhlevf;
 		    }
 		  */
-		  if ( nlevel == nhlevh )
-		    {
-		      hyb_press = half_press;
-		    }
-		  else if ( nlevel == nhlevf )
-		    {
-		      hyb_press = full_press;
-		    }
+                  double *hyb_press = NULL;
+		  if      ( nlevel == nhlevh ) hyb_press = half_press;
+		  else if ( nlevel == nhlevf ) hyb_press = full_press;
 		  else
 		    {
-		      param = vlistInqVarParam(vlistID1, varID);
+		      int param = vlistInqVarParam(vlistID1, varID);
 		      cdiParamToString(param, paramstr, sizeof(paramstr));
 		      cdoAbort("Number of hybrid level differ from full/half level (param=%s)!", paramstr);
 		    }
@@ -672,7 +565,7 @@ void *Vertintml(void *argument)
 		      if ( nlevel == nhlevh )
 			cdoAbort("Temperature on half level unsupported!");
 
-		      if ( opertype == type_log && Extrapolate )
+		      if ( opertype == type_log && extrapolate )
 			cdoAbort("Log. extrapolation of temperature unsupported!");
 
 		      interp_T(sgeopot, vardata1[varID], vardata2[varID],
@@ -681,7 +574,7 @@ void *Vertintml(void *argument)
 		    }
 		  else if ( varID == gheightID )
 		    {
-		      for ( i = 0; i < gridsize; ++i )
+		      for ( int i = 0; i < gridsize; ++i )
 			vardata1[varID][gridsize*nlevel+i] = sgeopot[i]/PlanetGrav;
 
 		      interp_Z(sgeopot, vardata1[varID], vardata2[varID],
@@ -694,8 +587,7 @@ void *Vertintml(void *argument)
 			       vert_index, plev, nplev, gridsize, nlevel, missval);
 		    }
 		  
-		  if ( Extrapolate == 0 )
-		    memcpy(varnmiss[varID], pnmiss, nplev*sizeof(int));
+		  if ( !extrapolate ) memcpy(varnmiss[varID], pnmiss, nplev*sizeof(int));
 		}
 	    }
 	}
@@ -704,14 +596,14 @@ void *Vertintml(void *argument)
 	{
 	  if ( vars[varID] )
 	    {
-	      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+	      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
 	      for ( levelID = 0; levelID < nlevel; levelID++ )
 		{
 		  //gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-		  offset   = gridsize*levelID;
-		  single2  = vardata2[varID] + offset;
+		  size_t offset  = gridsize*levelID;
+		  double *single = vardata2[varID] + offset;
 		  streamDefRecord(streamID2, varID, levelID);
-		  streamWriteRecord(streamID2, single2, varnmiss[varID][levelID]);
+		  streamWriteRecord(streamID2, single, varnmiss[varID][levelID]);
 		}
 	    }
 	}
@@ -730,16 +622,14 @@ void *Vertintml(void *argument)
     }
 
   if ( pnmiss     ) Free(pnmiss);
-
   if ( sgeopot    ) Free(sgeopot);
   if ( ps_prog    ) Free(ps_prog);
   if ( vert_index ) Free(vert_index);
   if ( full_press ) Free(full_press);
   if ( half_press ) Free(half_press);
   if ( vct        ) Free(vct);
-  if ( rvct       ) Free(rvct);
 
-  listDelete(flist);
+  lista_destroy(flista);
 
   cdoFinish();
 
