@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
       Vertwind    vertwind      Convert the vertical velocity to [m/s]
 */
 
-#include <ctype.h>
 
 #include <cdi.h>
 #include "cdo.h"
@@ -35,25 +34,14 @@
 
 void *Vertwind(void *argument)
 {
-  int streamID2;
-  int vlistID2;
-  int taxisID1, taxisID2;
-  int gridID, zaxisID, tsID;
-  int nlevel, nrecs, recID, code;
+  int nrecs;
   int varID, levelID;
-  int nvars, nvct = 0;
-  int gridsize, i;
-  int offset;
-  int nmiss, nmiss_out;
-  int temp_code, sq_code, ps_code, omega_code, lsp_code;
-  int tempID = -1, sqID = -1, psID = -1, omegaID = -1, lnpsID = -1;
+  int nvct = 0;
+  int nmiss;
+  int tempID = -1, sqID = -1, psID = -1, omegaID = -1;
   char varname[CDI_MAX_NAME];
   double *vct = NULL;
-  double tv, rho;
-  double *level = NULL;
-  double *temp = NULL, *sq = NULL, *omega = NULL, *wms = NULL;
-  double *fpress = NULL, *hpress = NULL, *ps_prog = NULL;
-  double missval_t, missval_sq, missval_wap, missval_out;
+  double *hpress = NULL, *ps_prog = NULL;
 
   cdoInitialize(argument);
 
@@ -61,42 +49,33 @@ void *Vertwind(void *argument)
 
   int vlistID1 = streamInqVlist(streamID1);
 
-  int ngp = vlist_check_gridsize(vlistID1);
+  vlist_check_gridsize(vlistID1);
 
-  temp_code  = 130;
-  sq_code    = 133;
-  ps_code    = 134;
-  omega_code = 135;
-  lsp_code   = 152;
+  int temp_code  = 130;
+  int sq_code    = 133;
+  int ps_code    = 134;
+  int omega_code = 135;
 
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
   for ( varID = 0; varID < nvars; ++varID )
     {
-      gridID   = vlistInqVarGrid(vlistID1, varID);
-      zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-      gridsize = gridInqSize(gridID);
-      nlevel   = zaxisInqSize(zaxisID);
-
-      code = vlistInqVarCode(vlistID1, varID);
+      int code = vlistInqVarCode(vlistID1, varID);
 
       if ( code <= 0 )
 	{
 	  vlistInqVarName(vlistID1, varID, varname);
-
 	  strtolower(varname);
 
-	  if      ( strcmp(varname, "st")    == 0 ) code = 130;
-	  else if ( strcmp(varname, "sq")    == 0 ) code = 133;
-	  else if ( strcmp(varname, "aps")   == 0 ) code = 134;
-	  else if ( strcmp(varname, "omega") == 0 ) code = 135;
-	  else if ( strcmp(varname, "lsp")   == 0 ) code = 152;
+	  if      ( strcmp(varname, "st")    == 0 ) code = temp_code;
+	  else if ( strcmp(varname, "sq")    == 0 ) code = sq_code;
+	  else if ( strcmp(varname, "aps")   == 0 ) code = ps_code;
+	  else if ( strcmp(varname, "omega") == 0 ) code = omega_code;
 	}
 
       if      ( code == temp_code  ) tempID  = varID;
       else if ( code == sq_code    ) sqID    = varID;
       else if ( code == ps_code    ) psID    = varID;
       else if ( code == omega_code ) omegaID = varID;
-      else if ( code == lsp_code   ) lnpsID  = varID;
     }
 
   if ( tempID == -1 || sqID == -1 || omegaID == -1 )
@@ -108,47 +87,47 @@ void *Vertwind(void *argument)
     }
 
   /* Get missing values */
-  missval_t   = vlistInqVarMissval(vlistID1, tempID);
-  missval_sq  = vlistInqVarMissval(vlistID1, sqID);
-  missval_wap = vlistInqVarMissval(vlistID1, omegaID);
-  missval_out = missval_wap;
+  double missval_t   = vlistInqVarMissval(vlistID1, tempID);
+  double missval_sq  = vlistInqVarMissval(vlistID1, sqID);
+  double missval_wap = vlistInqVarMissval(vlistID1, omegaID);
+  double missval_out = missval_wap;
 
-  gridID  = vlistInqVarGrid(vlistID1, omegaID);
-  zaxisID = vlistInqVarZaxis(vlistID1, omegaID);
+  int gridID  = vlistInqVarGrid(vlistID1, omegaID);
+  int zaxisID = vlistInqVarZaxis(vlistID1, omegaID);
 
   if ( psID == -1 && zaxisInqType(zaxisID) == ZAXIS_HYBRID )
     cdoAbort("Surface pressure (code 134) not found!");
 
-  gridsize = gridInqSize(gridID);
-  nlevel = zaxisInqSize(zaxisID);
-  level  = (double*) malloc(nlevel*sizeof(double));
-  zaxisInqLevels(zaxisID, level);
+  int gridsize = gridInqSize(gridID);
+  int nlevel = zaxisInqSize(zaxisID);
+  double *level = (double*) Malloc(nlevel*sizeof(double));
+  cdoZaxisInqLevels(zaxisID, level);
 
-  temp    = (double*) malloc(gridsize*nlevel*sizeof(double));
-  sq      = (double*) malloc(gridsize*nlevel*sizeof(double));
-  omega   = (double*) malloc(gridsize*nlevel*sizeof(double));
-  wms     = (double*) malloc(gridsize*nlevel*sizeof(double));
-  fpress  = (double*) malloc(gridsize*nlevel*sizeof(double));
+  double *temp    = (double*) Malloc(gridsize*nlevel*sizeof(double));
+  double *sq      = (double*) Malloc(gridsize*nlevel*sizeof(double));
+  double *omega   = (double*) Malloc(gridsize*nlevel*sizeof(double));
+  double *wms     = (double*) Malloc(gridsize*nlevel*sizeof(double));
+  double *fpress  = (double*) Malloc(gridsize*nlevel*sizeof(double));
 
 
   if ( zaxisInqType(zaxisID) == ZAXIS_PRESSURE )
     {
       for ( levelID = 0; levelID < nlevel; ++levelID )
 	{
-	  offset = levelID*gridsize;
-	  for ( i = 0; i < gridsize; ++i )
+	  size_t offset = (size_t)levelID*gridsize;
+	  for ( int i = 0; i < gridsize; ++i )
 	    fpress[offset+i] = level[levelID];
 	}
     }
   else if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID )
     {
-      ps_prog = (double*) malloc(gridsize*sizeof(double));
-      hpress  = (double*) malloc(gridsize*(nlevel+1)*sizeof(double));
+      ps_prog = (double*) Malloc(gridsize*sizeof(double));
+      hpress  = (double*) Malloc(gridsize*(nlevel+1)*sizeof(double));
   
       nvct = zaxisInqVctSize(zaxisID);
       if ( nlevel == (nvct/2 - 1) )
 	{
-	  vct = (double*) malloc(nvct*sizeof(double));
+	  vct = (double*) Malloc(nvct*sizeof(double));
 	  zaxisInqVct(zaxisID, vct);
 	}
       else
@@ -162,7 +141,7 @@ void *Vertwind(void *argument)
   for ( levelID = 0; levelID < nlevel; ++levelID )
     vlistDefFlag(vlistID1, omegaID, levelID, TRUE);
 
-  vlistID2 = vlistCreate();
+  int vlistID2 = vlistCreate();
   vlistCopyFlag(vlistID2, vlistID1);
   vlistDefVarCode(vlistID2, 0, 40);
   vlistDefVarName(vlistID2, 0, "W");
@@ -170,26 +149,25 @@ void *Vertwind(void *argument)
   vlistDefVarUnits(vlistID2, 0, "m/s");
   vlistDefVarMissval(vlistID2, 0, missval_out);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-
       streamDefTimestep(streamID2, tsID);
      
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
-	  offset = levelID*gridsize;
+	  size_t offset = (size_t)levelID*gridsize;
 
 	  if      ( varID == tempID )
 	    streamReadRecord(streamID1, temp+offset, &nmiss);
@@ -206,9 +184,9 @@ void *Vertwind(void *argument)
 
       for ( levelID = 0; levelID < nlevel; ++levelID )
 	{
-	  offset = levelID*gridsize;
+	  size_t offset = (size_t)levelID*gridsize;
 
-	  for ( i = 0; i < gridsize; ++i )
+	  for ( int i = 0; i < gridsize; ++i )
 	    {
 	      if ( DBL_IS_EQUAL(temp[offset+i],missval_t)    || 
 		   DBL_IS_EQUAL(omega[offset+i],missval_wap) ||
@@ -218,19 +196,14 @@ void *Vertwind(void *argument)
 		}
 	      else
 		{
-	          /* Virtuelle Temperatur bringt die Feuchteabhaengigkeit hinein */
-	          tv = temp[offset+i] * (1. + 0.608*sq[offset+i]);
+	          // Virtuelle Temperatur bringt die Feuchteabhaengigkeit hinein
+	          double tv = temp[offset+i] * (1. + 0.608*sq[offset+i]);
 
+	          // Die Dichte erhaelt man nun mit der Gasgleichung rho=p/(R*tv) Level in Pa!
+	          double rho = fpress[offset+i] / (R*tv);
 	          /*
-		    Die Dichte erhaelt man nun mit der Gasgleichung rho=p/(R*tv)
-		    Level in Pa!
-	          */
-	          rho = fpress[offset+i] / (R*tv);
-
-	          /*
-		    Nun daraus die Vertikalgeschwindigkeit im m/s, indem man die
-		    Vertikalgeschwindigkeit in Pa/s durch die Erdbeschleunigung
-		    und die Dichte teilt
+		    Nun daraus die Vertikalgeschwindigkeit im m/s, indem man die Vertikalgeschwindigkeit
+                    in Pa/s durch die Erdbeschleunigung und die Dichte teilt
 	          */
 	          wms[offset+i] = omega[offset+i]/(G*rho);
 	        }
@@ -239,13 +212,13 @@ void *Vertwind(void *argument)
 
       for ( levelID = 0; levelID < nlevel; ++levelID )
 	{
-	  nmiss_out = 0;
-	  for ( i = 0; i < gridsize; i++ )
+	  size_t offset = (size_t)levelID*gridsize;
+
+	  int nmiss_out = 0;
+	  for ( int i = 0; i < gridsize; i++ )
             if ( DBL_IS_EQUAL(wms[offset+i],missval_out) )
 	      nmiss_out++;
 
-	  offset = levelID*gridsize;
-
 	  streamDefRecord(streamID2, 0, levelID);
 	  streamWriteRecord(streamID2, wms+offset, nmiss_out);
 	}
@@ -258,19 +231,19 @@ void *Vertwind(void *argument)
  
   vlistDestroy(vlistID2);
 
-  free(temp);
-  free(sq);
-  free(omega);
-  free(wms);
-  free(fpress);
+  Free(temp);
+  Free(sq);
+  Free(omega);
+  Free(wms);
+  Free(fpress);
 
-  if ( ps_prog ) free(ps_prog);
-  if ( hpress )  free(hpress);
-  if ( vct ) free(vct);
+  if ( ps_prog ) Free(ps_prog);
+  if ( hpress )  Free(hpress);
+  if ( vct ) Free(vct);
 
-  free(level);
+  Free(level);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
