@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,22 +30,15 @@
 
 void *Writegrid(void *argument)
 {
-  int streamID;
-  int vlistID;
-  int gridID;
-  long gridsize, i;
-  int gridtype;
-  int *mask = NULL;
-
   cdoInitialize(argument);
 
-  streamID = streamOpenRead(cdoStreamName(0));
+  int streamID = streamOpenRead(cdoStreamName(0));
 
-  vlistID = streamInqVlist(streamID);
-  gridID  = vlistGrid(vlistID, 0);
+  int vlistID = streamInqVlist(streamID);
+  int gridID  = vlistGrid(vlistID, 0);
 
-  gridtype = gridInqType(gridID);
-  gridsize = gridInqSize(gridID);
+  int gridtype = gridInqType(gridID);
+  int gridsize = gridInqSize(gridID);
 
   if ( gridtype == GRID_GME ) gridID = gridToUnstructured(gridID, 1);
 
@@ -55,7 +48,7 @@ void *Writegrid(void *argument)
   if ( gridInqXbounds(gridID, NULL) == 0 || gridInqYbounds(gridID, NULL) == 0 )
     cdoAbort("Grid corner missing!");
 
-  mask = malloc(gridsize*sizeof(int));
+  int *mask = (int*) Malloc(gridsize*sizeof(int));
 
   if ( gridInqMask(gridID, NULL) )
     {
@@ -63,16 +56,16 @@ void *Writegrid(void *argument)
     }
   else
     {
-      for ( i = 0; i < gridsize; i++ ) mask[i] = 1;
+      for ( int i = 0; i < gridsize; i++ ) mask[i] = 1;
     }
       
   writeNCgrid(cdoStreamName(1)->args, gridID, mask);
 
   streamClose(streamID);
 
-  if ( mask ) free(mask);
+  if ( mask ) Free(mask);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
