@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -83,7 +83,7 @@ typedef struct {
   int tsIDnext;
   int streamID, nrecs;
   recinfo_t *recinfo;
-  field_t **vars;
+  field_type **vars;
 }
 readarg_t;
 
@@ -94,7 +94,7 @@ void *cdoReadTimestep(void *rarg)
 {
   int varID, levelID, nmiss;
   readarg_t *readarg = (readarg_t *) rarg;
-  field_t **input_vars = readarg->vars;
+  field_type **input_vars = readarg->vars;
   recinfo_t *recinfo = readarg->recinfo;
   int streamID = readarg->streamID;
   int tsIDnext = readarg->tsIDnext;
@@ -128,7 +128,7 @@ void *cdoReadTimestep(void *rarg)
 }
 
 static
-void cdoUpdateVars(int nvars, int vlistID, field_t **vars)
+void cdoUpdateVars(int nvars, int vlistID, field_type **vars)
 {
   void *tmp = NULL;
 
@@ -162,11 +162,10 @@ void *XTimstat(void *argument)
   int vdate = 0, vtime = 0;
   int vdate0 = 0, vtime0 = 0;
   int varID;
-  int nsets;
   int streamID3 = -1;
   int vlistID3, taxisID3 = -1;
   int nmiss;
-  int lvfrac = FALSE;
+  bool lvfrac = false;
   int nwpv; // number of words per value; real:1  complex:2
   char indate1[DATE_LEN+1], indate2[DATE_LEN+1];
   double vfrac = 1;
@@ -206,10 +205,10 @@ void *XTimstat(void *argument)
   int operfunc   = cdoOperatorF1(operatorID);
   int comparelen = cdoOperatorF2(operatorID);
 
-  int lmean   = operfunc == func_mean || operfunc == func_avg;
-  int lstd    = operfunc == func_std || operfunc == func_std1;
-  int lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  int divisor = operfunc == func_std1 || operfunc == func_var1;
+  bool lmean   = operfunc == func_mean || operfunc == func_avg;
+  bool lstd    = operfunc == func_std || operfunc == func_std1;
+  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  int divisor  = operfunc == func_std1 || operfunc == func_var1;
 
   if ( operfunc == func_mean )
     {
@@ -218,7 +217,7 @@ void *XTimstat(void *argument)
 
       if ( oargc == 1 )
 	{
-	  lvfrac = TRUE;
+	  lvfrac = true;
 	  vfrac = atof(oargv[0]);
 	  if ( cdoVerbose ) cdoPrint("Set vfrac to %g", vfrac);
 	  if ( vfrac < 0 || vfrac > 1 ) cdoAbort("vfrac out of range!");
@@ -252,7 +251,7 @@ void *XTimstat(void *argument)
   if      ( comparelen == DAY_LEN )  freq = "day";
   else if ( comparelen == MON_LEN )  freq = "mon";
   else if ( comparelen == YEAR_LEN ) freq = "year";
-  if ( freq ) vlistDefAttTxt(vlistID2, CDI_GLOBAL, "frequency", (int)strlen(freq), freq);
+  if ( freq ) cdiDefAttTxt(vlistID2, CDI_GLOBAL, "frequency", (int)strlen(freq), freq);
 
   int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
@@ -272,7 +271,7 @@ void *XTimstat(void *argument)
 
       for ( varID = 0; varID < nvars; ++varID )
 	{
-	  vlistDefVarDatatype(vlistID3, varID, DATATYPE_INT32);
+	  vlistDefVarDatatype(vlistID3, varID, CDI_DATATYPE_INT32);
 	  vlistDefVarMissval(vlistID3, varID, -1);
 	  vlistDefVarUnits(vlistID3, varID, "");
 	  vlistDefVarAddoffset(vlistID3, varID, 0);
@@ -294,10 +293,10 @@ void *XTimstat(void *argument)
 
   int FIELD_MEMTYPE = 0;
   if ( CDO_Memtype == MEMTYPE_FLOAT ) FIELD_MEMTYPE = FIELD_FLT;
-  field_t **input_vars = field_malloc(vlistID1, FIELD_PTR | FIELD_PTR2 | FIELD_MEMTYPE);
-  field_t **vars1 = field_malloc(vlistID1, FIELD_PTR);
-  field_t **samp1 = field_malloc(vlistID1, FIELD_NONE);
-  field_t **vars2 = NULL;
+  field_type **input_vars = field_malloc(vlistID1, FIELD_PTR | FIELD_PTR2 | FIELD_MEMTYPE);
+  field_type **vars1 = field_malloc(vlistID1, FIELD_PTR);
+  field_type **samp1 = field_malloc(vlistID1, FIELD_NONE);
+  field_type **vars2 = NULL;
   if ( lvarstd ) vars2 = field_malloc(vlistID1, FIELD_PTR);
 
   readarg_t readarg;
@@ -305,7 +304,7 @@ void *XTimstat(void *argument)
   readarg.vars = input_vars;
 
   int lparallelread = CDO_Parallel_Read;
-  int ltsfirst = TRUE;
+  bool ltsfirst = true;
   void *read_task = NULL;
   void *readresult = NULL;
 
@@ -323,12 +322,12 @@ void *XTimstat(void *argument)
   int otsID = 0;
   int nrecs = streamInqTimestep(streamID1, tsID);
   int maxrecs = nrecs;
-  recinfo_t *recinfo = (recinfo_t *) malloc(maxrecs*sizeof(recinfo_t));
+  recinfo_t *recinfo = (recinfo_t *) Malloc(maxrecs*sizeof(recinfo_t));
   
   tsID++;
   while ( TRUE )
     {
-      nsets = 0;
+      int nsets = 0;
       while ( nrecs > 0 )
 	{
 	  dtlist_taxisInqTimestep(dtlist, taxisID1, nsets);
@@ -346,7 +345,7 @@ void *XTimstat(void *argument)
 
           if ( ltsfirst || lparallelread == FALSE )
             {
-              ltsfirst = FALSE;
+              ltsfirst = false;
               readresult = cdoReadTimestep(&readarg);
             }
           else
@@ -374,8 +373,8 @@ void *XTimstat(void *argument)
                   int varID    = recinfo[recID].varID;
                   int levelID  = recinfo[recID].levelID;
 
-                  field_t *pvar1 = &vars1[varID][levelID];
-                  field_t *pinput_var = &input_vars[varID][levelID];
+                  field_type *pvar1 = &vars1[varID][levelID];
+                  field_type *pinput_var = &input_vars[varID][levelID];
 
                   int nwpv     = pvar1->nwpv;
                   int gridsize = pvar1->size;
@@ -406,8 +405,8 @@ void *XTimstat(void *argument)
                   int varID    = recinfo[recID].varID;
                   int levelID  = recinfo[recID].levelID;
                   
-                  field_t *pvar1 = &vars1[varID][levelID];
-                  field_t *pinput_var = &input_vars[varID][levelID];
+                  field_type *pvar1 = &vars1[varID][levelID];
+                  field_type *pinput_var = &input_vars[varID][levelID];
 
                   int nwpv     = pvar1->nwpv;
                   int gridsize = pvar1->size;
@@ -429,7 +428,7 @@ void *XTimstat(void *argument)
                   
 		  if ( lvarstd )
 		    {
-                      field_t *pvar2 = &vars2[varID][levelID];
+                      field_type *pvar2 = &vars2[varID][levelID];
 		      farsumq(pvar2, *pinput_var);
 		      farsum(pvar1, *pinput_var);
 		    }
@@ -445,8 +444,8 @@ void *XTimstat(void *argument)
               {
                 int varID   = recinfo[recID].varID;
                 int levelID = recinfo[recID].levelID;
-                field_t *pvar1 = &vars1[varID][levelID];
-                field_t *pvar2 = &vars2[varID][levelID];
+                field_type *pvar1 = &vars1[varID][levelID];
+                field_type *pvar2 = &vars2[varID][levelID];
 
 		if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 
@@ -462,43 +461,47 @@ void *XTimstat(void *argument)
       if ( nrecs == 0 && nsets == 0 ) break;
 
       if ( lmean )
+        {
 #if defined(_OPENMP)
 #pragma omp parallel for default(none) shared(vlistID1, nsets, maxrecs, recinfo, vars1, samp1) if(maxrecs>1)
 #endif
-        for ( int recID = 0; recID < maxrecs; recID++ )
-          {
-            int varID   = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
-            field_t *pvar1 = &vars1[varID][levelID];
+          for ( int recID = 0; recID < maxrecs; recID++ )
+            {
+              int varID   = recinfo[recID].varID;
+              int levelID = recinfo[recID].levelID;
+              field_type *pvar1 = &vars1[varID][levelID];
 
-	    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
+              if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 
-            if ( samp1[varID][levelID].ptr == NULL )
-              farcdiv(pvar1, (double)nsets);
-            else
-              fardiv(pvar1, samp1[varID][levelID]);
-	  }
+              if ( samp1[varID][levelID].ptr == NULL )
+                farcdiv(pvar1, (double)nsets);
+              else
+                fardiv(pvar1, samp1[varID][levelID]);
+            }
+        }
       else if ( lvarstd )
-        for ( int recID = 0; recID < maxrecs; recID++ )
-          {
-            int varID   = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
-            field_t *pvar1 = &vars1[varID][levelID];
-            field_t *pvar2 = &vars2[varID][levelID];
+        {
+          for ( int recID = 0; recID < maxrecs; recID++ )
+            {
+              int varID   = recinfo[recID].varID;
+              int levelID = recinfo[recID].levelID;
+              field_type *pvar1 = &vars1[varID][levelID];
+              field_type *pvar2 = &vars2[varID][levelID];
 
-            if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
+              if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 
-            if ( samp1[varID][levelID].ptr == NULL )
-              {
-                if ( lstd ) farcstd(pvar1, *pvar2, nsets, divisor);
-                else        farcvar(pvar1, *pvar2, nsets, divisor);
-              }
-            else
-              {
-                if ( lstd ) farstd(pvar1, *pvar2, samp1[varID][levelID], divisor);
-                else        farvar(pvar1, *pvar2, samp1[varID][levelID], divisor);
-	      }
-	  }
+              if ( samp1[varID][levelID].ptr == NULL )
+                {
+                  if ( lstd ) farcstd(pvar1, *pvar2, nsets, divisor);
+                  else        farcvar(pvar1, *pvar2, nsets, divisor);
+                }
+              else
+                {
+                  if ( lstd ) farstd(pvar1, *pvar2, samp1[varID][levelID], divisor);
+                  else        farvar(pvar1, *pvar2, samp1[varID][levelID], divisor);
+                }
+            }
+        }
 
       if ( cdoVerbose )
 	{
@@ -513,7 +516,7 @@ void *XTimstat(void *argument)
           {
             int varID   = recinfo[recID].varID;
             int levelID = recinfo[recID].levelID;
-            field_t *pvar1 = &vars1[varID][levelID];
+            field_type *pvar1 = &vars1[varID][levelID];
 
 	    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 
@@ -555,7 +558,7 @@ void *XTimstat(void *argument)
 	{
           int varID   = recinfo[recID].varID;
           int levelID = recinfo[recID].levelID;
-          field_t *pvar1 = &vars1[varID][levelID];
+          field_type *pvar1 = &vars1[varID][levelID];
 
 	  if ( otsID && vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 
