@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -172,7 +172,7 @@ void store_link_bilin(remapvars_t *rv, int dst_add, int src_add[4], double weigh
 
 } /* store_link_bilin */
 
-void yar_remap_bil(field_t *field1, field_t *field2)
+void yar_remap_bil(field_type *field1, field_type *field2)
 {
   int nlonIn, nlatIn;
   int nlonOut, nlatOut;
@@ -205,7 +205,7 @@ void yar_remap_bil(field_t *field1, field_t *field2)
 
   remap.src_grid.num_srch_bins = 0;
   remap.tgt_grid.num_srch_bins = 0;
-  remap.vars.pinit = FALSE;
+  remap.vars.pinit = false;
 
   if ( cdoTimer ) timer_start(timer_yar_remap_init);
   remap_grids_init(MAP_TYPE_BILINEAR, 0, gridIDin, &remap.src_grid, gridIDout, &remap.tgt_grid);
@@ -423,7 +423,7 @@ void yar_remap_bil(field_t *field1, field_t *field2)
 }
 
 
-void yar_remap_con(field_t *field1, field_t *field2)
+void yar_remap_con(field_type *field1, field_type *field2)
 {
   int nlonIn, nlatIn;
   int nlonOut, nlatOut;
@@ -454,7 +454,7 @@ void yar_remap_con(field_t *field1, field_t *field2)
 
   remap.src_grid.num_srch_bins = 0;
   remap.tgt_grid.num_srch_bins = 0;
-  remap.vars.pinit = FALSE;
+  remap.vars.pinit = false;
 
   if ( cdoTimer ) timer_start(timer_yar_remap_init);
   remap_grids_init(MAP_TYPE_CONSERV, 0, gridIDin, &remap.src_grid, gridIDout, &remap.tgt_grid);
@@ -739,22 +739,14 @@ void yar_remap_con(field_t *field1, field_t *field2)
 
 void *YAR(void *argument)
 {
-  int YARBIL, YARCON;
-  int operatorID;
-  int streamID1, streamID2;
-  int nrecs, ngrids;
+  int nrecs;
   int index;
-  int tsID, recID, varID, levelID;
-  int gridsize;
-  int vlistID1, vlistID2;
-  int gridID1 = -1, gridID2 = -1;
+  int varID, levelID;
+  int gridID1 = -1;
   int nmiss;
   int xinc = 0, yinc = 0;
   double missval;
   double slon, slat;
-  double *array1 = NULL, *array2 = NULL;
-  field_t field1, field2;
-  int taxisID1, taxisID2;
 
   if ( cdoTimer )
     {
@@ -767,27 +759,29 @@ void *YAR(void *argument)
 
   cdoInitialize(argument);
 
-  YARBIL = cdoOperatorAdd("yarbil",  0, 0, NULL);
-  YARCON = cdoOperatorAdd("yarcon",  0, 0, NULL);
+  int YARBIL = cdoOperatorAdd("yarbil",  0, 0, NULL);
+  int YARCON = cdoOperatorAdd("yarcon",  0, 0, NULL);
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   operatorInputArg("grid description file or name");
-  gridID2 = cdoDefineGrid(operatorArgv()[0]);
 
+  field_type field1, field2;
   field_init(&field1);
   field_init(&field2);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int gridID2 = cdoDefineGrid(operatorArgv()[0]);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
+
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  ngrids = vlistNgrids(vlistID1);
+  int ngrids = vlistNgrids(vlistID1);
   for ( index = 0; index < ngrids; index++ )
     {
       gridID1 = vlistGrid(vlistID1, index);
@@ -795,30 +789,26 @@ void *YAR(void *argument)
       if ( gridInqType(gridID1) != GRID_LONLAT && gridInqType(gridID1) != GRID_GAUSSIAN )
 	cdoAbort("Interpolation of %s data unsupported!", gridNamePtr(gridInqType(gridID1)) );
 
-      if ( gridIsRotated(gridID1) )
-	cdoAbort("Rotated grids not supported!");
-
       vlistChangeGridIndex(vlistID2, index, gridID2);
     }
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  gridsize = vlistGridsizeMax(vlistID1);
-  array1   = (double*) Malloc(gridsize*sizeof(double));
+  int gridsize = vlistGridsizeMax(vlistID1);
+  double *array1 = (double*) Malloc(gridsize*sizeof(double));
 
   gridsize = gridInqSize(gridID2);
-  array2   = (double*) Malloc(gridsize*sizeof(double));
+  double *array2 = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-
       streamDefTimestep(streamID2, tsID);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  streamReadRecord(streamID1, array1, &nmiss);
