@@ -24,14 +24,15 @@
       Ydrunstat    ydrunmean         Multi-year daily running mean
       Ydrunstat    ydrunavg          Multi-year daily running average
       Ydrunstat    ydrunvar          Multi-year daily running variance
-      Ydrunstat    ydrunvar1         Multi-year daily running variance [Divisor is (n-1)]
+      Ydrunstat    ydrunvar1         Multi-year daily running variance [Normalize by (n-1)]
       Ydrunstat    ydrunstd          Multi-year daily running standard deviation
-      Ydrunstat    ydrunstd1         Multi-year daily running standard deviation [Divisor is (n-1)]
+      Ydrunstat    ydrunstd1         Multi-year daily running standard deviation [Normalize by (n-1)]
 */
 
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
+#include "calendar.h"
 #include "pstream.h"
 
 
@@ -41,8 +42,8 @@
 typedef struct {
   int       vdate[NDAY];
   int       vtime[NDAY];  
-  field_t **vars1[NDAY]; 
-  field_t **vars2[NDAY];
+  field_type **vars1[NDAY]; 
+  field_type **vars2[NDAY];
   int       nsets[NDAY];
   int       vlist;
 }
@@ -52,33 +53,20 @@ YDAY_STATS;
 static YDAY_STATS *ydstatCreate(int vlistID);
 static void ydstatDestroy(YDAY_STATS *stats);
 static void ydstatUpdate(YDAY_STATS *stats, int vdate, int vtime, 
-  field_t **vars1, field_t **vars2, int nsets, int operfunc);
+  field_type **vars1, field_type **vars2, int nsets, int operfunc);
 static void ydstatFinalize(YDAY_STATS *stats, int operfunc);
 
 
 void *Ydrunstat(void *argument)
 {
-  int operatorID;
-  int operfunc;
   int varID;
-  int recID;
-  int nrecs, nrecords;
+  int nrecs;
   int levelID;
   int tsID;
-  int otsID;
-  int inp, its, ndates = 0;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
+  int inp, its;
   int nmiss;
-  int *recVarID, *recLevelID;
-  int lvarstd = FALSE;
-  field_t ***vars1 = NULL, ***vars2 = NULL;
-  datetime_t *datetime;
-  int taxisID1, taxisID2;
-  int calendar, dpy;
   int vdate, vtime;
   int dayoy;
-  YDAY_STATS *stats;
     
   cdoInitialize(argument);
 
@@ -92,42 +80,43 @@ void *Ydrunstat(void *argument)
   cdoOperatorAdd("ydrunstd",  func_std,  0, NULL);
   cdoOperatorAdd("ydrunstd1", func_std1, 0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
 
   operatorInputArg("number of timesteps");
-  ndates = atoi(operatorArgv()[0]);
+  int ndates = parameter2int(operatorArgv()[0]);
 
-  lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
   
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   if ( taxisHasBounds(taxisID2) ) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  calendar = taxisInqCalendar(taxisID1);
-  dpy      = calendar_dpy(calendar);
+  int calendar = taxisInqCalendar(taxisID1);
+  int dpy      = calendar_dpy(calendar);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  nrecords = vlistNrecs(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = (int*) malloc(nrecords*sizeof(int));
-  recLevelID = (int*) malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
-  datetime = (datetime_t*) malloc((ndates+1)*sizeof(datetime_t));
+  cdo_datetime_t *datetime = (cdo_datetime_t*) Malloc((ndates+1)*sizeof(cdo_datetime_t));
   
-  stats = ydstatCreate(vlistID1);
-  vars1 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
+  YDAY_STATS *stats = ydstatCreate(vlistID1);
+  field_type ***vars1 = (field_type ***) Malloc((ndates+1)*sizeof(field_type **));
+  field_type ***vars2 = NULL;
   if ( lvarstd )
-    vars2 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
+    vars2 = (field_type ***) Malloc((ndates+1)*sizeof(field_type **));
   
   for ( its = 0; its < ndates; its++ )
     {
@@ -145,7 +134,7 @@ void *Ydrunstat(void *argument)
       datetime[tsID].date = taxisInqVdate(taxisID1);
       datetime[tsID].time = taxisInqVtime(taxisID1);
 	
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -208,7 +197,7 @@ void *Ydrunstat(void *argument)
       datetime[ndates-1].date = taxisInqVdate(taxisID1);
       datetime[ndates-1].time = taxisInqVtime(taxisID1);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  
@@ -236,25 +225,27 @@ void *Ydrunstat(void *argument)
       tsID++;
     }
 
+  /*
+  // set the year to the minimum of years found on output timestep
   int outyear = 1e9;
+  int year, month, day;
   for ( dayoy = 0; dayoy < NDAY; dayoy++ )
     if ( stats->nsets[dayoy] )
       {
-	int year, month, day;
 	cdiDecodeDate(stats->vdate[dayoy], &year, &month, &day);
 	if ( year < outyear ) outyear = year;
       }
-
   for ( dayoy = 0; dayoy < NDAY; dayoy++ )
     if ( stats->nsets[dayoy] )
       {
-	int year, month, day;
 	cdiDecodeDate(stats->vdate[dayoy], &year, &month, &day);
-	stats->vdate[dayoy] = cdiEncodeDate(outyear, month, day);
+        // printf("vdates[%d] = %d  nsets = %d\n", dayoy, stats->vdate[dayoy], stats->nsets[dayoy]);
+	if ( year > outyear ) stats->vdate[dayoy] = cdiEncodeDate(outyear, month, day);
       }
-
+  */
   ydstatFinalize(stats, operfunc);
-  otsID = 0;
+
+  int otsID = 0;
 
   for ( dayoy = 0; dayoy < NDAY; dayoy++ )
     if ( stats->nsets[dayoy] )
@@ -263,10 +254,10 @@ void *Ydrunstat(void *argument)
 	taxisDefVtime(taxisID2, stats->vtime[dayoy]);
 	streamDefTimestep(streamID2, otsID);
 
-	for ( recID = 0; recID < nrecords; recID++ )
+	for ( int recID = 0; recID < nrecords; recID++ )
 	  {
-	    varID    = recVarID[recID];
-	    levelID  = recLevelID[recID];
+	    varID   = recVarID[recID];
+	    levelID = recLevelID[recID];
 
 	    if ( otsID && vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
 
@@ -285,20 +276,20 @@ void *Ydrunstat(void *argument)
     }
   
   ydstatDestroy(stats);
-  free(vars1);
-  if ( lvarstd ) free(vars2);
+  Free(vars1);
+  if ( lvarstd ) Free(vars2);
 
-  if ( datetime ) free(datetime);
+  if ( datetime ) Free(datetime);
 
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
 
 static
@@ -306,7 +297,7 @@ YDAY_STATS *ydstatCreate(int vlistID)
 {
   int dayoy;
   
-  YDAY_STATS *stats = (YDAY_STATS*) malloc(sizeof(YDAY_STATS));
+  YDAY_STATS *stats = (YDAY_STATS*) Malloc(sizeof(YDAY_STATS));
   
   for ( dayoy = 0; dayoy < NDAY; dayoy++ )
     {
@@ -338,10 +329,10 @@ void ydstatDestroy(YDAY_STATS *stats)
                 {
               	  nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
               	  for ( levelID = 0; levelID < nlevels; levelID++ )
-              	    free(stats->vars1[dayoy][varID][levelID].ptr);
-              	  free(stats->vars1[dayoy][varID]);
+              	    Free(stats->vars1[dayoy][varID][levelID].ptr);
+              	  Free(stats->vars1[dayoy][varID]);
                 }
-              free(stats->vars1[dayoy]);
+              Free(stats->vars1[dayoy]);
             }
           if ( stats->vars2[dayoy] != NULL )
             {
@@ -349,19 +340,19 @@ void ydstatDestroy(YDAY_STATS *stats)
                 {
               	  nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
               	  for ( levelID = 0; levelID < nlevels; levelID++ )
-              	    free(stats->vars2[dayoy][varID][levelID].ptr);
-              	  free(stats->vars2[dayoy][varID]);
+              	    Free(stats->vars2[dayoy][varID][levelID].ptr);
+              	  Free(stats->vars2[dayoy][varID]);
                 }
-              free(stats->vars2[dayoy]);
+              Free(stats->vars2[dayoy]);
             }
         }
-      free(stats);    
+      Free(stats);    
     }
 }
 
 static
 void ydstatUpdate(YDAY_STATS *stats, int vdate, int vtime, 
-		  field_t **vars1, field_t **vars2, int nsets, int operfunc)
+		  field_type **vars1, field_type **vars2, int nsets, int operfunc)
 {
   int varID, levelID, nvars, nlevels;
   int gridsize;
@@ -435,9 +426,7 @@ void ydstatFinalize(YDAY_STATS *stats, int operfunc)
 {
   int varID, levelID, nvars, nlevels;
   int dayoy;
-  double divisor;
-
-  divisor = operfunc == func_std1 || operfunc == func_var1;
+  int divisor = operfunc == func_std1 || operfunc == func_var1;
 
   nvars = vlistNvars(stats->vlist);
   
@@ -453,7 +442,7 @@ void ydstatFinalize(YDAY_STATS *stats, int operfunc)
 	          if ( vlistInqVarTsteptype(stats->vlist, varID) == TSTEP_CONSTANT ) continue;
 	          nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
 	          for ( levelID = 0; levelID < nlevels; levelID++ )
-		    farcmul(&stats->vars1[dayoy][varID][levelID], 1.0 / stats->nsets[dayoy]);
+		    farcdiv(&stats->vars1[dayoy][varID][levelID], (double) stats->nsets[dayoy]);
 	        }
 	      break;
 	      
@@ -464,8 +453,8 @@ void ydstatFinalize(YDAY_STATS *stats, int operfunc)
 	          if ( vlistInqVarTsteptype(stats->vlist, varID) == TSTEP_CONSTANT ) continue;
 	          nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
 	          for ( levelID = 0; levelID < nlevels; levelID++ )
-		    farcstdx(&stats->vars1[dayoy][varID][levelID], stats->vars2[dayoy][varID][levelID],
-			     stats->nsets[dayoy], divisor);
+		    farcstd(&stats->vars1[dayoy][varID][levelID], stats->vars2[dayoy][varID][levelID],
+                            stats->nsets[dayoy], divisor);
 	        }
 	      break;
 	      
@@ -476,7 +465,7 @@ void ydstatFinalize(YDAY_STATS *stats, int operfunc)
 	          if ( vlistInqVarTsteptype(stats->vlist, varID) == TSTEP_CONSTANT ) continue;
 	          nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
 	          for ( levelID = 0; levelID < nlevels; levelID++ )
-		    farcvarx(&stats->vars1[dayoy][varID][levelID], stats->vars2[dayoy][varID][levelID],
+		    farcvar(&stats->vars1[dayoy][varID][levelID], stats->vars2[dayoy][varID][levelID],
 			    stats->nsets[dayoy], divisor);
 	        }
 	      break;
