@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,9 +24,9 @@
       Yhourstat   yhourmean        Multi-year hourly mean
       Yhourstat   yhouravg         Multi-year hourly average
       Yhourstat   yhourvar         Multi-year hourly variance
-      Yhourstat   yhourvar1        Multi-year hourly variance [Divisor is (n-1)]
+      Yhourstat   yhourvar1        Multi-year hourly variance [Normalize by (n-1)]
       Yhourstat   yhourstd         Multi-year hourly standard deviation
-      Yhourstat   yhourstd1        Multi-year hourly standard deviation [Divisor is (n-1)]
+      Yhourstat   yhourstd1        Multi-year hourly standard deviation [Normalize by (n-1)]
 */
 
 #include <cdi.h>
@@ -59,35 +59,23 @@ int hour_of_year(int vdate, int vtime)
       cdoAbort("Hour of year %d out of range (%s %s)!", houroy, vdatestr, vtimestr);
     }
 
-  return (houroy);
+  return houroy;
 }
 
 
 void *Yhourstat(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int gridsize;
   int i;
   int varID;
-  int recID;
   int vdate, vtime;
   int houroy;
-  int nrecs, nrecords;
+  int nrecs;
   int levelID;
-  int tsID;
-  int otsID;
-  long nsets[MAX_HOUR];
-  int streamID1, streamID2;
-  int vlistID1, vlistID2, taxisID1, taxisID2;
+  int nsets[MAX_HOUR];
   int nmiss;
-  int nvars, nlevel;
-  int *recVarID, *recLevelID;
+  int nlevel;
   int vdates[MAX_HOUR], vtimes[MAX_HOUR];
-  int lmean = FALSE, lvarstd = FALSE, lstd = FALSE;
-  double divisor;
-  field_t **vars1[MAX_HOUR], **vars2[MAX_HOUR], **samp1[MAX_HOUR];
-  field_t field;
+  field_type **vars1[MAX_HOUR], **vars2[MAX_HOUR], **samp1[MAX_HOUR];
 
   cdoInitialize(argument);
 
@@ -101,13 +89,13 @@ void *Yhourstat(void *argument)
   cdoOperatorAdd("yhourstd",  func_std,  0, NULL);
   cdoOperatorAdd("yhourstd1", func_std1, 0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
 
-  lmean   = operfunc == func_mean || operfunc == func_avg;
-  lstd    = operfunc == func_std || operfunc == func_std1;
-  lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  divisor = operfunc == func_std1 || operfunc == func_var1;
+  bool lmean   = operfunc == func_mean || operfunc == func_avg;
+  bool lstd    = operfunc == func_std || operfunc == func_std1;
+  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  int divisor = operfunc == func_std1 || operfunc == func_var1;
 
   for ( houroy = 0; houroy < MAX_HOUR; ++houroy )
     {
@@ -117,32 +105,34 @@ void *Yhourstat(void *argument)
       nsets[houroy] = 0;
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   if ( taxisHasBounds(taxisID2) ) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  nvars    = vlistNvars(vlistID1);
-  nrecords = vlistNrecs(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = (int*) malloc(nrecords*sizeof(int));
-  recLevelID = (int*) malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
+
+  field_type field;
   field_init(&field);
-  field.ptr = (double*) malloc(gridsize*sizeof(double));
+  field.ptr = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID = 0;
-  otsID = 0;
+  int tsID = 0;
+  int otsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       vdate = taxisInqVdate(taxisID1);
@@ -163,7 +153,7 @@ void *Yhourstat(void *argument)
 	    vars2[houroy] = field_malloc(vlistID1, FIELD_PTR);
 	}
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -178,12 +168,12 @@ void *Yhourstat(void *argument)
 	  if ( nsets[houroy] == 0 )
 	    {
 	      streamReadRecord(streamID1, vars1[houroy][varID][levelID].ptr, &nmiss);
-	      vars1[houroy][varID][levelID].nmiss = nmiss;
+	      vars1[houroy][varID][levelID].nmiss = (size_t) nmiss;
 
 	      if ( nmiss > 0 || samp1[houroy][varID][levelID].ptr )
 		{
 		  if ( samp1[houroy][varID][levelID].ptr == NULL )
-		    samp1[houroy][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
+		    samp1[houroy][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 
 		  for ( i = 0; i < gridsize; i++ )
 		    if ( DBL_IS_EQUAL(vars1[houroy][varID][levelID].ptr[i],
@@ -195,7 +185,8 @@ void *Yhourstat(void *argument)
 	    }
 	  else
 	    {
-	      streamReadRecord(streamID1, field.ptr, &field.nmiss);
+	      streamReadRecord(streamID1, field.ptr, &nmiss);
+              field.nmiss   = (size_t) nmiss;
 	      field.grid    = vars1[houroy][varID][levelID].grid;
 	      field.missval = vars1[houroy][varID][levelID].missval;
 
@@ -203,7 +194,7 @@ void *Yhourstat(void *argument)
 		{
 		  if ( samp1[houroy][varID][levelID].ptr == NULL )
 		    {
-		      samp1[houroy][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
+		      samp1[houroy][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 		      for ( i = 0; i < gridsize; i++ )
 			samp1[houroy][varID][levelID].ptr[i] = nsets[houroy];
 		    }
@@ -229,7 +220,6 @@ void *Yhourstat(void *argument)
 	for ( varID = 0; varID < nvars; varID++ )
 	  {
 	    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
-	    gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
 	    nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	    for ( levelID = 0; levelID < nlevel; levelID++ )
 	      farmoq(&vars2[houroy][varID][levelID], vars1[houroy][varID][levelID]);
@@ -250,7 +240,7 @@ void *Yhourstat(void *argument)
 	      for ( levelID = 0; levelID < nlevel; levelID++ )
 		{
 		  if ( samp1[houroy][varID][levelID].ptr == NULL )
-		    farcmul(&vars1[houroy][varID][levelID], 1.0/nsets[houroy]);
+		    farcdiv(&vars1[houroy][varID][levelID], (double)nsets[houroy]);
 		  else
 		    fardiv(&vars1[houroy][varID][levelID], samp1[houroy][varID][levelID]);
 		}
@@ -265,16 +255,16 @@ void *Yhourstat(void *argument)
 		  if ( samp1[houroy][varID][levelID].ptr == NULL )
 		    {
 		      if ( lstd )
-			farcstdx(&vars1[houroy][varID][levelID], vars2[houroy][varID][levelID], nsets[houroy], divisor);
+			farcstd(&vars1[houroy][varID][levelID], vars2[houroy][varID][levelID], nsets[houroy], divisor);
 		      else
-			farcvarx(&vars1[houroy][varID][levelID], vars2[houroy][varID][levelID], nsets[houroy], divisor);
+			farcvar(&vars1[houroy][varID][levelID], vars2[houroy][varID][levelID], nsets[houroy], divisor);
 		    }
 		  else
 		    {
 		      if ( lstd )
-			farstdx(&vars1[houroy][varID][levelID], vars2[houroy][varID][levelID], samp1[houroy][varID][levelID], divisor);
+			farstd(&vars1[houroy][varID][levelID], vars2[houroy][varID][levelID], samp1[houroy][varID][levelID], divisor);
 		      else
-			farvarx(&vars1[houroy][varID][levelID], vars2[houroy][varID][levelID], samp1[houroy][varID][levelID], divisor);
+			farvar(&vars1[houroy][varID][levelID], vars2[houroy][varID][levelID], samp1[houroy][varID][levelID], divisor);
 		    }
 		}
 	    }
@@ -283,7 +273,7 @@ void *Yhourstat(void *argument)
 	taxisDefVtime(taxisID2, vtimes[houroy]);
 	streamDefTimestep(streamID2, otsID);
 
-	for ( recID = 0; recID < nrecords; recID++ )
+	for ( int recID = 0; recID < nrecords; recID++ )
 	  {
 	    varID    = recVarID[recID];
 	    levelID  = recLevelID[recID];
@@ -292,7 +282,7 @@ void *Yhourstat(void *argument)
 
 	    streamDefRecord(streamID2, varID, levelID);
 	    streamWriteRecord(streamID2, vars1[houroy][varID][levelID].ptr,
-			      vars1[houroy][varID][levelID].nmiss);
+			      (int)vars1[houroy][varID][levelID].nmiss);
 	  }
 
 	otsID++;
@@ -308,15 +298,15 @@ void *Yhourstat(void *argument)
 	}
     }
 
-  if ( field.ptr ) free(field.ptr);
+  if ( field.ptr ) Free(field.ptr);
 
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
