@@ -25,43 +25,40 @@
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
-#include "percentiles.h"
+#include "percentiles_hist.h"
 
 #define  NMONTH     17
 
+static
+int getmonth(int date)
+{
+  int year, month, day;
+  cdiDecodeDate(date, &year, &month, &day);
+  return month;
+}
+
+
 void *Ymonpctl(void *argument)
 {
-  int gridsize;
   int varID;
-  int recID;
   int gridID;
   int vdate, vtime;
   int year, month, day;
-  int nrecs, nrecords;
   int levelID;
-  int tsID;
-  int otsID;
-  long nsets[NMONTH];
-  int streamID1, streamID2, streamID3, streamID4;
-  int vlistID1, vlistID2, vlistID3, vlistID4, taxisID1, taxisID2, taxisID3, taxisID4;
   int nmiss;
-  int nvars, nlevels;
-  int *recVarID, *recLevelID;
+  int nrecs, nlevels;
   int vdates1[NMONTH], vtimes1[NMONTH];
-  int vdates2[NMONTH], vtimes2[NMONTH];
-  field_t **vars1[NMONTH];
-  field_t field;
-  double pn;
+  int vdates2[NMONTH];
+  long nsets[NMONTH];
+  field_type **vars1[NMONTH];
   HISTOGRAM_SET *hsets[NMONTH];
 
   cdoInitialize(argument);
   cdoOperatorAdd("ymonpctl", func_pctl, 0, NULL);
 
   operatorInputArg("percentile number");
-  pn = atof(operatorArgv()[0]);
-      
-  if ( !(pn > 0 && pn < 100) )
-    cdoAbort("Illegal argument: percentile number %g is not in the range 0..100!", pn);
+  double pn = parameter2double(operatorArgv()[0]);
+  percentile_check_number(pn);
 
   for ( month = 0; month < NMONTH; month++ )
     {
@@ -70,42 +67,44 @@ void *Ymonpctl(void *argument)
       nsets[month] = 0;
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
-  streamID3 = streamOpenRead(cdoStreamName(2));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID3 = streamOpenRead(cdoStreamName(2));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistID3 = streamInqVlist(streamID3);
-  vlistID4 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistID3 = streamInqVlist(streamID3);
+  int vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = vlistInqTaxis(vlistID2);
-  taxisID3 = vlistInqTaxis(vlistID3);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = vlistInqTaxis(vlistID2);
+  int taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  taxisID4 = taxisDuplicate(taxisID1);
+  int taxisID4 = taxisDuplicate(taxisID1);
   if ( taxisHasBounds(taxisID4) ) taxisDeleteBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  streamID4 = streamOpenWrite(cdoStreamName(3), cdoFiletype());
+  int streamID4 = streamOpenWrite(cdoStreamName(3), cdoFiletype());
 
   streamDefVlist(streamID4, vlistID4);
 
-  nvars    = vlistNvars(vlistID1);
-  nrecords = vlistNrecs(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = malloc(nrecords*sizeof(int));
-  recLevelID = malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
+
+  field_type field;
   field_init(&field);
-  field.ptr = malloc(gridsize*sizeof(double));
+  field.ptr = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID2, tsID)) )
     {
       if ( nrecs != streamInqTimestep(streamID3, tsID) )
@@ -114,7 +113,7 @@ void *Ymonpctl(void *argument)
       vdate = taxisInqVdate(taxisID2);
       vtime = taxisInqVtime(taxisID2);
       
-      if ( vdate != taxisInqVdate(taxisID3) || vtime != taxisInqVtime(taxisID3) )
+      if ( vdate != taxisInqVdate(taxisID3) )
         cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args);
         
       if ( cdoVerbose ) cdoPrint("process timestep: %d %d %d", tsID+1, vdate, vtime);
@@ -124,7 +123,6 @@ void *Ymonpctl(void *argument)
 	cdoAbort("Month %d out of range!", month);
 
       vdates2[month] = vdate;
-      vtimes2[month] = vtime;
 
       if ( vars1[month] == NULL )
 	{
@@ -140,13 +138,13 @@ void *Ymonpctl(void *argument)
 	    }
 	}
       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
         {
           streamInqRecord(streamID2, &varID, &levelID);
 	  streamReadRecord(streamID2, vars1[month][varID][levelID].ptr, &nmiss);
           vars1[month][varID][levelID].nmiss = nmiss;
         }
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
         {
           streamInqRecord(streamID3, &varID, &levelID);
 	  streamReadRecord(streamID3, field.ptr, &nmiss);
@@ -178,7 +176,7 @@ void *Ymonpctl(void *argument)
       if ( vars1[month] == NULL )
         cdoAbort("No data for month %d in %s and %s", month, cdoStreamName(1)->args, cdoStreamName(2)->args);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -198,14 +196,13 @@ void *Ymonpctl(void *argument)
       tsID++;
     }
 
-  otsID = 0;
+  int otsID = 0;
   for ( month = 0; month < NMONTH; month++ )
     if ( nsets[month] )
       {
-        if ( vdates1[month] != vdates2[month] )
-          cdoAbort("Verification dates for month %d of %s, %s and %s are different!", month, cdoStreamName(1)->args, cdoStreamName(2)->args, cdoStreamName(3)->args);
-        if ( vtimes1[month] != vtimes2[month] )
-          cdoAbort("Verification times for month %d of %s, %s and %s are different!", month, cdoStreamName(1)->args, cdoStreamName(2)->args, cdoStreamName(3)->args);
+        if ( getmonth(vdates1[month]) != getmonth(vdates2[month]) )
+          cdoAbort("Verification dates for the month %d of %s and %s are different!",
+                   month, cdoStreamName(0)->args, cdoStreamName(1)->args);
 
 	for ( varID = 0; varID < nvars; varID++ )
 	  {
@@ -220,7 +217,7 @@ void *Ymonpctl(void *argument)
 	taxisDefVtime(taxisID4, vtimes1[month]);
 	streamDefTimestep(streamID4, otsID);
 
-	for ( recID = 0; recID < nrecords; recID++ )
+	for ( int recID = 0; recID < nrecords; recID++ )
 	  {
 	    varID    = recVarID[recID];
 	    levelID  = recLevelID[recID];
@@ -243,10 +240,10 @@ void *Ymonpctl(void *argument)
 	}
     }
 
-  if ( field.ptr ) free(field.ptr);
+  if ( field.ptr ) Free(field.ptr);
 
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
 
   streamClose(streamID4);
   streamClose(streamID3);
@@ -255,5 +252,5 @@ void *Ymonpctl(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
