@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,9 @@
       Yseasstat  yseasmean       Multi-year seasonally mean
       Yseasstat  yseasavg        Multi-year seasonally average
       Yseasstat  yseasvar        Multi-year seasonally variance
+      Yseasstat  yseasvar1       Multi-year seasonally variance [Normalize by (n-1)]
       Yseasstat  yseasstd        Multi-year seasonally standard deviation
+      Yseasstat  yseasstd1       Multi-year seasonally standard deviation [Normalize by (n-1)]
 */
 
 #include <cdi.h>
@@ -42,12 +44,9 @@ typedef struct {
 }
 date_time_t;
 
-
-static 
 void set_date(int vdate_new, int vtime_new, date_time_t *datetime)
 {
   int year, month, day;
-
   cdiDecodeDate(vdate_new, &year, &month, &day);
   if ( month == 12 ) vdate_new = cdiEncodeDate(year-1, month, day);
 
@@ -61,28 +60,17 @@ void set_date(int vdate_new, int vtime_new, date_time_t *datetime)
 
 void *Yseasstat(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int gridsize;
   int i;
   int varID;
-  int recID;
   int vdate, vtime;
   int year, month, day, seas;
-  int nrecs, nrecords;
+  int nrecs;
   int levelID;
-  int tsID;
-  int otsID;
-  long nsets[NSEAS];
-  int streamID1, streamID2;
-  int vlistID1, vlistID2, taxisID1, taxisID2;
+  int nsets[NSEAS];
   int nmiss;
-  int nvars, nlevel;
-  int *recVarID, *recLevelID;
+  int nlevel;
   date_time_t datetime[NSEAS];
-  field_t **vars1[NSEAS], **vars2[NSEAS], **samp1[NSEAS];
-  field_t field;
-  int season_start;
+  field_type **vars1[NSEAS], **vars2[NSEAS], **samp1[NSEAS];
 
   cdoInitialize(argument);
 
@@ -92,12 +80,13 @@ void *Yseasstat(void *argument)
   cdoOperatorAdd("yseasmean", func_mean, 0, NULL);
   cdoOperatorAdd("yseasavg",  func_avg,  0, NULL);
   cdoOperatorAdd("yseasvar",  func_var,  0, NULL);
+  cdoOperatorAdd("yseasvar1", func_var1, 0, NULL);
   cdoOperatorAdd("yseasstd",  func_std,  0, NULL);
+  cdoOperatorAdd("yseasstd1", func_std1, 0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
 
-  season_start = get_season_start();
   for ( seas = 0; seas < NSEAS; seas++ )
     {
       vars1[seas]  = NULL;
@@ -108,57 +97,46 @@ void *Yseasstat(void *argument)
       datetime[seas].vtime = 0;
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  bool lmean   = operfunc == func_mean || operfunc == func_avg;
+  bool lstd    = operfunc == func_std || operfunc == func_std1;
+  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  int divisor  = operfunc == func_std1 || operfunc == func_var1;
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
+
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   if ( taxisHasBounds(taxisID2) ) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  nvars    = vlistNvars(vlistID1);
-  nrecords = vlistNrecs(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = (int*) malloc(nrecords*sizeof(int));
-  recLevelID = (int*) malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
+
+  field_type field;
   field_init(&field);
-  field.ptr = (double*) malloc(gridsize*sizeof(double));
+  field.ptr = (double*) Malloc(gridsize*sizeof(double));
 
-  tsID = 0;
-  otsID = 0;
+  int tsID = 0;
+  int otsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       vdate = taxisInqVdate(taxisID1);
       vtime = taxisInqVtime(taxisID1);
       cdiDecodeDate(vdate, &year, &month, &day);
-      if ( month < 0 || month > 16 )
-	cdoAbort("Month %d out of range!", month);
 
-      if ( season_start == START_DEC )
-	{
-	  if ( month <= 12 )
-	    seas = (month % 12) / 3;
-	  else
-	    seas = month - 13;
-	}
-      else
-	{
-	  if ( month <= 12 )
-	    seas = (month - 1) / 3;
-	  else
-	    seas = month - 13;
-	}
-
-      if ( seas < 0 || seas > 3 )
-	cdoAbort("Season %d out of range!", seas+1);
+      seas = month_to_season(month); 
 
       set_date(vdate, vtime, &datetime[seas]);
 
@@ -166,11 +144,11 @@ void *Yseasstat(void *argument)
 	{
 	  vars1[seas] = field_malloc(vlistID1, FIELD_PTR);
 	  samp1[seas] = field_malloc(vlistID1, FIELD_NONE);
-	  if ( operfunc == func_std || operfunc == func_var )
+	  if ( lvarstd )
 	    vars2[seas] = field_malloc(vlistID1, FIELD_PTR);
 	}
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
@@ -185,12 +163,12 @@ void *Yseasstat(void *argument)
 	  if ( nsets[seas] == 0 )
 	    {
 	      streamReadRecord(streamID1, vars1[seas][varID][levelID].ptr, &nmiss);
-	      vars1[seas][varID][levelID].nmiss = nmiss;
+	      vars1[seas][varID][levelID].nmiss = (size_t)nmiss;
 
 	      if ( nmiss > 0 || samp1[seas][varID][levelID].ptr )
 		{
 		  if ( samp1[seas][varID][levelID].ptr == NULL )
-		    samp1[seas][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
+		    samp1[seas][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 
 		  for ( i = 0; i < gridsize; i++ )
 		    if ( DBL_IS_EQUAL(vars1[seas][varID][levelID].ptr[i],
@@ -202,7 +180,8 @@ void *Yseasstat(void *argument)
 	    }
 	  else
 	    {
-	      streamReadRecord(streamID1, field.ptr, &field.nmiss);
+	      streamReadRecord(streamID1, field.ptr, &nmiss);
+              field.nmiss   = (size_t)nmiss;
 	      field.grid    = vars1[seas][varID][levelID].grid;
 	      field.missval = vars1[seas][varID][levelID].missval;
 
@@ -210,7 +189,7 @@ void *Yseasstat(void *argument)
 		{
 		  if ( samp1[seas][varID][levelID].ptr == NULL )
 		    {
-		      samp1[seas][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
+		      samp1[seas][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 		      for ( i = 0; i < gridsize; i++ )
 			samp1[seas][varID][levelID].ptr[i] = nsets[seas];
 		    }
@@ -220,7 +199,7 @@ void *Yseasstat(void *argument)
 		      samp1[seas][varID][levelID].ptr[i]++;
 		}
 
-	      if ( operfunc == func_std || operfunc == func_var )
+	      if ( lvarstd )
 		{
 		  farsumq(&vars2[seas][varID][levelID], field);
 		  farsum(&vars1[seas][varID][levelID], field);
@@ -232,11 +211,10 @@ void *Yseasstat(void *argument)
 	    }
 	}
 
-      if ( nsets[seas] == 0 && (operfunc == func_std || operfunc == func_var) )
+      if ( nsets[seas] == 0 && lvarstd )
 	for ( varID = 0; varID < nvars; varID++ )
 	  {
 	    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
-	    gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
 	    nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	    for ( levelID = 0; levelID < nlevel; levelID++ )
 	      farmoq(&vars2[seas][varID][levelID], vars1[seas][varID][levelID]);
@@ -249,40 +227,39 @@ void *Yseasstat(void *argument)
   for ( seas = 0; seas < NSEAS; seas++ )
     if ( nsets[seas] )
       {
-	if ( operfunc == func_mean || operfunc == func_avg )
+	if ( lmean )
 	  for ( varID = 0; varID < nvars; varID++ )
 	    {
 	      if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
-	      nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+	      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	      for ( levelID = 0; levelID < nlevel; levelID++ )
 		{
 		  if ( samp1[seas][varID][levelID].ptr == NULL )
-		    farcmul(&vars1[seas][varID][levelID], 1.0/nsets[seas]);
+		    farcdiv(&vars1[seas][varID][levelID], (double)nsets[seas]);
 		  else
 		    fardiv(&vars1[seas][varID][levelID], samp1[seas][varID][levelID]);
 		}
 	    }
-	else if ( operfunc == func_std || operfunc == func_var )
+	else if ( lvarstd )
 	  for ( varID = 0; varID < nvars; varID++ )
 	    {
 	      if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
-	      nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+	      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	      for ( levelID = 0; levelID < nlevel; levelID++ )
 		{
 		  if ( samp1[seas][varID][levelID].ptr == NULL )
 		    {
-		      if ( operfunc == func_std )
-			farcstd(&vars1[seas][varID][levelID], vars2[seas][varID][levelID], 1.0/nsets[seas]);
+		      if ( lstd )
+			farcstd(&vars1[seas][varID][levelID], vars2[seas][varID][levelID], nsets[seas], divisor);
 		      else
-			farcvar(&vars1[seas][varID][levelID], vars2[seas][varID][levelID], 1.0/nsets[seas]);
+			farcvar(&vars1[seas][varID][levelID], vars2[seas][varID][levelID], nsets[seas], divisor);
 		    }
 		  else
 		    {
-		      farinv(&samp1[seas][varID][levelID]);
-		      if ( operfunc == func_std )
-			farstd(&vars1[seas][varID][levelID], vars2[seas][varID][levelID], samp1[seas][varID][levelID]);
+		      if ( lstd )
+			farstd(&vars1[seas][varID][levelID], vars2[seas][varID][levelID], samp1[seas][varID][levelID], divisor);
 		      else
-			farvar(&vars1[seas][varID][levelID], vars2[seas][varID][levelID], samp1[seas][varID][levelID]);
+			farvar(&vars1[seas][varID][levelID], vars2[seas][varID][levelID], samp1[seas][varID][levelID], divisor);
 		    }
 		}
 	    }
@@ -291,7 +268,7 @@ void *Yseasstat(void *argument)
 	taxisDefVtime(taxisID2, datetime[seas].vtime);
 	streamDefTimestep(streamID2, otsID);
 
-	for ( recID = 0; recID < nrecords; recID++ )
+	for ( int recID = 0; recID < nrecords; recID++ )
 	  {
 	    varID    = recVarID[recID];
 	    levelID  = recLevelID[recID];
@@ -300,7 +277,7 @@ void *Yseasstat(void *argument)
 
 	    streamDefRecord(streamID2, varID, levelID);
 	    streamWriteRecord(streamID2, vars1[seas][varID][levelID].ptr,
-			      vars1[seas][varID][levelID].nmiss);
+			      (int)vars1[seas][varID][levelID].nmiss);
 	  }
 
 	otsID++;
@@ -312,19 +289,19 @@ void *Yseasstat(void *argument)
 	{
 	  field_free(vars1[seas], vlistID1);
 	  field_free(samp1[seas], vlistID1);
-	  if ( operfunc == func_std || operfunc == func_var ) free(vars2[seas]);
+	  if ( lvarstd ) Free(vars2[seas]);
 	}
     }
 
-  if ( field.ptr ) free(field.ptr);
+  if ( field.ptr ) Free(field.ptr);
 
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
