@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.1
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,9 @@
       Zonstat    zonmean         Zonal mean
       Zonstat    zonavg          Zonal average
       Zonstat    zonstd          Zonal standard deviation
+      Zonstat    zonstd1         Zonal standard deviation [Normalize by (n-1)]
       Zonstat    zonvar          Zonal variance
+      Zonstat    zonvar1         Zonal variance [Normalize by (n-1)]
       Zonstat    zonpctl         Zonal percentiles
 */
 
@@ -33,29 +35,18 @@
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
-#include "grid.h"
 #include "pstream.h"
+#include "grid.h"
 
 
 void *Zonstat(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
   int gridID1 = -1, gridID2 = -1;
   int zongridID = -1;
-  int nlatmax;
-  int index, ngrids;
-  int recID, nrecs;
-  int tsID, varID, levelID;
-  int lim;
-  int ndiffgrids;
-  int taxisID1, taxisID2;
-  field_t field1, field2;
-  /* RQ */
-  int pn = 0;
-  /* QR */
+  int index;
+  int nmiss;
+  int nrecs;
+  int varID, levelID;
 
   cdoInitialize(argument);
 
@@ -66,31 +57,32 @@ void *Zonstat(void *argument)
   cdoOperatorAdd("zonmean",  func_mean,  0, NULL);
   cdoOperatorAdd("zonavg",   func_avg,   0, NULL);
   cdoOperatorAdd("zonvar",   func_var,   0, NULL);
+  cdoOperatorAdd("zonvar1",  func_var1,  0, NULL);
   cdoOperatorAdd("zonstd",   func_std,   0, NULL);
+  cdoOperatorAdd("zonstd1",  func_std1,  0, NULL);
   cdoOperatorAdd("zonpctl",  func_pctl,  0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
 
+  double pn = 0;
   if ( operfunc == func_pctl )
     {
       operatorInputArg("percentile number");
-      pn = parameter2int(operatorArgv()[0]);
-      
-      if ( pn < 1 || pn > 99 )
-        cdoAbort("Illegal argument: percentile number %d is not in the range 1..99!", pn);
+      pn = parameter2double(operatorArgv()[0]);
+      percentile_check_number(pn);
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  ngrids = vlistNgrids(vlistID1);
+  int ngrids = vlistNgrids(vlistID1);
   for ( index = 0; index < ngrids; index++ )
     {
       if ( gridInqXsize(vlistGrid(vlistID1, index)) > 1 ) 
@@ -103,7 +95,7 @@ void *Zonstat(void *argument)
 	}
     }
 
-  ndiffgrids = 0;
+  int ndiffgrids = 0;
   for ( index = 0; index < ngrids; index++ )
     {
       if ( zongridID != -1 && zongridID == vlistGrid(vlistID1, index) ) continue;
@@ -136,32 +128,35 @@ void *Zonstat(void *argument)
   for ( index = 0; index < ngrids; index++ )
     vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
   gridID1 = vlistInqVarGrid(vlistID1, 0);
-  nlatmax = gridInqYsize(gridID1); /* max nlat ? */
+  int nlatmax = gridInqYsize(gridID1); /* max nlat ? */
 
-  lim = vlistGridsizeMax(vlistID1);
+  int lim = vlistGridsizeMax(vlistID1);
+
+  field_type field1, field2;
   field_init(&field2);
   field_init(&field2);
-  field1.ptr  = (double*) malloc(lim*sizeof(double));
-  field2.ptr  = (double*) malloc(nlatmax*sizeof(double));
+  field1.ptr  = (double*) Malloc(lim*sizeof(double));
+  field2.ptr  = (double*) Malloc(nlatmax*sizeof(double));
   field2.grid = gridID2;
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID);
 
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
-	  streamReadRecord(streamID1, field1.ptr, &field1.nmiss);
+	  streamReadRecord(streamID1, field1.ptr, &nmiss);
 
+          field1.nmiss   = (size_t) nmiss;
 	  field1.grid    = vlistInqVarGrid(vlistID1, varID);
 	  field1.missval = vlistInqVarMissval(vlistID1, varID);
 	  field2.missval = vlistInqVarMissval(vlistID1, varID);
@@ -180,7 +175,7 @@ void *Zonstat(void *argument)
 	    }
 
 	  streamDefRecord(streamID2, varID,  levelID);
-	  streamWriteRecord(streamID2, field2.ptr, field2.nmiss);
+	  streamWriteRecord(streamID2, field2.ptr, (int)field2.nmiss);
 	}
       tsID++;
     }
@@ -188,10 +183,10 @@ void *Zonstat(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( field1.ptr ) free(field1.ptr);
-  if ( field2.ptr ) free(field2.ptr);
+  if ( field1.ptr ) Free(field1.ptr);
+  if ( field2.ptr ) Free(field2.ptr);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
