@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,6 @@
 
 #include <signal.h>
 #include <fenv.h>
-#include <ctype.h>
 /*#include <malloc.h>*/ /* mallopt and malloc_stats */
 #include <sys/stat.h>
 #if defined(HAVE_GETRLIMIT)
@@ -58,7 +57,6 @@
 #endif
 
 #include "modules.h"
-#include "util.h"
 #include "error.h"
 
 #if defined(_OPENMP)
@@ -103,7 +101,7 @@ void gridsearch_set_method(const char *methodstr);
           } \
       }
 
-#define ISME  (strcmp(username, "\x6d\x32\x31\x34\x30\x30\x33") == 0)
+#define ITSME  (strcmp(username, "\x6d\x32\x31\x34\x30\x30\x33") == 0)
 
 static
 void cdo_stackframe(void)
@@ -157,11 +155,30 @@ void cdo_sig_handler(int signo)
     }
 }
 
+static
+void cdo_set_digits(const char *optarg)
+{
+  char *ptr1 = 0;
+  if ( optarg != 0 && (int) strlen(optarg) > 0 && optarg[0] != ',' )
+    CDO_flt_digits = (int)strtol(optarg, &ptr1, 10);
+
+  if ( CDO_flt_digits < 1 || CDO_flt_digits > 20 )
+    cdoAbort("Unreasonable value for float significant digits: %d", CDO_flt_digits);
+
+  if ( ptr1 && *ptr1 == ',' )
+    {
+      char *ptr2 = 0;
+      CDO_dbl_digits = (int)strtol(ptr1+1, &ptr2, 10);
+      if  ( ptr2 == ptr1+1 || CDO_dbl_digits < 1 || CDO_dbl_digits > 20 )
+        cdoAbort("Unreasonable value for double significant digits: %d", CDO_dbl_digits);
+    }
+}
+
 static
 void cdo_version(void)
 {
-  const int   filetypes[] = {FILETYPE_SRV, FILETYPE_EXT, FILETYPE_IEG, FILETYPE_GRB, FILETYPE_GRB2, FILETYPE_NC, FILETYPE_NC2, FILETYPE_NC4, FILETYPE_NC4C};
-  const char* typenames[] = {        "srv",        "ext",        "ieg",        "grb",        "grb2",        "nc",        "nc2",        "nc4",        "nc4c"};
+  const int   filetypes[] = {CDI_FILETYPE_SRV, CDI_FILETYPE_EXT, CDI_FILETYPE_IEG, CDI_FILETYPE_GRB, CDI_FILETYPE_GRB2, CDI_FILETYPE_NC, CDI_FILETYPE_NC2, CDI_FILETYPE_NC4, CDI_FILETYPE_NC4C};
+  const char* typenames[] = {        "srv",        "ext",        "ieg",       "grb1",        "grb2",       "nc1",        "nc2",        "nc4",        "nc4c"};
 
   fprintf(stderr, "%s\n", CDO_Version);
 #if defined(USER_NAME) && defined(HOST_NAME) && defined(SYSTEM_TYPE)
@@ -201,15 +218,14 @@ void cdo_usage(void)
   set_text_color(stderr, RESET, BLUE);
   fprintf(stderr, "    -a             Generate an absolute time axis\n");
   fprintf(stderr, "    -b <nbits>     Set the number of bits for the output precision\n");
-  fprintf(stderr, "                   (I8/I16/I32/F32/F64 for nc/nc2/nc4/nc4c; F32/F64 for grb2/srv/ext/ieg; P1 - P24 for grb/grb2)\n");
+  fprintf(stderr, "                   (I8/I16/I32/F32/F64 for nc1/nc2/nc4/nc4c; F32/F64 for grb2/srv/ext/ieg; P1 - P24 for grb1/grb2)\n");
   fprintf(stderr, "                   Add L or B to set the byteorder to Little or Big endian\n");
-  if ( ISME )
-    {
-      fprintf(stderr, "    --enableexcept <except>\n");
-      fprintf(stderr, "                   Set individual floating-point traps (DIVBYZERO, INEXACT, INVALID, OVERFLOW, UNDERFLOW, ALL_EXCEPT)\n");
-    }
+  fprintf(stderr, "    --cmor         CMOR conform NetCDF output\n");
+  fprintf(stderr, "    -C, --color    Colorized output messages\n");
+  fprintf(stderr, "    --enableexcept <except>\n");
+  fprintf(stderr, "                   Set individual floating-point traps (DIVBYZERO, INEXACT, INVALID, OVERFLOW, UNDERFLOW, ALL_EXCEPT)\n");
   fprintf(stderr, "    -f, --format <format>\n");
-  fprintf(stderr, "                   Format of the output file. (grb/grb2/nc/nc2/nc4/nc4c/srv/ext/ieg)\n");
+  fprintf(stderr, "                   Format of the output file. (grb1/grb2/nc1/nc2/nc4/nc4c/srv/ext/ieg)\n");
   fprintf(stderr, "    -g <grid>      Set default grid name or file. Available grids: \n");
   fprintf(stderr, "                   n<N>, t<RES>, tl<RES>, global_<DXY>, r<NX>x<NY>, g<NX>x<NY>, gme<NI>, lon=<LON>/lat=<LAT>\n");
   fprintf(stderr, "    -h, --help     Help information for the operators\n");
@@ -228,7 +244,7 @@ void cdo_usage(void)
   fprintf(stderr, "    -k <chunktype> NetCDF4 chunk type: auto, grid or lines\n");
   fprintf(stderr, "    -L             Lock IO (sequential access)\n");
   fprintf(stderr, "    -M             Switch to indicate that the I/O streams have missing values\n");
-  fprintf(stderr, "    -m <missval>   Set the default missing value (default: %g)\n", cdiInqMissval());
+  fprintf(stderr, "    -m <missval>   Set the missing value of non NetCDF files (default: %g)\n", cdiInqMissval());
   fprintf(stderr, "    --no_warnings  Inhibit warning messages\n");
   fprintf(stderr, "    -O             Overwrite existing output file, if checked\n");
   fprintf(stderr, "    --operators    List of all operators\n");
@@ -237,20 +253,24 @@ void cdo_usage(void)
 #endif
   fprintf(stderr, "    --percentile <method>\n");
   fprintf(stderr, "                   Percentile method: nrank, nist, numpy, numpy_lower, numpy_higher, numpy_nearest\n");
-  fprintf(stderr, "    -Q             Alphanumeric sorting of NetCDF parameter names\n");
+  fprintf(stderr, "    --precision <float_digits[,double_digits]>\n");
+  fprintf(stderr, "                   Precision to use in displaying floating-point data (default: 7,15)\n");
   fprintf(stderr, "    --reduce_dim   Reduce NetCDF dimensions (module: TIMSTAT, FLDSTAT)\n");
   fprintf(stderr, "    -R, --regular  Convert GRIB1 data from reduced to regular grid (cgribex only)\n");
   fprintf(stderr, "    -r             Generate a relative time axis\n");
   fprintf(stderr, "    -S             Create an extra output stream for the module TIMSTAT. This stream\n");
   fprintf(stderr, "                   contains the number of non missing values for each output period.\n");
   fprintf(stderr, "    -s, --silent   Silent mode\n");
-  fprintf(stderr, "    -t <partab>    Set GRIB1 default parameter table name or file (cgribex only)\n");
+  fprintf(stderr, "    --sort         Alphanumeric sorting of NetCDF parameter names\n");
+  fprintf(stderr, "    -t <codetab>   Set GRIB1 default parameter code table name or file (cgribex only)\n");
   fprintf(stderr, "                   Predefined tables: ");
   for ( int id = 0; id < tableInqNumber(); id++ )
     if ( (name = tableInqNamePtr(id)) )
       fprintf(stderr, " %s", name);
   fprintf(stderr, "\n");
 
+  fprintf(stderr, "    --timestat_date <srcdate>\n");
+  fprintf(stderr, "                   Target timestamp (time statistics): first, middle, midhigh or last source timestep.\n");
   fprintf(stderr, "    -V, --version  Print the version number\n");
   fprintf(stderr, "    -v, --verbose  Print extra details for some operators\n");
   fprintf(stderr, "    -W             Print extra warning messages\n");
@@ -269,7 +289,7 @@ void cdo_usage(void)
   */
 
   fprintf(stderr, "\n");
-  fprintf(stderr, "  CDO version %s, Copyright (C) 2003-2016 Uwe Schulzweida\n", VERSION);
+  fprintf(stderr, "  CDO version %s, Copyright (C) 2003-2017 Uwe Schulzweida\n", VERSION);
   //  fprintf(stderr, "  Available from <http://mpimet.mpg.de/cdo>\n");
   fprintf(stderr, "  This is free software and comes with ABSOLUTELY NO WARRANTY\n");
   fprintf(stderr, "  Report bugs to <http://mpimet.mpg.de/cdo>\n");
@@ -294,13 +314,10 @@ void cdoPrintHelp(const char *phelp[]/*, char *xoperator*/)
     fprintf(stderr, "No help available for this operator!\n");
   else
     {
-      int lprint;
+      bool lprint;
       while ( *phelp )
         {
-          lprint = TRUE;
-          if ( *phelp[0] == '\0' )
-            if ( *(phelp+1) )
-              if ( *(phelp+1)[0] == ' ' ) lprint = FALSE;
+          lprint = !(*phelp[0] == '\0' && *(phelp+1) && *(phelp+1)[0] == ' ');
           
           if ( lprint )
             {
@@ -371,54 +388,34 @@ void setDefaultDataType(const char *datatypestr)
   enum {D_UINT, D_INT, D_FLT, D_CPX};
   int dtype = -1;
 
-  if      ( *datatypestr == 'i' || *datatypestr == 'I' )
-    {
-      dtype = D_INT;
-      datatypestr++;
-    }
-  else if ( *datatypestr == 'u' || *datatypestr == 'U' )
-    {
-      dtype = D_UINT;
-      datatypestr++;
-    }
-  else if ( *datatypestr == 'f' || *datatypestr == 'F' )
-    {
-      dtype = D_FLT;
-      datatypestr++;
-    }
-  else if ( *datatypestr == 'c' || *datatypestr == 'C' )
-    {
-      dtype = D_CPX;
-      datatypestr++;
-    }
-  else if ( *datatypestr == 'p' || *datatypestr == 'P' )
-    {
-      datatypestr++;
-    }
+  int datatype = tolower(*datatypestr);
+  if      ( datatype == 'i' ) { dtype = D_INT;  datatypestr++; }
+  else if ( datatype == 'u' ) { dtype = D_UINT; datatypestr++; }
+  else if ( datatype == 'f' ) { dtype = D_FLT;  datatypestr++; }
+  else if ( datatype == 'c' ) { dtype = D_CPX;  datatypestr++; }
+  else if ( datatype == 'p' ) { datatypestr++; }
 
   if ( isdigit((int) *datatypestr) )
     {
       nbits = atoi(datatypestr);
-      if ( nbits < 10 )
-        datatypestr += 1;
-      else
-        datatypestr += 2;
+      datatypestr += 1;
+      if ( nbits >= 10 ) datatypestr += 1;
 
       if ( dtype == -1 )
         {
           if      ( nbits > 0 && nbits < 32 ) cdoDefaultDataType = nbits;
           else if ( nbits == 32 )
             {
-              if ( cdoDefaultFileType == FILETYPE_GRB )
-                cdoDefaultDataType = DATATYPE_PACK32;
+              if ( cdoDefaultFileType == CDI_FILETYPE_GRB )
+                cdoDefaultDataType = CDI_DATATYPE_PACK32;
               else
-                cdoDefaultDataType = DATATYPE_FLT32;
+                cdoDefaultDataType = CDI_DATATYPE_FLT32;
             }
-          else if ( nbits == 64 ) cdoDefaultDataType = DATATYPE_FLT64;
+          else if ( nbits == 64 ) cdoDefaultDataType = CDI_DATATYPE_FLT64;
           else
             {
               fprintf(stderr, "Unsupported number of bits %d!\n", nbits);
-              fprintf(stderr, "Use I8/I16/I32/F32/F64 for nc/nc2/nc4/nc4c; F32/F64 for grb2/srv/ext/ieg; P1 - P24 for grb/grb2.\n");
+              fprintf(stderr, "Use I8/I16/I32/F32/F64 for nc1/nc2/nc4/nc4c; F32/F64 for grb2/srv/ext/ieg; P1 - P24 for grb1/grb2.\n");
               exit(EXIT_FAILURE);
             }
         }
@@ -426,9 +423,9 @@ void setDefaultDataType(const char *datatypestr)
         {
           if ( dtype == D_INT )
             {
-              if      ( nbits ==  8 ) cdoDefaultDataType = DATATYPE_INT8;
-              else if ( nbits == 16 ) cdoDefaultDataType = DATATYPE_INT16;
-              else if ( nbits == 32 ) cdoDefaultDataType = DATATYPE_INT32;
+              if      ( nbits ==  8 ) cdoDefaultDataType = CDI_DATATYPE_INT8;
+              else if ( nbits == 16 ) cdoDefaultDataType = CDI_DATATYPE_INT16;
+              else if ( nbits == 32 ) cdoDefaultDataType = CDI_DATATYPE_INT32;
               else
                 {
                   fprintf(stderr, "Unsupported number of bits = %d for datatype INT!\n", nbits);
@@ -437,9 +434,9 @@ void setDefaultDataType(const char *datatypestr)
             }
           else if ( dtype == D_UINT )
             {
-              if      ( nbits ==  8 ) cdoDefaultDataType = DATATYPE_UINT8;
-              else if ( nbits == 16 ) cdoDefaultDataType = DATATYPE_UINT16;
-              else if ( nbits == 32 ) cdoDefaultDataType = DATATYPE_UINT32;
+              if      ( nbits ==  8 ) cdoDefaultDataType = CDI_DATATYPE_UINT8;
+              else if ( nbits == 16 ) cdoDefaultDataType = CDI_DATATYPE_UINT16;
+              else if ( nbits == 32 ) cdoDefaultDataType = CDI_DATATYPE_UINT32;
               else
                 {
                   fprintf(stderr, "Unsupported number of bits = %d for datatype UINT!\n", nbits);
@@ -448,8 +445,8 @@ void setDefaultDataType(const char *datatypestr)
             }
           else if ( dtype == D_FLT )
             {
-              if      ( nbits == 32 ) cdoDefaultDataType = DATATYPE_FLT32;
-              else if ( nbits == 64 ) cdoDefaultDataType = DATATYPE_FLT64;
+              if      ( nbits == 32 ) cdoDefaultDataType = CDI_DATATYPE_FLT32;
+              else if ( nbits == 64 ) cdoDefaultDataType = CDI_DATATYPE_FLT64;
               else
                 {
                   fprintf(stderr, "Unsupported number of bits = %d for datatype FLT!\n", nbits);
@@ -458,8 +455,8 @@ void setDefaultDataType(const char *datatypestr)
             }
           else if ( dtype == D_CPX )
             {
-              if      ( nbits == 32 ) cdoDefaultDataType = DATATYPE_CPX32;
-              else if ( nbits == 64 ) cdoDefaultDataType = DATATYPE_CPX64;
+              if      ( nbits == 32 ) cdoDefaultDataType = CDI_DATATYPE_CPX32;
+              else if ( nbits == 64 ) cdoDefaultDataType = CDI_DATATYPE_CPX64;
               else
                 {
                   fprintf(stderr, "Unsupported number of bits = %d for datatype CPX!\n", nbits);
@@ -500,15 +497,15 @@ void setDefaultDataTypeByte(char *datatypestr)
       datatype = atoi(datatypestr);
       datatypestr++;
 
-      if      ( datatype == 1 ) cdoDefaultDataType = DATATYPE_PACK8;
-      else if ( datatype == 2 ) cdoDefaultDataType = DATATYPE_PACK16;
-      else if ( datatype == 3 ) cdoDefaultDataType = DATATYPE_PACK24;
-      else if ( datatype == 4 ) cdoDefaultDataType = DATATYPE_FLT32;
-      else if ( datatype == 8 ) cdoDefaultDataType = DATATYPE_FLT64;
+      if      ( datatype == 1 ) cdoDefaultDataType = CDI_DATATYPE_PACK8;
+      else if ( datatype == 2 ) cdoDefaultDataType = CDI_DATATYPE_PACK16;
+      else if ( datatype == 3 ) cdoDefaultDataType = CDI_DATATYPE_PACK24;
+      else if ( datatype == 4 ) cdoDefaultDataType = CDI_DATATYPE_FLT32;
+      else if ( datatype == 8 ) cdoDefaultDataType = CDI_DATATYPE_FLT64;
       else
         {
           fprintf(stderr, "Unsupported datatype %d!\n", datatype);
-          fprintf(stderr, "Use 4/8 for filetype nc/srv/ext/ieg and 1/2/3 for grb/grb2.\n");
+          fprintf(stderr, "Use 4/8 for filetype nc/srv/ext/ieg and 1/2/3 for grb1/grb2.\n");
           exit(EXIT_FAILURE);
         }
     }
@@ -541,22 +538,23 @@ void setDefaultFileType(const char *filetypestr, int labort)
       const char *ftstr = filetypestr;
       size_t len;
 
-      if      ( cmpstrlen(filetypestr, "grb2", len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_GRB2;}
-      else if ( cmpstrlen(filetypestr, "grb1", len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_GRB; }
-      else if ( cmpstrlen(filetypestr, "grb",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_GRB; }
-      else if ( cmpstrlen(filetypestr, "nc2",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_NC2; }
-      else if ( cmpstrlen(filetypestr, "nc4c", len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_NC4C;}
-      else if ( cmpstrlen(filetypestr, "nc4",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_NC4; }
-      else if ( cmpstrlen(filetypestr, "nc",   len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_NC;  }
-      else if ( cmpstrlen(filetypestr, "srv",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_SRV; }
-      else if ( cmpstrlen(filetypestr, "ext",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_EXT; }
-      else if ( cmpstrlen(filetypestr, "ieg",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = FILETYPE_IEG; }
+      if      ( cmpstrlen(filetypestr, "grb2", len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_GRB2;}
+      else if ( cmpstrlen(filetypestr, "grb1", len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_GRB; }
+      else if ( cmpstrlen(filetypestr, "grb",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_GRB; }
+      else if ( cmpstrlen(filetypestr, "nc2",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_NC2; }
+      else if ( cmpstrlen(filetypestr, "nc4c", len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_NC4C;}
+      else if ( cmpstrlen(filetypestr, "nc4",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_NC4; }
+      else if ( cmpstrlen(filetypestr, "nc1",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_NC;  }
+      else if ( cmpstrlen(filetypestr, "nc",   len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_NC2; }
+      else if ( cmpstrlen(filetypestr, "srv",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_SRV; }
+      else if ( cmpstrlen(filetypestr, "ext",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_EXT; }
+      else if ( cmpstrlen(filetypestr, "ieg",  len)  == 0 ) { ftstr += len; cdoDefaultFileType = CDI_FILETYPE_IEG; }
       else
         {
           if ( labort )
             {
               fprintf(stderr, "Unsupported filetype %s!\n", filetypestr);
-              fprintf(stderr, "Available filetypes: grb/grb2/nc/nc2/nc4/nc4c/srv/ext/ieg\n");
+              fprintf(stderr, "Available filetypes: grb1/grb2/nc1/nc2/nc4/nc4c/srv/ext/ieg\n");
               exit(EXIT_FAILURE);
             }
           else
@@ -577,8 +575,8 @@ void setDefaultFileType(const char *filetypestr, int labort)
             {
               fprintf(stderr, "Unexpected character >%c< in file type >%s<!\n", *ftstr, filetypestr);
               fprintf(stderr, "Use format[_nbits] with:\n");
-              fprintf(stderr, "    format = grb, grb2, nc, nc2, nc4, nc4c, srv, ext or ieg\n");
-              fprintf(stderr, "    nbits  = 32/64 for grb2/nc/nc2/nc4/nc4c/srv/ext/ieg; 1 - 24 for grb/grb2\n");
+              fprintf(stderr, "    format = grb1, grb2, nc1, nc2, nc4, nc4c, srv, ext or ieg\n");
+              fprintf(stderr, "    nbits  = 32/64 for grb2/nc1/nc2/nc4/nc4c/srv/ext/ieg; 1 - 24 for grb1/grb2\n");
               exit(EXIT_FAILURE);
             }
         }
@@ -626,22 +624,22 @@ void defineCompress(const char *arg)
 
   if      ( strncmp(arg, "szip", len) == 0 )
     {
-      cdoCompType  = COMPRESS_SZIP;
+      cdoCompType  = CDI_COMPRESS_SZIP;
       cdoCompLevel = 0;
     }
   else if ( strncmp(arg, "jpeg", len) == 0 )
     {
-      cdoCompType = COMPRESS_JPEG;
+      cdoCompType = CDI_COMPRESS_JPEG;
       cdoCompLevel = 0;
     }
   else if ( strncmp(arg, "gzip", len) == 0 )
     {
-      cdoCompType  = COMPRESS_GZIP;
+      cdoCompType  = CDI_COMPRESS_GZIP;
       cdoCompLevel = 6;
     }
   else if ( strncmp(arg, "zip", 3) == 0 )
     {
-      cdoCompType  = COMPRESS_ZIP;
+      cdoCompType  = CDI_COMPRESS_ZIP;
       if ( len == 5 && arg[3] == '_' && isdigit(arg[4]) )
         cdoCompLevel = atoi(&arg[4]);
       else
@@ -657,9 +655,9 @@ void defineCompress(const char *arg)
 static
 void defineChunktype(const char *arg)
 {
-  if      ( strcmp("auto",  arg)   == 0 ) cdoChunkType = CHUNK_AUTO;
-  else if ( strcmp("grid",  arg)   == 0 ) cdoChunkType = CHUNK_GRID;
-  else if ( strcmp("lines", arg)   == 0 ) cdoChunkType = CHUNK_LINES;
+  if      ( strcmp("auto",  arg)   == 0 ) cdoChunkType = CDI_CHUNK_AUTO;
+  else if ( strcmp("grid",  arg)   == 0 ) cdoChunkType = CDI_CHUNK_GRID;
+  else if ( strcmp("lines", arg)   == 0 ) cdoChunkType = CDI_CHUNK_LINES;
   else
     {
       fprintf(stderr, "Chunk type '%s' unsupported!\n", arg);
@@ -705,8 +703,6 @@ void defineVarnames(const char *arg)
 static
 void get_env_vars(void)
 {
-  char *envstr;
-
   username = getenv("LOGNAME");
   if ( username == NULL )
     {
@@ -714,7 +710,7 @@ void get_env_vars(void)
       if ( username == NULL ) username = "unknown";
     }
 
-  envstr = getenv("CDO_GRID_SEARCH_DIR");
+  char *envstr = getenv("CDO_GRID_SEARCH_DIR");
   if ( envstr )
     {
       size_t len = strlen(envstr);
@@ -846,7 +842,7 @@ void get_env_vars(void)
             fprintf(stderr, "CDO_COLOR = %s\n", envstr);
         }
     }
-  else if ( CDO_Color == FALSE && ISME ) CDO_Color = TRUE;
+  else if ( CDO_Color == FALSE && ITSME ) CDO_Color = TRUE;
 }
 
 static
@@ -986,15 +982,13 @@ void check_stacksize()
 #if defined(HAVE_GETRLIMIT)
 #if defined(RLIMIT_STACK)
   {
-#define  MIN_STACK_SIZE  67108864L  /* 64MB */
-    int status;
     struct rlimit rlim;
-    RLIM_T min_stack_size = MIN_STACK_SIZE;
-
-    status = getrlimit(RLIMIT_STACK, &rlim);
+    int status = getrlimit(RLIMIT_STACK, &rlim);
 
     if ( status == 0 )
       {
+#define  MIN_STACK_SIZE  67108864L  /* 64MB */
+        RLIM_T min_stack_size = MIN_STACK_SIZE;
         if ( min_stack_size > rlim.rlim_max ) min_stack_size = rlim.rlim_max;
         if ( rlim.rlim_cur < min_stack_size )
           {
@@ -1037,14 +1031,12 @@ static
 long str_to_int(const char *intstring)
 {
   long intval = -1;
-  long fact = 1;
 
   if ( intstring )
     {
-      int loop, len;
-
-      len = (int) strlen(intstring);
-      for ( loop = 0; loop < len; loop++ )
+      long fact = 1;
+      int len = (int) strlen(intstring);
+      for ( int loop = 0; loop < len; loop++ )
         {
           if ( ! isdigit((int) intstring[loop]) )
             {
@@ -1074,12 +1066,19 @@ int parse_options_long(int argc, char *argv[])
   int lgridsearchnn;
   int lgridsearchradius;
   int lremap_genweights;
+  int lprecision;
   int lpercentile;
+  int lprintoperatorsno = 0;
   int lprintoperators = 0;
   int lenableexcept;
+  int ltimestat_date;
+  int ltimestat_bounds;
+  int lsortname;
+  int lsortparam;
 
   struct cdo_option opt_long[] =
     {
+      { "precision",         required_argument,        &lprecision,   1  },
       { "percentile",        required_argument,        &lpercentile,  1  },
       { "netcdf_hdr_pad",    required_argument,    &lnetcdf_hdr_pad,  1  },
       { "header_pad",        required_argument,    &lnetcdf_hdr_pad,  1  },
@@ -1089,18 +1088,25 @@ int parse_options_long(int argc, char *argv[])
       { "gridsearchradius",  required_argument,  &lgridsearchradius,  1  },
       { "remap_genweights",  required_argument,  &lremap_genweights,  1  },
       { "enableexcept",      required_argument,      &lenableexcept,  1  },
+      { "timestat_date",     required_argument,     &ltimestat_date,  1  },
+      { "timestat_bounds",         no_argument,   &ltimestat_bounds,  1  },
       { "cmor",                    no_argument,      &CDO_CMOR_Mode,  1  },
       { "reduce_dim",              no_argument,     &CDO_Reduce_Dim,  1  },
       { "float",                   no_argument,        &CDO_Memtype,  MEMTYPE_FLOAT  },
       { "rusage",                  no_argument,         &CDO_Rusage,  1  },
+      { "operators_no_output",     no_argument,  &lprintoperatorsno,  1  },
       { "operators",               no_argument,    &lprintoperators,  1  },
       { "no_warnings",             no_argument,           &_Verbose,  0  },
+      { "color",                   no_argument,                NULL, 'C' },
       { "format",            required_argument,                NULL, 'f' },
       { "help",                    no_argument,                NULL, 'h' },
       { "history",                 no_argument, &CDO_Append_History,  0  },
       { "no_history",              no_argument, &CDO_Append_History,  0  },
       { "regular",                 no_argument,                NULL, 'R' },
       { "silent",                  no_argument,                NULL, 's' },
+      { "sort",                    no_argument,                NULL, 'Q' },
+      { "sortname",                no_argument,          &lsortname,  1  },
+      { "sortparam",               no_argument,         &lsortparam,  1  },
       { "table",             required_argument,                NULL, 't' },
       { "verbose",                 no_argument,                NULL, 'v' },
       { "version",                 no_argument,                NULL, 'V' },
@@ -1111,6 +1117,7 @@ int parse_options_long(int argc, char *argv[])
 
   while ( 1 )
     {
+      lprecision = 0;
       lpercentile = 0;
       lnetcdf_hdr_pad = 0;
       luse_fftw = 0;
@@ -1118,6 +1125,10 @@ int parse_options_long(int argc, char *argv[])
       lgridsearchradius = 0;
       lremap_genweights = 0;
       lenableexcept = 0;
+      ltimestat_date = 0;
+      ltimestat_bounds = 0;
+      lsortname = 0;
+      lsortparam = 0;
 
       c = cdo_getopt_long(argc, argv, "f:b:e:P:g:i:k:l:m:n:t:D:z:aBCcdhLMOpQRrsSTuVvWXZ", opt_long, NULL);
       if ( c == -1 ) break;
@@ -1140,6 +1151,10 @@ int parse_options_long(int argc, char *argv[])
               int netcdf_hdr_pad = str_to_int(CDO_optarg);
               if ( netcdf_hdr_pad >= 0 ) CDO_netcdf_hdr_pad = netcdf_hdr_pad;
             }
+          else if ( lprecision )
+            {
+              cdo_set_digits(CDO_optarg);
+            }
           else if ( lpercentile )
             {
               percentile_set_method(CDO_optarg);
@@ -1157,6 +1172,22 @@ int parse_options_long(int argc, char *argv[])
               cdo_feenableexcept((unsigned)except);
               if ( signal(SIGFPE, cdo_sig_handler) == SIG_ERR ) cdoWarning("can't catch SIGFPE!");
             }
+          else if ( ltimestat_date )
+            {
+              int timestatdate = -1;
+              if      ( strcmp(CDO_optarg, "first")   == 0 ) timestatdate = TIMESTAT_FIRST;
+              else if ( strcmp(CDO_optarg, "last")    == 0 ) timestatdate = TIMESTAT_LAST;
+              else if ( strcmp(CDO_optarg, "middle")  == 0 ) timestatdate = TIMESTAT_MEAN;
+              else if ( strcmp(CDO_optarg, "midhigh") == 0 ) timestatdate = TIMESTAT_MIDHIGH;
+              if ( timestatdate < 0 ) cdoAbort("option --%s: unsupported argument: %s", "timestat_date", CDO_optarg);
+              extern int CDO_Timestat_Date;
+              CDO_Timestat_Date = timestatdate;
+            }
+          else if ( ltimestat_bounds )
+            {
+              extern bool CDO_Timestat_Bounds;
+              CDO_Timestat_Bounds = true;
+            }
           else if ( luse_fftw )
             {
               int intarg = parameter2int(CDO_optarg);
@@ -1171,11 +1202,9 @@ int parse_options_long(int argc, char *argv[])
           else if ( lgridsearchradius )
             {
               extern double gridsearch_radius;
-              double fval = atof(CDO_optarg);
-              if ( fval < 0 || fval > 180 )
-                cdoAbort("gridsearchradius=%g out of bounds (0-180)", fval);
-              else
-                gridsearch_radius = fval;
+              double fval = radius_str_to_deg(CDO_optarg);
+              if ( fval < 0 || fval > 180 ) cdoAbort("%s=%g out of bounds (0-180 deg)!", "gridsearchradius", fval);
+              gridsearch_radius = fval;
             }
           else if ( lremap_genweights )
             {
@@ -1184,6 +1213,14 @@ int parse_options_long(int argc, char *argv[])
                 cdoAbort("Unsupported value for option --remap_genweights=%d [range: 0-1]", intarg);
               remap_genweights = intarg;
             }
+          else if ( lsortname )
+            {
+              cdiDefGlobal("SORTNAME", TRUE);
+            }
+          else if ( lsortparam )
+            {
+              cdiDefGlobal("SORTPARAM", TRUE);
+            }
           break;
         case 'a':
           cdoDefaultTimeType = TAXIS_ABSOLUTE;
@@ -1228,7 +1265,7 @@ int parse_options_long(int argc, char *argv[])
           setDefaultFileType(CDO_optarg, 1);
           break;
         case 'g':
-          defineGrid(CDO_optarg);
+          cdo_set_grids(CDO_optarg);
           break;
         case 'h':        
           Help = 1;
@@ -1298,6 +1335,7 @@ int parse_options_long(int argc, char *argv[])
           break;
         case 'v':
           cdoVerbose = TRUE;
+          _Verbose = 1;
           break;
         case 'W': /* Warning messages */
           _Verbose = 1;
@@ -1314,15 +1352,16 @@ int parse_options_long(int argc, char *argv[])
         }
     }
 
-  if ( lprintoperators )
+  if ( lprintoperators || lprintoperatorsno )
     {
       set_text_color(stderr, RESET, GREEN);
-      operatorPrintList();
+      bool print_no_output = lprintoperatorsno > 0;
+      operatorPrintList(print_no_output);
       //operatorPrintAll();
       reset_text_color(stderr);
       return 1;
     }
-  
+ 
   return 0;
 }
 
@@ -1357,7 +1396,6 @@ int main(int argc, char *argv[])
   int lstop = FALSE;
   int noff = 0;
   int status = 0;
-  char *operatorName = NULL;
   char *operatorArg = NULL;
   argument_t *argument = NULL;
 
@@ -1410,11 +1448,13 @@ int main(int argc, char *argv[])
       fprintf(stderr, "OMP num procs       = %d\n", omp_get_num_procs());
       fprintf(stderr, "OMP max threads     = %d\n", omp_get_max_threads());
       fprintf(stderr, "OMP num threads     = %d\n", omp_get_num_threads());
+#if defined(HAVE_OPENMP3)
       fprintf(stderr, "OMP thread limit    = %d\n", omp_get_thread_limit());
       omp_sched_t kind;
       int modifer;
       omp_get_schedule(&kind, &modifer);
       fprintf(stderr, "OMP schedule        = %d (1:static; 2:dynamic; 3:guided; 4:auto)\n", (int) kind);
+#endif
 #if defined(HAVE_OPENMP4)
       fprintf(stderr, "OMP proc bind       = %d (0:false; 1:true; 2:master; 3:close; 4:spread)\n", (int) omp_get_proc_bind());
 #if !defined(__ICC)
@@ -1470,7 +1510,7 @@ int main(int argc, char *argv[])
 
   if ( cdoDefaultTableID != CDI_UNDEFID ) cdiDefTableID(cdoDefaultTableID);
 
-  operatorName = getOperatorName(operatorArg);
+  const char *operatorName = getOperatorName(operatorArg);
 
   if ( Help )
     {
