@@ -35,7 +35,7 @@ int equals(double expected, double actual, double eps)
 {
   return (int) fabs(expected - actual) < eps; 
 }
-
+/*
 static 
 double humidityIndex(double t, double p, double r, double missval)
 {
@@ -47,7 +47,7 @@ double humidityIndex(double t, double p, double r, double missval)
     
   return t + (5.0 / 9.0) * ((0.01 * r * p * 6.112 * pow(10.0, (7.5 * t) / (237.7 + t))) - 10.0);
 }
-
+*/
 /* reads a NetCDF file containing data for a single grid point */ 
 static
 void readNcFile(const char path[], double **vars, int nvars, int nts)
@@ -110,13 +110,13 @@ void writeNcFile(const char path[], const double array[], int length)
   
   varID = vlistDefVar(vlistID, gridID, zaxisID, TSTEP_INSTANT);
   vlistDefVarName(vlistID, varID, "test_values");
-  vlistDefVarDatatype(vlistID, varID, DATATYPE_FLT64);
+  vlistDefVarDatatype(vlistID, varID, CDI_DATATYPE_FLT64);
   vlistDefVarMissval(vlistID, varID, MISSVAL);
   
   taxisID = taxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID, taxisID);
   
-  streamID = streamOpenWrite(path, FILETYPE_NC);
+  streamID = streamOpenWrite(path, CDI_FILETYPE_NC);
   if ( streamID < 0 ) 
     {
       fprintf(stderr, "%s\n", cdiStringError(streamID));
@@ -154,8 +154,8 @@ void writeNcFile(const char path[], const double array[], int length)
 static
 double **createVars(int nvars, int nts)
 {  
-  double *array = malloc(nvars*nts*sizeof(double));
-  double **vars = malloc(nvars*sizeof(double));
+  double *array = (double*) Malloc(nvars*nts*sizeof(double));
+  double **vars = (double**) Malloc(nvars*sizeof(double*));
   
   int i;
   
@@ -171,14 +171,14 @@ void destroyVars(double **vars)
 {  
   if ( vars != NULL)
     {
-      free(vars[0]);
-      free(vars);
+      Free(vars[0]);
+      Free(vars);
     }
 }
 
 /* gets the path of the CDO binary executable */
 static 
-char *getCdoPath()
+const char *getCdoPath()
 {
   char *cdoPath = getenv("CDO_PATH");
   
@@ -204,7 +204,7 @@ static
 int submitCdoCommand(const char *argument)
 {  
   const char *cdoPath = getCdoPath();
-  char *cdoCommand = malloc(strlen(cdoPath) + strlen(argument) + 8);
+  char *cdoCommand = (char*) Malloc(strlen(cdoPath) + strlen(argument) + 8);
   
   int status;
   
@@ -214,7 +214,7 @@ int submitCdoCommand(const char *argument)
   strcat(cdoCommand, argument);
   
   status = system(cdoCommand);
-  free(cdoCommand);
+  Free(cdoCommand);
   
   return status;
 }
@@ -396,7 +396,7 @@ void testEcaGsl()
  
   destroyVars(vars);
 }
-
+/*
 static
 void testHi()
 {
@@ -424,7 +424,7 @@ void testHi()
 
   destroyVars(vars);
 }
-
+*/
 static
 void testTimcount()
 {
@@ -463,14 +463,14 @@ void testTimcount()
   
   destroyVars(vars);
 }
-
+/*
 static
 void testSeascount()
 {
   const double array[] = {MISSVAL, MISSVAL, TO_KELVIN(1.0), MISSVAL, 
     TO_KELVIN(1.0), TO_KELVIN(1.0)};
   
-  /* number of output variables and time steps */
+  // number of output variables and time steps
   int nvars = 1;
   int nts   = 1;
   
@@ -502,7 +502,7 @@ void testSeascount()
   
   destroyVars(vars);
 }
-
+*/
 static
 void testWct()
 {
