@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -39,15 +39,15 @@ int surfaceID = -1;
 
 enum {FT_STD, FT_CONST, FT_FLD, FT_VERT, FT_COORD, FT_1C};
 
-#define    COMPLT(x,y)  ((x) < (y) ? 1 : 0)
-#define    COMPGT(x,y)  ((x) > (y) ? 1 : 0)
-#define    COMPLE(x,y)  ((x) <= (y) ? 1 : 0)
-#define    COMPGE(x,y)  ((x) >= (y) ? 1 : 0)
-#define    COMPNE(x,y)  (IS_NOT_EQUAL(x,y) ? 1 : 0)
-#define    COMPEQ(x,y)  (IS_EQUAL(x,y) ? 1 : 0)
-#define   COMPLEG(x,y)  ((x) < (y) ? -1 : ((x) > (y) ? 1 : 0))
-#define   COMPAND(x,y)  (IS_NOT_EQUAL(x,0) && IS_NOT_EQUAL(y,0) ? 1 : 0)
-#define    COMPOR(x,y)  (IS_NOT_EQUAL(x,0) || IS_NOT_EQUAL(y,0) ? 1 : 0)
+#define    COMPLT(x,y)  ((x) < (y))
+#define    COMPGT(x,y)  ((x) > (y))
+#define    COMPLE(x,y)  ((x) <= (y))
+#define    COMPGE(x,y)  ((x) >= (y))
+#define    COMPNE(x,y)  IS_NOT_EQUAL(x,y)
+#define    COMPEQ(x,y)  IS_EQUAL(x,y)
+#define   COMPLEG(x,y)  ((x) < (y) ? -1. : ((x) > (y)))
+#define   COMPAND(x,y)  (IS_NOT_EQUAL(x,0) && IS_NOT_EQUAL(y,0))
+#define    COMPOR(x,y)  (IS_NOT_EQUAL(x,0) || IS_NOT_EQUAL(y,0))
 #define  MVCOMPLT(x,y)  (DBL_IS_EQUAL((x),missval1) ? missval1 : COMPLT(x,y))
 #define  MVCOMPGT(x,y)  (DBL_IS_EQUAL((x),missval1) ? missval1 : COMPGT(x,y))
 #define  MVCOMPLE(x,y)  (DBL_IS_EQUAL((x),missval1) ? missval1 : COMPLE(x,y))
@@ -59,7 +59,7 @@ enum {FT_STD, FT_CONST, FT_FLD, FT_VERT, FT_COORD, FT_1C};
 #define  MVCOMPOR(x,y)  (DBL_IS_EQUAL((x),missval1) ? missval1 : COMPOR(x,y))
 
 static double f_int(double x)          { return (int)(x); }
-static double f_nint(double x)         { return round(x); }
+static double f_nint(double x)         { return (double)lround(x); }
 static double f_sqr(double x)          { return x*x;      }
 static double f_rad(double x)          { return x*M_PI/180.; }
 static double f_deg(double x)          { return x*180./M_PI; }
@@ -72,67 +72,68 @@ typedef struct {
   int type;
   int flag;
   const char *name;  // function name
-  void (*func)();    // pointer to function
+  double (*func)();    // pointer to function
 }
 func_t;
 
 static func_t fun_sym_tbl[] =
 {
   // scalar functions
-  {FT_STD, 0, "abs",   (void (*)()) fabs},
-  {FT_STD, 0, "floor", (void (*)()) floor},
-  {FT_STD, 0, "ceil",  (void (*)()) ceil},
-  {FT_STD, 0, "int",   (void (*)()) f_int},
-  {FT_STD, 0, "nint",  (void (*)()) f_nint},
-  {FT_STD, 0, "sqr",   (void (*)()) f_sqr},
-  {FT_STD, 0, "sqrt",  (void (*)()) sqrt},
-  {FT_STD, 0, "exp",   (void (*)()) exp},
-  {FT_STD, 0, "erf",   (void (*)()) erf},
-  {FT_STD, 0, "log",   (void (*)()) log},
-  {FT_STD, 0, "log10", (void (*)()) log10},
-  {FT_STD, 0, "sin",   (void (*)()) sin},
-  {FT_STD, 0, "cos",   (void (*)()) cos},
-  {FT_STD, 0, "tan",   (void (*)()) tan},
-  {FT_STD, 0, "sinh",  (void (*)()) sinh},
-  {FT_STD, 0, "cosh",  (void (*)()) cosh},
-  {FT_STD, 0, "tanh",  (void (*)()) tanh},
-  {FT_STD, 0, "asin",  (void (*)()) asin},
-  {FT_STD, 0, "acos",  (void (*)()) acos},
-  {FT_STD, 0, "atan",  (void (*)()) atan},
-  {FT_STD, 0, "asinh", (void (*)()) asinh},
-  {FT_STD, 0, "acosh", (void (*)()) acosh},
-  {FT_STD, 0, "atanh", (void (*)()) atanh},
-  {FT_STD, 0, "gamma", (void (*)()) tgamma},
-  {FT_STD, 0, "rad",   (void (*)()) f_rad},
-  {FT_STD, 0, "deg",   (void (*)()) f_deg},
+  {FT_STD, 0, "abs",   (double (*)()) (double (*)(double)) fabs},
+  {FT_STD, 0, "floor", (double (*)()) (double (*)(double)) floor},
+  {FT_STD, 0, "ceil",  (double (*)()) (double (*)(double)) ceil},
+  {FT_STD, 0, "sqrt",  (double (*)()) (double (*)(double)) sqrt},
+  {FT_STD, 0, "exp",   (double (*)()) (double (*)(double)) exp},
+  {FT_STD, 0, "erf",   (double (*)()) (double (*)(double)) erf},
+  {FT_STD, 0, "log",   (double (*)()) (double (*)(double)) log},
+  {FT_STD, 0, "ln",    (double (*)()) (double (*)(double)) log},
+  {FT_STD, 0, "log10", (double (*)()) (double (*)(double)) log10},
+  {FT_STD, 0, "sin",   (double (*)()) (double (*)(double)) sin},
+  {FT_STD, 0, "cos",   (double (*)()) (double (*)(double)) cos},
+  {FT_STD, 0, "tan",   (double (*)()) (double (*)(double)) tan},
+  {FT_STD, 0, "sinh",  (double (*)()) (double (*)(double)) sinh},
+  {FT_STD, 0, "cosh",  (double (*)()) (double (*)(double)) cosh},
+  {FT_STD, 0, "tanh",  (double (*)()) (double (*)(double)) tanh},
+  {FT_STD, 0, "asin",  (double (*)()) (double (*)(double)) asin},
+  {FT_STD, 0, "acos",  (double (*)()) (double (*)(double)) acos},
+  {FT_STD, 0, "atan",  (double (*)()) (double (*)(double)) atan},
+  {FT_STD, 0, "asinh", (double (*)()) (double (*)(double)) asinh},
+  {FT_STD, 0, "acosh", (double (*)()) (double (*)(double)) acosh},
+  {FT_STD, 0, "atanh", (double (*)()) (double (*)(double)) atanh},
+  {FT_STD, 0, "gamma", (double (*)()) (double (*)(double)) tgamma},
+  {FT_STD, 0, "int",   (double (*)()) f_int},
+  {FT_STD, 0, "nint",  (double (*)()) f_nint},
+  {FT_STD, 0, "sqr",   (double (*)()) f_sqr},
+  {FT_STD, 0, "rad",   (double (*)()) f_rad},
+  {FT_STD, 0, "deg",   (double (*)()) f_deg},
 
   // constant functions
-  {FT_CONST, 0, "ngp",     (void (*)()) pt_ngp},      // number of horizontal grid points
-  {FT_CONST, 0, "nlev",    (void (*)()) pt_nlev},     // number of vertical levels
-  {FT_CONST, 0, "size",    (void (*)()) pt_size},     // ngp*nlev
-  {FT_CONST, 0, "missval", (void (*)()) pt_missval},  // Returns the missing value of a variable
+  {FT_CONST, 0, "ngp",     (double (*)()) pt_ngp},      // number of horizontal grid points
+  {FT_CONST, 0, "nlev",    (double (*)()) pt_nlev},     // number of vertical levels
+  {FT_CONST, 0, "size",    (double (*)()) pt_size},     // ngp*nlev
+  {FT_CONST, 0, "missval", (double (*)()) pt_missval},  // Returns the missing value of a variable
 
   // cdo field functions (Reduce grid to point)
-  {FT_FLD, 0, "fldmin",  (void (*)()) fldmin},
-  {FT_FLD, 0, "fldmax",  (void (*)()) fldmax},
-  {FT_FLD, 0, "fldsum",  (void (*)()) fldsum},
-  {FT_FLD, 1, "fldmean", (void (*)()) fldmean},
-  {FT_FLD, 1, "fldavg",  (void (*)()) fldavg},
-  {FT_FLD, 1, "fldstd",  (void (*)()) fldstd},
-  {FT_FLD, 1, "fldstd1", (void (*)()) fldstd1},
-  {FT_FLD, 1, "fldvar",  (void (*)()) fldvar},
-  {FT_FLD, 1, "fldvar1", (void (*)()) fldvar1},
+  {FT_FLD, 0, "fldmin",  (double (*)()) fldmin},
+  {FT_FLD, 0, "fldmax",  (double (*)()) fldmax},
+  {FT_FLD, 0, "fldsum",  (double (*)()) fldsum},
+  {FT_FLD, 1, "fldmean", (double (*)()) fldmean},
+  {FT_FLD, 1, "fldavg",  (double (*)()) fldavg},
+  {FT_FLD, 1, "fldstd",  (double (*)()) fldstd},
+  {FT_FLD, 1, "fldstd1", (double (*)()) fldstd1},
+  {FT_FLD, 1, "fldvar",  (double (*)()) fldvar},
+  {FT_FLD, 1, "fldvar1", (double (*)()) fldvar1},
 
   // cdo field functions (Reduce level to point)
-  {FT_VERT, 0, "vertmin",  (void (*)()) fldmin},
-  {FT_VERT, 0, "vertmax",  (void (*)()) fldmax},
-  {FT_VERT, 0, "vertsum",  (void (*)()) fldsum},
-  {FT_VERT, 1, "vertmean", (void (*)()) fldmean},
-  {FT_VERT, 1, "vertavg",  (void (*)()) fldavg},
-  {FT_VERT, 1, "vertstd",  (void (*)()) fldstd},
-  {FT_VERT, 1, "vertstd1", (void (*)()) fldstd1},
-  {FT_VERT, 1, "vertvar",  (void (*)()) fldvar},
-  {FT_VERT, 1, "vertvar1", (void (*)()) fldvar1},
+  {FT_VERT, 0, "vertmin",  (double (*)()) fldmin},
+  {FT_VERT, 0, "vertmax",  (double (*)()) fldmax},
+  {FT_VERT, 0, "vertsum",  (double (*)()) fldsum},
+  {FT_VERT, 1, "vertmean", (double (*)()) fldmean},
+  {FT_VERT, 1, "vertavg",  (double (*)()) fldavg},
+  {FT_VERT, 1, "vertstd",  (double (*)()) fldstd},
+  {FT_VERT, 1, "vertstd1", (double (*)()) fldstd1},
+  {FT_VERT, 1, "vertvar",  (double (*)()) fldvar},
+  {FT_VERT, 1, "vertvar1", (double (*)()) fldvar1},
   
   {FT_COORD, 0, "clon",       NULL},
   {FT_COORD, 0, "clat",       NULL},
@@ -700,8 +701,8 @@ nodeType *expr(int init, int oper, nodeType *p1, nodeType *p2)
         case '*':  coper = "*"; break;
         case '/':  coper = "/"; break;
         case '^':  coper = "^"; break;
-        case LT:   coper = ">"; break;
-        case GT:   coper = "<"; break;
+        case LT:   coper = "<"; break;
+        case GT:   coper = ">"; break;
         case LE:   coper = "<="; break;
         case GE:   coper = ">="; break;
         case NE:   coper = "!="; break;
@@ -836,7 +837,7 @@ nodeType *ex_fun_var(int init, int funcID, nodeType *p1)
         }
       else if ( functype == FT_FLD )
         {
-          field_t field;
+          field_type field;
           double *weights = NULL;
           //if ( funcflag == 1 ) weights = fld_weights(p1->param.gridID, ngp);
           if ( funcflag == 1 )
@@ -845,7 +846,7 @@ nodeType *ex_fun_var(int init, int funcID, nodeType *p1)
               assert(weights!=NULL);
             }
           
-          double (*exprfunc)(field_t) = (double (*)(field_t)) fun_sym_tbl[funcID].func;
+          double (*exprfunc)(field_type) = (double (*)(field_type)) fun_sym_tbl[funcID].func;
           for ( size_t k = 0; k < nlev; k++ )
             {
               fld_field_init(&field, nmiss, missval, ngp, p1data+k*ngp, weights);
@@ -855,11 +856,11 @@ nodeType *ex_fun_var(int init, int funcID, nodeType *p1)
         }
       else if ( functype == FT_VERT )
         {
-          field_t field;
+          field_type field;
           double *weights = NULL;
           if ( funcflag == 1 ) weights = vert_weights(p1->param.zaxisID, nlev);
           double *array = (double*) Malloc(nlev*sizeof(double));
-          double (*exprfunc)(field_t) = (double (*)(field_t)) fun_sym_tbl[funcID].func;
+          double (*exprfunc)(field_type) = (double (*)(field_type)) fun_sym_tbl[funcID].func;
           for ( size_t i = 0; i < ngp; i++ )
             {
               for ( size_t k = 0; k < nlev; k++ ) array[k] = p1data[k*ngp+i];
@@ -953,8 +954,8 @@ nodeType *fun1c(int init, int funcID, nodeType *p1, double value, parse_param_t
     {
       parse_arg->coords[coordID].needed = true;
 
-      data = (double*) malloc(nlev*sizeof(double));
-      zaxisInqLevels(zaxisID, data);
+      data = (double*) Malloc(nlev*sizeof(double));
+      cdoZaxisInqLevels(zaxisID, data);
     }
   else
     {
@@ -1075,12 +1076,12 @@ nodeType *ex_uminus_var(int init, nodeType *p1)
 
       if ( nmiss > 0 )
         {
-          for ( size_t i = 0; i < ngp*nlev; i++ )
+          for ( size_t i = 0; i < ngp*nlev; ++i )
             pdata[i] = DBL_IS_EQUAL(p1data[i], missval) ? missval : -(p1data[i]);
         }
       else
         {
-          for ( size_t i = 0; i < ngp*nlev; i++ )
+          for ( size_t i = 0; i < ngp*nlev; ++i )
             pdata[i] = -(p1data[i]);
         }
 
@@ -1135,13 +1136,13 @@ nodeType *ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
     {
       fprintf(stderr, "cdo expr:\t%s\tifelse\t%s[L%lu][N%lu] ? ", ExIn[init], p1->u.var.nm, p1->param.nlev, p1->param.ngp);
       if ( p2->type == typeCon )
-        printf("%g : ", p2->u.con.value);
+        fprintf(stderr, "%g : ", p2->u.con.value);
       else
-        printf("%s[L%lu][N%lu] : ", p2->u.var.nm, p2->param.nlev, p2->param.ngp);
+        fprintf(stderr, "%s[L%lu][N%lu] : ", p2->u.var.nm, p2->param.nlev, p2->param.ngp);
       if ( p3->type == typeCon )
-        printf("%g\n", p3->u.con.value);
+        fprintf(stderr, "%g\n", p3->u.con.value);
       else
-        printf("%s[L%lu][N%lu]\n", p3->u.var.nm, p3->param.nlev, p3->param.ngp);
+        fprintf(stderr, "%s[L%lu][N%lu]\n", p3->u.var.nm, p3->param.nlev, p3->param.ngp);
     }
 
   size_t nmiss1 = p1->param.nmiss;
@@ -1229,17 +1230,10 @@ nodeType *ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
 
       for ( size_t k = 0; k < nlev; ++k )
         {
-          size_t loff1, loff2, loff3;
-          size_t loff = k*ngp;
-
-          if ( nlev1 == 1 ) loff1 = 0;
-          else              loff1 = k*ngp;
-
-          if ( nlev2 == 1 ) loff2 = 0;
-          else              loff2 = k*ngp;
-
-          if ( nlev3 == 1 ) loff3 = 0;
-          else              loff3 = k*ngp;
+          size_t loff  = k*ngp;
+          size_t loff1 = (nlev1 == 1) ? 0 : loff;
+          size_t loff2 = (nlev2 == 1) ? 0 : loff;
+          size_t loff3 = (nlev3 == 1) ? 0 : loff;
 
           const double *restrict idat1 = pdata1+loff1;
           const double *restrict idat2 = pdata2+loff2;
