@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,71 +18,39 @@
 #include "cdo.h"
 #include "cdo_int.h"
 #include <cdi.h>
-/* RQ */
-#include "nth_element.h"
+#include "percentiles.h"
 #include "merge_sort2.h"
-/* QR */
 
 double crps_det_integrate(double *a, const double d, const size_t n);
 
-double _FADD_(const double x, const double y, const double missval1, const double missval2)
-{
-  return FADD(x,y);
-}
-
-double _FSUB_(const double x, const double y, const double missval1, const double missval2)
-{
-  return FSUB(x, y);
-}
-
-double _FMUL_(const double x, const double y, const double missval1, const double missval2)
-{
-  return FMUL(x, y);
-}
-
-double _FDIV_(const double x, const double y, const double missval1, const double missval2)
-{
-  return FDIV(x, y);
-}
-
-double _FPOW_(const double x, const double y, const double missval1, const double missval2)
-{
-  return FPOW(x, y);
-}
-
-double _FSQRT_(const double x, const double missval1)
-{
-  return FSQRT(x);
-}
-
-
-double fldfun(field_t field, const int function)
+double fldfun(field_type field, int function)
 {
   double rval = 0;
 
-  // Why is this not a switch-case statement?
-  if      ( function == func_min  )  rval = fldmin(field);
-  else if ( function == func_max  )  rval = fldmax(field);
-  else if ( function == func_sum  )  rval = fldsum(field);
-  else if ( function == func_mean )  rval = fldmean(field);
-  else if ( function == func_avg  )  rval = fldavg(field);
-  else if ( function == func_std  )  rval = fldstd(field);
-  else if ( function == func_std1 )  rval = fldstd1(field);
-  else if ( function == func_var  )  rval = fldvar(field);
-  else if ( function == func_var1 )  rval = fldvar1(field);
+  switch (function)
+    {
+    case func_range:  rval = fldrange(field);  break;
+    case func_min:    rval = fldmin(field);    break;
+    case func_max:    rval = fldmax(field);    break;
+    case func_sum:    rval = fldsum(field);    break;
+    case func_mean:   rval = fldmean(field);   break;
+    case func_avg:    rval = fldavg(field);    break;
+    case func_std:    rval = fldstd(field);    break;
+    case func_std1:   rval = fldstd1(field);   break;
+    case func_var:    rval = fldvar(field);    break;
+    case func_var1:   rval = fldvar1(field);   break;
+    case func_crps:   rval = fldcrps(field);   break;
+    case func_brs:    rval = fldbrs(field);    break;
+    case func_rank:   rval = fldrank(field);   break;
+    case func_roc:    rval = fldroc(field);    break;
+    default: cdoAbort("%s: function %d not implemented!", __func__, function);
+    }
   
-  else if ( function == func_crps )  rval = fldcrps(field);
-  else if ( function == func_brs )   rval = fldbrs(field);
-
-  else if ( function == func_rank )  rval = fldrank(field);
-  else if ( function == func_roc )   rval = fldroc(field);
-  else cdoAbort("%s: function %d not implemented!", __func__, function);
-
   return rval;
 }
 
 
-double fldrank(field_t field) 
+double fldrank(field_type field) 
 {
   double res = 0;
   // Using first value as reference (observation)
@@ -93,7 +61,7 @@ double fldrank(field_t field)
   const size_t len       = field.size-1;
   size_t j;
   
-  if ( nmiss )  return(missval);
+  if ( nmiss ) return missval;
 
   sort_iter_single(len,array, 1);
 
@@ -110,12 +78,12 @@ double fldrank(field_t field)
 }
 
 
-double fldroc(field_t field) 
+double fldroc(field_type field) 
 {
   return field.missval;
 }
 
-double fldcrps(field_t field)
+double fldcrps(field_type field)
 {
   const size_t len     = field.size;
   const int    nmiss   = field.nmiss;
@@ -136,10 +104,10 @@ double fldcrps(field_t field)
 }
 
 
-double fldbrs(field_t field) 
+double fldbrs(field_type field) 
 {
-  const size_t    len   = field.size;
   const int     nmiss   = field.nmiss;
+  const size_t    len   = field.size;
   double *array   = field.ptr;
   const double missval  = field.missval;
 
@@ -169,79 +137,119 @@ double fldbrs(field_t field)
 }
 
 
-double fldmin(field_t field)
+double fldrange(field_type field)
 {
-  size_t   i;
+  const int nmiss      = field.nmiss > 0;
   const size_t len     = field.size;
-  const int    nmiss   = field.nmiss;
   const double missval = field.missval;
-  double *array  = field.ptr;
-  double rmin = 0;
+  const double *restrict array = field.ptr;
+  double rmin =  DBL_MAX;
+  double rmax = -DBL_MAX;
+  double range = 0;
 
-  if ( nmiss > 0 )
+  assert(array!=NULL);
+
+  if ( nmiss )
     {
-      rmin = DBL_MAX;
-      for ( i = 0; i < len; i++ ) 
+      for ( size_t i = 0; i < len; i++ ) 
+	if ( !DBL_IS_EQUAL(array[i], missval) )
+          {
+            if ( array[i] < rmin ) rmin = array[i];
+            if ( array[i] > rmax ) rmax = array[i];
+          }
+
+      if ( IS_EQUAL(rmin,  DBL_MAX) && IS_EQUAL(rmax, -DBL_MAX) )
+        range = missval;
+      else
+        range = rmax-rmin;
+    }
+  else
+    {
+      //#pragma simd reduction(min:rmin) 
+      for ( size_t i = 0; i < len; i++ )
+        {
+          if ( array[i] < rmin ) rmin = array[i];
+          if ( array[i] > rmax ) rmax = array[i];
+        }
+      range = rmax-rmin;
+    }
+
+  return range;
+}
+
+
+double fldmin(field_type field)
+{
+  const int nmiss      = field.nmiss > 0;
+  const size_t len     = field.size;
+  const double missval = field.missval;
+  const double *restrict array = field.ptr;
+  double rmin = DBL_MAX;
+
+  assert(array!=NULL);
+
+  if ( nmiss )
+    {
+      for ( size_t i = 0; i < len; i++ ) 
 	if ( !DBL_IS_EQUAL(array[i], missval) )
 	  if ( array[i] < rmin ) rmin = array[i];
 
-      if ( IS_EQUAL(rmin, DBL_MAX) )
-	rmin = missval;
+      if ( IS_EQUAL(rmin, DBL_MAX) ) rmin = missval;
     }
   else
     {
-      rmin = array[0];
-      for ( i = 1; i < len; i++ ) 
-	if ( array[i] < rmin )  rmin = array[i];
+      //#pragma simd reduction(min:rmin) 
+      for ( size_t i = 0; i < len; i++ ) 
+	if ( array[i] < rmin ) rmin = array[i];
     }
 
-  return (rmin);
+  return rmin;
 }
 
 
-double fldmax(field_t field)
+double fldmax(field_type field)
 {
-  size_t   i;
+  const int nmiss      = field.nmiss > 0;
   const size_t len     = field.size;
-  const int    nmiss   = field.nmiss;
   const double missval = field.missval;
-  double *array  = field.ptr;
-  double rmax = 0;
+  const double *restrict array = field.ptr;
+  double rmax = -DBL_MAX;
+
+  assert(array!=NULL);
 
   if ( nmiss > 0 )
     {
-      rmax = -DBL_MAX;
-      for ( i = 0; i < len; i++ )
+      for ( size_t i = 0; i < len; i++ )
         if ( !DBL_IS_EQUAL(array[i], missval) )
           if ( array[i] > rmax ) rmax = array[i];
       
-      if ( IS_EQUAL(rmax, -DBL_MAX) )
-        rmax = missval;
+      if ( IS_EQUAL(rmax, -DBL_MAX) ) rmax = missval;
     }
   else
     {
-      rmax = array[0];
-      for ( i = 1; i < len; i++ ) 
-        if ( array[i] > rmax )  rmax = array[i];
+      for ( size_t i = 0; i < len; i++ ) 
+        if ( array[i] > rmax ) rmax = array[i];
     }
 
-  return (rmax);
+  return rmax;
 }
 
 
-double fldsum(field_t field)
+double fldsum(field_type field)
 {
-  size_t   i;
-  size_t   nvals   = 0;
+  const int nmiss      = field.nmiss > 0;
   const size_t len     = field.size;
-  const int    nmiss   = field.nmiss;
   const double missval = field.missval;
-  double *array  = field.ptr;
+  const double *restrict array = field.ptr;
   double rsum = 0;
 
-  if ( nmiss > 0 )
+  assert(array!=NULL);
+
+  if ( nmiss )
     {
-      for ( i = 0; i < len; i++ ) 
+      size_t nvals = 0;
+
+      for ( size_t i = 0; i < len; i++ ) 
 	if ( !DBL_IS_EQUAL(array[i], missval) )
 	  {
 	    rsum += array[i];
@@ -252,159 +260,159 @@ double fldsum(field_t field)
     }
   else
     {
-      for ( i = 0; i < len; i++ ) 
+      for ( size_t i = 0; i < len; i++ ) 
 	rsum += array[i];
     }
 
-  return (rsum);
+  return rsum;
 }
 
 
-double fldmean(field_t field)
+double fldmean(field_type field)
 {
-  size_t   i;
+  const int nmiss       = field.nmiss > 0;
   const size_t len      = field.size;
-  const int    nmiss    = field.nmiss;
   const double missval1 = field.missval;
   const double missval2 = field.missval;
-  double *array   = field.ptr;
-  double *w       = field.weight;
-  double rsum = 0, rsumw = 0, ravg = 0;
+  const double *restrict array = field.ptr;
+  const double *restrict w     = field.weight;
+  double rsum = 0, rsumw = 0;
+  double ravg = 0;
 
-  if ( nmiss > 0 )
+  assert(array!=NULL);
+  assert(w!=NULL);
+
+  if ( nmiss )
     {
-      for ( i = 0; i < len; i++ ) 
+      for ( size_t i = 0; i < len; i++ ) 
 	if ( !DBL_IS_EQUAL(array[i], missval1) && !DBL_IS_EQUAL(w[i], missval1) )
 	  {
 	    rsum  += w[i] * array[i];
 	    rsumw += w[i];
 	  }
+      ravg = DIVMN(rsum, rsumw);
     }
   else
     {
-      for ( i = 0; i < len; i++ ) 
-	{
-	  rsum  += w[i] * array[i];
-	  rsumw += w[i];
-	}
+      int fpeRaised = array_mean_val_weighted(len, array, w, missval1, &ravg);
     }
 
-  ravg = DIV(rsum, rsumw);
-
-  return (ravg);
+  return ravg;
 }
 
 
-double fldavg(field_t field)
+double fldavg(field_type field)
 {
-  size_t   i;
+  const int nmiss       = field.nmiss > 0;
   const size_t len      = field.size;
-  const int    nmiss    = field.nmiss;
   const double missval1 = field.missval;
   const double missval2 = field.missval;
-  double *array   = field.ptr;
-  double *w       = field.weight;
-  double rsum = 0, rsumw = 0, ravg = 0;
+  const double *restrict array = field.ptr;
+  const double *restrict w     = field.weight;
+  double rsum = 0, rsumw = 0;
 
-  if ( nmiss > 0 )
+  assert(array!=NULL);
+  assert(w!=NULL);
+
+  if ( nmiss )
     {
-      for ( i = 0; i < len; i++ ) 
+      for ( size_t i = 0; i < len; i++ ) 
 	if ( !DBL_IS_EQUAL(w[i], missval1) )
 	  {
-	    rsum  = ADD(rsum, MUL(w[i], array[i]));
-	    rsumw = ADD(rsumw, w[i]);
+	    rsum  = ADDMN(rsum, MULMN(w[i], array[i]));
+	    rsumw = ADDMN(rsumw, w[i]);
 	  }
     }
   else
     {
-      for ( i = 0; i < len; i++ ) 
+      for ( size_t i = 0; i < len; i++ ) 
 	{
 	  rsum  += w[i] * array[i];
 	  rsumw += w[i];
 	}
     }
 
-  ravg = DIV(rsum, rsumw);
+  double ravg = DIVMN(rsum, rsumw);
 
-  return (ravg);
+  return ravg;
 }
 
 static
-void prevarsum(const double *restrict array, const double *restrict w, size_t len, const int nmiss, 
+void prevarsum(const double *restrict array, const double *restrict w, size_t len, int nmiss, 
 	       double missval, double *rsum, double *rsumw, double *rsumq, double *rsumwq)
 { 
-  size_t i;
-  *rsum = *rsumw = 0;
-  *rsumq = *rsumwq = 0;
+  assert(array!=NULL);
+  assert(w!=NULL);
 
-  if ( nmiss > 0 )
+  double xsum = 0, xsumw = 0;
+  double xsumq = 0, xsumwq = 0;
+
+  if ( nmiss )
     {
-      for ( i = 0; i < len; i++ ) 
+      for ( size_t i = 0; i < len; i++ ) 
         if ( !DBL_IS_EQUAL(array[i], missval) && !DBL_IS_EQUAL(w[i], missval) )
           {
-            *rsum   += w[i] * array[i];
-            *rsumq  += w[i] * array[i] * array[i];
-            *rsumw  += w[i];
-            *rsumwq += w[i] * w[i];
+            xsum   += w[i] * array[i];
+            xsumq  += w[i] * array[i] * array[i];
+            xsumw  += w[i];
+            xsumwq += w[i] * w[i];
           }
     }
   else
     {
-      for ( i = 0; i < len; i++ ) 
+      for ( size_t i = 0; i < len; i++ ) 
         {
-          *rsum   += w[i] * array[i];
-          *rsumq  += w[i] * array[i] * array[i];
-          *rsumw  += w[i];
-          *rsumwq += w[i] * w[i];
+          xsum   += w[i] * array[i];
+          xsumq  += w[i] * array[i] * array[i];
+          xsumw  += w[i];
+          xsumwq += w[i] * w[i];
         }
     }
+
+  *rsum   = xsum;
+  *rsumq  = xsumq;
+  *rsumw  = xsumw;
+  *rsumwq = xsumwq;
 }
 
 
-double fldvar(field_t field)
+double fldvar(field_type field)
 {
+  const int    nmiss   = field.nmiss > 0;
   const size_t len     = field.size;
-  const int    nmiss   = field.nmiss;
   const double missval = field.missval;
-  double *array  = field.ptr;
-  double *w      = field.weight;
-  double rsum, rsumw, rvar;
+  double rsum, rsumw;
   double rsumq, rsumwq;
 
-  prevarsum(array, w, len, nmiss, missval, &rsum, &rsumw, &rsumq, &rsumwq);
+  prevarsum(field.ptr, field.weight, len, nmiss, missval, &rsum, &rsumw, &rsumq, &rsumwq);
 
-  rvar = IS_NOT_EQUAL(rsumw, 0) ? (rsumq*rsumw - rsum*rsum) / (rsumw*rsumw) : missval;
+  double rvar = IS_NOT_EQUAL(rsumw, 0) ? (rsumq*rsumw - rsum*rsum) / (rsumw*rsumw) : missval;
   if ( rvar < 0 && rvar > -1.e-5 ) rvar = 0;
 
-  return (rvar);
+  return rvar;
 }
 
 
-double fldvar1(field_t field)
+double fldvar1(field_type field)
 {
+  const int    nmiss   = field.nmiss > 0;
   const size_t len     = field.size;
-  const int    nmiss   = field.nmiss;
   const double missval = field.missval;
-  double *array  = field.ptr;
-  double *w      = field.weight;
-  double rsum, rsumw, rvar;
+  double rsum, rsumw;
   double rsumq, rsumwq;
 
-  prevarsum(array, w, len, nmiss, missval, &rsum, &rsumw, &rsumq, &rsumwq);
+  prevarsum(field.ptr, field.weight, len, nmiss, missval, &rsum, &rsumw, &rsumq, &rsumwq);
 
-  rvar = (rsumw*rsumw > rsumwq) ? (rsumq*rsumw - rsum*rsum) / (rsumw*rsumw - rsumwq) : missval;
+  double rvar = (rsumw*rsumw > rsumwq) ? (rsumq*rsumw - rsum*rsum) / (rsumw*rsumw - rsumwq) : missval;
   if ( rvar < 0 && rvar > -1.e-5 ) rvar = 0;
 
-  return (rvar);
+  return rvar;
 }
 
 
-double fldstd(field_t field)
+double var_to_std(double rvar, double missval)
 {
-  const double missval = field.missval;
-  double rvar, rstd;
-
-  rvar = fldvar(field);
+  double rstd;
 
   if ( DBL_IS_EQUAL(rvar, missval) || rvar < 0 )
     {
@@ -415,31 +423,22 @@ double fldstd(field_t field)
       rstd = IS_NOT_EQUAL(rvar, 0) ? sqrt(rvar) : 0;
     }
 
-  return (rstd);
+  return rstd;
 }
 
-
-double fldstd1(field_t field)
+double fldstd(field_type field)
 {
-  const double missval = field.missval;
-  double rvar, rstd;
-
-  rvar = fldvar1(field);
-
-  if ( DBL_IS_EQUAL(rvar, missval) || rvar < 0 )
-    {
-      rstd = missval;
-    }
-  else
-    {
-      rstd = IS_NOT_EQUAL(rvar, 0) ? sqrt(rvar) : 0;
-    }
-
-  return (rstd);
+  return var_to_std(fldvar(field), field.missval);
 }
 
 
-void fldrms(field_t field, field_t field2, field_t *field3)
+double fldstd1(field_type field)
+{
+  return var_to_std(fldvar1(field), field.missval);
+}
+
+
+void fldrms(field_type field, field_type field2, field_type *field3)
 {
   size_t   i;
   size_t len;
@@ -466,9 +465,9 @@ void fldrms(field_t field, field_t field2, field_t *field3)
       for ( i = 0; i < len; i++ ) 
 	if ( !DBL_IS_EQUAL(w[i], missval1) )
 	  {
-	    rsum  = ADD(rsum, MUL(w[i], MUL(SUB(array2[i], array1[i]),
-                                            SUB(array2[i], array1[i]))));
-	    rsumw = ADD(rsumw, w[i]);
+	    rsum  = ADDMN(rsum, MULMN(w[i], MULMN( SUBMN(array2[i], array1[i]),
+                                            SUBMN(array2[i], array1[i]))));
+	    rsumw = ADDMN(rsumw, w[i]);
 	  }
     }
     /*
@@ -482,7 +481,7 @@ void fldrms(field_t field, field_t field2, field_t *field3)
     }
     */
 
-  ravg = SQRT(DIV(rsum, rsumw));
+  ravg = SQRTMN( DIVMN(rsum, rsumw));
 
   if ( DBL_IS_EQUAL(ravg, missval1) ) rnmiss++;
 
@@ -491,7 +490,7 @@ void fldrms(field_t field, field_t field2, field_t *field3)
 }
 
 
-void varrms(field_t field, field_t field2, field_t *field3)
+void varrms(field_type field, field_type field2, field_type *field3)
 {
   size_t   i, k, nlev, len;
   int    rnmiss = 0;
@@ -520,9 +519,9 @@ void varrms(field_t field, field_t field2, field_t *field3)
 	for ( i = 0; i < len; i++ )
 	  /*	  if ( !DBL_IS_EQUAL(w[i], missval1) ) */
 	    {
-	      rsum  = ADD(rsum, MUL(w[i], MUL(SUB(array2[k*len+i], array1[k*len+i]),
-                                              SUB(array2[k*len+i], array1[k*len+i]))));
-	      rsumw = ADD(rsumw, w[i]);
+	      rsum  = ADDMN(rsum, MULMN(w[i], MULMN( SUBMN(array2[k*len+i], array1[k*len+i]),
+                                              SUBMN(array2[k*len+i], array1[k*len+i]))));
+	      rsumw = ADDMN(rsumw, w[i]);
 	    }
     }
     /*
@@ -536,7 +535,7 @@ void varrms(field_t field, field_t field2, field_t *field3)
     }
     */
 
-  ravg = SQRT(DIV(rsum, rsumw));
+  ravg = SQRTMN( DIVMN(rsum, rsumw));
 
   if ( DBL_IS_EQUAL(ravg, missval1) ) rnmiss++;
 
@@ -545,34 +544,32 @@ void varrms(field_t field, field_t field2, field_t *field3)
 }
 
 /* RQ */
-double fldpctl(field_t field, const int p)
+double fldpctl(field_type field, const double pn)
 {
   const size_t len     = field.size;
   const int    nmiss   = field.nmiss;
   const double missval = field.missval;
   double *array  = field.ptr;
-  double *array2;
-
-  size_t i, j;
   double pctl = missval;
 
   if ( len - nmiss > 0 )
     {
       if ( nmiss > 0 )
         {
-          array2 = malloc((len - nmiss)*sizeof(double));
+          double *array2 = (double*) Malloc((len - nmiss)*sizeof(double));
 
-          for ( i = 0, j = 0; i < len; i++ ) 
+          size_t j = 0;
+          for ( size_t i = 0; i < len; i++ ) 
             if ( !DBL_IS_EQUAL(array[i], missval) )
               array2[j++] = array[i];
 
-          pctl = nth_element(array2, j, (int)ceil(j*(p/100.0))-1);
+          pctl = percentile(array2, j, pn);
 
-          free(array2);
+          Free(array2);
         }
       else
         {
-          pctl = nth_element(array, len, (int)ceil(len*(p/100.0))-1);
+          pctl = percentile(array, len, pn);
         }
     }
 
@@ -580,9 +577,9 @@ double fldpctl(field_t field, const int p)
 }
 /* QR */
 
-/*  field_t UTILITIES */
+/*  field_type UTILITIES */
 /*  update the number non missing values */
-void fldunm(field_t *field)
+void fldunm(field_type *field)
 {
   size_t i;
 
@@ -592,10 +589,10 @@ void fldunm(field_t *field)
 }
 
 /*  check for non missval values */
-int fldhvs(field_t *fieldPtr, const size_t nlevels)
+int fldhvs(field_type *fieldPtr, const size_t nlevels)
 {
   size_t level;
-  field_t field;
+  field_type field;
 
   for ( level = 0; level < nlevels; level++)
     {
@@ -628,19 +625,19 @@ double crps_det_integrate(double *a, const double d, const size_t n)
 #if defined(_OPENMP)
 #pragma omp parallel for if ( n>10000 ) shared(a) private(i) \
   reduction(+:area) schedule(static,10000) 
-#endif                                                     /* **************************** */
-  for ( i=1; i<n; i++ ) {                                  /* INTEGRATE CURVE AREA         */
-    if ( a[i] < d )                                        /* left of heavyside            */
+#endif                                                         /* **************************** */
+  for ( i=1; i<n; i++ ) {                                      /* INTEGRATE CURVE AREA         */
+    if ( a[i] < d )                                            /* left of heavyside            */
       area += (a[i]-a[i-1])*(double)i*i/n/n;                   /*                              */
-    else if ( a[i-1] > d )                                 /* right of heavyside           */
-      area += (a[i]-a[i-1])*(1.-(double)i/n)*(1.-(double)i/n);              /*                              */
-    else if ( a[i-1] < d && a[i] > d ) {                   /* hitting jump pf heavyside    */
+    else if ( a[i-1] > d )                                     /* right of heavyside           */
+      area += (a[i]-a[i-1])*(1.-(double)i/n)*(1.-(double)i/n); /*                              */
+    else if ( a[i-1] < d && a[i] > d ) {                       /* hitting jump pf heavyside    */
       area += (d-a[i-1]) * (double)i*i/n/n;                    /* (occurs exactly once!)       */
-      area += (a[i]-d) * (1.-(double)i/n)*(1.-(double)i/n);                 /* **************************** */
+      area += (a[i]-d) * (1.-(double)i/n)*(1.-(double)i/n);    /* **************************** */
     }
   }
 
 
-  return(area);
+  return area;
 }
 
