@@ -5,8 +5,11 @@
 #include "config.h"
 #endif
 
+#include <math.h>
+#include <stdbool.h>
+
 #ifndef  M_PI
-#define  M_PI        3.14159265358979323846  /* pi */
+#define  M_PI        3.14159265358979323846264338327950288  /* pi */
 #endif
 
 
@@ -19,6 +22,21 @@
 #endif
 
 
+int nfc_to_nlat(int nfc, int ntr);
+int nlat_to_ntr(int nlat);
+int nlat_to_ntr_linear(int nlat);
+int ntr_to_nlat(int ntr);
+int ntr_to_nlat_linear(int ntr);
+int nlat_to_nlon(int nlat);
+
+void grid_copy_attributes(int gridID1, int gridID2);
+void grid_copy_mapping(int gridID1, int gridID2);
+
+void grid_def_param_laea(int gridID, double a, double lon0, double lat0);
+void grid_def_param_sinu(int gridID);
+
+bool grid_is_distance_generic(int gridID);
+
 void grid_to_radian(const char *units, long nvals, double *restrict values, const char *description);
 void grid_to_degree(const char *units, long nvals, double *restrict values, const char *description);
 
@@ -41,10 +59,11 @@ int referenceToGrid(int gridID);
 int gridToZonal(int gridID);
 int gridToMeridional(int gridID);
 int gridToUnstructured(int gridID, int lbounds);
+int gridToUnstructuredSelecton(int gridID1, int selectionSize, int *selectionIndexList, int nocoords ,int nobounds);
 int gridToCurvilinear(int gridID, int lbounds);
 int gridCurvilinearToRegular(int gridID);
 int gridToRegular(int gridID);
-void field2regular(int gridID1, int gridID2, double missval, double *array, int nmiss);
+void field2regular(int gridID1, int gridID2, double missval, double *array, int nmiss, int lnearest);
 
 /* GME grid */
 struct cart {
