@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -56,9 +56,10 @@ double yac_huiliers_area(int num_corners, double *cell_corner_lon, double *cell_
   double area = huiliers_area(cell);
   area /= (EarthRadius*EarthRadius);
 
-  return (area);
+  return area;
 }
 */
+
 static
 void cross_product(const double *restrict a, const double *restrict b, double *restrict c)
 {
@@ -67,31 +68,33 @@ void cross_product(const double *restrict a, const double *restrict b, double *r
   c[2] = a[0]*b[1] - a[1]*b[0];
 }
 
+/*
 static
 double scalar_product(const double *restrict a, const double *restrict b)
 {
   return (a[0]*b[0] + a[1]*b[1] + a[2]*b[2]);
 }
+*/
 
 static
 double norm(const double *restrict a)
 {
   return (a[0]*a[0] + a[1]*a[1] + a[2]*a[2]);
 }
-
+/*
 static
 double mod_cell_area(int num_corners, double *cell_corner_lon, double *cell_corner_lat)
 {
   if ( num_corners < 3 ) return 0;
 
-  /* generalised version based on the ICON code, mo_base_geometry.f90
-     provided by Luis Kornblueh, MPI-M. */
+  // generalised version based on the ICON code, mo_base_geometry.f90
+  // provided by Luis Kornblueh, MPI-M.
 
   int M = num_corners; // number of vertices
   int m; // loop index over number of vertices M
   int i; // loop index over the three dimensions
 
-  double area;
+  double area = 0.0;
   double s[M];
   double ca[M];
   double a[M];
@@ -99,27 +102,22 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double *cell_corn
   double p[M][3];
   double u[M][3];
 
-  /* Convert into cartesian coordinates */
-
+  // Convert into cartesian coordinates
   for ( m = 0; m < M; m++ )
     lonlat_to_xyz(cell_corner_lon[m], cell_corner_lat[m], p[m]);
 
-  /* First, compute cross products Uij = Vi x Vj. */
-
+  // First, compute cross products Uij = Vi x Vj.
   for ( m = 0; m < M; m++ )
     cross_product(p[m], p[(m+1)%M], u[m]);
 
-  /*  Normalize Uij to unit vectors. */
-
-  area = 0.0;
-
+  //  Normalize Uij to unit vectors.
   for ( m = 0; m < M; m++ )
     {
       s[m] = norm(u[m]);
       area += s[m];
     }
 
-  /* Test for a degenerated cells associated with collinear vertices. */
+  // Test for a degenerated cells associated with collinear vertices.
 
   if ( fabs(area) > 0.0 )
     {
@@ -130,19 +128,18 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double *cell_corn
 	for ( i = 0; i < 3; i++ )
 	  u[m][i] = u[m][i]/s[m];
 
-      /*  Compute interior angles Ai as the dihedral angles between planes
-	  by using the definition of the scalar product
+      //  Compute interior angles Ai as the dihedral angles between planes
+      //  by using the definition of the scalar product
+      //
+      //	    ab = |a| |b| cos (phi)
+      //
+      //  As a and b are already normalised this reduces to
+      //
+      //            ab = cos (phi)
 
-	            ab = |a| |b| cos (phi)
-
-	  As a and b are already normalised this reduces to
-
-                    ab = cos (phi)
-
-          There is no explanation so far for the - in the loop below.
-	  But otherwise we don't get the correct results for triangles
-	  and cells. Must have something to do with the theorem.
-      */
+      //  There is no explanation so far for the - in the loop below.
+      //  But otherwise we don't get the correct results for triangles
+      //  and cells. Must have something to do with the theorem.
 
       for ( m = 0; m < M; m++ )
 	{
@@ -152,22 +149,19 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double *cell_corn
 	  a[m] = acos(ca[m]);
 	}
 
-      /*  Compute areas = a1 + a2 + a3 - (M-2) * pi.
-
-	  here for a unit sphere: */
-
+      //  Compute areas = a1 + a2 + a3 - (M-2) * pi. here for a unit sphere:
       area = - (double) (M-2) * M_PI;
 
       for ( m = 0; m < M; m++ )
 	area += a[m];
 
-      //   area *= EarthRadius * EarthRadius;
-
+      // area *= EarthRadius * EarthRadius;
       if ( area < 0.0 ) area = 0.0;
     }
 
-  return (area);
+  return area;
 }
+*/
 
 /** area of a spherical triangle based on L'Huilier's Theorem
   *
@@ -206,7 +200,7 @@ double mod_tri_area(const double *restrict u, const double *restrict v, const do
 
   double area = fabs(4.0 * atan(sqrt(fabs(t))));
 
-  return (area);
+  return area;
 }
 
  /*
@@ -245,7 +239,7 @@ double mod_huiliers_area(int num_corners, double *cell_corner_lon, double *cell_
       if ( i < (num_corners-1) ) { pnt2[0] = pnt3[0]; pnt2[1] = pnt3[1]; pnt2[2] = pnt3[2]; }
     }
 
-  return (sum);
+  return sum;
 }
 
 static
@@ -279,33 +273,28 @@ double mod_huiliers_area2(int num_corners, double *cell_corner_lon, double *cell
       sum += mod_tri_area(pnt1, pnt2, pnt3);
     }
 
-  return (sum);
+  return sum;
 }
 
 
 int gridGenArea(int gridID, double* area)
 {
   int status = 0;
-  int gridtype;
-  int lgrid_gen_bounds = FALSE;
-  int lgriddestroy = FALSE;
-  long i;
-  long nv, gridsize;
-  int* grid_mask = NULL;
-  double* grid_center_lon = NULL;
-  double* grid_center_lat = NULL;
-  double* grid_corner_lon = NULL;
-  double* grid_corner_lat = NULL;
+  bool lgrid_gen_bounds = false;
+  bool lgriddestroy = false;
+  long nv;
 
-  gridsize = gridInqSize(gridID);
-  gridtype = gridInqType(gridID);
+  long gridsize = gridInqSize(gridID);
+  int gridtype = gridInqType(gridID);
+  int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1; 
 
   if ( gridtype != GRID_LONLAT      &&
        gridtype != GRID_GAUSSIAN    &&
        gridtype != GRID_LCC         &&
-       gridtype != GRID_LCC2        &&
-       gridtype != GRID_LAEA        &&
-       gridtype != GRID_SINUSOIDAL  &&
+       projtype != CDI_PROJ_RLL     &&
+       projtype != CDI_PROJ_LAEA    &&
+       projtype != CDI_PROJ_SINU    &&
+       projtype != CDI_PROJ_LCC     &&
        gridtype != GRID_GME         &&
        gridtype != GRID_CURVILINEAR &&
        gridtype != GRID_UNSTRUCTURED )
@@ -317,16 +306,19 @@ int gridGenArea(int gridID, double* area)
     {
       if ( gridtype == GRID_GME )
 	{
-	  lgriddestroy = TRUE;
+	  lgriddestroy = true;
 	  gridID = gridToUnstructured(gridID, 1);
-	  grid_mask = (int*) malloc(gridsize*sizeof(int));
+          /*
+	  grid_mask = (int*) Malloc(gridsize*sizeof(int));
 	  gridInqMaskGME(gridID, grid_mask);
+          if ( grid_mask ) Free(grid_mask);
+          */
 	}
       else
 	{
-	  lgriddestroy = TRUE;
+	  lgriddestroy = true;
 	  gridID = gridToCurvilinear(gridID, 1);
-	  lgrid_gen_bounds = TRUE;
+	  lgrid_gen_bounds = true;
 	}
     }
 
@@ -336,9 +328,9 @@ int gridGenArea(int gridID, double* area)
 	{
 	  if ( gridInqNumber(gridID) > 0 )
 	    {
-	      lgriddestroy = TRUE;
+	      lgriddestroy = true;
 	      gridID = referenceToGrid(gridID);
-	      if ( gridID == -1 ) return (1);
+	      if ( gridID == -1 ) return 1;
 	    }
 	}
     }
@@ -354,14 +346,14 @@ int gridGenArea(int gridID, double* area)
     {
       cdoWarning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
       status = 1;
-      return (status);
+      return status;
     }
 
   if ( nv == 0 )
     {
       cdoWarning("Computation of grid cell area weights failed, grid cell corner coordinates missing!");
       status = 1;
-      return (status);
+      return status;
     }
 
   char xunitstr[CDI_MAX_NAME];
@@ -369,14 +361,14 @@ int gridGenArea(int gridID, double* area)
   gridInqXunits(gridID, xunitstr);
   gridInqYunits(gridID, yunitstr);
 
-  grid_center_lon = (double*) malloc(gridsize*sizeof(double));
-  grid_center_lat = (double*) malloc(gridsize*sizeof(double));
+  double *grid_center_lon = (double*) Malloc(gridsize*sizeof(double));
+  double *grid_center_lat = (double*) Malloc(gridsize*sizeof(double));
 
   gridInqXvals(gridID, grid_center_lon);
   gridInqYvals(gridID, grid_center_lat);
 
-  grid_corner_lon = (double*) malloc(nv*gridsize*sizeof(double));
-  grid_corner_lat = (double*) malloc(nv*gridsize*sizeof(double));
+  double *grid_corner_lon = (double*) Malloc(nv*gridsize*sizeof(double));
+  double *grid_corner_lat = (double*) Malloc(nv*gridsize*sizeof(double));
 
   if ( gridInqYbounds(gridID, NULL) && gridInqXbounds(gridID, NULL) )
     {
@@ -398,7 +390,7 @@ int gridGenArea(int gridID, double* area)
       else
 	{
 	  status = 1;
-	  return (status);
+	  return status;
 	}
     }
   
@@ -415,11 +407,10 @@ int gridGenArea(int gridID, double* area)
   progressInit();
 
 #if defined(_OPENMP)
-#pragma omp parallel for default(none)        \
-  shared(findex, gridsize, area, nv, grid_corner_lon, grid_corner_lat, grid_center_lon, grid_center_lat) \
-  private(i)
+#pragma omp parallel for default(none)  \
+  shared(findex,gridsize,area,nv,grid_corner_lon,grid_corner_lat,grid_center_lon,grid_center_lat)
 #endif
-  for ( i = 0; i < gridsize; ++i )
+  for ( long i = 0; i < gridsize; ++i )
     {
       int lprogress = 1;
       if ( cdo_omp_get_thread_num() != 0 ) lprogress = 0;
@@ -440,15 +431,23 @@ int gridGenArea(int gridID, double* area)
   if ( cdoVerbose )
     {
       double total_area = 0;
-      for ( i = 0; i < gridsize; ++i ) total_area += area[i];
+      for ( long i = 0; i < gridsize; ++i ) total_area += area[i];
       cdoPrint("Total area = %g steradians", total_area);
     }
 
-  free(grid_center_lon);
-  free(grid_center_lat);
-  free(grid_corner_lon);
-  free(grid_corner_lat);
-  if ( grid_mask ) free(grid_mask);
+  if ( gridsize < 20 )
+    {
+      double total_area = 0;
+      for ( long i = 0; i < gridsize; ++i ) total_area += area[i];
+      int nzero = 0;
+      for ( long i = 0; i < gridsize; ++i ) if ( IS_EQUAL(area[i], 0.) ) nzero++;
+      if ( IS_EQUAL(total_area, 0.) ) status = 2;
+    }
 
-  return (status);
+  Free(grid_center_lon);
+  Free(grid_center_lat);
+  Free(grid_corner_lon);
+  Free(grid_corner_lat);
+
+  return status;
 }
