@@ -42,7 +42,7 @@ int cdf_openread(const char *filename)
   cdoWarning("NetCDF support not compiled in!");
 #endif
 
-  return (fileID);
+  return fileID;
 }
 
 
@@ -87,7 +87,7 @@ int gridFromNCfile(const char *gridfile)
 	   nc_inq_varid(nc_file_id, "grid_center_lat", &nc_gridlat_id)  != NC_NOERR || 
 	   nc_inq_varid(nc_file_id, "grid_center_lon", &nc_gridlon_id)  != NC_NOERR || 
 	   nc_inq_varid(nc_file_id, "grid_corner_lat", &nc_gridclat_id) != NC_NOERR || 
-	   nc_inq_varid(nc_file_id, "grid_corner_lon", &nc_gridclon_id) != NC_NOERR ) return (gridID);
+	   nc_inq_varid(nc_file_id, "grid_corner_lon", &nc_gridclon_id) != NC_NOERR ) return gridID;
 
       nce(nc_get_var_int(nc_file_id, nc_griddims_id, grid_dims));
 
@@ -115,8 +115,8 @@ int gridFromNCfile(const char *gridfile)
       grid.ybounds = (double*) Malloc(grid.nvertex*grid.size*sizeof(double));
 
       nce(nc_inq_vartype(nc_file_id, nc_gridlat_id, &xtype));
-      if ( xtype == NC_FLOAT )  grid.prec = DATATYPE_FLT32;
-      else                      grid.prec = DATATYPE_FLT64;
+      if ( xtype == NC_FLOAT )  grid.prec = CDI_DATATYPE_FLT32;
+      else                      grid.prec = CDI_DATATYPE_FLT64;
 
       nce(nc_get_var_double(nc_file_id, nc_gridlon_id, grid.xvals));
       nce(nc_get_var_double(nc_file_id, nc_gridlat_id, grid.yvals));
@@ -154,7 +154,7 @@ int gridFromNCfile(const char *gridfile)
   cdoWarning("NetCDF support not compiled in!");
 #endif
 
-  return (gridID);
+  return gridID;
 }
 
 
@@ -185,7 +185,7 @@ void writeNCgrid(const char *gridfile, int gridID, int *grid_imask)
   gridtype = gridInqType(gridID);
   gridsize = gridInqSize(gridID);
 
-  if ( gridInqPrec(gridID) == DATATYPE_FLT64 ) xtype = NC_DOUBLE;
+  if ( gridInqPrec(gridID) == CDI_DATATYPE_FLT64 ) xtype = NC_DOUBLE;
   else                                         xtype = NC_FLOAT;
 
   if ( gridtype == GRID_CURVILINEAR )
