@@ -30,7 +30,7 @@ long find_element(double x, long nelem, const double *restrict array)
   if ( array[0] < array[nelem-1] ) // ascending order
     {
       /* return the length of the array if x is out of bounds */
-      if ( x < array[0] || x > array[nelem-1] ) return (nelem);
+      if ( x < array[0] || x > array[nelem-1] ) return nelem;
 
       /* search for the interval in which x fits */
       // implementation: binary search algorithm
@@ -56,7 +56,7 @@ long find_element(double x, long nelem, const double *restrict array)
   else
     {
       /* return the length of the array if x is out of bounds */
-      if ( x < array[nelem-1] || x > array[0] ) return (nelem);
+      if ( x < array[nelem-1] || x > array[0] ) return nelem;
 
       /* search for the interval in which x fits */
       // implementation: binary search algorithm
@@ -82,7 +82,7 @@ long find_element(double x, long nelem, const double *restrict array)
 
   if ( mid > 1 && IS_EQUAL(x,array[mid-1]) ) mid--;
 
-  return (mid);
+  return mid;
 }
 
 /*
@@ -101,7 +101,7 @@ long find_element(double x, long nelem, const double *array)
 	if ( x >= array[ii] && x <= array[ii-1] ) break;
     }
 
-  return (ii);
+  return ii;
 }
 */
 
@@ -118,7 +118,7 @@ int rect_grid_search(long *ii, long *jj, double x, double y, long nxm, long nym,
       if ( *ii < nxm ) lfound = 1;
     }
 
-  return (lfound);
+  return lfound;
 }
 
 
@@ -171,7 +171,7 @@ int rect_grid_search2(long *imin, long *imax, double xmin, double xmax, long nxm
 	}
     }
 
-  return (lfound);
+  return lfound;
 }
 
 
@@ -202,60 +202,31 @@ double intlinarr2p(long nxm, long nym, double **fieldm, const double *xm, const
   return value;
 }
 
-static
-void rect_find_ij_weights(double plon, double plat, long ii, long jj, const double *restrict xm, const double *restrict ym, double *ig, double *jg)
-{
-  /*
-    wgts[0] = (plon-xm[ii])   * (plat-ym[jj])   / ((xm[ii-1]-xm[ii]) * (ym[jj-1]-ym[jj]));
-    wgts[1] = (plon-xm[ii-1]) * (plat-ym[jj])   / ((xm[ii]-xm[ii-1]) * (ym[jj-1]-ym[jj]));
-    wgts[2] = (plon-xm[ii-1]) * (plat-ym[jj-1]) / ((xm[ii]-xm[ii-1]) * (ym[jj]-ym[jj-1]));
-    wgts[3] = (plon-xm[ii])   * (plat-ym[jj-1]) / ((xm[ii-1]-xm[ii]) * (ym[jj]-ym[jj-1]));
-  */
-  double iw, jw;
-  double wgts0 = (plon-xm[ii])   * (plat-ym[jj])   / ((xm[ii-1]-xm[ii]) * (ym[jj-1]-ym[jj]));
-  double wgts1 = (plon-xm[ii-1]) * (plat-ym[jj])   / ((xm[ii]-xm[ii-1]) * (ym[jj-1]-ym[jj]));
-
-  iw = 1./(wgts0/wgts1 + 1.);
-  jw = 1. - wgts1/iw;
-
-  *ig = iw;
-  *jg = jw;
-}
-
 static
 void intlinarr2(double missval, int lon_is_circular,
 		long nxm, long nym,  const double *restrict fieldm, const double *restrict xm, const double *restrict ym,
 		long gridsize2, double *field, const double *restrict x, const double *restrict y)
 {
-  long i, ii, jj;
-  long gridsize1;
   long nlon1 = nxm;
   double findex = 0;
-  int *grid1_mask = NULL;
 
   if ( lon_is_circular ) nlon1--;
-  gridsize1 = nlon1*nym;
+  long gridsize1 = nlon1*nym;
 
-  grid1_mask = (int*) calloc(1, gridsize1*sizeof(int));
-  for ( jj = 0; jj < nym; ++jj )
-    for ( ii = 0; ii < nlon1; ++ii )
-      {
-	if ( !DBL_IS_EQUAL(fieldm[jj*nlon1+ii], missval) ) grid1_mask[jj*nlon1+ii] = 1;
-      }
+  bool *grid1_mask = (bool*) Malloc(gridsize1*sizeof(bool));
+  for ( long jj = 0; jj < nym; ++jj )
+    for ( long ii = 0; ii < nlon1; ++ii )
+      grid1_mask[jj*nlon1+ii] = !DBL_IS_EQUAL(fieldm[jj*nlon1+ii], missval);
 
   progressInit();
 
 #if defined(_OPENMP)
 #pragma omp parallel for default(none) \
-  shared(ompNumThreads, field, fieldm, x, y, xm, ym, nxm, nym, gridsize2, missval, findex, nlon1, lon_is_circular, grid1_mask) \
-  private(i, jj, ii)
+  shared(ompNumThreads, field, fieldm, x, y, xm, ym, nxm, nym, gridsize2, missval, findex, nlon1, lon_is_circular, grid1_mask)
 #endif
-  for ( i = 0; i < gridsize2; ++i )
+  for ( int i = 0; i < gridsize2; ++i )
     {
       int src_add[4];                /*  address for the four source points    */
-      long n;
-      long iix;
-      int lfound;
       int lprogress = 1;
       if ( cdo_omp_get_thread_num() != 0 ) lprogress = 0;
 
@@ -267,11 +238,12 @@ void intlinarr2(double missval, int lon_is_circular,
       findex++;
       if ( lprogress ) progressStatus(0, 1, findex/gridsize2);
 
-      lfound = rect_grid_search(&ii, &jj, x[i], y[i], nxm, nym, xm, ym); 
+      long ii, jj;
+      int lfound = rect_grid_search(&ii, &jj, x[i], y[i], nxm, nym, xm, ym); 
 
       if ( lfound )
 	{
-	  iix = ii;
+	  long iix = ii;
 	  if ( lon_is_circular && iix == (nxm-1) ) iix = 0;
 	  src_add[0] = (jj-1)*nlon1+(ii-1);
 	  src_add[1] = (jj-1)*nlon1+(iix);
@@ -279,14 +251,13 @@ void intlinarr2(double missval, int lon_is_circular,
 	  src_add[3] = (jj)*nlon1+(iix);
 
 	  /* Check to see if points are missing values */
-	  for ( n = 0; n < 4; ++n )
+	  for ( int n = 0; n < 4; ++n )
 	    if ( ! grid1_mask[src_add[n]] ) lfound = 0;
 	}
 
       if ( lfound )
 	{
 	  double wgts[4];
-
 	  wgts[0] = (x[i]-xm[ii])   * (y[i]-ym[jj])   / ((xm[ii-1]-xm[ii]) * (ym[jj-1]-ym[jj]));
 	  wgts[1] = (x[i]-xm[ii-1]) * (y[i]-ym[jj])   / ((xm[ii]-xm[ii-1]) * (ym[jj-1]-ym[jj]));
 	  wgts[3] = (x[i]-xm[ii-1]) * (y[i]-ym[jj-1]) / ((xm[ii]-xm[ii-1]) * (ym[jj]-ym[jj-1]));
@@ -309,350 +280,14 @@ void intlinarr2(double missval, int lon_is_circular,
 
 	  
 	  field[i] = 0;
-	  for ( n = 0; n < 4; ++n )
+	  for ( int n = 0; n < 4; ++n )
 	    field[i] += fieldm[src_add[n]] * wgts[n];
 	}
     }
  
   if ( findex < gridsize2 ) progressStatus(0, 1, 1);
 
-  if ( grid1_mask ) free(grid1_mask);
-}
-
-static
-void restrict_boundbox(const double *restrict grid_bound_box, double *restrict bound_box)
-{
-  if ( bound_box[0] < grid_bound_box[0] && bound_box[1] > grid_bound_box[0] ) bound_box[0] = grid_bound_box[0];
-  if ( bound_box[1] > grid_bound_box[1] && bound_box[0] < grid_bound_box[1] ) bound_box[1] = grid_bound_box[1];
-  //  if ( bound_box[2] < grid_bound_box[2] && bound_box[3] > grid_bound_box[2] ) bound_box[2] = grid_bound_box[2];
-  //  if ( bound_box[3] > grid_bound_box[3] && bound_box[2] < grid_bound_box[3] ) bound_box[3] = grid_bound_box[3];
-}
-
-static
-void boundbox_from_corners(long ic, long nc, const double *restrict corner_lon,
-			   const double *restrict corner_lat, double *restrict bound_box)
-{
-  long inc, j;
-  double clon, clat;
-
-  inc = ic*nc;
-  clat = corner_lat[inc];
-  clon = corner_lon[inc];
-
-  bound_box[0] = clat;
-  bound_box[1] = clat;
-  bound_box[2] = clon;
-  bound_box[3] = clon;
-
-  for ( j = 1; j < nc; ++j )
-    {
-      clat = corner_lat[inc+j];
-      clon = corner_lon[inc+j];
-      if ( clat < bound_box[0] ) bound_box[0] = clat;
-      if ( clat > bound_box[1] ) bound_box[1] = clat;
-      if ( clon < bound_box[2] ) bound_box[2] = clon;
-      if ( clon > bound_box[3] ) bound_box[3] = clon;
-    }
-}
-
-#if defined(HAVE_LIBYAC)
-#include "clipping.h"
-#include "area.h"
-#endif
-
-static
-void intconarr2(double missval, int lon_is_circular,
-		long nxm, long nym,  const double *restrict fieldm, const double *restrict xm, const double *restrict ym,
-		long nc2, long gridsize2, double *field, const double *restrict x, const double *restrict y)
-{
-  long ndeps;
-  int *deps;
-  long i, ii = -1, jj = -1;
-  long gridsize1;
-  long nlon1 = nxm;
-  long nx, ny;
-  double findex = 0;
-  int *grid1_mask = NULL;
-
-  nx = nxm - 1;
-  ny = nym - 1;
-  printf(" nx, ny %ld %ld\n", nx, ny);
-  //if ( lon_is_circular ) nlon1--;
-  gridsize1 = nx*ny;
-
-  deps = (int*) malloc(gridsize1*sizeof(int));
-
-  grid1_mask = (int*) calloc(1, gridsize1*sizeof(int));
-  for ( jj = 0; jj < ny; ++jj )
-    for ( ii = 0; ii < nx; ++ii )
-      {
-	if ( !DBL_IS_EQUAL(fieldm[jj*nx+ii], missval) ) grid1_mask[jj*nx+ii] = 1;
-      }
-
-  double grid1_bound_box[4];
-  grid1_bound_box[0] = ym[0];
-  grid1_bound_box[1] = ym[ny];
-  if ( ym[0] > ym[ny] )
-    {
-      grid1_bound_box[0] = ym[ny];
-      grid1_bound_box[1] = ym[0];
-    }
-  grid1_bound_box[2] = xm[0];
-  grid1_bound_box[3] = xm[nx];
- 
-  progressInit();
-
-#if defined(HAVE_LIBYAC)
-  enum edge_type quad_type[] = {GREAT_CIRCLE, GREAT_CIRCLE, GREAT_CIRCLE, GREAT_CIRCLE}; // not used !
-  // enum edge_type quad_type[] = {LON_CIRCLE, LON_CIRCLE, LON_CIRCLE, LON_CIRCLE};
-
-  int n;
-  double weight_sum;
-
-  double *weight;
-  weight = (double*) malloc(gridsize1*sizeof(double));
-
-  double tgt_area;
-  double *area;
-  area = (double*) malloc(gridsize1*sizeof(double));
-
-  struct grid_cell *SourceCell;
-  SourceCell = (struct grid_cell*) malloc(gridsize1  * sizeof(struct grid_cell));
-
-  for ( int n = 0; n <  gridsize1; n++ ) {
-    SourceCell[n].num_corners   = 4;
-    SourceCell[n].edge_type     = quad_type;
-    SourceCell[n].coordinates_x = (double*) malloc(4 * sizeof(double));
-    SourceCell[n].coordinates_y = (double*) malloc(4 * sizeof(double));
-  }
-
-  struct grid_cell  TargetCell;
-
-  TargetCell.num_corners   = nc2;
-  TargetCell.edge_type     = quad_type;
-  TargetCell.coordinates_x = (double*) malloc(nc2 * sizeof(double));
-  TargetCell.coordinates_y = (double*) malloc(nc2 * sizeof(double));
-
-  unsigned const * curr_deps;
-#endif
-
-  /*
-#if defined(_OPENMP)
-#pragma omp parallel for default(none) \
-  shared(ompNumThreads, field, fieldm, x, y, xm, ym, nxm, nym, gridsize2, missval, findex, nlon1, lon_is_circular, grid1_mask, nc2) \
-  private(i, jj, ii)
-#endif
-  */
-  for ( i = 0; i < gridsize2; ++i )
-    {
-      int src_add[4];                /*  address for the four source points    */
-      long n;
-      long iix;
-      int lfound;
-      int lprogress = 1;
-      ndeps = 0;
-      /*
-#if defined(_OPENMP)
-      if ( cdo_omp_get_thread_num() != 0 ) lprogress = 0;
-#endif
-      */
-      field[i] = missval;
-      
-      // printf("%ld lonb: %g %g %g %g latb: %g %g %g %g\n", i+1, x[i*nc2]*RAD2DEG, x[i*nc2+1]*RAD2DEG, x[i*nc2+2]*RAD2DEG, x[i*nc2+3]*RAD2DEG, y[i*nc2]*RAD2DEG, y[i*nc2+1]*RAD2DEG, y[i*nc2+2]*RAD2DEG, y[i*nc2+3]*RAD2DEG);
-
-
-      double bound_lon1, bound_lon2;
-      double bound_box[4];
-      boundbox_from_corners(i, nc2, x, y, bound_box);
-      restrict_boundbox(grid1_bound_box, bound_box);
-      bound_lon1 = bound_box[2];
-      bound_lon2 = bound_box[3];
-      //printf("bound_box %ld  lon: %g %g lat: %g %g\n", i, bound_box[2]*RAD2DEG, bound_box[3]*RAD2DEG, bound_box[0]*RAD2DEG, bound_box[1]*RAD2DEG);
-      /*
-#if defined(_OPENMP)
-#include "pragma_omp_atomic_update.h"
-#endif
-      */
-      findex++;
-      if ( lprogress ) progressStatus(0, 1, findex/gridsize2);
-
-      //      lfound = rect_grid_search(&ii, &jj, x[i], y[i], nxm, nym, xm, ym);
-      //     for ( int k = 0; k < nxm; ++k ) printf("x: %d %g\n", k+1, xm[k]);
-      //    for ( int k = 0; k < nym; ++k ) printf("y: %d %g\n", k+1, ym[k]*RAD2DEG);
-      long imin = nxm, imax = -1, jmin = nym, jmax = -1;
-      long im, jm;
-
-      lfound = rect_grid_search2(&jmin, &jmax, bound_box[0], bound_box[1], nym, ym);
-      bound_lon1 = bound_box[2];
-      bound_lon2 = bound_box[3];
-      if ( bound_lon1 <= grid1_bound_box[3] && bound_lon2 >= grid1_bound_box[2] )
-	{
-	  //printf("b1 %g %g\n", bound_lon1*RAD2DEG, bound_lon2*RAD2DEG);
-	  if ( bound_lon1 < grid1_bound_box[2] && bound_lon2 > grid1_bound_box[2] ) bound_lon1 = grid1_bound_box[2];
-	  if ( bound_lon2 > grid1_bound_box[3] && bound_lon1 < grid1_bound_box[3] ) bound_lon2 = grid1_bound_box[3];
-	  lfound = rect_grid_search2(&imin, &imax, bound_lon1, bound_lon2, nxm, xm);
-	  //printf("imin %ld  imax %ld  jmin %ld jmax %ld\n", imin, imax, jmin, jmax);
-	  for ( jm = jmin; jm <= jmax; ++jm )
-	    for ( im = imin; im <= imax; ++im )
-	      deps[ndeps++] = jm*nx + im;
-	}
-
-
-      bound_lon1 = bound_box[2];
-      bound_lon2 = bound_box[3];
-      if ( bound_lon1 <= grid1_bound_box[2] && bound_lon2 > grid1_bound_box[2] )
-	{
-	  bound_lon1 += 2*M_PI;
-	  bound_lon2 += 2*M_PI;
-	  //printf("b2 %g %g\n", bound_lon1*RAD2DEG, bound_lon2*RAD2DEG);
-	  if ( bound_lon1 < grid1_bound_box[2] && bound_lon2 > grid1_bound_box[2] ) bound_lon1 = grid1_bound_box[2];
-	  if ( bound_lon2 > grid1_bound_box[3] && bound_lon1 < grid1_bound_box[3] ) bound_lon2 = grid1_bound_box[3];
-	  lfound = rect_grid_search2(&imin, &imax, bound_lon1, bound_lon2, nxm, xm);
-	  //printf("imin %ld  imax %ld  jmin %ld jmax %ld\n", imin, imax, jmin, jmax);
-	  for ( jm = jmin; jm <= jmax; ++jm )
-	    for ( im = imin; im <= imax; ++im )
-	      deps[ndeps++] = jm*nx + im;
-	}
-
-      bound_lon1 = bound_box[2];
-      bound_lon2 = bound_box[3];
-      if ( bound_lon1 < grid1_bound_box[3] && bound_lon2 >= grid1_bound_box[3] )
-	{
-	  bound_lon1 -= 2*M_PI;
-	  bound_lon2 -= 2*M_PI;
-	  //printf("b3 %g %g\n", bound_lon1*RAD2DEG, bound_lon2*RAD2DEG);
-	  if ( bound_lon1 < grid1_bound_box[2] && bound_lon2 > grid1_bound_box[2] ) bound_lon1 = grid1_bound_box[2];
-	  if ( bound_lon2 > grid1_bound_box[3] && bound_lon1 < grid1_bound_box[3] ) bound_lon2 = grid1_bound_box[3];
-	  lfound = rect_grid_search2(&imin, &imax, bound_lon1, bound_lon2, nxm, xm);
-	  //printf("imin %ld  imax %ld  jmin %ld jmax %ld\n", imin, imax, jmin, jmax);
-	  for ( jm = jmin; jm <= jmax; ++jm )
-	    for ( im = imin; im <= imax; ++im )
-	      deps[ndeps++] = jm*nx + im;
-	}
-
-      //for ( long id = 0; id < ndeps; ++id )
-	//	printf("dep %ld %d\n", id+1, deps[id]);
-      /*
-      if ( bound_lon1 < grid1_bound_box[2] && bound_box[3] >= grid1_bound_box[2] )
-	lfound = rect_grid_search2(&imin, &imax, bound_box[2], bound_box[3], nxm, xm);
-      */
-
-#if defined(HAVE_LIBYAC)
-      int index2 = i;
-      /*
-      int ilat2 = index2/nlonOut;
-      int ilon2 = index2 - ilat2*nlonOut;
-      */
-      double addtest = 0;
-      //if ( i == 296 ) addtest = 360*DEG2RAD;
-      for ( int ic = 0; ic < nc2; ++ic )
-	{
-	  TargetCell.coordinates_x[ic] =  addtest+x[index2*nc2+ic];
-	  TargetCell.coordinates_y[ic] =          y[index2*nc2+ic];
-	}
-
-      if ( cdoVerbose )
-	{
-	  printf("target:       ");
-	  for ( int n = 0; n < nc2; ++n )
-	    printf(" %g %g", TargetCell.coordinates_x[n]/DEG2RAD, TargetCell.coordinates_y[n]/DEG2RAD);
-	  printf("\n");
-	}
-      /*
-      if ( cdoVerbose )
-	printf("num_deps_per_element %d %d\n", i, ndeps);
-      */
-      int num_deps = ndeps;
-      int nSourceCells = num_deps;
-
-      for ( int k = 0; k < num_deps; ++k )
-	{
-	  int index1 = deps[k];
-	  int ilat1 = index1/nx;
-	  int ilon1 = index1 - ilat1*nx;
-	  /*
-	  if ( cdoVerbose )
-	    printf("  dep: %d %d %d %d %d %d\n", k, nlonOut, nlatOut, index1, ilon1, ilat1);
-	  */
-	  if ( ym[ilat1] < ym[ilat1+1] )
-	    {
-	      SourceCell[k].coordinates_x[0] =  addtest+xm[ilon1];
-	      SourceCell[k].coordinates_y[0] =          ym[ilat1];
-	      SourceCell[k].coordinates_x[1] =  addtest+xm[ilon1+1];
-	      SourceCell[k].coordinates_y[1] =          ym[ilat1];
-	      SourceCell[k].coordinates_x[2] =  addtest+xm[ilon1+1];
-	      SourceCell[k].coordinates_y[2] =          ym[ilat1+1];
-	      SourceCell[k].coordinates_x[3] =  addtest+xm[ilon1];
-	      SourceCell[k].coordinates_y[3] =          ym[ilat1+1];
-	    }
-	  else
-	    {
-	      SourceCell[k].coordinates_x[0] =  addtest+xm[ilon1];
-	      SourceCell[k].coordinates_y[0] =          ym[ilat1+1];
-	      SourceCell[k].coordinates_x[1] =  addtest+xm[ilon1+1];
-	      SourceCell[k].coordinates_y[1] =          ym[ilat1+1];
-	      SourceCell[k].coordinates_x[2] =  addtest+xm[ilon1+1];
-	      SourceCell[k].coordinates_y[2] =          ym[ilat1];
-	      SourceCell[k].coordinates_x[3] =  addtest+xm[ilon1];
-	      SourceCell[k].coordinates_y[3] =          ym[ilat1];
-	    }
-	  if ( cdoVerbose )
-	    {
-	      printf("source: %d %d", num_deps, k);
-	      for ( int n = 0; n < 4; ++n )
-		printf(" %g %g", SourceCell[k].coordinates_x[n]/DEG2RAD, SourceCell[k].coordinates_y[n]/DEG2RAD);
-	      printf("\n");
-	    }
-	}
-      
-      compute_overlap_areas ( nSourceCells, SourceCell, TargetCell, area);
-
-      tgt_area = huiliers_area(TargetCell);
-      // tgt_area = cell_area(TargetCell);
-      for (n = 0; n < nSourceCells; ++n)
-	weight[n] = area[n] / tgt_area;
-
-      correct_weights ( nSourceCells, weight );
-
-      field[i] = missval;
-      if ( num_deps ) field[i] = 0;
-      for ( int k = 0; k < num_deps; ++k )
-	{
-	  int index1 = deps[k];
-	  /*
-	  int ilat1 = index1/nx;
-	  int ilon1 = index1 - ilat1*nx;
-	  long add1, add2;
-
-	  add1 = index1;
-	  add2 = index2;
-
-	  yar_store_link_cnsrv(&remap.vars, add1, add2, weight[k]);
-	  */
-	  if ( weight[k] > 0 )
-	    {
-	      if ( cdoVerbose )
-		printf("tgt_cell_add %ld, src_cell_add %ld,  weight[n] %g, tgt_area  %g\n", i, index1,  weight[k], tgt_area);
-	      //printf("tgt_cell_add %ld, n %ld, src_cell_add %ld,  weight[n] %g, tgt_area  %g\n", i, k, index1,  weight[k], tgt_area);
-	      field[i] += fieldm[index1] * weight[k];
-	    }
-	  /*
-	  if ( cdoVerbose )
-	    printf("  result dep: %ld %d %d  %g\n", i, k, index1, weight[k]);
-	  */
-	}
-#endif
-    }
- 
-#if defined(HAVE_LIBYAC)
-  free(weight);
-  free(area);
-#endif
-
-  if ( findex < gridsize2 ) progressStatus(0, 1, 1);
-
-  if ( deps ) free(deps);
-  if ( grid1_mask ) free(grid1_mask);
+  if ( grid1_mask ) Free(grid1_mask);
 }
 
 
@@ -663,11 +298,9 @@ double intlin(double x, double y1, double x1, double y2, double x2)
 
     Uwe Schulzweida  04/05/1995
   */
-  double value;
-  
-  value = (y2*(x-x1)+y1*(x2-x)) / (x2-x1);
+  double value = (y2*(x-x1)+y1*(x2-x)) / (x2-x1);
 
-  return (value);
+  return value;
 }
 
 
@@ -678,89 +311,85 @@ void intlinarr(long nxm, double *ym, double *xm, int nx, double *y, double *x)
 
     Uwe Schulzweida  04/05/1995
   */
-  long j, jj;
-
-  for ( jj = 1; jj < nxm; jj++ )
-    for ( j = 0; j < nx; j++ )
+  for ( long jj = 1; jj < nxm; jj++ )
+    for ( long j = 0; j < nx; j++ )
       if ( x[j] >= xm[jj-1] && x[j] <= xm[jj] )
 	y[j] = intlin(x[j], ym[jj-1], xm[jj-1], ym[jj], xm[jj]);
 }
 
 
-void intgridbil(field_t *field1, field_t *field2)
+void intgridbil(field_type *field1, field_type *field2)
 {
-  int nlon1, nlat1;
-  int nlon2, nlat2;
-  int ilat;
-  int gridID1, gridID2;
-  int i, nmiss;
-  int lon_is_circular;
-  double *lon1, *lat1;
-  double **array1_2D = NULL;
-  double **field;
-  double *array = NULL;
-  double *array1, *array2;
-  double missval;
-  char units[CDI_MAX_NAME];
-  /* static int index = 0; */
+  char xunits[CDI_MAX_NAME];
+  int gridID1 = field1->grid;
+  int gridID2 = field2->grid;
+  double *array1 = field1->ptr;
+  double *array2 = field2->ptr;
+  double missval = field1->missval;
 
-  gridID1 = field1->grid;
-  gridID2 = field2->grid;
-  array1  = field1->ptr;
-  array2  = field2->ptr;
-  missval = field1->missval;
+  int nlon1 = gridInqXsize(gridID1);
+  int nlat1 = gridInqYsize(gridID1);
 
-  if ( ! (gridInqXvals(gridID1, NULL) && gridInqYvals(gridID1, NULL)) )
-    cdoAbort("Source grid has no values");
+  int lon_is_circular = 0;
+  
+  bool lgeorefgrid = true;
+  if ( grid_is_distance_generic(gridID1) && grid_is_distance_generic(gridID2) ) lgeorefgrid = false;
 
-  lon_is_circular = gridIsCircular(gridID1);
-
-  nlon1 = gridInqXsize(gridID1);
-  nlat1 = gridInqYsize(gridID1);
-
-  array1_2D = (double **) malloc(nlat1*sizeof(double *));
-  for ( ilat = 0; ilat < nlat1; ilat++ )
+  double **array1_2D = (double **) Malloc(nlat1*sizeof(double *));
+  for ( int ilat = 0; ilat < nlat1; ilat++ )
     array1_2D[ilat] = array1 + ilat*nlon1;
 
-  if ( lon_is_circular ) nlon1 += 1;
-  lon1 = (double*) malloc(nlon1*sizeof(double));
-  lat1 = (double*) malloc(nlat1*sizeof(double));
+  if ( lgeorefgrid )
+    {
+      if ( ! (gridInqXvals(gridID1, NULL) && gridInqYvals(gridID1, NULL)) )
+        cdoAbort("Source grid has no coordinate values!");
+  
+      lon_is_circular = gridIsCircular(gridID1);
+
+      if ( lon_is_circular ) nlon1 += 1;
+    }
+  
+  double *lon1 = (double*) Malloc(nlon1*sizeof(double));
+  double *lat1 = (double*) Malloc(nlat1*sizeof(double));
   gridInqXvals(gridID1, lon1);
   gridInqYvals(gridID1, lat1);
-  if ( lon_is_circular ) lon1[nlon1-1] = 0;
 
-  gridInqXunits(gridID1, units);
+  if ( lgeorefgrid )
+    {
+      if ( lon_is_circular ) lon1[nlon1-1] = 0;
 
-  grid_to_radian(units, nlon1, lon1, "grid1 center lon"); 
-  grid_to_radian(units, nlat1, lat1, "grid1 center lat"); 
+      gridInqXunits(gridID1, xunits);
 
-  if ( lon_is_circular ) lon1[nlon1-1] = lon1[0] + 2*M_PI;
+      grid_to_radian(xunits, nlon1, lon1, "grid1 center lon"); 
+      grid_to_radian(xunits, nlat1, lat1, "grid1 center lat"); 
 
-  nlon2 = gridInqXsize(gridID2);
-  nlat2 = gridInqYsize(gridID2);
+      if ( lon_is_circular ) lon1[nlon1-1] = lon1[0] + 2*M_PI;
+    }
+  
+  int xsize2 = gridInqXsize(gridID2);
+  int ysize2 = gridInqYsize(gridID2);
 
-  if ( nlon2 == 1 && nlat2 == 1 )
+  if ( xsize2 == 1 && ysize2 == 1 )
     {
       double lon2, lat2;
-
       gridInqXvals(gridID2, &lon2);
       gridInqYvals(gridID2, &lat2);
 
-      gridInqXunits(gridID2, units);
+      gridInqXunits(gridID2, xunits);
 
-      grid_to_radian(units, nlon2, &lon2, "grid2 center lon"); 
-      grid_to_radian(units, nlat2, &lat2, "grid2 center lat"); 
+      grid_to_radian(xunits, xsize2, &lon2, "grid2 center lon"); 
+      grid_to_radian(xunits, ysize2, &lat2, "grid2 center lat"); 
 
       if ( lon2 < lon1[0] ) lon2 += 2*M_PI;
 
       if ( lon2 > lon1[nlon1-1] )
 	{
-	  field  = array1_2D;
-	  array1_2D = (double **) malloc(nlat1*sizeof(double *));
-	  lon1 = (double*) realloc(lon1, (nlon1+1)*sizeof(double));
-	  array = (double*) malloc(nlat1*(nlon1+1)*sizeof(double));
+	  double **field = array1_2D;
+	  array1_2D = (double **) Malloc(nlat1*sizeof(double *));
+	  lon1 = (double*) Realloc(lon1, (nlon1+1)*sizeof(double));
+	  double *array = (double*) Malloc(nlat1*(nlon1+1)*sizeof(double));
 
-	  for ( ilat = 0; ilat < nlat1; ilat++ )
+	  for ( int ilat = 0; ilat < nlat1; ilat++ )
 	    {
 	      array1_2D[ilat] = array + ilat*(nlon1+1);  
 	      memcpy(array1_2D[ilat], field[ilat], nlon1*sizeof(double));
@@ -768,7 +397,8 @@ void intgridbil(field_t *field1, field_t *field2)
 	      lon1[nlon1] = lon1[0] + 2*M_PI;
 	    }
 	  nlon1++;
-	  free(field);
+	  Free(field);
+          Free(array);
 	}
 
       if ( lon2 < lon1[0] || lon2 > lon1[nlon1-1] )
@@ -785,184 +415,78 @@ void intgridbil(field_t *field1, field_t *field2)
     }
   else
     {
-      int gridsize2;
-      double *lon2, *lat2;
+      if ( lgeorefgrid )
+        {
+          if ( gridInqType(gridID2) == GRID_GME ) gridID2 = gridToUnstructured(gridID2, 0);
 
-      if ( gridInqType(gridID2) == GRID_GME ) gridID2 = gridToUnstructured(gridID2, 0);
+          if ( gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR )
+            gridID2 = gridToCurvilinear(gridID2, 0);
 
-      if ( gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR )
-	gridID2 = gridToCurvilinear(gridID2, 0);
+          if ( ! (gridInqXvals(gridID2, NULL) && gridInqYvals(gridID2, NULL)) )
+            cdoAbort("Target grid has no coordinate values!");
+        }
+      
+      int gridsize2 = gridInqSize(gridID2);
 
-      if ( ! (gridInqXvals(gridID2, NULL) && gridInqYvals(gridID2, NULL)) )
-	cdoAbort("Target grid has no values");
+      double *xvals2 = (double*) Malloc(gridsize2*sizeof(double));
+      double *yvals2 = (double*) Malloc(gridsize2*sizeof(double));
 
-      gridsize2 = gridInqSize(gridID2);
+      if ( lgeorefgrid )
+        {
+          gridInqXvals(gridID2, xvals2);
+          gridInqYvals(gridID2, yvals2);
 
-      lon2 = (double*) malloc(gridsize2*sizeof(double));
-      lat2 = (double*) malloc(gridsize2*sizeof(double));
-      gridInqXvals(gridID2, lon2);
-      gridInqYvals(gridID2, lat2);
+          gridInqXunits(gridID2, xunits);
+          
+          grid_to_radian(xunits, gridsize2, xvals2, "grid2 center lon"); 
+          grid_to_radian(xunits, gridsize2, yvals2, "grid2 center lat"); 
 
-      gridInqXunits(gridID2, units);
+          for ( int i = 0; i < gridsize2; ++i )
+            {
+              if ( xvals2[i] < lon1[0]       ) xvals2[i] += 2*M_PI;
+              if ( xvals2[i] > lon1[nlon1-1] ) xvals2[i] -= 2*M_PI;
+            }
+        }
+      else
+        {
+          double *xcoord = (double*) Malloc(xsize2*sizeof(double));
+          double *ycoord = (double*) Malloc(ysize2*sizeof(double));
 
-      grid_to_radian(units, gridsize2, lon2, "grid2 center lon"); 
-      grid_to_radian(units, gridsize2, lat2, "grid2 center lat"); 
-
-      for ( int i = 0; i < gridsize2; ++i )
-	{
-	  if ( lon2[i] < lon1[0]       ) lon2[i] += 2*M_PI;
-	  if ( lon2[i] > lon1[nlon1-1] ) lon2[i] -= 2*M_PI;
-	}
+          gridInqXvals(gridID2, xcoord);
+          gridInqYvals(gridID2, ycoord);
+  
+          for ( int j = 0; j < ysize2; ++j )
+            for ( int i = 0; i < xsize2; ++i )
+              {
+                xvals2[j*xsize2+i] = xcoord[i]; 
+                yvals2[j*xsize2+i] = ycoord[j]; 
+              }
+  
+          Free(xcoord);
+          Free(ycoord);
+        }
 
       intlinarr2(missval, lon_is_circular, 
 		 nlon1, nlat1, array1, lon1, lat1,
-		 gridsize2, array2, lon2, lat2);
+		 gridsize2, array2, xvals2, yvals2);
 
-      nmiss = 0;
-      for ( i = 0; i < gridsize2; ++i )
+      int nmiss = 0;
+      for ( int i = 0; i < gridsize2; ++i )
 	if ( DBL_IS_EQUAL(array2[i], missval) ) nmiss++;
 
       field2->nmiss = nmiss;
 
-      free(lon2);
-      free(lat2);
+      Free(xvals2);
+      Free(yvals2);
     }
 
-  if (array) free(array);
-  free(lon1);
-  free(lat1);
-  free(array1_2D);
+  Free(lon1);
+  Free(lat1);
+  Free(array1_2D);
 }
 
-
-void intgridcon(field_t *field1, field_t *field2)
-{
-  int nlon1, nlat1;
-  int nlon1b, nlat1b;
-  int nlon2, nlat2;
-  int ilat;
-  int gridID1, gridID2;
-  int i, nmiss;
-  int lon_is_circular;
-  double *lon1, *lat1;
-  double *lon1bounds, *lat1bounds;
-  double **field;
-  double *array = NULL;
-  double *array1, *array2;
-  double missval;
-  char units[CDI_MAX_NAME];
-  /* static int index = 0; */
-
-  gridID1 = field1->grid;
-  gridID2 = field2->grid;
-  array1  = field1->ptr;
-  array2  = field2->ptr;
-  missval = field1->missval;
-
-  if ( ! (gridInqXvals(gridID1, NULL) && gridInqYvals(gridID1, NULL)) )
-    cdoAbort("Source grid has no values");
-
-  lon_is_circular = gridIsCircular(gridID1);
-
-  nlon1 = gridInqXsize(gridID1);
-  nlat1 = gridInqYsize(gridID1);
-
-  lon1 = (double*) malloc(nlon1*sizeof(double));
-  lat1 = (double*) malloc(nlat1*sizeof(double));
-  gridInqXvals(gridID1, lon1);
-  gridInqYvals(gridID1, lat1);
-
-  gridInqXunits(gridID1, units);
-
-  grid_to_radian(units, nlon1, lon1, "grid1 center lon"); 
-  grid_to_radian(units, nlat1, lat1, "grid1 center lat"); 
-
-  nlon1b = nlon1 + 1;
-  nlat1b = nlat1 + 1;
-  lon1bounds = (double*) malloc(nlon1b*sizeof(double));
-  lat1bounds = (double*) malloc(nlat1b*sizeof(double));
-
-  grid_gen_corners(nlon1, lon1, lon1bounds);
-  grid_gen_corners(nlat1, lat1, lat1bounds);
-
-  if ( cdoVerbose )
-    {
-      for ( int i = 0; i < nlat1; ++i )
-	printf("lat1 %d %g\n", i+1, lat1[i]*RAD2DEG);
-      printf("lat1bounds: %g %g %g ... %g %g %g\n", lat1bounds[0]*RAD2DEG, lat1bounds[1]*RAD2DEG, lat1bounds[2]*RAD2DEG, lat1bounds[nlat1b-3]*RAD2DEG, lat1bounds[nlat1b-2]*RAD2DEG, lat1bounds[nlat1b-1]*RAD2DEG);
-      printf("lon1bounds: %g %g %g ... %g %g %g\n", lon1bounds[0]*RAD2DEG, lon1bounds[1]*RAD2DEG, lon1bounds[2]*RAD2DEG, lon1bounds[nlon1b-3]*RAD2DEG, lon1bounds[nlon1b-2]*RAD2DEG, lon1bounds[nlon1b-1]*RAD2DEG);
-    }
-
-  nlon2 = gridInqXsize(gridID2);
-  nlat2 = gridInqYsize(gridID2);
-
-  int gridsize2;
-
-  if ( gridInqType(gridID2) == GRID_GME ) gridID2 = gridToUnstructured(gridID2, 0);
-
-  if ( gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR )
-    gridID2 = gridToCurvilinear(gridID2, 0);
-
-  if ( ! (gridInqXvals(gridID2, NULL) && gridInqYvals(gridID2, NULL)) )
-    cdoAbort("Target grid has no values");
-
-  gridsize2 = gridInqSize(gridID2);
-
-  int nc2;
-
-  if ( gridInqType(gridID2) == GRID_UNSTRUCTURED )
-    nc2 = gridInqNvertex(gridID2);
-  else
-    nc2 = 4;
-
-  double *grid2_corner_lon = NULL, *grid2_corner_lat = NULL;
-
-  if ( gridInqYbounds(gridID2, NULL) && gridInqXbounds(gridID2, NULL) )
-    {
-      grid2_corner_lon = (double*) malloc(nc2*gridsize2*sizeof(double));
-      grid2_corner_lat = (double*) malloc(nc2*gridsize2*sizeof(double));
-      gridInqXbounds(gridID2, grid2_corner_lon);
-      gridInqYbounds(gridID2, grid2_corner_lat);
-    }
-  else
-    {
-      cdoAbort("grid2 corner missing!");
-    }
- 
-  gridInqXunits(gridID2, units);
-
-  grid_to_radian(units, nc2*gridsize2, grid2_corner_lon, "grid2 corner lon"); 
-  grid_to_radian(units, nc2*gridsize2, grid2_corner_lat, "grid2 corner lat"); 
-  /*
-  for ( int i = 0; i < gridsize2; ++i )
-    {
-      if ( lon2[i] < lon1[0]       ) lon2[i] += 2*M_PI;
-      if ( lon2[i] > lon1[nlon1-1] ) lon2[i] -= 2*M_PI;
-    }
-  */
-  for ( i = 0; i < gridsize2; ++i ) array2[i] = missval;
-
-  intconarr2(missval, lon_is_circular, 
-	     nlon1b, nlat1b, array1, lon1bounds, lat1bounds,
-	     nc2, gridsize2, array2, grid2_corner_lon, grid2_corner_lat);
-
-  nmiss = 0;
-  for ( i = 0; i < gridsize2; ++i )
-    if ( DBL_IS_EQUAL(array2[i], missval) ) nmiss++;
-
-  field2->nmiss = nmiss;
-
-  if (grid2_corner_lon) free(grid2_corner_lon);
-  if (grid2_corner_lat) free(grid2_corner_lat);
-
-  if (array) free(array);
-  free(lon1);
-  free(lat1);
-}
-
-
 /* source code from pingo */
-void interpolate(field_t *field1, field_t *field2)
+void interpolate(field_type *field1, field_type *field2)
 {
   int i;
   double *lono_array, *lato_array, *lono, *lato;
@@ -1009,8 +533,8 @@ void interpolate(field_t *field1, field_t *field2)
   out_nlon = gridInqXsize(gridIDo);
   out_nlat = gridInqYsize(gridIDo);
 
-  lon_array = (double*) malloc((nlon + 2) * sizeof(double));
-  lat_array = (double*) malloc((nlat + 2) * sizeof(double));
+  lon_array = (double*) Malloc((nlon + 2) * sizeof(double));
+  lat_array = (double*) Malloc((nlat + 2) * sizeof(double));
   lon = lon_array + 1;
   lat = lat_array + 1;
 
@@ -1061,9 +585,9 @@ void interpolate(field_t *field1, field_t *field2)
   if ( lat[nlat] < -90 ) lat[nlat] = -99;
   if ( lat[nlat] >  90 ) lat[nlat] =  99;
 
-  lono_array = (double*) malloc((out_nlon < 2 ? 4 : out_nlon + 2) * sizeof(double));
+  lono_array = (double*) Malloc((out_nlon < 2 ? 4 : out_nlon + 2) * sizeof(double));
   lono = lono_array + 1;
-  lato_array = (double*) malloc((out_nlat < 2 ? 4 : out_nlat + 2) * sizeof(double));
+  lato_array = (double*) Malloc((out_nlat < 2 ? 4 : out_nlat + 2) * sizeof(double));
   lato = lato_array + 1;
 
   gridInqXvals(gridIDo, lono);
@@ -1135,13 +659,13 @@ void interpolate(field_t *field1, field_t *field2)
 
   nxlon = 2*nlon + 1;
   nxlat = 2*nlat + 1;
-  xin_array = (double*) malloc(nxlon * nxlat * sizeof(double));
-  xin = (double **) malloc(nxlat * sizeof(double *));
+  xin_array = (double*) Malloc(nxlon * nxlat * sizeof(double));
+  xin = (double **) Malloc(nxlat * sizeof(double *));
 
   for (ilat = 0; ilat < nxlat; ilat++)
     xin[ilat] = xin_array + ilat * nxlon;
 
-  xlon = (double *) malloc(nxlon * sizeof(double));
+  xlon = (double *) Malloc(nxlon * sizeof(double));
   for ( ilon = 0; ilon < nlon; ilon++ )
     {
       xlon[2*ilon + 1] = lon[ilon];
@@ -1149,7 +673,7 @@ void interpolate(field_t *field1, field_t *field2)
     }
   xlon[2 * nlon] = (lon[nlon - 1] + lon[nlon]) / 2;
 
-  xlat = (double*) malloc((2 * nlat + 1) * sizeof(double));
+  xlat = (double*) Malloc((2 * nlat + 1) * sizeof(double));
   for ( ilat = 0; ilat < nlat; ilat++ )
     {
       xlat[2*ilat + 1] = lat[ilat];
@@ -1157,18 +681,18 @@ void interpolate(field_t *field1, field_t *field2)
     }
   xlat[2 * nlat] = (lat[nlat - 1] + lat[nlat]) / 2;
 
-  in0 = (double**) malloc(nlat * sizeof(double*));
+  in0 = (double**) Malloc(nlat * sizeof(double*));
   for (ilat = 0; ilat < nlat; ilat++)
     in0[ilat] = arrayIn + ilat * nlon;
 
-  ilon11 = (long*) malloc(out_nlon * sizeof(long));
-  ilon12 = (long*) malloc(out_nlon * sizeof(long));
-  ilon21 = (long*) malloc(out_nlon * sizeof(long));
-  ilon22 = (long*) malloc(out_nlon * sizeof(long));
-  volon11 = (double*) malloc(out_nlon * sizeof(double));
-  volon12 = (double*) malloc(out_nlon * sizeof(double));
-  volon21 = (double*) malloc(out_nlon * sizeof(double));
-  volon22 = (double*) malloc(out_nlon * sizeof(double));
+  ilon11 = (long*) Malloc(out_nlon * sizeof(long));
+  ilon12 = (long*) Malloc(out_nlon * sizeof(long));
+  ilon21 = (long*) Malloc(out_nlon * sizeof(long));
+  ilon22 = (long*) Malloc(out_nlon * sizeof(long));
+  volon11 = (double*) Malloc(out_nlon * sizeof(double));
+  volon12 = (double*) Malloc(out_nlon * sizeof(double));
+  volon21 = (double*) Malloc(out_nlon * sizeof(double));
+  volon22 = (double*) Malloc(out_nlon * sizeof(double));
 
   for (olon = 0; olon < out_nlon; olon++)
     {
@@ -1193,8 +717,8 @@ void interpolate(field_t *field1, field_t *field2)
       ilon22[olon] = l22;
     }
 
-  ilat1 = (long*) malloc(out_nlat * sizeof(long));
-  ilat2 = (long*) malloc(out_nlat * sizeof(long));
+  ilat1 = (long*) Malloc(out_nlat * sizeof(long));
+  ilat2 = (long*) Malloc(out_nlat * sizeof(long));
 
   xlat_is_ascending = xlat[0] <= xlat[nxlat - 1];
   for ( olat = 0; olat < out_nlat; olat++ )
@@ -1232,7 +756,7 @@ void interpolate(field_t *field1, field_t *field2)
       ilat2[olat] = l2;
     }
 
-  xout = (double**) malloc(out_nlat * sizeof(double*));
+  xout = (double**) Malloc(out_nlat * sizeof(double*));
   for (olat = 0; olat < out_nlat; olat++)
     xout[olat] = arrayOut + olat * out_nlon;
 
@@ -1462,26 +986,26 @@ void interpolate(field_t *field1, field_t *field2)
 
   field2->nmiss = nmiss;
 
-  free(lon_array);
-  free(lat_array);
-  free(lono_array);
-  free(lato_array);
-  free(xin);
-  free(xin_array);
-  free(xlon);
-  free(xlat);
-  free(in0);
-  free(ilon11);
-  free(ilon12);
-  free(ilon21);
-  free(ilon22);
-  free(volon11);
-  free(volon12);
-  free(volon21);
-  free(volon22);
-  free(ilat1);
-  free(ilat2);
-  free(xout);
+  Free(lon_array);
+  Free(lat_array);
+  Free(lono_array);
+  Free(lato_array);
+  Free(xin);
+  Free(xin_array);
+  Free(xlon);
+  Free(xlat);
+  Free(in0);
+  Free(ilon11);
+  Free(ilon12);
+  Free(ilon21);
+  Free(ilon22);
+  Free(volon11);
+  Free(volon12);
+  Free(volon21);
+  Free(volon22);
+  Free(ilat1);
+  Free(ilat2);
+  Free(xout);
 }
 
 
@@ -1537,17 +1061,17 @@ void contrast(void)
 
   if (rec == 1)
     {
-      xin_array = (double*) malloc((nlat + 2) * (nlon + 2) * sizeof(double));
-      xin = (double **) malloc((nlat + 2) * sizeof(double *));
+      xin_array = (double*) Malloc((nlat + 2) * (nlon + 2) * sizeof(double));
+      xin = (double **) Malloc((nlat + 2) * sizeof(double *));
       *xin = *(xin + 1);
       for (ilat = -1; ilat <= nlat; ilat++)
 	xin[ilat] = xin_array + (ilat + 1) * (nlon + 2) + 1;
-      xout = (double **) malloc(nlat * sizeof(double *));
+      xout = (double **) Malloc(nlat * sizeof(double *));
       for (ilat = 0; ilat < nlat; ilat++)
 	xout[ilat] = out[0] + ilat * nlon;
       for (j = 0; j < 17; j++)
 	{
-	  xwork[j] = (double **) malloc(nlat * sizeof(double *));
+	  xwork[j] = (double **) Malloc(nlat * sizeof(double *));
 	  for (ilat = 0; ilat < nlat; ilat++)
 	    xwork[j][ilat] = work[j] + ilat * nlon;
 	}
@@ -2017,9 +1541,9 @@ void contrast(void)
   iter_sum = iter_sum * 0.9 + iter;
   iter_n = iter_n * 0.9 + 1;
 
-  free(xin_array);
-  free(xin);
-  free(xout);
+  Free(xin_array);
+  Free(xin);
+  Free(xout);
   for (j = 0; j < 17; j++)
-    free(xwork[j]);
+    Free(xwork[j]);
 }
