@@ -6,16 +6,12 @@
 #define _XOPEN_SOURCE 600 /* gethostname */
 #endif
 
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
-#include <stdlib.h>
+#include "cdo_int.h"
 
 #include <sys/types.h>  /* fstat */
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "cdo.h"
 
 #if defined(HAVE_LIBDRMAA)
 #  include "drmaa.h"
@@ -59,7 +55,7 @@ static drmaa_job_template_t *create_job_template(const char *expname, const char
   /* determine current path */
 
   size = pathconf(".", _PC_PATH_MAX);
-  if ( (dir = (char*) malloc((size_t)size)) != NULL )
+  if ( (dir = (char*) Malloc((size_t)size)) != NULL )
     {
       ptr = getcwd(dir, (size_t)size);
     }
@@ -71,7 +67,7 @@ static drmaa_job_template_t *create_job_template(const char *expname, const char
   len2 = strlen(GRID_TMPDIR);
   len = len1+len2+2;
 
-  output_path = (char*) malloc(len*sizeof(char));
+  output_path = (char*) Malloc(len*sizeof(char));
   /*
   strcpy(output_path, host);
   strcat(output_path, ":");
@@ -124,7 +120,7 @@ static drmaa_job_template_t *create_job_template(const char *expname, const char
       fprintf (stderr, "name: %-25s \t %s\n", name, value);
     }
 
-  free(dir);
+  Free(dir);
 
   return job;
 }
@@ -339,14 +335,19 @@ static int drmaa_submit(const char *expname, const char *jobfilename, const char
 
 int job_submit(const char *expname, const char *jobfilename, const char *jobname, const char *tmppath, const char *ftppath)
 {
-  int status = 0;
+  int status = -1;
 #if defined(HAVE_LIBDRMAA)
 
   status = drmaa_submit(expname, jobfilename, jobname, tmppath, ftppath);
 #else
+  UNUSED(expname);
+  UNUSED(jobfilename);
+  UNUSED(jobname);
+  UNUSED(tmppath);
+  UNUSED(ftppath);
   fprintf(stderr, "DRMAA support not compiled in!\n");
 #endif
-  return (status);
+  return status;
 }
 
 
@@ -374,11 +375,14 @@ size_t my_fwrite(void *buffer, size_t size, size_t nmemb, void *stream)
 
 
 int my_progress_func(void *stdout_is_tty,
-			double t, /* dltotal */
-			double d, /* dlnow */
-			double ultotal,
-			double ulnow)
+                     double t, /* dltotal */
+                     double d, /* dlnow */
+                     double ultotal,
+                     double ulnow)
 {
+  UNUSED(ultotal);
+  UNUSED(ulnow);
+
   if ( *(char*)stdout_is_tty )
     {
       fprintf(stdout, "\b\b\b\b\b%4d%%", (int) (d*100/t));
@@ -502,10 +506,15 @@ int ftpget(int flag, const char *url, const char *path, const char *target, cons
 
   if ( flag ) fprintf(stdout, "\n");
 #else
+  UNUSED(flag);
+  UNUSED(url);
+  UNUSED(path);
+  UNUSED(target);
+  UNUSED(source);
   fprintf(stderr, "CURL support not compiled in!\n");
 #endif
 
-  return (status);
+  return status;
 }
 
 
@@ -573,17 +582,19 @@ int ftprmd(const char *url, const char *path)
   curl_global_cleanup();
 
 #else
+  UNUSED(url);
+  UNUSED(path);
   fprintf(stderr, "CURL support not compiled in!\n");
 #endif
 
-  return (status);
+  return status;
 }
 
 
 #define  DEFAULT_CDO_REMOTE_PATH  "/client/bin/cdo"
 const char *cdojobfiles  = "ftp_files";
 
-void exp_run(int argc, char *argv[], char *cdoExpName)
+void exp_run(int argc, char *argv[], const char *cdoExpName)
 {
   char commandline[65536];
   int i;
