@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -54,6 +54,8 @@ void *Change(void *argument);
 void *Change_e5slm(void *argument);
 void *Cloudlayer(void *argument);
 void *CMOR(void *argument);
+void *CMOR_lite(void *argument);
+void *CMOR_table(void *argument);
 void *Collgrid(void *argument);
 void *Command(void *argument);
 void *Comp(void *argument);
@@ -64,6 +66,7 @@ void *Cond2(void *argument);
 void *Condc(void *argument);
 void *Consecstat(void *argument);
 void *Copy(void *argument);
+void *Deltat(void *argument);
 void *Deltime(void *argument);
 void *Derivepar(void *argument);
 void *Detrend(void *argument);
@@ -112,8 +115,8 @@ void *Intyear(void *argument);
 void *Invert(void *argument);
 void *Invertlev(void *argument);
 void *Isosurface(void *argument);
-void *Kvl(void *argument);
 void *Log(void *argument);
+void *MapReduce(void *argument);
 void *Maskbox(void *argument);
 void *Mastrfu(void *argument);
 void *Math(void *argument);
@@ -125,7 +128,7 @@ void *Monarith(void *argument);
 void *Mrotuv(void *argument);
 void *Mrotuvb(void *argument);
 void *Ninfo(void *argument);
-void *Nmltest(void *argument);
+void *Nmldump(void *argument);
 void *Output(void *argument);
 void *Outputgmt(void *argument);
 void *Pack(void *argument);
@@ -145,12 +148,14 @@ void *Seascount(void *argument);
 void *Seaspctl(void *argument);
 void *Seasstat(void *argument);
 void *Selbox(void *argument);
+void *Selindex(void *argument);
 void *Select(void *argument);
 void *Selvar(void *argument);
 void *Seloperator(void *argument);
 void *Selrec(void *argument);
 void *Seltime(void *argument);
 void *Set(void *argument);
+void *Setattribute(void *argument);
 void *Setbox(void *argument);
 void *Setgatt(void *argument);
 void *Setgrid(void *argument);
@@ -160,9 +165,10 @@ void *Setpartab(void *argument);
 void *Setrcaname(void *argument);
 void *Settime(void *argument);
 void *Setzaxis(void *argument);
+void *Shiftxy(void *argument);
 void *Showinfo(void *argument);
 void *Sinfo(void *argument);
-void *Smooth9(void *argument);
+void *Smooth(void *argument);
 void *Sort(void *argument);
 void *Sorttimestamp(void *argument);
 void *Specinfo(void *argument);
@@ -173,7 +179,6 @@ void *Splitrec(void *argument);
 void *Splitsel(void *argument);
 void *Splittime(void *argument);
 void *Splityear(void *argument);
-void *SSOpar(void *argument);
 void *Subtrend(void *argument);
 void *Tee(void *argument);
 void *Template1(void *argument);
@@ -182,9 +187,9 @@ void *Test(void *argument);
 void *Test2(void *argument);
 void *Testdata(void *argument);
 void *Tests(void *argument);
-void *Timedt(void *argument);
 void *Timsort(void *argument);
 void *Timcount(void *argument);
+void *Timcumsum(void *argument);
 void *Timpctl(void *argument);
 void *Timselpctl(void *argument);
 void *Timselstat(void *argument);
@@ -289,6 +294,8 @@ void *Maggraph(void *argument);
 #define  Change_e5slmOperators  {"change_e5slm", "change_e5lsm", "change_e5mask"}
 #define  CloudlayerOperators    {"cloudlayer"}
 #define  CMOROperators          {"cmor"}
+#define  CMORliteOperators      {"cmorlite"}
+#define  CMORtableOperators     {"dump_cmor_table", "conv_cmor_table"}
 #define  CollgridOperators      {"collgrid"}
 #define  CommandOperators       {"command", "com", "cmd"}
 #define  CompOperators          {"eq",  "ne",  "le",  "lt",  "ge",  "gt"}
@@ -299,14 +306,14 @@ void *Maggraph(void *argument);
 #define  CondcOperators         {"ifthenc", "ifnotthenc"}
 #define  ConsecstatOperators    {"consects", "consecsum"}
 #define  CopyOperators          {"copy", "selall", "szip"}
+#define  DeltatOperators        {"deltat"}
 #define  DeltimeOperators       {"delday", "del29feb"}
 #define  DeriveparOperators     {"gheight", "sealevelpressure"}
 #define  DetrendOperators       {"detrend"}
 #define  DiffOperators          {"diff", "diffp", "diffn", "diffc"}
 #define  DistgridOperators      {"distgrid"}
 #define  DuplicateOperators     {"duplicate"}
-#define  Echam5iniOperators     {"import_e5ml", "import_e5res", \
-                                 "export_e5ml", "export_e5res"}
+#define  Echam5iniOperators     {"import_e5ml", "import_e5res", "export_e5ml", "export_e5res"}
 #define  EnlargeOperators       {"enlarge"}
 #define  EnlargegridOperators   {"enlargegrid"}
 #define  EnsstatOperators       {"ensmin", "ensmax", "enssum", "ensmean", "ensavg", "ensvar", "ensvar1", "ensstd", "ensstd1", "enspctl"}
@@ -318,12 +325,12 @@ void *Maggraph(void *argument);
 #define  EOF3dOperators         {"eof3d","eof3dspatial","eof3dtime"}
 #define  ExprOperators          {"expr", "exprf", "aexpr", "aexprf"}
 #define  FCOperators            {"fc2sp", "sp2fc", "fc2gp", "gp2fc"}
-#define  FiledesOperators       {"filedes", "griddes", "griddes2", "zaxisdes", "vct", "vct2", "pardes", \
+#define  FiledesOperators       {"filedes", "griddes", "griddes2", "zaxisdes", "vct", "vct2", "codetab", \
                                  "vlist", "partab", "partab2", "spartab"}
 #define  FillmissOperators      {"fillmiss", "fillmiss2"}
 #define  FilterOperators        {"bandpass", "highpass", "lowpass"}
 #define  FldrmsOperators        {"fldrms"}
-#define  FldstatOperators       {"fldmin", "fldmax", "fldsum", "fldmean", "fldavg", "fldstd", "fldstd1", "fldvar", "fldvar1", "fldpctl"}
+#define  FldstatOperators       {"fldrange", "fldmin", "fldmax", "fldsum", "fldmean", "fldavg", "fldstd", "fldstd1", "fldvar", "fldvar1", "fldpctl"}
 #define  FldcorOperators        {"fldcor"}
 #define  FldcovarOperators      {"fldcovar"}
 #define  FourierOperators       {"fourier"}
@@ -340,7 +347,7 @@ void *Maggraph(void *argument);
 #define  ImportobsOperators     {"import_obs"}
 #define  InfoOperators          {"info", "infop", "infon", "infoc", "map"}
 #define  InputOperators         {"input", "inputsrv", "inputext"}
-#define  IntgridOperators       {"intgridbil", "intgridcon", "intpoint", "interpolate", "boxavg", "thinout"}
+#define  IntgridOperators       {"intgridbil", "intpoint", "interpolate", "boxavg", "thinout"}
 #define  IntgridtrajOperators   {"intgridtraj"}
 #define  IntlevelOperators      {"intlevel", "intlevelx"}
 #define  Intlevel3dOperators    {"intlevel3d", "intlevelx3d"}
@@ -350,8 +357,8 @@ void *Maggraph(void *argument);
 #define  InvertOperators        {"invertlat", "invertlon", "invertlatdes", "invertlondes", "invertlatdata", "invertlondata"}
 #define  InvertlevOperators     {"invertlev"}
 #define  IsosurfaceOperators    {"isosurface"}
-#define  KvlOperators           {"read_cmor_table", "conv_cmor_table"}
 #define  LogOperators           {"dumplogs", "daylogs", "monlogs", "dumplogo", "snamelogo", "scalllogo", "smemlogo", "stimelogo", "sperclogo"}
+#define  MapReduceOperators     {"reducegrid"}
 #define  MaskboxOperators       {"masklonlatbox", "maskindexbox"}
 #define  MaskregionOperators    {"maskregion"}
 #define  MastrfuOperators       {"mastrfu"}
@@ -364,8 +371,8 @@ void *Maggraph(void *argument);
 #define  MonarithOperators      {"monadd", "monsub", "monmul", "mondiv"}
 #define  MrotuvOperators        {"mrotuv"}
 #define  MrotuvbOperators       {"mrotuvb"}
-#define  NinfoOperators         {"nyear", "nmon", "ndate", "ntime", "ncode", "npar", "nlevel"}
-#define  NmltestOperators       {"nmltest"}
+#define  NinfoOperators         {"nyear", "nmon", "ndate", "ntime", "ncode", "npar", "nlevel", "ngridpoints", "ngrids"}
+#define  NmldumpOperators       {"nmldump", "kvldump"}
 #define  OutputOperators        {"output", "outputint", "outputsrv", "outputext", "outputf", "outputts", \
                                  "outputfld", "outputarr", "outputxyz"}
 #define  OutputtabOperators     {"outputtab"}
@@ -397,6 +404,7 @@ void *Maggraph(void *argument);
 #define  SeaspctlOperators      {"seaspctl"}
 #define  SeasstatOperators      {"seasmin", "seasmax", "seassum", "seasmean", "seasavg", "seasstd", "seasstd1", "seasvar", "seasvar1"}
 #define  SelboxOperators        {"sellonlatbox", "selindexbox"}
+#define  SelindexOperators      {"selindex"}
 #define  SelectOperators        {"select", "delete"}
 #define  SelvarOperators        {"selparam", "selcode", "selname", "selstdname", "sellevel", "sellevidx", "selgrid", \
                                  "selzaxis", "selzaxisname", "seltabnum", "delparam", "delcode", "delname", "selltype"}
@@ -405,22 +413,24 @@ void *Maggraph(void *argument);
 #define  SeltimeOperators       {"seltimestep", "selyear", "selseason", "selmonth", "selday", "selhour", "seldate", \
                                  "seltime", "selsmon"}
 #define  SetOperators           {"setcode", "setparam", "setname", "setunit", "setlevel", "setltype", "settabnum"}
+#define  SetattributeOperators  {"setattribute"}
 #define  SetboxOperators        {"setclonlatbox", "setcindexbox"}
 #define  SetgattOperators       {"setgatt", "setgatts"}
 #define  SetgridOperators       {"setgrid", "setgridtype", "setgridarea", "setgridmask", "unsetgridmask", "setgridnumber", "setgriduri"}
 #define  SethaloOperators       {"sethalo", "tpnhalo"}
 #define  SetmissOperators       {"setmissval", "setctomiss", "setmisstoc", "setrtomiss", "setvrange"}
 #define  SetmisstonnOperators   {"setmisstonn", "setmisstodis"}
-#define  Setpartab0Operators    {"setpartab"}
+#define  SetcodetabOperators    {"setcodetab"}
 #define  SetpartabOperators     {"setpartabc", "setpartabp", "setpartabn"}
 #define  SetrcanameOperators    {"setrcaname"}
 #define  SettimeOperators       {"setyear", "setmon", "setday", "setdate", "settime", "settunits", \
-                                 "settaxis", "setreftime", "setcalendar", "shifttime"}
+                                 "settaxis", "settbounds", "setreftime", "setcalendar", "shifttime"}
 #define  SetzaxisOperators      {"setzaxis", "genlevelbounds"}
+#define  ShiftxyOperators       {"shiftx", "shifty"}
 #define  ShowinfoOperators      {"showyear", "showmon", "showdate", "showtime", "showtimestamp", "showcode", "showunit", \
                                  "showparam", "showname", "showstdname", "showlevel", "showltype", "showformat"}
 #define  SinfoOperators         {"sinfo", "sinfop", "sinfon", "sinfoc", "seinfo", "seinfop", "seinfon", "seinfoc"}
-#define  Smooth9Operators       {"smooth9"}
+#define  SmoothOperators        {"smooth", "smooth9"}
 #define  SortOperators          {"sortcode", "sortparam", "sortname", "sortlevel"}
 #define  SorttimestampOperators {"sorttimestamp", "sorttaxis"}
 #define  SpecinfoOperators      {"specinfo"}
@@ -431,7 +441,6 @@ void *Maggraph(void *argument);
 #define  SplitselOperators      {"splitsel"}
 #define  SplittimeOperators     {"splithour", "splitday", "splitmon", "splitseas"}
 #define  SplityearOperators     {"splityear", "splityearmon"}
-#define  SSOparOperators        {"ssopar"}
 #define  SubtrendOperators      {"subtrend"}
 #define  TeeOperators           {"tee"}
 #define  Template1Operators     {"template1"}
@@ -440,13 +449,13 @@ void *Maggraph(void *argument);
 #define  Test2Operators         {"test2"}
 #define  TestdataOperators      {"testdata"}
 #define  TestsOperators         {"normal", "studentt", "chisquare", "beta", "fisher"}
-#define  TimedtOperators        {"timedt"}
 #define  TimsortOperators       {"timsort"}
 #define  TimcountOperators      {"timcount"}
 #define    YearcountOperators   {"yearcount"}
 #define    MoncountOperators    {"moncount"}
 #define    DaycountOperators    {"daycount"}
 #define    HourcountOperators   {"hourcount"}
+#define  TimcumsumOperators     {"timcumsum"}
 #define  TimpctlOperators       {"timpctl"}
 #define    YearpctlOperators    {"yearpctl"}
 #define    MonpctlOperators     {"monpctl"}
@@ -457,11 +466,11 @@ void *Maggraph(void *argument);
 #define  XTimstatOperators      {"xtimmin",  "xtimmax",  "xtimsum",  "xtimmean",  "xtimavg",  "xtimvar",  "xtimvar1",  "xtimstd",  "xtimstd1", \
                                  "xyearmin", "xyearmax", "xyearsum", "xyearmean", "xyearavg", "xyearvar", "xyearvar1", "xyearstd", "xyearstd1", \
                                  "xmonmin",  "xmonmax",  "xmonsum",  "xmonmean",  "xmonavg",  "xmonvar",  "xmonvar1",  "xmonstd",  "xmonstd1"}
-#define  TimstatOperators       {"timmin",  "timmax",  "timsum",  "timmean",  "timavg",  "timvar",  "timvar1",  "timstd",  "timstd1"}
-#define    YearstatOperators    {"yearmin", "yearmax", "yearsum", "yearmean", "yearavg", "yearvar", "yearvar1", "yearstd", "yearstd1"}
-#define    MonstatOperators     {"monmin",  "monmax",  "monsum",  "monmean",  "monavg",  "monvar",  "monvar1",  "monstd",  "monstd1"}
-#define    DaystatOperators     {"daymin",  "daymax",  "daysum",  "daymean",  "dayavg",  "dayvar",  "dayvar1",  "daystd",  "daystd1"}
-#define    HourstatOperators    {"hourmin", "hourmax", "hoursum", "hourmean", "houravg", "hourvar", "hourvar1", "hourstd", "hourstd1"}
+#define  TimstatOperators       {"timrange",  "timmin",  "timmax",  "timsum",  "timmean",  "timavg",  "timvar",  "timvar1",  "timstd",  "timstd1"}
+#define    YearstatOperators    {"yearrange", "yearmin", "yearmax", "yearsum", "yearmean", "yearavg", "yearvar", "yearvar1", "yearstd", "yearstd1"}
+#define    MonstatOperators     {"monrange",  "monmin",  "monmax",  "monsum",  "monmean",  "monavg",  "monvar",  "monvar1",  "monstd",  "monstd1"}
+#define    DaystatOperators     {"dayrange",  "daymin",  "daymax",  "daysum",  "daymean",  "dayavg",  "dayvar",  "dayvar1",  "daystd",  "daystd1"}
+#define    HourstatOperators    {"hourrange", "hourmin", "hourmax", "hoursum", "hourmean", "houravg", "hourvar", "hourvar1", "hourstd", "hourstd1"}
 #define  TimcorOperators        {"timcor"}
 #define  TimcovarOperators      {"timcovar"}
 #define  Timstat3Operators      {"meandiff2test", "varquot2test"}
@@ -474,11 +483,11 @@ void *Maggraph(void *argument);
 #define  VargenOperators        {"random", "const", "sincos", "coshill", "for", "topo", "temp", "mask", "stdatm"}
 #define  VarrmsOperators        {"varrms"}
 #define  VertintmlOperators     {"ml2pl", "ml2hl", "ml2plx", "ml2hlx", "ml2pl_lp", "ml2hl_lp", "ml2plx_lp", "ml2hlx_lp"}
-#define  VertintapOperators     {"ap2pl", "ap2plx", "ap2pl_lp", "ap2plx_lp"}
+#define  VertintapOperators     {"ap2pl", "ap2plx", "ap2pl_lp", "ap2plx_lp", "ap2hl", "ap2hlx"}
 #define  VertstatOperators      {"vertmin", "vertmax", "vertsum", "vertint", "vertmean", "vertavg", "vertstd", "vertstd1", "vertvar", "vertvar1"}
 #define  VertcumOperators       {"vertcum", "vertcumhl"}
 #define  VertwindOperators      {"vertwind"}
-#define  VerifygridOperators    {"verifygrid", "verifygridtest"}
+#define  VerifygridOperators    {"verifygrid"}
 #define  WindOperators          {"uv2dv", "uv2dvl", "dv2uv", "dv2uvl", "dv2ps"}
 #define  WritegridOperators     {"writegrid"}
 #define  WriterandomOperators   {"writerandom"}
@@ -575,6 +584,8 @@ static modules_t Modules[] =
   { Change_e5slm,   NULL,              Change_e5slmOperators,  0,   CDI_REAL,  1,  1 },
   { Cloudlayer,     NULL,              CloudlayerOperators,    1,   CDI_REAL,  1,  1 },
   { CMOR,           NULL,              CMOROperators,          1,   CDI_REAL,  1,  0 },
+  { CMOR_lite,      CMORliteHelp,      CMORliteOperators,      1,   CDI_REAL,  1,  1 },
+  { CMOR_table,     NULL,              CMORtableOperators,     1,   CDI_REAL,  0,  0 },
   { Collgrid,       CollgridHelp,      CollgridOperators,      1,   CDI_REAL, -1,  1 },
   { Command,        NULL,              CommandOperators,       0,   CDI_REAL,  1,  0 },
   { Comp,           CompHelp,          CompOperators,          1,   CDI_REAL,  2,  1 },
@@ -585,6 +596,7 @@ static modules_t Modules[] =
   { Condc,          CondcHelp,         CondcOperators,         1,   CDI_REAL,  1,  1 },
   { Consecstat,     ConsecstatHelp,    ConsecstatOperators,    1,   CDI_REAL,  1,  1 },
   { Copy,           CopyHelp,          CopyOperators,          1,   CDI_REAL, -1,  1 },
+  { Deltat,         NULL,              DeltatOperators,        1,   CDI_REAL,  1,  1 },
   { Deltime,        NULL,              DeltimeOperators,       1,   CDI_REAL,  1,  1 },
   { Derivepar,      DeriveparHelp,     DeriveparOperators,     1,   CDI_REAL,  1,  1 },
   { Detrend,        DetrendHelp,       DetrendOperators,       1,   CDI_REAL,  1,  1 },
@@ -634,8 +646,8 @@ static modules_t Modules[] =
   { Invert,         InvertHelp,        InvertOperators,        1,   CDI_REAL,  1,  1 },
   { Invertlev,      InvertlevHelp,     InvertlevOperators,     1,   CDI_REAL,  1,  1 },
   { Isosurface,     NULL,              IsosurfaceOperators,    1,   CDI_REAL,  1,  1 },
-  { Kvl,            NULL,              KvlOperators,           1,   CDI_REAL,  0,  0 },
   { Log,            NULL,              LogOperators,           0,   CDI_REAL,  1,  0 },
+  { MapReduce,      MapReduceHelp,     MapReduceOperators,     1,   CDI_REAL,  1,  1 },
   { Maskbox,        MaskboxHelp,       MaskboxOperators,       1,   CDI_REAL,  1,  1 },
   { Maskbox,        MaskregionHelp,    MaskregionOperators,    1,   CDI_REAL,  1,  1 },
   { Mastrfu,        MastrfuHelp,       MastrfuOperators,       1,   CDI_REAL,  1,  1 },
@@ -648,7 +660,7 @@ static modules_t Modules[] =
   { Mrotuv,         NULL,              MrotuvOperators,        1,   CDI_REAL,  1,  2 },
   { Mrotuvb,        NULL,              MrotuvbOperators,       1,   CDI_REAL,  2,  1 },
   { Ninfo,          NinfoHelp,         NinfoOperators,         1,   CDI_BOTH,  1,  0 },
-  { Nmltest,        NULL,              NmltestOperators,       0,   CDI_REAL,  0,  0 },
+  { Nmldump,        NULL,              NmldumpOperators,       0,   CDI_REAL,  0,  0 },
   { Output,         OutputHelp,        OutputOperators,        1,   CDI_REAL, -1,  0 },
   { Output,         OutputtabHelp,     OutputtabOperators,     1,   CDI_REAL, -1,  0 },
   { Outputgmt,      OutputgmtHelp,     OutputgmtOperators,     1,   CDI_REAL,  1,  0 },
@@ -678,26 +690,29 @@ static modules_t Modules[] =
   { Seaspctl,       SeaspctlHelp,      SeaspctlOperators,      1,   CDI_REAL,  3,  1 },
   { Seasstat,       SeasstatHelp,      SeasstatOperators,      1,   CDI_REAL,  1,  1 },
   { Selbox,         SelboxHelp,        SelboxOperators,        1,   CDI_BOTH,  1,  1 },
+  { Selindex,       NULL,              SelindexOperators,      1,   CDI_BOTH,  1,  1 },
   { Select,         SelectHelp,        SelectOperators,        1,   CDI_BOTH, -1,  1 },
   { Selvar,         SelvarHelp,        SelvarOperators,        1,   CDI_BOTH,  1,  1 },
   { Selrec,         SelvarHelp,        SelrecOperators,        1,   CDI_BOTH,  1,  1 },
   { Seloperator,    NULL,              SeloperatorOperators,   1,   CDI_REAL,  1,  1 },
   { Seltime,        SeltimeHelp,       SeltimeOperators,       1,   CDI_BOTH,  1,  1 },
   { Set,            SetHelp,           SetOperators,           1,   CDI_BOTH,  1,  1 },
+  { Setattribute,   SetattributeHelp,  SetattributeOperators,  1,   CDI_BOTH,  1,  1 },
   { Setbox,         SetboxHelp,        SetboxOperators,        1,   CDI_REAL,  1,  1 },
-  { Setgatt,        SetgattHelp,       SetgattOperators,       1,   CDI_BOTH,  1,  1 },
+  { Setgatt,        NULL,              SetgattOperators,       1,   CDI_BOTH,  1,  1 },
   { Setgrid,        SetgridHelp,       SetgridOperators,       1,   CDI_BOTH,  1,  1 },
   { Sethalo,        SethaloHelp,       SethaloOperators,       1,   CDI_REAL,  1,  1 },
   { Setmiss,        SetmissHelp,       SetmissOperators,       1,   CDI_REAL,  1,  1 },
   { Fillmiss,       SetmissHelp,       SetmisstonnOperators,   1,   CDI_REAL,  1,  1 },
-  { Setpartab,      SetHelp,           Setpartab0Operators,    1,   CDI_REAL,  1,  1 },
+  { Setpartab,      SetHelp,           SetcodetabOperators,    1,   CDI_REAL,  1,  1 },
   { Setpartab,      SetpartabHelp,     SetpartabOperators,     1,   CDI_REAL,  1,  1 },
   { Setrcaname,     NULL,              SetrcanameOperators,    1,   CDI_REAL,  1,  1 },
   { Settime,        SettimeHelp,       SettimeOperators,       1,   CDI_BOTH,  1,  1 },
   { Setzaxis,       SetzaxisHelp,      SetzaxisOperators,      1,   CDI_BOTH,  1,  1 },
+  { Shiftxy,        ShiftxyHelp,       ShiftxyOperators,       1,   CDI_REAL,  1,  1 },
   { Showinfo,       ShowinfoHelp,      ShowinfoOperators,      1,   CDI_BOTH,  1,  0 },
   { Sinfo,          SinfoHelp,         SinfoOperators,         1,   CDI_BOTH, -1,  0 },
-  { Smooth9,        Smooth9Help,       Smooth9Operators,       1,   CDI_REAL,  1,  1 },
+  { Smooth,         SmoothHelp,        SmoothOperators,        1,   CDI_REAL,  1,  1 },
   { Sort,           NULL,              SortOperators,          1,   CDI_REAL,  1,  1 },
   { Sorttimestamp,  NULL,              SorttimestampOperators, 1,   CDI_REAL, -1,  1 },
   { Specinfo,       NULL,              SpecinfoOperators,      1,   CDI_REAL,  0,  0 },
@@ -708,7 +723,6 @@ static modules_t Modules[] =
   { Splitsel,       SplitselHelp,      SplitselOperators,      1,   CDI_BOTH,  1, -1 },
   { Splittime,      SplittimeHelp,     SplittimeOperators,     1,   CDI_BOTH,  1, -1 },
   { Splityear,      SplittimeHelp,     SplityearOperators,     1,   CDI_BOTH,  1, -1 },
-  { SSOpar,         NULL,              SSOparOperators,        0,   CDI_REAL,  1,  1 },
   { Subtrend,       SubtrendHelp,      SubtrendOperators,      1,   CDI_REAL,  3,  1 },
   { Tee,            NULL,              TeeOperators,           1,   CDI_REAL,  2,  1 },
   { Template1,      NULL,              Template1Operators,     0,   CDI_REAL,  1,  1 },
@@ -722,13 +736,13 @@ static modules_t Modules[] =
   { Timcount,       NULL,              MoncountOperators,      1,   CDI_BOTH,  1,  1 },
   { Timcount,       NULL,              DaycountOperators,      1,   CDI_BOTH,  1,  1 },
   { Timcount,       NULL,              HourcountOperators,     1,   CDI_BOTH,  1,  1 },
+  { Timcumsum,      TimcumsumHelp,     TimcumsumOperators,     1,   CDI_BOTH,  1,  1 },
   { Timpctl,        TimpctlHelp,       TimpctlOperators,       1,   CDI_REAL,  3,  1 },
   { Timpctl,        YearpctlHelp,      YearpctlOperators,      1,   CDI_REAL,  3,  1 },
   { Timpctl,        MonpctlHelp,       MonpctlOperators,       1,   CDI_REAL,  3,  1 },
   { Timpctl,        DaypctlHelp,       DaypctlOperators,       1,   CDI_REAL,  3,  1 },
   { Timpctl,        HourpctlHelp,      HourpctlOperators,      1,   CDI_REAL,  3,  1 },
   { Timselpctl,     TimselpctlHelp,    TimselpctlOperators,    1,   CDI_REAL,  3,  1 },
-  { Timedt,         NULL,              TimedtOperators,        1,   CDI_REAL,  1,  1 },
   { Timsort,        TimsortHelp,       TimsortOperators,       1,   CDI_REAL,  1,  1 },
   { Timselstat,     TimselstatHelp,    TimselstatOperators,    1,   CDI_REAL,  1,  1 },
   { XTimstat,       NULL,              XTimstatOperators,      0,   CDI_BOTH,  1,  1 },
@@ -848,7 +862,8 @@ static const char *opalias[][2] =
   {"ncode",               "npar"},
   {"nvar",                "npar"},
   {"outputkey",           "outputtab"},
-  {"vardes",              "pardes"},
+  {"vardes",              "codetab"},
+  {"pardes",              "codetab"},
   {"selvar",              "selname"},
   {"delvar",              "delname"},
   {"remapcon1",           "remaplaf"},
@@ -858,6 +873,7 @@ static const char *opalias[][2] =
   {"selgridname",         "selgrid"},
   {"setvar",              "setname"},
   {"setpartabv",          "setpartabn"},
+  {"setpartab",           "setcodetab"},
   {"sinfov",              "sinfon"},
   {"sortvar",             "sortname"},
   {"splitvar",            "splitname"},
@@ -878,10 +894,10 @@ static int nopalias = sizeof(opalias) / (2*sizeof(opalias[0][0]));
 
 
 static
-int similar(const char *a, const char *b, int alen, int blen)
+bool similar(const char *a, const char *b, int alen, int blen)
 {
   if ( alen > 2 && blen > 2 && strstr(b, a) )
-    return TRUE;
+    return true;
 
   while ( *a && *b && *a == *b )
     { 
@@ -889,21 +905,21 @@ int similar(const char *a, const char *b, int alen, int blen)
       b++;
     }
   if ( !*a && !*b )
-    return TRUE;
+    return true;
   /*
     printf("%d %d %s %s\n", alen, blen, a, b);
   */
   if ( alen >= 2 && blen >= 1 && *a && similar(a+1, b, alen-2, blen-1) )
-    return TRUE;
+    return true;
 
   if ( alen >= 1 && blen >= 2 && *b && similar(a, b+1, alen-1, blen-2) )
-    return TRUE;
+    return true;
 
-  return FALSE; 
+  return false; 
 }
 
 
-const char *operatorAlias(char *operatorName)
+const char *operatorAlias(const char *operatorName)
 {
   int i;
 
@@ -955,41 +971,47 @@ int operatorInqModID(const char *operatorName)
 
   if ( modID == -1 )
     {
-      int nbyte;
-      int error = TRUE;
+      bool lfound = false;
       FILE *fp = fopen(operatorName, "r");
       if ( fp )
 	{
 	  fclose(fp);
 	  fprintf(stderr, "Use commandline option -h for help.");
-	  Error("operator missing! %s is a file on disk!", operatorName);
+	  Error("Operator missing, %s is a file on disk!", operatorName);
 	}
 
       fprintf(stderr, "Operator >%s< not found!\n", operatorName);
       fprintf(stderr, "Similar operators are:\n");
-      nbyte = fprintf(stderr, "   ");
+      int nbyte = fprintf(stderr, "   ");
       if ( operatorName )
-	for ( i = 0; i < NumModules; i++ )
-	  {
-	    if ( Modules[i].help == NULL ) continue;
-	    j = 0;
-	    while ( Modules[i].operators[j] )
-	      {
-		if( similar(operatorName, Modules[i].operators[j],
-			    strlen(operatorName), strlen(Modules[i].operators[j])) )
-		  {
-		    if ( nbyte > 75 )
-		      {
-			fprintf(stdout, "\n");
-			nbyte = fprintf(stderr, "   ");
-		      }
-		    nbyte += fprintf(stderr, " %s", Modules[i].operators[j]);
-		    error = FALSE ;
-		  }
-		j++;
-	      }
-	  }
-      if ( error )
+        {
+          int len = strlen(operatorName);
+          char *opname = strdup(operatorName);
+          strtolower(opname);
+          for ( i = 0; i < NumModules; i++ )
+            {
+              if ( Modules[i].help == NULL ) continue;
+              j = 0;
+              while ( Modules[i].operators[j] )
+                {
+                  if ( similar(opname, Modules[i].operators[j],
+                               len, strlen(Modules[i].operators[j])) )
+                    {
+                      if ( nbyte > 75 )
+                        {
+                          fprintf(stdout, "\n");
+                          nbyte = fprintf(stderr, "   ");
+                        }
+                      nbyte += fprintf(stderr, " %s", Modules[i].operators[j]);
+                      lfound = true;
+                    }
+                  j++;
+                }
+            }
+          free(opname);
+        }
+
+      if ( !lfound )
 	fprintf(stderr, "(not found)\n") ;
       else
 	fprintf(stderr, "\n");
@@ -1004,31 +1026,31 @@ int operatorInqModID(const char *operatorName)
   return modID;
 }
 
-void *(*operatorModule(char *operatorName))(void *)
+void *(*operatorModule(const char *operatorName))(void *)
 {
   int modID = operatorInqModID(operatorName);
   return Modules[modID].func;
 }
 
-const char **operatorHelp(char *operatorName)
+const char **operatorHelp(const char *operatorName)
 {
   int modID = operatorInqModID(operatorName);
   return Modules[modID].help;
 }
 
-int operatorStreamInCnt(char *operatorName)
+int operatorStreamInCnt(const char *operatorName)
 {
   int modID = operatorInqModID(operatorAlias(operatorName));
   return Modules[modID].streamInCnt;
 }
 
-int operatorStreamOutCnt(char *operatorName)
+int operatorStreamOutCnt(const char *operatorName)
 {
   int modID = operatorInqModID(operatorAlias(operatorName));
   return Modules[modID].streamOutCnt;
 }
 
-int operatorStreamNumber(char *operatorName)
+int operatorStreamNumber(const char *operatorName)
 {
   int modID = operatorInqModID(operatorAlias(operatorName));
   return Modules[modID].number;
@@ -1080,7 +1102,7 @@ void operatorPrintAll(void)
 }
 
 
-void operatorPrintList(void)
+void operatorPrintList(bool print_no_output)
 {
   int i, j, nbyte, nop = 0;
   const char *opernames[4096];
@@ -1097,10 +1119,7 @@ void operatorPrintList(void)
     }
 
   // Add operator aliases
-  for ( i = 0; i < nopalias; i++ )
-    {
-      opernames[nop++] = opalias[i][0];
-    }
+  for ( i = 0; i < nopalias; i++ ) opernames[nop++] = opalias[i][0];
 
   qsort(opernames, nop, sizeof(char *), cmpname);
 
@@ -1180,10 +1199,13 @@ void operatorPrintList(void)
             }      
         }
 
-      nbyte = fprintf(pout, "%s ", opernames[i]);
-      for ( int i = nbyte; i <= 16; ++i ) fprintf(pout, " ");
-      if ( pdes ) fprintf(pout, "%s", pdes);
-      else if ( ialias >= 0 )  fprintf(pout, "--> %s", opalias[ialias][1]);
-      fprintf(pout, "\n");
+      if ( !print_no_output || operatorStreamOutCnt(opernames[i]) == 0 )
+        {
+          nbyte = fprintf(pout, "%s ", opernames[i]);
+          for ( int i = nbyte; i <= 16; ++i ) fprintf(pout, " ");
+          if ( pdes ) fprintf(pout, "%s", pdes);
+          else if ( ialias >= 0 )  fprintf(pout, "--> %s", opalias[ialias][1]);
+          fprintf(pout, "\n");
+        }
     }
 }
