@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,14 +18,17 @@
 #ifndef _MODULES_H
 #define _MODULES_H
 
-void *(*operatorModule(char *operatorName))(void *);
+#include <stdbool.h>
 
-char **operatorHelp(char *operatorName);
+void *(*operatorModule(const char *operatorName))(void *);
 
-int operatorStreamInCnt(char *operatorName);
-int operatorStreamOutCnt(char *operatorName);
-int operatorStreamNumber(char *operatorName);
+const char **operatorHelp(const char *operatorName);
+
+int operatorStreamInCnt(const char *operatorName);
+int operatorStreamOutCnt(const char *operatorName);
+int operatorStreamNumber(const char *operatorName);
 
 void operatorPrintAll(void);
+void operatorPrintList(bool print_no_output);
 
 #endif  /* _MODULES_H */
