@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -77,7 +77,7 @@ typedef struct {
   short       streamCnt;
   argument_t *streamNames;
   char       *xoperator;
-  char       *operatorName;
+  const char *operatorName;
   char       *operatorArg;
   int         oargc;
   char       *oargv[MAX_OARGC];
@@ -304,7 +304,7 @@ const char *processInqOpername(void)
 }
 
 
-void processDefPrompt(char *opername)
+void processDefPrompt(const char *opername)
 {
   int processID = processSelf();
 
@@ -700,7 +700,7 @@ int checkStreamCnt(void)
     {
       if ( Process[processID].streamNames[i].args[0] == '-' )
         {
-          cdoAbort("Output file name %s must not begin with \"-\"!\n",
+          cdoAbort("Output file name %s must not begin with \"-\"!",
                    Process[processID].streamNames[i].args);
         }
       else if ( !obase )
