@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,10 @@
 #ifndef _PSTREAM_H
 #define _PSTREAM_H
 
-#define  streamOpenWrite          pstreamOpenWrite
+#include "pstream_write.h"
+
+#include <sys/types.h> /* off_t */
+
 #define  streamOpenRead           pstreamOpenRead
 #define  streamOpenAppend         pstreamOpenAppend
 #define  streamClose              pstreamClose
@@ -27,24 +30,21 @@
 #define  streamInqByteorder       pstreamInqByteorder
 
 #define  streamInqVlist           pstreamInqVlist
-#define  streamDefVlist           pstreamDefVlist
 
-#define  streamDefTimestep        pstreamDefTimestep
 #define  streamInqTimestep        pstreamInqTimestep
 
-#define  streamDefRecord          pstreamDefRecord
 #define  streamInqRecord          pstreamInqRecord
 
-#define  streamWriteRecord        pstreamWriteRecord
-#define  streamWriteRecordF       pstreamWriteRecordF
 #define  streamReadRecord         pstreamReadRecord
+#define  streamReadRecordF        pstreamReadRecordF
 
 #define  streamCopyRecord         pstreamCopyRecord
 
-#define  streamInqGinfo           pstreamInqGinfo
+#define  streamInqGRIBinfo        pstreamInqGRIBinfo
+
+#define  vlistCopyFlag            cdoVlistCopyFlag
 
 
-int     pstreamOpenWrite(const argument_t *argument, int filetype);
 int     pstreamOpenRead(const argument_t *argument);
 int     pstreamOpenAppend(const argument_t *argument);
 void    pstreamClose(int pstreamID);
@@ -52,21 +52,20 @@ void    pstreamClose(int pstreamID);
 int     pstreamInqFiletype(int pstreamID);
 int     pstreamInqByteorder(int pstreamID);
 
-void    pstreamDefVlist(int pstreamID, int vlistID);
 int     pstreamInqVlist(int pstreamID);
 
-void    pstreamDefTimestep(int pstreamID, int tsID);
 int     pstreamInqTimestep(int pstreamID, int tsID);
 
-void    pstreamDefRecord(int pstreamID, int  varID, int  levelID);
 int     pstreamInqRecord(int pstreamID, int *varID, int *levelID);
 
-void    pstreamWriteRecord(int pstreamID, double *data, int nmiss);
-void    pstreamWriteRecordF(int pstreamID, float *data, int nmiss);
 void    pstreamReadRecord(int pstreamID, double *data, int *nmiss);
+void    pstreamReadRecordF(int pstreamID, float *data, int *nmiss);
 void    pstreamCopyRecord(int pstreamIDdest, int pstreamIDsrc);
 
-void    pstreamInqGinfo(int pstreamID, int *intnum, float *fltnum, off_t *bignum);
+void    pstreamInqGRIBinfo(int pstreamID, int *intnum, float *fltnum, off_t *bignum);
 
+int     pstreamFileID(int pstreamID);
+
+void    cdoVlistCopyFlag(int vlistID2, int vlistID1);
 
 #endif  /* _PSTREAM_H */
