@@ -35,10 +35,10 @@ void search_realloc(long num_srch_cells, search_t *search)
 
   if ( num_srch_cells > max_srch_cells )
     {
-      partial_areas   = (double*) realloc(partial_areas,   num_srch_cells*sizeof(double));
-      partial_weights = (double*) realloc(partial_weights, num_srch_cells*sizeof(double));
-      overlap_buffer = (struct grid_cell*) realloc(overlap_buffer, num_srch_cells*sizeof(struct grid_cell));
-      src_grid_cells = (struct grid_cell*) realloc(src_grid_cells, num_srch_cells*sizeof(struct grid_cell));
+      partial_areas   = (double*) Realloc(partial_areas,   num_srch_cells*sizeof(double));
+      partial_weights = (double*) Realloc(partial_weights, num_srch_cells*sizeof(double));
+      overlap_buffer = (struct grid_cell*) Realloc(overlap_buffer, num_srch_cells*sizeof(struct grid_cell));
+      src_grid_cells = (struct grid_cell*) Realloc(src_grid_cells, num_srch_cells*sizeof(struct grid_cell));
 
       for ( long n = max_srch_cells; n < num_srch_cells; ++n )
         {
@@ -55,9 +55,9 @@ void search_realloc(long num_srch_cells, search_t *search)
           src_grid_cells[n].array_size      = search->srch_corners;
           src_grid_cells[n].num_corners     = search->srch_corners;
           src_grid_cells[n].edge_type       = search->src_edge_type;
-          src_grid_cells[n].coordinates_x   = (double*) malloc(search->srch_corners*sizeof(double));
-          src_grid_cells[n].coordinates_y   = (double*) malloc(search->srch_corners*sizeof(double));
-          src_grid_cells[n].coordinates_xyz = (double*) malloc(3*search->srch_corners*sizeof(double));
+          src_grid_cells[n].coordinates_x   = (double*) Malloc(search->srch_corners*sizeof(double));
+          src_grid_cells[n].coordinates_y   = (double*) Malloc(search->srch_corners*sizeof(double));
+          src_grid_cells[n].coordinates_xyz = (double*) Malloc(3*search->srch_corners*sizeof(double));
         }
 
       max_srch_cells = num_srch_cells;
@@ -301,20 +301,17 @@ static
 void boundbox_from_corners1r(long ic, long nc, const double *restrict corner_lon,
 			     const double *restrict corner_lat, restr_t *restrict bound_box)
 {
-  long inc, j;
-  restr_t clon, clat;
+  long inc = ic*nc;
 
-  inc = ic*nc;
-
-  clat = RESTR_SCALE(corner_lat[inc]);
-  clon = RESTR_SCALE(corner_lon[inc]);
+  restr_t clat = RESTR_SCALE(corner_lat[inc]);
+  restr_t clon = RESTR_SCALE(corner_lon[inc]);
 
   bound_box[0] = clat;
   bound_box[1] = clat;
   bound_box[2] = clon;
   bound_box[3] = clon;
 
-  for ( j = 1; j < nc; ++j )
+  for ( long j = 1; j < nc; ++j )
     {
       clat = RESTR_SCALE(corner_lat[inc+j]);
       clon = RESTR_SCALE(corner_lon[inc+j]);
@@ -608,9 +605,9 @@ void remapNormalizeWeights(remapgrid_t *tgt_grid, remapvars_t *rv)
 	  tgt_cell_add = rv->tgt_cell_add[n];
 
           if ( IS_NOT_EQUAL(tgt_grid->cell_area[tgt_cell_add], 0) )
-	    norm_factor = ONE/tgt_grid->cell_area[tgt_cell_add];
+	    norm_factor = 1./tgt_grid->cell_area[tgt_cell_add];
           else
-            norm_factor = ZERO;
+            norm_factor = 0.;
 
 	  rv->wts[n*num_wts] *= norm_factor;
 	}
@@ -630,9 +627,9 @@ void remapNormalizeWeights(remapgrid_t *tgt_grid, remapvars_t *rv)
 	  tgt_cell_add = rv->tgt_cell_add[n];
 
           if ( IS_NOT_EQUAL(tgt_grid->cell_frac[tgt_cell_add], 0) )
-	    norm_factor = ONE/tgt_grid->cell_frac[tgt_cell_add];
+	    norm_factor = 1./tgt_grid->cell_frac[tgt_cell_add];
           else
-            norm_factor = ZERO;
+            norm_factor = 0.;
 
 	  rv->wts[n*num_wts] *= norm_factor;
 	}
