@@ -1,4 +1,4 @@
-#include "cdo.h"
+#include "cdo_int.h"
 #include "remap.h"
 /*
 #if defined(_OPENMP)
@@ -43,7 +43,7 @@ void calc_bin_addr_omp(long gridsize, long nbins, const restr_t* restrict bin_la
   restr_t cell_bound_box_lat1, cell_bound_box_lat2;
 #if defined(_OPENMP)
   extern int ompNumThreads;
-  restr_t (*omp_bin_addr)[ompNumThreads] = malloc(nbins*sizeof(*omp_bin_addr));
+  restr_t (*omp_bin_addr)[ompNumThreads] = Malloc(nbins*sizeof(*omp_bin_addr));
 
   for ( int ompthID = 0; ompthID < ompNumThreads; ++ompthID )
     for ( n = 0; n < nbins; ++n )
@@ -101,7 +101,7 @@ void calc_bin_addr_omp(long gridsize, long nbins, const restr_t* restrict bin_la
 	}
     }
 
-  free(omp_bin_addr);
+  Free(omp_bin_addr);
 #endif
 }
 */
@@ -121,7 +121,7 @@ void calc_lat_bins(remapgrid_t* src_grid, remapgrid_t* tgt_grid, int map_type)
 
   if ( nbins > 0 )
     {
-      bin_lats = src_grid->bin_lats = (restr_t*) realloc(src_grid->bin_lats, 2*nbins*sizeof(restr_t));
+      bin_lats = src_grid->bin_lats = (restr_t*) Realloc(src_grid->bin_lats, 2*nbins*sizeof(restr_t));
 
       for ( n = 0; n < nbins; ++n )
 	{
@@ -130,28 +130,28 @@ void calc_lat_bins(remapgrid_t* src_grid, remapgrid_t* tgt_grid, int map_type)
 	  bin_lats[n2+1] = RESTR_SCALE((n+1)*dlat - PIH);
 	}
 
-      src_grid->bin_addr = (int*) realloc(src_grid->bin_addr, 2*nbins*sizeof(int));
+      src_grid->bin_addr = (int*) Realloc(src_grid->bin_addr, 2*nbins*sizeof(int));
 
       calc_bin_addr(src_grid->size, nbins, bin_lats, src_grid->cell_bound_box, src_grid->bin_addr);
 
       if ( map_type == MAP_TYPE_CONSERV || map_type == MAP_TYPE_CONSERV_YAC )
 	{
-	  tgt_grid->bin_addr = (int*) realloc(tgt_grid->bin_addr, 2*nbins*sizeof(int));
+	  tgt_grid->bin_addr = (int*) Realloc(tgt_grid->bin_addr, 2*nbins*sizeof(int));
 
 	  calc_bin_addr(tgt_grid->size, nbins, bin_lats, tgt_grid->cell_bound_box, tgt_grid->bin_addr);
 
-	  free(src_grid->bin_lats); src_grid->bin_lats = NULL;
+	  Free(src_grid->bin_lats); src_grid->bin_lats = NULL;
 	}
    }
 
   if ( map_type == MAP_TYPE_CONSERV_YAC )
     {
-      free(tgt_grid->cell_bound_box); tgt_grid->cell_bound_box = NULL;
+      Free(tgt_grid->cell_bound_box); tgt_grid->cell_bound_box = NULL;
     }
  
   if ( map_type == MAP_TYPE_DISTWGT )
     {
-      free(src_grid->cell_bound_box); src_grid->cell_bound_box = NULL;
+      Free(src_grid->cell_bound_box); src_grid->cell_bound_box = NULL;
     }
 }
 
@@ -241,7 +241,7 @@ long get_srch_cells(long tgt_cell_add, long nbins, int *bin_addr1, int *bin_addr
 	}
     }
 
-  return (num_srch_cells);
+  return num_srch_cells;
 }
 
 static
