@@ -2,12 +2,11 @@
 #  include "config.h"
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <math.h>
+#if defined(_OPENMP)
+#include <omp.h>
+#endif
 
-#include "cdo.h"
+#include "cdo_int.h"
 #include "remap.h"
 
 /*****************************************************************************/
@@ -114,7 +113,7 @@ void remap_heapsort(const long num_links, int *restrict add1, int *restrict add2
 
       /* Loop until proper level is found for this link, or reach bottom */
 
-      for ( i = 0; i < num_links; ++i ) /* while ( TRUE ) */
+      for ( i = 0; i < num_links; ++i )
 	{
 	  /* Find the largest of the two daughters */
 
@@ -191,7 +190,7 @@ void remap_heapsort(const long num_links, int *restrict add1, int *restrict add2
 
       final_lvl = 0;
 
-      for ( i = 0; i < num_links; ++i ) /* while ( TRUE ) */
+      for ( i = 0; i < num_links; ++i )
 	{
 	  /* Find the largest of the two daughters */
 
@@ -287,20 +286,20 @@ void sort_add(long num_links, long num_wts, int *restrict add1, int *restrict ad
 
   if ( num_links <= 1 ) return;
 
-  idx = (int*) malloc(num_links*sizeof(int));
+  idx = (int*) Malloc(num_links*sizeof(int));
   for ( i = 0; i < num_links; ++i ) idx[i] = i;
 
   remap_heapsort(num_links, add1, add2, idx);
 
-  wgt_tmp = (double*) malloc(num_wts*num_links*sizeof(double));
+  wgt_tmp = (double*) Malloc(num_wts*num_links*sizeof(double));
   memcpy(wgt_tmp, weights, num_wts*num_links*sizeof(double));
 
   for ( i = 0; i < num_links; ++i )
     for ( n = 0; n < num_wts; ++n )
       weights[num_wts*i+n] = wgt_tmp[num_wts*idx[i]+n];
 
-  free(wgt_tmp);
-  free(idx);
+  Free(wgt_tmp);
+  Free(idx);
 
   if ( cdoVerbose ) 
     if ( !isSorted(add1, add2, num_links) ) fprintf(stderr, ">>>> sort_add failed!!!\n");
@@ -353,7 +352,7 @@ void sort_add_orig(long num_links, long num_wts, int *restrict add1, int *restri
 
       /* Loop until proper level is found for this link, or reach bottom */
 
-      for ( i = 0; i < num_links; i++ ) /* while ( TRUE ) */
+      for ( i = 0; i < num_links; i++ )
 	{
 	  /* Find the largest of the two daughters */
 
@@ -436,7 +435,7 @@ void sort_add_orig(long num_links, long num_wts, int *restrict add1, int *restri
 
       final_lvl = 0;
 
-      for ( i = 0; i < num_links; i++ ) /* while ( TRUE ) */
+      for ( i = 0; i < num_links; i++ )
 	{
 	  /* Find the largest of the two daughters */
 
@@ -620,7 +619,7 @@ void sort_par(long num_links, long num_wts, int *restrict add1, int *restrict ad
       exit(-1);
     }
 
-  idx = (long*) malloc(num_links*sizeof(long));
+  idx = (long*) Malloc(num_links*sizeof(long));
 
   /* SPLIT AND SORT THE DATA FRAGMENTS */
   /*
@@ -660,7 +659,7 @@ void sort_par(long num_links, long num_wts, int *restrict add1, int *restrict ad
 
 #if defined(_OPENMP)
 #pragma omp parallel for if ( depth < par_depth ) \
-        private(i, n, m, wgttmp, who_am_i)   \
+        private(n, m, wgttmp, who_am_i)    \
         shared(weights) num_threads(2)
 #endif
   for ( i=0; i < nsplit; i++ )
@@ -675,7 +674,7 @@ void sort_par(long num_links, long num_wts, int *restrict add1, int *restrict ad
       //	       who_am_i,parent,my_depth,omp_get_thread_num()+1,omp_get_num_threads());
 #endif
             
-      wgttmp = (double*) malloc(num_wts*nl[i]*sizeof(double));
+      wgttmp = (double*) Malloc(num_wts*nl[i]*sizeof(double));
        
       for ( m = 0; m < nl[i]; m++ )
 	for ( n = 0; n < num_wts; n++ )                      
@@ -687,7 +686,7 @@ void sort_par(long num_links, long num_wts, int *restrict add1, int *restrict ad
 	for ( n = 0; n < num_wts; n++ )
 	  weights[num_wts*(add_srt[i]+m)+n] = wgttmp[num_wts*m+n];
 
-      free(wgttmp);
+      Free(wgttmp);
     }
 
   /* ********************************* */
@@ -698,7 +697,7 @@ void sort_par(long num_links, long num_wts, int *restrict add1, int *restrict ad
                                                               /* ********************** */
   merge_lists(nl,add1s[0],add2s[0],add1s[1],add2s[1], idx);   /* MERGE THE SEGMENTS     */
                                                               /* ********************** */
-  tmp = (int*) malloc(num_links*sizeof(int));
+  tmp = (int*) Malloc(num_links*sizeof(int));
   
 #if defined(_OPENMP)
 #pragma omp parallel for if ( depth < par_depth ) private(i) num_threads(2)
@@ -721,10 +720,10 @@ void sort_par(long num_links, long num_wts, int *restrict add1, int *restrict ad
   for ( i = 0; i < num_links; i++ )
     add2[i] = tmp[i];
   
-  free(tmp);
+  Free(tmp);
   tmp = NULL;
   
-  tmp2 = (double*) malloc(num_links*num_wts*sizeof(double));
+  tmp2 = (double*) Malloc(num_links*num_wts*sizeof(double));
   
 #if defined(_OPENMP)
 #pragma omp parallel for if ( depth < par_depth ) private(i,n) num_threads(2)
@@ -740,10 +739,10 @@ void sort_par(long num_links, long num_wts, int *restrict add1, int *restrict ad
     for ( n = 0; n < num_wts; n++ )
       weights[num_wts*i+n] = tmp2[num_wts*i+n];
   
-  free(tmp2);
+  Free(tmp2);
   tmp2 = NULL;
   
-  free(idx);
+  Free(idx);
 }
 
 
