@@ -16,7 +16,6 @@ void uv2dv(double *fu, double *fv, double *sd, double *sv,
            double *pol2, double *pol3, int klev, int nlat, int nt);
 void dv2uv(double *d, double *o, double *u, double *v, double *f, double *g,
 	   int nt, int nsp, int nlev);
-void dv2ps(const double * restrict div, double * restrict pot, long nlev, long ntr);
 
 void after_legini_full(int ntr, int nlat, double *restrict poli, double *restrict pold, double *restrict pdev,
 		       double *restrict pol2, double *restrict pol3, double *restrict coslat);
@@ -32,12 +31,12 @@ void grid2spec(SPTRANS *sptrans, int gridIDin, double *arrayIn, int gridIDout, d
   int waves = ntr + 1;
   int nfc   = waves * 2;
 
-  double *fpwork = (double*) malloc(nlat*nfc*nlev*sizeof(double));
+  double *fpwork = (double*) Malloc(nlat*nfc*nlev*sizeof(double));
 
   gp2fc(sptrans->trig, sptrans->ifax, arrayIn, fpwork, nlat, nlon, nlev, nfc);
   fc2sp(fpwork, arrayOut, sptrans->pold, nlev, nlat, nfc, ntr);
 
-  free(fpwork);
+  Free(fpwork);
 }
 	   
    
@@ -50,17 +49,18 @@ void spec2grid(SPTRANS *sptrans, int gridIDin, double *arrayIn, int gridIDout, d
   int waves = ntr + 1;
   int nfc   = waves * 2;
 
-  double *fpwork = (double*) malloc(nlat*nfc*nlev*sizeof(double));
+  double *fpwork = (double*) Malloc(nlat*nfc*nlev*sizeof(double));
 
   sp2fc(arrayIn, fpwork, sptrans->poli, nlev, nlat, nfc, ntr);
   fc2gp(sptrans->trig, sptrans->ifax, fpwork, arrayOut, nlat, nlon, nlev, nfc);
 
-  free(fpwork);
+  Free(fpwork);
 }
 
 
 void four2spec(SPTRANS *sptrans, int gridIDin, double *arrayIn, int gridIDout, double *arrayOut)
 {
+  (void)gridIDin;
   int nlev  = 1;
   int ntr   = gridInqTrunc(gridIDout);
   int nlat  = sptrans->nlat;
@@ -76,7 +76,7 @@ void spec2four(SPTRANS *sptrans, int gridIDin, double *arrayIn, int gridIDout, d
   int nlev  = 1;
   int ntr   = gridInqTrunc(gridIDin);
   int nfc   = gridInqSize(gridIDout);
-  int nlat  = nfc2nlat(nfc, ntr);
+  int nlat  = nfc_to_nlat(nfc, ntr);
   int waves = ntr + 1;
   nfc   = waves * 2;
 
@@ -129,7 +129,7 @@ void speccut(int gridIDin, double *arrayIn, double *arrayOut, int *waves)
 
 SPTRANS *sptrans_new(int nlon, int nlat, int ntr, int flag)
 {
-  SPTRANS *sptrans = (SPTRANS*) malloc(sizeof(SPTRANS));
+  SPTRANS *sptrans = (SPTRANS*) Malloc(sizeof(SPTRANS));
 
   sptrans->nlon = nlon;
   sptrans->nlat = nlat;
@@ -138,16 +138,16 @@ SPTRANS *sptrans_new(int nlon, int nlat, int ntr, int flag)
   int nsp = (ntr + 1)*(ntr + 2);
   sptrans->poldim = nsp / 2 * nlat;
 
-  sptrans->trig = (double*) malloc(nlon * sizeof(double));
+  sptrans->trig = (double*) Malloc(nlon * sizeof(double));
   fft_set(sptrans->trig, sptrans->ifax, nlon);
 
-  sptrans->poli = (double*) malloc(sptrans->poldim * sizeof(double));
-  sptrans->pold = (double*) malloc(sptrans->poldim * sizeof(double));
+  sptrans->poli = (double*) Malloc(sptrans->poldim * sizeof(double));
+  sptrans->pold = (double*) Malloc(sptrans->poldim * sizeof(double));
   
   if ( flag )
     {
-      sptrans->pol2 = (double*) malloc(sptrans->poldim * sizeof(double));
-      sptrans->pol3 = (double*) malloc(sptrans->poldim * sizeof(double));
+      sptrans->pol2 = (double*) Malloc(sptrans->poldim * sizeof(double));
+      sptrans->pol3 = (double*) Malloc(sptrans->poldim * sizeof(double));
     }
   else
     {
@@ -155,8 +155,8 @@ SPTRANS *sptrans_new(int nlon, int nlat, int ntr, int flag)
       sptrans->pol3 = NULL;
     }
 
-  sptrans->coslat  = (double*) malloc(nlat * sizeof(double));
-  sptrans->rcoslat = (double*) malloc(nlat * sizeof(double));
+  sptrans->coslat  = (double*) Malloc(nlat * sizeof(double));
+  sptrans->rcoslat = (double*) Malloc(nlat * sizeof(double));
 
   if ( flag )
     after_legini_full(ntr, nlat, sptrans->poli, sptrans->pold, NULL,
@@ -167,7 +167,7 @@ SPTRANS *sptrans_new(int nlon, int nlat, int ntr, int flag)
   for ( int jgl = 0; jgl < nlat; ++jgl )
     sptrans->rcoslat[jgl] = 1.0 / sptrans->coslat[jgl];
 
-  return (sptrans);
+  return sptrans;
 }
 
 
@@ -175,34 +175,34 @@ void sptrans_delete(SPTRANS *sptrans)
 {
   if ( sptrans )
     {
-      if ( sptrans->trig ) { free(sptrans->trig);  sptrans->trig = NULL; }
-      if ( sptrans->poli ) { free(sptrans->poli);  sptrans->poli = NULL; }
-      if ( sptrans->pold ) { free(sptrans->pold);  sptrans->pold = NULL; }
-      if ( sptrans->pol2 ) { free(sptrans->pol2);  sptrans->pol2 = NULL; }
-      if ( sptrans->pol3 ) { free(sptrans->pol3);  sptrans->pol3 = NULL; }
-      if ( sptrans->coslat  ) { free(sptrans->coslat);   sptrans->coslat = NULL; }
-      if ( sptrans->rcoslat ) { free(sptrans->rcoslat);  sptrans->rcoslat = NULL; }
+      if ( sptrans->trig ) { Free(sptrans->trig);  sptrans->trig = NULL; }
+      if ( sptrans->poli ) { Free(sptrans->poli);  sptrans->poli = NULL; }
+      if ( sptrans->pold ) { Free(sptrans->pold);  sptrans->pold = NULL; }
+      if ( sptrans->pol2 ) { Free(sptrans->pol2);  sptrans->pol2 = NULL; }
+      if ( sptrans->pol3 ) { Free(sptrans->pol3);  sptrans->pol3 = NULL; }
+      if ( sptrans->coslat  ) { Free(sptrans->coslat);   sptrans->coslat = NULL; }
+      if ( sptrans->rcoslat ) { Free(sptrans->rcoslat);  sptrans->rcoslat = NULL; }
 
-      free(sptrans); sptrans = NULL;
+      Free(sptrans); sptrans = NULL;
     }
 }
 
 
 DVTRANS *dvtrans_new(int ntr)
 {
-  DVTRANS *dvtrans = (DVTRANS*) malloc(sizeof(DVTRANS));
+  DVTRANS *dvtrans = (DVTRANS*) Malloc(sizeof(DVTRANS));
 
   dvtrans->ntr = ntr;
 
   int dimsp = (ntr + 1)*(ntr + 2);
   dvtrans->fdim = dimsp / 2;
 
-  dvtrans->f1 = (double*) malloc(dvtrans->fdim * sizeof(double));
-  dvtrans->f2 = (double*) malloc(dvtrans->fdim * sizeof(double));
+  dvtrans->f1 = (double*) Malloc(dvtrans->fdim * sizeof(double));
+  dvtrans->f2 = (double*) Malloc(dvtrans->fdim * sizeof(double));
 
   geninx(ntr, dvtrans->f1, dvtrans->f2);
 
-  return (dvtrans);
+  return dvtrans;
 }
 
 
@@ -210,10 +210,10 @@ void dvtrans_delete(DVTRANS *dvtrans)
 {
   if ( dvtrans )
     {
-      if ( dvtrans->f1 ) { free(dvtrans->f1);  dvtrans->f1 = NULL; }
-      if ( dvtrans->f2 ) { free(dvtrans->f2);  dvtrans->f2 = NULL; }
+      if ( dvtrans->f1 ) { Free(dvtrans->f1);  dvtrans->f1 = NULL; }
+      if ( dvtrans->f2 ) { Free(dvtrans->f2);  dvtrans->f2 = NULL; }
 
-      free(dvtrans); dvtrans = NULL;
+      Free(dvtrans); dvtrans = NULL;
     }
 }
 
@@ -234,8 +234,8 @@ void trans_uv2dv(SPTRANS *sptrans, int nlev,
   int waves = ntr + 1;
   int nfc   = waves * 2;
 
-  double *fpwork1 = (double*) malloc(nlat*nfc*nlev*sizeof(double));
-  double *fpwork2 = (double*) malloc(nlat*nfc*nlev*sizeof(double));
+  double *fpwork1 = (double*) Malloc(nlat*nfc*nlev*sizeof(double));
+  double *fpwork2 = (double*) Malloc(nlat*nfc*nlev*sizeof(double));
 
   gp2fc(sptrans->trig, sptrans->ifax, gu, fpwork1, nlat, nlon, nlev, nfc);
   gp2fc(sptrans->trig, sptrans->ifax, gv, fpwork2, nlat, nlon, nlev, nfc);
@@ -245,8 +245,8 @@ void trans_uv2dv(SPTRANS *sptrans, int nlev,
 
   uv2dv(fpwork1, fpwork2, sd, svo, sptrans->pol2, sptrans->pol3, nlev, nlat, ntr);
 
-  free(fpwork1);
-  free(fpwork2);
+  Free(fpwork1);
+  Free(fpwork2);
 }
 
 
@@ -272,7 +272,7 @@ void trans_dv2uv(SPTRANS *sptrans, DVTRANS *dvtrans, int nlev,
 
   dv2uv(sd, svo, su, sv, dvtrans->f1, dvtrans->f2, ntr, dimsp, nlev);
 
-  double *fpwork = (double*) malloc(nlat*nfc*nlev*sizeof(double));
+  double *fpwork = (double*) Malloc(nlat*nfc*nlev*sizeof(double));
 
   sp2fc(su, fpwork, sptrans->poli, nlev, nlat, nfc, ntr);
   scaluv(fpwork, sptrans->rcoslat, nlat, nfc*nlev);
@@ -282,6 +282,5 @@ void trans_dv2uv(SPTRANS *sptrans, DVTRANS *dvtrans, int nlev,
   scaluv(fpwork, sptrans->rcoslat, nlat, nfc*nlev);
   fc2gp(sptrans->trig, sptrans->ifax, fpwork, gv, nlat, nlon, nlev, nfc);
 
-  free(fpwork);
+  Free(fpwork);
 }
-
