@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,45 +22,33 @@
 
 #include <cdi.h>
 #include "cdo.h"
+#include "util.h"
 #include "error.h"
 
 
-#ifndef  UNDEFID
-#define  UNDEFID  CDI_UNDEFID
-#endif
-
-
-int defineTable(char *tablearg)
+int defineTable(const char *tablearg)
 {
-  char *tablename;
-  int tableID = UNDEFID;
+  const char *tablename = tablearg;
 
-  tablename = tablearg;
+  int tableID = fileExists(tablename) ? tableRead(tablename) : CDI_UNDEFID;
 
-  tableID = tableRead(tablename);
-
-  if ( tableID == UNDEFID )
+  if ( tableID == CDI_UNDEFID )
     {
       char *tablepath = getenv("CD_TABLEPATH");
-
       if ( tablepath )
 	{
-	  char *tablefile = NULL;
 	  int len = sizeof(tablepath) + sizeof(tablename) + 3;
-	  tablefile = (char*) malloc(len);
+	  char tablefile[len];
 	  strcpy(tablefile, tablepath);
 	  strcat(tablefile, "/");
 	  strcat(tablefile, tablename);
-	  tableID = tableRead(tablefile);
-	  free(tablefile);
+	  if ( fileExists(tablename) ) tableID = tableRead(tablefile);
 	}
     }
 
-  if ( tableID == UNDEFID )
-    tableID = tableInq(-1, 0, tablename);
+  if ( tableID == CDI_UNDEFID ) tableID = tableInq(-1, 0, tablename);
 
-  if ( tableID == UNDEFID )
-    Error("table <%s> not found", tablename);
+  if ( tableID == CDI_UNDEFID ) Error("table <%s> not found", tablename);
 
-  return (tableID);
+  return tableID;
 }
