@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2017 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -78,6 +78,9 @@ int cdoDefaultTimeType   = CDI_UNDEFID;
 int cdoLockIO            = FALSE;
 int cdoCheckDatarange    = FALSE;
 
+int CDO_flt_digits       = 7;
+int CDO_dbl_digits       = 15;
+
 int CDO_Color            = FALSE;
 int CDO_Use_FFTW         = TRUE;
 int CDO_Version_Info     = TRUE;
@@ -92,7 +95,7 @@ int CDO_Reduce_Dim       = FALSE;
 int CDO_Append_History   = TRUE;
 int CDO_Reset_History    = FALSE;
 
-int cdoCompType          = COMPRESS_NONE;  // compression type
+int cdoCompType          = CDI_COMPRESS_NONE;  // compression type
 int cdoCompLevel         = 0;              // compression level
 int cdoDebug             = 0;
 int cdoChunkType         = CDI_UNDEFID;
@@ -122,14 +125,13 @@ int timer_read, timer_write;
 const char *cdoComment(void)
 {
   static char comment[256];
-  static int init = 0;
+  static bool init = false;
 
   if ( ! init )
     {
-      init = 1;
+      init = true;
 
       int size = strlen(CDO_Version);
-
       strncat(comment, CDO_Version, size);
       comment[size] = 0;
     }
@@ -182,56 +184,47 @@ int cdo_omp_get_thread_num(void)
 
 char *getProgname(char *string)
 {
-  char *progname;
-
 #if defined(_WIN32)
   /*  progname = strrchr(string, '\\'); */
-  progname = " cdo";
+  char *progname = " cdo";
 #else
-  progname = strrchr(string, '/');
+  char *progname = strrchr(string, '/');
 #endif
 
   if ( progname == NULL ) progname = string;
   else                    progname++;
 
-  return (progname);
+  return progname;
 }
 
 char *getOperator(const char *argument)
 {
   char *operatorArg = NULL;
-  size_t len;
 
   if ( argument )
     {
-      len = 1 + strlen(argument);
+      size_t len = 1 + strlen(argument);
 
       operatorArg = (char*) Malloc(len);
 
       memcpy(operatorArg, argument, len);
     }
 
-  return (operatorArg);
+  return operatorArg;
 }
 
-char *operatorAlias(char *operatorName);
+const char *operatorAlias(const char *operatorName);
 
-char *getOperatorName(const char *operatorArg)
+const char *getOperatorName(const char *operatorArg)
 {
-  char *commapos;
   char *operatorName = NULL;
-  size_t len;
 
   if ( operatorArg )
     {
       if ( operatorArg[0] == '-' ) operatorArg++;
 
-      commapos = (char *)strchr(operatorArg, ',');
-
-      if ( commapos )
-        len = commapos - operatorArg;
-      else
-        len = strlen(operatorArg);
+      char *commapos = (char *)strchr(operatorArg, ',');
+      size_t len = (commapos != NULL) ? (size_t)(commapos - operatorArg) : strlen(operatorArg);
 
       operatorName = (char*) Malloc(len+1);
 
@@ -239,24 +232,21 @@ char *getOperatorName(const char *operatorArg)
       operatorName[len] = '\0';
     }
 
-  /*  return (operatorName); */
-  char * alias = operatorAlias(operatorName);
-  return alias;
+  /*  return operatorName; */
+  return operatorAlias(operatorName);
 }
 
 
 argument_t *file_argument_new(const char *filename)
 {
-  argument_t *argument;
-
-  argument = (argument_t*) Calloc(1, sizeof(argument_t));
+  argument_t *argument = (argument_t*) Calloc(1, sizeof(argument_t));
 
   argument->argc = 1;
   argument->argv = (char **) Calloc(1, sizeof(char *));
   argument->argv[0] = (char *) filename;
   argument->args = (char *) filename;
 
-  return (argument);
+  return argument;
 }
 
 
@@ -276,9 +266,7 @@ void file_argument_free(argument_t *argument)
 
 argument_t *argument_new(size_t argc, size_t len)
 {
-  argument_t *argument;
-
-  argument = (argument_t*) Calloc(1, sizeof(argument_t));
+  argument_t *argument = (argument_t*) Calloc(1, sizeof(argument_t));
 
   if ( argc > 0 )
     {
@@ -289,7 +277,7 @@ argument_t *argument_new(size_t argc, size_t len)
   if ( len > 0 )
     argument->args = (char*) Calloc(len, sizeof(char));
 
-  return (argument);
+  return argument;
 }
 
 
@@ -327,11 +315,9 @@ void argument_free(argument_t *argument)
 
 void argument_fill(argument_t *argument, int argc, char *argv[])
 {
-  int iarg;
-
   assert(argument->argc == argc);
 
-  for ( iarg = 0; iarg < argc; ++iarg )
+  for ( int iarg = 0; iarg < argc; ++iarg )
     argument->argv[iarg] = strdup(argv[iarg]);
 }
 
@@ -339,24 +325,48 @@ void argument_fill(argument_t *argument, int argc, char *argv[])
 char *getFileArg(char *argument)
 {
   char *fileArg = NULL;
-  char *parg;
-  char *blankpos;
-  size_t len;
 
   if ( argument )
     {
-      blankpos = strchr(argument, ' ');
+      char *blankpos = strchr(argument, ' ');
 
       if ( blankpos )
         {
-          parg = blankpos + 1;
-          len = strlen(parg);
+          char *parg = blankpos + 1;
+          size_t len = strlen(parg);
           fileArg = (char*) Malloc(len+1);
           strcpy(fileArg, parg);
         }
     }
 
-  return (fileArg);
+  return fileArg;
+}
+
+static
+void trim_flt(char *ss)
+{
+  char *cp = ss;
+  if ( *cp == '-' ) cp++;
+  while ( isdigit((int)*cp ) || *cp == '.' ) cp++;
+  if ( *--cp == '.' ) return;
+
+  char *ep = cp+1;
+  while ( *cp == '0' ) cp--;
+  cp++;
+  if ( cp == ep ) return;
+  while ( *ep ) *cp++ = *ep++;
+  *cp = '\0';
+
+  return;
+}
+
+
+char *double_to_attstr(int digits, char *str, size_t len, double value)
+{
+  int ret = snprintf(str, len, "%#.*g", digits, value);
+  assert(ret != -1 && ret < (int)len);
+  trim_flt(str);
+  return str;
 }
 
 
@@ -375,56 +385,80 @@ void input_int(char *arg, int intarr[], int maxint, int *nintfound)
 
 void strtolower(char *str)
 {
-  int i, len;
-
   if ( str )
+    for ( size_t i = 0; str[i]; ++i )
+      str[i] = (char)tolower((int)str[i]);
+}
+
+
+const char *parameter2word(const char *string)
+{
+  size_t len = strlen(string);
+
+  for ( size_t i = 0; i < len; ++i )
     {
-      len = (int) strlen(str);
-      for ( i = 0; i < len; i++ )
-        str[i] = tolower((int) str[i]);
+      int c = string[i];
+      if ( iscntrl(c) || isblank(c) )
+        cdoAbort("Word parameter >%s< contains invalid character at position %d!", string, i+1);
     }
+
+  if ( len == 0 ) cdoAbort("Word parameter >%s< is empty!", string);
+
+  return string;
+}
+
+
+bool parameter2bool(const char *str)
+{
+  size_t len = strlen(str);
+
+  if ( len == 1 )
+    {
+      if ( *str == 't' || *str == 'T' || *str == '1' ) return true;
+      if ( *str == 'f' || *str == 'F' || *str == '0' ) return false;
+    }
+  else if ( len == 4 && (STR_IS_EQ(str, "true")  || STR_IS_EQ(str, "TRUE")  || STR_IS_EQ(str, "True"))  ) return true;
+  else if ( len == 5 && (STR_IS_EQ(str, "false") || STR_IS_EQ(str, "FALSE") || STR_IS_EQ(str, "False")) ) return false;
+
+  cdoAbort("Boolean parameter >%s< contains invalid characters!", str);
+
+  return false;
 }
 
 
 double parameter2double(const char *string)
 {
   char *endptr = NULL;
-
   double fval = strtod(string, &endptr);
-
   if ( *endptr != 0 )
     cdoAbort("Float parameter >%s< contains invalid character at position %d!",
 	     string, (int)(endptr-string+1));
 
-  return (fval);
+  return fval;
 }
 
 
 int parameter2int(const char *string)
 {
   char *endptr = NULL;
-
   int ival = (int) strtol(string, &endptr, 10);
-
   if ( *endptr != 0 )
     cdoAbort("Integer parameter >%s< contains invalid character at position %d!",
 	     string, (int)(endptr-string+1));
 
-  return (ival);
+  return ival;
 }
 
 
 int parameter2intlist(const char *string)
 {
   char *endptr = NULL;
-
   int ival = (int) strtol(string, &endptr, 10);
-
   if ( *endptr != 0 && *endptr != '/' && (endptr - string) == 0 )
     cdoAbort("Integer parameter >%s< contains invalid character at position %d!",
 	     string, (int)(endptr-string+1));
 
-  return (ival);
+  return ival;
 }
 
 
@@ -435,11 +469,11 @@ static int season_start = START_DEC;
 
 int get_season_start(void)
 {
-  static int lgetenv = TRUE;
+  static bool lgetenv = true;
 
   if ( lgetenv )
     {
-      lgetenv = FALSE;
+      lgetenv = false;
   
       char *envstr = getenv("CDO_SEASON_START");
       if ( envstr )
@@ -463,12 +497,10 @@ int get_season_start(void)
 
 void get_season_name(const char *seas_name[])
 {
-  long i;
-
   if ( get_season_start() == START_DEC )
-    for ( i = 0; i < 4; ++i ) seas_name[i] = seas_name_dec[i];
+    for ( int i = 0; i < 4; ++i ) seas_name[i] = seas_name_dec[i];
   else
-    for ( i = 0; i < 4; ++i ) seas_name[i] = seas_name_jan[i];
+    for ( int i = 0; i < 4; ++i ) seas_name[i] = seas_name_jan[i];
 }
 
 
@@ -503,25 +535,25 @@ int month_to_season(int month)
 #include <sys/stat.h>
 //#include <unistd.h>
 
-int fileExists(const char *restrict filename)
+bool fileExists(const char *restrict filename)
 {
-  int status = 0;
+  bool status = false;
   struct stat buf;
 
   if ( stat(filename, &buf) == 0 )
     {
-      if ( S_ISREG(buf.st_mode) && buf.st_size > 0 ) status = 1;
+      if ( S_ISREG(buf.st_mode) && buf.st_size > 0 ) status = true;
     }
 
   return status;
 }
 
 
-int userFileOverwrite(const char *restrict filename)
+bool userFileOverwrite(const char *restrict filename)
 {
-  int status = 0;
+  bool status = false;
 
-  if ( stdin_is_tty && stderr_is_tty )
+  if ( !cdoSilentMode && stdin_is_tty && stderr_is_tty )
     {
       fprintf(stderr, "File %s already exists, overwrite? (yes/no): ", filename);
       char line[1024];
@@ -532,13 +564,13 @@ int userFileOverwrite(const char *restrict filename)
       if ( len == 3 )
         {
           if ( pline[0] == 'y' && pline[1] == 'e' && pline[2] == 's' )
-            status = 1;
+            status = true;
           else if ( pline[0] == 'Y' && pline[1] == 'E' && pline[2] == 'S' )
-            status = 1;
+            status = true;
         }
       else if ( len == 1 )
         {
-          if ( pline[0] == 'y' || pline[0] == 'Y' ) status = 1;
+          if ( pline[0] == 'y' || pline[0] == 'Y' ) status = true;
         }
     }
 
@@ -546,13 +578,13 @@ int userFileOverwrite(const char *restrict filename)
 }
 
 
-int ps_lhead = FALSE;
+bool ps_lhead = false;
 int ps_nch   = 0;
 int ps_cval  = -1;
 
 void progressInit(void)
 {
-  ps_lhead = FALSE;
+  ps_lhead = false;
   ps_nch   = 0;
   ps_cval  = -1;
 }
@@ -560,8 +592,6 @@ void progressInit(void)
 
 void progressStatus(double offset, double refval, double curval)
 {
-  int ival;
-
   if ( cdoSilentMode ) return;
   if ( !stdout_is_tty ) return;
 
@@ -572,13 +602,13 @@ void progressStatus(double offset, double refval, double curval)
   curval = curval < 0 ? 0: curval;
   curval = curval > 1 ? 1: curval;
 
-  ival = (offset + refval*curval)*100;
+  int ival = (offset + refval*curval)*100;
 
   if ( ps_cval == -1 )
     {
       ps_nch = fprintf(stdout, "%s: %3d%%", processInqPrompt(), 0);
       fflush(stdout);
-      ps_lhead = TRUE;
+      ps_lhead = true;
     }
 
   if ( ival != ps_cval )
@@ -590,7 +620,7 @@ void progressStatus(double offset, double refval, double curval)
 
   if ( ps_cval == 100 && ps_lhead )
     {
-      ps_lhead = FALSE;
+      ps_lhead = false;
       while ( ps_nch-- ) fprintf(stdout, "\b \b");
       fflush(stdout);
     }
@@ -601,52 +631,50 @@ int datatype2str(int datatype, char *datatypestr)
 {
   int status = 0;
 
-  if      ( datatype == DATATYPE_PACK   ) strcpy(datatypestr, "P0");
-  else if ( datatype > 0 && datatype <= 32  ) sprintf(datatypestr, "P%d", datatype);
-  else if ( datatype == DATATYPE_CPX32  ) strcpy(datatypestr, "C32");
-  else if ( datatype == DATATYPE_CPX64  ) strcpy(datatypestr, "C64");
-  else if ( datatype == DATATYPE_FLT32  ) strcpy(datatypestr, "F32");
-  else if ( datatype == DATATYPE_FLT64  ) strcpy(datatypestr, "F64");
-  else if ( datatype == DATATYPE_INT8   ) strcpy(datatypestr, "I8");
-  else if ( datatype == DATATYPE_INT16  ) strcpy(datatypestr, "I16");
-  else if ( datatype == DATATYPE_INT32  ) strcpy(datatypestr, "I32");
-  else if ( datatype == DATATYPE_UINT8  ) strcpy(datatypestr, "U8");
-  else if ( datatype == DATATYPE_UINT16 ) strcpy(datatypestr, "U16");
-  else if ( datatype == DATATYPE_UINT32 ) strcpy(datatypestr, "U32");
+  if      ( datatype == CDI_DATATYPE_PACK   ) strcpy(datatypestr, "P0");
+  else if ( datatype > 0 && datatype <= 32  ) snprintf(datatypestr, 4, "P%d", datatype);
+  else if ( datatype == CDI_DATATYPE_CPX32  ) strcpy(datatypestr, "C32");
+  else if ( datatype == CDI_DATATYPE_CPX64  ) strcpy(datatypestr, "C64");
+  else if ( datatype == CDI_DATATYPE_FLT32  ) strcpy(datatypestr, "F32");
+  else if ( datatype == CDI_DATATYPE_FLT64  ) strcpy(datatypestr, "F64");
+  else if ( datatype == CDI_DATATYPE_INT8   ) strcpy(datatypestr, "I8");
+  else if ( datatype == CDI_DATATYPE_INT16  ) strcpy(datatypestr, "I16");
+  else if ( datatype == CDI_DATATYPE_INT32  ) strcpy(datatypestr, "I32");
+  else if ( datatype == CDI_DATATYPE_UINT8  ) strcpy(datatypestr, "U8");
+  else if ( datatype == CDI_DATATYPE_UINT16 ) strcpy(datatypestr, "U16");
+  else if ( datatype == CDI_DATATYPE_UINT32 ) strcpy(datatypestr, "U32");
   else                                  { strcpy(datatypestr, "-1"); status = -1;}
 
-  return (status);
+  return status;
 }
 
 
 int str2datatype(const char *datatypestr)
 {
   int datatype = -1;
-  size_t len;
-
-  len = strlen(datatypestr);
+  size_t len = strlen(datatypestr);
 
   if ( len > 1 )
     {
       int ilen = atoi(datatypestr+1);
-      if      ( strncmp(datatypestr, "P0",  len) == 0 ) datatype = DATATYPE_PACK;
+      if      ( strncmp(datatypestr, "P0",  len) == 0 ) datatype = CDI_DATATYPE_PACK;
       else if ( strncmp(datatypestr, "P",     1) == 0 &&
-                ilen > 0 && ilen <= 32 )               datatype = atoi(datatypestr+1);
-      else if ( strncmp(datatypestr, "C32", len) == 0 ) datatype = DATATYPE_CPX32;
-      else if ( strncmp(datatypestr, "C64", len) == 0 ) datatype = DATATYPE_CPX64;
-      else if ( strncmp(datatypestr, "F32", len) == 0 ) datatype = DATATYPE_FLT32;
-      else if ( strncmp(datatypestr, "F64", len) == 0 ) datatype = DATATYPE_FLT64;
-      else if ( strncmp(datatypestr, "I8",  len) == 0 ) datatype = DATATYPE_INT8;
-      else if ( strncmp(datatypestr, "I16", len) == 0 ) datatype = DATATYPE_INT16;
-      else if ( strncmp(datatypestr, "I32", len) == 0 ) datatype = DATATYPE_INT32;
-      else if ( strncmp(datatypestr, "U8",  len) == 0 ) datatype = DATATYPE_UINT8;
-      else if ( strncmp(datatypestr, "U16", len) == 0 ) datatype = DATATYPE_UINT16;
-      else if ( strncmp(datatypestr, "U32", len) == 0 ) datatype = DATATYPE_UINT32;
-      else if ( strncmp(datatypestr, "real",   len) == 0 ) datatype = DATATYPE_FLT32;
-      else if ( strncmp(datatypestr, "double", len) == 0 ) datatype = DATATYPE_FLT64;
+                ilen > 0 && ilen <= 32 )                datatype = atoi(datatypestr+1);
+      else if ( strncmp(datatypestr, "C32", len) == 0 ) datatype = CDI_DATATYPE_CPX32;
+      else if ( strncmp(datatypestr, "C64", len) == 0 ) datatype = CDI_DATATYPE_CPX64;
+      else if ( strncmp(datatypestr, "F32", len) == 0 ) datatype = CDI_DATATYPE_FLT32;
+      else if ( strncmp(datatypestr, "F64", len) == 0 ) datatype = CDI_DATATYPE_FLT64;
+      else if ( strncmp(datatypestr, "I8",  len) == 0 ) datatype = CDI_DATATYPE_INT8;
+      else if ( strncmp(datatypestr, "I16", len) == 0 ) datatype = CDI_DATATYPE_INT16;
+      else if ( strncmp(datatypestr, "I32", len) == 0 ) datatype = CDI_DATATYPE_INT32;
+      else if ( strncmp(datatypestr, "U8",  len) == 0 ) datatype = CDI_DATATYPE_UINT8;
+      else if ( strncmp(datatypestr, "U16", len) == 0 ) datatype = CDI_DATATYPE_UINT16;
+      else if ( strncmp(datatypestr, "U32", len) == 0 ) datatype = CDI_DATATYPE_UINT32;
+      else if ( strncmp(datatypestr, "real",   len) == 0 ) datatype = CDI_DATATYPE_FLT32;
+      else if ( strncmp(datatypestr, "double", len) == 0 ) datatype = CDI_DATATYPE_FLT64;
     }
 
-  return (datatype);
+  return datatype;
 }
 
 
@@ -668,24 +696,23 @@ off_t fileSize(const char *restrict filename)
 
 
 /* 
- * Return the filetype extension (const char)
- * for a given filetype (int)
+ * Return the filetype extension (const char) for a given filetype (int)
  * TODO: handle lists of extensions i.e. grb and grb2 for GRIB2-format
  */
 const char *filetypeext(int filetype)
 {
   switch ( filetype )
     {
-    case FILETYPE_GRB:
-    case FILETYPE_GRB2: return (".grb");   break;
-    case FILETYPE_NC:
-    case FILETYPE_NC2:
-    case FILETYPE_NC4:
-    case FILETYPE_NC4C: return (".nc");    break;
-    case FILETYPE_SRV:  return (".srv");   break;
-    case FILETYPE_EXT:  return (".ext");   break;
-    case FILETYPE_IEG:  return (".ieg");   break;
-    default:            return ("");
+    case CDI_FILETYPE_GRB:
+    case CDI_FILETYPE_GRB2: return ".grb";   break;
+    case CDI_FILETYPE_NC:
+    case CDI_FILETYPE_NC2:
+    case CDI_FILETYPE_NC4:
+    case CDI_FILETYPE_NC4C: return ".nc";    break;
+    case CDI_FILETYPE_SRV:  return ".srv";   break;
+    case CDI_FILETYPE_EXT:  return ".ext";   break;
+    case CDI_FILETYPE_IEG:  return ".ieg";   break;
+    default:                return "";
     }
 }
 
@@ -736,8 +763,8 @@ void cdoGenFileSuffix(char *filesuffix, size_t maxlen, int filetype, int vlistID
         }
       else
         {
-          int lready = FALSE;
-          int lcompsz = FALSE;
+          bool lready = false;
+          bool lcompsz = false;
           
           if ( filetype == cdoDefaultFileType && cdoDefaultDataType == -1 && cdoDefaultByteorder == -1 )
             {
@@ -753,20 +780,20 @@ void cdoGenFileSuffix(char *filesuffix, size_t maxlen, int filetype, int vlistID
                       switch (firstchar)
                         {
                         case 'g':
-                          if ( cdoDefaultFileType == FILETYPE_GRB || cdoDefaultFileType == FILETYPE_GRB2 ) lready = TRUE;
+                          if ( cdoDefaultFileType == CDI_FILETYPE_GRB || cdoDefaultFileType == CDI_FILETYPE_GRB2 ) lready = true;
                           break;
                         case 'n':
-                          if ( cdoDefaultFileType == FILETYPE_NC || cdoDefaultFileType == FILETYPE_NC2 ||
-                               cdoDefaultFileType == FILETYPE_NC4 || cdoDefaultFileType == FILETYPE_NC4C ) lready = TRUE;
+                          if ( cdoDefaultFileType == CDI_FILETYPE_NC || cdoDefaultFileType == CDI_FILETYPE_NC2 ||
+                               cdoDefaultFileType == CDI_FILETYPE_NC4 || cdoDefaultFileType == CDI_FILETYPE_NC4C ) lready = true;
                           break;
                         case 's':
-                          if ( cdoDefaultFileType == FILETYPE_SRV ) lready = TRUE;
+                          if ( cdoDefaultFileType == CDI_FILETYPE_SRV ) lready = true;
                           break;
                         case 'e':
-                          if ( cdoDefaultFileType == FILETYPE_EXT ) lready = TRUE;
+                          if ( cdoDefaultFileType == CDI_FILETYPE_EXT ) lready = true;
                           break;
                         case 'i':
-                          if ( cdoDefaultFileType == FILETYPE_IEG ) lready = TRUE;
+                          if ( cdoDefaultFileType == CDI_FILETYPE_IEG ) lready = true;
                           break;
                         }
                     }
@@ -787,10 +814,10 @@ void cdoGenFileSuffix(char *filesuffix, size_t maxlen, int filetype, int vlistID
           if ( !lready )
             {
               strncat(filesuffix, streamFilesuffix(cdoDefaultFileType), maxlen-1);
-              if ( cdoDefaultFileType == FILETYPE_GRB && vlistIsSzipped(vlistID) ) lcompsz = TRUE;
+              if ( cdoDefaultFileType == CDI_FILETYPE_GRB && vlistIsSzipped(vlistID) ) lcompsz = true;
             }
 
-          if ( cdoDefaultFileType == FILETYPE_GRB && cdoCompType == COMPRESS_SZIP ) lcompsz = TRUE;
+          if ( cdoDefaultFileType == CDI_FILETYPE_GRB && cdoCompType == CDI_COMPRESS_SZIP ) lcompsz = true;
           if ( lcompsz ) strncat(filesuffix, ".sz", maxlen-1);
         }
     }
@@ -801,11 +828,54 @@ int cdoFiletype(void)
 {
   if ( cdoDefaultFileType == CDI_UNDEFID )
     {
-      cdoDefaultFileType = FILETYPE_GRB;
+      cdoDefaultFileType = CDI_FILETYPE_GRB;
       if ( ! cdoSilentMode )
         cdoPrint("Set default filetype to GRIB");
     }
 
-  return (cdoDefaultFileType);
+  return cdoDefaultFileType;
 }
 
+
+void cdoSetNAN(double missval, size_t gridsize, double *array)
+{
+  if ( DBL_IS_NAN(missval) )
+    {
+      double newmissval = -9e33;
+      for ( size_t i = 0; i < gridsize; ++i )
+        if ( DBL_IS_EQUAL(array[i], missval) )
+          array[i] = newmissval;
+    }
+}
+
+
+void minmaxval(long nvals, double *array, int *imiss, double *minval, double *maxval)
+{
+  double xmin =  DBL_MAX;
+  double xmax = -DBL_MAX;
+
+  if ( imiss )
+    {
+      for ( long i = 0; i < nvals; ++i )
+	{
+	  if ( ! imiss[i] )
+	    {
+	      if      ( array[i] > xmax ) xmax = array[i];
+	      else if ( array[i] < xmin ) xmin = array[i];
+	    }
+	}
+    }
+  else
+    {
+      xmin = array[0];
+      xmax = array[0];
+      for ( long i = 1; i < nvals; ++i )
+	{
+	  if      ( array[i] > xmax ) xmax = array[i];
+	  else if ( array[i] < xmin ) xmin = array[i];
+	}
+    }
+
+  *minval = xmin;
+  *maxval = xmax;
+}
